#include <stddef.h>
#include <stdio.h>

#include <thread>
#include <chrono>
#include <fstream>
#include <sstream>

#include "SMWrapper.hpp"
#include "SM_GeneralWrapper.h"
#include "dpc_sm/executable.h"
#include "SMWrapper.hpp"
#ifdef __cplusplus
extern "C"
{
#endif

// C header here
#include "SM_Itg.h"

#ifdef __cplusplus
}
#endif


int prevLifeCycle = -1;

// To do: Load header file of data types
std::string LogFileNameDated(const std::string& inputName);
double getCurrentTimeStamp();
void logSMData(std::ofstream *ptr_File, CAN2SWC *canSignal, dpc::Decision4SM *sm_Decision4SM, dpc::VED4DPC *sm_VED4DPC, 
                dpc::ObjectTrim *sm_ObjectTrim, dpc::LaneInfo *sm_LaneInfo, dpc::LeftLine *sm_LeftLine, 
                dpc::LeftRoadEdge *sm_LeftRoadEdge, dpc::ODDInfo *sm_ODDInfo, dpc::RMEgoLftLnegeometry *sm_RMEgoLftLnegeometry, 
                dpc::RMEgoRgtLnegeometry *sm_RMEgoRgtLnegeometry, dpc::RMRdEdgeLftgeometry *sm_RMRdEdgeLftgeometry, 
                dpc::RMRdEdgeRgtgeometry *sm_RMRdEdgeRgtgeometry, dpc::RMSMMessage *sm_RMSMMessage, dpc::RightLine *sm_RightLine,
                dpc::RightRoadEdge *sm_RightRoadEdge, dpc::SM4Decision *provide);

void assignSMInputsFromBA(dpc::Decision4SM data) {
    // Call this function in call back of Deccision to TRPCLat and assigned the inputs from Decision
    SM_Itg_U.DC_OptAction_In = data.ba_opt_action; // TODO: Roy please check this assign
    
}

void assignSMInputsFromCAN(dpc::VED4DPC ved, CAN2SWC can) {
    uint8_t standStill = 0;
    if (ved.MotionState == 3) //TODO: check whether 3 is correct or not
    {
        standStill = 1;
    }

    // Call this function in call back of VED to TRPCLat and assigned the inputs from VED
    SM_Itg_U.vehicleStandStill_In = standStill;
    SM_Itg_U.VehicleStanstill_Valid_in = 1;

    SM_Itg_U.FL_WhlSpdValid_in = 1;
    SM_Itg_U.FR_WhlSpdValid_in = 1;
    SM_Itg_U.RL_WhlSpdValid_in = 1;
    SM_Itg_U.RR_WhlSpdValid_in = 1;
    SM_Itg_U.FL_WheelDirectionValid_in = 1;
    SM_Itg_U.FR_WheelDirectionValid_in = 1;
    SM_Itg_U.RL_WheelDirectionValid_in = 1;
    SM_Itg_U.RR_WheelDirectionValid_in = 1;

    SM_Itg_U.VehDynLongAccValid_in = 1;
    SM_Itg_U.VehDynLatAccValid_in = 1;
    SM_Itg_U.VehDynYawRateValid_in = 1;
    SM_Itg_U.YawRateCalSts_in = 1; // To be defined
    SM_Itg_U.LongitudeCtrlAvailable_in = 1;
    SM_Itg_U.VehicleSpeedValid_in = 1;

    // TODO: the following signals have not been defined yet!
    SM_Itg_U.EmergencyStop_In = 0; // has been set
    SM_Itg_U.EmergencyStopSwitchSts_in = 1; // has been set
    SM_Itg_U.IndctDrct_In = 0; // has been set
    SM_Itg_U.EPB_State_In = can.VCU_Info_1_Signals.ses_en_sts;
    SM_Itg_U.VCU_Ready_In = can.VCU_Info_1_Signals.vcu_mode;
    SM_Itg_U.ContainerLightSt_in = 0; // has been set
    SM_Itg_U.CurrentActGearValid_in = 1; // has been set
    SM_Itg_U.VehicleFailureLevel_in = 0; // has been set
    SM_Itg_U.VCU_SBW_Available_in = (prevLifeCycle!=SM_Itg_U.EPS_SteeringAngleValid_in); // caution!! SHOULD BE CHECKED
    SM_Itg_U.EPS_SBW_Available_in = can.VCU_Info_1_Signals.ses_en_sts;
    SM_Itg_U.EPS_SteeringAngleValid_in = 1; // has been set
    SM_Itg_U.EPS_SteeringAnglSPDValid_in = 1; // has been set
    SM_Itg_U.EPS_FaultLevel_in = 0; // has been set
    SM_Itg_U.EPS_SASCalSts_in = 1; // has been set
    SM_Itg_U.VCU_BBW_Available_in = 1; // has been set
    SM_Itg_U.EHBFaultLevel_in = 0; // has been set
    SM_Itg_U.EPBSysSts_in = 1; // has been set
    SM_Itg_U.RPS_AvailableSts_in = 1; // has been set
    SM_Itg_U.RPS_TakeoverReq_in = 0; // has been set
    SM_Itg_U.RPS_ShiftGearPosReq_in = 1; // has been set
    SM_Itg_U.MDC_Checksum_in = prevLifeCycle;

    prevLifeCycle = can.VCU_Info_1_Signals.vcu_life_signal;
}

void assignSMOutpusToDECISION (dpc::SM4Decision& provide) {
    provide.sm_lfcmfrtavlbhvr = SM_Itg_Y.Lfcmfrtavlbhvr;
    provide.sm_lcpavlbhvr = SM_Itg_Y.Lcpavlbhvr;
    provide.sm_lcavlbhvr = SM_Itg_Y.Lcavlbhvr;
    provide.sm_latactvfunc = SM_Itg_Y.LatActvFunc;
    provide.sm_drvmd = SM_Itg_Y.DrvMd;
    provide.sm_drvr_indctdrct = SM_Itg_Y.IndctDrct;
    provide.sm_drvrsetspd = SM_Itg_Y.DrvSetSpd;
    provide.sm_seltdistlvl = SM_Itg_Y.SeltDistLvl;
    provide.sm_spdlmtid01 = SM_Itg_Y.spdLimitid01;
    provide.sm_spdlmtid02 = SM_Itg_Y.spdLimitid02;
    provide.sm_speedlimit01 = SM_Itg_Y.spdLimit01;
    provide.sm_speedlimit02 = SM_Itg_Y.spdLimit02;
    provide.sm_controlwaypoint_offset01 = SM_Itg_Y.spdLimitOffset02;
    provide.sm_controlwaypoint_offset02 = SM_Itg_Y.spdLimitOffset01;
}


const double SM_LogFileDuration = 120; // seconds

int main() {
    dpc::Decision4SM sm_Decision4SM;
    dpc::VED4DPC sm_VED4DPC;
    dpc::ObjectTrim sm_ObjectTrim;
    dpc::LaneInfo sm_LaneInfo;
    dpc::LeftLine sm_LeftLine;
    dpc::LeftRoadEdge sm_LeftRoadEdge;
    dpc::ODDInfo sm_ODDInfo;
    dpc::RMEgoLftLnegeometry sm_RMEgoLftLnegeometry;
    dpc::RMEgoRgtLnegeometry sm_RMEgoRgtLnegeometry;
    dpc::RMRdEdgeLftgeometry sm_RMRdEdgeLftgeometry;
    dpc::RMRdEdgeRgtgeometry sm_RMRdEdgeRgtgeometry;
    dpc::RMSMMessage sm_RMSMMessage;
    dpc::RightLine sm_RightLine;
    dpc::RightRoadEdge sm_RightRoadEdge;

    CAN2SWC canSignal;

    dpc::SM4Decision provide;


    std::ofstream SMLogFile;
    SMLogFile.open(LogFileNameDated("SMLogFile"), std::ios::out);

    uint32_t SM_run_counter = 0;

    double SM_dataLoggingTimerStart = getCurrentTimeStamp();

    ara::log::InitLogging("DPC", "EM_TEST", ara::log::LogLevel::kVerbose, (ara::log::LogMode::kConsole | ara::log::LogMode::kRemote));

	ara::log::Logger &mainLog { ara::log::CreateLogger("main", "em sample context", ara::log::LogLevel::kVerbose) };

	mainLog.LogInfo() << "Let's produce some SM data...";

	GeneralWrapper gw(mainLog);

    while (1)
    {
        if (SM_run_counter == 0) {
            SM_Itg_initialize();
	mainLog.LogDebug() << "here is " << __LINE__;

            // 
			// ptr_reqPorts = p_VED_in();
			// ptr_proPorts = p_VED_out();
        }
        else {
	mainLog.LogDebug() << "here is " << __LINE__;

            gw._DM2SM_Required.s.GetData(sm_Decision4SM);
            gw._VED2SM_Required.s.GetData(sm_VED4DPC);
            gw._DM2SMObjectTrim_Required.s.GetData(sm_ObjectTrim);
            gw._EM2SMLaneInfo_Required.s.GetData(sm_LaneInfo);
            gw._EM2SMLeftLine_Required.s.GetData(sm_LeftLine);
            gw._EM2SMLeftRoadEdge_Required.s.GetData(sm_LeftRoadEdge);
            gw._ODDInfo_Required.s.GetData(sm_ODDInfo);
            gw._EM2SMRMEgoLftLnegeometry_Required.s.GetData(sm_RMEgoLftLnegeometry);
            gw._EM2SMRMEgoRgtLnegeometry_Required.s.GetData(sm_RMEgoRgtLnegeometry);
            gw._EM2SMRMRdEdgeLftgeometry_Required.s.GetData(sm_RMRdEdgeLftgeometry);
            gw._EM2SMRMRdEdgeRgtgeometry_Required.s.GetData(sm_RMRdEdgeRgtgeometry);
            gw._EM2SMRMSMMessage_Required.s.GetData(sm_RMSMMessage);
            gw._EM2SMRightLine_Required.s.GetData(sm_RightLine);
            gw._EM2SMRightRoadEdge_Required.s.GetData(sm_RightRoadEdge);
            gw._CAN2SM_Required.s.GetData(canSignal);

            // TO DO: Control data sync --> Must have the same arival time
            assignSMInputsFromBA(sm_Decision4SM);
            assignSMInputsFromCAN(sm_VED4DPC, canSignal);
	mainLog.LogDebug() << "here is " << __LINE__;

            SM_Itg_step();
	mainLog.LogDebug() << "here is " << __LINE__;

            assignSMOutpusToDECISION(provide);            
        }
	mainLog.LogDebug() << "here is " << __LINE__;

        gw._SM2Decision_Provide.s.SetData(provide);
        	mainLog.LogDebug() << "here is " << __LINE__;

        gw._SM2Decision_Provide.Publish();
	mainLog.LogDebug() << "here is " << __LINE__;

        SM_run_counter++;

        double SM_dataLoggingTimerNow = getCurrentTimeStamp();
        if ((SM_dataLoggingTimerNow - SM_dataLoggingTimerStart)<= SM_LogFileDuration) {
            logSMData(&SMLogFile, &canSignal, &sm_Decision4SM, &sm_VED4DPC, &sm_ObjectTrim, &sm_LaneInfo, &sm_LeftLine, &sm_LeftRoadEdge, &sm_ODDInfo, &sm_RMEgoLftLnegeometry, &sm_RMEgoRgtLnegeometry, &sm_RMRdEdgeLftgeometry, &sm_RMRdEdgeRgtgeometry, &sm_RMSMMessage, &sm_RightLine, &sm_RightRoadEdge, &provide);
        }
        else {
            SMLogFile.close();
        }

        // TODO: check time step!!!
        std::this_thread::sleep_for(std::chrono::milliseconds(10)); // TRPCLat cycle time: 10 ms
    }

    return 0;

}

void logSMData(
            std::ofstream *ptr_File,
            CAN2SWC *canSignal,
            dpc::Decision4SM *sm_Decision4SM,
            dpc::VED4DPC *sm_VED4DPC,
            dpc::ObjectTrim *sm_ObjectTrim,
            dpc::LaneInfo *sm_LaneInfo,
            dpc::LeftLine *sm_LeftLine,
            dpc::LeftRoadEdge *sm_LeftRoadEdge,
            dpc::ODDInfo *sm_ODDInfo,
            dpc::RMEgoLftLnegeometry *sm_RMEgoLftLnegeometry,
            dpc::RMEgoRgtLnegeometry *sm_RMEgoRgtLnegeometry,
            dpc::RMRdEdgeLftgeometry *sm_RMRdEdgeLftgeometry,
            dpc::RMRdEdgeRgtgeometry *sm_RMRdEdgeRgtgeometry,
            dpc::RMSMMessage *sm_RMSMMessage,
            dpc::RightLine *sm_RightLine,
            dpc::RightRoadEdge *sm_RightRoadEdge,
            dpc::SM4Decision *provide
        ){
    *ptr_File << 
        getCurrentTimeStamp()<< "," <<

        canSignal->VCU_Info_1_Signals.ses_en_sts << "," <<
        canSignal->VCU_Info_1_Signals.vcu_life_signal << "," <<
        canSignal->VCU_Info_1_Signals.vcu_mode << "," <<
        
        prevLifeCycle << "," <<

        SM_Itg_U.EmergencyStop_In << "," <<
        SM_Itg_U.IndctDrct_In << "," <<
        SM_Itg_U.DC_OptAction_In << "," <<
        SM_Itg_U.vehicleStandStill_In << "," <<
        SM_Itg_U.EPB_State_In << "," <<
        SM_Itg_U.VCU_Ready_In << "," <<
        SM_Itg_U.CurrentActGearValid_in << "," <<
        SM_Itg_U.VehicleFailureLevel_in << "," <<
        SM_Itg_U.VCU_SBW_Available_in << "," <<
        SM_Itg_U.EPS_SBW_Available_in << "," <<
        SM_Itg_U.EPS_SteeringAngleValid_in << "," <<
        SM_Itg_U.EPS_SteeringAnglSPDValid_in << "," <<
        SM_Itg_U.EPS_FaultLevel_in << "," <<
        SM_Itg_U.EPS_SASCalSts_in << "," <<
        SM_Itg_U.VCU_BBW_Available_in << "," <<
        SM_Itg_U.LongitudeCtrlAvailable_in << "," <<
        SM_Itg_U.EmergencyStopSwitchSts_in << "," <<
        SM_Itg_U.VehicleSpeedValid_in << "," <<
        SM_Itg_U.EHBFaultLevel_in << "," <<
        SM_Itg_U.EPBSysSts_in << "," <<
        SM_Itg_U.RPS_AvailableSts_in << "," <<
        SM_Itg_U.RPS_TakeoverReq_in << "," <<
        SM_Itg_U.RPS_ShiftGearPosReq_in << "," <<
        SM_Itg_U.FL_WhlSpdValid_in << "," <<
        SM_Itg_U.FR_WhlSpdValid_in << "," <<
        SM_Itg_U.RL_WhlSpdValid_in << "," <<
        SM_Itg_U.RR_WhlSpdValid_in << "," <<
        SM_Itg_U.FL_WheelDirectionValid_in << "," <<
        SM_Itg_U.FR_WheelDirectionValid_in << "," <<
        SM_Itg_U.RL_WheelDirectionValid_in << "," <<
        SM_Itg_U.RR_WheelDirectionValid_in << "," <<
        SM_Itg_U.VehicleStanstill_Valid_in << "," <<
        SM_Itg_U.VehDynLongAccValid_in << "," <<
        SM_Itg_U.VehDynLatAccValid_in << "," <<
        SM_Itg_U.VehDynYawRateValid_in << "," <<
        SM_Itg_U.YawRateCalSts_in << "," <<
        SM_Itg_U.ContainerLightSt_in << "," <<
        SM_Itg_U.MDC_Checksum_in << "," <<

        SM_Itg_Y.Lfcmfrtavlbhvr << "," <<
        SM_Itg_Y.Lcpavlbhvr << "," <<
        SM_Itg_Y.Lcavlbhvr << "," <<
        SM_Itg_Y.LatActvFunc << "," <<
        SM_Itg_Y.DrvMd << "," <<
        SM_Itg_Y.IndctDrct << "," <<
        SM_Itg_Y.DrvSetSpd << "," <<
        SM_Itg_Y.SeltDistLvl << "," <<
        SM_Itg_Y.spdLimitid01 << "," <<
        SM_Itg_Y.spdLimitid02 << "," <<
        SM_Itg_Y.spdLimit01 << "," <<
        SM_Itg_Y.spdLimit02 << "," <<
        SM_Itg_Y.spdLimitOffset02 << "," <<
        SM_Itg_Y.spdLimitOffset01 << "," <<

        sm_Decision4SM->ba_elk_tgt_id << "," <<
        sm_Decision4SM->ba_ldp_elk_md << "," <<
        sm_Decision4SM->ba_elk_tgt_dngr_lvl << "," <<
        sm_Decision4SM->ba_lc_end_pnt_x << "," <<
        sm_Decision4SM->ba_lc_end_pnt_y << "," <<
        sm_Decision4SM->ba_alc_abl_tgt_id << "," <<
        sm_Decision4SM->ba_alc_abl_tgt_dngr_lvl << "," <<
        sm_Decision4SM->ba_dyn_allw_max_spd << "," <<
        sm_Decision4SM->ba_opt_corridor_id << "," <<
        sm_Decision4SM->ba_opt_target_obj_id << "," <<
        sm_Decision4SM->ba_opt_action << "," <<
        sm_Decision4SM->ba_takeover_req << "," <<
        sm_Decision4SM->ba_lane_change_abolition_avlbl << "," <<
        // sm_Decision4SM->ba_obj_list2sm << "," <<
        sm_Decision4SM->ba_drcn_ind_req << "," <<

        sm_VED4DPC->LongitudinalVelocity << "," <<
        sm_VED4DPC->LongitudinalAcceleration << "," <<
        sm_VED4DPC->LateralVelocity << "," <<
        sm_VED4DPC->LateralAcceleration << "," <<
        sm_VED4DPC->MotionState << "," <<
        sm_VED4DPC->Yawrate << "," <<

        sm_ObjectTrim->object_id << "," <<
        sm_ObjectTrim->object_exist_probability << "," <<
        sm_ObjectTrim->classification << "," <<
        sm_ObjectTrim->relative_center_x << "," <<
        sm_ObjectTrim->relative_center_y << "," <<
        sm_ObjectTrim->relative_position_x << "," <<
        sm_ObjectTrim->relative_position_y << "," <<
        sm_ObjectTrim->absolute_velocity_x << "," <<
        sm_ObjectTrim->absolute_velocity_y << "," <<
        sm_ObjectTrim->absolute_acceleration_x << "," <<
        sm_ObjectTrim->absolute_acceleration_y << "," <<
        sm_ObjectTrim->absolute_yawrate << "," <<
        sm_ObjectTrim->size_width << "," <<
        // sm_ObjectTrim->size_length << "," <<

        sm_LaneInfo->lane_id << "," <<
        sm_LaneInfo->max_speed << "," <<
        sm_LaneInfo->min_speed << "," <<
        sm_LaneInfo->start_offset << "," <<
        sm_LaneInfo->end_offset << "," <<
        sm_LaneInfo->lane_merge_distance << "," <<
        sm_LaneInfo->lane_type << "," <<
        // sm_LaneInfo->reference_line << "," <<
        // sm_LaneInfo->odd_info << "," <<

        sm_LeftLine->rm_egolftlne_id << "," <<
        // sm_LeftLine->rm_egolftlne_geometry << "," <<
        sm_LeftLine->rm_egolftlne_color << "," <<
        sm_LeftLine->rm_egolftlne_type << "," <<
        sm_LeftLine->rm_egolftlne_edoutview << "," <<
        sm_LeftLine->rm_lftlnesnsrsource << "," <<

        sm_LeftRoadEdge->rm_rdedgelft_id << "," <<
        // sm_LeftRoadEdge->rm_rdedgelft_geometry << "," <<
        sm_LeftRoadEdge->rm_rdedgelft_edoutview << "," <<
        sm_LeftRoadEdge->rm_lftrdedgesnsrsource << "," <<

        sm_ODDInfo->valid << "," <<
        sm_ODDInfo->lane_id << "," <<
        sm_ODDInfo->type << "," <<
        sm_ODDInfo->sub_type << "," <<
        sm_ODDInfo->start_offset << "," <<
        sm_ODDInfo->end_offset << "," <<

        sm_RMEgoLftLnegeometry->c0_llne << "," <<
        sm_RMEgoLftLnegeometry->c1_llne << "," <<
        sm_RMEgoLftLnegeometry->c2_llne << "," <<
        sm_RMEgoLftLnegeometry->c3_llne << "," <<
        sm_RMEgoLftLnegeometry->start_rnge_llne << "," <<
        sm_RMEgoLftLnegeometry->end_rnge_llne << "," <<
        sm_RMEgoLftLnegeometry->probability_llne << "," <<
        sm_RMEgoLftLnegeometry->valid_llne << "," <<

        sm_RMEgoRgtLnegeometry->c0_rlne << "," <<
        sm_RMEgoRgtLnegeometry->c1_rlne << "," <<
        sm_RMEgoRgtLnegeometry->c2_rlne << "," <<
        sm_RMEgoRgtLnegeometry->c3_rlne << "," <<
        sm_RMEgoRgtLnegeometry->start_rnge_rlne << "," <<
        sm_RMEgoRgtLnegeometry->end_rnge_rlne << "," <<
        sm_RMEgoRgtLnegeometry->probability_rlne << "," <<
        sm_RMEgoRgtLnegeometry->valid_rlne << "," <<

        sm_RMRdEdgeLftgeometry->c0_lrdedge << "," <<
        sm_RMRdEdgeLftgeometry->c1_lrdedge << "," <<
        sm_RMRdEdgeLftgeometry->c2_lrdedge << "," <<
        sm_RMRdEdgeLftgeometry->c3_lrdedge << "," <<
        sm_RMRdEdgeLftgeometry->start_rnge_lrdedge << "," <<
        sm_RMRdEdgeLftgeometry->end_rnge_lrdedge << "," <<
        sm_RMRdEdgeLftgeometry->probability_lrdedge << "," <<
        sm_RMRdEdgeLftgeometry->valid_lrdedge << "," <<

        sm_RMRdEdgeRgtgeometry->c0_rrdedge << "," <<
        sm_RMRdEdgeRgtgeometry->c1_rrdedge << "," <<
        sm_RMRdEdgeRgtgeometry->c2_rrdedge << "," <<
        sm_RMRdEdgeRgtgeometry->c3_rrdedge << "," <<
        sm_RMRdEdgeRgtgeometry->start_rnge_rrdedge << "," <<
        sm_RMRdEdgeRgtgeometry->end_rnge_rrdedge << "," <<
        sm_RMRdEdgeRgtgeometry->probability_rrdedge << "," <<
        sm_RMRdEdgeRgtgeometry->valid_rrdedge << "," <<

        // sm_RMSMMessage->rm_header << "," <<
        // sm_RMSMMessage->rm_crrdr_id << "," <<
        // sm_RMSMMessage->rm_featuretag << "," <<
        // sm_RMSMMessage->rm_dsblrsn << "," <<
        // sm_RMSMMessage->rm_ilc_target << "," <<
        // sm_RMSMMessage->left_line << "," <<
        // sm_RMSMMessage->right_line << "," <<
        // sm_RMSMMessage->left_lane << "," <<
        // sm_RMSMMessage->ego_lane << "," <<
        // sm_RMSMMessage->right_lane << "," <<
        // sm_RMSMMessage->left_roadedge << "," <<
        // sm_RMSMMessage->right_roadedge << "," <<
        // sm_RMSMMessage->float_reserve << "," <<
        // sm_RMSMMessage->double_reserve << "," <<
        sm_RMSMMessage->rm_lne_crss << "," <<
        sm_RMSMMessage->rm_lanewidth << "," <<
        sm_RMSMMessage->control_way_point_target_road_type << "," <<
        sm_RMSMMessage->spd_lmt_id01 << "," <<
        sm_RMSMMessage->control_way_point_speed_limit01 << "," <<
        sm_RMSMMessage->control_way_point_offset01 << "," <<
        sm_RMSMMessage->spd_lmt_id02 << "," <<
        sm_RMSMMessage->control_way_point_speed_limit02 << "," <<
        sm_RMSMMessage->control_way_point_offset02 << "," <<
        sm_RMSMMessage->reminder_way_point_targe_road_type << "," <<
        sm_RMSMMessage->reminder_way_point_offset << "," <<
        sm_RMSMMessage->remain_distance_major_path << "," <<
        sm_RMSMMessage->remain_distance_major_path_type << "," <<
        sm_RMSMMessage->camr_line_mapping << "," <<
        sm_RMSMMessage->location_qulity << "," <<
        sm_RMSMMessage->enter_ramp_dist << "," <<
        sm_RMSMMessage->merge_main_road_dist << "," <<
        sm_RMSMMessage->inavlts << "," <<

        sm_RightLine->rm_egorgtlne_id << "," <<
        // sm_RightLine->rm_egorgtlne_geometry << "," <<
        sm_RightLine->rm_egorgtlne_color << "," <<
        sm_RightLine->rm_egorgtlne_type << "," <<
        sm_RightLine->rm_egorgtlne_edoutview << "," <<
        sm_RightLine->rm_rgtlnesnsrsource << "," <<

        sm_RightRoadEdge->rm_rdedgergt_id << "," <<
        // sm_RightRoadEdge->rm_rdedgergt_geometry << "," <<
        sm_RightRoadEdge->rm_rdedgergt_edoutview << "," <<
        sm_RightRoadEdge->rm_rgtrdedgesnsrsource << "," <<

        provide->sm_lfcmfrtavlbhvr << "," <<
        provide->sm_lfcmfrtavlbhvr_crdrid << "," <<
        provide->sm_lcavlbhvr << "," <<
        provide->sm_lcavlbhvr_crdrid << "," <<
        provide->sm_lcpavlbhvr << "," <<
        provide->sm_lcpavlbhvr_crdrid << "," <<
        provide->sm_lssavlbhvr << "," <<
        provide->sm_lgtavlbhvr << "," <<
        provide->sm_sftystp << "," <<
        provide->sm_latactvfunc << "," <<
        provide->sm_drvmd << "," <<
        provide->sm_nrp_lcavl << "," <<
        provide->sm_drvr_indctdrct << "," <<
        provide->sm_drvrsetspd << "," <<
        provide->sm_seltdistlvl << "," <<
        provide->sm_fdout_strgy_lat << "," <<
        provide->sm_fdin_strgy_lat << "," <<
        provide->sm_fdout_strgy_lgt << "," <<
        provide->sm_fdin_strgy_lgt << "," <<
        provide->sm_spdlmtid01 << "," <<
        provide->sm_speedlimit01 << "," <<
        provide->sm_controlwaypoint_offset01 << "," <<
        provide->sm_spdlmtid02 << "," <<
        provide->sm_speedlimit02 << "," <<
        provide->sm_controlwaypoint_offset02 << "," <<

        "\n";
}

std::string LogFileNameDated(const std::string& inputName) {
    time_t now = time(0);
    tm* localTime = localtime(&now);

    int year = 1900 + localTime->tm_year;
    int month = 1 + localTime->tm_mon;
    int day = localTime->tm_mday;
    int hour = localTime->tm_hour;
    int min = localTime->tm_min;
    int sec = localTime->tm_sec;

    std::string fileName = inputName + "_" + std::to_string(year) + "_" +
                           std::to_string(month) + "_" + std::to_string(day) + "_" +
                           std::to_string(hour) + "_" + std::to_string(min) + "_" +
                           std::to_string(sec) + ".csv";

    return fileName;
}

double getCurrentTimeStamp() {
    auto current_time = std::chrono::system_clock::now();
    auto timestamp = std::chrono::duration_cast<std::chrono::seconds>(current_time.time_since_epoch());
    uint32_t stamp_sec = static_cast<uint32_t>(timestamp.count());
    uint32_t stamp_nanosec = static_cast<uint32_t>((current_time.time_since_epoch() - timestamp).count());
    double CurrentTimeStamp = stamp_sec + (stamp_nanosec * 1e-9);
    return CurrentTimeStamp;
    }
