#include <stddef.h>
#include <stdio.h>

#include <thread>
#include <chrono>
#include <fstream>
#include <sstream>

#include "TRPCLongWrapper.hpp"
#include "TRPCLong_GeneralWrapper.h"
#include "dpc_trpc_long/executable.h"

#ifdef __cplusplus
extern "C"
{
#endif

// C header here
#include "TRPC_Long_Itg.h"

#ifdef __cplusplus
}
#endif

typedef enum {
 IPC_Target_Motor_En,       // uint8        --> AccVlcMainReq = 1 
 IPC_Target_Gear,           // uint8        --> AccVlcMainReq = 1 -> D, else -> N
 IPC_Target_Mode,           // uint8        --> Set as 1
 IPC_Life_Signal,           // uint8        --> Counter 0-15
 IPC_Target_Speed,          // float32      --> Vx_des_tar
 IPC_Target_SpdValid,       // uint8        --> Set as 1
 IPC_Target_Brk_Press,      // double       --> Ax_tar < 0 and VlcDCEnable == 1
 IPC_Target_Angle,          // float32      --> EPSRoadWheelReqValue
 IPC_Target_Steering_En,    // uint8        --> EPSRoadWheelReqSts
 IPC_Target_Torque,         // double       --> MProp
 IPC_Torque_Rpm_Mode,       // uint8        --> Set 1  
 IPC_TurnLight,             // uint8        --> Brake pressure and DC enable
 IPC_Target_SpdMode         // uint8        --> based on Vx_des_tar value range
} PendingSignals;

struct TRPCLongSWC2CANStruct {
    SWC2CAN ControlMode;
    uint8_t control_mode;

    SWC2CAN TargetGear;
    uint8_t target_gear_value;

    SWC2CAN DriveMotorEnable;
    uint8_t drive_motor_enable;

    SWC2CAN TorqueOrRPMMode;
    uint8_t torque_or_rpm_mode;

    SWC2CAN SpeedMode;
    uint8_t speed_mode;

    SWC2CAN TargerSpeed;
    float Vx_des_tar;

    SWC2CAN TargerSpeedValid;
    uint8_t TargerSpeedValid_value;

    SWC2CAN TargetTorque;
    double MProp;

    SWC2CAN BrakePressure;
    double brake_pressure_command;

    SWC2CAN TurnLight;
    uint8_t turn_signal;

    SWC2CAN LifeCycle;
    uint8_t life_cycle;
};

std::string LogFileNameDated(const std::string& inputName);
void logTRPCLongData(std::ofstream *ptr_File, dpc::VED4DPC* const ptr_trpclong_data_from_VED, 
                    dpc::Decision4Long* const  ptr_trpclong_data_from_DM,
                    dpc::TargetsInfo* const ptr_trpclong_data_from_EM_TargetsInfo, 
                    dpc::TargetInfo* const ptr_trpclong_data_from_EM_TargetInfo, 
                    dpc::RoadEdgeInfo* const ptr_trpclong_data_from_Em_RoadEdgeInfo, 
                    dpc::LaneMarkerInfo* const ptr_trpclong_data_from_EM_LaneMarkerInf,
                    TRPCLongSWC2CANStruct* const ptr_trpclong_data_to_CAN
					);
double getCurrentTimeStamp();
void assignTRPCLongInputsFromBAandEM(const dpc::Decision4Long data_From_DM_4_TRPCLong,
                                     const dpc::TargetsInfo data_From_EMTargetsInfo_4_TRPCLong,
                                     const dpc::TargetInfo data_From_EMTargetInfo_4_TRPCLong,
                                     const dpc::RoadEdgeInfo data_From_EMRoadEdgeInfo_4_TRPCLong, 
                                     const dpc::LaneMarkerInfo data_Form_EMLaneMarkerInfo_4_TRPCLong) {                                                          
  TRPC_Long_Itg_U.ba_trp_v_sv_setspd_in =  data_From_DM_4_TRPCLong.ba_trp_v_sv_setspd;      
  TRPC_Long_Itg_U.ba_trp_indx_trp_lgt_sts_in = data_From_DM_4_TRPCLong.ba_trp_indx_trp_lgt_sts;
  TRPC_Long_Itg_U.ba_trp_a_tgt_longt_accel_iv_in = data_From_DM_4_TRPCLong.ba_trp_a_tgt_longt_accel_iv;                              
  TRPC_Long_Itg_U.ba_trp_indx_tgt_id_iv_in = data_From_DM_4_TRPCLong.ba_trp_indx_tgt_id_iv;
  TRPC_Long_Itg_U.ba_trp_v_tgt_longt_spd_iv_in = data_From_DM_4_TRPCLong.ba_trp_v_tgt_longt_spd_iv;
  TRPC_Long_Itg_U.ba_trp_d_tgt_longt_dist_iv_in = data_From_DM_4_TRPCLong.ba_trp_d_tgt_longt_dist_iv;
  TRPC_Long_Itg_U.ba_trp_indx_spdlim_typ_in = data_From_DM_4_TRPCLong.ba_trp_indx_spdlim_typ;
  TRPC_Long_Itg_U.ba_trp_mps_spdlim_act_val_in = data_From_DM_4_TRPCLong.ba_trp_kmh_spdlim_act_val/3.6;
  TRPC_Long_Itg_U.ba_trp_d_spdlim_nxt_dst_in = data_From_DM_4_TRPCLong.ba_trp_d_spdlim_nxt_dst;
  TRPC_Long_Itg_U.ba_trp_indx_opt_action_in = data_From_DM_4_TRPCLong.ba_trp_indx_opt_action;

  TRPC_Long_Itg_U.rm_iv_indx_tgt_id = data_From_EMTargetInfo_4_TRPCLong.indx_tgtid; // Assumed the assignment from TargetInfo is OK
  TRPC_Long_Itg_U.rm_iv_d_tgtLongtdist = data_From_EMTargetInfo_4_TRPCLong.d_tgtlongtdist;
  TRPC_Long_Itg_U.rm_iv_v_tgtlongtspd = data_From_EMTargetInfo_4_TRPCLong.v_tgtlongtspd;
  TRPC_Long_Itg_U.rm_iv_a_tgtlongtaccel = data_From_EMTargetInfo_4_TRPCLong.a_tgtlongtaccel;
}

void assignTRPCLongInputsFromVED(const dpc::VED4DPC data_FromVED4TRPCLong) {
    uint8_t _is_standstill = 0;
    if (data_FromVED4TRPCLong.MotionState == 3) //check whether 3 is correct or not
    {
        _is_standstill = 1;
    }
    TRPC_Long_Itg_U.ivehsdslsts_in = _is_standstill; 
    TRPC_Long_Itg_U.v_ego_mps = data_FromVED4TRPCLong.LongitudinalVelocity;  
    TRPC_Long_Itg_U.a_ego = data_FromVED4TRPCLong.LongitudinalAcceleration;
    TRPC_Long_Itg_U.AxSlopeDyn = 0.; // If required, we can calculate it using IMU data
    TRPC_Long_Itg_U.SpdActl_Esti_mps_e = data_FromVED4TRPCLong.LongitudinalVelocity;
}

void assignTRPCLongInputsFromCAN(const CAN2SWC canData) {
    TRPC_Long_Itg_U.istrgwhlang_in = canData.VCU_Info_1_Signals.current_angle;   
    TRPC_Long_Itg_U.AccelPedal = 0.;
    TRPC_Long_Itg_U.iwhlbrkprssts = 2;
    TRPC_Long_Itg_U.MPropAct = canData.VCU_Info_1_Signals.motor_torque;
    TRPC_Long_Itg_U.MPropMin = 0.;
    TRPC_Long_Itg_U.MPropMax = 1000.;
    
}




uint8_T TRPC_Long_Run_Counter = 0;

const double TRPCLong_LogFileDuration = 120; // seconds

int main() {
    dpc::VED4DPC trpclong_data_from_VED;
    dpc::Decision4Long  trpclong_data_from_DM;
    dpc::TargetsInfo trpclong_data_from_EM_TargetsInfo;
    dpc::TargetInfo trpclong_data_from_EM_TargetInfo;
    dpc::RoadEdgeInfo trpclong_data_from_Em_RoadEdgeInfo;
    dpc::LaneMarkerInfo trpclong_data_from_EM_LaneMarkerInfo;
    CAN2SWC CAN_data_4_TRPCLong;

    PendingSignals TargerSpeed_enum = IPC_Target_Speed;
    SWC2CAN TargerSpeed;

    PendingSignals TargerSpeedValid_enum = IPC_Target_SpdValid;
    SWC2CAN TargerSpeedValid;

    PendingSignals TargetTorque_enum = IPC_Target_Torque;
    SWC2CAN TargetTorque;

    PendingSignals BrakePressure_enum = IPC_Target_Brk_Press;
    SWC2CAN BrakePressure;

    PendingSignals ControlMode_enum = IPC_Target_Mode;
    SWC2CAN ControlMode;

    PendingSignals TargetGear_enum = IPC_Target_Gear;
    SWC2CAN TargetGear;

    PendingSignals DriveMotorEnable_enum = IPC_Target_Motor_En;
    SWC2CAN DriveMotorEnable;

    PendingSignals SpeedMode_enum = IPC_Target_SpdMode;
    SWC2CAN SpeedMode;

    PendingSignals LifeCycle_enum = IPC_Life_Signal;
    SWC2CAN LifeCycle;
    uint8_t life_cycle = 0;

    PendingSignals TorqueOrRPMMode_enum = IPC_Torque_Rpm_Mode;
    SWC2CAN TorqueOrRPMMode;

    PendingSignals TurnLight_enum = IPC_TurnLight;
    SWC2CAN TurnLight;
    
    TRPCLongSWC2CANStruct LongToCANstruct4log;
    
    std::ofstream TRPCLongLogFile;
    TRPCLongLogFile.open(LogFileNameDated("TRPCLongLogFile"), std::ios::out);

    uint32_t TRPCLong_run_counter = 0;

    double TRPCLong_dataLoggingTimerStart = getCurrentTimeStamp();

    ara::log::InitLogging("DPC", "EM_TEST", ara::log::LogLevel::kVerbose, (ara::log::LogMode::kConsole | ara::log::LogMode::kRemote));

	ara::log::Logger &mainLog { ara::log::CreateLogger("main", "em sample context", ara::log::LogLevel::kVerbose) };

	mainLog.LogInfo() << "Let's produce some trpc long data...";

	GeneralWrapper gw(mainLog);

    while (1)
    {
        if (TRPCLong_run_counter == 0) {
            TRPC_Long_Itg_initialize();
        }
        else {
            gw._VED2TRPCLong_Required.s.GetData(trpclong_data_from_VED);
            gw._DM2TRPCLong_Required.s.GetData(trpclong_data_from_DM);
            gw._EM2TRPCLongTargetsInfo_Required.s.GetData(trpclong_data_from_EM_TargetsInfo);
            gw._EM2TRPCLongTargetInfo_Required.s.GetData(trpclong_data_from_EM_TargetInfo);
            gw._EM2TRPCLongRoadEdgeInfo_Required.s.GetData(trpclong_data_from_Em_RoadEdgeInfo);
            gw._EM2TRPCLongLaneMarkerInfo_Required.s.GetData(trpclong_data_from_EM_LaneMarkerInfo);
            gw._CAN2Long_Required.s.GetData(CAN_data_4_TRPCLong);
            
            // TODO: Control data sync --> Must have the same arival time
            assignTRPCLongInputsFromCAN(CAN_data_4_TRPCLong);
            assignTRPCLongInputsFromVED(trpclong_data_from_VED);
            assignTRPCLongInputsFromBAandEM(trpclong_data_from_DM, 
                                            trpclong_data_from_EM_TargetsInfo, 
                                            trpclong_data_from_EM_TargetInfo, 
                                            trpclong_data_from_Em_RoadEdgeInfo, 
                                            trpclong_data_from_EM_LaneMarkerInfo);

            TRPC_Long_Itg_step();
        }

        // TODO publish data

        ControlMode.signal = (UInt8)(ControlMode_enum);
        uint8_t control_mode = 1;
        // memcpy(&(ControlMode.value), &(control_mode), 
        //                         sizeof(ControlMode.value));
        ControlMode.value = static_cast<uint64_t>(control_mode);
        gw._TRPCLong2CAN_Provide.s.SetData(ControlMode);
        gw._TRPCLong2CAN_Provide.Publish();
        LongToCANstruct4log.ControlMode = ControlMode;
        LongToCANstruct4log.control_mode = control_mode;

        TargetGear.signal = (UInt8)(TargetGear_enum);
        DriveMotorEnable.signal = (UInt8)(DriveMotorEnable_enum);
        uint8_t target_gear_value = 0;
        uint8_t drive_motor_enable = 0;
        if (TRPC_Long_Itg_Y.AccVlcMainReq == 1) {
            target_gear_value = 1;
            drive_motor_enable = 1;
        }
        // memcpy(&(TargetGear.value), &(target_gear_value), 
        //                         sizeof(TargetGear.value));
        TargetGear.value = static_cast<uint64_t>(target_gear_value);
        // memcpy(&(DriveMotorEnable.value), &(drive_motor_enable), 
        //                         sizeof(DriveMotorEnable.value));
        DriveMotorEnable.value = static_cast<uint64_t>(drive_motor_enable);
        gw._TRPCLong2CAN_Provide.s.SetData(TargetGear);
        gw._TRPCLong2CAN_Provide.Publish();
        LongToCANstruct4log.TargetGear = TargetGear;
        LongToCANstruct4log.target_gear_value = target_gear_value;
        gw._TRPCLong2CAN_Provide.s.SetData(DriveMotorEnable);
        gw._TRPCLong2CAN_Provide.Publish();
        LongToCANstruct4log.DriveMotorEnable = DriveMotorEnable;
        LongToCANstruct4log.drive_motor_enable = drive_motor_enable;

        TorqueOrRPMMode.signal = (UInt8)(TorqueOrRPMMode_enum);
        uint8_t torque_or_rpm_mode = 0;
        // memcpy(&(TorqueOrRPMMode.value), &(torque_or_rpm_mode), 
        //                         sizeof(TorqueOrRPMMode.value));
        TorqueOrRPMMode.value = static_cast<uint64_t>(torque_or_rpm_mode);
        gw._TRPCLong2CAN_Provide.s.SetData(TorqueOrRPMMode);
        gw._TRPCLong2CAN_Provide.Publish();
        LongToCANstruct4log.TorqueOrRPMMode = TorqueOrRPMMode;
        LongToCANstruct4log.torque_or_rpm_mode = torque_or_rpm_mode;
        
        SpeedMode.signal = (UInt8)(SpeedMode_enum);
        uint8_t speed_mode = 0;
        if (TRPC_Long_Itg_Y.Vx_des_tar >= 0. && TRPC_Long_Itg_Y.Vx_des_tar < 5.) {speed_mode = 0;}
        else if (TRPC_Long_Itg_Y.Vx_des_tar >= 5. && TRPC_Long_Itg_Y.Vx_des_tar < 10.) {speed_mode = 1;}
        else {speed_mode = 2;}
        // memcpy(&(SpeedMode.value), &(speed_mode), 
        //                         sizeof(SpeedMode.value));
        SpeedMode.value = static_cast<uint64_t>(speed_mode);
        gw._TRPCLong2CAN_Provide.s.SetData(SpeedMode);
        gw._TRPCLong2CAN_Provide.Publish();
        LongToCANstruct4log.SpeedMode = SpeedMode;
        LongToCANstruct4log.speed_mode = speed_mode;

        TargerSpeed.signal = (UInt8)(TargerSpeed_enum);
        double Vx_des_tar_double = static_cast<double>(TRPC_Long_Itg_Y.Vx_des_tar);
        memcpy(&(TargerSpeed.value), &(Vx_des_tar_double), 
                                sizeof(TargerSpeed.value));
        gw._TRPCLong2CAN_Provide.s.SetData(TargerSpeed);
        gw._TRPCLong2CAN_Provide.Publish();
        LongToCANstruct4log.TargerSpeed = TargerSpeed;
        LongToCANstruct4log.Vx_des_tar = TRPC_Long_Itg_Y.Vx_des_tar;

        TargerSpeedValid.signal = (UInt8)(TargerSpeedValid_enum);
        uint8_t TargerSpeedValid_value = 1; // Faked it
        // memcpy(&(TargerSpeedValid.value), &(TargerSpeedValid_value), 
        //                         sizeof(TargerSpeedValid.value));
        TargerSpeedValid.value = static_cast<uint64_t>(TargerSpeedValid_value);
        gw._TRPCLong2CAN_Provide.s.SetData(TargerSpeedValid);
        gw._TRPCLong2CAN_Provide.Publish();
        LongToCANstruct4log.TargerSpeedValid = TargerSpeedValid;
        LongToCANstruct4log.TargerSpeedValid_value = TargerSpeedValid_value;

        TargetTorque.signal = (UInt8)(TargetTorque_enum);
        memcpy(&(TargetTorque.value), &(TRPC_Long_Itg_Y.MProp), 
                                sizeof(TargetTorque.value));
        gw._TRPCLong2CAN_Provide.s.SetData(TargetTorque);
        gw._TRPCLong2CAN_Provide.Publish();
        LongToCANstruct4log.TargetTorque = TargetTorque;
        LongToCANstruct4log.MProp = TRPC_Long_Itg_Y.MProp;

        BrakePressure.signal = (UInt8)(BrakePressure_enum);
        TurnLight.signal = (UInt8)(TurnLight_enum);
        uint8_t turn_signal = 0;
        double param_decel_2_pressure = 10.;  // Can be tuned later!
        double brake_pressure_command = 0.;
        if (TRPC_Long_Itg_Y.VlcDCEnable == 1 && TRPC_Long_Itg_Y.Ax_tar < 0) {
            brake_pressure_command = static_cast<double>(TRPC_Long_Itg_Y.Ax_tar) * param_decel_2_pressure;
            turn_signal = 3;
        }
        memcpy(&(BrakePressure.value), &(brake_pressure_command), 
                                sizeof(BrakePressure.value));
        gw._TRPCLong2CAN_Provide.s.SetData(BrakePressure);
        gw._TRPCLong2CAN_Provide.Publish();
        LongToCANstruct4log.BrakePressure = BrakePressure;
        LongToCANstruct4log.brake_pressure_command = brake_pressure_command;


        // memcpy(&(TurnLight.value), &(turn_signal), 
        //                     sizeof(TurnLight.value));
        TurnLight.value = static_cast<uint64_t>(turn_signal);
        gw._TRPCLong2CAN_Provide.s.SetData(TurnLight);
        gw._TRPCLong2CAN_Provide.Publish();
        LongToCANstruct4log.TurnLight = TurnLight;
        LongToCANstruct4log.turn_signal = turn_signal;


        LifeCycle.signal = (UInt8)(LifeCycle_enum);
        // memcpy(&(LifeCycle.value), &(life_cycle), 
        //                         sizeof(LifeCycle.value));
        LifeCycle.value = static_cast<uint64_t>(life_cycle);
        gw._TRPCLong2CAN_Provide.s.SetData(LifeCycle);
        gw._TRPCLong2CAN_Provide.Publish();
        LongToCANstruct4log.LifeCycle = LifeCycle;
        LongToCANstruct4log.life_cycle = life_cycle;
        life_cycle++;
        if(life_cycle>15) {
            life_cycle = 0; 
        }              


        TRPCLong_run_counter++;

        double TRPCLong_dataLoggingTimerNow = getCurrentTimeStamp();
        if ((TRPCLong_dataLoggingTimerNow - TRPCLong_dataLoggingTimerStart)<= TRPCLong_LogFileDuration) {
            logTRPCLongData(&TRPCLongLogFile, &trpclong_data_from_VED, &trpclong_data_from_DM,
                             &trpclong_data_from_EM_TargetsInfo, &trpclong_data_from_EM_TargetInfo, 
                             &trpclong_data_from_Em_RoadEdgeInfo, &trpclong_data_from_EM_LaneMarkerInfo,
                             &LongToCANstruct4log);
        }
        else {
            TRPCLongLogFile.close();
        }
        // TODO, check the trpclong cycle time
        std::this_thread::sleep_for(std::chrono::milliseconds(10)); // TRPCLong cycle time: ? ms
    }
    

    return 0;
}

std::string LogFileNameDated(const std::string& inputName) {
    time_t now = time(0);
    tm* localTime = localtime(&now);

    int year = 1900 + localTime->tm_year;
    int month = 1 + localTime->tm_mon;
    int day = localTime->tm_mday;
    int hour = localTime->tm_hour;
    int min = localTime->tm_min;
    int sec = localTime->tm_sec;

    std::string fileName = inputName + "_" + std::to_string(year) + "_" +
                           std::to_string(month) + "_" + std::to_string(day) + "_" +
                           std::to_string(hour) + "_" + std::to_string(min) + "_" +
                           std::to_string(sec) + ".csv";

    return fileName;
}

// TODO log data
void logTRPCLongData(std::ofstream *ptr_File, dpc::VED4DPC* const ptr_trpclong_data_from_VED, 
                    dpc::Decision4Long* const  ptr_trpclong_data_from_DM,
                    dpc::TargetsInfo* const ptr_trpclong_data_from_EM_TargetsInfo, 
                    dpc::TargetInfo* const ptr_trpclong_data_from_EM_TargetInfo, 
                    dpc::RoadEdgeInfo* const ptr_trpclong_data_from_Em_RoadEdgeInfo, 
                    dpc::LaneMarkerInfo* const ptr_trpclong_data_from_EM_LaneMarkerInf,
                    TRPCLongSWC2CANStruct* const ptr_trpclong_data_to_CAN
					){
                        *ptr_File << getCurrentTimeStamp() << ",";

                        //Log main received data From VED
                        // *ptr_File << ptr_trpclong_data_from_VED->  // todo, add ivehsdslsts to log
                        *ptr_File << ptr_trpclong_data_from_VED->LongitudinalVelocity << ",";
                        *ptr_File << ptr_trpclong_data_from_VED->LongitudinalAcceleration << ",";
                        *ptr_File << ptr_trpclong_data_from_VED->LateralVelocity << ","; //here we lat and lon acc seperated, do we need the overal acc?
                        *ptr_File << ptr_trpclong_data_from_VED->LateralAcceleration << ",";
                        *ptr_File << ptr_trpclong_data_from_VED->MotionState << ",";
                        *ptr_File << ptr_trpclong_data_from_VED->SteeringAngle << ",";
                        *ptr_File << ptr_trpclong_data_from_VED->SteeringRate << ",";
                        *ptr_File << ptr_trpclong_data_from_VED->Yawrate << ",";
                        
                        //Log main received data from DM
                        *ptr_File << ptr_trpclong_data_from_DM->ba_trp_v_sv_setspd << ",";
                        *ptr_File << ptr_trpclong_data_from_DM->ba_trp_indx_trp_lgt_sts << ",";
                        *ptr_File << ptr_trpclong_data_from_DM->ba_trp_a_tgt_longt_accel_iv << ",";
                        *ptr_File << ptr_trpclong_data_from_DM->ba_trp_indx_tgt_id_iv << ",";
                        *ptr_File << ptr_trpclong_data_from_DM->ba_trp_v_tgt_longt_spd_iv << ",";
                        *ptr_File << ptr_trpclong_data_from_DM->ba_trp_d_tgt_longt_dist_iv << ",";
                        *ptr_File << ptr_trpclong_data_from_DM->ba_trp_indx_spdlim_typ << ",";
                        *ptr_File << ptr_trpclong_data_from_DM->ba_trp_kmh_spdlim_act_val << ",";
                        *ptr_File << ptr_trpclong_data_from_DM->ba_trp_d_spdlim_nxt_dst << ",";
                        *ptr_File << ptr_trpclong_data_from_DM->ba_trp_indx_opt_action << ",";

                        //Log main received data from EM
                        *ptr_File << ptr_trpclong_data_from_EM_TargetInfo->indx_tgtid << ",";
                        *ptr_File << ptr_trpclong_data_from_EM_TargetInfo->d_tgtlongtdist << ",";
                        *ptr_File << ptr_trpclong_data_from_EM_TargetInfo->v_tgtlongtspd << ",";
                        *ptr_File << ptr_trpclong_data_from_EM_TargetInfo->a_tgtlongtaccel << ",";

                        //Log the Can Command from TRPCLon 
                        *ptr_File << ptr_trpclong_data_to_CAN->ControlMode.signal<< ",";
                        *ptr_File << ptr_trpclong_data_to_CAN->ControlMode.value<< ",";
                        *ptr_File << ptr_trpclong_data_to_CAN->control_mode<< ",";

                        *ptr_File << ptr_trpclong_data_to_CAN->TargetGear.signal<< ",";
                        *ptr_File << ptr_trpclong_data_to_CAN->TargetGear.value<< ",";
                        *ptr_File << ptr_trpclong_data_to_CAN->target_gear_value<< ",";

                        *ptr_File << ptr_trpclong_data_to_CAN->DriveMotorEnable.signal<< ",";
                        *ptr_File << ptr_trpclong_data_to_CAN->DriveMotorEnable.value<< ",";
                        *ptr_File << ptr_trpclong_data_to_CAN->drive_motor_enable<< ",";

                        *ptr_File << ptr_trpclong_data_to_CAN->TorqueOrRPMMode.signal<< ",";
                        *ptr_File << ptr_trpclong_data_to_CAN->TorqueOrRPMMode.value<< ",";
                        *ptr_File << ptr_trpclong_data_to_CAN->torque_or_rpm_mode<< ",";

                        *ptr_File << ptr_trpclong_data_to_CAN->SpeedMode.signal<< ",";
                        *ptr_File << ptr_trpclong_data_to_CAN->SpeedMode.value<< ",";
                        *ptr_File << ptr_trpclong_data_to_CAN->speed_mode<< ",";

                        *ptr_File << ptr_trpclong_data_to_CAN->TargerSpeed.signal<< ",";
                        *ptr_File << ptr_trpclong_data_to_CAN->TargerSpeed.value<< ",";
                        *ptr_File << ptr_trpclong_data_to_CAN->Vx_des_tar<< ",";

                        *ptr_File << ptr_trpclong_data_to_CAN->TargerSpeedValid.signal<< ",";
                        *ptr_File << ptr_trpclong_data_to_CAN->TargerSpeedValid.value<< ",";
                        *ptr_File << ptr_trpclong_data_to_CAN->TargerSpeedValid_value<< ",";

                        *ptr_File << ptr_trpclong_data_to_CAN->TargetTorque.signal<< ",";
                        *ptr_File << ptr_trpclong_data_to_CAN->TargetTorque.value<< ",";
                        *ptr_File << ptr_trpclong_data_to_CAN->MProp<< ",";

                        *ptr_File << ptr_trpclong_data_to_CAN->BrakePressure.signal<< ",";
                        *ptr_File << ptr_trpclong_data_to_CAN->BrakePressure.value<< ",";
                        *ptr_File << ptr_trpclong_data_to_CAN->brake_pressure_command<< ",";

                        *ptr_File << ptr_trpclong_data_to_CAN->TurnLight.signal<< ",";
                        *ptr_File << ptr_trpclong_data_to_CAN->TurnLight.value<< ",";
                        *ptr_File << ptr_trpclong_data_to_CAN->turn_signal<< ",";

                        *ptr_File << ptr_trpclong_data_to_CAN->LifeCycle.signal<< ",";
                        *ptr_File << ptr_trpclong_data_to_CAN->LifeCycle.value<< ",";
                        *ptr_File << ptr_trpclong_data_to_CAN->life_cycle<< ",";

                        //Go to the next line
                        *ptr_File << "\n";
                    }

double getCurrentTimeStamp() {
    auto current_time = std::chrono::system_clock::now();
    auto timestamp = std::chrono::duration_cast<std::chrono::seconds>(current_time.time_since_epoch());
    uint32_t stamp_sec = static_cast<uint32_t>(timestamp.count());
    uint32_t stamp_nanosec = static_cast<uint32_t>((current_time.time_since_epoch() - timestamp).count());
    double CurrentTimeStamp = stamp_sec + (stamp_nanosec * 1e-9);
    return CurrentTimeStamp;
    }
