#include <iostream>
#include <vector> 
#include <map>
#include <thread>
#include <chrono>
#include <fstream>
#include <sstream>

using namespace std;
#ifdef __cplusplus
#include "VED_Rte_Wrapper.h"
#include "ved_ext.h"
#include "connected_data.h"
#include "time_handler.h"
#include "ara/log/logging.h"
#include "ara/exec/execution_client.h"
#include "pub/PubWrapperHelper.h"
#include "pub/PubDataWrapper.h"
#include "VED_PubDataWrapper.h"
#include "dpc_ved/executable.h"
// #include "impl_type_ved4dpc.h"
#include "impl_type_can2swc.h"
#include "VED_GeneralWrapper.h"
extern "C" {
#endif

#ifdef __cplusplus
}
#endif

#define VED_EXEX_CANIN_STATUS_MAP(status) (status == 0 ? 1 : 0)


// added for wrapper
//GENERATE_PUB_WRAPPER(VEDPubWrapper, VEDPubDataWrapper, DPCServiceSkeleton, VED4DPC, VED4DPCEvent);

unsigned long long timeStampUs=0;

void get_VED_outputs(dpc::VED4DPC* const vedData, proVEDPrtList_t* const proPortsData);
void set_VED_inputs(reqVEDPrtList_t* const reqPortsData, CAN2SWC const CanCallbackData);
void set_VED_params(reqVEDParams_t* const reqParamsValues);
std::string LogFileNameDated(const std::string& inputName);
void logVEDData(std::ofstream *ptr_File, CAN2SWC* const ptrcan2ved_data,
                dpc::VED4DPC* const ptr_vedGWData, proVEDPrtList_t* const ptr_proPortsData, 
                reqVEDPrtList_t* ptr_reqPortsData, reqVEDParams_t* ptr_reqParamsData);
double getCurrentTimeStamp();

void get_data_for_fusion(dpc::VED4FusionEgoMotion &pub_data_object, proVEDPrtList_t* const proPortsData);

const double VED_LogFileDuration = 120; // seconds


int main(int argc, char **argv) {

	reqVEDPrtList_t* ptr_reqPorts;
	reqVEDParams_t* ptr_reqParams;
	proVEDPrtList_t* ptr_proPorts;
	dpc::VED4DPC ved_data; // Executable Pub data
    dpc::VED4FusionEgoMotion ved_data_4_fusion; // Executable Pub data
    CAN2SWC can2ved_data; // Executable Sub data

    std::ofstream VEDLogFile;
    VEDLogFile.open(LogFileNameDated("VEDLogFile"), std::ios::out);

	uint32_t VED_run_counter = 0;

    double VED_dataLoggingTimerStart = getCurrentTimeStamp();

    ara::log::InitLogging("DPC", "EM_TEST", ara::log::LogLevel::kVerbose, (ara::log::LogMode::kConsole | ara::log::LogMode::kRemote));

	ara::log::Logger &mainLog { ara::log::CreateLogger("main", "em sample context", ara::log::LogLevel::kVerbose) };

	mainLog.LogInfo() << "Let's produce some trpc long data...";

	GeneralWrapper gw(mainLog);

    while (1){

        if (VED_run_counter == 0) {

            VED_Initialize();
			ptr_reqPorts = p_VED_in();
			ptr_proPorts = p_VED_out();
        }
        else {
            gw._CAN2VED_Required.s.GetData(can2ved_data);
            set_VED_inputs(ptr_reqPorts, can2ved_data);
			VED_StepExec();
        }
        
        get_VED_outputs(&ved_data, ptr_proPorts);
        gw._VED2DPC_Provide.s.SetData(ved_data);
        gw._VED2DPC_Provide.Publish();

        get_data_for_fusion(ved_data_4_fusion, ptr_proPorts);
        gw._VED2FusionEgoMotion_Provide.s.SetData(ved_data_4_fusion);
        gw._VED2FusionEgoMotion_Provide.Publish();


        VED_run_counter++;

        double VED_dataLoggingTimerNow = getCurrentTimeStamp();
        if ((VED_dataLoggingTimerNow - VED_dataLoggingTimerStart)<= VED_LogFileDuration) {
            logVEDData(&VEDLogFile, &can2ved_data, &ved_data, ptr_proPorts, ptr_reqPorts, ptr_reqParams);
        }
        else {
            VEDLogFile.close();
        }
        
        std::this_thread::sleep_for(std::chrono::milliseconds(20)); // VED cycle time: 20 ms
    }

    return 0;
}

void get_VED_outputs(dpc::VED4DPC* const vedData, proVEDPrtList_t* const proPortsData) {
	vedData->LongitudinalVelocity = proPortsData->pVehicleDynamicSignals->Longitudinal.VeloCorr.corrVelo;
	vedData->LongitudinalAcceleration = proPortsData->pVehicleDynamicSignals->Longitudinal.AccelCorr.corrAccel;
	vedData->LateralVelocity =  0.0;
	vedData->LateralAcceleration = proPortsData->pVehicleDynamicSignals->Lateral.Accel.LatAccel;
	vedData->MotionState = proPortsData->pVehicleDynamicSignals->MotionState.MotState; 
    // VED_LONG_MOT_STATE_MOVE 0U, VED_LONG_MOT_STATE_MOVE_FWD 1U, 
    // VED_LONG_MOT_STATE_MOVE_RWD 2U, VED_LONG_MOT_STATE_STDST 3U
	vedData->Yawrate = proPortsData->pVehicleDynamicSignals->Lateral.YawRate.YawRate;
    vedData->SteeringAngle = proPortsData->pOriginVehicleSignals->VehSigMain.StWheelAngle;
    vedData->SteeringRate = proPortsData->pVehicleDynamicSignals->Lateral.fStWheelAngleGradient_rps;
}

void set_VED_inputs(reqVEDPrtList_t* const reqPortsData, CAN2SWC const CanCallbackData) {
    // We must set inputs based on the data provided in calback function
	
	// Mule CAN
	reqPortsData->pVehicleInputSignals->VehSigMain.YawRate = CanCallbackData.IMU_BIAS_CORRECTED_ANGRATE_Signals.imu_bias_corrected_ang_rate_z; // IMU_BIAS_CORRECTED_angRateZ
	reqPortsData->pVehicleInputSignals->VehSigMain.StWheelAngle = CanCallbackData.VCU_Info_1_Signals.current_angle/57.3f; // Current_Angle
	reqPortsData->pVehicleInputSignals->VehSigMain.LatAccel = 0. - CanCallbackData.IMU_BIAS_CORRECTED_ACC_Signals.imu_bias_corrected_acc_y; // 0.0 - IMU_BIAS_CORRECTED_accY
	reqPortsData->pVehicleInputSignals->VehSigMain.WhlVelFrLeft = CanCallbackData.VCU_Info_1_Signals.current_speed; // Current_Speed
    reqPortsData->pVehicleInputSignals->VehSigMain.WhlVelFrRight = CanCallbackData.VCU_Info_1_Signals.current_speed; // Current_Speed
    reqPortsData->pVehicleInputSignals->VehSigMain.WhlVelReLeft = CanCallbackData.VCU_Info_1_Signals.current_speed; // Current_Speed
    reqPortsData->pVehicleInputSignals->VehSigMain.WhlVelReRight = CanCallbackData.VCU_Info_1_Signals.current_speed; // Current_Speed
	reqPortsData->pVehicleInputSignals->VehSigMain.VehVelocityExt = CanCallbackData.VCU_Info_1_Signals.current_speed; // Current_Speed
    reqPortsData->pVehicleInputSignals->VehSigMain.VehLongAccelExt = CanCallbackData.IMU_BIAS_CORRECTED_ACC_Signals.imu_bias_corrected_acc_x; // IMU_BIAS_CORRECTED_accX
	// Mule whee dir: 0-Still, 1-Forward, 2-Reverse, 3-Reserved but in EP40: 1 forward, 2 backward (0 standstill,  3 not identified)
	uint8 mule_motor_direction = CanCallbackData.VCU_Info_2_Signals.motor_direction; // Motor_Direction
    reqPortsData->pVehicleInputSignals->VehSigMain.WhlDirFrLeft = mule_motor_direction;
	reqPortsData->pVehicleInputSignals->VehSigMain.WhlDirFrRight = mule_motor_direction;
	reqPortsData->pVehicleInputSignals->VehSigMain.WhlDirReLeft = mule_motor_direction;
	reqPortsData->pVehicleInputSignals->VehSigMain.WhlDirReRight = mule_motor_direction;
	// ActGearPos
    // VED:0_N/1_P/2_D/3_R/10_invalid
    // Mule: 0-N gear, 1-D gear, 2-R gear, 3-reserved
    // EP40:0_default/1_P/2_R/3_N/4_D/5_S/7_Invalid
	uint8 mule_current_gear = CanCallbackData.VCU_Info_1_Signals.current_gear; // Current_Gear
    switch (mule_current_gear) {
       case 0:
           reqPortsData->pVehicleInputSignals->VehSigMain.ActGearPos = 0u;
           break;
       case 1:
           reqPortsData->pVehicleInputSignals->VehSigMain.ActGearPos = 2u;
           break;
       case 2:
           reqPortsData->pVehicleInputSignals->VehSigMain.ActGearPos = 3u;
           break;
       case 3:
           reqPortsData->pVehicleInputSignals->VehSigMain.ActGearPos = 10u;
           break;
       default:
           reqPortsData->pVehicleInputSignals->VehSigMain.ActGearPos = 0u;
           break;
   }
	uint8 mule_SES_En_Sts = CanCallbackData.VCU_Info_1_Signals.ses_en_sts; //SES_En_Sts Steering motor enable status: 0-Not En, 1-En
	reqPortsData->pVehicleInputSignals->VehSigMain.State[2] = VED_EXEX_CANIN_STATUS_MAP(mule_SES_En_Sts);

	uint8 mule_Motor_En_Sts = CanCallbackData.VCU_Info_1_Signals.motor_en_sts; // Motor_En_Sts Drive motor enable status: 0-Not En, 1-En
	reqPortsData->pVehicleInputSignals->VehSigMain.State[12] = VED_EXEX_CANIN_STATUS_MAP(mule_Motor_En_Sts);

    reqPortsData->pVehicleInputSignals->VehSigMain.ParkBrakeState = 0u; // No available signal
    
}

void set_VED_params(reqVEDParams_t* const reqParamsValues) {
	reqParamsValues->SteeringRatio.swa.ang[0] = 0.0f;
	reqParamsValues->SteeringRatio.swa.ang[1] = 1000.0f;
	reqParamsValues->SteeringRatio.swa.rat[0] = 14.52000046f;
	reqParamsValues->SteeringRatio.swa.rat[1] = 17.0f;
	reqParamsValues->SteeringRatio.vel.vel[0] = 0.0f;
	reqParamsValues->SteeringRatio.vel.vel[1] = 0.0f;
	reqParamsValues->SteeringRatio.vel.rat[0] = 14.52000046f;
	reqParamsValues->SteeringRatio.vel.rat[1] = 14.52000046f;
	reqParamsValues->VED_Kf_SelfSteerGrad_nu = 0.003734f;
	reqParamsValues->VED_Kf_CntrOfGravHeight_met = 0.63422f;
	reqParamsValues->VED_Kf_WhlLoadDepFrontAxle_nu = 3.6329999f;
	reqParamsValues->VED_Kf_WhlLoadDepRearAxle_nu = 2.92000008f;
	reqParamsValues->VED_Ku_WhlTcksPerRev_nu = 96.0f;
}



std::string LogFileNameDated(const std::string& inputName) {
    time_t now = time(0);
    tm* localTime = localtime(&now);

    int year = 1900 + localTime->tm_year;
    int month = 1 + localTime->tm_mon;
    int day = localTime->tm_mday;
    int hour = localTime->tm_hour;
    int min = localTime->tm_min;
    int sec = localTime->tm_sec;

    std::string fileName = inputName + "_" + std::to_string(year) + "_" +
                           std::to_string(month) + "_" + std::to_string(day) + "_" +
                           std::to_string(hour) + "_" + std::to_string(min) + "_" +
                           std::to_string(sec) + ".csv";

    return fileName;
}

void logVEDData(std::ofstream *ptr_File, CAN2SWC* const ptrcan2ved_data,
                dpc::VED4DPC* const ptr_vedGWData, proVEDPrtList_t* const ptr_proPortsData, 
                reqVEDPrtList_t* ptr_reqPortsData, reqVEDParams_t* ptr_reqParamsData) {
    
   *ptr_File << getCurrentTimeStamp()<< ",";

   // Log main received data from CAN
   *ptr_File << ptrcan2ved_data->VCU_Info_1_Signals.current_angle << ",";
   *ptr_File << ptrcan2ved_data->VCU_Info_1_Signals.current_speed << ",";
   *ptr_File << ptrcan2ved_data->VCU_Info_2_Signals.motor_direction << ",";
   *ptr_File << ptrcan2ved_data->VCU_Info_1_Signals.current_gear << ",";
   *ptr_File << ptrcan2ved_data->VCU_Info_1_Signals.ses_en_sts << ",";
   *ptr_File << ptrcan2ved_data->VCU_Info_1_Signals.motor_en_sts << ",";
   *ptr_File << ptrcan2ved_data->IMU_BIAS_CORRECTED_ACC_Signals.imu_bias_corrected_acc_x << ",";
   *ptr_File << ptrcan2ved_data->IMU_BIAS_CORRECTED_ACC_Signals.imu_bias_corrected_acc_y << ",";
   *ptr_File << ptrcan2ved_data->IMU_BIAS_CORRECTED_ANGRATE_Signals.imu_bias_corrected_ang_rate_z << ",";


   // Log main published data by VED
   *ptr_File << ptr_vedGWData->LongitudinalVelocity<< ",";
   *ptr_File << ptr_vedGWData->LongitudinalAcceleration<< ",";
   *ptr_File << ptr_vedGWData->LateralVelocity<< ",";
   *ptr_File << ptr_vedGWData->LateralAcceleration<< ",";
   *ptr_File << ptr_vedGWData->MotionState<< ",";
   *ptr_File << ptr_vedGWData->Yawrate<< ",";
   *ptr_File << ptr_vedGWData->SteeringAngle<< ",";
   *ptr_File << ptr_vedGWData->SteeringRate<< ",";

   // Go to the next line
   *ptr_File << "\n";
}

double getCurrentTimeStamp() {
    auto current_time = std::chrono::system_clock::now();
    auto timestamp = std::chrono::duration_cast<std::chrono::seconds>(current_time.time_since_epoch());
    uint32_t stamp_sec = static_cast<uint32_t>(timestamp.count());
    uint32_t stamp_nanosec = static_cast<uint32_t>((current_time.time_since_epoch() - timestamp).count());
    double CurrentTimeStamp = stamp_sec + (stamp_nanosec * 1e-9);
    return CurrentTimeStamp;
    }


void get_data_for_fusion(dpc::VED4FusionEgoMotion &pub_data_object, proVEDPrtList_t* const proPortsData){
	pub_data_object.LongAcc = proPortsData->pVehicleDynamicSignals->Longitudinal.AccelCorr.corrAccel;
    pub_data_object.LongVel = proPortsData->pVehicleDynamicSignals->Longitudinal.VeloCorr.corrVelo;
	pub_data_object.SigStatus =  true;  // Set it
	pub_data_object.LatYawRate = proPortsData->pVehicleDynamicSignals->Lateral.YawRate.YawRate;
    pub_data_object.TimeStamp = getCurrentTimeStamp();
}