/**
 * @Author: wzy
 * @Date: 2021-11-19
 * @LastEditTime: 
 * @LastEditors: Please set LastEditors
 * @Description: In User Settings Edit
 * @filePath:
 */

#include "PD_Comfort.h"
using namespace ADC::PD_Comfort;

Comfort::Comfort(const float pd_LatTrj_Lt_coefficient_d[6],
                 const std::vector<ADC::PD_EgoVehTrj::EgoVeh_TrjPnt>& EgoVeh_trace_list){

    if (init(pd_LatTrj_Lt_coefficient_d, EgoVeh_trace_list) == false){
        // std::cout << "ATTENTION: comfort init is false!" << std::endl;
        return;
    }
}

bool Comfort::init(const float pd_LatTrj_Lt_coefficient_d[6],
                   const std::vector<ADC::PD_EgoVehTrj::EgoVeh_TrjPnt>& EgoVeh_trace_list){
    if (EgoVeh_trace_list.empty()){
        initFlag = false;
        // std::cout << "ATTENTION: <Comfort> EgoVeh_trace_list ipos empty!" << std::endl;
    }
    else{
        initFlag = true;
        // _plantime = EgoVeh_trace_list.size() * timesampl;
        
        for (int i = 0; i < 6; i++){
            _LT_coefficient_d[i] = pd_LatTrj_Lt_coefficient_d[i];
        }
        PD_DynmcObjRsk::EgoVeh_Point egoveh_TrjPnt;
        for (int i = 0; i < EgoVeh_trace_list.size(); i++){
            egoveh_TrjPnt.t = EgoVeh_trace_list[i].t;
            egoveh_TrjPnt.s = EgoVeh_trace_list[i].s;
            egoveh_TrjPnt.l = EgoVeh_trace_list[i].l;
            egoveh_TrjPnt.vs = EgoVeh_trace_list[i].vx;
            egoveh_TrjPnt.as = EgoVeh_trace_list[i].ax;
            egoveh_TrjPnt.jks = EgoVeh_trace_list[i].jkx;
            _egoVeh_TrjPnt.push_back(egoveh_TrjPnt);
        }
    }
    return initFlag;
}

float Comfort::CalculateLatAccMax(){
    std::vector<float> cftlatacc;
    float tem_cftlatacc = 0.0f;
    for (int i = 0; i < _egoVeh_TrjPnt.size(); i++){
    // for (float t = 0.0; t <= _plantime; t += 0.1){
        tem_cftlatacc = 2 * _LT_coefficient_d[2]
                      + 6 * _LT_coefficient_d[3] * pow(0.1*i, 1)
                     + 12 * _LT_coefficient_d[4] * pow(0.1*i, 2)
                     + 20 * _LT_coefficient_d[5] * pow(0.1*i, 3);
        cftlatacc.push_back(tem_cftlatacc);
    }
    m_CftLatAccMax = *max_element(cftlatacc.begin(), cftlatacc.end());
    return m_CftLatAccMax;
}

float Comfort::CalculateLongAccMax(){
    std::vector<float> cftlongacc;
    for (int i = 0; i < _egoVeh_TrjPnt.size(); i++){
        cftlongacc.push_back(_egoVeh_TrjPnt[i].as);
    }
    m_CftLongAccMax = *max_element(cftlongacc.begin(), cftlongacc.end());
    return m_CftLongAccMax;
}

float Comfort::CalculateLatJerkMax(){
    std::vector<float> cftlataccjerk;
    float tem_cftlataccjerk = 0.0f;
    for (int i = 0; i < _egoVeh_TrjPnt.size(); i++){
    // for (float t = 0.0; t <= _plantime; t += 0.1){
        tem_cftlataccjerk = 6 * _LT_coefficient_d[3]
                         + 24 * _LT_coefficient_d[4] * pow(0.1*i, 1)
                         + 60 * _LT_coefficient_d[5] * pow(0.1*i, 2);
        cftlataccjerk.push_back(tem_cftlataccjerk);
    }
    m_CftLatAccJerkMax = *max_element(cftlataccjerk.begin(), cftlataccjerk.end());
    return m_CftLatAccJerkMax;
}

float Comfort::CalculateLongJerkMax(){
    std::vector<float> cftlongaccjerk;
    for (int i = 0; i < _egoVeh_TrjPnt.size(); i++){
        cftlongaccjerk.push_back(_egoVeh_TrjPnt[i].jks);
    }
    m_CftLongAccJerkMax = *max_element(cftlongaccjerk.begin(), cftlongaccjerk.end());
    return m_CftLongAccJerkMax;
}

// float Comfort::CalculateLatAccMin(){
//     std::vector<float> cftlatacc;
//     for (int i = 0; i < _egoVeh_TrjPnt.size(); i++){
//     // for (float t = 0.0; t <= _plantime; t += 0.1){
//         float tem_cftlatacc = 0.0f;
//         tem_cftlatacc = 2 * _LT_coefficient_d[2]
//                       + 6 * _LT_coefficient_d[3] * pow(0.1*i, 1)
//                      + 12 * _LT_coefficient_d[4] * pow(0.1*i, 2)
//                      + 20 * _LT_coefficient_d[5] * pow(0.1*i, 3);
//         cftlatacc.push_back(tem_cftlatacc);
//     }
//     m_CftLatAccMin = *min_element(cftlatacc.begin(), cftlatacc.end());
//     return m_CftLatAccMin;
// }

// float Comfort::CalculateLongAccMin(){
//     std::vector<float> cftlongacc;
//     for (int i = 0; i < _egoVeh_TrjPnt.size(); i++){
//         cftlongacc.push_back(_egoVeh_TrjPnt[i].as);
//     }
//     m_CftLongAccMin = *min_element(cftlongacc.begin(), cftlongacc.end());
//     return m_CftLongAccMin;
// }

// float Comfort::CalculateLatJerkMin(){
//     std::vector<float> cftlataccjerk;
//     for (int i = 0; i < _egoVeh_TrjPnt.size(); i++){
//     // for (float t = 0.0; t <= _plantime; t += 0.1){
//         float tem_cftlataccjerk = 0.0f;
//         tem_cftlataccjerk = 6 * _LT_coefficient_d[3]
//                          + 24 * _LT_coefficient_d[4] * pow(0.1*i, 1)
//                          + 60 * _LT_coefficient_d[5] * pow(0.1*i, 2);
//         cftlataccjerk.push_back(tem_cftlataccjerk);
//     }
//     m_CftLatAccJerkMin = *min_element(cftlataccjerk.begin(), cftlataccjerk.end());
//     return m_CftLatAccJerkMin;
// }

// float Comfort::CalculateLongJerkMin(){
//     std::vector<float> cftlongaccjerk;
//     for (int i = 0; i < _egoVeh_TrjPnt.size(); i++){
//         cftlongaccjerk.push_back(_egoVeh_TrjPnt[i].jks);
//     }
//     m_CftLongAccJerkMin = *min_element(cftlongaccjerk.begin(), cftlongaccjerk.end());
//     return m_CftLongAccJerkMin;
// }

void Comfort::process(){
    if (initFlag != true){
        // std::cout << "ATTENTION: Data reading of PD_Comfort is error" << std::endl;
        return;
    }
    //TJA赋默认值
    // m_CftLatAccMax = 0.0f;
    // m_CftLongAccMax = 0.0f;
    // m_CftLatAccJerkMax = 0.0f;
    // m_CftLongAccJerkMax = 0.0f;
    // m_CftLatAccMin = 0.0f;
    // m_CftLongAccMin = 0.0f;
    // m_CftLatAccJerkMin = 0.0f;
    // m_CftLongAccJerkMin = 0.0f;

    CalculateLatAccMax();
    CalculateLongAccMax();
    CalculateLatJerkMax();
    CalculateLongJerkMax();

    // CalculateLatAccMin();
    // CalculateLongAccMin();
    // CalculateLatJerkMin();
    // CalculateLongJerkMin();
}