/**
 * @Author: wzy
 * @Date: 2021-11-19
 * @LastEditTime: 
 * @LastEditors: Please set LastEditors
 * @Description: In User Settings Edit
 * @filePath:
 */

#pragma once

#include <iostream>
#include <vector>
#include <algorithm>
#include <cmath>

#include "PD_EgoVehStsArb.h"
#include "PD_DynamicObjRisk.h"

namespace ADC
{
namespace PD_Comfort
{

class Comfort
{
public:
    Comfort(const float pd_LatTrj_Lt_coefficient_d[6],
            const std::vector<ADC::PD_EgoVehTrj::EgoVeh_TrjPnt>& EgoVeh_trace_list);
    ~Comfort() = default;

private:
    bool init(const float pd_LatTrj_Lt_coefficient_d[6],
              const std::vector<ADC::PD_EgoVehTrj::EgoVeh_TrjPnt>& EgoVeh_trace_list);
    bool initFlag = true;

/**
  * @brief   Calculate the max acceleration of lateral
  * @param   
  * @return  the max acceleration of lateral
  */
    float CalculateLatAccMax();
/**
  * @brief   Calculate the max acceleration of longitudinal
  * @param   
  * @return  the max acceleration of longitudinal
  */
    float CalculateLongAccMax();
/**
  * @brief   Calculate the max jerk of lateral
  * @param   
  * @return  the max jerk of lateral
  */
    float CalculateLatJerkMax();
/**
  * @brief   Calculate the max jerk of longitudinal
  * @param   
  * @return  the max jerk of longitudinal
  */
    float CalculateLongJerkMax();

// /**
//   * @brief   Calculate the min acceleration of lateral
//   * @param   
//   * @return  the min acceleration of lateral
//   */
//     float CalculateLatAccMin();
// /**
//   * @brief   Calculate the min acceleration of longitudinal
//   * @param   
//   * @return  the min acceleration of longitudinal
//   */
//     float CalculateLongAccMin();
// /**
//   * @brief   Calculate the min jerk of lateral
//   * @param   
//   * @return  the min jerk of lateral
//   */
//     float CalculateLatJerkMin();
// /**
//   * @brief   Calculate the min jerk of longitudinal
//   * @param   
//   * @return  the min jerk of longitudinal
//   */
//     float CalculateLongJerkMin();

private:
    // float _plantime;
    // float timesampl = 0.1f;
    float _LT_coefficient_d[6];
    std::vector<ADC::PD_DynmcObjRsk::EgoVeh_Point> _egoVeh_TrjPnt;

public:
/**
  * @brief   Calculate the comfort
  * @return  the comfort
  */
    void process();
    float m_CftLatAccMax = 0.0f;
    float m_CftLongAccMax = 0.0f;
    float m_CftLatAccJerkMax = 0.0f;
    float m_CftLongAccJerkMax = 0.0f;

    // float m_CftLatAccMin = 0.0f;
    // float m_CftLongAccMin = 0.0f;
    // float m_CftLatAccJerkMin = 0.0f;
    // float m_CftLongAccJerkMin = 0.0f;
};

}   //namespace PD_Comfort
}   //namespace ADC
