/**
 * @Author: wzy
 * @Date: 2021-11-9
 * @LastEditTime: 
 * @LastEditors: Please set LastEditors
 * @Description: In User Settings Edit
 * @filePath:
 */

#include "PD_LegalPunishment.h"
using namespace ADC::PD_LegalPnsh;

LegalPunishment::LegalPunishment(const PD::PD_Input &pd_input, 
                                 const uint8_t pd_LatTrj_EV_ActionType,
                                 const std::vector<ADC::PD_EgoVehTrj::EgoVeh_TrjPnt>& EgoVeh_trace_list){
    if (init(pd_input, pd_LatTrj_EV_ActionType, EgoVeh_trace_list) == false){
        // std::cout << "ATTENTION: LegalPunishment init is false!" << std::endl;
        return;
    }
}

/**
 * @brief get data
 * @param pd_input input from RM
 * @param pd_LatTrj_EV_ActionType action flag from LatTrajectory
 * @param EgoVeh_trace_list ego vehicle trajectory point
 */
bool LegalPunishment::init(const PD::PD_Input &pd_input, 
                           const uint8_t pd_LatTrj_EV_ActionType,
                           const std::vector<ADC::PD_EgoVehTrj::EgoVeh_TrjPnt>& EgoVeh_trace_list){
    bool Flag = true;
    Flag = Flag && !(pd_input.rm_roadmodel.empty());     //check rm_roadmodel

    if(Flag == false){
        initFlag = false;
    }
    else{
        _left_lane_market_id_list.clear();
        _right_lane_market_id_list.clear();
        _left_line_type_attribute_list.clear();
        _right_line_type_attribute_list.clear();
        _left_front_lane_boundary_list.clear();
        _right_front_lane_boundary_list.clear();
        _egoVeh_TrjPnt.clear();
        initFlag = true;
        
        _ego_action_type = pd_LatTrj_EV_ActionType;

        _ego_lane_id = pd_input.rm_roadmodel.at(rm_Num).car_position.ego_lane_id;
        _lane_list = pd_input.rm_roadmodel.at(rm_Num).lane_list;

        for (auto &element : _lane_list){
            if (element.start_id == _ego_lane_id){
                _left_lane_market_id_list = element.left_lane_marker_id_list;
                _right_lane_market_id_list = element.right_lane_marker_id_list;
                break;
            }
        }
        
        _lane_marker_list = pd_input.rm_roadmodel.at(rm_Num).lane_marker_list;
        for (auto &element : _lane_marker_list){
            if (!(_left_lane_market_id_list.empty())){
                if (element.id == _left_lane_market_id_list.at(0)){
                    _left_line_type_attribute_list = element.line_type_attribute_list;
                    _left_front_lane_boundary_list = element.front_lane_boundary_list;
                    break;
                }
            }
        }
        for (auto &element : _lane_marker_list){
            if (!(_right_lane_market_id_list.empty())){
                if (element.id == _right_lane_market_id_list.at(0)){
                    _right_line_type_attribute_list = element.line_type_attribute_list;
                    _right_front_lane_boundary_list = element.front_lane_boundary_list;
                    break;
                }
            }
        }
        // bool Flag1 = false;
        // bool Flag2 = false;
        // for (auto &element : _lane_marker_list){
        //     if ((!(_left_lane_market_id_list.empty())) && (!(_right_lane_market_id_list.empty()))){
        //         if (element.id == _left_lane_market_id_list.at(0)){
        //             _left_front_lane_boundary_list = element.front_lane_boundary_list;
        //             Flag1 = true;
        //         }
        //         else if (element.id == _right_lane_market_id_list.at(0)){
        //             _right_front_lane_boundary_list = element.front_lane_boundary_list;
        //             Flag2 = true;
        //         }
        //         if (Flag1 && Flag2){
        //             break;
        //         }
        //     }
        // }

        bool flagFindCorridorID = false;
        uint32_t passage_corridor_ID = pd_input.rm_roadmodel.at(rm_Num).passage_list.at(psg_Num).corridor_id;
        for (auto& element : _lane_list){
            for (auto& element1 : element.corridor_list){
                if (element1.corridor_id == passage_corridor_ID){
                    _pos_ego_start = element1.reference_line.c0;
                    flagFindCorridorID = true;
                }
                if (flagFindCorridorID)
                {break;}
            }
            if (flagFindCorridorID)
            {break;}
        }

        PD_DynmcObjRsk::EgoVeh_Point egoveh_TrjPnt;
        for (int i = 0; i < EgoVeh_trace_list.size(); i++){
            egoveh_TrjPnt.t = EgoVeh_trace_list[i].t;
            egoveh_TrjPnt.s = EgoVeh_trace_list[i].s;
            egoveh_TrjPnt.l = EgoVeh_trace_list[i].l;
            egoveh_TrjPnt.vs = EgoVeh_trace_list[i].vx;
            egoveh_TrjPnt.as = EgoVeh_trace_list[i].ax;
            egoveh_TrjPnt.jks = EgoVeh_trace_list[i].jkx;
            _egoVeh_TrjPnt.push_back(egoveh_TrjPnt);
        }
    }
    return initFlag;
}

// TJA阶段置false
// bool LegalPunishment::calculateLegalPunishment(){
//     return false;   
// }

bool LegalPunishment::calculateLegalPunishment(){
    if (_lane_list.empty() || _lane_marker_list.empty() || _egoVeh_TrjPnt.empty()){
        return false;
    }
    else{
        float threshold = 0.5f;
        // float tem_line_start_range = 0.0f;
        // float tem_line_end_range = 0.0f;
        float tem_C0 = 0.0f;
        float tem_LongDis_C0 = 0.0f;
        bool PnshFlag1 = false;
        bool PnshFlag2 = false;

        //LC
        if (_ego_action_type == 2){
            if (_left_line_type_attribute_list.empty() || _left_front_lane_boundary_list.empty()){
                return false;
            }
            else{
                tem_C0 = _left_front_lane_boundary_list.at(0).line_geometry.c0;
                // if(_egoVeh_TrjPnt.empty()){
                //         std::cout << "ATTENTION: _egoVeh_TrjPnt is empty!" << std::endl;
                //     }
                for (int i = 1; i < _egoVeh_TrjPnt.size(); i++){
                    if ((_pos_ego_start + _egoVeh_TrjPnt[i].l) > tem_C0){
                        tem_LongDis_C0 = _egoVeh_TrjPnt[i].s;
                        break;
                    }
                    // 插值
                    // if ((_pos_ego_start + _egoVeh_TrjPnt[i-1].l) < tem_C0 && (_pos_ego_start + _egoVeh_TrjPnt[i+1].l) > tem_C0){
                    //     tem_LongDis_C0 = (_egoVeh_TrjPnt[i+1].s - _egoVeh_TrjPnt[i-1].s) 
                    //                    / (_egoVeh_TrjPnt[i+1].l - _egoVeh_TrjPnt[i-1].l)
                    //                    * (-tem_C0 - _egoVeh_TrjPnt[i-1].l) + _egoVeh_TrjPnt[i-1].s;
                    //     break;
                    // }
                }
                for(auto &ele : _left_line_type_attribute_list){
                    // if(_egoVeh_TrjPnt.empty()){
                    //         std::cout << "ATTENTION: _egoVeh_TrjPnt is empty!" << std::endl;
                    //     }
                    if ((_egoVeh_TrjPnt.at(0).s >= (ele.start_offset - 1.0)) && (_egoVeh_TrjPnt.at(0).s < ele.end_offset)){
                    //     if (ele.linetype == 1 || ele.linetype == 5){
                    //         if ((ele.end_offset - _egoVeh_TrjPnt.front().s) > threshold * _egoVeh_TrjPnt.front().vs){
                    //             return true;
                    //         }
                    //     }
                        if ((ele.linetype == 1 || ele.linetype == 5) && ((ele.end_offset - _egoVeh_TrjPnt.front().s) > threshold * _egoVeh_TrjPnt.front().vs)){
                            PnshFlag1 = true;
                        }
                        else{
                            PnshFlag1 = false;
                        }
                        break;
                    }
                    // else{
                    //     PnshFlag1 = false;
                    // }
                }
                for(auto &ele : _left_line_type_attribute_list){
                    if ((tem_LongDis_C0 > ele.start_offset) && (tem_LongDis_C0 < ele.end_offset)){

                        #if FLAG_LGPNSH_IOOUT != 0
                        _linerange = ele.end_offset;
                        #endif
                        
                        if((ele.linetype == 1) || (ele.linetype == 5)){
                            PnshFlag2 = true;
                        }
                        else{
                            PnshFlag2 = false;
                        }
                        break;
                    }
                }
                if (PnshFlag1 || PnshFlag2){
                    return true;
                }
                else{
                    return false;
                }
            }
        }
        //RC
        else if (_ego_action_type == 3){
            if (_right_line_type_attribute_list.empty() || _right_front_lane_boundary_list.empty()){
                return false;
            }
            else{
                tem_C0 = _right_front_lane_boundary_list.at(0).line_geometry.c0;
                for (int i = 1; i < _egoVeh_TrjPnt.size(); i++){
                    if ((-_pos_ego_start - _egoVeh_TrjPnt[i].l) > (-tem_C0)){
                        tem_LongDis_C0 = _egoVeh_TrjPnt[i].s;
                        break;
                    }
                    // 插值
                    // if ((-_pos_ego_start - _egoVeh_TrjPnt[i-1].l) < (-tem_C0) && (-_pos_ego_start - _egoVeh_TrjPnt[i+1].l) > (-tem_C0)){
                    //     tem_LongDis_C0 = (_egoVeh_TrjPnt[i+1].s - _egoVeh_TrjPnt[i-1].s) 
                    //                    / (_egoVeh_TrjPnt[i-1].l - _egoVeh_TrjPnt[i+1].l)
                    //                    * (_egoVeh_TrjPnt[i-1].l + tem_C0) + _egoVeh_TrjPnt[i-1].s;
                    //     break;
                    // }
                }
                for(auto &ele : _right_line_type_attribute_list){
                    if ((_egoVeh_TrjPnt.front().s >= ele.start_offset - 1.0) && (_egoVeh_TrjPnt.front().s < ele.end_offset)){
                        if ((ele.linetype == 1 || ele.linetype == 5) && ((ele.end_offset - _egoVeh_TrjPnt.front().s) > threshold * _egoVeh_TrjPnt.front().vs)){
                            PnshFlag1 = true;
                        }
                        else{
                            PnshFlag1 = false;
                        }
                        break;
                    }
                }
                for(auto &ele : _right_line_type_attribute_list){
                    if ((tem_LongDis_C0 > ele.start_offset) && (tem_LongDis_C0 < ele.end_offset)){

                        #if FLAG_LGPNSH_IOOUT != 0
                        _linerange = ele.end_offset;
                        #endif

                        if((ele.linetype == 1) || (ele.linetype == 5)){
                            PnshFlag2 = true;
                        }
                        else{
                            PnshFlag2 = false;
                        }
                        break;
                    }
                }
                if (PnshFlag1 || PnshFlag2){
                    return true;
                }
                else{
                    return false;
                }
            }
        }
        else{
            return false;       
        }

        #if FLAG_LGPNSH_IOOUT != 0
        _LineType = _left_line_type_attribute_list.at(0).line_type;
        _carpos = tem_LongDis_C0;
        #endif

    } 
}

void LegalPunishment::process(){
    if (initFlag != true){
        // std::cout << "ATTENTION: Data reading of PD_LegalPunishment is error" << std::endl;
        return;
    }
    m_PnshSldnene = calculateLegalPunishment();

    #if FLAG_LGPNSH_IOOUT != 0
    std::ofstream ofs;
    struct timeval time;
    gettimeofday(&time, NULL);
    ofs.open("./src/mpc_drv_ctAp_ba/core/source/PD_module/LogFile/Draw/Draw_PnshSldnene.txt", std::ios::app);
    ofs << "action: " << int(_ego_action_type) <<", PnshSldene: " << int(m_PnshSldnene) << ", linetype: " << int(_LineType) 
        << ", linerange: " << _linerange << ", carpos: " << _carpos 
        << ", time: " << time.tv_sec << "." << time.tv_usec << std::endl;
    ofs.close();
    #endif

}

