/**
 * @file
 * @brief  add 2021-11-22
 *        instruction manual: include the head file in prediction.h;
 *                              in prediction.cpp function process():  
 */



#include "PD_RoadModelVector_PDtoBA.h"
using namespace ADC::PD_RoadModelVector_PDtoBA;


#if FLAG_PDFP_SAVECSV!=0
std::string timeStampBag;
#endif

/**
 * @brief
 * @param
 */
RoadModelVector_PDtoBA::RoadModelVector_PDtoBA(const PD::PD_Input &m_pd_input,
                            const dpc::Index &m_pd_updatecorridor_next_NodeID, 
                            const std::vector<ADC::PD_EgoVehTrj::EgoVeh_TrjPnt> &m_egoveh_tp,
                            const std::vector<ADC::PD_NodeUpdate::PD_NodeHVInfo> &m_nodeHVinfo){
    if(init(m_pd_input,m_pd_updatecorridor_next_NodeID,m_egoveh_tp,m_nodeHVinfo) == false)
    return;
}


/**
 * @brief 
 * @param
 * @return
 */
bool RoadModelVector_PDtoBA::init(const PD::PD_Input &m_pd_input,
                            const dpc::Index &m_pd_updatecorridor_next_NodeID, 
                            const std::vector<ADC::PD_EgoVehTrj::EgoVeh_TrjPnt> &m_egoveh_tp,
                            const std::vector<ADC::PD_NodeUpdate::PD_NodeHVInfo> &m_nodeHVinfo){
    bool Flag=true;
    Flag = Flag && (static_cast<decltype(m_pd_input.rm_roadmodel.size())>(rm_Num)<m_pd_input.rm_roadmodel.size());                                 //chech rm_roadmodel
    if(Flag){
        Flag = Flag && (static_cast<decltype(m_pd_input.rm_roadmodel.at(rm_Num).passage_list.size())>(psg_Num)<m_pd_input.rm_roadmodel.at(rm_Num).passage_list.size());    //chech passage_list
    }
    // Flag = Flag && !(m_pd_input.rm_roadmodel.empty());                //chech rm_roadmodel
    // Flag = Flag && !(m_egoveh_tp.empty());                          //chech egoveh_tp

    pd_RoadModelOutput_invalid = false;
    if(Flag==false){
        initFlag=false;
        pd_RoadModelOutput_invalid = true;
    }
    else{

        initFlag=true;
        _pd_input_last = m_pd_input;
        // _pd_last_RoadModelVector = m_pd_input.rm_roadmodel;
        _pd_UpdateCorridor_index = m_pd_updatecorridor_next_NodeID;
        if(m_egoveh_tp.empty()){
            _EV_end_s = 0.0;
            _EV_end_l = 0.0;
            _EV_delta_l = 0.0;
            _EV_end_vs = 0.0;
            _EV_end_vl = 0.0;
        }
        else{
            _EV_end_s = m_egoveh_tp.back().s;
            _EV_end_l = m_egoveh_tp.back().l;
            _EV_delta_l = m_egoveh_tp.back().l - m_egoveh_tp.front().l;
            _EV_end_vs = m_egoveh_tp.back().vx;
            _EV_end_vl = m_egoveh_tp.back().vy;
        }
        _EV_end_laneID = m_nodeHVinfo.empty() ? 0 : m_nodeHVinfo.back().locationID;


        // estimate availability of input
        bool flagInputInvalid = false;
        bool flagFindCorridorID = false;
        for(auto& element : _pd_input_last.rm_roadmodel.at(rm_Num).lane_list){
            for(auto& element2 : element.corridor_list){
                if(element2.corridor_id == _pd_input_last.rm_roadmodel.at(rm_Num).passage_list.at(psg_Num).corridor_id){
                    flagInputInvalid = flagInputInvalid || (fabs(element2.reference_line.c0) > 2000.0f);
                    flagFindCorridorID = true;
                }
                if(flagFindCorridorID)
                {break;}
            }
            if(flagFindCorridorID)
            {break;}
        }
        if(flagInputInvalid){
            pd_RoadModelOutput_invalid = true;
            initFlag = false;
        }

        // 20220729 reduce the times of calculations
        #if FLAG_PDFP_SAVECSV == 0
            int x_meanful_updateRoadModelVector = -1;
            if(ADC::common::k_BA_TotalDurationTimeThreshold < 3.0f)         x_meanful_updateRoadModelVector=-1;
            else if(ADC::common::k_BA_TotalDurationTimeThreshold < 5.0f)    x_meanful_updateRoadModelVector=0;
            else if(ADC::common::k_BA_TotalDurationTimeThreshold < 7.0f)    x_meanful_updateRoadModelVector=1;
            else if(ADC::common::k_BA_TotalDurationTimeThreshold < 9.0f)    x_meanful_updateRoadModelVector=2;
            else if(ADC::common::k_BA_TotalDurationTimeThreshold < 11.0f)   x_meanful_updateRoadModelVector=3;
            else if(ADC::common::k_BA_TotalDurationTimeThreshold < 13.0f)   x_meanful_updateRoadModelVector=4;
            else if(ADC::common::k_BA_TotalDurationTimeThreshold < 15.0f)   x_meanful_updateRoadModelVector=5;
            else if(ADC::common::k_BA_TotalDurationTimeThreshold < 17.0f)   x_meanful_updateRoadModelVector=6;
            else if(ADC::common::k_BA_TotalDurationTimeThreshold < 19.0f)   x_meanful_updateRoadModelVector=7;
            else if(ADC::common::k_BA_TotalDurationTimeThreshold < 21.0f)   x_meanful_updateRoadModelVector=8;
            else if(ADC::common::k_BA_TotalDurationTimeThreshold < 23.0f)   x_meanful_updateRoadModelVector=9;
            else x_meanful_updateRoadModelVector=(static_cast<int>(ADC::common::k_BA_TotalDurationTimeThreshold)-3)/2;
            if(static_cast<int>(_pd_input_last.rm_roadmodel.at(rm_Num).road_id.x) > x_meanful_updateRoadModelVector){
                pd_RoadModelOutput_invalid = pd_RoadModelOutput_invalid && false;
                initFlag = initFlag && false;
            }
        #endif

    }

    return initFlag;
}





/**
 * @brief
 * @param
 */
void RoadModelVector_PDtoBA::process(){

    if(initFlag != true){
        // std::cout<< "ATTENTION: Data reading of RoadModeVector_PDtoBA error" <<std::endl;
        return;
    }
#if FLAG_PDRM_HEAP == 0
    // road id
    ADC::PD_RoadModel_RoadID_PDtoBA::RoadModel_RoadID_PDtoBA _roadModel_RoadID_PDtoBA(_pd_UpdateCorridor_index);
    _roadModel_RoadID_PDtoBA.process();
    
    // car_position
    ADC::PD_RoadModel_CarPosition_PDtoBA::RoadModel_CarPosition_PDtoBA _roadModel_CarPosition_PDtoBA(_pd_input_last,_EV_end_s,_EV_end_l,_EV_end_laneID);
    _roadModel_CarPosition_PDtoBA.process();
    
    // lane marker list
    ADC::PD_RoadModel_LaneMarkerList_PDtoBA::RoadModel_LaneMarkerList_PDtoBA _roadModel_LaneMarkerList_PDtoBA(_pd_input_last,_EV_end_s,_EV_end_l,_EV_end_vs,_EV_end_vl);
    #if FLAG_PDRM_THREAD == 0
    _roadModel_LaneMarkerList_PDtoBA.process();
    #else
    std::thread thread_laneboun(&ADC::PD_RoadModel_LaneMarkerList_PDtoBA::RoadModel_LaneMarkerList_PDtoBA::process(), _roadModel_LaneMarkerList_PDtoBA);
    thread_laneboun.join();
    #endif

    // lane list
    ADC::PD_RoadModel_LaneList_PDtoBA::RoadModel_LaneList_PDtoBA _roadModel_LaneList_PDtoBA(_pd_input_last,_EV_end_s,_EV_end_l,_EV_end_vs,_EV_end_vl,_roadModel_LaneMarkerList_PDtoBA.pd_RoadModel_LaneMarkerList_output);
    #if FLAG_PDRM_THREAD == 0
    _roadModel_LaneList_PDtoBA.process();
    #else
    std::thread thread_lanelist(&ADC::PD_RoadModel_LaneList_PDtoBA::RoadModel_LaneList_PDtoBA::process(), _roadModel_LaneList_PDtoBA);
    thread_lanelist.join();
    #endif

    // object list
    ADC::PD_RoadModel_ObjectList_PDtoBA::RoadModel_ObjectList_PDtoBA _roadModel_ObjectList_PDtoBA(_pd_input_last,_EV_end_s,_EV_end_l,_EV_end_vs,_EV_end_vl);
    #if FLAG_PDRM_THREAD == 0
    _roadModel_ObjectList_PDtoBA.process();
    #else
    std::thread thread_obj(&ADC::PD_RoadModel_ObjectList_PDtoBA::RoadModel_ObjectList_PDtoBA::process(), _roadModel_ObjectList_PDtoBA);
    thread_obj.join();
    #endif

    // road edge list
    ADC::PD_RoadModel_RoadEdge_PDtoBA::RoadModel_RoadEdge_PDtoBA _roadModel_RoadEdge_PDtoBA(_pd_input_last,_EV_end_s,_EV_end_l,_EV_end_vs,_EV_end_vl);
    #if FLAG_PDRM_THREAD == 0
    _roadModel_RoadEdge_PDtoBA.process();
    #else
    std::thread thread_roadedge(&ADC::PD_RoadModel_RoadEdge_PDtoBA::RoadModel_RoadEdge_PDtoBA::process(), _roadModel_RoadEdge_PDtoBA);
    thread_roadedge.join();
    #endif

    // passage list


    // collect
    RoadInfo m_roadInfo;
    m_roadInfo.road_id              =   _roadModel_RoadID_PDtoBA.pd_RoadModel_RoadID_output;
    m_roadInfo.car_position         =   _roadModel_CarPosition_PDtoBA.PD_RoadModel_CarPosition_output;
    m_roadInfo.lane_list            =   _roadModel_LaneList_PDtoBA.pd_RoadModel_LaneList_output;
    m_roadInfo.lane_marker_list     =   _roadModel_LaneMarkerList_PDtoBA.pd_RoadModel_LaneMarkerList_output;
    m_roadInfo.object_list          =   _roadModel_ObjectList_PDtoBA.pd_RoadModel_ObjectList_output;
    m_roadInfo.road_edge_list       =   _roadModel_RoadEdge_PDtoBA.pd_RoadModel_RoadEdge_output;
    m_roadInfo.passage_list  =  _pd_input_last.rm_roadmodel.at(rm_Num).passage_list;//.clear();
#else   //FLAG_PDRM_HEAP
    // road id
    std::unique_ptr<ADC::PD_RoadModel_RoadID_PDtoBA::RoadModel_RoadID_PDtoBA> \
        _roadModel_RoadID_PDtoBA(new ADC::PD_RoadModel_RoadID_PDtoBA::RoadModel_RoadID_PDtoBA(_pd_UpdateCorridor_index));
    
    // car_position
    std::unique_ptr<ADC::PD_RoadModel_CarPosition_PDtoBA::RoadModel_CarPosition_PDtoBA> \
        _roadModel_CarPosition_PDtoBA(new ADC::PD_RoadModel_CarPosition_PDtoBA::RoadModel_CarPosition_PDtoBA(_pd_input_last,_EV_end_s,_EV_end_l,_EV_end_laneID));

    // lane marker list
    std::unique_ptr<ADC::PD_RoadModel_LaneMarkerList_PDtoBA::RoadModel_LaneMarkerList_PDtoBA> \
        _roadModel_LaneMarkerList_PDtoBA(new ADC::PD_RoadModel_LaneMarkerList_PDtoBA::RoadModel_LaneMarkerList_PDtoBA(_pd_input_last,_EV_end_s,_EV_end_l,_EV_end_vs,_EV_end_vl));
    
    // lane list
    std::unique_ptr<ADC::PD_RoadModel_LaneList_PDtoBA::RoadModel_LaneList_PDtoBA> \
        _roadModel_LaneList_PDtoBA(new ADC::PD_RoadModel_LaneList_PDtoBA::RoadModel_LaneList_PDtoBA(_pd_input_last,_EV_end_s,_EV_end_l,_EV_end_vs,_EV_end_vl,_roadModel_LaneMarkerList_PDtoBA->pd_RoadModel_LaneMarkerList_output));

    // object list
    std::unique_ptr<ADC::PD_RoadModel_ObjectList_PDtoBA::RoadModel_ObjectList_PDtoBA> \
        _roadModel_ObjectList_PDtoBA(new ADC::PD_RoadModel_ObjectList_PDtoBA::RoadModel_ObjectList_PDtoBA(_pd_input_last,_EV_end_s,_EV_end_l,_EV_end_vs,_EV_end_vl));

    // road edge list
    std::unique_ptr<ADC::PD_RoadModel_RoadEdge_PDtoBA::RoadModel_RoadEdge_PDtoBA> \
        _roadModel_RoadEdge_PDtoBA(new ADC::PD_RoadModel_RoadEdge_PDtoBA::RoadModel_RoadEdge_PDtoBA(_pd_input_last,_EV_end_s,_EV_end_l,_EV_end_vs,_EV_end_vl));
    
    // passage list


    // collect
    RoadInfo m_roadInfo;

    bool flagNotNull = true;
    if(!(_roadModel_RoadID_PDtoBA))             flagNotNull = flagNotNull && false;
    if(!(_roadModel_CarPosition_PDtoBA))        flagNotNull = flagNotNull && false;
    if(!(_roadModel_LaneList_PDtoBA))           flagNotNull = flagNotNull && false;
    if(!(_roadModel_LaneMarkerList_PDtoBA))     flagNotNull = flagNotNull && false;
    if(!(_roadModel_ObjectList_PDtoBA))         flagNotNull = flagNotNull && false;
    if(!(_roadModel_RoadEdge_PDtoBA))           flagNotNull = flagNotNull && false;
    if(flagNotNull){
        _roadModel_RoadID_PDtoBA->process();
        _roadModel_CarPosition_PDtoBA->process();
        _roadModel_LaneList_PDtoBA->process();
        _roadModel_LaneMarkerList_PDtoBA->process();
        _roadModel_ObjectList_PDtoBA->process();
        _roadModel_RoadEdge_PDtoBA->process();

        m_roadInfo.road_id              =   _roadModel_RoadID_PDtoBA->pd_RoadModel_RoadID_output;
        m_roadInfo.car_position         =   _roadModel_CarPosition_PDtoBA->PD_RoadModel_CarPosition_output;
        m_roadInfo.lane_list            =   _roadModel_LaneList_PDtoBA->pd_RoadModel_LaneList_output;
        m_roadInfo.lane_marker_list   =   _roadModel_LaneMarkerList_PDtoBA->pd_RoadModel_LaneMarkerList_output;
        m_roadInfo.object_list          =   _roadModel_ObjectList_PDtoBA->pd_RoadModel_ObjectList_output;
        m_roadInfo.road_edge_list       =   _roadModel_RoadEdge_PDtoBA->pd_RoadModel_RoadEdge_output;
        m_roadInfo.passage_list  =  _pd_input_last.rm_roadmodel.at(rm_Num).passage_list;//.clear();
    }
    else{
        // std::cout << "ATTENTION: Object not created of RoadModelVector in heap" << std::endl;
    }
#endif   //FLAG_PDRM_HEAP
    pd_RoadModel_output=m_roadInfo;


#if FLAG_PDFP_SAVECSV != 0
    std::ofstream plotfile_PDroadmodel;
    plotfile_PDroadmodel.open("./pd_roadmodel_file.txt", std::ios::app);

    plotfile_PDroadmodel<<"timestamp,"<<timeStampBag<<",roadID_x,"<<int(_pd_input_last.rm_roadmodel.at(rm_Num).road_id.x);
    // 1. last roadinfo
    // 1.1 lane
    plotfile_PDroadmodel<<",size_lastlane,"<<_pd_input_last.rm_roadmodel.at(rm_Num).lane_list.size();
    for(auto &element :  _pd_input_last.rm_roadmodel.at(rm_Num).lane_list){
        // if(element.start_id != _pd_input_last.rm_roadmodel.at(rm_Num).car_position.ego_lane_id) continue;
        plotfile_PDroadmodel<<",lastlane_start_id,"<<int(element.start_id)<<",lastlane_end_id,"<<int(element.end_id)<< \
                            ",lastlane_start_range,"<<element.start_range<<",lastlane_end_range,"<<element.end_range;
        for(auto& element2 : element.corridor_list){
            plotfile_PDroadmodel<<",lastlane_corridor_id,"<<element2.corridor_id<< \
                                ",lastlane_corridor_geometry_c0,"<<element2.reference_line.c0<< \
                                ",lastlane_corridor_geometry_c1,"<<element2.reference_line.c1<< \
                                ",lastlane_corridor_geometry_c2,"<<element2.reference_line.c2<< \
                                ",lastlane_corridor_geometry_c3,"<<element2.reference_line.c3<< \
                                ",lastlane_corridor_geometry_start_range,"<<element2.reference_line.start_range<< \
                                ",lastlane_corridor_geometry_end_range,"<<element2.reference_line.end_range;
            for(auto& element3 : element2.object_in_corridor_list){
                plotfile_PDroadmodel<<",lastlane_obj_in_id,"<<element3.object_id<< \
                                    ",lastlane_obj_in_s,"<<element3.reference_point2corridor_frenet.s<< \
                                    ",lastlane_obj_in_l,"<<element3.reference_point2corridor_frenet.l<< \
                                    ",lastlane_obj_in_x,"<<element3.relative_center_x<< \
                                    ",lastlane_obj_in_y,"<<element3.relative_center_y;
            }
            for(auto& element3 : element2.relevant_object_corridor_list){
                plotfile_PDroadmodel<<",lastlane_obj_out_id,"<<element3.object_id<< \
                                    ",lastlane_obj_out_s,"<<element3.reference_point2corridor_frenet.s<< \
                                    ",lastlane_obj_out_l,"<<element3.reference_point2corridor_frenet.l<< \
                                    ",lastlane_obj_out_x,"<<element3.relative_center_x<< \
                                    ",lastlane_obj_out_y,"<<element3.relative_center_y;
            }
        }
        for(auto& element2 : element.back_corridor_list){
            plotfile_PDroadmodel<<",lastlane_corridor_id,"<<element2.corridor_id<< \
                                ",lastlane_corridor_geometry_c0,"<<element2.reference_line.c0<< \
                                ",lastlane_corridor_geometry_c1,"<<element2.reference_line.c1<< \
                                ",lastlane_corridor_geometry_c2,"<<element2.reference_line.c2<< \
                                ",lastlane_corridor_geometry_c3,"<<element2.reference_line.c3<< \
                                ",lastlane_corridor_geometry_start_range,"<<element2.reference_line.start_range<< \
                                ",lastlane_corridor_geometry_end_range,"<<element2.reference_line.end_range;
            /*for(auto& element3 : element2.object_in_corridor_list){
                plotfile_PDroadmodel<<",lastlane_obj_in_id,"<<element3.object_id<< \
                                    ",lastlane_obj_in_s,"<<element3.reference_point2corridor_frenet.s<< \
                                    ",lastlane_obj_in_l,"<<element3.reference_point2corridor_frenet.l<< \
                                    ",lastlane_obj_in_x,"<<element3.relative_center_x<< \
                                    ",lastlane_obj_in_y,"<<element3.relative_center_y;
            }
            for(auto& element3 : element2.relevant_object_corridor_list){
                plotfile_PDroadmodel<<",lastlane_obj_out_id,"<<element3.object_id<< \
                                    ",lastlane_obj_out_s,"<<element3.reference_point2corridor_frenet.s<< \
                                    ",lastlane_obj_out_l,"<<element3.reference_point2corridor_frenet.l<< \
                                    ",lastlane_obj_out_x,"<<element3.relative_center_x<< \
                                    ",lastlane_obj_out_y,"<<element3.relative_center_y;
            }*/
        }
    }
    // 1.2 lane_boundary
    for(auto &element :  _pd_input_last.rm_roadmodel.at(rm_Num).lane_marker_list){
        for(auto &element2 : element.front_lane_boundary_list){
            plotfile_PDroadmodel<<",lastlane_boundary_id,"<<element2.line_boundary_id<< \
                                ",lastlane_boundary_geometry_c0,"<<element2.line_geometry.c0<< \
                                ",lastlane_boundary_geometry_c1,"<<element2.line_geometry.c1<< \
                                ",lastlane_boundary_geometry_c2,"<<element2.line_geometry.c2<< \
                                ",lastlane_boundary_geometry_c3,"<<element2.line_geometry.c3<< \
                                ",lastlane_boundary_geometry_start_range,"<<element2.line_geometry.start_range<< \
                                ",lastlane_boundary_geometry_end_range,"<<element2.line_geometry.end_range<< 
                            #if USING_ROS2 == 0     // ros1
                                ",lastlane_boundary_type,"<<int(element2.line_type);
                            #else                   // ros2
                                ",lastlane_boundary_type,";
                                int linetype_number = 2;
                                for(auto & element3 : element.line_type_attribute_list){
                                    if((element2.line_geometry.start_range > element3.start_offset-0.01f)&&(element2.line_geometry.start_range < element3.end_offset+0.01f)){
                                        linetype_number = int(element3.linetype);
                                        break;
                                    }
                                }
                                plotfile_PDroadmodel<<int(linetype_number);
                            #endif
        }
        for(auto &element2 : element.back_lane_boundary_list){
            plotfile_PDroadmodel<<",lastlane_boundary_id,"<<element2.line_boundary_id<< \
                                ",lastlane_boundary_geometry_c0,"<<element2.line_geometry.c0<< \
                                ",lastlane_boundary_geometry_c1,"<<element2.line_geometry.c1<< \
                                ",lastlane_boundary_geometry_c2,"<<element2.line_geometry.c2<< \
                                ",lastlane_boundary_geometry_c3,"<<element2.line_geometry.c3<< \
                                ",lastlane_boundary_geometry_start_range,"<<element2.line_geometry.start_range<< \
                                ",lastlane_boundary_geometry_end_range,"<<element2.line_geometry.end_range<< 
                            #if USING_ROS2 == 0     // ros1
                                ",lastlane_boundary_type,"<<int(element2.line_type);
                            #else                   // ros2
                                ",lastlane_boundary_type,";
                                int linetype_number = 2;
                                for(auto & element3 : element.line_type_attribute_list){
                                    if((element2.line_geometry.start_range > element3.start_offset-0.01f)&&(element2.line_geometry.start_range < element3.end_offset+0.01f)){
                                        linetype_number = int(element3.linetype);
                                        break;
                                    }
                                }
                                plotfile_PDroadmodel<<int(linetype_number);
                            #endif
        }

    }
    // 1.3 road_edge
    for(auto &element :  _pd_input_last.rm_roadmodel.at(rm_Num).road_edge_list){
        for(auto &element2 : element.road_edge_segment_list){
            plotfile_PDroadmodel<<",lastlane_roadedge_id,"<<int(element2.id)<< \
                                ",lastlane_roadedge_geometry_c0,"<<element2.line_geometry.c0<< \
                                ",lastlane_roadedge_geometry_c1,"<<element2.line_geometry.c1<< \
                                ",lastlane_roadedge_geometry_c2,"<<element2.line_geometry.c2<< \
                                ",lastlane_roadedge_geometry_c3,"<<element2.line_geometry.c3<< \
                                ",lastlane_roadedge_geometry_start_range,"<<element2.line_geometry.start_range<< \
                                ",lastlane_roadedge_geometry_end_range,"<<element2.line_geometry.end_range;
        }
    }


    // 2. ego vehicle
    plotfile_PDroadmodel<<",EV_end_s,"<<_EV_end_s<<",EV_end_l,"<<_EV_end_l<< \
                        ",EV_delta_l,"<<_EV_delta_l<<",EV_end_vs,"<<_EV_end_vs<<",EV_end_vl,"<<_EV_end_vl<<",EV_end_laneID,"<<int(_EV_end_laneID)<< \
                        ",target_cor_id,"<< _pd_input_last.rm_roadmodel.at(rm_Num).passage_list.at(psg_Num).corridor_id;

    // 3. next roadinfo
    // 3.1 lane
    plotfile_PDroadmodel<<",size_nextlane,"<<pd_RoadModel_output.lane_list.size();
    for(auto &element :  pd_RoadModel_output.lane_list){
        // if(element.start_id != pd_RoadModel_output.car_position.ego_lane_id) continue;
        plotfile_PDroadmodel<<",nextlane_start_id,"<<int(element.start_id)<<",nextlane_end_id,"<<int(element.end_id)<< \
                            ",nextlane_start_range,"<<element.start_range<<",nextlane_end_range,"<<element.end_range;
        for(auto& element2 : element.corridor_list){
            plotfile_PDroadmodel<<",nextlane_corridor_id,"<<element2.corridor_id<< \
                                ",nextlane_corridor_geometry_c0,"<<element2.reference_line.c0<< \
                                ",nextlane_corridor_geometry_c1,"<<element2.reference_line.c1<< \
                                ",nextlane_corridor_geometry_c2,"<<element2.reference_line.c2<< \
                                ",nextlane_corridor_geometry_c3,"<<element2.reference_line.c3<< \
                                ",nextlane_corridor_geometry_start_range,"<<element2.reference_line.start_range<< \
                                ",nextlane_corridor_geometry_end_range,"<<element2.reference_line.end_range;
            for(auto& element3 : element2.object_in_corridor_list){
                plotfile_PDroadmodel<<",nextlane_obj_in_id,"<<element3.object_id<< \
                                    ",nextlane_obj_in_s,"<<element3.reference_point2corridor_frenet.s<< \
                                    ",nextlane_obj_in_l,"<<element3.reference_point2corridor_frenet.l<< \
                                    ",nextlane_obj_in_x,"<<element3.relative_center_x<< \
                                    ",nextlane_obj_in_y,"<<element3.relative_center_y;
            }
            for(auto& element3 : element2.relevant_object_corridor_list){
                plotfile_PDroadmodel<<",nextlane_obj_out_id,"<<element3.object_id<< \
                                    ",nextlane_obj_out_s,"<<element3.reference_point2corridor_frenet.s<< \
                                    ",nextlane_obj_out_l,"<<element3.reference_point2corridor_frenet.l<< \
                                    ",nextlane_obj_out_x,"<<element3.relative_center_x<< \
                                    ",nextlane_obj_out_y,"<<element3.relative_center_y;
            }
        }
        for(auto& element2 : element.back_corridor_list){
            plotfile_PDroadmodel<<",nextlane_corridor_id,"<<element2.corridor_id<< \
                                ",nextlane_corridor_geometry_c0,"<<element2.reference_line.c0<< \
                                ",nextlane_corridor_geometry_c1,"<<element2.reference_line.c1<< \
                                ",nextlane_corridor_geometry_c2,"<<element2.reference_line.c2<< \
                                ",nextlane_corridor_geometry_c3,"<<element2.reference_line.c3<< \
                                ",nextlane_corridor_geometry_start_range,"<<element2.reference_line.start_range<< \
                                ",nextlane_corridor_geometry_end_range,"<<element2.reference_line.end_range;
            /*for(auto& element3 : element2.object_in_corridor_list){
                plotfile_PDroadmodel<<",nextlane_obj_in_id,"<<element3.object_id<< \
                                    ",nextlane_obj_in_s,"<<element3.reference_point2corridor_frenet.s<< \
                                    ",nextlane_obj_in_l,"<<element3.reference_point2corridor_frenet.l<< \
                                    ",nextlane_obj_in_x,"<<element3.relative_center_x<< \
                                    ",nextlane_obj_in_y,"<<element3.relative_center_y;
            }
            for(auto& element3 : element2.relevant_object_corridor_list){
                plotfile_PDroadmodel<<",nextlane_obj_out_id,"<<element3.object_id<< \
                                    ",nextlane_obj_out_s,"<<element3.reference_point2corridor_frenet.s<< \
                                    ",nextlane_obj_out_l,"<<element3.reference_point2corridor_frenet.l<< \
                                    ",nextlane_obj_out_x,"<<element3.relative_center_x<< \
                                    ",nextlane_obj_out_y,"<<element3.relative_center_y;
            }*/
        }
    }
    // 3.2 lane_boundary
    for(auto &element :  pd_RoadModel_output.lane_marker_list){

        for(auto &element2 : element.front_lane_boundary_list){
            plotfile_PDroadmodel<<",nextlane_boundary_id,"<<element2.line_boundary_id<< \
                                ",nextlane_boundary_geometry_c0,"<<element2.line_geometry.c0<< \
                                ",nextlane_boundary_geometry_c1,"<<element2.line_geometry.c1<< \
                                ",nextlane_boundary_geometry_c2,"<<element2.line_geometry.c2<< \
                                ",nextlane_boundary_geometry_c3,"<<element2.line_geometry.c3<< \
                                ",nextlane_boundary_geometry_start_range,"<<element2.line_geometry.start_range<< \
                                ",nextlane_boundary_geometry_end_range,"<<element2.line_geometry.end_range<< 
                            #if USING_ROS2 == 0     // ros1
                                ",nextlane_boundary_type,"<<int(element2.line_type);
                            #else                   // ros2
                                ",nextlane_boundary_type,";
                                int linetype_number = 2;
                                for(auto & element3 : element.line_type_attribute_list){
                                    if((element2.line_geometry.start_range > element3.start_offset-0.01f)&&(element2.line_geometry.start_range < element3.end_offset+0.01f)){
                                        linetype_number = int(element3.linetype);
                                        break;
                                    }
                                }
                                plotfile_PDroadmodel<<int(linetype_number);
                            #endif
        }
        for(auto &element2 : element.back_lane_boundary_list){
            plotfile_PDroadmodel<<",nextlane_boundary_id,"<<element2.line_boundary_id<< \
                                ",nextlane_boundary_geometry_c0,"<<element2.line_geometry.c0<< \
                                ",nextlane_boundary_geometry_c1,"<<element2.line_geometry.c1<< \
                                ",nextlane_boundary_geometry_c2,"<<element2.line_geometry.c2<< \
                                ",nextlane_boundary_geometry_c3,"<<element2.line_geometry.c3<< \
                                ",nextlane_boundary_geometry_start_range,"<<element2.line_geometry.start_range<< \
                                ",nextlane_boundary_geometry_end_range,"<<element2.line_geometry.end_range<< 
                            #if USING_ROS2 == 0     // ros1
                                ",nextlane_boundary_type,"<<int(element2.line_type);
                            #else                   // ros2
                                ",nextlane_boundary_type,";
                                int linetype_number = 2;
                                for(auto & element3 : element.line_type_attribute_list){
                                    if((element2.line_geometry.start_range > element3.start_offset-0.01f)&&(element2.line_geometry.start_range < element3.end_offset+0.01f)){
                                        linetype_number = int(element3.linetype);
                                        break;
                                    }
                                }
                                plotfile_PDroadmodel<<int(linetype_number);
                            #endif
        }

    }
    // 3.3 road_edge
    for(auto &element : pd_RoadModel_output.road_edge_list){
        for(auto &element2 : element.road_edge_segment_list){
            plotfile_PDroadmodel<<",nextlane_roadedge_id,"<<int(element2.id)<< \
                                ",nextlane_roadedge_geometry_c0,"<<element2.line_geometry.c0<< \
                                ",nextlane_roadedge_geometry_c1,"<<element2.line_geometry.c1<< \
                                ",nextlane_roadedge_geometry_c2,"<<element2.line_geometry.c2<< \
                                ",nextlane_roadedge_geometry_c3,"<<element2.line_geometry.c3<< \
                                ",nextlane_roadedge_geometry_start_range,"<<element2.line_geometry.start_range<< \
                                ",nextlane_roadedge_geometry_end_range,"<<element2.line_geometry.end_range;
        }
    }

    plotfile_PDroadmodel<<std::endl;
    plotfile_PDroadmodel.close();

#endif





    // pd_RoadModelVector_output = _pd_last_RoadModelVector;
}




#if FLAG_PDFP_SAVECSV!=0
#if USING_ROS2 == 0
// ros::Subscriber clock_info = n.subscribe<rosgraph_msgs::Clock>("/clock",3,getRosbagTimeCallback);  // add to main()
void getRosbagTimeCallback(const rosgraph_msgs::Clock::ConstPtr& msg){
    timeStampBag = std::to_string(msg->clock.sec) + std::to_string(msg->clock.nsec);
}
#endif  // USING_ROS2

#if USING_ROS2 != 0
// auto clock_info = behavior_analyzer_node->create_subscription<rosgraph_msgs::msg::Clock>("clock",3,getRosbagTimeCallback);  // add to main()
void getRosbagTimeCallback(const rosgraph_msgs::msg::Clock& timemsg){
    // timeStampBag = std::to_string(timemsg.clock.sec) + std::to_string(timemsg.clock.nanosec);
    std::string str_s = std::to_string(timemsg.clock.sec);
    std::string str_ns = std::to_string(timemsg.clock.nanosec);
    int n_zero = 9 - str_ns.size();
    timeStampBag = str_s;
    for(int i=0;i<n_zero;i++){
        timeStampBag = timeStampBag + '0';
    }
    timeStampBag = timeStampBag + str_ns;
}
#endif  // USING_ROS2

#endif



