/**
 * @file
 * @brief  add 2021-11-23
 *        usage: include the head file in prediction.h;
 *               in prediction.cpp function process():  
 */

#include "PD_RoadModel_CarPosition_PDtoBA.h"
using namespace ADC::PD_RoadModel_CarPosition_PDtoBA;
#if FLAG_PD_CARPOS_SAVECSV != 0
extern std::string timeStampBag;
#endif

/**
 * @brief
 * @param
 */
RoadModel_CarPosition_PDtoBA::RoadModel_CarPosition_PDtoBA(const PD::PD_Input &m_pd_input, float m_EV_end_s, float m_EV_end_l, uint8_t m_EV_end_laneID){
    if(init(m_pd_input,m_EV_end_s,m_EV_end_l,m_EV_end_laneID) == false)
    return;
}


/**
 * @brief 
 * @param
 * @return
 */
bool RoadModel_CarPosition_PDtoBA::init(const PD::PD_Input &m_pd_input, float m_EV_end_s, float m_EV_end_l, uint8_t m_EV_end_laneID){

    bool Flag=true;
    Flag = Flag && (static_cast<decltype(m_pd_input.rm_roadmodel.size())>(rm_Num)<m_pd_input.rm_roadmodel.size());                                 //chech rm_roadmodel
    if(Flag){
        Flag = Flag && (static_cast<decltype(m_pd_input.rm_roadmodel.at(rm_Num).passage_list.size())>(psg_Num)<m_pd_input.rm_roadmodel.at(rm_Num).passage_list.size());    //chech passage_list
    }
    /*Flag = Flag && !(m_pd_input.rm_roadmodel.empty());                //chech rm_roadmodel
    for(auto& element : m_pd_input.rm_roadmodel){
        Flag = Flag && !(element.passage_list.empty());             //check passage_list
        // Flag = Flag && !(element.lane_list.empty());                //check lane_list
        // Flag = Flag && !(element.lane_marker_list.empty());       //check lane_marker_list
    }*/




    if(Flag==false){
        initFlag=false;
    }
    else{

        initFlag=true;

        // #if FlagCalculateCarPos != 0
        _m_last_RoadInfo = m_pd_input.rm_roadmodel.at(rm_Num);
        _EV_EndPositionToReferenceLine_s = m_EV_end_s;
        _EV_EndPositionToReferenceLine_l = m_EV_end_l;
        // #else
        _EV_end_laneID = m_EV_end_laneID;
        // #endif

        // output init
        PD_RoadModel_CarPosition_output.ego_lane_id = 0;
        PD_RoadModel_CarPosition_output.lane_line_crossed = 0;
        PD_RoadModel_CarPosition_output.target_lane_id_list.clear();
        // PD_RoadModel_CarPosition_output.target_lane_id_list.emplace_back(static_cast<uint8_t>(0));

    }
    return initFlag;

}


/**
 * @brief
 * @param
 */
void RoadModel_CarPosition_PDtoBA::calcu_CarPosition_based_DurationTime(){



#if FlagCalculateCarPos != 0
    // find ego vehicle last lateral position
    uint32_t passage_corridor_ID = _m_last_RoadInfo.passage_list.at(psg_Num).corridor_id;
    bool flagFindPassageCorridorID = false;
    for(auto& element : _m_last_RoadInfo.lane_list){
        for(auto& element2 : element.corridor_list){
            if(element2.corridor_id == passage_corridor_ID){
                _EV_LastPositionToReferenceLine_l = element2.reference_line.c0;
                flagFindPassageCorridorID = true;
            }
            if(flagFindPassageCorridorID){break;}
        }
        if(flagFindPassageCorridorID){break;}
    }


    // find lane marker
    uint8_t _last_egovehicle_lane_id = _m_last_RoadInfo.car_position.ego_lane_id;
    std::vector<uint32_t> _left_lanemarker_id_list;     _left_lanemarker_id_list.reserve(20);
    std::vector<uint32_t> _right_lanemarker_id_list;    _right_lanemarker_id_list.reserve(20);
    for(auto& element : _m_last_RoadInfo.lane_list){
        if(element.start_id == _last_egovehicle_lane_id){
            _left_lanemarker_id_list = element.left_lane_marker_id_list;
            _right_lanemarker_id_list = element.right_lane_marker_id_list;
            _LaneWidth_egoVehicle = element.lane_width;
            break;
        }
    }
    // get the lateral distance between line and ego vehicle
    bool flag_findLeftBoundary = false;
    bool flag_findRightBoundary = false;
    for(auto& element : _m_last_RoadInfo.lane_marker_list){
        if(!flag_findLeftBoundary){
            for(auto& marker_id : _left_lanemarker_id_list){
                if(element.id == marker_id){
                    for(auto& element2 : element.front_lane_boundary_list){
                        if( (element2.line_geometry.start_range <= 0.1) && 
                            (element2.line_geometry.end_range > 0.1)    ){
                                flag_findLeftBoundary = true;
                                _leftLine_LastPositionToEV_l = element2.line_geometry.c0;
                                break;
                        }
                    }
                }
                if(flag_findLeftBoundary){break;}
            }
        }
        if(!flag_findLeftBoundary){
            _leftLine_LastPositionToEV_l = 200.0;
        }
        if(!flag_findRightBoundary){
            for(auto& marker_id : _right_lanemarker_id_list){
                if(element.id == marker_id){
                    for(auto& element2 : element.front_lane_boundary_list){
                        if( (element2.line_geometry.start_range <= 0.1) && 
                            (element2.line_geometry.end_range > 0.1)    ){
                                flag_findRightBoundary = true;
                                _rightLine_LastPositionToEV_l = element2.line_geometry.c0;
                                break;
                        }
                    }
                }
                if(flag_findRightBoundary){break;}
            }
        }
        if(!flag_findRightBoundary){
            _rightLine_LastPositionToEV_l = 200.0;
        }
        if(flag_findLeftBoundary && flag_findRightBoundary){break;}
    }

    // calculate line Lateral position at end
    _leftLine_EndPositionToEV_l = _leftLine_LastPositionToEV_l - _EV_LastPositionToReferenceLine_l - _EV_EndPositionToReferenceLine_l;
    _rightLine_EndPositionToEV_l = _rightLine_LastPositionToEV_l - _EV_LastPositionToReferenceLine_l - _EV_EndPositionToReferenceLine_l;

    // arbitration
    if((_leftLine_EndPositionToEV_l < 0.0) && (_rightLine_EndPositionToEV_l > 0.0)){
        /*wrong*/
    }
    else{
        if(_m_last_RoadInfo.passage_list.at(psg_Num).action_type == 2){
            if(_leftLine_EndPositionToEV_l < -0.5){
                /*left lane change*/
                PD_RoadModel_CarPosition_output.ego_lane_id = _m_last_RoadInfo.car_position.ego_lane_id - 1;
            }
            else if(_leftLine_EndPositionToEV_l < 0.5){
                /*left lane change cross*/
                PD_RoadModel_CarPosition_output.lane_line_crossed = 1;
                PD_RoadModel_CarPosition_output.ego_lane_id = _m_last_RoadInfo.car_position.ego_lane_id;
            }
            else{
                /*lane follow*/
                PD_RoadModel_CarPosition_output.ego_lane_id = _m_last_RoadInfo.car_position.ego_lane_id;
            }
        }
        else if(_m_last_RoadInfo.passage_list.at(psg_Num).action_type == 3){
            if(_rightLine_EndPositionToEV_l > 0.5){
                /*right lane change*/
                PD_RoadModel_CarPosition_output.ego_lane_id = _m_last_RoadInfo.car_position.ego_lane_id + 1;
            }
            else if(_rightLine_EndPositionToEV_l > -0.5){
                /*right lane change cross*/
                PD_RoadModel_CarPosition_output.lane_line_crossed = 2;
                PD_RoadModel_CarPosition_output.ego_lane_id = _m_last_RoadInfo.car_position.ego_lane_id;
            }
            else{
                /*lane follow*/
                PD_RoadModel_CarPosition_output.ego_lane_id = _m_last_RoadInfo.car_position.ego_lane_id;
            }
        }
        else{
            /*lane follow*/
            PD_RoadModel_CarPosition_output.ego_lane_id = _m_last_RoadInfo.car_position.ego_lane_id;
        }

    }



    // longitudinal
#else

    PD_RoadModel_CarPosition_output.ego_lane_id = _EV_end_laneID;
    PD_RoadModel_CarPosition_output.lane_line_crossed = 0;

#endif


}


/**
 * @brief
 * @param
 */
void RoadModel_CarPosition_PDtoBA::process(){
    if(initFlag != true){
        // std::cout<< "ATTENTION: Data reading of RoadMode_CarPosition_PDtoBA error" <<std::endl;
        return;
    }


    calcu_CarPosition_based_DurationTime();

    #if FLAG_PD_CARPOS_SAVECSV != 0
    std::ofstream outfile_PDRM_CarPos;
    outfile_PDRM_CarPos.open("./pd_roadmodel_carposition_file.csv", std::ios::app);
    outfile_PDRM_CarPos << "timestamp," << timeStampBag << \
                            ",input_end_laneid," << _EV_end_laneID << \
                            ",output_laneid," << int(PD_RoadModel_CarPosition_output.ego_lane_id) << \
                            ",output_linecrossed," << int(PD_RoadModel_CarPosition_output.lane_line_crossed);



    outfile_PDRM_CarPos<<std::endl;
    outfile_PDRM_CarPos.close();
    #endif

}