/**
 * @file
 * @brief  add 2021-11-24
 *        usage: include the head file in prediction.h;
 *               in prediction.cpp function process():  
 */

#include "PD_RoadModel_LaneList_PDtoBA.h"
using namespace ADC::PD_RoadModel_LaneList_PDtoBA;

#if SAVE_Timer_FILE != 0
std::ofstream outfile_lanelist;
#endif

/**
 * @brief
 * @param
 */
RoadModel_LaneList_PDtoBA::RoadModel_LaneList_PDtoBA(const PD::PD_Input &m_pd_input, float m_EV_end_s, float m_EV_end_l, float m_EV_end_vs, float m_EV_end_vl,
                                const std::vector<LaneMarker> &m_pd_lanemarkerlist_output):
                                ADC::PD_RoadModel_LaneList_PDtoBA::RoadModel_base(m_pd_input,m_EV_end_s,m_EV_end_l,m_EV_end_vs,m_EV_end_vl){
    _pd_lanemarkerlist_output = m_pd_lanemarkerlist_output;
    if(init(m_pd_input,m_EV_end_s,m_EV_end_l,m_EV_end_vs,m_EV_end_vl) == false)
    return;
}





/**
 * @brief
 * @param
 */
void RoadModel_LaneList_PDtoBA::calcu_LaneList_based_DurationTime(){
    
    #if SAVE_Timer_FILE != 0
    outfile_lanelist.open("./time_lane_ms.txt", std::ios::app);
    auto time0 = std::chrono::high_resolution_clock::now();
    #endif

    pd_RoadModel_LaneList_output.clear();
    pd_RoadModel_LaneList_output.reserve(16);
    calcu_referenceLineList_Cartesian();
    calcu_transforFactor_BaseOnEVMove();

    #if SAVE_Timer_FILE != 0
    auto time1 = std::chrono::high_resolution_clock::now();
    std::chrono::duration<double, std::milli> tm1=time1 - time0;
    outfile_lanelist<<"lane_first_tm,"<<tm1.count()<<",";
    #endif
    std::map<uint32_t,Position_Cartesian> objectXY_inCorridor_list;     // save only one object Cartesian for different corridor
    /*uint32_t corridorIDwithObject=2147483647;           // find the corridor id which contains object
    {
        bool isPassageCorridorNoObject = false;    // linefollow=1,carfollow=2,ilc=3  // There would be object on corridor only in linefollow mode
        bool isFindCorridor = false;
        bool isFindCorridorWithObject = false;
        for(auto& element:_m_last_RoadInfo.lane_list){
            for(auto& element2:element.corridor_list){
                if(element2.corridor_id == _m_last_RoadInfo.passage_list.at(psg_Num).corridor_id){
                    isPassageCorridorNoObject = !((element2.feature_tag == static_cast<decltype(element2.feature_tag)>(1)) ||   // line follow
                                                  (element2.feature_tag == static_cast<decltype(element2.feature_tag)>(2)) );   // car follow
                    isFindCorridor = true;
                    break;
                }
            }
            if(isFindCorridor)  {
                if(!isPassageCorridorNoObject){
                    corridorIDwithObject = static_cast<decltype(corridorIDwithObject)>(_m_last_RoadInfo.passage_list.at(psg_Num).corridor_id);
                    isFindCorridorWithObject = true;
                }
                else{
                    for(auto& element2:element.corridor_list){
                        if(element2.feature_tag == static_cast<decltype(element2.feature_tag)>(1)){
                            corridorIDwithObject = static_cast<decltype(corridorIDwithObject)>(element2.corridor_id);
                            isFindCorridorWithObject = true;
                            break;
                        }
                    }
                }
                break;
            }
        }
        if(!isFindCorridorWithObject){    // not find available corridor in its lane
            for(auto& element:_m_last_RoadInfo.lane_list){
                for(auto& element2:element.corridor_list){
                    if((!element2.object_in_corridor_list.empty())||(!element2.relevant_object_corridor_list.empty())){
                        corridorIDwithObject = static_cast<decltype(corridorIDwithObject)>(element2.corridor_id);
                        isFindCorridorWithObject = true;
                        break;
                    }
                }
                if(isFindCorridorWithObject) break;
            }
        }
    }*/
    // attach object to corridor if corridor is ilc corridor
    float deltaL_ilc2baseCorridor = 0.0f;
    bool flagFindPassageCorridorID = false;
    if(static_cast<uint8_t>(_m_last_RoadInfo.passage_list.at(psg_Num).feature_label) == static_cast<uint8_t>(3)){     // ilc
        float deltaL_ilc2baseCorridor = 0.0f;
        float c0_baseCorridor = 0.0f;
        float c0_ilcCorridor = 0.0f;
        Corridor baseCorridor_bak;
        Corridor* ilcCorridor_change=nullptr;
        for(auto& element:_m_last_RoadInfo.lane_list){
            for(auto& element2:element.corridor_list){
                if(element2.corridor_id == _m_last_RoadInfo.passage_list.at(psg_Num).corridor_id){
                    c0_ilcCorridor = element2.reference_line.c0;
                    ilcCorridor_change = &element2;
                    flagFindPassageCorridorID = true;
                    break;
                }
            }
            if(flagFindPassageCorridorID){
                for(auto& element2:element.corridor_list){
                    if(static_cast<uint8_t>(element2.feature_tag) == static_cast<uint8_t>(1)){      // line follow
                        c0_baseCorridor = element2.reference_line.c0;
                        baseCorridor_bak = element2;
                        break;
                    }
                }
                break;
            }
        }
        deltaL_ilc2baseCorridor = c0_ilcCorridor - c0_baseCorridor;
        if(flagFindPassageCorridorID)
        if(ilcCorridor_change){
            if((ilcCorridor_change->object_in_corridor_list.empty())&&(ilcCorridor_change->relevant_object_corridor_list.empty())){
                for(auto element1:baseCorridor_bak.object_in_corridor_list){
                    ilcCorridor_change->object_in_corridor_list.emplace_back(element1);
                    ilcCorridor_change->object_in_corridor_list.back().reference_point2corridor_frenet.l = 
                        ilcCorridor_change->object_in_corridor_list.back().reference_point2corridor_frenet.l - deltaL_ilc2baseCorridor;
                }
                for(auto element1:baseCorridor_bak.relevant_object_corridor_list){
                    ilcCorridor_change->relevant_object_corridor_list.emplace_back(element1);
                    ilcCorridor_change->relevant_object_corridor_list.back().reference_point2corridor_frenet.l = 
                        ilcCorridor_change->relevant_object_corridor_list.back().reference_point2corridor_frenet.l - deltaL_ilc2baseCorridor;
                }
            }
        }
    }

    // 20220729 only update left_left to right_right
    std::set<uint8_t> laneID_ll2rr;
    {
        // ego lane id
        laneID_ll2rr.insert(_m_last_RoadInfo.car_position.ego_lane_id);
        std::pair<bool,uint8_t> laneID_left = {false,0}, laneID_leftleft = {false,0}, laneID_right = {false,0}, laneID_rightright = {false,0};
        for(auto& element:_m_last_RoadInfo.lane_list){
            if(element.start_id == _m_last_RoadInfo.car_position.ego_lane_id){
                // left
                for(auto& ele2:element.left_neighbor_lane){
                    if(element.left_neighbor_lane.size() == 1){
                        laneID_left={true,ele2.lane_id};
                    }
                    else{
                        if((ele2.start_range < 10.0f)&&(ele2.end_range > 10.0f)){
                            laneID_left={true,ele2.lane_id};
                            break;                    
                        }
                    }
                }
                // right
                for(auto& ele2:element.right_neighbor_lane){
                    if(element.right_neighbor_lane.size() == 1){
                        laneID_right={true,ele2.lane_id};
                    }
                    else{
                        if((ele2.start_range < 10.0f)&&(ele2.end_range > 10.0f)){
                            laneID_right={true,ele2.lane_id};
                            break;                    
                        }
                    }
                }
                break;
            }
        }
        // left_left and right_right
        for(auto& element:_m_last_RoadInfo.lane_list){
            // left_left
            if(laneID_left.first)
            if(element.start_id == laneID_left.second){
                for(auto& ele2:element.left_neighbor_lane){
                    if(element.left_neighbor_lane.size() == 1){
                        laneID_leftleft={true,ele2.lane_id};
                    }
                    else{
                        if((ele2.start_range < 10.0f)&&(ele2.end_range > 10.0f)){
                            laneID_leftleft={true,ele2.lane_id};
                            break;
                        }
                    }
                }
            }
            // right_right
            if(laneID_right.first)
            if(element.start_id == laneID_right.second){
                for(auto& ele2:element.right_neighbor_lane){
                    if(element.right_neighbor_lane.size()==1){
                        laneID_rightright={true,ele2.lane_id};
                    }
                    else{
                        if((ele2.start_range < 10.0f)&&(ele2.end_range > 10.0f)){
                            laneID_rightright={true,ele2.lane_id};
                            break;
                        }
                    }
                }
            }
            if(laneID_leftleft.first&&laneID_rightright.first) break;
        }
        if(laneID_left.first)   laneID_ll2rr.insert(laneID_left.second);
        if(laneID_leftleft.first)   laneID_ll2rr.insert(laneID_leftleft.second);
        if(laneID_right.first)   laneID_ll2rr.insert(laneID_right.second);
        if(laneID_rightright.first)   laneID_ll2rr.insert(laneID_rightright.second);
    }

    for(auto& element : _m_last_RoadInfo.lane_list){

        #if SAVE_Timer_FILE != 0
        auto time2 = std::chrono::high_resolution_clock::now();
        #endif
        Lane m_next_lane;

        if(true){         // vaild in this range Frenet

            // 20220729 only update left_left to right_right no more than 5 lanes
            if(laneID_ll2rr.count(element.start_id)==0)continue;

            // start_id
            m_next_lane.start_id   = element.start_id;
            // end_id
            m_next_lane.end_id     = element.end_id;
            // start_range
            m_next_lane.start_range = 0.0f;//calcu_EndCartesianPoint_BaseOnEVMove(element.start_range,lane_start_range_y).x_Cartesian;
            // end_range
            float end_range_extend = 0.0f;
            #if USING_ROS2 != 0
            if(static_cast<bool>(element.is_end_out_of_range) == static_cast<bool>(false)){     // lane is not end
            #endif  // USING_ROS2
                end_range_extend = std::max( 4.0f*_EV_EndPositionToReferenceLine_vs, const_minCorridorLength);
            #if USING_ROS2 != 0
            }
            #endif  // USING_ROS2
            m_next_lane.end_range = std::max( element.end_range - _EV_EndPositionToReferenceLine_s, end_range_extend); // calcu_EndCartesianPoint_BaseOnEVMove(element.end_range,lane_end_range_y).x_Cartesian
            // left_neighbor_lane
            m_next_lane.left_neighbor_lane.reserve(element.left_neighbor_lane.size());
            NeighborLane m_next_neighborlane;
            for(auto& element2 : element.left_neighbor_lane){
                m_next_neighborlane.start_range = element2.start_range - _EV_EndPositionToReferenceLine_s;
                m_next_neighborlane.end_range = element2.end_range - _EV_EndPositionToReferenceLine_s;
                m_next_neighborlane.lane_id = element2.lane_id;
                m_next_lane.left_neighbor_lane.emplace_back(m_next_neighborlane);
            }
            // right_neighbor_lane
            m_next_lane.right_neighbor_lane.reserve(element.right_neighbor_lane.size());
            for(auto& element2 : element.right_neighbor_lane){
                m_next_neighborlane.start_range = element2.start_range - _EV_EndPositionToReferenceLine_s;
                m_next_neighborlane.end_range = element2.end_range - _EV_EndPositionToReferenceLine_s;
                m_next_neighborlane.lane_id = element2.lane_id;
                m_next_lane.right_neighbor_lane.emplace_back(m_next_neighborlane);
            }
            
            // left_lane_marker_id_list
            m_next_lane.left_lane_marker_id_list = element.left_lane_marker_id_list;
            // right_lane_marker_id_list
            m_next_lane.right_lane_marker_id_list = element.right_lane_marker_id_list;

            #if SAVE_Timer_FILE != 0
            auto time3 = std::chrono::high_resolution_clock::now();
            std::chrono::duration<double, std::milli> tm3=time3 - time2;
            outfile_lanelist<<"lane_boundary_tm,"<<tm3.count()<<",";
            #endif

            // sampling_point_list      // not use, so no update
            /*m_next_lane.sampling_point_list.reserve(element.sampling_point_list.size());
            SamplingPoint m_next_samplingPoint;   m_next_samplingPoint.curve = 0.0;
            for(auto& element2 : element.sampling_point_list){
                Position_Cartesian next_samplingPoint_Cartesian = calcu_EndCartesianPoint_BaseOnEVMove(element2.x,element2.y);
                m_next_samplingPoint.x = next_samplingPoint_Cartesian.x_Cartesian;
                m_next_samplingPoint.y = next_samplingPoint_Cartesian.y_Cartesian;
                m_next_lane.sampling_point_list.emplace_back(m_next_samplingPoint);
            }*/

            // front corridor_list
            m_next_lane.corridor_list.reserve(64);
            m_next_lane.back_corridor_list.reserve(64);
            for(auto& element2 : element.corridor_list){
                if(fabs(element2.reference_line.c0)>1000.0f) continue;  // estimate availability
                Corridor m_next_corridor;
                float corridor_start_range_y =  element2.reference_line.c0 + 
                                                element2.reference_line.c1 * element2.reference_line.start_range +
                                                element2.reference_line.c2 * element2.reference_line.start_range * element2.reference_line.start_range +
                                                element2.reference_line.c3 * element2.reference_line.start_range * element2.reference_line.start_range * element2.reference_line.start_range;
                float corridor_end_range_y =    element2.reference_line.c0 + 
                                                element2.reference_line.c1 * element2.reference_line.end_range +
                                                element2.reference_line.c2 * element2.reference_line.end_range * element2.reference_line.end_range +
                                                element2.reference_line.c3 * element2.reference_line.end_range * element2.reference_line.end_range * element2.reference_line.end_range;
                if(true){
                    m_next_corridor.corridor_id = element2.corridor_id;
                    m_next_corridor.feature_tag = element2.feature_tag;
                    m_next_corridor.ilc_object_target = element2.ilc_object_target;
                    m_next_corridor.disenable_reason = element2.disenable_reason;
                    m_next_corridor.drivable_width = element2.drivable_width;
                    
                    // reference_line
                    float x_range = element2.reference_line.end_range-element2.reference_line.start_range;
                    if(fabs(x_range)<0.005)         {x_range = std::max(x_range,0.01f);}
                    Position_Cartesian selectNewPointFit;
                    std::vector<Position_Cartesian> selectNewPointFitList;
                    selectNewPointFitList.reserve(sizeof(const_interpolation)/sizeof(float));
                    for(decltype(sizeof(const_interpolation)/sizeof(float)) i=0;i<sizeof(const_interpolation)/sizeof(float);i++){
                        float last_x =  element2.reference_line.start_range + const_interpolation[i] * x_range;
                        float last_y =  element2.reference_line.c0 + 
                                        element2.reference_line.c1 * last_x +
                                        element2.reference_line.c2 * last_x * last_x +
                                        element2.reference_line.c3 * last_x * last_x * last_x;
                        selectNewPointFit = calcu_EndCartesianPoint_BaseOnEVMove(last_x,last_y);
                        selectNewPointFitList.emplace_back(selectNewPointFit);
                    }

                    #if SAVE_Timer_FILE != 0
                    auto time5 = std::chrono::high_resolution_clock::now();
                    std::chrono::duration<double, std::milli> tm5=time5 - time3;
                    outfile_lanelist<<"lane_corr_beforefit_tm,"<<tm5.count()<<",";
                    #endif

                    std::unique_ptr<float[]> newReferenceLineCoeff(new float[4]);
                    int num_fit=0;
                    while(PolynomialFitting(selectNewPointFitList,3,newReferenceLineCoeff)>const_fittingErrorThreshold){
                        num_fit++;
                        if(num_fit>3){
                            newReferenceLineCoeff[1]=(fabs(selectNewPointFitList.back().x_Cartesian-selectNewPointFitList.front().x_Cartesian)<0.001) ? 0.0 : \
                                                        (selectNewPointFitList.back().y_Cartesian-selectNewPointFitList.front().y_Cartesian) / \
                                                        (selectNewPointFitList.back().x_Cartesian-selectNewPointFitList.front().x_Cartesian);
                            newReferenceLineCoeff[0]=selectNewPointFitList.front().y_Cartesian - \
                                                        newReferenceLineCoeff[1] * selectNewPointFitList.front().x_Cartesian;
                            newReferenceLineCoeff[2]=0.0;
                            newReferenceLineCoeff[3]=0.0;
                            break;
                        }
                    }

                    #if SAVE_Timer_FILE != 0
                    auto time6 = std::chrono::high_resolution_clock::now();
                    std::chrono::duration<double, std::milli> tm6=time6 - time6;
                    outfile_lanelist<<"lane_fit_tm,"<<tm6.count()<<",";
                    #endif

                    m_next_corridor.reference_line.start_range = calcu_EndCartesianPoint_BaseOnEVMove(element2.reference_line.start_range,corridor_start_range_y).x_Cartesian;
                    m_next_corridor.reference_line.end_range = calcu_EndCartesianPoint_BaseOnEVMove(element2.reference_line.end_range,corridor_end_range_y).x_Cartesian;
                    m_next_corridor.reference_line.c0 = newReferenceLineCoeff[0];
                    m_next_corridor.reference_line.c1 = newReferenceLineCoeff[1];
                    m_next_corridor.reference_line.c2 = newReferenceLineCoeff[2];
                    m_next_corridor.reference_line.c3 = newReferenceLineCoeff[3];

                    
                    // object           // TODO add new interface in future
                    m_next_corridor.object_in_corridor_list.reserve(128);
                    m_next_corridor.relevant_object_corridor_list.reserve(128);
                    ObjectLocation m_next_object;
                    float durat_time = _m_last_RoadInfo.passage_list.at(psg_Num).duration_time;
                    for(auto& element3 : element2.object_in_corridor_list){
                        // objectLocation object_id
                        m_next_object.object_id = element3.object_id;

                        // objectLocation reference_point2corridor_frenet
                        SLPoint obj_next_temporary = element3.reference_point2corridor_frenet;
                        float t_last = 0.0;
                        float ds_last = 0.0f;
                        float dl_last = 0.0f;
                        for(float t=0.0; t<=durat_time; t=t+0.5){

                            float dds_SV_temporary = 0.0f;
                            if(t<element3.reference_point2corridor_frenet.dds_valid_time)  {
                                dds_SV_temporary = element3.reference_point2corridor_frenet.dds;
                                obj_next_temporary.dds_valid_time = obj_next_temporary.dds_valid_time - (t-t_last);
                                obj_next_temporary.dds_valid_time = std::max(obj_next_temporary.dds_valid_time, 0.0f);
                            }
                            else{
                                dds_SV_temporary = 0.0f;
                                obj_next_temporary.dds = 0.0;
                                obj_next_temporary.dds_valid_time = 0.0;
                            }
                            float dl_SV_temporary  = 0.0f;
                            if(t<element3.reference_point2corridor_frenet.dl_valid_time)   {
                                dl_SV_temporary = element3.reference_point2corridor_frenet.dl;
                                obj_next_temporary.dl_valid_time = obj_next_temporary.dl_valid_time - (t-t_last);
                                obj_next_temporary.dl_valid_time = std::max(obj_next_temporary.dl_valid_time, 0.0f);
                            }
                            else{
                                dl_SV_temporary = 0.0;
                                obj_next_temporary.dl = 0.0;
                                obj_next_temporary.dl_valid_time = 0.0;
                            }
                            /*float ddl_SV_temporary = 0.0f;
                            if(t<element3.reference_point2corridor_frenet.ddl_valid_time)  {
                                ddl_SV_temporary = element3.reference_point2corridor_frenet.ddl;
                                obj_next_temporary.ddl_valid_time = obj_next_temporary.ddl_valid_time - (t-t_last);
                                obj_next_temporary.ddl_valid_time = std::max(obj_next_temporary.ddl_valid_time, 0.0f);
                            }
                            else{
                                ddl_SV_temporary = 0.0;
                                obj_next_temporary.ddl = 0.0;
                                obj_next_temporary.ddl_valid_time = 0.0;
                            }*/

                            if(t<0.3f){
                                ds_last = obj_next_temporary.ds;
                                dl_last = dl_SV_temporary;
                            }
                            obj_next_temporary.ds = obj_next_temporary.ds + dds_SV_temporary * (t-t_last);
                            obj_next_temporary.s = obj_next_temporary.s + 0.5f*(ds_last+obj_next_temporary.ds) * (t-t_last);
                            obj_next_temporary.l = obj_next_temporary.l + 0.5f*(dl_last+dl_SV_temporary) * (t-t_last);

                            ds_last = obj_next_temporary.ds;
                            dl_last = dl_SV_temporary;
                            t_last = t;
                        }
                        obj_next_temporary.s = obj_next_temporary.s - _EV_EndPositionToReferenceLine_s;
                        m_next_object.reference_point2corridor_frenet = obj_next_temporary;

                        Position_Cartesian objectXY;    objectXY.x_Cartesian = 9999.9f; objectXY.y_Cartesian = 9999.9f;
                        if(m_next_corridor.corridor_id == _m_last_RoadInfo.passage_list.at(psg_Num).corridor_id){       // only work when corridor is the passage corridor
                        // if(m_next_corridor.corridor_id == static_cast<decltype(m_next_corridor.corridor_id)>(corridorIDwithObject)){       // only work when corridor is the passage corridor
                            objectXY = FrenetToCartesian(_m_referenceLine_Cartesian_list, 
                                                        m_next_object.reference_point2corridor_frenet.s,
                                                        m_next_object.reference_point2corridor_frenet.l);
                            objectXY = calcu_EndCartesianPoint_BaseOnEVMove(objectXY.x_Cartesian, objectXY.y_Cartesian);
                            
                            if(objectXY_inCorridor_list.count(static_cast<uint32_t>(m_next_object.object_id))==0){
                                objectXY_inCorridor_list[static_cast<uint32_t>(m_next_object.object_id)] = objectXY;
                            }
                        }
                        // objectLocation relative_center_x
                        m_next_object.relative_center_x = objectXY.x_Cartesian;
                        // objectLocation relative_center_y
                        m_next_object.relative_center_y = objectXY.y_Cartesian;
                        // objectLocation size_width
                        m_next_object.size_width = element3.size_width;
                        // objectLocation size_length
                        m_next_object.size_length = element3.size_length;
                        // objectLocation classification
                        m_next_object.classification = element3.classification;

                        if(fabs(m_next_object.reference_point2corridor_frenet.l)<=element2.drivable_width*0.5){   // object_in_corridor_list
                            m_next_corridor.object_in_corridor_list.emplace_back(m_next_object);
                        }
                        else{   // relevant_object_corridor_list
                            m_next_corridor.relevant_object_corridor_list.emplace_back(m_next_object);
                        }
                    }
                    for(auto& element3 : element2.relevant_object_corridor_list){
                        // objectLocation object_id
                        m_next_object.object_id = element3.object_id;

                        // objectLocation reference_point2corridor_frenet
                        SLPoint obj_next_temporary = element3.reference_point2corridor_frenet;
                        float t_last = 0.0;
                        for(float t=0.0; t<=durat_time; t=t+0.5){

                            float dds_SV_temporary = 0.0f;
                            if(t<element3.reference_point2corridor_frenet.dds_valid_time)  {
                                dds_SV_temporary = element3.reference_point2corridor_frenet.dds;
                                obj_next_temporary.dds_valid_time = obj_next_temporary.dds_valid_time - (t-t_last);
                                obj_next_temporary.dds_valid_time = std::max(obj_next_temporary.dds_valid_time, 0.0f);
                            }
                            else{
                                dds_SV_temporary = 0.0f;
                                obj_next_temporary.dds = 0.0;
                                obj_next_temporary.dds_valid_time = 0.0;
                            }
                            float dl_SV_temporary  = 0.0f;
                            if(t<element3.reference_point2corridor_frenet.dl_valid_time)   {
                                dl_SV_temporary = element3.reference_point2corridor_frenet.dl;
                                obj_next_temporary.dl_valid_time = obj_next_temporary.dl_valid_time - (t-t_last);
                                obj_next_temporary.dl_valid_time = std::max(obj_next_temporary.dl_valid_time, 0.0f);
                            }
                            else{
                                dl_SV_temporary = 0.0;
                                obj_next_temporary.dl = 0.0;
                                obj_next_temporary.dl_valid_time = 0.0;
                            }
                            /*float ddl_SV_temporary = 0.0f;
                            if(t<element3.reference_point2corridor_frenet.ddl_valid_time)  {
                                ddl_SV_temporary = element3.reference_point2corridor_frenet.ddl;
                                obj_next_temporary.ddl_valid_time = obj_next_temporary.ddl_valid_time - (t-t_last);
                                obj_next_temporary.ddl_valid_time = std::max(obj_next_temporary.ddl_valid_time, 0.0f);
                            }
                            else{
                                ddl_SV_temporary = 0.0;
                                obj_next_temporary.ddl = 0.0;
                                obj_next_temporary.ddl_valid_time = 0.0;
                            }*/

                            obj_next_temporary.ds = obj_next_temporary.ds + dds_SV_temporary * (t-t_last);
                            obj_next_temporary.s = obj_next_temporary.s + obj_next_temporary.ds * (t-t_last);
                            obj_next_temporary.l = obj_next_temporary.l + dl_SV_temporary * (t-t_last);

                            t_last = t;
                        }
                        obj_next_temporary.s = obj_next_temporary.s - _EV_EndPositionToReferenceLine_s;
                        m_next_object.reference_point2corridor_frenet = obj_next_temporary;

                        Position_Cartesian objectXY;    objectXY.x_Cartesian = 9999.9f; objectXY.y_Cartesian = 9999.9f;
                        if(m_next_corridor.corridor_id == _m_last_RoadInfo.passage_list.at(psg_Num).corridor_id){       // only work when corridor is the passage corridor
                        // if(m_next_corridor.corridor_id == static_cast<decltype(m_next_corridor.corridor_id)>(corridorIDwithObject)){       // only work when corridor is the passage corridor
                            objectXY = FrenetToCartesian(_m_referenceLine_Cartesian_list, 
                                                        m_next_object.reference_point2corridor_frenet.s,
                                                        m_next_object.reference_point2corridor_frenet.l);

                            if(objectXY_inCorridor_list.count(static_cast<uint32_t>(m_next_object.object_id))==0){
                                objectXY_inCorridor_list[static_cast<uint32_t>(m_next_object.object_id)] = objectXY;
                            }
                        }
                        // objectLocation relative_center_x
                        m_next_object.relative_center_x = objectXY.x_Cartesian;
                        // objectLocation relative_center_y
                        m_next_object.relative_center_y = objectXY.y_Cartesian;
                        // objectLocation size_width
                        m_next_object.size_width = element3.size_width;
                        // objectLocation size_length
                        m_next_object.size_length = element3.size_length;
                        // objectLocation classification
                        m_next_object.classification = element3.classification;

                        if(fabs(m_next_object.reference_point2corridor_frenet.l)<=element2.drivable_width*0.5){   // object_in_corridor_list
                            m_next_corridor.object_in_corridor_list.emplace_back(m_next_object);
                        }
                        else{   // relevant_object_corridor_list
                            m_next_corridor.relevant_object_corridor_list.emplace_back(m_next_object);
                        }
                    }
                    std::vector<ObjectLocation>(m_next_corridor.object_in_corridor_list).swap(m_next_corridor.object_in_corridor_list); //capacity()
                    std::vector<ObjectLocation>(m_next_corridor.relevant_object_corridor_list).swap(m_next_corridor.relevant_object_corridor_list); //capacity()
                    if(m_next_corridor.reference_line.end_range>=0.0){
                        if(m_next_corridor.reference_line.start_range>=0.0){
                            m_next_lane.corridor_list.emplace_back(m_next_corridor);
                        }
                        else{
                            Corridor m_next_front_corridor=m_next_corridor;
                            Corridor m_next_back_corridor=m_next_corridor;
                            m_next_front_corridor.reference_line.start_range=0.0;
                            m_next_back_corridor.reference_line.end_range=0.0;
                            m_next_lane.corridor_list.emplace_back(m_next_front_corridor);
                            m_next_lane.back_corridor_list.emplace_back(m_next_back_corridor);
                        }
                    }
                    else{
                        m_next_lane.back_corridor_list.emplace_back(m_next_corridor);
                    }

                    #if SAVE_Timer_FILE != 0
                    auto time7 = std::chrono::high_resolution_clock::now();
                    std::chrono::duration<double, std::milli> tm7=time7 - time6;
                    outfile_lanelist<<"lane_corr_afterfit_tm,"<<tm7.count()<<",";
                    #endif

                }
            }
            if(m_next_lane.corridor_list.empty()){      // build corridor by point
                Corridor m_extend_corridor;
                if(!m_next_lane.back_corridor_list.empty()) {
                    m_extend_corridor = m_next_lane.back_corridor_list.front();
                }
                m_extend_corridor.reference_line.start_range = 0.0;
                m_extend_corridor.reference_line.end_range = std::max( const_factorExtendReferenceline*_EV_EndPositionToReferenceLine_vs, const_minCorridorLength);
                m_extend_corridor.reference_line.end_range = std::min( m_extend_corridor.reference_line.end_range, const_maxCorridorLength);
                #if USING_ROS2 != 0
                if(static_cast<bool>(element.is_end_out_of_range) == static_cast<bool>(true)){  // lane is end in range
                    float boundary_end_range = const_maxCorridorLength;
                    if(!element.left_lane_marker_id_list.empty()){
                        uint32_t left_lanemarker_id = element.left_lane_marker_id_list.front();
                        for(auto& element2 : _pd_lanemarkerlist_output){
                            if(static_cast<uint32_t>(element2.id) == left_lanemarker_id){
                                if(!element2.front_lane_boundary_list.empty()){
                                    boundary_end_range = std::min(boundary_end_range, element2.front_lane_boundary_list.back().line_geometry.end_range);//sampling point will not be updated if not provied by RM, so do not use it
                                    break;
                                }
                            }
                        }
                    }
                    if(!element.right_lane_marker_id_list.empty()){
                        uint32_t right_lanemarker_id = element.right_lane_marker_id_list.front();
                        for(auto& element2 : _pd_lanemarkerlist_output){
                            if(static_cast<uint32_t>(element2.id) == right_lanemarker_id){
                                if(!element2.front_lane_boundary_list.empty()){
                                    boundary_end_range = std::min(boundary_end_range, element2.front_lane_boundary_list.back().line_geometry.end_range);//sampling point will not be updated if not provied by RM, so do not use it
                                    break;
                                }
                            }
                        }
                    }
                    m_extend_corridor.reference_line.end_range = std::min( m_extend_corridor.reference_line.end_range, boundary_end_range );
                }
                #endif  // USING_ROS2

                if((m_extend_corridor.reference_line.end_range-m_extend_corridor.reference_line.start_range) > 0.1){    // meanful to update
                    uint32_t left_lanemarker_id = element.left_lane_marker_id_list.empty()? 0 : element.left_lane_marker_id_list.front();
                    std::vector<LaneBoundary> left_laneboundary_list;
                    for(auto& element2 : _pd_lanemarkerlist_output){
                        if(static_cast<uint32_t>(element2.id) == left_lanemarker_id){
                            left_laneboundary_list = element2.front_lane_boundary_list;
                            break;
                        }
                    }
                    uint32_t right_lanemarker_id = element.right_lane_marker_id_list.empty()? 0 : element.right_lane_marker_id_list.front();
                    std::vector<LaneBoundary> right_laneboundary_list;
                    for(auto& element2 : _pd_lanemarkerlist_output){
                        if(static_cast<uint32_t>(element2.id) == right_lanemarker_id){
                            right_laneboundary_list = element2.front_lane_boundary_list;
                            break;
                        }
                    }
                    Position_Cartesian selectNewPointFit;
                    std::vector<Position_Cartesian> selectNewPointFitList;
                    selectNewPointFitList.reserve(sizeof(const_interpolation)/sizeof(float));
                    for(decltype(sizeof(const_interpolation)/sizeof(float)) i=0;i<sizeof(const_interpolation)/sizeof(float);i++){
                        selectNewPointFit.x_Cartesian = m_extend_corridor.reference_line.start_range + const_interpolation[i] * (m_extend_corridor.reference_line.end_range - m_extend_corridor.reference_line.start_range);
                        selectNewPointFit.y_Cartesian = 0.0;
                        bool flag_find_leftpoint = false;
                        float leftpoint_y = 0.0;
                        for(auto& element2 : left_laneboundary_list){
                            if((element2.line_geometry.start_range<=selectNewPointFit.x_Cartesian) && (element2.line_geometry.end_range>=selectNewPointFit.x_Cartesian)){
                                leftpoint_y = element2.line_geometry.c0 + 
                                                element2.line_geometry.c1 * selectNewPointFit.x_Cartesian + 
                                                element2.line_geometry.c2 * selectNewPointFit.x_Cartesian * selectNewPointFit.x_Cartesian + 
                                                element2.line_geometry.c3 * selectNewPointFit.x_Cartesian * selectNewPointFit.x_Cartesian * selectNewPointFit.x_Cartesian; 
                                flag_find_leftpoint=true;
                                break;
                            }
                        }
                        bool flag_find_rightpoint = false;
                        float rightpoint_y = 0.0;
                        for(auto& element2 : right_laneboundary_list){
                            if((element2.line_geometry.start_range<=selectNewPointFit.x_Cartesian) && (element2.line_geometry.end_range>=selectNewPointFit.x_Cartesian)){
                                rightpoint_y = element2.line_geometry.c0 + 
                                                element2.line_geometry.c1 * selectNewPointFit.x_Cartesian + 
                                                element2.line_geometry.c2 * selectNewPointFit.x_Cartesian * selectNewPointFit.x_Cartesian + 
                                                element2.line_geometry.c3 * selectNewPointFit.x_Cartesian * selectNewPointFit.x_Cartesian * selectNewPointFit.x_Cartesian; 
                                flag_find_rightpoint=true;
                                break;
                            }
                        }
                        if(flag_find_leftpoint && flag_find_rightpoint){
                            selectNewPointFit.y_Cartesian = 0.5 * (leftpoint_y+rightpoint_y);
                            selectNewPointFitList.emplace_back(selectNewPointFit);
                        }
                    }
                    std::unique_ptr<float[]> newReferenceLineCoeff(new float[4]);
                    int num_fit=0;
                    while(PolynomialFitting(selectNewPointFitList,3,newReferenceLineCoeff)>const_fittingErrorThreshold){
                        num_fit++;
                        if(num_fit>3){
                            newReferenceLineCoeff[1]=(fabs(selectNewPointFitList.back().x_Cartesian-selectNewPointFitList.front().x_Cartesian)<0.001) ? 0.0 : \
                                                        (selectNewPointFitList.back().y_Cartesian-selectNewPointFitList.front().y_Cartesian) / \
                                                        (selectNewPointFitList.back().x_Cartesian-selectNewPointFitList.front().x_Cartesian);
                            newReferenceLineCoeff[0]=selectNewPointFitList.front().y_Cartesian - \
                                                        newReferenceLineCoeff[1] * selectNewPointFitList.front().x_Cartesian;
                            newReferenceLineCoeff[2]=0.0;
                            newReferenceLineCoeff[3]=0.0;
                            break;
                        }
                    }
                    m_extend_corridor.reference_line.c0 = newReferenceLineCoeff[0];
                    m_extend_corridor.reference_line.c1 = newReferenceLineCoeff[1];
                    m_extend_corridor.reference_line.c2 = newReferenceLineCoeff[2];
                    m_extend_corridor.reference_line.c3 = newReferenceLineCoeff[3];

                    if(selectNewPointFitList.size()>1){
                        m_next_lane.corridor_list.emplace_back(m_extend_corridor);
                    }
                }
            }
            else{                                       // combine corridor by line and point, then replace origin referenceline
                LineGeometry m_extend_lineGeometry;
                for(decltype(m_next_lane.corridor_list.size()) i_corr=0; i_corr<m_next_lane.corridor_list.size();i_corr++){
                    m_extend_lineGeometry = m_next_lane.corridor_list.at(i_corr).reference_line;
                    m_extend_lineGeometry.start_range = std::max(static_cast<float>(0.0), m_next_lane.corridor_list.at(i_corr).reference_line.start_range);
                    m_extend_lineGeometry.end_range = std::max( const_factorExtendReferenceline*_EV_EndPositionToReferenceLine_vs, const_minCorridorLength);
                    m_extend_lineGeometry.end_range = std::min( m_extend_lineGeometry.end_range, const_maxCorridorLength);
                    #if USING_ROS2 != 0
                    if(static_cast<bool>(element.is_end_out_of_range) == static_cast<bool>(true)){  // lane is end in range
                        float boundary_end_range = const_maxCorridorLength;
                        if(!element.left_lane_marker_id_list.empty()){
                            uint32_t left_lanemarker_id = element.left_lane_marker_id_list.front();
                            for(auto& element2 : _pd_lanemarkerlist_output){
                                if(static_cast<uint32_t>(element2.id) == left_lanemarker_id){
                                    if(!element2.front_lane_boundary_list.empty()){
                                        boundary_end_range = std::min(boundary_end_range, element2.front_lane_boundary_list.back().line_geometry.end_range);//sampling point will not be updated if not provied by RM, so do not use it
                                        break;
                                    }
                                }
                            }
                        }
                        if(!element.right_lane_marker_id_list.empty()){
                            uint32_t right_lanemarker_id = element.right_lane_marker_id_list.front();
                            for(auto& element2 : _pd_lanemarkerlist_output){
                                if(static_cast<uint32_t>(element2.id) == right_lanemarker_id){
                                    if(!element2.front_lane_boundary_list.empty()){
                                        boundary_end_range = std::min(boundary_end_range, element2.front_lane_boundary_list.back().line_geometry.end_range);//sampling point will not be updated if not provied by RM, so do not use it
                                        break;
                                    }
                                }
                            }
                        }
                        m_extend_lineGeometry.end_range = std::min( m_extend_lineGeometry.end_range, boundary_end_range );
                    }
                    #endif  // USING_ROS2
                    if((m_extend_lineGeometry.end_range-m_extend_lineGeometry.start_range) > 0.1){    // meanful to update
                        if(m_next_lane.corridor_list.at(i_corr).reference_line.end_range < m_extend_lineGeometry.end_range){    //length is not enough
                            uint32_t left_lanemarker_id = element.left_lane_marker_id_list.empty()? 0 : element.left_lane_marker_id_list.front();
                            std::vector<LaneBoundary> left_laneboundary_list;
                            for(auto& element2 : _pd_lanemarkerlist_output){
                                if(static_cast<uint32_t>(element2.id) == left_lanemarker_id){
                                    left_laneboundary_list = element2.front_lane_boundary_list;
                                    break;
                                }
                            }
                            uint32_t right_lanemarker_id = element.right_lane_marker_id_list.empty()? 0 : element.right_lane_marker_id_list.front();
                            std::vector<LaneBoundary> right_laneboundary_list;
                            for(auto& element2 : _pd_lanemarkerlist_output){
                                if(static_cast<uint32_t>(element2.id) == right_lanemarker_id){
                                    right_laneboundary_list = element2.front_lane_boundary_list;
                                    break;
                                }
                            }
                            Position_Cartesian selectNewPointFit;
                            std::vector<Position_Cartesian> selectNewPointFitList;
                            selectNewPointFitList.reserve(sizeof(const_interpolation)/sizeof(float));
                            for(decltype(sizeof(const_interpolation)/sizeof(float)) i=0;i<sizeof(const_interpolation)/sizeof(float);i++){
                                selectNewPointFit.x_Cartesian = m_extend_lineGeometry.start_range + const_interpolation[i] * (m_extend_lineGeometry.end_range - m_extend_lineGeometry.start_range);
                                selectNewPointFit.y_Cartesian = 0.0;

                                if(selectNewPointFit.x_Cartesian<=m_next_lane.corridor_list.at(i_corr).reference_line.end_range){
                                    selectNewPointFit.y_Cartesian = m_next_lane.corridor_list.at(i_corr).reference_line.c0 + 
                                                                    m_next_lane.corridor_list.at(i_corr).reference_line.c1 * selectNewPointFit.x_Cartesian + 
                                                                    m_next_lane.corridor_list.at(i_corr).reference_line.c2 * selectNewPointFit.x_Cartesian * selectNewPointFit.x_Cartesian + 
                                                                    m_next_lane.corridor_list.at(i_corr).reference_line.c3 * selectNewPointFit.x_Cartesian * selectNewPointFit.x_Cartesian * selectNewPointFit.x_Cartesian; 
                                    selectNewPointFitList.emplace_back(selectNewPointFit);
                                }
                                else{
                                    bool flag_find_leftpoint = false;
                                    float leftpoint_y = 0.0;
                                    for(auto& element2 : left_laneboundary_list){
                                        if((element2.line_geometry.start_range<=selectNewPointFit.x_Cartesian) && (element2.line_geometry.end_range>=selectNewPointFit.x_Cartesian)){
                                            leftpoint_y = element2.line_geometry.c0 + 
                                                            element2.line_geometry.c1 * selectNewPointFit.x_Cartesian + 
                                                            element2.line_geometry.c2 * selectNewPointFit.x_Cartesian * selectNewPointFit.x_Cartesian + 
                                                            element2.line_geometry.c3 * selectNewPointFit.x_Cartesian * selectNewPointFit.x_Cartesian * selectNewPointFit.x_Cartesian; 
                                            flag_find_leftpoint=true;
                                            break;
                                        }
                                    }
                                    bool flag_find_rightpoint = false;
                                    float rightpoint_y = 0.0;
                                    for(auto& element2 : right_laneboundary_list){
                                        if((element2.line_geometry.start_range<=selectNewPointFit.x_Cartesian) && (element2.line_geometry.end_range>=selectNewPointFit.x_Cartesian)){
                                            rightpoint_y = element2.line_geometry.c0 + 
                                                            element2.line_geometry.c1 * selectNewPointFit.x_Cartesian + 
                                                            element2.line_geometry.c2 * selectNewPointFit.x_Cartesian * selectNewPointFit.x_Cartesian + 
                                                            element2.line_geometry.c3 * selectNewPointFit.x_Cartesian * selectNewPointFit.x_Cartesian * selectNewPointFit.x_Cartesian; 
                                            flag_find_rightpoint=true;
                                            break;
                                        }
                                    }
                                    if(flag_find_leftpoint && flag_find_rightpoint){
                                        selectNewPointFit.y_Cartesian = 0.5 * (leftpoint_y+rightpoint_y);
                                        selectNewPointFitList.emplace_back(selectNewPointFit);
                                    }
                                }
                            }
                            std::unique_ptr<float[]> newReferenceLineCoeff(new float[4]);
                            int num_fit=0;
                            while(PolynomialFitting(selectNewPointFitList,3,newReferenceLineCoeff)>const_fittingErrorThreshold){
                                num_fit++;
                                if(num_fit>3){
                                    newReferenceLineCoeff[1]=(fabs(selectNewPointFitList.back().x_Cartesian-selectNewPointFitList.front().x_Cartesian)<0.001) ? 0.0 : \
                                                                (selectNewPointFitList.back().y_Cartesian-selectNewPointFitList.front().y_Cartesian) / \
                                                                (selectNewPointFitList.back().x_Cartesian-selectNewPointFitList.front().x_Cartesian);
                                    newReferenceLineCoeff[0]=selectNewPointFitList.front().y_Cartesian - \
                                                                newReferenceLineCoeff[1] * selectNewPointFitList.front().x_Cartesian;
                                    newReferenceLineCoeff[2]=0.0;
                                    newReferenceLineCoeff[3]=0.0;
                                    break;
                                }
                            }
                            m_extend_lineGeometry.c0 = newReferenceLineCoeff[0];
                            m_extend_lineGeometry.c1 = newReferenceLineCoeff[1];
                            m_extend_lineGeometry.c2 = newReferenceLineCoeff[2];
                            m_extend_lineGeometry.c3 = newReferenceLineCoeff[3];
                            
                            if(selectNewPointFitList.size()>1){
                                m_next_lane.corridor_list.at(i_corr).reference_line = m_extend_lineGeometry;
                            }
                        }
                    }
                }
            }

            std::vector<Corridor>(m_next_lane.corridor_list).swap(m_next_lane.corridor_list); //capacity()
            // back_corridor_list    back is not necessary to BA, necessary for plot
            #if FLAG_PDFP_SAVECSV != 0
            for(auto& element2 : element.back_corridor_list){
                if(fabs(element2.reference_line.c0)>1000.0f) continue;  // estimate availability
                Corridor m_next_corridor;
                float corridor_start_range_y =  element2.reference_line.c0 + 
                                                element2.reference_line.c1 * element2.reference_line.start_range +
                                                element2.reference_line.c2 * element2.reference_line.start_range * element2.reference_line.start_range +
                                                element2.reference_line.c3 * element2.reference_line.start_range * element2.reference_line.start_range * element2.reference_line.start_range;
                float corridor_end_range_y =    element2.reference_line.c0 + 
                                                element2.reference_line.c1 * element2.reference_line.end_range +
                                                element2.reference_line.c2 * element2.reference_line.end_range * element2.reference_line.end_range +
                                                element2.reference_line.c3 * element2.reference_line.end_range * element2.reference_line.end_range * element2.reference_line.end_range;
                if(true/*corridor_end_range_s > _EV_EndPositionToReferenceLine_s*/){
                    m_next_corridor.corridor_id = element2.corridor_id;
                    m_next_corridor.feature_tag = element2.feature_tag;
                    m_next_corridor.ilc_object_target = element2.ilc_object_target;
                    m_next_corridor.disenable_reason = element2.disenable_reason;
                    m_next_corridor.drivable_width = element2.drivable_width;
                    
                    // reference_line
                    float x_range = element2.reference_line.end_range-element2.reference_line.start_range;
                    if(fabs(x_range)<0.005)         {x_range = std::max(x_range,0.01f);}
                    Position_Cartesian selectNewPointFit;
                    std::vector<Position_Cartesian> selectNewPointFitList;
                    selectNewPointFitList.reserve(sizeof(const_interpolation)/sizeof(float));
                    for(decltype(sizeof(const_interpolation)/sizeof(float)) i=0;i<sizeof(const_interpolation)/sizeof(float);i++){
                        float last_x =  element2.reference_line.start_range + const_interpolation[i] * x_range;
                        float last_y =  element2.reference_line.c0 + 
                                        element2.reference_line.c1 * last_x +
                                        element2.reference_line.c2 * last_x * last_x +
                                        element2.reference_line.c3 * last_x * last_x * last_x;
                        selectNewPointFit = calcu_EndCartesianPoint_BaseOnEVMove(last_x,last_y);
                        selectNewPointFitList.emplace_back(selectNewPointFit);
                    }
                    std::unique_ptr<float[]> newReferenceLineCoeff(new float[4]);
                    int num_fit=0;
                    while(PolynomialFitting(selectNewPointFitList,3,newReferenceLineCoeff)>const_fittingErrorThreshold){
                        num_fit++;
                        if(num_fit>3){
                            newReferenceLineCoeff[1]=(fabs(selectNewPointFitList.back().x_Cartesian-selectNewPointFitList.front().x_Cartesian)<0.001) ? 0.0 : \
                                                        (selectNewPointFitList.back().y_Cartesian-selectNewPointFitList.front().y_Cartesian) / \
                                                        (selectNewPointFitList.back().x_Cartesian-selectNewPointFitList.front().x_Cartesian);
                            newReferenceLineCoeff[0]=selectNewPointFitList.front().y_Cartesian - \
                                                        newReferenceLineCoeff[1] * selectNewPointFitList.front().x_Cartesian;
                            newReferenceLineCoeff[2]=0.0;
                            newReferenceLineCoeff[3]=0.0;
                            break;
                        }
                    }

                    m_next_corridor.reference_line.start_range = calcu_EndCartesianPoint_BaseOnEVMove(element2.reference_line.start_range,corridor_start_range_y).x_Cartesian;
                    m_next_corridor.reference_line.end_range = calcu_EndCartesianPoint_BaseOnEVMove(element2.reference_line.end_range,corridor_end_range_y).x_Cartesian;
                    m_next_corridor.reference_line.c0 = newReferenceLineCoeff[0];
                    m_next_corridor.reference_line.c1 = newReferenceLineCoeff[1];
                    m_next_corridor.reference_line.c2 = newReferenceLineCoeff[2];
                    m_next_corridor.reference_line.c3 = newReferenceLineCoeff[3];
                    
                    // object           // TODO add new interface in future
                    m_next_corridor.object_in_corridor_list.reserve(128);
                    m_next_corridor.relevant_object_corridor_list.reserve(128);
                    ObjectLocation m_next_object;
                    float durat_time = _m_last_RoadInfo.passage_list.at(psg_Num).duration_time;
                    for(auto& element3 : element2.object_in_corridor_list){
                        m_next_object.object_id = element3.object_id;

                        SLPoint obj_next_temporary = element3.reference_point2corridor_frenet;
                        float t_last = 0.0;
                        for(float t=0.0; t<=durat_time; t=t+0.5){

                            float dds_SV_temporary = 0.0f;
                            if(t<element3.reference_point2corridor_frenet.dds_valid_time)  {
                                dds_SV_temporary = element3.reference_point2corridor_frenet.dds;
                                obj_next_temporary.dds_valid_time = obj_next_temporary.dds_valid_time - (t-t_last);
                                obj_next_temporary.dds_valid_time = std::max(obj_next_temporary.dds_valid_time, 0.0f);
                            }
                            else{
                                dds_SV_temporary = 0.0f;
                                obj_next_temporary.dds = 0.0;
                                obj_next_temporary.dds_valid_time = 0.0;
                            }
                            float dl_SV_temporary  = 0.0f;
                            if(t<element3.reference_point2corridor_frenet.dl_valid_time)   {
                                dl_SV_temporary = element3.reference_point2corridor_frenet.dl;
                                obj_next_temporary.dl_valid_time = obj_next_temporary.dl_valid_time - (t-t_last);
                                obj_next_temporary.dl_valid_time = std::max(obj_next_temporary.dl_valid_time, 0.0f);
                            }
                            else{
                                dl_SV_temporary = 0.0;
                                obj_next_temporary.dl = 0.0;
                                obj_next_temporary.dl_valid_time = 0.0;
                            }
                            /*float ddl_SV_temporary = 0.0f;
                            if(t<element3.reference_point2corridor_frenet.ddl_valid_time)  {
                                ddl_SV_temporary = element3.reference_point2corridor_frenet.ddl;
                                obj_next_temporary.ddl_valid_time = obj_next_temporary.ddl_valid_time - (t-t_last);
                                obj_next_temporary.ddl_valid_time = std::max(obj_next_temporary.ddl_valid_time, 0.0f);
                            }
                            else{
                                ddl_SV_temporary = 0.0;
                                obj_next_temporary.ddl = 0.0;
                                obj_next_temporary.ddl_valid_time = 0.0;
                            }*/

                            obj_next_temporary.ds = obj_next_temporary.ds + dds_SV_temporary * (t-t_last);
                            obj_next_temporary.s = obj_next_temporary.s + obj_next_temporary.ds * (t-t_last);
                            obj_next_temporary.l = obj_next_temporary.l + dl_SV_temporary * (t-t_last);

                            t_last = t;
                        }
                        obj_next_temporary.s = obj_next_temporary.s - _EV_EndPositionToReferenceLine_s;
                        m_next_object.reference_point2corridor_frenet = obj_next_temporary;

                        Position_Cartesian objectXY;    objectXY.x_Cartesian = 9999.9f; objectXY.y_Cartesian = 9999.9f;
                        if(m_next_corridor.corridor_id == _m_last_RoadInfo.passage_list.at(psg_Num).corridor_id){       // only work when corridor is the passage corridor
                        // if(m_next_corridor.corridor_id == static_cast<decltype(m_next_corridor.corridor_id)>(corridorIDwithObject)){       // only work when corridor is the passage corridor
                            objectXY = FrenetToCartesian(_m_referenceLine_Cartesian_list, 
                                                        m_next_object.reference_point2corridor_frenet.s,
                                                        m_next_object.reference_point2corridor_frenet.l);

                            if(objectXY_inCorridor_list.count(static_cast<uint32_t>(m_next_object.object_id))==0){
                                objectXY_inCorridor_list[static_cast<uint32_t>(m_next_object.object_id)] = objectXY;
                            }
                        }
                        // objectLocation relative_center_x
                        m_next_object.relative_center_x = objectXY.x_Cartesian;
                        // objectLocation relative_center_y
                        m_next_object.relative_center_y = objectXY.y_Cartesian;
                        // objectLocation size_width
                        m_next_object.size_width = element3.size_width;
                        // objectLocation size_length
                        m_next_object.size_length = element3.size_length;
                        // objectLocation classification
                        m_next_object.classification = element3.classification;

                        if(fabs(m_next_object.reference_point2corridor_frenet.l)<=element2.drivable_width*0.5){   // object_in_corridor_list
                            m_next_corridor.object_in_corridor_list.emplace_back(m_next_object);
                        }
                        else{   // relevant_object_corridor_list
                            m_next_corridor.relevant_object_corridor_list.emplace_back(m_next_object);
                        }
                    }
                    for(auto& element3 : element2.relevant_object_corridor_list){
                        m_next_object.object_id = element3.object_id;

                        SLPoint obj_next_temporary = element3.reference_point2corridor_frenet;
                        float t_last = 0.0;
                        for(float t=0.0; t<=durat_time; t=t+0.5){

                            float dds_SV_temporary = 0.0f;
                            if(t<element3.reference_point2corridor_frenet.dds_valid_time)  {
                                dds_SV_temporary = element3.reference_point2corridor_frenet.dds;
                                obj_next_temporary.dds_valid_time = obj_next_temporary.dds_valid_time - (t-t_last);
                                obj_next_temporary.dds_valid_time = std::max(obj_next_temporary.dds_valid_time, 0.0f);
                            }
                            else{
                                dds_SV_temporary = 0.0f;
                                obj_next_temporary.dds = 0.0;
                                obj_next_temporary.dds_valid_time = 0.0;
                            }
                            float dl_SV_temporary  = 0.0f;
                            if(t<element3.reference_point2corridor_frenet.dl_valid_time)   {
                                dl_SV_temporary = element3.reference_point2corridor_frenet.dl;
                                obj_next_temporary.dl_valid_time = obj_next_temporary.dl_valid_time - (t-t_last);
                                obj_next_temporary.dl_valid_time = std::max(obj_next_temporary.dl_valid_time, 0.0f);
                            }
                            else{
                                dl_SV_temporary = 0.0;
                                obj_next_temporary.dl = 0.0;
                                obj_next_temporary.dl_valid_time = 0.0;
                            }
                            /*float ddl_SV_temporary = 0.0f;
                            if(t<element3.reference_point2corridor_frenet.ddl_valid_time)  {
                                ddl_SV_temporary = element3.reference_point2corridor_frenet.ddl;
                                obj_next_temporary.ddl_valid_time = obj_next_temporary.ddl_valid_time - (t-t_last);
                                obj_next_temporary.ddl_valid_time = std::max(obj_next_temporary.ddl_valid_time, 0.0f);
                            }
                            else{
                                ddl_SV_temporary = 0.0;
                                obj_next_temporary.ddl = 0.0;
                                obj_next_temporary.ddl_valid_time = 0.0;
                            }*/

                            obj_next_temporary.ds = obj_next_temporary.ds + dds_SV_temporary * (t-t_last);
                            obj_next_temporary.s = obj_next_temporary.s + obj_next_temporary.ds * (t-t_last);
                            obj_next_temporary.l = obj_next_temporary.l + dl_SV_temporary * (t-t_last);

                            t_last = t;
                        }
                        obj_next_temporary.s = obj_next_temporary.s - _EV_EndPositionToReferenceLine_s;
                        m_next_object.reference_point2corridor_frenet = obj_next_temporary;

                        Position_Cartesian objectXY;    objectXY.x_Cartesian = 9999.9f; objectXY.y_Cartesian = 9999.9f;
                        if(m_next_corridor.corridor_id == _m_last_RoadInfo.passage_list.at(psg_Num).corridor_id){       // only work when corridor is the passage corridor
                        // if(m_next_corridor.corridor_id == static_cast<decltype(m_next_corridor.corridor_id)>(corridorIDwithObject)){       // only work when corridor is the passage corridor
                            objectXY = FrenetToCartesian(_m_referenceLine_Cartesian_list, 
                                                        m_next_object.reference_point2corridor_frenet.s,
                                                        m_next_object.reference_point2corridor_frenet.l);

                            if(objectXY_inCorridor_list.count(static_cast<uint32_t>(m_next_object.object_id))==0){
                                objectXY_inCorridor_list[static_cast<uint32_t>(m_next_object.object_id)] = objectXY;
                            }
                        }
                        // objectLocation relative_center_x
                        m_next_object.relative_center_x = objectXY.x_Cartesian;
                        // objectLocation relative_center_y
                        m_next_object.relative_center_y = objectXY.y_Cartesian;
                        // objectLocation size_width
                        m_next_object.size_width = element3.size_width;
                        // objectLocation size_length
                        m_next_object.size_length = element3.size_length;
                        // objectLocation classification
                        m_next_object.classification = element3.classification;

                        if(fabs(m_next_object.reference_point2corridor_frenet.l)<=element2.drivable_width*0.5){   // object_in_corridor_list
                            m_next_corridor.object_in_corridor_list.emplace_back(m_next_object);
                        }
                        else{   // relevant_object_corridor_list
                            m_next_corridor.relevant_object_corridor_list.emplace_back(m_next_object);
                        }
                    }
                    std::vector<ObjectLocation>(m_next_corridor.object_in_corridor_list).swap(m_next_corridor.object_in_corridor_list); //capacity()
                    std::vector<ObjectLocation>(m_next_corridor.relevant_object_corridor_list).swap(m_next_corridor.relevant_object_corridor_list); //capacity()
                    
                    m_next_lane.back_corridor_list.emplace_back(m_next_corridor);
                    
                }
            }
            std::vector<Corridor>(m_next_lane.back_corridor_list).swap(m_next_lane.back_corridor_list); //capacity()
            #endif

            // lane_type
            m_next_lane.lane_type = element.lane_type;
            // max_speed_limit
            std::vector<LaneControl> m_next_LaneControl_list;
            m_next_LaneControl_list.reserve(32);
            for(auto& element2 : element.max_speed_limit){
                LaneControl m_next_laneControl;

                if(true){                     // available
                    m_next_laneControl.start_range = element2.start_range - _EV_EndPositionToReferenceLine_s;
                    m_next_laneControl.end_range = element2.end_range - _EV_EndPositionToReferenceLine_s;
                    m_next_laneControl.value = element2.value;
                    m_next_LaneControl_list.emplace_back(m_next_laneControl);
                }
            }
            std::vector<LaneControl>(m_next_LaneControl_list).swap(m_next_LaneControl_list);
            m_next_lane.max_speed_limit = m_next_LaneControl_list;
            // lane_width
            m_next_lane.lane_width = element.lane_width;
            #if USING_ROS2 != 0
            // is_start_out_of_range
            m_next_lane.is_start_out_of_range = element.is_start_out_of_range;
            // is_end_out_of_range
            m_next_lane.is_end_out_of_range = element.is_end_out_of_range;
            #endif  // USING_ROS2
            // lane_start_split_merge_type
            m_next_lane.lane_start_split_merge_type = element.lane_start_split_merge_type;
            // lane_end_split_merge_type
            m_next_lane.lane_end_split_merge_type = element.lane_end_split_merge_type;

        }


        pd_RoadModel_LaneList_output.emplace_back(m_next_lane);
    }
    std::vector<Lane>(pd_RoadModel_LaneList_output).swap(pd_RoadModel_LaneList_output); //capacity()

    /*************** update object Cartesian base on the same objectXY_inCorridor_list ****************/
    for(auto& element1 : pd_RoadModel_LaneList_output){
        for(auto& element2:element1.corridor_list){
            for(auto& element3 : element2.object_in_corridor_list){
                if((element3.relative_center_x > 9000.0f)&&(element3.relative_center_y > 9000.0f)){
                    element3.relative_center_x = objectXY_inCorridor_list[element3.object_id].x_Cartesian;
                    element3.relative_center_y = objectXY_inCorridor_list[element3.object_id].y_Cartesian;
                }
            }
            for(auto& element3 : element2.relevant_object_corridor_list){
                if((element3.relative_center_x > 9000.0f)&&(element3.relative_center_y > 9000.0f)){
                    element3.relative_center_x = objectXY_inCorridor_list[element3.object_id].x_Cartesian;
                    element3.relative_center_y = objectXY_inCorridor_list[element3.object_id].y_Cartesian;
                }
            }
        }
        for(auto& element2:element1.back_corridor_list){
            for(auto& element3 : element2.object_in_corridor_list){
                if((element3.relative_center_x > 9000.0f)&&(element3.relative_center_y > 9000.0f)){
                    element3.relative_center_x = objectXY_inCorridor_list[element3.object_id].x_Cartesian;
                    element3.relative_center_y = objectXY_inCorridor_list[element3.object_id].y_Cartesian;
                }
            }
            for(auto& element3 : element2.relevant_object_corridor_list){
                if((element3.relative_center_x > 9000.0f)&&(element3.relative_center_y > 9000.0f)){
                    element3.relative_center_x = objectXY_inCorridor_list[element3.object_id].x_Cartesian;
                    element3.relative_center_y = objectXY_inCorridor_list[element3.object_id].y_Cartesian;
                }
            }
        }
    }


    #if SAVE_Timer_FILE != 0
    outfile_lanelist<<std::endl;
    outfile_lanelist.close();
    #endif

}


/**
 * @brief
 * @param
 */
void RoadModel_LaneList_PDtoBA::process(){

    if(initFlag != true){
        // std::cout<< "ATTENTION: Data reading of RoadMode_LaneList_PDtoBA error" <<std::endl;
        return;
    }

    calcu_LaneList_based_DurationTime();
}


/***************************************************************************************************
****************************************************************************************************
***************************************************************************************************/

/**
 * @brief
 * @param
 */
RoadModel_base::RoadModel_base(const PD::PD_Input &m_pd_input, float m_EV_end_s, float m_EV_end_l, float m_EV_end_vs, float m_EV_end_vl){
    if(init(m_pd_input,m_EV_end_s,m_EV_end_l,m_EV_end_vs,m_EV_end_vl) == false)
    return;
}


/**
 * @brief 
 * @param
 * @return
 */
bool RoadModel_base::init(const PD::PD_Input &m_pd_input, float m_EV_end_s, float m_EV_end_l, float m_EV_end_vs, float m_EV_end_vl){
    bool Flag=true;
    Flag = Flag && (static_cast<decltype(m_pd_input.rm_roadmodel.size())>(rm_Num)<m_pd_input.rm_roadmodel.size());                                 //chech rm_roadmodel
    if(Flag){
        Flag = Flag && (static_cast<decltype(m_pd_input.rm_roadmodel.at(rm_Num).passage_list.size())>(psg_Num)<m_pd_input.rm_roadmodel.at(rm_Num).passage_list.size());    //chech passage_list
    }
    /*Flag = Flag && !(m_pd_input.rm_roadmodel.empty());                //chech rm_roadmodel
    for(auto& element : m_pd_input.rm_roadmodel){
        // Flag = Flag && !(element.lane_list.empty());                //check lane_list
        Flag = Flag && !(element.passage_list.empty());             //check passage_list
        // for(auto& element2 : element.lane_list){
        //     Flag = Flag && !(element2.corridor_list.empty());       //check corridor_list
        // }


    }*/

    if(Flag==false){
        initFlag=false;
    }
    else{

        initFlag=true;

        _m_referenceLine_Cartesian_list.clear();


        _m_last_RoadInfo = m_pd_input.rm_roadmodel.at(rm_Num);
        _EV_EndPositionToReferenceLine_s = m_EV_end_s;
        _EV_EndPositionToReferenceLine_l = m_EV_end_l;
        _EV_EndPositionToReferenceLine_vs = m_EV_end_vs;
        _EV_EndPositionToReferenceLine_vl = m_EV_end_vl;

    }
    return initFlag;
}


/**
 * @brief
 * @param
 */
void RoadModel_base::calcu_referenceLineList_Cartesian(){

    bool flagFindReferenceLine = false;
    for(auto& element : _m_last_RoadInfo.lane_list){
        for(auto& element2 : element.corridor_list){
            if(element2.corridor_id == _m_last_RoadInfo.passage_list.at(psg_Num).corridor_id){

                _m_referenceLine_Cartesian_list.insert(_m_referenceLine_Cartesian_list.end(), element2.reference_line);
                _m_referenceLine_Cartesian_list.front().start_range = 0.0f;
                flagFindReferenceLine = true;
            }
            if(flagFindReferenceLine)
            {break;}
        }
        if(flagFindReferenceLine)
        {break;}
    }
    if(!flagFindReferenceLine) return;

    #if 1       // extend const_referencelineStart to const_referencelineEnd   -250 or 0 to 1000 m 
        LineGeometry back_referenceLine;
        back_referenceLine.start_range = const_referencelineStart;
        back_referenceLine.end_range = _m_referenceLine_Cartesian_list.front().start_range;
        back_referenceLine.c3 = 0.0f;
        back_referenceLine.c2 = 0.0f;
        back_referenceLine.c1 = calcu_Y_polynomial(_m_referenceLine_Cartesian_list, _m_referenceLine_Cartesian_list.front().start_range, 1);
        back_referenceLine.c0 = calcu_Y_polynomial(_m_referenceLine_Cartesian_list, _m_referenceLine_Cartesian_list.front().start_range, 0)
                                    - back_referenceLine.c1 * _m_referenceLine_Cartesian_list.front().start_range;

        LineGeometry front_referenceLine;
        front_referenceLine.start_range = _m_referenceLine_Cartesian_list.front().end_range;
        front_referenceLine.end_range = const_referencelineEnd;
        front_referenceLine.c3 = 0.0f;
        front_referenceLine.c2 = 0.0f;
        front_referenceLine.c1 = calcu_Y_polynomial(_m_referenceLine_Cartesian_list, _m_referenceLine_Cartesian_list.front().end_range, 1);
        front_referenceLine.c0 = calcu_Y_polynomial(_m_referenceLine_Cartesian_list, _m_referenceLine_Cartesian_list.front().end_range, 0)
                                    - front_referenceLine.c1 * _m_referenceLine_Cartesian_list.front().end_range;
        
        _m_referenceLine_Cartesian_list.insert(_m_referenceLine_Cartesian_list.begin(), back_referenceLine);
        _m_referenceLine_Cartesian_list.insert(_m_referenceLine_Cartesian_list.end(), front_referenceLine);
    #endif
}


/**
 * @brief
 * @param
 */
float RoadModel_base::calcu_Y_polynomial(const std::list<LineGeometry>& m_polynomials, const float m_x, int m_derivative){

    float y_derivative = 999.9f;
    if(m_polynomials.empty()) return y_derivative;

    LineGeometry line_selected;
    auto it = m_polynomials.begin();
    for(; it != m_polynomials.end(); it++){
        if((m_x > (it->start_range-0.01f))&&(m_x < (it->end_range+0.01f))){
            line_selected = *it;
            break;
        }
    }
    if(it == m_polynomials.end()) line_selected=*(it--);

    y_derivative = calcu_Y_polynomial(line_selected, m_x, m_derivative);
    return y_derivative;
}
float RoadModel_base::calcu_Y_polynomial(const LineGeometry& m_polynomial, const float m_x, int m_derivative){
    float y_derivative = 999.9f;
    switch (m_derivative){
        case 0:
            y_derivative = m_polynomial.c0 + 
                            m_polynomial.c1 * m_x + 
                            m_polynomial.c2 * m_x * m_x + 
                            m_polynomial.c3 * m_x * m_x * m_x; 
            break;
        case 1:
            y_derivative = m_polynomial.c1 + 
                            m_polynomial.c2 * m_x * 2.0f + 
                            m_polynomial.c3 * m_x * m_x * 3.0f; 
            break;
        case 2:
            y_derivative = m_polynomial.c2 * 2.0f + 
                            m_polynomial.c3 * m_x * 3.0f * 2.0f; 
            break;
        case 3:
            y_derivative = m_polynomial.c3 * 3.0f * 2.0f; 
            break;
        default:
            break;
    }
    return y_derivative;
}


/**
 * @brief
 * @param
 */
Position_Frenet RoadModel_base::CartesianToFrenet(const std::list<LineGeometry>& referenceLine_Cartesian_list, const float m_x, const float m_y){

    Position_Frenet position_Frenet;

    if(referenceLine_Cartesian_list.empty()) {return position_Frenet;}

    float k_s = 1.0f;
    float k_l = 1.0f;

    float refer_x0 = m_x;
    findReferPointNewtonMethod(refer_x0, m_x, m_y, referenceLine_Cartesian_list);

    if(refer_x0<0.0f) k_s = -1.0f;
    if(m_y<calcu_Y_polynomial(referenceLine_Cartesian_list, refer_x0, 0)) k_l = -1.0f;

    position_Frenet.s_Frenet = k_s * calculateArcLengthAbs(referenceLine_Cartesian_list, refer_x0);
    position_Frenet.l_Frenet = k_l * calculateDisBetweenPoints({m_x,m_y},{refer_x0,calcu_Y_polynomial(referenceLine_Cartesian_list, refer_x0, 0)});

    return position_Frenet;
}
void RoadModel_base::findReferPointNewtonMethod(float& refer_x0, const float m_x, const float m_y, 
                                    const std::list<LineGeometry>& m_polynomials,
                                    const float step_x, const float accuracy_diff, const int cycleNum){
    int count = 0;
    float refer_x1 = refer_x0 + 0.1f;
    float dis_x_midd = calculateDisBetweenPoints({m_x,m_y},{refer_x0,calcu_Y_polynomial(m_polynomials, refer_x0, 0)});
    float dis_x_right = calculateDisBetweenPoints({m_x,m_y},{refer_x0+step_x,calcu_Y_polynomial(m_polynomials, refer_x0+step_x, 0)});
    float diff_x = (dis_x_right - dis_x_midd)/step_x;

    while(fabs(diff_x)>accuracy_diff){
        if(count>cycleNum) break;

        dis_x_midd = calculateDisBetweenPoints({m_x,m_y},{refer_x0,calcu_Y_polynomial(m_polynomials, refer_x0, 0)});
        dis_x_right = calculateDisBetweenPoints({m_x,m_y},{refer_x0+step_x,calcu_Y_polynomial(m_polynomials, refer_x0+step_x, 0)});
        diff_x = (dis_x_right - dis_x_midd)/step_x;

        refer_x0 = refer_x1;
        refer_x1 = refer_x1 - 0.07f*diff_x;   //modified Newton's Method

        count++;
    }
}
float RoadModel_base::calculateDisBetweenPoints(const std::pair<float,float>& point1, const std::pair<float,float>& point2){
    return pow((point1.first-point2.first)*(point1.first-point2.first) + 
                (point1.second-point2.second)*(point1.second-point2.second),0.5);
}
float RoadModel_base::calculateArcLengthAbs(const std::list<LineGeometry>& m_polynomials, const float refer_x){
    float s_referPoint = 0.0f;
    if(refer_x>0.0f){   // front
        for(auto it = m_polynomials.begin(); it != m_polynomials.end(); ++it){
            if(it->start_range > -0.1f){    // front
                if(refer_x < it->end_range){// in this polynomial
                    s_referPoint = s_referPoint + calculateArcIntegralSimpsonMethod(*it,refer_x);
                    break;
                }
                else{
                    s_referPoint = s_referPoint + calculateArcIntegralSimpsonMethod(*it,it->end_range);
                }
            }
        }
    }
    else{               // back
        for(auto it = m_polynomials.rbegin(); it != m_polynomials.rend(); ++it){
            if(it->end_range<0.1f){              // back
                if(refer_x > it->start_range){   // in this corridor
                    s_referPoint = s_referPoint + calculateArcIntegralSimpsonMethod(*it,refer_x);
                    break;
                }
                else{
                    s_referPoint = s_referPoint + calculateArcIntegralSimpsonMethod(*it,it->start_range);
                }
            }
        }
    }
    return s_referPoint;
}
float RoadModel_base::calculateArcIntegralSimpsonMethod(const LineGeometry& m_polynomial, const float refer_x){

    float arcStart = 0.0f;
    float arcEnd = 0.0f;
    if(m_polynomial.start_range<-0.1f){  // back
        arcStart = refer_x;
        arcEnd = m_polynomial.end_range;
    }
    else{   // front
        arcStart = m_polynomial.start_range;
        arcEnd = refer_x;
    }

    if(arcEnd<=arcStart) return 0.0f;
    // Simpson Method
    return (arcEnd-arcStart)/6.0f*(calculateDeltaArc(m_polynomial,arcStart) + 
                4.0f*calculateDeltaArc(m_polynomial,0.5*(arcStart+arcEnd)) + calculateDeltaArc(m_polynomial,arcEnd));
}
float RoadModel_base::calculateDeltaArc(const LineGeometry& m_polynomial, const float point_x){
    float gradient = calcu_Y_polynomial(m_polynomial, point_x, 1);
    return pow(1+pow(gradient,2),0.5);    
}


/**
 * @brief
 * @param
 */
Position_Cartesian RoadModel_base::FrenetToCartesian(const std::list<LineGeometry>& referenceLine_Cartesian_list, const float s, const float l){

    Position_Cartesian position_Cartesian;

    if(referenceLine_Cartesian_list.empty()) {return position_Cartesian;}

    float refer_x0 = s;
    findReferPointNewtonMethod(refer_x0, s, referenceLine_Cartesian_list);

    float refer_y0 = calcu_Y_polynomial(referenceLine_Cartesian_list, refer_x0, 0);
    float diff_y0 = calcu_Y_polynomial(referenceLine_Cartesian_list, refer_x0, 1);

    position_Cartesian.x_Cartesian = refer_x0 + l * (diff_y0 / pow(diff_y0*diff_y0+1.0f,0.5f));
    position_Cartesian.y_Cartesian = refer_y0 + l * (1.0f / pow(diff_y0*diff_y0+1.0f, 0.5f));

    return position_Cartesian;
}
void RoadModel_base::findReferPointNewtonMethod(float& refer_x0, const float m_s, 
                                    const std::list<LineGeometry>& m_polynomials,
                                    const float step_x, const float accuracy_diff, const int cycleNum){

    int count = 0;
    // float refer_x1 = refer_x0 + 0.1f;
    float k_arc = 1.0f; if(refer_x0<0.0f)k_arc=-1.0f;
    float s_x_midd = k_arc*calculateArcLengthAbs(m_polynomials, refer_x0) - m_s;
    // float s_x_right = calculateArcLengthAbs(m_polynomials, refer_x0+step_x) - m_s;

    while(fabs(s_x_midd) > accuracy_diff){
        if(count>cycleNum) break;

        k_arc = 1.0f; if(refer_x0<0.0f)k_arc=-1.0f;
        s_x_midd = k_arc*calculateArcLengthAbs(m_polynomials, refer_x0) - m_s;
        // s_x_right = calculateArcLengthAbs(m_polynomials, refer_x0+step_x)-m_s;

        // refer_x0 = refer_x1;
        refer_x0 = refer_x0 - 0.07f * s_x_midd; //modified Newton's Method

        count += 1;
    }

}


/**
 * @brief
 * @param
 */
int16_t RoadModel_base::binarySearchNearestMark(const std::vector<Position_Cartesian>& m_referencePoint_Cartesian_list, const float x, const float y){
    int16_t nearest_Mark = 0;
    if(m_referencePoint_Cartesian_list.size() < 3)
    {return nearest_Mark;}
    int16_t leftMark = 0;
    int16_t rightMark = m_referencePoint_Cartesian_list.size()-1;    rightMark=std::max(rightMark,leftMark);
    int16_t midMark = (leftMark+rightMark)/2;
    std::map<int16_t, float> mapNear;
    while(true){
        if((rightMark-leftMark)<5){
            mapNear.clear();
            for(int16_t i=std::max(static_cast<int16_t>(0),static_cast<int16_t>(leftMark-1));
                i<=std::min(static_cast<int16_t>(rightMark+1),static_cast<int16_t>(m_referencePoint_Cartesian_list.size()-1));
                i++){
                mapNear.insert(std::map<int16_t, float>::value_type (i, 
                        pow(m_referencePoint_Cartesian_list.at(i).x_Cartesian - x,2.0) + 
                        pow(m_referencePoint_Cartesian_list.at(i).y_Cartesian - y,2.0) ));
            }
            break;
        }
        float left_dis_square = pow(m_referencePoint_Cartesian_list.at(leftMark).x_Cartesian - x,2.0) + 
                                pow(m_referencePoint_Cartesian_list.at(leftMark).y_Cartesian - y,2.0);
        float right_dis_square =pow(m_referencePoint_Cartesian_list.at(rightMark).x_Cartesian - x,2.0) + 
                                pow(m_referencePoint_Cartesian_list.at(rightMark).y_Cartesian - y,2.0);
        /*float mid_dis =   pow(  pow(m_referencePoint_Cartesian_list.at(midMark).x_Cartesian - x,2.0) + 
                                pow(m_referencePoint_Cartesian_list.at(midMark).y_Cartesian - y,2.0),0.5);*/
        if(left_dis_square>right_dis_square){leftMark = midMark;}
        else{rightMark = midMark;}
        midMark = (leftMark+rightMark)/2;
    }

    float nearset_dis_square=200.0f * 200.0f;
    for(std::map<int16_t, float>::iterator iter = mapNear.begin(); iter != mapNear.end(); iter++){
        if(iter->second < nearset_dis_square){
            nearset_dis_square = iter->second;
            nearest_Mark = iter->first;
        }
    }
    return nearest_Mark;
}




/**
 * @brief
 * @param
 */
void RoadModel_base::calcu_transforFactor_BaseOnEVMove(){

    // calculate the xy position
    _deltaXY_translation = FrenetToCartesian(_m_referenceLine_Cartesian_list, _EV_EndPositionToReferenceLine_s, _EV_EndPositionToReferenceLine_l);

    _deltaXY_translation.x_Cartesian = mapSaturationMeter(_deltaXY_translation.x_Cartesian);
    _deltaXY_translation.y_Cartesian = mapSaturationMeter(_deltaXY_translation.y_Cartesian);

    // calculate the angles/rad
    float Heading_EVEnd_To_FrenetEnd = 0.0;
    {
        // float t = _m_last_RoadInfo.passage_list.at(psg_Num).duration_time;
        float vl = _EV_EndPositionToReferenceLine_vl;
        float vs = _EV_EndPositionToReferenceLine_vs;
        vs=std::max(vs,0.001f);
        Heading_EVEnd_To_FrenetEnd = atan(vl/vs);
    }
    float Angle_FrenetEnd_To_EVStart_ = 0.0;
    {
        Position_Cartesian position_Cartesian_FrenetEnd = FrenetToCartesian(_m_referenceLine_Cartesian_list, _EV_EndPositionToReferenceLine_s, 0.0);
        float tange =   calcu_Y_polynomial(_m_referenceLine_Cartesian_list, position_Cartesian_FrenetEnd.x_Cartesian, 1);
        Angle_FrenetEnd_To_EVStart_ = atan(tange);
    }
    float deltaHeading_EV = Heading_EVEnd_To_FrenetEnd + Angle_FrenetEnd_To_EVStart_;
    _deltaHeading_EV = std::max(std::min(deltaHeading_EV,3.14159f/2.0f),-3.14159f/2.0f);


}


/**
 * @brief
 * @param
 */
Position_Cartesian RoadModel_base::calcu_EndCartesianPoint_BaseOnEVMove(const float point_x_start, const float point_y_start){

    Position_Cartesian _position_Cartesian;

    // 1 translation 平移
    float x_translation = point_x_start - _deltaXY_translation.x_Cartesian;
    float y_translation = point_y_start - _deltaXY_translation.y_Cartesian;
    // 2 rotate 旋转
    _position_Cartesian.x_Cartesian = x_translation * cos(-_deltaHeading_EV) - y_translation * sin(-_deltaHeading_EV);
    _position_Cartesian.y_Cartesian = y_translation * cos(-_deltaHeading_EV) + x_translation * sin(-_deltaHeading_EV);

    _position_Cartesian.x_Cartesian = std::max(_position_Cartesian.x_Cartesian, -1000.0f);
    _position_Cartesian.x_Cartesian = std::min(_position_Cartesian.x_Cartesian,  1000.0f);
    _position_Cartesian.y_Cartesian = std::max(_position_Cartesian.y_Cartesian, -1000.0f);
    _position_Cartesian.y_Cartesian = std::min(_position_Cartesian.y_Cartesian,  1000.0f);

    return _position_Cartesian;
}


/**
 * @brief
 * @param
 */
float RoadModel_base::PolynomialFitting(const std::vector<Position_Cartesian>& point, const int degree, std::unique_ptr<float[]> &new_coeff){
#if 0
    int N = std::min(static_cast<int>(point.size()-1), degree);
    new_coeff.reset(new float[degree+1]);

    Eigen::MatrixXd A(point.size(), N + 1);
    Eigen::MatrixXd B(point.size(), 1);

    for (unsigned int i = 0; i < point.size(); ++i) {  
        for (int n = 0; n < N+1; n++) {
            A(i, n) = pow(point.at(i).x_Cartesian, n);
        }
    }
    for (unsigned int i = 0; i < point.size(); ++i) {
        B(i, 0) = point.at(i).y_Cartesian;
    }

    Eigen::MatrixXd W(N+1, 1);
    W = (A.transpose() * A).inverse() * A.transpose() * B;

    for(int i=0; i<N+1; i++){
        if(std::isnan(W(i,0))){
            new_coeff[i] = 0.0;
        }
        else{
            new_coeff[i] = W(i,0);  
        }
    }
    if(N<degree){
        for(unsigned int i = N+1; i < degree+1; i++){
            new_coeff[i] = 0.0;
        }
    }
#endif
    int N = std::min(static_cast<int>(point.size()-1), degree);
    new_coeff.reset(new float[degree+1]);

    // float a[N+1][point.size()];
    // float b[N+1][N+1+1];
    // float c[N+1];
    std::vector< std::vector<float> > a(N+1, std::vector<float>(point.size(),0.0f));
    std::vector< std::vector<float> > b(N+1, std::vector<float>(N+1+1,0.0f));
    std::vector<float> c(N+1,0.0f);

    {
        int i,j,k;
        float temp;
        for(i=0;i<N+1;i++)
        for(j=0;static_cast<decltype(point.size())>(j)<point.size();j++)
        {
            temp=1.0f;
            for(k=0;k<i;k++)
                temp*=point.at(j).x_Cartesian;
            a[i][j]=temp;
        }
    }

    {
        int i,j,k;
        float temp2;
        for(i=0;i<N+1;i++)
        {
            for(j=0;j<N+1;j++)
            {
                temp2=0.0;
                for(k=0;static_cast<decltype(point.size())>(k)<point.size();k++)
                    temp2+= a[i][k]*a[j][k];
                b[i][j] = temp2;
            }
            temp2=0;
            for(k=0;static_cast<decltype(point.size())>(k)<point.size();k++)
            {
                temp2+=point.at(k).y_Cartesian * a[i][k];
                b[i][j] = temp2;
            }
        }
    }

    {
        int i,r,k,j;
        // float s[N+1],t[N+1];
        std::vector<float> s(N+1,0.0f),t(N+1,0.0f);
        float max;
        for(r=0;r<N+1;r++)
        {
            max=0.0;
            j=r;
            for(i=r;i<N+1;i++)
            {
                s[i]=b[i][r];
                for(k=0;k<r;k++)
                    s[i]-=b[i][k]*b[k][r];
                s[i]=s[i]>0?s[i]:-s[i];
                if(s[i]>max){
                    j=i;
                    max=s[i];
                }
            }
            if(j!=r)
            {
                for(i=0;i<N+2;i++)
                {
                    float swap = b[r][i];
                    b[r][i]=b[j][i];
                    b[j][i]=swap;
                }
            }
            for(i=r;i<N+2;i++)
                for(k=0;k<r;k++){
                    b[r][i]-=b[r][k]*b[k][i];
                }
            for(i=r+1;i<N+1;i++)
            {
                for(k=0;k<r;k++)
                    b[i][r]-=b[i][k]*b[k][r];
                b[i][r]/=b[r][r];
            }
        }
        for(i=0;i<N+1;i++)
            t[i]=b[i][N+1];
        for(i=N;i>=0;i--)
        {
            for(r=N;r>i;r--)
                t[i]-=b[i][r]*c[r];
            c[i]=t[i]/b[i][i];
        }
    }

    for(int i=0;i<N+1;i++){
        if(std::isnan(c[i]) || std::isinf(c[i])){
            new_coeff[i] = 0.0;
        }
        else{
            new_coeff[i] = c[i];
        }
    }
    if(N<degree){
        for(unsigned int i = N+1; static_cast<decltype(N+1)>(i) < degree+1; i++){
            new_coeff[i] = 0.0;
        }
    }

    float totalErr=0.0;
    for(int i=0;static_cast<decltype(point.size())>(i)<point.size();i++){
	    float theErr=0.0;
	    for(int j=0;j<degree+1;j++){
	        theErr=theErr+new_coeff[j]*pow(point.at(i).x_Cartesian,j);
	    }
	    theErr=theErr-point.at(i).y_Cartesian;
        totalErr = totalErr + theErr*theErr;
    }
    return totalErr;
}


/**
 * @brief
 * @param
 */
float RoadModel_base::curveArcLength(const LineGeometry& m_polynomial, const float start_x, const float end_x){
    float arcLength = 0.0f;
    if(end_x<=start_x) return arcLength;

    std::list<LineGeometry> polynomials;
    polynomials.insert(polynomials.begin(), m_polynomial);
    float s_end = calculateArcLengthAbs(polynomials, end_x);
    if(end_x<0.0f) s_end *= -1.0f;
    float s_start = calculateArcLengthAbs(polynomials, start_x);
    if(start_x<0.0f) s_start *= -1.0f;

    arcLength = s_end - s_start;

    return arcLength;
}

