/**
 * @file
 * @brief  add 20211124
 */

#ifndef __PD_ROADMODEL_LANELIST_PDTOBA_H__
#define __PD_ROADMODEL_LANELIST_PDTOBA_H__

#include "PD_Input.h"
#include "PD_LgtTrajectory.h"
#include "PD_LatTrajectory.h"
#include <cmath>
#include <memory>
#include <map>
#include <set>
#include <list>

#include "../../source/PD_commonfunc.h"
#define SAVE_Timer_FILE 0    /***************** switch 0 ******************/
#if SAVE_Timer_FILE != 0
#include <fstream>
#include <chrono>
#endif

using namespace dpc;
// const float const_interpolation[10] = {0.02, 0.13, 0.23, 0.34, 0.45, 0.55, 0.66, 0.77, 0.87, 0.98};// select 10 points
// const float const_interpolation[16] = {0.01, 0.08, 0.14, 0.21, 0.27, 0.34, 0.40, 0.47, 0.53, 0.60, 0.66, 0.73, 0.79, 0.86, 0.92, 0.99};// select 16 points
const float const_interpolation[6] = {0.020, 0.212, 0.404, 0.596, 0.788, 0.980};// 20220729 select 6 points to reduce load of cpu
const float const_fittingErrorThreshold = 0.4f;
const float const_factorExtendReferenceline = 8.0;        // s_fit = const_factorExtendReferenceline * egoSpeed_mps
const float const_maxCorridorLength = 40.0f;
const float const_minCorridorLength = 10.0f;
const float const_minBoundaryLength = 10.0f;
const float const_referencelineStart = -500.0f;
const float const_referencelineEnd = 1000.0f;
// #define Fre2CarteAccuracy(x) std::min(std::max(static_cast<float>(0.1125*fabs(x)-1.25),1.0f),10.0f)         // 1m(s<=20m) - 10m(s>=100m)

namespace ADC
{
namespace PD_RoadModel_LaneList_PDtoBA
{


struct Position_Frenet
{
    float s_Frenet = 0.0;
    float l_Frenet = 0.0;
};
struct Position_Cartesian
{
    float x_Cartesian = 0.0;
    float y_Cartesian = 0.0;
};


class RoadModel_base
{
public:
    RoadModel_base(const PD::PD_Input &m_pd_input, float m_EV_end_s, float m_EV_end_l, float m_EV_end_vs, float m_EV_end_vl);
    ~RoadModel_base()=default;

protected:
    bool init(const PD::PD_Input &m_pd_input, float m_EV_end_s, float m_EV_end_l, float m_EV_end_vs, float m_EV_end_vl);
    bool initFlag=false;
    RoadInfo _m_last_RoadInfo;
    float _EV_EndPositionToReferenceLine_s = 0.0;
    float _EV_EndPositionToReferenceLine_l = 0.0;
    float _EV_EndPositionToReferenceLine_vs = 0.0;
    float _EV_EndPositionToReferenceLine_vl = 0.0;

protected:
    std::list<LineGeometry> _m_referenceLine_Cartesian_list;
    void calcu_referenceLineList_Cartesian();
    float calcu_Y_polynomial(const std::list<LineGeometry>& m_polynomials, const float m_x, int m_derivative=0);
    float calcu_Y_polynomial(const LineGeometry& m_polynomial, const float m_x, int m_derivative=0);
    
    Position_Frenet CartesianToFrenet(const std::list<LineGeometry>& referenceLine_Cartesian_list, const float m_x, const float m_y);
    void findReferPointNewtonMethod(float& refer_x0, const float m_x, const float m_y, 
                                    const std::list<LineGeometry>& m_polynomials,
                                    const float step_x=0.1f, const float accuracy_diff=0.02f, const int cycleNum=80);
    float calculateDisBetweenPoints(const std::pair<float,float>& point1, const std::pair<float,float>& point2);
    float calculateArcLengthAbs(const std::list<LineGeometry>& m_polynomials, const float refer_x);
    float calculateArcIntegralSimpsonMethod(const LineGeometry& m_polynomial, const float refer_x);
    float calculateDeltaArc(const LineGeometry& m_polynomial, const float point_x);
    
    Position_Cartesian FrenetToCartesian(const std::list<LineGeometry>& referenceLine_Cartesian_list, const float s, const float l);
    void findReferPointNewtonMethod(float& refer_x0, const float m_s, 
                                    const std::list<LineGeometry>& m_polynomials,
                                    const float step_x=0.1f, const float accuracy_diff=0.5f, const int cycleNum=80);

    void calcu_transforFactor_BaseOnEVMove();
    Position_Cartesian calcu_EndCartesianPoint_BaseOnEVMove(const float point_x_start, const float point_y_start);
    float PolynomialFitting(const std::vector<Position_Cartesian>& point, const int degree, std::unique_ptr<float[]> &new_coeff);
    int16_t binarySearchNearestMark(const std::vector<Position_Cartesian>& m_referencePoint_Cartesian_list, const float x, const float y);
    float curveArcLength(const LineGeometry& m_polynomial, const float start_x, const float end_x);

public:
    Position_Cartesian _deltaXY_translation;        // 平移
    float _deltaHeading_EV=0.0;                     // 旋转 rad
};




class RoadModel_LaneList_PDtoBA:public RoadModel_base
{
public:
    RoadModel_LaneList_PDtoBA(const PD::PD_Input &m_pd_input, float m_EV_end_s, float m_EV_end_l, float m_EV_end_vs, float m_EV_end_vl,
                                const std::vector<LaneMarker> &m_pd_lanemarkerlist_output);
    ~RoadModel_LaneList_PDtoBA()=default;

private:
    std::vector<LaneMarker> _pd_lanemarkerlist_output;

private:
    void calcu_LaneList_based_DurationTime();
    Lane _m_Lane;

public:
    void process();
    std::vector<Lane> pd_RoadModel_LaneList_output;    //output
};


}   //namespace PD_RoadModel_LaneList_PDtoBA
}   //namespace ADC

#endif  //__PD_ROADMODEL_LANELIST_PDTOBA_H__

