/**
 * @file
 * @brief  add 2021-11-29
 *        usage: include the head file in prediction.h;
 *               in prediction.cpp function process():  
 */

#include "PD_RoadModel_LaneMarkerList_PDtoBA.h"
using namespace ADC::PD_RoadModel_LaneMarkerList_PDtoBA;

#if FLAG_PD_LANMAR_SAVECSV != 0
extern std::string timeStampBag;
#endif

/**
 * @brief
 * @param
 */
RoadModel_LaneMarkerList_PDtoBA::RoadModel_LaneMarkerList_PDtoBA(const PD::PD_Input &m_pd_input, float m_EV_end_s, float m_EV_end_l, float m_EV_end_vs, float m_EV_end_vl):
                        ADC::PD_RoadModel_LaneList_PDtoBA::RoadModel_base(m_pd_input,m_EV_end_s,m_EV_end_l,m_EV_end_vs,m_EV_end_vl){
    if(init(m_pd_input,m_EV_end_s,m_EV_end_l,m_EV_end_vs,m_EV_end_vl) == false)
    return;
}


/**
 * @brief
 * @param
 */
void RoadModel_LaneMarkerList_PDtoBA::calcu_LaneMarkerList_based_DurationTime(){

    pd_RoadModel_LaneMarkerList_output.clear();
    pd_RoadModel_LaneMarkerList_output.reserve(32);
    calcu_referenceLineList_Cartesian();
    calcu_transforFactor_BaseOnEVMove();

    #if FLAG_PD_LANMAR_SAVECSV != 0
    std::ofstream outfile_PDlanemarker;
    outfile_PDlanemarker.open("./pd_lanemarker_file.txt", std::ios::app);
    outfile_PDlanemarker<< "timestamp," << timeStampBag;
    #endif
    #if FLAG_PD_LANMAR_SAVECSV != 0
    for(auto it = _m_referenceLine_Cartesian_list.begin(); it != _m_referenceLine_Cartesian_list.end(); it++){
        for(float x = it->start_range; x <= it->end_range; x=x+1.0f){
            outfile_PDlanemarker<<",refline_x,"<<x<<",refline_y,"<<calcu_Y_polynomial(*it, x, 0);
        }
    }
    outfile_PDlanemarker<<",trans_x_m,"<<_deltaXY_translation.x_Cartesian<<",trans_y_m,"<<_deltaXY_translation.y_Cartesian
                        <<",trans_heading_rad,"<<_deltaHeading_EV;
    #endif


    for(auto& element : _m_last_RoadInfo.lane_marker_list){
        LaneMarker m_next_LaneMarker;
        // id
        m_next_LaneMarker.id = element.id;
        // front_lane_boundary_list
        LaneBoundary m_next_LaneBoundary;
        m_next_LaneMarker.front_lane_boundary_list.reserve(64);
        m_next_LaneMarker.back_lane_boundary_list.reserve(64);
        for(auto & element2 : element.front_lane_boundary_list){
            if(fabs(element2.line_geometry.c0)>1000.0f) continue;  // estimate availability
            if(fabs(element2.line_geometry.end_range - element2.line_geometry.start_range)<0.02f) continue;
            m_next_LaneBoundary.line_boundary_id = element2.line_boundary_id;
            float lineboundary_start_range_y =  element2.line_geometry.c0 + 
                                                element2.line_geometry.c1 * element2.line_geometry.start_range +
                                                element2.line_geometry.c2 * element2.line_geometry.start_range * element2.line_geometry.start_range +
                                                element2.line_geometry.c3 * element2.line_geometry.start_range * element2.line_geometry.start_range * element2.line_geometry.start_range;
            float lineboundary_end_range_y =    element2.line_geometry.c0 + 
                                                element2.line_geometry.c1 * element2.line_geometry.end_range +
                                                element2.line_geometry.c2 * element2.line_geometry.end_range * element2.line_geometry.end_range +
                                                element2.line_geometry.c3 * element2.line_geometry.end_range * element2.line_geometry.end_range * element2.line_geometry.end_range;
            float x_range = element2.line_geometry.end_range - element2.line_geometry.start_range;  
            if(fabs(x_range)<0.005)         {x_range = std::max(x_range,0.01f);}
            Position_Cartesian selectNewPointFit;
            std::vector<Position_Cartesian> selectNewPointFitList;
            selectNewPointFitList.reserve(sizeof(const_interpolation)/sizeof(float));
            for(decltype(sizeof(const_interpolation)/sizeof(float)) i=0;i<sizeof(const_interpolation)/sizeof(float);i++){
                float last_x =  element2.line_geometry.start_range + const_interpolation[i] * x_range;
                float last_y =  element2.line_geometry.c0 + 
                                element2.line_geometry.c1 * last_x +
                                element2.line_geometry.c2 * last_x * last_x +
                                element2.line_geometry.c3 * last_x * last_x * last_x;
                selectNewPointFit = calcu_EndCartesianPoint_BaseOnEVMove(last_x,last_y);
                #if FLAG_PD_LANMAR_SAVECSV != 0
                outfile_PDlanemarker<<",old_point_x,"<<last_x<<",old_point_y,"<<last_y
                                    <<",new_point_x,"<<selectNewPointFit.x_Cartesian<<",new_point_y,"<<selectNewPointFit.y_Cartesian;
                #endif
                selectNewPointFitList.emplace_back(selectNewPointFit);
            }
            std::unique_ptr<float[]> newReferenceLineCoeff(new float[4]);
            int num_fit=0;
            while(PolynomialFitting(selectNewPointFitList,3,newReferenceLineCoeff)>const_fittingErrorThreshold){
                num_fit++;
                if(num_fit>3){
                    newReferenceLineCoeff[1]=(fabs(selectNewPointFitList.back().x_Cartesian-selectNewPointFitList.front().x_Cartesian)<0.001) ? 0.0 : \
                                                (selectNewPointFitList.back().y_Cartesian-selectNewPointFitList.front().y_Cartesian) / \
                                                (selectNewPointFitList.back().x_Cartesian-selectNewPointFitList.front().x_Cartesian);
                    newReferenceLineCoeff[0]=selectNewPointFitList.front().y_Cartesian - \
                                                newReferenceLineCoeff[1] * selectNewPointFitList.front().x_Cartesian;
                    newReferenceLineCoeff[2]=0.0;
                    newReferenceLineCoeff[3]=0.0;
                    break;
                }
            }
            m_next_LaneBoundary.line_geometry.c0 = newReferenceLineCoeff[0];
            m_next_LaneBoundary.line_geometry.c1 = newReferenceLineCoeff[1];
            m_next_LaneBoundary.line_geometry.c2 = newReferenceLineCoeff[2];
            m_next_LaneBoundary.line_geometry.c3 = newReferenceLineCoeff[3];
            m_next_LaneBoundary.line_geometry.start_range = calcu_EndCartesianPoint_BaseOnEVMove(element2.line_geometry.start_range,lineboundary_start_range_y).x_Cartesian;
            m_next_LaneBoundary.line_geometry.end_range = calcu_EndCartesianPoint_BaseOnEVMove(element2.line_geometry.end_range,lineboundary_end_range_y).x_Cartesian;
            #if USING_ROS2 == 0     // ros1
            m_next_LaneBoundary.line_color = element2.line_color;
            m_next_LaneBoundary.line_type = element2.line_type;
            #endif

            // sampling_point_list
            m_next_LaneBoundary.sampling_point_list.reserve(element2.sampling_point_list.size());
            SamplingPoint m_next_samplingPoint;   m_next_samplingPoint.curve = 0.0;
            for(auto& element3 : element2.sampling_point_list){
                Position_Cartesian next_samplingPoint_Cartesian = calcu_EndCartesianPoint_BaseOnEVMove(element3.x,element3.y);
                m_next_samplingPoint.x = next_samplingPoint_Cartesian.x_Cartesian;
                m_next_samplingPoint.y = next_samplingPoint_Cartesian.y_Cartesian;

                // Linear interpolation   Protection logic
                /*if(!m_next_LaneBoundary.sampling_point_list.empty()){
                    SamplingPoint m_point_interpolation;
                    if(((m_next_samplingPoint.x - m_next_LaneBoundary.sampling_point_list.back().x) > 2.5f) && ((m_next_samplingPoint.x - m_next_LaneBoundary.sampling_point_list.back().x) <200.0f)){      // +
                        for(float interpolation_x = m_next_LaneBoundary.sampling_point_list.back().x+1.0; 
                                    interpolation_x < m_next_samplingPoint.x-0.7; interpolation_x=interpolation_x+1.0){
                            m_point_interpolation.x = interpolation_x;
                            m_point_interpolation.y = (m_next_samplingPoint.y - m_next_LaneBoundary.sampling_point_list.back().y)/ \
                                                    (m_next_samplingPoint.x - m_next_LaneBoundary.sampling_point_list.back().x)* \
                                                    (interpolation_x - m_next_LaneBoundary.sampling_point_list.back().x)+m_next_LaneBoundary.sampling_point_list.back().y;
                            m_next_LaneBoundary.sampling_point_list.emplace_back(m_point_interpolation);
                        }
                    }
                    else if(((m_next_LaneBoundary.sampling_point_list.back().x - m_next_samplingPoint.x) > 2.5f) && ((m_next_LaneBoundary.sampling_point_list.back().x - m_next_samplingPoint.x) < 200.0f)){ // -
                        for(float interpolation_x = m_next_LaneBoundary.sampling_point_list.back().x-1.0; 
                                    interpolation_x > m_next_samplingPoint.x+0.7; interpolation_x=interpolation_x-1.0){
                            m_point_interpolation.x = interpolation_x;
                            m_point_interpolation.y = (m_next_samplingPoint.y - m_next_LaneBoundary.sampling_point_list.back().y)/ \
                                                    (m_next_samplingPoint.x - m_next_LaneBoundary.sampling_point_list.back().x)* \
                                                    (interpolation_x - m_next_LaneBoundary.sampling_point_list.back().x)+m_next_LaneBoundary.sampling_point_list.back().y;
                            m_next_LaneBoundary.sampling_point_list.emplace_back(m_point_interpolation);
                        }
                    }
                }*/
                m_next_LaneBoundary.sampling_point_list.emplace_back(m_next_samplingPoint);
            }

            #if FLAG_PD_LANMAR_SAVECSV != 0
            if(!(element2.sampling_point_list.empty())){
                Position_Cartesian new_samplingPoint_Cartesian = calcu_EndCartesianPoint_BaseOnEVMove(element2.sampling_point_list.back().x,element2.sampling_point_list.back().y);
                outfile_PDlanemarker<<",old_sampling_x,"<<element2.sampling_point_list.back().x<<",old_sampling_y,"<<element2.sampling_point_list.back().y
                                    <<",new_sampling_x,"<<new_samplingPoint_Cartesian.x_Cartesian<<",new_sampling_y,"<<new_samplingPoint_Cartesian.y_Cartesian;
            }
            #endif


            // arc_length
            // calculate in follow part

            // distinguish front and back lane_boundary
            if(m_next_LaneBoundary.line_geometry.end_range>=0.0){
                if(m_next_LaneBoundary.line_geometry.start_range>=0.0){
                    // arc_length
                    m_next_LaneBoundary.arc_length = element2.arc_length;
                    m_next_LaneMarker.front_lane_boundary_list.emplace_back(m_next_LaneBoundary);
                }
                else{
                    LaneBoundary m_next_front_LaneBoundary=m_next_LaneBoundary;
                    LaneBoundary m_next_back_LaneBoundary=m_next_LaneBoundary;
                    {
                        m_next_front_LaneBoundary.line_geometry.start_range=0.0;
                        // arc_length
                        m_next_front_LaneBoundary.arc_length = curveArcLength(m_next_front_LaneBoundary.line_geometry, m_next_front_LaneBoundary.line_geometry.start_range, m_next_front_LaneBoundary.line_geometry.end_range);
                    }
                    {
                        m_next_back_LaneBoundary.line_geometry.end_range=0.0;
                        // arc_length
                        m_next_back_LaneBoundary.arc_length = curveArcLength(m_next_back_LaneBoundary.line_geometry, m_next_back_LaneBoundary.line_geometry.start_range, m_next_back_LaneBoundary.line_geometry.end_range);
                    }
                    m_next_LaneMarker.front_lane_boundary_list.emplace_back(m_next_front_LaneBoundary);
                    m_next_LaneMarker.back_lane_boundary_list.emplace_back(m_next_back_LaneBoundary);
                }
            }
            else{
                // arc_length
                // std::vector<float> line_geometry_coeff_list;
                // line_geometry_coeff_list.emplace_back(m_next_LaneBoundary.line_geometry.c0);
                // line_geometry_coeff_list.emplace_back(m_next_LaneBoundary.line_geometry.c1);
                // line_geometry_coeff_list.emplace_back(m_next_LaneBoundary.line_geometry.c2);
                // line_geometry_coeff_list.emplace_back(m_next_LaneBoundary.line_geometry.c3);
                m_next_LaneBoundary.arc_length = element2.arc_length;
                m_next_LaneMarker.back_lane_boundary_list.emplace_back(m_next_LaneBoundary);
            }
        }

        LaneBoundary m_extend_laneBoundary;
        {
            if(m_next_LaneMarker.front_lane_boundary_list.empty()){
                if(!m_next_LaneMarker.back_lane_boundary_list.empty())
                m_extend_laneBoundary = m_next_LaneMarker.back_lane_boundary_list.back();
            }
            else{
                m_extend_laneBoundary = m_next_LaneMarker.front_lane_boundary_list.back();
            }
            if(!(m_next_LaneMarker.front_lane_boundary_list.empty() && m_next_LaneMarker.back_lane_boundary_list.empty())){     // if there is no boundary in this lane_marker, the lane_marker is not meanful to extend
                float desired_boundary_x = std::max( const_factorExtendReferenceline*_EV_EndPositionToReferenceLine_vs, const_minBoundaryLength);
                float laneboundaryline_final_x = 0.0;
                /*for(auto& element2 : m_next_LaneMarker.front_lane_boundary_list){
                    laneboundaryline_final_x = std::max(laneboundaryline_final_x, element2.line_geometry.end_range);
                }*/
                laneboundaryline_final_x = m_extend_laneBoundary.line_geometry.end_range;

                float laneboundaryline_final_y = m_extend_laneBoundary.line_geometry.c0 + 
                                                    m_extend_laneBoundary.line_geometry.c1 * laneboundaryline_final_x + 
                                                    m_extend_laneBoundary.line_geometry.c2 * laneboundaryline_final_x * laneboundaryline_final_x + 
                                                    m_extend_laneBoundary.line_geometry.c3 * laneboundaryline_final_x * laneboundaryline_final_x * laneboundaryline_final_x;
                float sampling_point_final_x = m_extend_laneBoundary.sampling_point_list.empty() ? -120.0 : m_extend_laneBoundary.sampling_point_list.back().x;
                float sampling_point_final_y = m_extend_laneBoundary.sampling_point_list.empty() ? 0.0 : m_extend_laneBoundary.sampling_point_list.back().y;
                if(static_cast<bool>(element.is_end_out_of_range) == static_cast<bool>(true)){  // laneMarker is end
                    desired_boundary_x = sampling_point_final_x;
                }
                float k_extend = 0.0f;  float b_extend = 0.0f;
                {
                    float x0 = 0.0f;  float y0 = 0.0f;  float x1 = 1.0f;  float y1 = 0.0f;
                    if((sampling_point_final_x - laneboundaryline_final_x) > 5.0f){      // calculate coeff by sampling_point
                        if(m_extend_laneBoundary.sampling_point_list.size()>1){
                            x0 = m_extend_laneBoundary.sampling_point_list.at(m_extend_laneBoundary.sampling_point_list.size()-2).x;
                            y0 = m_extend_laneBoundary.sampling_point_list.at(m_extend_laneBoundary.sampling_point_list.size()-2).y;
                            x1 = m_extend_laneBoundary.sampling_point_list.at(m_extend_laneBoundary.sampling_point_list.size()-1).x;
                            y1 = m_extend_laneBoundary.sampling_point_list.at(m_extend_laneBoundary.sampling_point_list.size()-1).y;
                            if(fabs(x0-x1)<0.001){
                                k_extend = 0.0f;
                                b_extend = y1;
                            }
                            else{
                                k_extend = (y1-y0)/(x1-x0);
                                b_extend = y1 - k_extend*x1;
                            }
                            #if FLAG_PD_LANMAR_SAVECSV != 0
                            outfile_PDlanemarker<<",ext_bysample1_x0,"<<x0<<",ext_bysample1_y0,"<<y0<<",ext_bysample1_x1,"<<x1<<",ext_bysample1_y1,"<<y1;
                            #endif
                        }
                        else{
                            k_extend = 0.0f;
                            b_extend = sampling_point_final_y;
                            #if FLAG_PD_LANMAR_SAVECSV != 0
                            outfile_PDlanemarker<<",ext_bysample2_y,"<<sampling_point_final_y;
                            #endif
                        }
                    }
                    else{                                                          // calculate coeff by laneboundaryline
                        if(laneboundaryline_final_x>0.1){
                            x1 = m_extend_laneBoundary.line_geometry.end_range;
                            y1 = m_extend_laneBoundary.line_geometry.c0 + 
                                m_extend_laneBoundary.line_geometry.c1 * x1 + 
                                m_extend_laneBoundary.line_geometry.c2 * x1 * x1 + 
                                m_extend_laneBoundary.line_geometry.c3 * x1 * x1 * x1;
                            x0 = m_extend_laneBoundary.line_geometry.end_range-1.0f;
                            y0 = m_extend_laneBoundary.line_geometry.c0 + 
                                m_extend_laneBoundary.line_geometry.c1 * x0 + 
                                m_extend_laneBoundary.line_geometry.c2 * x0 * x0 + 
                                m_extend_laneBoundary.line_geometry.c3 * x0 * x0 * x0;
                            if(fabs(x0-x1)<0.001){
                                k_extend = 0.0f;
                                b_extend = y1;
                            }
                            else{
                                k_extend = (y1-y0)/(x1-x0);
                                b_extend = y1 - k_extend*x1;
                            }
                            #if FLAG_PD_LANMAR_SAVECSV != 0
                            outfile_PDlanemarker<<",ext_byline1_x0,"<<x0<<",ext_byline1_y0,"<<y0<<",ext_byline1_x1,"<<x1<<",ext_byline1_y1,"<<y1;
                            #endif
                        }
                        else{
                            k_extend = 0.0f;
                            b_extend = laneboundaryline_final_y;
                            #if FLAG_PD_LANMAR_SAVECSV != 0
                            outfile_PDlanemarker<<",ext_byline2_y,"<<laneboundaryline_final_y;
                            #endif
                        }
                    }
                }


                std::vector<Position_Cartesian> samplingPointExtend_list(m_extend_laneBoundary.sampling_point_list.size());
                Position_Cartesian samplingPoint;
                for(auto& element2 : m_extend_laneBoundary.sampling_point_list){
                    if((element2.x - laneboundaryline_final_x) < -10.0f) continue;
                    samplingPoint.x_Cartesian = element2.x;
                    samplingPoint.y_Cartesian = 0.0;
                    samplingPointExtend_list.emplace_back(samplingPoint);
                }
                samplingPointExtend_list.shrink_to_fit();

                float x_perSection=40.0f;
                for(float start_x =laneboundaryline_final_x; start_x<=desired_boundary_x; start_x=start_x+x_perSection){  // x_perSection meter per boundary section

                    m_extend_laneBoundary.line_geometry.start_range = start_x;
                    m_extend_laneBoundary.line_geometry.end_range = std::min(start_x+x_perSection, desired_boundary_x);

                    Position_Cartesian selectNewPointFit;
                    std::vector<Position_Cartesian> selectNewPointFitList;
                    selectNewPointFitList.reserve(sizeof(const_interpolation)/sizeof(float));
                    for(decltype(sizeof(const_interpolation)/sizeof(float)) i=0;i<sizeof(const_interpolation)/sizeof(float);i++){
                        float selPoint_x  = m_extend_laneBoundary.line_geometry.start_range + const_interpolation[i] * (m_extend_laneBoundary.line_geometry.end_range - m_extend_laneBoundary.line_geometry.start_range);
                        
                        if(selPoint_x<=sampling_point_final_x){
                            int16_t selPointMark=binarySearchNearestMark(samplingPointExtend_list, selPoint_x, 0.0);
                            if(static_cast<decltype(m_next_LaneBoundary.sampling_point_list.size())>(selPointMark)<m_next_LaneBoundary.sampling_point_list.size()){        // select point by sampling_point
                                selectNewPointFit.x_Cartesian = m_next_LaneBoundary.sampling_point_list.at(selPointMark).x;
                                selectNewPointFit.y_Cartesian = m_next_LaneBoundary.sampling_point_list.at(selPointMark).y;
                            }
                            else{                                                                   // select point by extend_point
                                selectNewPointFit.x_Cartesian = selPoint_x;
                                selectNewPointFit.y_Cartesian = k_extend*selPoint_x+b_extend; //(sampling_point_final_x > laneboundaryline_final_x) ? sampling_point_final_y : laneboundaryline_final_y;
                            }
                        }
                        else{                                                                       // select point by extend_point
                            selectNewPointFit.x_Cartesian = selPoint_x;
                            selectNewPointFit.y_Cartesian = k_extend*selPoint_x+b_extend; //(sampling_point_final_x > laneboundaryline_final_x) ? sampling_point_final_y : laneboundaryline_final_y;
                        }
                        selectNewPointFitList.emplace_back(selectNewPointFit);
                        #if FLAG_PD_LANMAR_SAVECSV != 0
                        outfile_PDlanemarker<<",ext_point_x,"<<selectNewPointFit.x_Cartesian<<",ext_point_y,"<<selectNewPointFit.y_Cartesian;
                        #endif
                    }
                    std::unique_ptr<float[]> newReferenceLineCoeff(new float[4]);
                    int num_fit=0;
                    while(PolynomialFitting(selectNewPointFitList,3,newReferenceLineCoeff)>const_fittingErrorThreshold){
                        num_fit++;
                        if(num_fit>3){
                            newReferenceLineCoeff[1]=(fabs(selectNewPointFitList.back().x_Cartesian-selectNewPointFitList.front().x_Cartesian)<0.001) ? 0.0 : \
                                                        (selectNewPointFitList.back().y_Cartesian-selectNewPointFitList.front().y_Cartesian) / \
                                                        (selectNewPointFitList.back().x_Cartesian-selectNewPointFitList.front().x_Cartesian);
                            newReferenceLineCoeff[0]=selectNewPointFitList.front().y_Cartesian - \
                                                        newReferenceLineCoeff[1] * selectNewPointFitList.front().x_Cartesian;
                            newReferenceLineCoeff[2]=0.0;
                            newReferenceLineCoeff[3]=0.0;
                            break;
                        }
                    }

                    m_extend_laneBoundary.line_geometry.c0 = newReferenceLineCoeff[0];
                    m_extend_laneBoundary.line_geometry.c1 = newReferenceLineCoeff[1];
                    m_extend_laneBoundary.line_geometry.c2 = newReferenceLineCoeff[2];
                    m_extend_laneBoundary.line_geometry.c3 = newReferenceLineCoeff[3];
                    // arc_length
                    m_extend_laneBoundary.arc_length = curveArcLength(m_extend_laneBoundary.line_geometry, m_extend_laneBoundary.line_geometry.start_range, m_extend_laneBoundary.line_geometry.end_range);
                    m_next_LaneMarker.front_lane_boundary_list.emplace_back(m_extend_laneBoundary);
                }
            }
        }

        std::vector<LaneBoundary>(m_next_LaneMarker.front_lane_boundary_list).swap(m_next_LaneMarker.front_lane_boundary_list); //capacity()

        // back_lane_boundary_list
        #if FLAG_PDFP_SAVECSV != 0
        for(auto & element2 : element.back_lane_boundary_list){
            if(fabs(element2.line_geometry.c0)>1000.0f) continue;  // estimate availability
            m_next_LaneBoundary.line_boundary_id = element2.line_boundary_id;
            float lineboundary_start_range_y =  element2.line_geometry.c0 + 
                                                element2.line_geometry.c1 * element2.line_geometry.start_range +
                                                element2.line_geometry.c2 * element2.line_geometry.start_range * element2.line_geometry.start_range +
                                                element2.line_geometry.c3 * element2.line_geometry.start_range * element2.line_geometry.start_range * element2.line_geometry.start_range;
            float lineboundary_end_range_y =    element2.line_geometry.c0 + 
                                                element2.line_geometry.c1 * element2.line_geometry.end_range +
                                                element2.line_geometry.c2 * element2.line_geometry.end_range * element2.line_geometry.end_range +
                                                element2.line_geometry.c3 * element2.line_geometry.end_range * element2.line_geometry.end_range * element2.line_geometry.end_range;
            float x_range = element2.line_geometry.end_range - element2.line_geometry.start_range;  
            if(fabs(x_range)<0.005)         {x_range = std::max(x_range,0.01f);}
            Position_Cartesian selectNewPointFit;
            std::vector<Position_Cartesian> selectNewPointFitList;
            selectNewPointFitList.reserve(sizeof(const_interpolation)/sizeof(float));
            for(decltype(sizeof(const_interpolation)/sizeof(float)) i=0;i<sizeof(const_interpolation)/sizeof(float);i++){
                float last_x =  element2.line_geometry.start_range + const_interpolation[i] * x_range;
                float last_y =  element2.line_geometry.c0 + 
                                element2.line_geometry.c1 * last_x +
                                element2.line_geometry.c2 * last_x * last_x +
                                element2.line_geometry.c3 * last_x * last_x * last_x;
                selectNewPointFit = calcu_EndCartesianPoint_BaseOnEVMove(last_x,last_y);
                #if FLAG_PD_LANMAR_SAVECSV != 0
                outfile_PDlanemarker<<",old_point_x,"<<last_x<<",old_point_y,"<<last_y
                                    <<",new_point_x,"<<selectNewPointFit.x_Cartesian<<",new_point_y,"<<selectNewPointFit.y_Cartesian;
                #endif
                selectNewPointFitList.emplace_back(selectNewPointFit);
            }
            std::unique_ptr<float[]> newReferenceLineCoeff(new float[4]);
            int num_fit=0;
            while(PolynomialFitting(selectNewPointFitList,3,newReferenceLineCoeff)>const_fittingErrorThreshold){
                num_fit++;
                if(num_fit>3){
                    newReferenceLineCoeff[1]=(fabs(selectNewPointFitList.back().x_Cartesian-selectNewPointFitList.front().x_Cartesian)<0.001) ? 0.0 : \
                                                (selectNewPointFitList.back().y_Cartesian-selectNewPointFitList.front().y_Cartesian) / \
                                                (selectNewPointFitList.back().x_Cartesian-selectNewPointFitList.front().x_Cartesian);
                    newReferenceLineCoeff[0]=selectNewPointFitList.front().y_Cartesian - \
                                                newReferenceLineCoeff[1] * selectNewPointFitList.front().x_Cartesian;
                    newReferenceLineCoeff[2]=0.0;
                    newReferenceLineCoeff[3]=0.0;
                    break;
                }
            }
            m_next_LaneBoundary.line_geometry.c0 = newReferenceLineCoeff[0];
            m_next_LaneBoundary.line_geometry.c1 = newReferenceLineCoeff[1];
            m_next_LaneBoundary.line_geometry.c2 = newReferenceLineCoeff[2];
            m_next_LaneBoundary.line_geometry.c3 = newReferenceLineCoeff[3];
            m_next_LaneBoundary.line_geometry.start_range = calcu_EndCartesianPoint_BaseOnEVMove(element2.line_geometry.start_range,lineboundary_start_range_y).x_Cartesian;
            m_next_LaneBoundary.line_geometry.end_range = calcu_EndCartesianPoint_BaseOnEVMove(element2.line_geometry.end_range,lineboundary_end_range_y).x_Cartesian;
            // arc_length
            {
                // std::vector<float> line_geometry_coeff_list;
                // line_geometry_coeff_list.emplace_back(m_next_LaneBoundary.line_geometry.c0);
                // line_geometry_coeff_list.emplace_back(m_next_LaneBoundary.line_geometry.c1);
                // line_geometry_coeff_list.emplace_back(m_next_LaneBoundary.line_geometry.c2);
                // line_geometry_coeff_list.emplace_back(m_next_LaneBoundary.line_geometry.c3);
                m_next_LaneBoundary.arc_length = element2.arc_length;
            }
            #if USING_ROS2 == 0     // ros1
            m_next_LaneBoundary.line_color = element2.line_color;
            m_next_LaneBoundary.line_type = element2.line_type;
            #endif
            
            m_next_LaneMarker.back_lane_boundary_list.emplace_back(m_next_LaneBoundary);
            
        }
        std::vector<LaneBoundary>(m_next_LaneMarker.back_lane_boundary_list).swap(m_next_LaneMarker.back_lane_boundary_list); //capacity()
        #endif

        #if USING_ROS2 != 0     // ros2
        // line_color_attribute_list
        LineColor m_next_LineColor;
        m_next_LaneMarker.line_color_attribute_list.reserve(16);
        for(auto& element2 : element.line_color_attribute_list){
            m_next_LineColor.start_offset = element2.start_offset - _EV_EndPositionToReferenceLine_s;
            m_next_LineColor.end_offset = element2.end_offset - _EV_EndPositionToReferenceLine_s;
            m_next_LineColor.linecolor = element2.linecolor;
            m_next_LaneMarker.line_color_attribute_list.emplace_back(m_next_LineColor);
        }
        std::vector<LineColor>(m_next_LaneMarker.line_color_attribute_list).swap(m_next_LaneMarker.line_color_attribute_list); //capacity()

        // line_type_attribute_list
        LineType m_next_LineType;
        m_next_LaneMarker.line_type_attribute_list.reserve(16);
        for(auto& element2 : element.line_type_attribute_list){
            m_next_LineType.start_offset = element2.start_offset - _EV_EndPositionToReferenceLine_s;
            m_next_LineType.end_offset = element2.end_offset - _EV_EndPositionToReferenceLine_s;
            m_next_LineType.linetype = element2.linetype;
            m_next_LaneMarker.line_type_attribute_list.emplace_back(m_next_LineType);
        }
        std::vector<LineType>(m_next_LaneMarker.line_type_attribute_list).swap(m_next_LaneMarker.line_type_attribute_list); //capacity()

        #endif


        // bool
        m_next_LaneMarker.is_start_out_of_range = element.is_start_out_of_range;
        m_next_LaneMarker.is_end_out_of_range = element.is_end_out_of_range;
        pd_RoadModel_LaneMarkerList_output.emplace_back(m_next_LaneMarker);
    }
    std::vector<LaneMarker>(pd_RoadModel_LaneMarkerList_output).swap(pd_RoadModel_LaneMarkerList_output); //capacity()

    #if FLAG_PD_LANMAR_SAVECSV != 0
    outfile_PDlanemarker<<std::endl;
    outfile_PDlanemarker.close();
    #endif

}


/**
 * @brief
 * @param
 */
void RoadModel_LaneMarkerList_PDtoBA::process(){

    if(initFlag != 1){
        // std::cout<< "ATTENTION: Data reading of RoadMode_LaneMarkerList_PDtoBA error" <<std::endl;
        return;
    }

    calcu_LaneMarkerList_based_DurationTime();
}
