/**
 * @file
 * @brief  add 2021-11-29
 *        usage: include the head file in prediction.h;
 *               in prediction.cpp function process():  
 */

#include "PD_RoadModel_ObjectList_PDtoBA.h"
using namespace ADC::PD_RoadModel_ObjectList_PDtoBA;

#if FLAG_PD_OBJECT_SAVECSV != 0
extern std::string timeStampBag;
#endif

/**
 * @brief
 * @param
 */
RoadModel_ObjectList_PDtoBA::RoadModel_ObjectList_PDtoBA(const PD::PD_Input &m_pd_input, float m_EV_end_s, float m_EV_end_l, float m_EV_end_vs, float m_EV_end_vl):
                        ADC::PD_RoadModel_LaneList_PDtoBA::RoadModel_base(m_pd_input,m_EV_end_s,m_EV_end_l,m_EV_end_vs,m_EV_end_vl){
    if(init(m_pd_input,m_EV_end_s,m_EV_end_l,m_EV_end_vs,m_EV_end_vl) == false)
    return;
}


/**
 * @brief
 * @param
 */
void RoadModel_ObjectList_PDtoBA::calcu_ObjectList_based_DurationTime(){

    pd_RoadModel_ObjectList_output.clear();
    pd_RoadModel_ObjectList_output.reserve(128);
    calcu_referenceLineList_Cartesian();
    calcu_transforFactor_BaseOnEVMove();

    #if FLAG_PD_OBJECT_SAVECSV != 0
    std::ofstream outfile_PDobject;
    outfile_PDobject.open("./pd_object_file.txt", std::ios::app);
    outfile_PDobject<< "timestamp," << timeStampBag;
    #endif

    for(auto& element : _m_last_RoadInfo.object_list){
        Object m_next_object;
        m_next_object.object_id = element.object_id;
        m_next_object.object_exist_probability = element.object_exist_probability;
        m_next_object.classification = element.classification;
        Position_Cartesian next_objectCenter_Cartesian = calcu_EndCartesianPoint_BaseOnEVMove(
                element.relative_center_x+element.absolute_velocity_x*_m_last_RoadInfo.passage_list.at(psg_Num).duration_time,
                element.relative_center_y+element.absolute_velocity_y*_m_last_RoadInfo.passage_list.at(psg_Num).duration_time);
        m_next_object.relative_center_x = next_objectCenter_Cartesian.x_Cartesian;
        m_next_object.relative_center_y = next_objectCenter_Cartesian.y_Cartesian;
        Position_Cartesian next_objectPosition_Cartesian = calcu_EndCartesianPoint_BaseOnEVMove(
                element.relative_position_x+element.absolute_velocity_x*_m_last_RoadInfo.passage_list.at(psg_Num).duration_time,
                element.relative_position_y+element.absolute_velocity_y*_m_last_RoadInfo.passage_list.at(psg_Num).duration_time);
        m_next_object.relative_position_x = next_objectPosition_Cartesian.x_Cartesian;
        m_next_object.relative_position_y = next_objectPosition_Cartesian.y_Cartesian;
        // m_next_object.relative_center_x = calcu_EndCartesianPoint_BaseOnEVMove(
        //         element.relative_center_x+element.absolute_velocity_x*_m_last_RoadInfo.passage_list.at(psg_Num).duration_time,
        //         element.relative_center_y+element.absolute_velocity_y*_m_last_RoadInfo.passage_list.at(psg_Num).duration_time).x_Cartesian;
        // m_next_object.relative_center_y = calcu_EndCartesianPoint_BaseOnEVMove(
        //         element.relative_center_x+element.absolute_velocity_x*_m_last_RoadInfo.passage_list.at(psg_Num).duration_time,
        //         element.relative_center_y+element.absolute_velocity_y*_m_last_RoadInfo.passage_list.at(psg_Num).duration_time).y_Cartesian;
        // m_next_object.relative_position_x = calcu_EndCartesianPoint_BaseOnEVMove(
        //         element.relative_position_x+element.absolute_velocity_x*_m_last_RoadInfo.passage_list.at(psg_Num).duration_time,
        //         element.relative_position_y+element.absolute_velocity_y*_m_last_RoadInfo.passage_list.at(psg_Num).duration_time).x_Cartesian;
        // m_next_object.relative_position_y = calcu_EndCartesianPoint_BaseOnEVMove(
        //         element.relative_position_x+element.absolute_velocity_x*_m_last_RoadInfo.passage_list.at(psg_Num).duration_time,
        //         element.relative_position_y+element.absolute_velocity_y*_m_last_RoadInfo.passage_list.at(psg_Num).duration_time).y_Cartesian;
        m_next_object.absolute_velocity_x = element.absolute_velocity_x;    //const velocity
        m_next_object.absolute_velocity_y = element.absolute_velocity_y;    //const velocity
        m_next_object.absolute_acceleration_x = 0.0;        // 0
        m_next_object.absolute_acceleration_y = 0.0;        // 0
        m_next_object.absolute_yawrate = 0.0;               // 0
        m_next_object.size_width = element.size_width;
        m_next_object.size_length = element.size_length;
        m_next_object.intentions_valid = element.intentions_valid;
        m_next_object.intentions_number = element.intentions_number;
        m_next_object.intention_list = element.intention_list;
        pd_RoadModel_ObjectList_output.emplace_back(m_next_object);

        #if FLAG_PD_OBJECT_SAVECSV != 0
        outfile_PDobject<< ",objectid," << element.object_id << \
                            ",centerX," << element.relative_center_x << ",centerY," << element.relative_center_y << \
                            ",width," << element.size_width << ",length," << element.size_length;
        #endif

    }
    std::vector<Object>(pd_RoadModel_ObjectList_output).swap(pd_RoadModel_ObjectList_output); //capacity()

    #if FLAG_PD_OBJECT_SAVECSV != 0
    outfile_PDobject<<std::endl;
    outfile_PDobject.close();
    #endif

}


/**
 * @brief
 * @param
 */
void RoadModel_ObjectList_PDtoBA::process(){

    if(initFlag != 1){
        // std::cout<< "ATTENTION: Data reading of RoadMode_ObjectList_PDtoBA error" <<std::endl;
        return;
    }

    calcu_ObjectList_based_DurationTime();
}
