/**
 * @Author: wzy
 * @Date: 2021-11-16
 * @LastEditTime: 
 * @LastEditors: Please set LastEditors
 * @Description: In User Settings Edit
 * @filePath:
 */

#include "PD_StnObstcRisk.h"
using namespace ADC::PD_StnObstcRsk;

StnObstcRisk::StnObstcRisk(const PD::PD_Input &pd_input,
                           const uint8_t pd_LatTrj_EV_ActionType,
                           const std::vector<ADC::PD_EgoVehTrj::EgoVeh_TrjPnt>& EgoVeh_trace_list){
    if (init(pd_input, pd_LatTrj_EV_ActionType, EgoVeh_trace_list) == false){
        // std::cout << "ATTENTION: StnObstcRisk init is false!" << std::endl;
        return;
    }
}

bool StnObstcRisk::init(const PD::PD_Input &pd_input,
                        const uint8_t pd_LatTrj_EV_ActionType,
                        const std::vector<ADC::PD_EgoVehTrj::EgoVeh_TrjPnt>& EgoVeh_trace_list){
    bool Flag = true;
    Flag = Flag && !(pd_input.rm_roadmodel.empty());             //chech rm_roadmodel

    if(Flag==false){
        initFlag=false;
    }
    else{
        _left_road_edge_segment_list.clear();
        _right_road_edge_segment_list.clear();
        _road_edge_list.clear();
        _egoVeh_TrjPnt.clear();
        _lat_dis_list.clear();
        initFlag = true;

        _ego_action_type = pd_LatTrj_EV_ActionType;

        _road_edge_list = pd_input.rm_roadmodel.at(rm_Num).road_edge_list;

        #if FLAG_STN_IOOUT != 0
        if (_road_edge_list.empty()){
            std::cout << "ATTENTION: road_edge_list is empty!" << std::endl;
        }
        #endif

        for(auto& element : _road_edge_list){
            for(auto& ele1 : element.road_edge_segment_list){
                if(ele1.line_geometry.c0 > 0){
                    _left_road_edge_segment_list.push_back(ele1);
                }
                else{
                    _right_road_edge_segment_list.push_back(ele1);
                }
            }
        }

        bool flagFindCorridorID = false;
        uint32_t passage_corridor_ID = pd_input.rm_roadmodel.at(rm_Num).passage_list.at(psg_Num).corridor_id;
        for (auto& element : pd_input.rm_roadmodel.at(rm_Num).lane_list){
            for (auto& element1 : element.corridor_list){
                if (element1.corridor_id == passage_corridor_ID){
                    _pos_ego_start = element1.reference_line.c0;
                    flagFindCorridorID = true;
                }
                if (flagFindCorridorID)
                {break;}
            }
            if (flagFindCorridorID)
            {break;}
        }

        PD_DynmcObjRsk::EgoVeh_Point egoveh_TrjPnt;
        for (int i = 0; i < EgoVeh_trace_list.size(); i++){
            egoveh_TrjPnt.t = EgoVeh_trace_list[i].t;
            egoveh_TrjPnt.s = EgoVeh_trace_list[i].s;
            egoveh_TrjPnt.l = EgoVeh_trace_list[i].l;
            egoveh_TrjPnt.vs = EgoVeh_trace_list[i].vx;
            egoveh_TrjPnt.as = EgoVeh_trace_list[i].ax;
            egoveh_TrjPnt.jks = EgoVeh_trace_list[i].jkx;
            _egoVeh_TrjPnt.push_back(egoveh_TrjPnt);
        }
    }
    return initFlag;
}

float StnObstcRisk::CalculateBdrLatMinDis(){
    if (_road_edge_list.empty() || _egoVeh_TrjPnt.empty()){
        m_BdrLatMinDis = 999.0f;
        return m_BdrLatMinDis;
    }
    float LatDis = 0.0f;
    //LC
    if ((_ego_action_type == 2) || (_ego_action_type == 4)){
        if(_left_road_edge_segment_list.empty()){
            return 999.0f;
        }
        else{
            for(auto& ele_Trj : _egoVeh_TrjPnt){
                for(auto& ele_edge : _left_road_edge_segment_list){
                    if ((ele_Trj.s >= ele_edge.line_geometry.start_range)
                    && (ele_Trj.s < ele_edge.line_geometry.end_range)){
                        LatDis = abs(ele_edge.line_geometry.c0 - _pos_ego_start - ele_Trj.l);              
                        _lat_dis_list.push_back(LatDis);
                        break;
                    }
                }
            }
        }
    }
    //RC
    else if((_ego_action_type == 3) || (_ego_action_type == 5)){
        if(_right_road_edge_segment_list.empty()){
            return 999.0f;
        }
        else{
            for (auto& ele_Trj : _egoVeh_TrjPnt){
                for (auto& ele_edge : _right_road_edge_segment_list){
                    if ((ele_Trj.s >= ele_edge.line_geometry.start_range)
                    && (ele_Trj.s < ele_edge.line_geometry.end_range)){
                        LatDis = abs(_pos_ego_start - ele_edge.line_geometry.c0 + ele_Trj.l);
                        _lat_dis_list.push_back(LatDis);
                        break;
                    }
                }
            } 
        }
    }
    else{
        if(_left_road_edge_segment_list.empty() && _right_road_edge_segment_list.empty()){
            LatDis = 999.0f;
        }  
        else if(_left_road_edge_segment_list.empty()){
            LatDis = abs(_right_road_edge_segment_list.front().line_geometry.c0);
        }  
        else if(_right_road_edge_segment_list.empty()){
            LatDis = abs(_left_road_edge_segment_list.front().line_geometry.c0);
        }  
        else{
            LatDis = std::min(abs(_left_road_edge_segment_list.front().line_geometry.c0), abs(_right_road_edge_segment_list.front().line_geometry.c0));
        }  
        _lat_dis_list.push_back(LatDis);
    }
    if(_lat_dis_list.empty()){
        return 999.0f;
    }
    else{
        m_BdrLatMinDis = *min_element(_lat_dis_list.begin(), _lat_dis_list.end());
        return m_BdrLatMinDis;
    }
}

float StnObstcRisk::CalculateBdrLatCrsSfZnItg(){
    if (_road_edge_list.empty() || _egoVeh_TrjPnt.empty()){
        m_BdrLatCrsSfZnItg = 0.0f;
        return m_BdrLatCrsSfZnItg;
    }
    float LatDis = 0.0f;
    uint8_t edge_type;
    float LatCrsZn = 0.0f;
    // float BdrCrsZn_Threshold = 0.5f;
    // 0=unknown; 1=flat; 2=curb; 3=elevated; 4=cone and pole; 5=parked cars
    float BdrCrsZn_Threshold[6] = {0.5, 0.2, 0.2, 0.3, 0.4, 0.5};
    std::vector<float> BdrLatCrsSfZn;

    if ((_ego_action_type == 2) || (_ego_action_type == 4)){
        if(_left_road_edge_segment_list.empty()){
            return 0.0f;
        }
        else{
            for(auto& ele_Trj : _egoVeh_TrjPnt){
                for(auto& ele_edge : _left_road_edge_segment_list){
                    if ((ele_Trj.s >= ele_edge.line_geometry.start_range)
                    && (ele_Trj.s < ele_edge.line_geometry.end_range)){
                        LatDis = abs(ele_edge.line_geometry.c0 - _pos_ego_start - ele_Trj.l);
                        edge_type = ele_edge.type;
                        LatCrsZn = BdrCrsZn_Threshold[edge_type] - LatDis;
                        if (LatCrsZn > 0){
                            BdrLatCrsSfZn.push_back(LatCrsZn);
                        }
                        else{
                            BdrLatCrsSfZn.push_back(0);
                        }
                        break;
                    }
                }
            }
        }
    }
    else if((_ego_action_type == 3) || (_ego_action_type == 5)){
        if(_right_road_edge_segment_list.empty()){
            return 0.0f;
        }
        else{
            for (auto& ele_Trj : _egoVeh_TrjPnt){
                for (auto& ele_edge : _right_road_edge_segment_list){
                    if ((ele_Trj.s >= ele_edge.line_geometry.start_range)
                    && (ele_Trj.s < ele_edge.line_geometry.end_range)){
                        LatDis = abs(_pos_ego_start - ele_edge.line_geometry.c0 + ele_Trj.l);
                        edge_type = ele_edge.type;
                        LatCrsZn = BdrCrsZn_Threshold[edge_type] - LatDis;
                        if (LatCrsZn > 0){
                            BdrLatCrsSfZn.push_back(LatCrsZn);
                        }
                        else{
                            BdrLatCrsSfZn.push_back(0);
                        }
                        break;
                    }
                }
            } 
        }
    }
    else{
        if(_left_road_edge_segment_list.empty() && _right_road_edge_segment_list.empty()){
            LatDis = 999.0f;
        }  
        else if(_left_road_edge_segment_list.empty()){
            LatDis = abs(_right_road_edge_segment_list.front().line_geometry.c0);
        }  
        else if(_right_road_edge_segment_list.empty()){
            LatDis = abs(_left_road_edge_segment_list.front().line_geometry.c0);
        }  
        else{
            LatDis = std::min(abs(_left_road_edge_segment_list.front().line_geometry.c0), abs(_right_road_edge_segment_list.front().line_geometry.c0));
        }  
        LatCrsZn = 0.5 - LatDis;
        if (LatCrsZn > 0){
            BdrLatCrsSfZn.push_back(LatCrsZn);
        }
        else{
            BdrLatCrsSfZn.push_back(0);
        }
    }

    if(BdrLatCrsSfZn.empty()){
        return 0.0f;
    }
    else{
        for(auto& ele : BdrLatCrsSfZn){
            m_BdrLatCrsSfZnItg += ele * timesampl;
        }
        return m_BdrLatCrsSfZnItg;
    }
}

void StnObstcRisk::process(){
    if (initFlag != true){
        // std::cout << "ATTENTION: Data reading of PD_StnObstcRisk is error" << std::endl;
        return;
    }
    CalculateBdrLatMinDis();
    CalculateBdrLatCrsSfZnItg();

    #if FLAG_STN_IOOUT != 0
    std::ofstream ofs;
    ofs.open("./src/BA_Node/source/core/source/PD_module/LogFile/StnObstcOutPut.txt", std::ios::app);
    struct timeval time;
    gettimeofday(&time, NULL);
    ofs << "time: " << time.tv_sec << "." << time.tv_usec << std::endl;
    ofs << "m_BdrLatMinDis: " << m_BdrLatMinDis << " / " << "m_BdrLatCrsSfZnItg: " << m_BdrLatCrsSfZnItg << std::endl;
    ofs << std::endl;
    ofs.close();
    #endif

    //TJA阶段赋默认值
    // m_BdrLatMinDis = 999.0f;
    // m_BdrLatCrsSfZnItg = 0.0f;
}