#include "PD_Debug_Process.h"
#include "glog/logging.h"

using namespace ADC::PD_DebugProc;

PD_AnalysisData::PD_AnalysisData(const PD::PD_Input &pd_input,
                                 const PD_DynmcObjRsk::DynamicObjRisk &dynmcobjrsk,
                                 const PD_EgoVehTrj::EgoVehTrj &egovehtrj,
                                 const PD_LgtSpdLmt::LgtSpdLmt &lgtspdlmt,
                                 const PD_NodeUpdate::NodeUpdate &nodeinfo,
                                 const PD_ObjAttention::RiskObjAttention &attobjinfo,
                                 const PD_LgtTrj::LgtTrajectory &lgttrjinfo)

{
    pdobsvar.pd_passage_id.x = pd_input.rm_roadmodel.at(rm_Num).passage_list.at(psg_Num).passage_id.x;
    pdobsvar.pd_passage_id.y = pd_input.rm_roadmodel.at(rm_Num).passage_list.at(psg_Num).passage_id.y;
    pdobsvar.pd_ba2pd_actiontype = pd_input.rm_roadmodel.at(rm_Num).passage_list.at(psg_Num).action_type;
    // pdobsvar.pd_obj_clsn_flg = nodeinfo.NodeClsnFlag;
    if (!nodeinfo.clsnobjinfo.empty())
    {

        for (int m = 0; m < nodeinfo.clsnobjinfo.size(); m++)
        {
            out_clsninfo.pd_clsnobj_id = nodeinfo.clsnobjinfo[m].ClsnObjID;

            // get x,y,vx
            for (int i = 0; i < pd_input.rm_roadmodel.at(rm_Num).object_list.size(); i++)
            {
                if (out_clsninfo.pd_clsnobj_id == pd_input.rm_roadmodel.at(rm_Num).object_list[i].object_id)
                {
                    out_clsninfo.pd_clsnobj_x = pd_input.rm_roadmodel.at(rm_Num).object_list[i].relative_center_x;
                    out_clsninfo.pd_clsnobj_y = pd_input.rm_roadmodel.at(rm_Num).object_list[i].relative_center_y;
                    out_clsninfo.pd_clsnobj_vx = pd_input.rm_roadmodel.at(rm_Num).object_list[i].absolute_velocity_x;
                }
            }

            // get ttc,td
            for (int j = 0; j < dynmcobjrsk.m_DynObjRsk_list.size(); j++)
            {
                if (out_clsninfo.pd_clsnobj_id == dynmcobjrsk.m_DynObjRsk_list[j].ObjID)
                {
                    out_clsninfo.pd_clsnobj_ttc = dynmcobjrsk.m_DynObjRsk_list[j].m_ObjLongPreClsnTTC.front();
                    out_clsninfo.pd_clsnobj_td = dynmcobjrsk.m_DynObjRsk_list[j].m_ObjLongPreClsnTD.front();
                }
            }
            pdclsninfo.emplace_back(out_clsninfo);
        }
    }

    // lgtspd limit info
    pdobsvar.pd_desiredobj_id = lgttrjinfo.pd_lgttrjinput.PD_LgtTrj_TgtObjID;
    pdobsvar.pd_desireddist = lgttrjinfo.pd_lgttrjinput.PD_LgtTrj_DesDist;
    pdobsvar.pd_desiredspd = lgttrjinfo.pd_lgttrjinput.PD_LgtTrj_DesSpd;
    pdobsvar.m_pd_obj_clsninfo = pdclsninfo;
    // pdobsvar.m_pd_dynmicobj_list = dynmcobjrsk.pd_eva_val_;

    // get obj dynamic asssement variabl
    PD::PDAsmtVar pdasmtvar;
    if (!dynmcobjrsk.pd_eva_val_.empty())
    {

        for (int i = 0; i < dynmcobjrsk.pd_eva_val_.size(); i++)
        {
            pdasmtvar.pd_obj_id = dynmcobjrsk.pd_eva_val_[i].val_obj_id;
            pdasmtvar.pd_ttc_abnml = dynmcobjrsk.pd_eva_val_[i].val_ttc;
            pdasmtvar.pd_td_abnml = dynmcobjrsk.pd_eva_val_[i].val_td;
            pdasmtvar.pd_latdist = dynmcobjrsk.pd_eva_val_[i].val_lat_dist;
            pdasmtvar.pd_lgtdist = dynmcobjrsk.pd_eva_val_[i].val_lgt_dist;

            pdasmtinfo.emplace_back(pdasmtvar);
        }
    }
    pdobsvar.m_pd_dynmicobj_list = pdasmtinfo;

    pddebuginfo.m_pd_obs_psg_list.emplace_back(pdobsvar);
}
