#include "BA_BaseInfoProcess.h"
#include "glog/logging.h"

// float P_BIP_t_CycleTime = 0.1;  // unit:s  100ms
float P_BIP_t_CycleTime = BEHAVIORANALYZERCYCLE / 1000.0;  // unit:s modify to constant value

using namespace BA;
using namespace std;
using namespace BIP;

void AvailableBehaviorDetermine(
    BA::LFCmfrtAvlblBhvr SM_LFComfortAvlblBehavior,
    BA::LCAvlblBhvr SM_LCAvlblBehavior, BA::LCPAvlblBhvr SM_LCPAvlblBehavior,
    BA::LongtAvlblBhvr SM_LongtAvlblBehavior, bool BA_LaneChangeAbolitionAvlbl,
    std::vector<BA::BA_ActionTyp> &BA_LatAvlblBehavior,
    BA::LongtAvlblBhvr &BA_LongtAvlblBehavior) {
  /*   SM_LFComfortAvlblBehavior signal correlation logic
    1. When SM_LFComfortAvlblBehavior is LFCmfrtAvlblBhvr_None or
    LFCmfrtAvlblBhvr_LongtStandAlone, BA_LatAvlblBehavior does not
    contain BA_ActionTyp_Lanefollow behavior;
    2. When SM_LFComfortAvlblBehavior is LFCmfrtAvlblBhvr_LatStandAlone or
    LFCmfrtAvlblBhvr_LongtLat_HO or LFCmfrtAvlblBhvr_LongtLat_HOInclILC or
    LFCmfrtAvlblBhvr_LongtLat_HF, BA_LatAvlblBehavior contains
    BA_ActionTyp_LaneFollow behavior;
    3. When SM_LFComfortAvlblBehavior is LFCmfrtAvlblBhvr_LongtLat_HOInclILC,
    BA_LatAvlblBehavior contains BA_ActionTyp_ILC; */
  BA_LatAvlblBehavior.clear();

  switch (SM_LFComfortAvlblBehavior) {
    case LFCmfrtAvlblBhvr_None:
      BA_LatAvlblBehavior.clear();
      break;
    case LFCmfrtAvlblBhvr_LongtStandAlone:
      BA_LatAvlblBehavior.push_back(BA_ActionTyp_LaneFollow);
      break;
    case LFCmfrtAvlblBhvr_LatStandAlone:
      BA_LatAvlblBehavior.push_back(BA_ActionTyp_LaneFollow);
      break;
    case LFCmfrtAvlblBhvr_LongtLat_HO:
      BA_LatAvlblBehavior.push_back(BA_ActionTyp_LaneFollow);
      break;
    case LFCmfrtAvlblBhvr_LongtLat_HOInclILC:
      BA_LatAvlblBehavior.push_back(BA_ActionTyp_LaneFollow);
      BA_LatAvlblBehavior.push_back(BA_ActionTyp_ILC);
      break;
    case LFCmfrtAvlblBhvr_LongtLat_HF:
      BA_LatAvlblBehavior.push_back(BA_ActionTyp_LaneFollow);
      break;
    default:
      BA_LatAvlblBehavior.clear();
      break;
  }
  /* SM_LCPAvlblBehavior signal correlation logic
  1. When SM_LCPAvlblBehavior is BA_ActionTyp_LeftLaneChangePrepare,
  BA_LatAvlblBehavior contains BA_ActionTyp_LeftLaneChangePrepare behavior;
  2. When SM_LCPAvlblBehavior is
  BA_ActionTyp_RightLaneChangePrepare,BA_LatAvlblBehavior contains
  BA_ActionTyp_RightLaneChangePrepare behavior;
  3. When SM_LCPAvlblBehavior is
  LCPAvlblBhvr_LaneChangePrepare_Auto,BA_LatAvlblBehavior contains
  BA_ActionTyp_LeftLaneChangePrepare and BA_ActionTyp_RightLaneChangePrepare
  behavior;
   */
  switch (SM_LCPAvlblBehavior) {
    case LCPAvlblBhvr_LeftLaneChangePrepare:
      BA_LatAvlblBehavior.push_back(BA_ActionTyp_LeftLaneChangePrepare);
      break;
    case LCPAvlblBhvr_RightLaneChangePrepare:
      BA_LatAvlblBehavior.push_back(BA_ActionTyp_RightLaneChangePrepare);
      break;
    case LCPAvlblBhvr_LaneChangePrepare_Auto:
      BA_LatAvlblBehavior.push_back(BA_ActionTyp_LeftLaneChangePrepare);
      BA_LatAvlblBehavior.push_back(BA_ActionTyp_RightLaneChangePrepare);
      break;
    default:
      break;
  }
  /* SM_LCAvlblBehavior signal related logic
  1. SM_LCAvlblBehavior is LCAvlblBhvr_LeftLaneChange_C, BA_LatAvlblBehavior
  behavior includes BA_ActionTyp_LeftLaneChange behavior, when the lane change
  return is available (BA_LaneChangeAbolitionAvlbl is true),
  BA_LatTyp_LaneChange_LaneChange behavior is not included in
  BA_LatAvlblBehavior behavior.
  2. SM_LCAvlblBehavior is LCAvlblBhvr_RightLaneChange_C, and
  BA_LatAvlblBehavior behavior includes BA_ActionTyp_RightLaneChange behavior.
  When lane change return is available (BA_LaneChangeAbolitionAvlbl is true),
  BA_LatTyp_LaneChange behavior does not include BA_LatTyp_LaneBehavior
  behavior.
  3. SM_LCAvlblBehavior is LCAvlblBhvr_LaneChange_Auto, BA_LatAvlblBehavior
  behavior includes BA_ActionTyp_LeftLaneChange and BA_ActionTyp_RightLaneChange
  behaviors. When the lane change return is available
  (BA_LaneChangeAbolitionAvlbl is trueAvl when BA_LaneChangeAbolitionAvlBeh is
  included in the inverse behavior, BA_LaneChangeAbolitionAvlBeh is trueAvl)
  4. SM_LCAvlblBehavior is LCAvlblBhvr_LeftLaneChangeAbolition_C, then
  BA_LatAvlblBehavior only contains the BA_ActionTyp_LeftLaneChangeAbolition
  behavior available (regardless of other conditions);
  5. SM_LCAvlblBehavior is LCAvlblBhvr_RightLaneChangeAbolition_C, then
  BA_LatAvlblBehavior only contains BA_ActionTyp_RightLaneChangeAbolition
  behavior available (regardless of other conditions)
   */
  switch (SM_LCAvlblBehavior) {
    case LCAvlblBhvr_LeftLaneChange_C:
      BA_LatAvlblBehavior.push_back(BA_ActionTyp_LeftLaneChange);
      /*       if (BA_LaneChangeAbolitionAvlbl)
              BA_LatAvlblBehavior.push_back(BA_ActionTyp_LeftLaneChangeAbolition);
       */
      break;
    case LCAvlblBhvr_RightLaneChange_C:
      BA_LatAvlblBehavior.push_back(BA_ActionTyp_RightLaneChange);
      break;
    case LCAvlblBhvr_LaneChange_Auto:
      BA_LatAvlblBehavior.push_back(BA_ActionTyp_LeftLaneChange);
      BA_LatAvlblBehavior.push_back(BA_ActionTyp_RightLaneChange);
      break;
    case LCAvlblBhvr_LeftLaneChange_Auto:
      BA_LatAvlblBehavior.push_back(BA_ActionTyp_LeftLaneChange);
      break;
    case LCAvlblBhvr_RightLaneChange_Auto:
      BA_LatAvlblBehavior.push_back(BA_ActionTyp_RightLaneChange);
      break;
    case LCAvlblBhvr_LeftLaneChangeAbolition_C:
      if (BA_LaneChangeAbolitionAvlbl) {
        // BA_LatAvlblBehavior.clear();
        BA_LatAvlblBehavior.push_back(BA_ActionTyp_LeftLaneChangeAbolition);
      } else {
        BA_LatAvlblBehavior.push_back(BA_ActionTyp_LeftLaneChange);
      }
      break;
    case LCAvlblBhvr_RightLaneChangeAbolition_C:
      if (BA_LaneChangeAbolitionAvlbl) {
        // BA_LatAvlblBehavior.clear();
        BA_LatAvlblBehavior.push_back(BA_ActionTyp_RightLaneChangeAbolition);
      } else {
        BA_LatAvlblBehavior.push_back(BA_ActionTyp_RightLaneChange);
      }
      break;
    default:
      break;
  }

  if (!BA_LatAvlblBehavior.empty()) {
    sort(BA_LatAvlblBehavior.begin(), BA_LatAvlblBehavior.end());

    BA_LatAvlblBehavior.erase(
        unique(BA_LatAvlblBehavior.begin(), BA_LatAvlblBehavior.end()),
        BA_LatAvlblBehavior.end());
  }

  BA_LongtAvlblBehavior = SM_LongtAvlblBehavior;
}

void DirectionIndicatorSignalProcess(
    BA::LCAvlblBhvr SM_LCAvlblBhvr,
    DM::DirectionIndicatorDuration &BA_DirectionIndicatorInfo)
{
  static uint64_t leftcount = 0;
  static uint64_t rightcount = 0;

  if (SM_LCAvlblBhvr == LCAvlblBhvr::LCAvlblBhvr_LeftLaneChange_C || SM_LCAvlblBhvr == BA::LCAvlblBhvr_LeftLaneChange_Auto)
  {
    leftcount++;
    rightcount = 0;
  }
  else
  {
    if (SM_LCAvlblBhvr == LCAvlblBhvr::LCAvlblBhvr_RightLaneChange_C || SM_LCAvlblBhvr == BA::LCAvlblBhvr_RightLaneChange_Auto)
    {
      leftcount = 0;
      rightcount++;
      
    }
    else
    {
      leftcount = 0;
      rightcount = 0;
      
    }
  }

  BA_DirectionIndicatorInfo.m_left_direction_indicator_duration =
      leftcount * P_BIP_t_CycleTime;
  BA_DirectionIndicatorInfo.m_right_direction_indicator_duration =
      rightcount * P_BIP_t_CycleTime;
  //LOG(INFO)<<"indicator time is "<<BA_DirectionIndicatorInfo.m_right_direction_indicator_duration;

  return;
}

void HistoryBehaviorStack(BA::BA_ActionTyp BA_ActionProc,
                          std::vector<HistoryBehavior> &BehaviorStackVector) {
  const int StackSize = 5;

  static vector<HistoryBehavior> HistoryBehaviorDeque(
      StackSize, HistoryBehavior(BA_ActionTyp_None, float(0), float(0)));

  HistoryBehavior temp;

  HistoryBehaviorDeque[0].PassTime = HistoryBehaviorDeque[0].Duration;

  HistoryBehaviorDeque[0].Duration = float(0);

  if (BA_ActionProc != HistoryBehaviorDeque[0].Action) {
    temp.Action = BA_ActionProc;

    temp.Duration = float(0);

    temp.PassTime = float(0);

    HistoryBehaviorDeque[0].Duration = HistoryBehaviorDeque[0].PassTime;

    HistoryBehaviorDeque[0].PassTime = 0;

    HistoryBehaviorDeque.insert(HistoryBehaviorDeque.begin(), temp);

    HistoryBehaviorDeque.pop_back();
  }

  for_each(HistoryBehaviorDeque.begin(), HistoryBehaviorDeque.end(),
           [](HistoryBehavior &HistoryBehaviorElement) {
             HistoryBehaviorElement.PassTime += P_BIP_t_CycleTime;
             return;
           });

  HistoryBehaviorDeque[0].Duration = HistoryBehaviorDeque[0].PassTime;

  HistoryBehaviorDeque[0].PassTime = float(0);

  BehaviorStackVector = HistoryBehaviorDeque;
  
  #ifdef GLOG
  int cnt = 0;
  // DLOG(INFO) << "behavior stack vector size = " << int(BehaviorStackVector.size()) << std::endl;
	// for (auto behaviorElem : BehaviorStackVector)
	// {
  //   cnt = cnt + 1;
  //   DLOG(INFO) << "num = " << cnt << std::endl;
	//   DLOG(INFO) << "action = " << int(behaviorElem.Action) << std::endl;
  //   DLOG(INFO) << "duration = " << behaviorElem.Duration << std::endl;
  //   DLOG(INFO) << "pass time = " << behaviorElem.PassTime << std::endl;
	// }
	#endif

  return;
}
