#pragma once

#include <vector>
#include "../include/CtApBehaviorAnalysis_/BA_Enum_List.h"
// #include "mpc_drv_msgs/msg/direction_indicator_duration.hpp"
#include "../include/CtApBehaviorAnalysis_/interface/dm_internal_interface.h"

#include <algorithm>
#include <deque>
#include "../commonfunc.h"

extern float P_BIP_t_CycleTime;

namespace BIP {
struct HistoryBehavior {
  HistoryBehavior(BA::BA_ActionTyp _Action, float _Duration, float _PassTime)
      : Action(_Action), Duration(_Duration), PassTime(_PassTime) {}
  HistoryBehavior(){};

  BA::BA_ActionTyp Action;

  float Duration;

  float PassTime;
};
}  // namespace BIP

/**
 *The module outputs the list of behaviors available for the current BA behavior
 *decision according to the driver command input signal processed by SM.
 *@param  SM_LFComfortAvlblBehavior lane follow comfort available behavior.
 *@param  SM_LCAvlblBehavior lane change available behavior.
 *@param  SM_LCPAvlblBehavior lane change preparing available behavior.
 *@param  SM_LongtAvlblBehavior longitudinal available behavior.
 *@param  BA_LaneChangeAbolitionAvlbl a boolean indicating whether the vehicle
 *can cancle the lane change behavior or not..
 *@param  BA_LatAvlblBehavior the lateral available behavior list.
 *@param  BA_LongtAvlblBehavior the longitudinal available behavior list.
 *@return void.
 */
void AvailableBehaviorDetermine(
    BA::LFCmfrtAvlblBhvr SM_LFComfortAvlblBehavior,
    BA::LCAvlblBhvr SM_LCAvlblBehavior, BA::LCPAvlblBhvr SM_LCPAvlblBehavior,
    BA::LongtAvlblBhvr SM_LongtAvlblBehavior, bool BA_LaneChangeAbolitionAvlbl,
    std::vector<BA::BA_ActionTyp> &BA_LatAvlblBehavior,
    BA::LongtAvlblBhvr &BA_LongtAvlblBehavior);

/**
 *Used to obtain the on time of left and right direction indicator.
 *@param  SM_DirectionIndicatorSts the status of  direction indicator.
 *@param  BA_DirectionIndicatorInfo the on time of  direction indicator.
 *@return void.
 */
void DirectionIndicatorSignalProcess(
    BA::LCAvlblBhvr SM_LCAvlblBhvr,
    DM::DirectionIndicatorDuration &BA_DirectionIndicatorInfo);

void HistoryBehaviorStack(
    BA::BA_ActionTyp BA_ActionProc,
    std::vector<BIP::HistoryBehavior> &BehaviorStackVector);
