/**
 * @Author: Zhang Bin
 * @Date: 2021-11-17 10:37:00
 * @LastEditTime: 2021-09-28 13:34:06
 * @LastEditors: Please set LastEditors
 * @Description: In User Settings Editd
 * @FilePath: ******.h
 */

#include "BA_CurrentBehaviorJudgement.h"
#include "glog/logging.h"
#include "math.h"

#define pi (float)3.141592
const float C_CBJ_d_WheelBase_sg = 1.571; // WheelBase 1.571m
const float C_CBJ_d_FrtwheelTrd_sg = 2.715;
float P_CBJ_d_CenteringJudgmentDisThrdHigh = 0.35; // 0.25
float P_CBJ_d_CenteringJudgmentDisThrdLow = 0.3;   // 0.2
float P_CBJ_d_CenteringJudgmentA1ThrdHigh = 0.174; // 0.174
float P_CBJ_d_CenteringJudgmentAglThrdLow = 0.087; // 0.087
float P_CBJ_d_CrosJudgmentFrtDisThrdHigh = 0.15;
float P_CBJ_d_CrosJudgmentFrtDisThrdLow = -0.15;
float P_CBJ_d_CrosJudgmentMidDisThrdHigh = 0.1;
float P_CBJ_d_CrosJudgmentMidDisThrdLow = -0.1;
float P_CBJ_d_ClsToLnCntrJudgmentDisThrd = 0.65;
// float P_CBJ_t_CycleTime = 0.1;                     // unit:s
float P_CBJ_t_CycleTime = BEHAVIORANALYZERCYCLE / 1000.0; // unit:s modify to constant value
float P_CBJ_t_CenteringJudgmentTimeThrd = 0.2;            // unit:s
float P_CBJ_t_NonCenteringJudgmentTimeThrd = 0.2;         // unit:s

using namespace std;
using namespace BA;
// using namespace mpc_drv_msgs;
using namespace CBJ;

bool CenteringConditionJudgment(RoadInfo &RM_RoadModel,
                                uint32_t BA_OptCorridorID,
                                const uint8_t &BA_OptLnID)
{
  Corridor _TargertCorridor;

  LineGeometry TgtCrdCtrLn_LGParam;

  // 20220528: get optCorridorID from BA_OptLnID instead of BA_OptCorridorID
  bool _CorridorFindResult = GetTargetCorridor(RM_RoadModel, BA_OptLnID, _TargertCorridor);

  // bool _CorridorFindResult = GetTargertCorridor(RM_RoadModel, BA_OptCorridorID, _TargertCorridor);

  if (_CorridorFindResult)
  {
    TgtCrdCtrLn_LGParam = _TargertCorridor.reference_line;

    LineFunctionParameters TgtCrdCtrLn_LFParam;

    LineInterfaceMatching(TgtCrdCtrLn_LGParam, TgtCrdCtrLn_LFParam);

    enum CenteringConditionState
    {
      eCCS_NoneCenteringState = 0,
      eCCS_CenteringState = 1
    };
    // static variable used to record whether the car is centered or not.
    static CenteringConditionState CurrentState_eCCS = eCCS_NoneCenteringState;

    static float _Counter_Centering = 0;
    static float _Counter_NonCentering = 0;
    bool _CenteringJudgment_bl = false;
    bool _NonCenteringJudgment_bl = false;

    // return value of the function
    static bool BA_VehInLnCenter = false;

    // the distance from the origin of the coordinate axis to centerline.
    float distance_origin_frenet = 0;
    // the angle between the cooridinate of x axis and center line.
    float angle_car_centerline = atan(TgtCrdCtrLn_LFParam.A1);
    // A approximate algorithm is used to obtain the variable
    // distance_origin_frenet. distance_origin_frenet = (0 - Ln_Param.A0) *
    // cos(fabs(atan(Ln_Param.A1)));
    FrenetLateralDistanceConversion(TgtCrdCtrLn_LFParam, 0, 0,
                                    distance_origin_frenet);
    // A small state machine is used to judge whether the vehicle is centered.

    switch (CurrentState_eCCS)
    {
    case eCCS_NoneCenteringState:
      // when the distance is smaller than the low
      // threshold:P_CBJ_d_CenteringJudgmentDisThrdLow, and the angle is
      // smaller than the low threshold:P_CBJ_d_CenteringJudgmentAglThrdLow,
      // the transition condition is fulfiled,Current state change from
      // eCCS_NoneCenteringState to eCCS_CenteringState. BA_VehInLnCenter
      // change to true

      // original version
      /* _CenteringJudgment_bl =
            (fabs(distance_origin_frenet) <=
             P_CBJ_d_CenteringJudgmentDisThrdLow) &&
            (fabs(angle_car_centerline) <= P_CBJ_d_CenteringJudgmentAglThrdLow); */

      // 20220401: modify dist judgement condition to A0 instead of frenet coordination switch
      _CenteringJudgment_bl =
          (fabs(TgtCrdCtrLn_LFParam.A0) <=
           P_CBJ_d_CenteringJudgmentDisThrdLow) &&
          (fabs(angle_car_centerline) <= P_CBJ_d_CenteringJudgmentAglThrdLow);

      if (_CenteringJudgment_bl)
      {
        _Counter_Centering++;
      }
      else
      {
        _Counter_Centering = 0;
      }

      if (_Counter_Centering * P_CBJ_t_CycleTime >=
          P_CBJ_t_CenteringJudgmentTimeThrd)
      {
        _Counter_Centering = 0;
        CurrentState_eCCS = eCCS_CenteringState;
        BA_VehInLnCenter = true;
      }
      else
      {
        BA_VehInLnCenter = false;
      }
      break;

    case eCCS_CenteringState:
      // when the distance is bigger than the hign
      // threshold:P_CBJ_d_CenteringJudgmentDisThrdHigh, and the angle is
      // bigger than the hign threshold:P_CBJ_d_CenteringJudgmentA1ThrdHigh,
      // the transition condition is fulfiled, Currentstate change from
      // eCCS_CenteringState to eCCS_NoneCenteringState. BA_VehInLnCenter
      // change to true.

      // original version
      /* _NonCenteringJudgment_bl =
            (fabs(distance_origin_frenet) >
             P_CBJ_d_CenteringJudgmentDisThrdHigh) ||
            (fabs(angle_car_centerline) > P_CBJ_d_CenteringJudgmentA1ThrdHigh); */

      _NonCenteringJudgment_bl =
          (fabs(TgtCrdCtrLn_LFParam.A0) >
           P_CBJ_d_CenteringJudgmentDisThrdHigh) ||
          (fabs(angle_car_centerline) > P_CBJ_d_CenteringJudgmentA1ThrdHigh);

      if (_NonCenteringJudgment_bl)
      {
        _Counter_NonCentering++;
      }
      else
      {
        _Counter_NonCentering = 0;
      }

      if (_Counter_NonCentering * P_CBJ_t_CycleTime >=
          P_CBJ_t_NonCenteringJudgmentTimeThrd)
      {
        _Counter_NonCentering = 0;
        CurrentState_eCCS = eCCS_NoneCenteringState;
        BA_VehInLnCenter = false;
      }
      else
      {
        BA_VehInLnCenter = true;
      }

      break;
    default:
      CurrentState_eCCS = eCCS_NoneCenteringState;
      BA_VehInLnCenter = false;
    }

    return BA_VehInLnCenter;
  }
  else
  {
    return false;
  }
}

bool LaneChangeCancelAvailableJudgement(RoadInfo &RM_RoadModel,
                                        behavior behavior_current,
                                        uint8_t CurrentlaneId,
                                        bool VehInLnCenter,
                                        bool &isLCLnIDNotChg)
{
  static uint8_t _history_line_id = CurrentlaneId;

  bool _available_result = false;

  if (VehInLnCenter)
  {
    _history_line_id = CurrentlaneId;
  }

  bool isApprLftBndry = false;
  bool isApprRghtBndry = false;
  isVehApprToLLnBndry(RM_RoadModel, isApprLftBndry);
  isVehApprToRLnBndry(RM_RoadModel, isApprRghtBndry);
  DLOG(INFO) << "isApprLftBndry = " << int(isApprLftBndry) << std::endl;
  DLOG(INFO) << "isApprRghtBndry = " << int(isApprRghtBndry) << std::endl;

  switch (behavior_current)
  {
  case BA_ActionTyp_LaneFollow:
    // _available_result = false;
    _available_result = true;
    isLCLnIDNotChg = true;
    _history_line_id = CurrentlaneId;
    break;
  case BA_ActionTyp_LeftLaneChange:
    if (CurrentlaneId != _history_line_id || isApprLftBndry == true)
    {
      _available_result = false;
    }
    else
    {
      _available_result = true;
    }
    // assign flag whether lane id has changed during lane change action
    if (CurrentlaneId != _history_line_id)
    {
      isLCLnIDNotChg = false;
    }
    else
    {
      isLCLnIDNotChg = true;
    }
    break;
  case BA_ActionTyp_RightLaneChange:
    if (CurrentlaneId != _history_line_id || isApprRghtBndry == true)
    {
      _available_result = false;
    }
    else
    {
      _available_result = true;
    }
    // assign flag whether lane id has changed during lane change action
    if (CurrentlaneId != _history_line_id)
    {
      isLCLnIDNotChg = false;
    }
    else
    {
      isLCLnIDNotChg = true;
    }
    break;
  /* case BA_ActionTyp_LeftLaneChangeAbolition:
    _available_result = true;
    _history_line_id = CurrentlaneId;
    break;
  case BA_ActionTyp_RightLaneChangeAbolition:
    _available_result = true;
    _history_line_id = CurrentlaneId;
    break; */
  default:
    _available_result = true;
    isLCLnIDNotChg = true;
    _history_line_id = CurrentlaneId;
    break;
  }

  return _available_result;
}

bool LaneChangeCancelAvailableJudgement(RoadInfo &RM_RoadModel,
                                        behavior behavior_current,
                                        _start_id_type target_lane_start_id,
                                        _end_id_type target_lane_end_id)
{
  LineFunctionParameters TgtLnLftLnMrk_LFParam;
  LineFunctionParameters TgtLnRgtLnMrk_LFParam;

  LineGeometry TgtLnLftLnMrk_LGParam;
  LineGeometry TgtLnRgtLnMrk_LGParam;

  // bool _LaneMarkFindResult = GetTargetLaneLeftAndRightMarker(
  //     RM_RoadModel, target_lane_start_id, target_lane_end_id,
  //     TgtLnLftLnMrk_LGParam, TgtLnRgtLnMrk_LGParam);
  bool _LaneMarkFindResult = false;
  if (_LaneMarkFindResult)
  {
    LineInterfaceMatching(TgtLnLftLnMrk_LGParam, TgtLnLftLnMrk_LFParam);
    LineInterfaceMatching(TgtLnRgtLnMrk_LGParam, TgtLnRgtLnMrk_LFParam);

    typedef enum
    {
      eLCA_LnChgCncNotAvlb = 0,
      eLCA_LnChgCncAvlb = 1
    } LnChngAbltAvlbState;

    // The current state of the state machine which is used to get the value of
    // the variable BA_LaneChangeAbolitionAvlbl.
    LnChngAbltAvlbState CurrentState_eLCA = eLCA_LnChgCncNotAvlb;

    // return value :a boolean quantity indicating whether the vehicle can
    // cancle the lane change behavior or not.
    bool BA_LaneChangeAbolitionAvlbl = false;

    // Target lane marker which is used for lane change cancellation judgment
    // logic
    LineFunctionParameters TgtLn_Param;

    // distance from origin to Target lane marker
    float distance_front_axis = 0;
    // distance from vehicle center to Target lane marker
    float distance_car_center = 0;
    // unify lane Changing to left side situation by multiplying the distance by
    // the direction variable
    float direction = 1;

    // Judge which line marker is used for lane change cancellation judgment
    // logic according to the variables behavior_current and
    // behavior_last_different.
    if ((behavior_current == BA_ActionTyp_LeftLaneChange))
    {
      TgtLn_Param = TgtLnRgtLnMrk_LFParam;
      direction = 1;
    }
    else if ((behavior_current == BA_ActionTyp_LeftLaneChangeAbolition))
    {
      TgtLn_Param = TgtLnLftLnMrk_LFParam;
      direction = 1;
    }
    else if ((behavior_current == BA_ActionTyp_RightLaneChange))
    {
      TgtLn_Param = TgtLnLftLnMrk_LFParam;
      direction = -1;
    }
    else if ((behavior_current == BA_ActionTyp_RightLaneChangeAbolition))
    {
      TgtLn_Param = TgtLnRgtLnMrk_LFParam;
      direction = -1;
    }
    else
    {
      TgtLn_Param = TgtLnLftLnMrk_LFParam;
      direction = 1;
    }

    // Calculate the distance between coordinate axis origin (vehicle front axle
    // center) and the vehicle center from the reference line, which is unified
    // into Lane Changing to left situation by mutiplying the direction variable
    FrenetLateralDistanceConversion(TgtLn_Param, 0, 0, distance_front_axis);

    FrenetLateralDistanceConversion(TgtLn_Param, -C_CBJ_d_FrtwheelTrd_sg / 2, 0,
                                    distance_car_center);

    distance_front_axis = direction * distance_front_axis;
    distance_car_center = direction * distance_car_center;

    // A small state machine used to judge whether ego car can abort lane change
    // behavior
    switch (CurrentState_eLCA)
    {
    //(when the variable distance_front_axis is smaller then low threshold
    // P_CBJ_d_CrosJudgmentFrtDisThrdLow and the variable distance_car_center
    // is smaller then low threshold P_CBJ_d_CrosJudgmentMidDisThrdLow)
    // transition condition if fulfiled and Current state changes
    // from eLCA_LnChgCncNotAvlb to eLCA_LnChgCncAvlb.
    case eLCA_LnChgCncNotAvlb:
      if (((distance_front_axis <= P_CBJ_d_CrosJudgmentFrtDisThrdLow) &&
           (distance_car_center <= P_CBJ_d_CrosJudgmentMidDisThrdLow)))
      {
        CurrentState_eLCA = eLCA_LnChgCncAvlb;
        BA_LaneChangeAbolitionAvlbl = true;
      }
      break;
    //(when the variable distance_front_axis is bigger then hign threshold
    // P_CBJ_d_CrosJudgmentFrtDisThrdHigh and  the variable
    // distance_car_center is bigger then hign threshold
    // P_CBJ_d_CrosJudgmentMidDisThrdHigh) transition condition if fulfiled
    // and Current state changes from eLCA_LnChgCncAvlb to
    // eLCA_LnChgCncNotAvlb.
    case eLCA_LnChgCncAvlb:
      if (((distance_front_axis >= P_CBJ_d_CrosJudgmentFrtDisThrdHigh) &&
           (distance_car_center >= P_CBJ_d_CrosJudgmentMidDisThrdHigh)))
      {
        CurrentState_eLCA = eLCA_LnChgCncNotAvlb;
        BA_LaneChangeAbolitionAvlbl = false;
      }
      break;
    default:
      CurrentState_eLCA = eLCA_LnChgCncNotAvlb;
      BA_LaneChangeAbolitionAvlbl = false;
    }

    return BA_LaneChangeAbolitionAvlbl;
  }
  else
  {
    return true;
  }
}

bool CurrentBehaviorJudgement(RoadInfo &RM_RoadModel,
                              _start_id_type target_lane_start_id,
                              _end_id_type target_lane_end_id,
                              uint32_t BA_OptCorridorID,
                              BA_ActionTyp BA_OptAction,
                              bool &BA_VehInLnCenter,
                              bool &BA_LaneChangeAbolitionAvlbl,
                              behavior &BA_ActionProc,
                              bool &BA_ISVehCls2LnCntr,
                              bool &BA_ISLCLnIDnotChg,
                              uint8_t &BA_OptLaneID)
{
  behavior behavior_last_cycle = BA_OptAction;

  BA_VehInLnCenter = CenteringConditionJudgment(RM_RoadModel, BA_OptCorridorID, BA_OptLaneID);

  /*   BA_LaneChangeAbolitionAvlbl = LaneChangeCancelAvailableJudgement(
        RM_RoadModel, behavior_last_cycle, target_lane_start_id,
        target_lane_end_id); */

  BA_LaneChangeAbolitionAvlbl = LaneChangeCancelAvailableJudgement(
      RM_RoadModel, behavior_last_cycle, RM_RoadModel.car_position.ego_lane_id,
      BA_VehInLnCenter, BA_ISLCLnIDnotChg);

  BA_ActionProc = CurrentBehaviorProcess(behavior_last_cycle, BA_VehInLnCenter,
                                         BA_LaneChangeAbolitionAvlbl);
  
  BA_ISVehCls2LnCntr = isVehClsToLnCntr(RM_RoadModel, BA_OptCorridorID, BA_OptLaneID);

  return true;
}

bool GetTargertCorridor(RoadInfo &RM_RoadModel,
                        uint32_t BA_OptCorridorID,
                        Corridor &TargertCorridor)
{
  auto it_lane =
       find_if(RM_RoadModel.lane_list.begin(), RM_RoadModel.lane_list.end(),
              [&](Lane &_lane)
              {
                auto it_cor = find_if(
                    _lane.corridor_list.begin(), _lane.corridor_list.end(),
                    [&](Corridor &_corridor)
                    {
                      if (_corridor.corridor_id == BA_OptCorridorID)
                        return true;
                      else
                        return false;
                    });

                if (!_lane.corridor_list.empty() &&
                    it_cor != _lane.corridor_list.end())
                {
                  TargertCorridor = *it_cor;
                  return true;
                }
                else
                  return false;
              });

  if (!RM_RoadModel.lane_list.empty() &&
      it_lane != RM_RoadModel.lane_list.end())
    return true;
  else
    return false;
  ///////////////////////////////////////////////////////////////////////
  /*   typedef std::vector<Lane> _lane_list_type;
    typedef std::vector<Corridor> _corridor_list_type;

    for (_lane_list_type::iterator it_i = RM_RoadModel.lane_list.begin();
         it_i != RM_RoadModel.lane_list.end(); it_i++)
    {
      for (_corridor_list_type::iterator it_j = (it_i->corridor_list).begin();
           it_j != (it_i->corridor_list).end(); it_j++)
      {
        if (it_j->corridor_id == BA_OptCorridorID)
        {
          TargertCorridor = *it_j;
          return true;
          break;
        }
      }
    }
    return false; */
}

bool GetTargertCorridorLaneGeometry(
    RoadInfo &RM_RoadModel, uint32_t BA_OptCorridorID,
    LineGeometry &TgtCrdCtrLn_Param)
{
  auto it_lane =
      find_if(RM_RoadModel.lane_list.begin(), RM_RoadModel.lane_list.end(),
              [&](Lane &_lane)
              {
                auto it_cor = find_if(
                    _lane.corridor_list.begin(), _lane.corridor_list.end(),
                    [&](Corridor &_corridor)
                    {
                      if (_corridor.corridor_id == BA_OptCorridorID)
                        return true;
                      else
                        return false;
                    });

                if (!_lane.corridor_list.empty() &&
                    it_cor != _lane.corridor_list.end())
                {
                  TgtCrdCtrLn_Param = it_cor->reference_line;
                  return true;
                }
                else
                  return false;
              });

  if (!RM_RoadModel.lane_list.empty() &&
      it_lane != RM_RoadModel.lane_list.end())
    return true;
  else
    return false;

  /*   typedef std::vector<Lane> _lane_list_type;
    typedef std::vector<Corridor> _corridor_list_type;

    for (_lane_list_type::iterator it_i = RM_RoadModel.lane_list.begin();
         it_i != RM_RoadModel.lane_list.end(); it_i++)
    {
      for (_corridor_list_type::iterator it_j = (it_i->corridor_list).begin();
           it_j != (it_i->corridor_list).end(); it_j++)
      {
        if (it_j->corridor_id == BA_OptCorridorID)
        {
          TgtCrdCtrLn_Param = it_j->reference_line;
          return true;
          break;
        }
      }
    }
    return false; */
}

// bool GetTargetLaneLeftAndRightMarker(
//     RoadInfo &RM_RoadModel, _start_id_type target_lane_start_id,
//     _end_id_type target_lane_end_id,
//     LineGeometry &TgtLnLftLnMrk_LGParam,
//     LineGeometry &TgtLnRgtLnMrk_LGParam)
// {
  // typedef std::vector<Lane> _lane_list_type;

  // uint32_t left_boundary_id = 0;
  // uint32_t right_boundary_id = 0;

  /* for (_lane_list_type::iterator it = RM_RoadModel.lane_list.begin();
       it != RM_RoadModel.lane_list.end(); it++) {
    if (it->start_id == target_lane_start_id &&
        it->end_id == target_lane_end_id) {
      if (!it->left_boundary_id_list.empty() &&
          !it->right_boundary_id_list.empty()) {
        left_boundary_id = it->left_boundary_id_list.at(0);
        right_boundary_id = it->right_boundary_id_list.at(0);
        break;
      }
    }
  }

  int _counter = 0;

  if (left_boundary_id != 0 && right_boundary_id != 0) {
    typedef std::vector<LaneBoundary> _lane_boundary_list_type;

    for (_lane_boundary_list_type::iterator it =
             RM_RoadModel.lane_boundary_list.begin();
         it != RM_RoadModel.lane_boundary_list.end(); it++) {
      if (it->line_boundary_id == left_boundary_id) {
        TgtLnLftLnMrk_LGParam = it->line_geometry;
        _counter++;
        if (_counter == 2) break;
      }

      if (it->line_boundary_id == right_boundary_id) {
        TgtLnRgtLnMrk_LGParam = it->line_geometry;
        _counter++;
        if (_counter == 2) break;
      }
    }
  } */

  /* if (_counter == 2)
    return true;
  else */
//   return false;
// }

void LineInterfaceMatching(const LineGeometry &LnGmt_Param,
                           LineFunctionParameters &Ln_Param)
{
  Ln_Param.A0 = LnGmt_Param.c0;
  Ln_Param.A1 = LnGmt_Param.c1;
  Ln_Param.A2 = LnGmt_Param.c2;
  Ln_Param.A3 = LnGmt_Param.c0;
  Ln_Param.start_range = LnGmt_Param.start_range;
  Ln_Param.end_range = LnGmt_Param.end_range;
  return;
}

bool CenteringConditionJudgment(LineFunctionParameters Ln_Param)
{
  enum CenteringConditionState
  {
    eCCS_NoneCenteringState = 0,
    eCCS_CenteringState = 1
  };
  // static variable used to record whether the car is centered or not.
  static CenteringConditionState CurrentState_eCCS = eCCS_NoneCenteringState;

  static float _Counter_Centering = 0;
  static float _Counter_NonCentering = 0;
  bool _CenteringJudgment_bl = false;
  bool _NonCenteringJudgment_bl = false;

  // return value of the function
  bool BA_VehInLnCenter = false;

  // the distance from the origin of the coordinate axis to centerline.
  float distance_origin_frenet = 0;
  // the angle between the cooridinate of x axis and center line.
  float angle_car_centerline = atan(Ln_Param.A1);
  // A approximate algorithm is used to obtain the variable
  // distance_origin_frenet. distance_origin_frenet = (0 - Ln_Param.A0) *
  // cos(fabs(atan(Ln_Param.A1)));
  FrenetLateralDistanceConversion(Ln_Param, 0, 0, distance_origin_frenet);
  // A small state machine is used to judge whether the vehicle is centered.

  switch (CurrentState_eCCS)
  {
  case eCCS_NoneCenteringState:
    // when the distance is smaller than the low
    // threshold:P_CBJ_d_CenteringJudgmentDisThrdLow, and the angle is smaller
    // than the low threshold:P_CBJ_d_CenteringJudgmentAglThrdLow, the
    // transition condition is fulfiled,Current state change from
    // eCCS_NoneCenteringState to eCCS_CenteringState. BA_VehInLnCenter change
    // to true

    _CenteringJudgment_bl =
        (fabs(distance_origin_frenet) <=
         P_CBJ_d_CenteringJudgmentDisThrdLow) &&
        (fabs(angle_car_centerline) <= P_CBJ_d_CenteringJudgmentAglThrdLow);

    if (_CenteringJudgment_bl)
    {
      _Counter_Centering++;
    }
    else
    {
      _Counter_Centering = 0;
    }

    if (_Counter_Centering * P_CBJ_t_CycleTime >=
        P_CBJ_t_CenteringJudgmentTimeThrd)
    {
      _Counter_Centering = 0;
      CurrentState_eCCS = eCCS_CenteringState;
      BA_VehInLnCenter = true;
    }
    else
    {
      BA_VehInLnCenter = false;
    }
    break;

  case eCCS_CenteringState:
    // when the distance is bigger than the hign
    // threshold:P_CBJ_d_CenteringJudgmentDisThrdHigh, and the angle is bigger
    // than the hign threshold:P_CBJ_d_CenteringJudgmentA1ThrdHigh, the
    // transition condition is fulfiled, Currentstate change from
    // eCCS_CenteringState to eCCS_NoneCenteringState. BA_VehInLnCenter change
    // to true.
    _NonCenteringJudgment_bl =
        (fabs(distance_origin_frenet) >
         P_CBJ_d_CenteringJudgmentDisThrdHigh) ||
        (fabs(angle_car_centerline) > P_CBJ_d_CenteringJudgmentA1ThrdHigh);

    if (_NonCenteringJudgment_bl)
    {
      _Counter_NonCentering++;
    }
    else
    {
      _Counter_NonCentering = 0;
    }

    if (_Counter_NonCentering * P_CBJ_t_CycleTime >=
        P_CBJ_t_NonCenteringJudgmentTimeThrd)
    {
      _Counter_NonCentering = 0;
      CurrentState_eCCS = eCCS_NoneCenteringState;
      BA_VehInLnCenter = false;
    }
    else
    {
      BA_VehInLnCenter = true;
    }

    break;
  default:
    CurrentState_eCCS = eCCS_NoneCenteringState;
    BA_VehInLnCenter = false;
  }

  return BA_VehInLnCenter;
}

bool LaneChangeCancelAvailableJudgement(
    behavior behavior_last_cycle, LineFunctionParameters TgtLnLftLnMrk_Param,
    LineFunctionParameters TgtLnRgtLnMrk_Param)
{
  typedef enum
  {
    eLCA_LnChgCncNotAvlb = 0,
    eLCA_LnChgCncAvlb = 1
  } LnChngAbltAvlbState;

  // The current state of the state machine which is used to get the value of
  // the variable BA_LaneChangeAbolitionAvlbl.
  LnChngAbltAvlbState CurrentState_eLCA = eLCA_LnChgCncNotAvlb;

  // return value :a boolean quantity indicating whether the vehicle can cancle
  // the lane change behavior or not.
  bool BA_LaneChangeAbolitionAvlbl = false;

  // Target lane marker which is used for lane change cancellation judgment
  // logic
  LineFunctionParameters TgtLn_Param;

  // distance from origin to Target lane marker
  float distance_front_axis = 0;
  // distance from vehicle center to Target lane marker
  float distance_car_center = 0;
  // unify lane Changing to left side situation by multiplying the distance by
  // the direction variable
  float direction = 1;

  // Judge which line marker is used for lane change cancellation judgment logic
  // according to the variables behavior_last_cycle and behavior_last_different.
  if ((behavior_last_cycle == BA_ActionTyp_LeftLaneChange))
  {
    TgtLn_Param = TgtLnRgtLnMrk_Param;
    direction = 1;
  }
  else if ((behavior_last_cycle == BA_ActionTyp_LeftLaneChangeAbolition))
  {
    TgtLn_Param = TgtLnLftLnMrk_Param;
    direction = 1;
  }
  else if ((behavior_last_cycle == BA_ActionTyp_RightLaneChange))
  {
    TgtLn_Param = TgtLnLftLnMrk_Param;
    direction = -1;
  }
  else if ((behavior_last_cycle == BA_ActionTyp_RightLaneChangeAbolition))
  {
    TgtLn_Param = TgtLnRgtLnMrk_Param;
    direction = -1;
  }
  else
  {
    TgtLn_Param = TgtLnLftLnMrk_Param;
    direction = 1;
  }

  // Calculate the distance between coordinate axis origin (vehicle front axle
  // center) and the vehicle center from the reference line, which is unified
  // into Lane Changing to left situation by mutiplying the direction variable
  FrenetLateralDistanceConversion(TgtLn_Param, 0, 0, distance_front_axis);

  FrenetLateralDistanceConversion(TgtLn_Param, -C_CBJ_d_FrtwheelTrd_sg / 2, 0,
                                  distance_car_center);

  distance_front_axis = direction * distance_front_axis;
  distance_car_center = direction * distance_car_center;

  // A small state machine used to judge whether ego car can abort lane change
  // behavior
  switch (CurrentState_eLCA)
  {
  //(when the variable distance_front_axis is smaller then low threshold
  // P_CBJ_d_CrosJudgmentFrtDisThrdLow and the variable distance_car_center is
  // smaller then low threshold P_CBJ_d_CrosJudgmentMidDisThrdLow)
  // transition condition if fulfiled and Current state changes
  // from eLCA_LnChgCncNotAvlb to eLCA_LnChgCncAvlb.
  case eLCA_LnChgCncNotAvlb:
    if (((distance_front_axis <= P_CBJ_d_CrosJudgmentFrtDisThrdLow) &&
         (distance_car_center <= P_CBJ_d_CrosJudgmentMidDisThrdLow)))
    {
      CurrentState_eLCA = eLCA_LnChgCncAvlb;
      BA_LaneChangeAbolitionAvlbl = true;
    }
    break;
  //(when the variable distance_front_axis is bigger then hign threshold
  // P_CBJ_d_CrosJudgmentFrtDisThrdHigh and  the variable distance_car_center
  // is bigger then hign threshold P_CBJ_d_CrosJudgmentMidDisThrdHigh)
  // transition condition if fulfiled and Current state changes from
  // eLCA_LnChgCncAvlb to eLCA_LnChgCncNotAvlb.
  case eLCA_LnChgCncAvlb:
    if (((distance_front_axis >= P_CBJ_d_CrosJudgmentFrtDisThrdHigh) &&
         (distance_car_center >= P_CBJ_d_CrosJudgmentMidDisThrdHigh)))
    {
      CurrentState_eLCA = eLCA_LnChgCncNotAvlb;
      BA_LaneChangeAbolitionAvlbl = false;
    }
    break;
  default:
    CurrentState_eLCA = eLCA_LnChgCncNotAvlb;
    BA_LaneChangeAbolitionAvlbl = false;
  }

  return BA_LaneChangeAbolitionAvlbl;
}

behavior CurrentBehaviorProcess(const behavior behavior_last_cycle,
                                bool BA_VehInLnCenter,
                                bool BA_LaneChangeAbolitionAvlbl)
{
  behavior BA_ActionProc;

  if (((behavior_last_cycle == BA_ActionTyp_LeftLaneChange ||
        behavior_last_cycle == BA_ActionTyp_RightLaneChange) &&
       BA_LaneChangeAbolitionAvlbl == false) &&
      (BA_VehInLnCenter == true))
  {
    BA_ActionProc = BA_ActionTyp_LaneFollow;
  }
  else if ((behavior_last_cycle == BA_ActionTyp_LeftLaneChangeAbolition ||
            behavior_last_cycle == BA_ActionTyp_RightLaneChangeAbolition) &&
           BA_LaneChangeAbolitionAvlbl == true)
  {
    BA_ActionProc = BA_ActionTyp_LaneFollow;
  }
  else
  {
    BA_ActionProc = behavior_last_cycle;
  }
  return BA_ActionProc;
}

float YCtsPntLineFuction(LineFunctionParameters Ln_Param,
                         const float X_CtsPnt)
{
  float Y_CtsPnt = 0;
  Y_CtsPnt = Ln_Param.A0 + Ln_Param.A1 * X_CtsPnt +
             Ln_Param.A2 * X_CtsPnt * X_CtsPnt / 2.0 +
             Ln_Param.A3 * X_CtsPnt * X_CtsPnt * X_CtsPnt / 6.0;
  return Y_CtsPnt;
}

float ReferenceLineDistance(LineFunctionParameters Ln_Param,
                            float nearest_point_x,
                            int interpolation_point_num)
{
  float distance = 0;

  float iterator_num = interpolation_point_num + 1;
  float tolerance = nearest_point_x / iterator_num;

  float last_point[2] = {0, 0};
  float current_point[2] = {0, YCtsPntLineFuction(Ln_Param, current_point[0])};

  for (int i = 0; current_point[0] < nearest_point_x; i++)
  {
    last_point[0] = current_point[0];
    last_point[1] = current_point[1];

    current_point[0] += tolerance;
    current_point[1] = YCtsPntLineFuction(Ln_Param, current_point[0]);

    distance += float(sqrt(pow((double)(current_point[0] - last_point[0]), 2) +
                           pow((double)(current_point[1] - last_point[1]), 2)));
  }

  return distance;
}

bool FrenetLateralDistanceConversion(LineFunctionParameters Ln_Param,
                                     float X_CtsPnt, float Y_CtsPnt,
                                     float &D_Frnpnt)
{
  bool if_valid = false;

  bool valid_X_point =
      (Ln_Param.start_range <= X_CtsPnt) && (X_CtsPnt <= Ln_Param.end_range);

  if (valid_X_point)
  {
    if_valid = true;
  }
  else
  {
    if_valid = false;
  }

  float theta_point_x = 0;

  float delta_y = 0;

  theta_point_x = atan(Ln_Param.A1 + Ln_Param.A2 * X_CtsPnt +
                       Ln_Param.A3 * X_CtsPnt * X_CtsPnt / 2.0);

  delta_y = Y_CtsPnt - YCtsPntLineFuction(Ln_Param, X_CtsPnt);

  D_Frnpnt = delta_y * cos(fabs(theta_point_x));

  return if_valid;
}

bool PointCoordinateSystemConversion(LineFunctionParameters Ln_Param,
                                     CartesianPoint Cts_Pnt,
                                     FrenetPoint &FntPnt)
{
  bool if_valid = false;

  bool valid_X_point = (Ln_Param.start_range <= Cts_Pnt.X_CtsPnt) &&
                       (Cts_Pnt.X_CtsPnt <= Ln_Param.end_range);

  if (valid_X_point)
  {
    if_valid = true;
  }
  else
  {
    if_valid = false;
  }

  float theta_point_x = 0;
  float delta_y = 0;
  float sign_delta_y = 0;
  float vector_pnt2nearest[2] = {0, 0};
  float vector_org2nearest[2] = {0, 0};
  float theta_point_nearest = 0;
  float vector_TrsMtrx[2] = {0, 0};

  theta_point_x = atan(Ln_Param.A1 + Ln_Param.A2 * Cts_Pnt.X_CtsPnt +
                       Ln_Param.A3 * Cts_Pnt.X_CtsPnt * Cts_Pnt.X_CtsPnt / 2.0);

  delta_y = Cts_Pnt.Y_CtsPnt - YCtsPntLineFuction(Ln_Param, Cts_Pnt.X_CtsPnt);

  sign_delta_y = sign<float>(delta_y);

  FntPnt.D_Frnpnt = delta_y * cos(fabs(theta_point_x));

  vector_pnt2nearest[0] =
      fabs(FntPnt.D_Frnpnt) * cos(theta_point_x - sign_delta_y * pi / 2);
  vector_pnt2nearest[1] =
      fabs(FntPnt.D_Frnpnt) * sin(theta_point_x - sign_delta_y * pi / 2);

  vector_org2nearest[0] = Cts_Pnt.X_CtsPnt + vector_pnt2nearest[0];
  vector_org2nearest[1] = Cts_Pnt.Y_CtsPnt + vector_pnt2nearest[1];

  theta_point_nearest =
      atan(Ln_Param.A1 + Ln_Param.A2 * vector_org2nearest[0] +
           Ln_Param.A3 * Cts_Pnt.X_CtsPnt * vector_org2nearest[0] / 2.0);

  vector_TrsMtrx[0] = cos(theta_point_nearest);
  vector_TrsMtrx[1] = sin(theta_point_nearest);

  FntPnt.S_FrnPnt = ReferenceLineDistance(Ln_Param, vector_org2nearest[0], 100);

  FntPnt.Vs_Frnpnt = vector_TrsMtrx[0] * Cts_Pnt.Vx_CtsPnt +
                     vector_TrsMtrx[1] * Cts_Pnt.Vy_CtsPnt;
  FntPnt.Vd_Frnpnt = vector_TrsMtrx[0] * Cts_Pnt.Vy_CtsPnt -
                     vector_TrsMtrx[1] * Cts_Pnt.Vx_CtsPnt;

  FntPnt.As_Frnpnt = vector_TrsMtrx[0] * Cts_Pnt.Ax_CtsPnt +
                     vector_TrsMtrx[1] * Cts_Pnt.Ay_CtsPnt;
  FntPnt.Ad_Frnpnt = vector_TrsMtrx[0] * Cts_Pnt.Ay_CtsPnt -
                     vector_TrsMtrx[1] * Cts_Pnt.Ax_CtsPnt;

  return if_valid;
}

float LaneChangeCompletionPercentage(RoadInfo &RM_RoadModel,
                                     uint32_t BA_OptCorridorID,
                                     BA_ActionTyp BA_OptAction/* ,
                                     bool BA_LaneChangeAbolitionAvlbl,
                                     bool BA_VehInLnCenter */)
{
  LineGeometry _TgtCrdCtrLn_Param;

  LineFunctionParameters _Ln_Param;

  Corridor TargertCorridor;

  GetTargertCorridor(RM_RoadModel, BA_OptCorridorID, TargertCorridor);

  float _LaneChangePercentage = 0;

  float _distance_origin_frenet = 0;

  float _temp, _temp_up_limit, _temp_limit = 0;

  _TgtCrdCtrLn_Param = TargertCorridor.reference_line;

  LineInterfaceMatching(_TgtCrdCtrLn_Param, _Ln_Param);

  FrenetLateralDistanceConversion(_Ln_Param, 0, 0, _distance_origin_frenet);

  switch (BA_OptAction)
  {
  case BA_ActionTyp_LeftLaneChange:

    _temp =
        -(_distance_origin_frenet / (TargertCorridor.drivable_width + 1e-6));

    _temp_up_limit = min(float(1), _temp);

    _temp_limit = max(float(0), _temp_up_limit);

    _LaneChangePercentage = 1 - _temp_limit;

    break;
  case BA_ActionTyp_RightLaneChange:

    _temp = _distance_origin_frenet / (TargertCorridor.drivable_width + 1e-6);

    _temp_up_limit = min(float(1), _temp);

    _temp_limit = max(float(0), _temp_up_limit);

    _LaneChangePercentage = 1 - _temp_limit;

    break;
  default:
    _LaneChangePercentage = 0;
    break;
  }

  return _LaneChangePercentage;
}

bool isVehClsToLnCntr(RoadInfo &RM_RoadModel,
                      uint32_t BA_OptCorridorID,
                      const uint8_t &BA_OptLnID)
{
  Corridor _TargertCorridor;

  LineGeometry TgtCrdCtrLn_LGParam;

  // 20220528: get optCorridorID from BA_OptLnID instead of BA_OptCorridorID
  bool _CorridorFindResult = GetTargetCorridor(RM_RoadModel, BA_OptLnID, _TargertCorridor);

  // bool _CorridorFindResult = GetTargertCorridor(RM_RoadModel, BA_OptCorridorID, _TargertCorridor);

  if (_CorridorFindResult)
  {
    TgtCrdCtrLn_LGParam = _TargertCorridor.reference_line;

    LineFunctionParameters TgtCrdCtrLn_LFParam;

    LineInterfaceMatching(TgtCrdCtrLn_LGParam, TgtCrdCtrLn_LFParam);

    bool isClose2LaneCenter_bl = false;

    isClose2LaneCenter_bl = fabs(TgtCrdCtrLn_LFParam.A0) <= P_CBJ_d_ClsToLnCntrJudgmentDisThrd;
    if(BA_OptLnID != RM_RoadModel.car_position.ego_lane_id){
      return false;
    }
    return isClose2LaneCenter_bl;
  }
  else
  {
    return false;
  }
}

// direction values 0: left side, 1: right side
void isVehApprToLLnBndry(RoadInfo &RM_RoadModel, bool &leftApprFlg)
{
  static bool isApprLeftLnBndry = false;
  uint8_t ego_lane_id = RM_RoadModel.car_position.ego_lane_id;
  auto it_lane = std::find_if(RM_RoadModel.lane_list.begin(), RM_RoadModel.lane_list.end(),
                              [ego_lane_id](const Lane &lane_)
                              {
                                return (ego_lane_id == lane_.start_id);
                              });
  if (it_lane != RM_RoadModel.lane_list.end())
  {
    // get first lane marker id of left side
    uint32_t trgtLftLnMrkID = uint32_t(0);
    
    if (!it_lane->left_lane_marker_id_list.empty())
    {
      trgtLftLnMrkID = it_lane->left_lane_marker_id_list.front();
    }
    else
    {
      return;
    }
    // get first front lane boundary of the lane marker object
    auto it_laneMarker = std::find_if(RM_RoadModel.lane_marker_list.begin(), RM_RoadModel.lane_marker_list.end(),
                                      [trgtLftLnMrkID](const LaneMarker &laneMarker_)
                                      {
                                        return (trgtLftLnMrkID == laneMarker_.id);
                                      });
    if (it_laneMarker != RM_RoadModel.lane_marker_list.end())
    {
      LaneBoundary trgtLnBndry;
      if (!it_laneMarker->front_lane_boundary_list.empty())
      {
        trgtLnBndry = it_laneMarker->front_lane_boundary_list.front();
        LineGeometry trgtLineGeo = trgtLnBndry.line_geometry;
        float latDist = trgtLineGeo.c0;
        DLOG(INFO) << "@@latDistToLeft = " << latDist << std::endl;
        if (latDist <= ADC::common::k_BA_LftApprLnBndryLowThr && isApprLeftLnBndry == false)
        {
          isApprLeftLnBndry = true;
        }
        else if (latDist > ADC::common::k_BA_LftApprLnBndryHighThr && isApprLeftLnBndry == true)
        {
          isApprLeftLnBndry = false;
        }
        leftApprFlg = isApprLeftLnBndry;
      }
      else
      {
        return;
      }  
    } 
    
  }
  else
  {
    return;
  }
}

void isVehApprToRLnBndry(RoadInfo &RM_RoadModel, bool &rightApprFlg)
{
  static bool isApprRightLnBndry = false;
  uint8_t ego_lane_id = RM_RoadModel.car_position.ego_lane_id;
  auto it_lane = std::find_if(RM_RoadModel.lane_list.begin(), RM_RoadModel.lane_list.end(),
                              [ego_lane_id](const Lane &lane_)
                              {
                                return (ego_lane_id == lane_.start_id);
                              });
  if (it_lane != RM_RoadModel.lane_list.end())
  {
    // get first lane marker id of left side
    uint32_t trgtRghtLnMrkID = uint32_t(0);
    
    if (!it_lane->right_lane_marker_id_list.empty())
    {
      trgtRghtLnMrkID = it_lane->right_lane_marker_id_list.front();
    }
    else
    {
      return;
    }
    // get first front lane boundary of the lane marker object
    auto it_laneMarker = std::find_if(RM_RoadModel.lane_marker_list.begin(), RM_RoadModel.lane_marker_list.end(),
                                      [trgtRghtLnMrkID](const LaneMarker &laneMarker_)
                                      {
                                        return (trgtRghtLnMrkID == laneMarker_.id);
                                      });
    if (it_laneMarker != RM_RoadModel.lane_marker_list.end())
    {
      LaneBoundary trgtLnBndry;
      if (!it_laneMarker->front_lane_boundary_list.empty())
      {
        trgtLnBndry = it_laneMarker->front_lane_boundary_list.front();
        LineGeometry trgtLineGeo = trgtLnBndry.line_geometry;
        float latDist = trgtLineGeo.c0;
        DLOG(INFO) << "@@latDistToRight = " << latDist << std::endl;
        if (latDist >= ADC::common::k_BA_RghtApprLnBndryLowThr && isApprRightLnBndry == false)
        {
          isApprRightLnBndry = true;
        }
        else if (latDist < ADC::common::k_BA_RghtApprLnBndryHighThr && isApprRightLnBndry == true)
        {
          isApprRightLnBndry = false;
        }
        rightApprFlg = isApprRightLnBndry;
      }
      else
      {
        return;
      }  
    } 
    
  }
  else
  {
    return;
  }
}

bool GetTargetCorridor(RoadInfo &RM_RoadModel,
                        uint8_t BA_OptLaneID_LastCycle,
                        Corridor &TargetCorridor)
{
	auto it_Ln = std::find_if(RM_RoadModel.lane_list.begin(), RM_RoadModel.lane_list.end(),
										       [BA_OptLaneID_LastCycle](const Lane &lane_)
										       {
											        return (lane_.start_id == BA_OptLaneID_LastCycle);
										       });
	if (it_Ln != RM_RoadModel.lane_list.end())
	{
		uint8_t trgtFeatureTag = BA::FeatureTag_LineFllw;
		auto it_crrd = std::find_if(it_Ln->corridor_list.begin(), it_Ln->corridor_list.end(),
								               [trgtFeatureTag](const Corridor &corridor_)
											         {
													        return (corridor_.feature_tag == trgtFeatureTag);
											         });
		if (it_crrd != it_Ln->corridor_list.end())
		{
			TargetCorridor = *it_crrd;
      return true;
		}
  }
  return false;
}
