//#pragma once
#pragma once
#include <math.h>

#include "../include/CtApBehaviorAnalysis_/BA_Enum_List.h"
#include "../include/CtApBehaviorAnalysis_/input_BA.h"
// #include "mpc_drv_msgs/msg/line_geometry.hpp"
#include "../commonfunc.h"
#include "../include/CtApBehaviorAnalysis_/interface/rm_road_info.h"
#include "dpc/impl_type_roadinfo.h"
#include <algorithm>

extern const float C_CBJ_d_WheelBase_sg;
extern const float C_CBJ_d_FrtwheelTrd_sg;
extern float P_CBJ_d_CenteringJudgmentDisThrdHigh;
extern float P_CBJ_d_CenteringJudgmentDisThrdLow;
extern float P_CBJ_d_CenteringJudgmentA1ThrdHigh;
extern float P_CBJ_d_CenteringJudgmentAglThrdLow;
extern float P_CBJ_d_CrosJudgmentFrtDisThrdHigh;
extern float P_CBJ_d_CrosJudgmentFrtDisThrdLow;
extern float P_CBJ_d_CrosJudgmentMidDisThrdHigh;
extern float P_CBJ_d_CrosJudgmentMidDisThrdLow;

namespace CBJ
{
  typedef struct CartesianPoint
  {
    CartesianPoint(float x, float y, float vx, float vy, float ax, float ay)
        : X_CtsPnt(x),
          Y_CtsPnt(y),
          Vx_CtsPnt(vx),
          Vy_CtsPnt(vy),
          Ax_CtsPnt(ax),
          Ay_CtsPnt(ay) {}

    CartesianPoint()
        : X_CtsPnt(0),
          Y_CtsPnt(0),
          Vx_CtsPnt(0),
          Vy_CtsPnt(0),
          Ax_CtsPnt(0),
          Ay_CtsPnt(0) {}

    float X_CtsPnt;
    float Y_CtsPnt;
    float Vx_CtsPnt;
    float Vy_CtsPnt;
    float Ax_CtsPnt;
    float Ay_CtsPnt;
  } CartesianPoint;

  typedef struct FrenetPoint
  {
    FrenetPoint(float s, float d, float vs, float vd, float as, float ad)
        : S_FrnPnt(s),
          D_Frnpnt(d),
          Vs_Frnpnt(vs),
          Vd_Frnpnt(vd),
          As_Frnpnt(as),
          Ad_Frnpnt(ad) {}

    FrenetPoint()
        : S_FrnPnt(0),
          D_Frnpnt(0),
          Vs_Frnpnt(0),
          Vd_Frnpnt(0),
          As_Frnpnt(0),
          Ad_Frnpnt(0) {}

    bool operator==(const FrenetPoint &fpnt) const
    {
      if (this->S_FrnPnt == fpnt.S_FrnPnt && this->D_Frnpnt == fpnt.D_Frnpnt &&
          this->Vs_Frnpnt == fpnt.Vs_Frnpnt &&
          this->Vd_Frnpnt == fpnt.Vd_Frnpnt &&
          this->As_Frnpnt == fpnt.As_Frnpnt &&
          this->Ad_Frnpnt == fpnt.Ad_Frnpnt)
      {
        return true;
      }
      else
      {
        return false;
      }
    };

    float S_FrnPnt;
    float D_Frnpnt;
    float Vs_Frnpnt;
    float Vd_Frnpnt;
    float As_Frnpnt;
    float Ad_Frnpnt;
  } FrenetPoint;

  typedef struct LineFunctionParameters
  {
    LineFunctionParameters(float a0, float a1, float a2, float a3,
                           float startrange, float endrange)
        : A0(a0),
          A1(a1),
          A2(a2),
          A3(a3),
          start_range(startrange),
          end_range(endrange) {}

    LineFunctionParameters()
        : A0(0), A1(0), A2(0), A3(0), start_range(0), end_range(0) {}

    float A0;
    float A1;
    float A2;
    float A3;
    float start_range;
    float end_range;
  } LineFunctionParameters;

  typedef enum BA::BA_ActionTyp behavior;
  typedef uint8_t _start_id_type;
  typedef uint8_t _end_id_type;

} // namespace CBJ

/**
 *Gets the target corridor parameters according to optimal corridor ID.
 *@param  RM_RoadModel the road model used to find the target lane according to
 *the start id and end id.
 *@param  BA_OptCorridorID_LastCycle last cycle's optimal corridor used to
 *controll the lateral motion.
 *@return A Boolean quantity indicating whether the vehicle is centered.
 */
bool CenteringConditionJudgment(RoadInfo &RM_RoadModel,
                                uint32_t BA_OptCorridorID,
                                const uint8_t &BA_OptLnID);

bool LaneChangeCancelAvailableJudgement(RoadInfo &RM_RoadModel,
                                        BA::BA_ActionTyp behavior_current,
                                        uint8_t CurrentlaneId,
                                        bool VehInLnCenter,
                                        bool &isLCLnIDNotChg);

/**
 *Gets the target corridor parameters according to optimal corridor ID.
 *@param  RM_RoadModel the road model used to find the target lane according to
 *the start id and end id.
 *@param  behavior_last_cycle last cycle's optimal behavior selected to
 *controll the vehicle.
 *@param  target_lane_start_id the start id used to determine the targer lane.
 *@param  target_lane_end_id the end id used to determine the targer lane.
 *@return A boolean quantity indicating whether the vehicle can cancle the lane
 *change behavior or not.
 */
bool LaneChangeCancelAvailableJudgement(
    RoadInfo &RM_RoadModel, CBJ::behavior behavior_current,
    CBJ::_start_id_type target_lane_start_id,
    CBJ::_end_id_type target_lane_end_id);

/**
 *Gets the target corridor parameters according to optimal corridor ID.
 *@param  RM_RoadModel the road model used to find the target lane according to
 *the start id and end id.
 *@param  target_lane_start_id the start id used to determine the targer lane.
 *@param  target_lane_end_id the end id used to determine the targer lane.
 *@param  BA_OptCorridorID_LastCycle last cycle's optimal corridor used to
 *controll the lateral motion.
 *@param  BA_OptAction last cycle's optimal behavior selected to
 *controll the vehicle.
 *@param  BA_VehInLnCenter  Reference variable : a Boolean quantity indicating
 *whether the vehicle is centered.
 *@param  BA_LaneChangeAbolitionAvlbl  Reference variable : a boolean quantity
 *indicating whether the vehicle can cancle the lane change behavior or not.
 *@param  BA_ActionProc Reference variable : Current vehicle behavior after
 *judgement
 */
bool CurrentBehaviorJudgement(RoadInfo &RM_RoadModel,
                              CBJ::_start_id_type target_lane_start_id,
                              CBJ::_end_id_type target_lane_end_id,
                              uint32_t BA_OptCorridorID_LastCycle,
                              BA::BA_ActionTyp BA_OptAction,
                              bool &BA_VehInLnCenter,
                              bool &BA_LaneChangeAbolitionAvlbl,
                              CBJ::behavior &BA_ActionProc,
                              bool &BA_ISVehCls2LnCntr,
                              bool &BA_ISLCLnIDnotChg,
                              uint8_t &BA_OptLaneID);

/**
 *Gets the target corridor parameters according to optimal corridor ID.
 *@param  RM_RoadModel the road model used to find the target lane according to
 *the start id and end id.
 *@param  BA_OptCorridorID_LastCycle last cycle's optimal corridor used to
 *controll the lateral motion.
 *@param TargertCorridor the target corridor parameters according to optimal
 *corridor ID.
 *@return Return true when the target corridor parameters are found, otherwise
 *it return false.
 */
bool GetTargertCorridor(RoadInfo &RM_RoadModel,
                        uint32_t BA_OptCorridorID_LastCycle,
                        Corridor &TargertCorridor);

/**
 *Gets the target corridor parameters according to optimal corridor ID.
 *@param  RM_RoadModel the road model used to find the target lane according to
 *the start id and end id.
 *@param  BA_OptCorridorID_LastCycle last cycle's optimal corridor used to
 *controll the lateral motion.
 *@param TgtCrdCtrLn_Param the target corridor parameters according to optimal
 *corridor ID.
 *@return Return true when the target corridor Lane Geometry parameters are
 *found, otherwise it return false.
 */
bool GetTargertCorridorLaneGeometry(
    RoadInfo &RM_RoadModel, uint32_t BA_OptCorridorID_LastCycle,
    LineGeometry &TgtCrdCtrLn_Param);

/**
 *Gets the left and right lane marker of the target lane.
 *@param  RM_RoadModel the road model used to find the target lane according to
 *the start id and end id.
 *@param  target_lane_start_id the start id used to determine the targer lane.
 *@param  target_lane_end_id the end id used to determine the targer lane.
 *@param  TgtLnLftLnMrk_LGParam left lane marker parameters of the target lane.
 *@param  TgtLnRgtLnMrk_LGParam right lane marker parameters of the target lane
 *@return Return true when the target left and right lane marker parameters are
 *found, otherwise it return false.
 */
/* bool GetTargetLaneLeftAndRightMarker(
    RoadInfo &RM_RoadModel,
    CBJ::_start_id_type target_lane_start_id,
    CBJ::_end_id_type target_lane_end_id,
    LineGeometry &TgtLnLftLnMrk_LGParam,
    LineGeometry &TgtLnRgtLnMrk_LGParam); */

/**
 *Match the interface of lane line parameters.
 *@param  LnGmt_Param Interface of lane line parameters provided by RM module.
 *@param  Ln_Param Interface of lane line parameters used by CBJ
 *module(y=A0+A1*x+A2/2*x^2+A3/6*x^3).
 *@return Void.
 */
void LineInterfaceMatching(const LineGeometry &LnGmt_Param,
                           CBJ::LineFunctionParameters &Ln_Param);
/**
 *Judge whether the Current vehicle is centered.
 *@param  Ln_Param center line coefficients of cubic polynomial equation
 *(physical meaning).
 *@return A Boolean quantity indicating whether the vehicle is centered.
 */
bool CenteringConditionJudgment(CBJ::LineFunctionParameters Ln_Param);

/**
 *Judge whether ego vehicle can cancle the lane change behavior.
 *@param  behavior_last_cycle last cycle's behavior.
 *@param  LftLn_Param Target lane's left lane marker.
 *@param  RgtLn_Param Target lane's Right lane marker.
 *@return A boolean quantity indicating whether the vehicle can cancle the lane
 *change behavior or not.
 */
bool LaneChangeCancelAvailableJudgement(
    CBJ::behavior behavior_last_cycle, CBJ::LineFunctionParameters LftLn_Param,
    CBJ::LineFunctionParameters RgtLn_Param);

/**
*Current vehicle behavior judgement,After the vehicle performs lane change
behavior, when the vehicle completes lane change (the vehicle centering
condition is judged as true) or when the vehicle completes lane change return
behavior (the vehicle centering condition is judged as true), the current
vehicle behavior needs to be treated as lanefollow behavior, which is not
handled in other cases.
*@param  behavior_last_cycle last cycle's behavior.
*@param  BA_VehInLnCenter  Reference variable : a Boolean quantity indicating
whether the vehicle is centered.
*@param  BA_LaneChangeAbolitionAvlbl  Reference variable : a boolean quantity
indicating whether the vehicle can cancle the lane change behavior or not.
*@return Current vehicle behavior after judgement.
*/
BA::BA_ActionTyp CurrentBehaviorProcess(
    const BA::BA_ActionTyp behavior_last_cycle, bool BA_VehInLnCenter,
    bool BA_LaneChangeAbolitionAvlbl);

/**
 *Calculate the y value corresponding to the x value according to the cubic
 *polynomial of the lane marker(y=A0+A1*x+A2/2*x^2+A3/6*x^3).
 *@param  Ln_Param the cubic polynomial's coefficient of the lane marker
 *(physical meaning).
 *@return The y value calculated by the cubic polynomial fuction.
 */
float YCtsPntLineFuction(CBJ::LineFunctionParameters Ln_Param,
                         const float X_CtsPnt);

/**
 *Using interpolation method to obtain the approximate length of the reference
 *line.
 *@param  Ln_Param the cubic polynomial's coefficient of the lane marker
 *(physical meaning).
 *@param  nearest_point_x the X coordinate value of the end point.
 *@param  interpolation_point_num the points number used for interpolation.
 *@return The approximate length of the reference line.
 */
float ReferenceLineDistance(CBJ::LineFunctionParameters Ln_Param,
                            float nearest_point_x, int interpolation_point_num);

/**
 *Obtained the distance from the point to the reference line by approximate
 *method.
 *@param  Ln_Param the cubic polynomial's coefficient of the lane marker
 *(physical meaning).
 *@param  X_CtsPnt the X coordinate value of the point.
 *@param  Y_CtsPnt the Y coordinate value of the point.
 *@param  D_Frnpnt the distance from the point to the reference line.
 *@return a boolean used to indicate whether the calculating is valid.
 **/
bool FrenetLateralDistanceConversion(CBJ::LineFunctionParameters Ln_Param,
                                     float X_CtsPnt, float Y_CtsPnt,
                                     float &D_Frnpnt);

/**
 *The numerical values of coordinate points are transformed
 *from Cartesian coordinate system to frenet coordinate system by approximate
 *method.
 *@param  Ln_Param the cubic polynomial's coefficient of the lane marker
 *(physical meaning).
 *@param  Cts_Pnt Parameters of points represented by Cartesian coordinate
 *system.
 *@param  FntPnt Parameters of points represented by frenet coordinate system.
 *@return a boolean used to indicate whether the calculating is valid.
 **/
bool PointCoordinateSystemConversion(CBJ::LineFunctionParameters Ln_Param,
                                     CBJ::CartesianPoint Cts_Pnt,
                                     CBJ::FrenetPoint &FntPnt);

/**
 *Get the symbol of the input variable.
 *@param  data the variable that needs to get the symbol.
 *@return the symbol of the variable.
 **/
template <class Type>
inline Type sign(Type data)
{
  Type temp = (data >= 0) ? (1) : (-1);
  return temp;
}

float LaneChangeCompletionPercentage(RoadInfo &RM_RoadModel,
                                     uint32_t BA_OptCorridorID_LastCycle,
                                     BA::BA_ActionTyp BA_OptAction/* ,
                                     bool BA_LaneChangeAbolitionAvlbl,
                                     bool BA_VehInLnCenter */);

bool isVehClsToLnCntr(RoadInfo &RM_RoadModel,
                      uint32_t BA_OptCorridorID,
                      const uint8_t &BA_OptLnID); // the flag is to switch lane change action to lane follow action when the vehicle is close to lane center

void isVehApprToLLnBndry(RoadInfo &RM_RoadModel, bool &leftApprFlg); // the flag is used to denote whether vehicle is approaching left lane boundary, which will be applied to assist lane change abolition availability judgement

void isVehApprToRLnBndry(RoadInfo &RM_RoadModel, bool &rightApprFlg); // the flag is used to denote whether vehicle is approaching right lane boundary, which will be applied to assist lane change abolition availability judgement

bool GetTargetCorridor(RoadInfo &RM_RoadModel,
                        uint8_t BA_OptLaneID_LastCycle,
                        Corridor &TargertCorridor); // 20220528: select the corridor whose feature tag is line_follow as target corridor 
