#include <memory>
#include <vector>
#include <array>
#include <algorithm>

#include "BehaviorNodesAdd.h"
#include "glog/logging.h"

void BehaviorNodesAdd::Init(vector<BA::behaviorNodeGlobal> *BA_BehaviorNodeGlobalVector,
                            vector<PD::BehaviorNodeLocal> &PD_BehaviorNodeLocalVector_,
                            vector<RoadInfo> *RoadModel_,
                            int &BA_StepNum,
                            BA_ActionTyp BA_OptAction_LastCycle,
                            uint32_t BA_OptCorridorID_LastCycle,
                            uint32_t BA_OptTargetObjID_LastCycle,
                            BA::Input_BA Input_BA,
                            vector<BA::BA_ActionTyp> BA_LatAvlblBehavior)
{
    //  BA_BehaviorNodeGlobalVector_=BA_BehaviorNodeGlobalVector;
    BA_StepNum_ = BA_StepNum;
    BA_OptAction_LastCycle_ = BA_OptAction_LastCycle;
    BA_OptCorridorID_LastCycle_ = BA_OptCorridorID_LastCycle;
    BA_OptTargetObjID_LastCycle_ = BA_OptTargetObjID_LastCycle;
    BA_OptLCPCls_LastCycle_ = Input_BA.BA_LCPCls_LastCycle;
    BA_OptLCCls_LastCycle_ = Input_BA.BA_LCCls_LastCycle;
    ReminderObjLst_.clear();
    Process(BA_BehaviorNodeGlobalVector, PD_BehaviorNodeLocalVector_, RoadModel_, BA_StepNum, Input_BA, BA_LatAvlblBehavior);
}

void BehaviorNodesAdd::Process(vector<BA::behaviorNodeGlobal> *BA_BehaviorNodeGlobalVector_,
                               vector<PD::BehaviorNodeLocal> &PD_BehaviorNodeLocalVector_,
                               vector<RoadInfo> *RoadModel_,
                               int &BA_StepNum,
                               BA::Input_BA Input_BA,
                               vector<BA::BA_ActionTyp> BA_LatAvlblBehavior)
{
    if (FatherNodeJudge())
    {
        FatherNodeGen(BA_BehaviorNodeGlobalVector_);
    }

    if (CycleNodeJudge())
    {
        CycleNodeGen(BA_BehaviorNodeGlobalVector_,
                     PD_BehaviorNodeLocalVector_,
                     RoadModel_,
                     Input_BA,
                     BA_LatAvlblBehavior);
    }

    CycleCounter(BA_StepNum);
}

void BehaviorNodesAdd::CycleCounter(int &BA_StepNum)
{
    BA_StepNum = BA_StepNum + 1;
}

bool BehaviorNodesAdd::FatherNodeJudge()
{
    if (BA_StepNum_ == -1)
    {
        return true;
    }
    else
    {
        return false;
    }
}

void BehaviorNodesAdd::FatherNodeGen(vector<BA::behaviorNodeGlobal> *BA_BehaviorNodeGlobalVector_)
{
    BA_BehaviorNodeGlobalVector_->clear();
    //    BA_BehaviorNodeGlobalVector->push_back(S)
    ///    BA_BehaviorNodeGlobalVector->end()->m_TotalCost
    //    (*BA_BehaviorNodeGlobalVector)[2].m_TotalCost

    BA::behaviorNodeGlobal FatherNode_;

    FatherNode_.m_NodeID.m_X = uint8_t(0);
    FatherNode_.m_NodeID.m_Y = uint8_t(0); // 2021/12/03: change from 1 to 0 temperarily to match rm definition
    FatherNode_.m_TotalCost = 0;
    FatherNode_.m_ProcessCost = 0;
    FatherNode_.m_BACalcCost = 0;
    FatherNode_.m_DurationTimeArr.clear();
    FatherNode_.m_DurationTimeArr.emplace_back(0);
    FatherNode_.m_ActionTypArr.clear();
    FatherNode_.m_ActionTypArr.emplace_back(BA_OptAction_LastCycle_);
    FatherNode_.m_CorridorIDArr.clear();
    FatherNode_.m_CorridorIDArr.emplace_back(BA_OptCorridorID_LastCycle_);
    FatherNode_.m_TargetObjIDArr.clear();
    FatherNode_.m_TargetObjIDArr.emplace_back(BA_OptTargetObjID_LastCycle_);
    FatherNode_.m_TotalDurationTime = 0;
    FatherNode_.m_IsNodeTerminated = false;
    FatherNode_.m_FirstCorridorID = uint32_t(0); // modified as this assignment should be done when BA_StepNum == 0
    FatherNode_.m_FirstPassageID.m_X = uint8_t(0);
    FatherNode_.m_FirstPassageID.m_Y = uint8_t(1);

    BA_BehaviorNodeGlobalVector_->push_back(FatherNode_);
    if (BA_BehaviorNodeGlobalVector_->size() != 1)
    {
        DLOG(INFO) << "Father Node" << std::endl;
    }
}

bool BehaviorNodesAdd::CycleNodeJudge()
{
    if (BA_StepNum_ != -1)
    {
        return true;
    }
    else
    {
        return false;
    }
}

bool BehaviorNodesAdd::CollideTerminalNodeJudge(PD::BehaviorNodeLocal PD_BehaviorNodeLocal)
{
    if (PD_BehaviorNodeLocal.m_node_clsn_sts == 1)
    {
        //        DLOG(INFO) << "CLSN " << std::endl;
        return true;
    }
    else
    {
        return false;
    }
}

void BehaviorNodesAdd::CycleNodeGen(vector<BA::behaviorNodeGlobal> *BA_BehaviorNodeGlobalVector_,
                                    vector<PD::BehaviorNodeLocal> &PD_BehaviorNodeLocalVector_,
                                    vector<RoadInfo> *RoadModel_,
                                    BA::Input_BA Input_BA,
                                    vector<BA::BA_ActionTyp> BA_LatAvlblBehavior)
{
    // PD new node gen& Cost Update process
    //    unsigned int NodeIndex = 0;
    uint8_t NodeIndex = uint8_t(0);
    for (auto BehaviorNodeLocal : PD_BehaviorNodeLocalVector_)
    {
        //        BehaviorNodeLocal.m_node_clsn_sts = 0; // templogic  for PD Test     PD Logic Loop in 0330
        if (!CollideTerminalNodeJudge(BehaviorNodeLocal))
        {

            BA::behaviorNodeGlobal ParentNode_ = GetParentNode(BA_BehaviorNodeGlobalVector_, BehaviorNodeLocal);
            RoadInfo RoadModelInfo_ = GetParentRMInfo(RoadModel_, BehaviorNodeLocal);
            Passage ParentPassage_ = GetParentPassage(RoadModelInfo_, BehaviorNodeLocal);

            if (GPN_RunNormal == true && GPR_RunNormal == true && GPP_RunNormal == true) // chk data prepare all right
            {
                NodeIndex++;
                // std::cout << "~~~~~NodeIndex: " << int(NodeIndex) << std::endl;
                BA::behaviorNodeGlobal NewNode_ = UpdateNewNode(ParentNode_, ParentPassage_, BehaviorNodeLocal, NodeIndex, Input_BA, BA_LatAvlblBehavior);

                // std::cout << "~~~~~NewNode duration time: " << NewNode_.m_TotalDurationTime << std::endl;
                // JustFor Ob
                // if (NewNode_.m_NodeID.m_X == 1)
                // {
                //     std::cout << "Action_1:" << int(NewNode_.m_ActionTypArr[0]) << std::endl;
                //     std::cout << "Action_2:" << int(NewNode_.m_ActionTypArr[1]) << std::endl;
                //     std::cout << "Cost:" << float(NewNode_.m_TotalCost) << std::endl;
                // }
                // NewNode Push work
                BA_BehaviorNodeGlobalVector_->push_back(NewNode_);
            }
        }
    } // BNG_node_update
    // chk step = 0 ,node normal
    FirstEstNodeChk(BA_BehaviorNodeGlobalVector_, BA_LatAvlblBehavior);
    // clear passagelist
    if (!RoadModel_->empty())
    {
        for (auto iter = RoadModel_->begin(); iter != RoadModel_->end(); ++iter)
        {
            iter->passage_list.clear();
        }
    }
}

BA::behaviorNodeGlobal BehaviorNodesAdd::GetParentNode(vector<BA::behaviorNodeGlobal> *BA_behaviornodeglobalvector_, PD::BehaviorNodeLocal localnode_)
{
    GPN_RunNormal = false;
    BA::behaviorNodeGlobal _ParentNode_;
    if (!BA_behaviornodeglobalvector_->empty())
    {

        for (auto behaviorglobeNode : *BA_behaviornodeglobalvector_)
        {
            if ((behaviorglobeNode.m_NodeID.m_X == localnode_.m_parent_node_id.x) && (behaviorglobeNode.m_NodeID.m_Y == localnode_.m_parent_node_id.y))
            {
                _ParentNode_ = behaviorglobeNode;
                GPN_RunNormal = true;
                //        std::cout << ": " << NewNode_.m_TotalDurationTime << std::endl
                break;
            }
        }
        if (!GPN_RunNormal)
        {
            DLOG(INFO) << "Find ParentNode failed" << std::endl;
        }
    }
    else
    {
        DLOG(INFO) << "ParentNode_list is empty" << std::endl;
    }
    return _ParentNode_;
}

Passage BehaviorNodesAdd::GetParentPassage(RoadInfo RoadModelInfo_, PD::BehaviorNodeLocal localnode_)
{
    GPP_RunNormal = false;
    Passage _ParentPassage_;
    if (!RoadModelInfo_.passage_list.empty())
    {

        for (auto Passage_ : RoadModelInfo_.passage_list)
        {
            if ((Passage_.passage_id.x == localnode_.m_node_id.x) && (Passage_.passage_id.y == localnode_.m_node_id.y))
            {
                _ParentPassage_ = Passage_;
                GPP_RunNormal = true;
                break;
            }
        }

        if (!GPP_RunNormal)
        {
            DLOG(INFO) << "Find ParentPassage failed" << std::endl;
        }
    }
    else
    {
        DLOG(INFO) << "ParentPassage_list is empty" << std::endl;
    }
    return _ParentPassage_;
}

BA::behaviorNodeGlobal BehaviorNodesAdd::UpdateNewNode(BA::behaviorNodeGlobal ParentNode_,
                                                       Passage ParentPassage_,
                                                       PD::BehaviorNodeLocal BehaviorNodeLocal,
                                                       uint8_t NodeIndex,
                                                       BA::Input_BA Input_BA,
                                                       vector<BA::BA_ActionTyp> BA_LatAvlblBehavior)
{
    BA::behaviorNodeGlobal _NewNode_;

    _NewNode_.m_NodeID.m_X = ParentNode_.nodeID().m_X + uint8_t(1);
    _NewNode_.m_NodeID.m_Y = NodeIndex;

    _NewNode_.m_DurationTimeArr = ParentNode_.m_DurationTimeArr;            // inherit
    _NewNode_.m_DurationTimeArr.emplace_back(ParentPassage_.duration_time); // durtime         use pd or passage????

    _NewNode_.m_ActionTypArr = ParentNode_.m_ActionTypArr; // inherit
    _NewNode_.m_ActionTypArr.emplace_back(BA::BA_ActionTyp(ParentPassage_.action_type));

    _NewNode_.m_CorridorIDArr = ParentNode_.m_CorridorIDArr; // inherit
    _NewNode_.m_CorridorIDArr.emplace_back(ParentPassage_.corridor_id);

    _NewNode_.m_TargetObjIDArr = ParentNode_.m_TargetObjIDArr; // inherit
    _NewNode_.m_TargetObjIDArr.emplace_back(ParentPassage_.longt_info.target_object_id);

    _NewNode_.m_HVInfoGlobal = ParentNode_.m_HVInfoGlobal; // inherit
    BANodeGlobalHVpushFcn(_NewNode_, BehaviorNodeLocal);   // update HVinfo

    if (this->BA_StepNum_ == 0) // 20211203: FirstCorridorID should be assigned when BA_StepNum is equal to 1 instead of 0
    {
        _NewNode_.m_FirstCorridorID = ParentPassage_.corridor_id;
        _NewNode_.m_FirstPassageID.m_X = ParentPassage_.passage_id.x;
        _NewNode_.m_FirstPassageID.m_Y = ParentPassage_.passage_id.y;
    }
    else
    {
        _NewNode_.m_FirstCorridorID = ParentNode_.m_FirstCorridorID;
        _NewNode_.m_FirstPassageID = ParentNode_.m_FirstPassageID;
    }

    _NewNode_.m_IsNodeTerminated = ParentNode_.m_IsNodeTerminated;
    _NewNode_.m_TotalDurationTime = ParentNode_.m_TotalDurationTime + BehaviorNodeLocal.m_duration_time;
    //    std::cout << "++++parent total duration time: " << ParentNode_.m_TotalDurationTime << std::endl;
    //    std::cout << "++++single duration time: " << BehaviorNodeLocal.m_duration_time << std::endl;
    //    std::cout << "++++parent node id x: " << int(ParentNode_.nodeID().m_X) << std::endl;

    // spd limit ref
    if (this->BA_StepNum_ == 0) // 20220420
    {
        if (BehaviorNodeLocal.spd_limit_type == BA::SpdLmtTyp_Sign1st) // spd limit =1 curve limit = 1;
        {
            _NewNode_.m_SpdLmtFlg = true;
            _NewNode_.m_SpdLmtTyp = BA::SpdLmtTyp_Sign1st;
            _NewNode_.m_SpdLmtID = ParentPassage_.longt_info.sign_spd_limit_id_01;
            _NewNode_.m_SpdLmtVal = ParentPassage_.longt_info.sign_spd_limit_val_01;
            _NewNode_.m_SpdLmtDst = ParentPassage_.longt_info.sign_spd_limit_dist_01;
        }
        else if (BehaviorNodeLocal.spd_limit_type == BA::SpdLmtTyp_Sign2nd)
        {
            _NewNode_.m_SpdLmtFlg = true;
            _NewNode_.m_SpdLmtTyp = BA::SpdLmtTyp_Sign2nd;
            _NewNode_.m_SpdLmtID = ParentPassage_.longt_info.sign_spd_limit_id_02;
            _NewNode_.m_SpdLmtVal = ParentPassage_.longt_info.sign_spd_limit_val_02;
            _NewNode_.m_SpdLmtDst = ParentPassage_.longt_info.sign_spd_limit_dist_02;
        }
        else if (BehaviorNodeLocal.spd_limit_type == BA::SpdLmtTyp_CurvRd)
        {
            _NewNode_.m_SpdLmtFlg = true;
            _NewNode_.m_SpdLmtTyp = BA::SpdLmtTyp_CurvRd;
            _NewNode_.m_SpdLmtID = 11;
            _NewNode_.m_SpdLmtVal = ParentPassage_.longt_info.curv_spd_limit_val;
            _NewNode_.m_SpdLmtDst = ParentPassage_.longt_info.curv_spd_limit_dist;
        }
        else // spd limit =0 curve limit = 0;
        {
            _NewNode_.m_SpdLmtFlg = false;
            _NewNode_.m_SpdLmtTyp = BA::SpdLmtTyp_NotAvlbl;
            _NewNode_.m_SpdLmtID = 0;
            _NewNode_.m_SpdLmtVal = 150;
            _NewNode_.m_SpdLmtDst = 9999;
        }
    }
    else
    {
        _NewNode_.m_SpdLmtFlg = ParentNode_.m_SpdLmtFlg;
        _NewNode_.m_SpdLmtTyp = ParentNode_.m_SpdLmtTyp;
        _NewNode_.m_SpdLmtID = ParentNode_.m_SpdLmtID;
        _NewNode_.m_SpdLmtVal = ParentNode_.m_SpdLmtVal;
        _NewNode_.m_SpdLmtDst = ParentNode_.m_SpdLmtDst;
    }
    DLOG(INFO) << "m_SpdLmtFlg = " << int(_NewNode_.m_SpdLmtFlg) << std::endl;
    DLOG(INFO) << "m_SpdLmtID = " << int(_NewNode_.m_SpdLmtID) << std::endl;
    DLOG(INFO) << "m_SpdLmtDist = " << int(_NewNode_.m_SpdLmtDst) << std::endl;
    //  cost ref
    CostProcess(ParentNode_, ParentPassage_, _NewNode_, BehaviorNodeLocal, Input_BA, BA_LatAvlblBehavior);

    return _NewNode_;
}

void BehaviorNodesAdd::FirstEstNodeChk(vector<BA::behaviorNodeGlobal> *BA_BehaviorNodeGlobalVector_,
                                       vector<BA::BA_ActionTyp> BA_LatAvlblBehavior)
{
    if (BA_StepNum_ == 0) // only work at 0 step
    {
        BNGNode_Unusual = false; // inital
        if (BA_BehaviorNodeGlobalVector_->size() > 1 && BA_LatAvlblBehavior.empty() == false)
        {
            vector<BA::BA_ActionTyp>::iterator result = find(BA_LatAvlblBehavior.begin() + 1, BA_LatAvlblBehavior.end(), BA_ActionTyp_LaneFollow);
            if (result != BA_LatAvlblBehavior.end())
            {
                // for (auto it_i = BA_BehaviorNodeGlobalVector_->begin();
                //      it_i != BA_BehaviorNodeGlobalVector_->end(); it_i++)
                // {
                //         if(it_i->m_NodeID.m_X == uint8_t(1) && it_i->m_ActionTypArr.back() == BA_ActionTyp_LaneFollow && it_i->m_IsNodeTerminated == 0)
                // }

                // auto it = find_if(BA_BehaviorNodeGlobalVector_->begin() + 1, BA_BehaviorNodeGlobalVector_->end(),
                //                   [](BA::behaviorNodeGlobal &behaviorNodeGlobal)
                //                   {
                //                       bool res1 = behaviorNodeGlobal.m_NodeID.m_X == uint8_t(1);
                //                       bool res2 = behaviorNodeGlobal.m_ActionTypArr.back() == BA_ActionTyp_LaneFollow;
                //                       if (res1 && res2)
                //                       {
                //                           return true;
                //                       }
                //                       else
                //                       {
                //                           return false;
                //                       }
                //                   })
                BNGNode_Unusual = false;
            }
            else if (BA_BehaviorNodeGlobalVector_->front().m_ActionTypArr.front() == BA_ActionTyp_LeftLaneChange || BA_BehaviorNodeGlobalVector_->front().m_ActionTypArr.front() == BA_ActionTyp_RightLaneChange)
            {
                BNGNode_Unusual = false;
            }
            else
            {
                BNGNode_Unusual = true;
            }
        }
        else
        {
            BNGNode_Unusual = true;
        }
    }
}

void BehaviorNodesAdd::HVInfoAdd(PD::HVInfo BehaviorNodeLocalHVInfo, BA::behaviorNodeGlobal &NewNode)
{
    // BA::HV_Info HVTemp;
    NewNode.m_HVInfoGlobal.m_DistanceAlongS.emplace_back(BehaviorNodeLocalHVInfo.m_distance_alongs);
    NewNode.m_HVInfoGlobal.m_LaneLocation.emplace_back(BehaviorNodeLocalHVInfo.m_lane_location);
    NewNode.m_HVInfoGlobal.m_Velocity.emplace_back(BehaviorNodeLocalHVInfo.m_velocity);

    // std::cout << "++++BNLHV Dist: " << BehaviorNodeLocalHVInfo.m_DistanceAlongS << std::endl;
    // std::cout << "++++HVTemp Dist: " << HVTemp.m_DistanceAlongS << std::endl;
    // std::cout << "++++BNLHV LaneLocation: " << BehaviorNodeLocalHVInfo.m_LaneLocation << std::endl;
    // std::cout << "++++HVTemp LaneLocation: " << HVTemp.m_LaneLocation << std::endl;
    // std::cout << "++++BNLHV Velocity: " << BehaviorNodeLocalHVInfo.m_Velocity << std::endl;
    // std::cout << "++++HVTemp Velocity: " << HVTemp.m_Velocity << std::endl;
    // std::cout << "DONE " << std::endl;
}

void BehaviorNodesAdd::BANodeGlobalHVpushFcn(BA::behaviorNodeGlobal &NewNode_, PD::BehaviorNodeLocal &BehaviorNodeLocal)
{
    if (!BehaviorNodeLocal.m_hvinfo.empty())
    {
        auto ct = int(BehaviorNodeLocal.m_hvinfo.size());
        for (int i = 0; i < ct; i++)
        {
            // NewNode_.m_HVInfoGlobal.emplace_back(HVInfoTransf(BehaviorNodeLocal.m_hvinfo[i]));
            HVInfoAdd(BehaviorNodeLocal.m_hvinfo[i], NewNode_);
        }
    }
    else
    {
        cout << "BNL HVInfo is empty" << endl;
    }
}

RoadInfo BehaviorNodesAdd::GetParentRMInfo(vector<RoadInfo> *RoadModel_, PD::BehaviorNodeLocal localnode_)
{
    GPR_RunNormal = false;
    RoadInfo _RoadModelInfo_;
    if (!RoadModel_->empty())
    {
        for (auto roadmodelsingle : *RoadModel_)
        {
            if ((roadmodelsingle.road_id.x == localnode_.m_parent_node_id.x) && (roadmodelsingle.road_id.y == localnode_.m_parent_node_id.y))
            {
                _RoadModelInfo_ = roadmodelsingle;
                GPR_RunNormal = true;
                break;
            }
        }
        // std::cout << "BNLIndex_x: " << int(localnode_.m_NodeID.x) << std::endl;
        // std::cout << "BNLIndex_y: " << int(localnode_.m_NodeID.y) << std::endl;
        // std::cout << "BNLParentIndex_x: " << int(localnode_.m_parent_node_id.x) << std::endl;
        // std::cout << "BNLParentIndex_y: " << int(localnode_.m_parent_node_id.y) << std::endl;
        // std::cout << "RMIIndex_x: " << int(_RoadModelInfo_.road_id.x) << std::endl;
        // std::cout << "RMIIndex_y: " << int(_RoadModelInfo_.road_id.y) << std::endl;
        if (!GPR_RunNormal)
        {
            DLOG(INFO) << "Find ParentRMInfo failed" << std::endl;
        }
    }
    else
    {
        DLOG(INFO) << "Find ParentRMInfo failed" << std::endl;
    }
    return _RoadModelInfo_;
}

void BehaviorNodesAdd::CostProcess(BA::behaviorNodeGlobal ParentNode_,
                                   Passage ParentPassage_,
                                   BA::behaviorNodeGlobal &_NewNode_,
                                   PD::BehaviorNodeLocal BehaviorNodeLocal,
                                   BA::Input_BA Input_BA,
                                   vector<BA::BA_ActionTyp> BA_LatAvlblBehavior)
{
    BAChgCostEstimate(ParentNode_, ParentPassage_, Input_BA, BA_LatAvlblBehavior, _NewNode_);
    CostGainCalc(_NewNode_, BehaviorNodeLocal);
    PDStepCostEstimate(BehaviorNodeLocal, ParentPassage_); // need repaired
    LayerCostGen();
    _NewNode_.m_ProcessCost = BAPDCost_;
    _NewNode_.m_BACalcCost = BAChgCost_ + BAALCCourageCost_;
    _NewNode_.m_TotalCost = ParentNode_.m_TotalCost + LayerCost_;
    _NewNode_.m_TargetObjIDArr.push_back(ReminderObjID_);

    if (ParentPassage_.action_type == BA_ActionTyp::BA_ActionTyp_LaneFollow)
    {
        DLOG(INFO) << "LF_LayerCost is: " << LayerCost_ << std::endl;
        DLOG(INFO) << "LF_BACalcCost is: " << _NewNode_.m_BACalcCost << std::endl;
        DLOG(INFO) << "LF_TotalCost is: " << _NewNode_.m_TotalCost << std::endl;
    }
    else if ((ParentPassage_.action_type == BA_ActionTyp::BA_ActionTyp_LeftLaneChange) || (ParentPassage_.action_type == BA_ActionTyp::BA_ActionTyp_RightLaneChange))
    {
        DLOG(INFO) << "LC_LayerCost is: " << LayerCost_ << std::endl;
        DLOG(INFO) << "LC_BACalcCost is: " << _NewNode_.m_BACalcCost << std::endl;
        DLOG(INFO) << "LC_TotalCost is: " << _NewNode_.m_TotalCost << std::endl;
    }
    else if ((ParentPassage_.action_type == BA_ActionTyp::BA_ActionTyp_LeftLaneChangePrepare) || (ParentPassage_.action_type == BA_ActionTyp::BA_ActionTyp_RightLaneChangePrepare))
    {
        DLOG(INFO) << "LCP_LayerCost is: " << LayerCost_ << std::endl;
        DLOG(INFO) << "LCP_BACalcCost is: " << _NewNode_.m_BACalcCost << std::endl;
        DLOG(INFO) << "LCP_TotalCost is: " << _NewNode_.m_TotalCost << std::endl;
    }
    else
    {
        DLOG(INFO) << "STANDBY " << std::endl;
    }
}

void BehaviorNodesAdd::BAChgCostEstimate(BA::behaviorNodeGlobal ParentNode_,
                                         Passage ParentPassage_,
                                         BA::Input_BA Input_BA,
                                         vector<BA::BA_ActionTyp> BA_LatAvlblBehavior,
                                         BA::behaviorNodeGlobal &NewNode_)
{
    BAChgCost_ = 0;        // initialize for BehaviorChgCost;
    BAALCCourageCost_ = 0; // initialize for couragecost;
    BehaviorChgCourageCost(ParentNode_, ParentPassage_, Input_BA);
    BehaviorChgCost_S1(ParentNode_, ParentPassage_);
    BehaviorChgCost_S2(ParentNode_, ParentPassage_);
    BehaviorChgCost_S3(ParentNode_, ParentPassage_, Input_BA, BA_LatAvlblBehavior);
    BehaviorChgCost_S4(ParentNode_, ParentPassage_, Input_BA, BA_LatAvlblBehavior);

    BehaviorChgCostS2_S4ALLFail();
    /* if(ParentNode_.m_ActionTypArr[0] == BA_ActionTyp_LaneFollow && (ParentPassage_.action_type == BA_ActionTyp_LeftLaneChange || ParentPassage_.action_type == BA_ActionTyp_RightLaneChange))
    {
        DLOG(INFO) << "   S2 is : = " << int(BAChgWork_S2_) << std::endl;
        DLOG(INFO) << "   S3 is : = " << int(BAChgWork_S3_) << std::endl;
        DLOG(INFO) << "   S4 is : = " << int(BAChgWork_S4_) << std::endl;
        DLOG(INFO) << "   ALLFail state: = " << int(BAS2_S4Allfail_) << std::endl;
    }    */
    BehaviorChgGuideCost(ParentNode_, ParentPassage_, Input_BA, NewNode_);
}

void BehaviorNodesAdd::CostGainCalc(BA::behaviorNodeGlobal &NewNode_, PD::BehaviorNodeLocal BehaviorNodeLocal)
{
    Gain_share_ = 1;
    Gain_decay_ = 1; // initialize

    if (NewNode_.m_TotalDurationTime != 0)
    {
        if (NewNode_.m_TotalDurationTime <= ADC::common::k_BA_TotalDurationTimeThreshold)
        {
            Gain_share_ = BehaviorNodeLocal.m_duration_time / ADC::common::k_BA_TotalDurationTimeThreshold;
        }
        else
        {
            float DurationLawTime = ADC::common::k_BA_TotalDurationTimeThreshold - (NewNode_.m_TotalDurationTime - BehaviorNodeLocal.m_duration_time);
            Gain_share_ = DurationLawTime / ADC::common::k_BA_TotalDurationTimeThreshold;
        }

        float chk_time = NewNode_.m_TotalDurationTime - BehaviorNodeLocal.m_duration_time;
        MathFcn::LinearIntrPltn<float> DecayTb;

        Gain_decay_ = DecayTb.InterPolation(k_BA_BNACostDecayTmTb, k_BA_BNACostDecayGainTb, chk_time);
    }
    else
    {
        Gain_share_ = 1;
        Gain_decay_ = 1;
    }
    DLOG(INFO) << "Gain_share_ = " << float(Gain_share_) << std::endl;
    DLOG(INFO) << "Gain_decay_ = " << float(Gain_decay_) << std::endl;
}

void BehaviorNodesAdd::BehaviorChgCourageCost(BA::behaviorNodeGlobal ParentNode_,
                                              Passage ParentPassage_,
                                              BA::Input_BA Input_BA) // work for stepNum ==0, use for encourage driver lc/lcp action
{
    if (BA_StepNum_ == 0 && ParentNode_.m_ActionTypArr.empty() == 0)
    {
        BA::BA_ActionTyp typ1 = BA_ActionTyp_None;
        if (int(Input_BA.SM_LCAvlblBehavior) == int(LCAvlblBhvr::LCAvlblBhvr_LeftLaneChange_C) || Input_BA.SM_LCAvlblBehavior == LCAvlblBhvr::LCAvlblBhvr_LeftLaneChange_Auto)
        {
            typ1 = BA_ActionTyp_LeftLaneChange;
        }
        else if (Input_BA.SM_LCAvlblBehavior == LCAvlblBhvr::LCAvlblBhvr_RightLaneChange_C || Input_BA.SM_LCAvlblBehavior == LCAvlblBhvr::LCAvlblBhvr_RightLaneChange_Auto)
        {
            typ1 = BA_ActionTyp_RightLaneChange;
        }
        else if (Input_BA.SM_LCPAvlblBehavior == LCPAvlblBhvr::LCPAvlblBhvr_LeftLaneChangePrepare)
        {
            typ1 = BA_ActionTyp_LeftLaneChangePrepare;
        }
        else if (Input_BA.SM_LCPAvlblBehavior == LCPAvlblBhvr::LCPAvlblBhvr_RightLaneChangePrepare)
        {
            typ1 = BA_ActionTyp_RightLaneChangePrepare;
        }
        else
        {
            typ1 = BA_ActionTyp_None;
        }

        if (ParentPassage_.action_type != typ1)
        {
            BAALCCourageCost_ = k_BA_BNAALCCourage_A;
        }
    }
    else
    {
        BAALCCourageCost_ = 0;
    }
}

void BehaviorNodesAdd::BehaviorChgCost_S1(BA::behaviorNodeGlobal ParentNode_,
                                          Passage ParentPassage_)
{
    if (ParentNode_.m_ActionTypArr.empty() == 0 && ParentNode_.m_ActionTypArr.size() >= max((BA_StepNum_ - 1), int(0)))
    { // std::cout << " " << ParentNode_.m_ActionTypArr[BA_StepNum_ - 1] << std::endl;
        if (BA_StepNum_ >= 1 && (((ParentNode_.m_ActionTypArr[BA_StepNum_] == BA_ActionTyp_LaneFollow) && (ParentPassage_.action_type == BA_ActionTyp_LaneFollow)) || ((ParentNode_.m_ActionTypArr[BA_StepNum_] == BA_ActionTyp_LeftLaneChange) && (ParentPassage_.action_type == BA_ActionTyp_LaneFollow)) || ((ParentNode_.m_ActionTypArr[BA_StepNum_] == BA_ActionTyp_RightLaneChange) && (ParentPassage_.action_type == BA_ActionTyp_LaneFollow))))
        {
            BAChgCost_ = ADC::common::k_BA_BNACommonChgCost_nutrl;
        }

        else if (BA_StepNum_ >= 1 && (((ParentNode_.m_ActionTypArr[BA_StepNum_] == BA_ActionTyp_LaneFollow) && (ParentPassage_.action_type == BA_ActionTyp_LeftLaneChange)) || ((ParentNode_.m_ActionTypArr[BA_StepNum_] == BA_ActionTyp_LaneFollow) && (ParentPassage_.action_type == BA_ActionTyp_RightLaneChange)) || ((ParentNode_.m_ActionTypArr[BA_StepNum_] == BA_ActionTyp_LeftLaneChangePrepare) && (ParentPassage_.action_type == BA_ActionTyp_LaneFollow)) || ((ParentNode_.m_ActionTypArr[BA_StepNum_] == BA_ActionTyp_RightLaneChangePrepare) && (ParentPassage_.action_type == BA_ActionTyp_LaneFollow))))
        {
            BAChgCost_ = ADC::common::k_BA_BNACommonChgCost_pnshA;
        }

        else if (BA_StepNum_ >= 1 && (((ParentNode_.m_ActionTypArr[BA_StepNum_] == BA_ActionTyp_LaneFollow) && (ParentPassage_.action_type == BA_ActionTyp_LeftLaneChangePrepare)) || ((ParentNode_.m_ActionTypArr[BA_StepNum_] == BA_ActionTyp_LaneFollow) && (ParentPassage_.action_type == BA_ActionTyp_RightLaneChangePrepare))))
        {
            BAChgCost_ = ADC::common::k_BA_BNACommonChgCost_pnshB;
        }

        else if (BA_StepNum_ >= 1 && (((ParentNode_.m_ActionTypArr[BA_StepNum_] == BA_ActionTyp_LeftLaneChange) && (ParentPassage_.action_type == BA_ActionTyp_LeftLaneChange)) || ((ParentNode_.m_ActionTypArr[BA_StepNum_] == BA_ActionTyp_LeftLaneChangePrepare) && (ParentPassage_.action_type == BA_ActionTyp_LeftLaneChangePrepare)) || ((ParentNode_.m_ActionTypArr[BA_StepNum_] == BA_ActionTyp_RightLaneChange) && (ParentPassage_.action_type == BA_ActionTyp_RightLaneChange)) || ((ParentNode_.m_ActionTypArr[BA_StepNum_] == BA_ActionTyp_RightLaneChangePrepare) && (ParentPassage_.action_type == BA_ActionTyp_RightLaneChangePrepare))))
        {
            BAChgCost_ = ADC::common::k_BA_BNACommonChgCost_pnshC;
        }

        else if (BA_StepNum_ >= 1 && (((ParentNode_.m_ActionTypArr[BA_StepNum_] == BA_ActionTyp_LeftLaneChangePrepare) && (ParentPassage_.action_type == BA_ActionTyp_LeftLaneChange)) || ((ParentNode_.m_ActionTypArr[BA_StepNum_] == BA_ActionTyp_RightLaneChangePrepare) && (ParentPassage_.action_type == BA_ActionTyp_RightLaneChange))))
        {
            BAChgCost_ = ADC::common::k_BA_BNACommonChgCost_encrgA;
        }

        else if (BA_StepNum_ >= 1)
        {
            BAChgCost_ = k_BA_BNAChgCostCommon;
        }

        else
        {
            BAChgCost_ = 0;
        }
    }
    else
    {
        DLOG(INFO) << "BAChgCostS1 is Abnormal" << std::endl;
        BAChgCost_ = 0;
    }
}

void BehaviorNodesAdd::BehaviorChgCost_S2(BA::behaviorNodeGlobal ParentNode_,
                                          Passage ParentPassage_)
{
    if (!ParentNode_.m_ActionTypArr.empty())
    {
        if (BA_StepNum_ == 0 && ((ParentNode_.m_ActionTypArr[0] == BA_ActionTyp_LeftLaneChange) || (ParentNode_.m_ActionTypArr[0] == BA_ActionTyp_RightLaneChange)) && ((ParentPassage_.action_type == BA_ActionTyp_LeftLaneChangeAbolition) || (ParentPassage_.action_type == BA_ActionTyp_RightLaneChangeAbolition)))
        {
            BAChgCost_ = BAChgCost_ + ADC::common::k_BA_BNAChgCost_S2;
            BAChgWork_S2_ = true;
        }

        else
        {
            BAChgWork_S2_ = false;
            BAChgCost_ = BAChgCost_ + 0;
        }
    }
}

void BehaviorNodesAdd::BehaviorChgCost_S3(BA::behaviorNodeGlobal ParentNode_,
                                          Passage ParentPassage_,
                                          BA::Input_BA Input_BA,
                                          vector<BA::BA_ActionTyp> BA_LatAvlblBehavior)
{
    BA::BA_ActionTyp typ1 = BA_ActionTyp_LeftLaneChange;
    BA::BA_ActionTyp typ2 = BA_ActionTyp_RightLaneChange;
    BA::BA_ActionTyp typ3 = BA_ActionTyp_LeftLaneChangePrepare;
    BA::BA_ActionTyp typ4 = BA_ActionTyp_RightLaneChangePrepare;
    BA::BA_ActionTyp typ5 = BA_ActionTyp_LaneFollow;
    if (!ParentNode_.m_ActionTypArr.empty())
    {
        if (BA_StepNum_ == 0 && ParentNode_.m_ActionTypArr[0] == BA_ActionTyp_LaneFollow && (((Input_BA.SM_LCAvlblBehavior == LCAvlblBhvr_LeftLaneChange_C) || (Input_BA.SM_LCAvlblBehavior == LCAvlblBhvr_LeftLaneChange_Auto)) && (TargetBehaviorChk(typ1, BA_LatAvlblBehavior))) // for ALC manner judge
            && (ParentPassage_.action_type == BA_ActionTyp_LeftLaneChange))
        {
            BAChgWork_S3_ = true;
            BAChgCost_ = BAChgCost_ + ADC::common::k_BA_BNAChgCost_S3_encrgB;
        } // ALC/iALC leftchg

        else if (BA_StepNum_ == 0 && ParentNode_.m_ActionTypArr[0] == BA_ActionTyp_LaneFollow && (((Input_BA.SM_LCAvlblBehavior == LCAvlblBhvr_RightLaneChange_C) || (Input_BA.SM_LCAvlblBehavior == LCAvlblBhvr_RightLaneChange_Auto)) && (TargetBehaviorChk(typ2, BA_LatAvlblBehavior))) // for ALC manner judge
                 && (ParentPassage_.action_type == BA_ActionTyp_RightLaneChange))
        {
            BAChgWork_S3_ = true;
            BAChgCost_ = BAChgCost_ + ADC::common::k_BA_BNAChgCost_S3_encrgB;
        } // ALC/iALC rightchg

        else if (BA_StepNum_ == 0 && ParentNode_.m_ActionTypArr[0] == BA_ActionTyp_LaneFollow && (((Input_BA.SM_LCAvlblBehavior == LCAvlblBhvr_LeftLaneChange_C) || (Input_BA.SM_LCAvlblBehavior == LCAvlblBhvr_LeftLaneChange_Auto)) && (TargetBehaviorChk(typ3, BA_LatAvlblBehavior))) // for ALC manner judge
                 && (ParentPassage_.action_type == BA_ActionTyp_LeftLaneChangePrepare))
        {
            BAChgWork_S3_ = true;
            BAChgCost_ = BAChgCost_ + ADC::common::k_BA_BNAChgCost_S3_encrgA;
            if ((ParentPassage_.lcp_cls == 1 || ParentPassage_.lcp_cls == 2 || ParentPassage_.lcp_cls == 5 || ParentPassage_.lcp_cls == 6) && (k_BA_BNALCPFrtPnsh == 1))
            {
                BAChgCost_ = BAChgCost_ + k_BA_BNALCPFrtPnshGain;
            } // Tst For Back Space Gen 0407
        }     // iALC leftchgPrepare

        else if (BA_StepNum_ == 0 && ParentNode_.m_ActionTypArr[0] == BA_ActionTyp_LaneFollow && (((Input_BA.SM_LCAvlblBehavior == LCAvlblBhvr_RightLaneChange_C) || (Input_BA.SM_LCAvlblBehavior == LCAvlblBhvr_RightLaneChange_Auto)) && (TargetBehaviorChk(typ4, BA_LatAvlblBehavior))) // for ALC manner judge
                 && (ParentPassage_.action_type == BA_ActionTyp_RightLaneChangePrepare))
        {
            BAChgWork_S3_ = true;
            BAChgCost_ = BAChgCost_ + ADC::common::k_BA_BNAChgCost_S3_encrgA;
            if ((ParentPassage_.lcp_cls == 1 || ParentPassage_.lcp_cls == 2 || ParentPassage_.lcp_cls == 5 || ParentPassage_.lcp_cls == 6) && (k_BA_BNALCPFrtPnsh == 1))
            {
                BAChgCost_ = BAChgCost_ + k_BA_BNALCPFrtPnshGain;
            } // Tst For Back Space Gen 0407
        }     // iALC rightchgPrepare

        else if (BA_StepNum_ == 0 && ParentNode_.m_ActionTypArr[0] == BA_ActionTyp_LaneFollow && ((((Input_BA.SM_LCAvlblBehavior == LCAvlblBhvr_LeftLaneChange_C) || (Input_BA.SM_LCAvlblBehavior == LCAvlblBhvr_LeftLaneChange_Auto)) && (TargetBehaviorChk(typ1, BA_LatAvlblBehavior))) || (((Input_BA.SM_LCAvlblBehavior == LCAvlblBhvr_RightLaneChange_C) || (0) /*(Input_BA.SM_LCAvlblBehavior == LCAvlblBhvr_RightLaneChange_Auto)*/) && (TargetBehaviorChk(typ2, BA_LatAvlblBehavior))) || (((Input_BA.SM_LCAvlblBehavior == LCAvlblBhvr_LeftLaneChange_C) || (0) /*(Input_BA.SM_LCAvlblBehavior == LCAvlblBhvr_LeftLaneChange_Auto)*/) && (TargetBehaviorChk(typ3, BA_LatAvlblBehavior))) || (((Input_BA.SM_LCAvlblBehavior == LCAvlblBhvr_RightLaneChange_C) || (0) /*(Input_BA.SM_LCAvlblBehavior == LCAvlblBhvr_RightLaneChange_Auto)*/) && (TargetBehaviorChk(typ4, BA_LatAvlblBehavior)))) // for ALC manner judge
                 && ParentPassage_.action_type == BA_ActionTyp_LaneFollow)
        {
            BAChgWork_S3_ = true;
            BAChgCost_ = BAChgCost_ + ADC::common::k_BA_BNAChgCost_S3_pnshA;
        } // punish laneflo (above 4 choices in cmd lc)
        else if (BA_StepNum_ == 0 && ParentNode_.m_ActionTypArr[0] == BA_ActionTyp_LaneFollow && (((Input_BA.SM_LCAvlblBehavior == LCAvlblBhvr_LaneChange_Auto)) && (TargetBehaviorChk(typ5, BA_LatAvlblBehavior)) && ParentPassage_.action_type == BA_ActionTyp_LaneFollow))
        {
            BAChgWork_S3_ = true;
            BAChgCost_ = BAChgCost_ + ADC::common::k_BA_BNAChgCost_S3_encrgB;
        } // encrg laneflo in NRP(0311)
        else if ((BA_StepNum_ == 0) && (ParentNode_.m_ActionTypArr[0] == BA_ActionTyp_LeftLaneChange) && ((Input_BA.SM_LCAvlblBehavior == LCAvlblBhvr_LaneChange_Auto) && (TargetBehaviorChk(typ1, BA_LatAvlblBehavior)) && (ParentPassage_.action_type == BA_ActionTyp_LeftLaneChange)))
        {
            BAChgWork_S3_ = true;
            BAChgCost_ = BAChgCost_ + ADC::common::k_BA_BNAChgCost_S3_encrgB;
        } // encrg Left lanechg go on in NRP(0530) LC-LC
        else if ((BA_StepNum_ == 0) && (ParentNode_.m_ActionTypArr[0] == BA_ActionTyp_RightLaneChange) && ((Input_BA.SM_LCAvlblBehavior == LCAvlblBhvr_LaneChange_Auto) && (TargetBehaviorChk(typ2, BA_LatAvlblBehavior)) && (ParentPassage_.action_type == BA_ActionTyp_RightLaneChange)))
        {
            BAChgWork_S3_ = true;
            BAChgCost_ = BAChgCost_ + ADC::common::k_BA_BNAChgCost_S3_encrgB;
        } // encrg Right lanechg go on in NRP(0530) LC-LC
        else if ((BA_StepNum_ == 0) && (ParentNode_.m_ActionTypArr[0] == BA_ActionTyp_LeftLaneChangePrepare) && ((Input_BA.SM_LCAvlblBehavior == LCAvlblBhvr_LaneChange_Auto) && (TargetBehaviorChk(typ1, BA_LatAvlblBehavior)) && (ParentPassage_.action_type == BA_ActionTyp_LeftLaneChange)))
        {
            BAChgWork_S3_ = true;
            BAChgCost_ = BAChgCost_ + ADC::common::k_BA_BNAChgCost_S3_encrgB;
        } // encrg Left lanechg go on in NRP(0530) LCP-LC
        else if ((BA_StepNum_ == 0) && (ParentNode_.m_ActionTypArr[0] == BA_ActionTyp_RightLaneChangePrepare) && ((Input_BA.SM_LCAvlblBehavior == LCAvlblBhvr_LaneChange_Auto) && (TargetBehaviorChk(typ2, BA_LatAvlblBehavior)) && (ParentPassage_.action_type == BA_ActionTyp_RightLaneChange)))
        {
            BAChgWork_S3_ = true;
            BAChgCost_ = BAChgCost_ + ADC::common::k_BA_BNAChgCost_S3_encrgB;
        } // encrg Right lanechg go on in NRP(0530) LCP-LC
        else if ((BA_StepNum_ == 0) && (ParentNode_.m_ActionTypArr[0] == BA_ActionTyp_LeftLaneChangePrepare) && ((Input_BA.SM_LCAvlblBehavior == LCAvlblBhvr_LaneChange_Auto) && (TargetBehaviorChk(typ3, BA_LatAvlblBehavior)) && (ParentPassage_.action_type == BA_ActionTyp_LeftLaneChangePrepare)))
        {
            BAChgWork_S3_ = true;
            BAChgCost_ = BAChgCost_ + ADC::common::k_BA_BNAChgCost_S3_encrgA;
        } // encrg LITTLE Left lanechg go on in NRP(0530) LCP-LCP
        else if ((BA_StepNum_ == 0) && (ParentNode_.m_ActionTypArr[0] == BA_ActionTyp_RightLaneChangePrepare) && ((Input_BA.SM_LCAvlblBehavior == LCAvlblBhvr_LaneChange_Auto) && (TargetBehaviorChk(typ4, BA_LatAvlblBehavior)) && (ParentPassage_.action_type == BA_ActionTyp_RightLaneChangePrepare)))
        {
            BAChgWork_S3_ = true;
            BAChgCost_ = BAChgCost_ + ADC::common::k_BA_BNAChgCost_S3_encrgA;
        } // encrg LITTLE Right lanechg go on in NRP(0530) LCP-LCP
        else
        {
            BAChgWork_S3_ = false;
            BAChgCost_ = BAChgCost_ + 0;
            /* if(ParentNode_.m_ActionTypArr[0] == BA_ActionTyp_LaneFollow && (ParentPassage_.action_type == BA_ActionTyp_LeftLaneChange || ParentPassage_.action_type == BA_ActionTyp_RightLaneChange))
            {
                DLOG(INFO) << "   LF-LC Case = " << BAChgCost_ << std::endl;
            } */
        }
    }
}

void BehaviorNodesAdd::BehaviorChgCost_S4(BA::behaviorNodeGlobal ParentNode_,
                                          Passage ParentPassage_,
                                          BA::Input_BA Input_BA,
                                          vector<BA::BA_ActionTyp> BA_LatAvlblBehavior)
{
    BA::BA_ActionTyp typ1 = BA_ActionTyp_LeftLaneChange;
    BA::BA_ActionTyp typ2 = BA_ActionTyp_RightLaneChange;
    BA::BA_ActionTyp typ3 = BA_ActionTyp_LeftLaneChangePrepare;
    BA::BA_ActionTyp typ4 = BA_ActionTyp_RightLaneChangePrepare;
    BA::BA_ActionTyp typ5 = BA_ActionTyp_LaneFollow;
    if ((!ParentNode_.m_ActionTypArr.empty()) && (!BA_LatAvlblBehavior.empty()) && (Input_BA.SM_LCAvlblBehavior != LCAvlblBhvr_LaneChange_Auto))
    {
        if (BA_StepNum_ == 0 && ParentNode_.m_ActionTypArr[0] == BA_ActionTyp_LeftLaneChangePrepare && TargetBehaviorChk(typ1, BA_LatAvlblBehavior) && (ParentPassage_.action_type == BA_ActionTyp_LeftLaneChange))
        {
            BAChgWork_S4_ = true;
            BAChgCost_ = BAChgCost_ + ADC::common::k_BA_BNAChgCost_S4_encrgA;
        }

        else if (BA_StepNum_ == 0 && ParentNode_.m_ActionTypArr[0] == BA_ActionTyp_RightLaneChangePrepare && TargetBehaviorChk(typ2, BA_LatAvlblBehavior) && (ParentPassage_.action_type == BA_ActionTyp_RightLaneChange))
        {
            BAChgWork_S4_ = true;
            BAChgCost_ = BAChgCost_ + ADC::common::k_BA_BNAChgCost_S4_encrgA;
        }

        else if (BA_StepNum_ == 0 && ParentNode_.m_ActionTypArr[0] == BA_ActionTyp_LeftLaneChangePrepare && TargetBehaviorChk(typ3, BA_LatAvlblBehavior) && (ParentPassage_.action_type == BA_ActionTyp_LeftLaneChangePrepare))
        {
            BAChgWork_S4_ = true;
            BAChgCost_ = BAChgCost_ + ADC::common::k_BA_BNAChgCost_S4_pnshA;
        }

        else if (BA_StepNum_ == 0 && ParentNode_.m_ActionTypArr[0] == BA_ActionTyp_RightLaneChangePrepare && TargetBehaviorChk(typ4, BA_LatAvlblBehavior) && (ParentPassage_.action_type == BA_ActionTyp_RightLaneChangePrepare))
        {
            BAChgWork_S4_ = true;
            BAChgCost_ = BAChgCost_ + ADC::common::k_BA_BNAChgCost_S4_pnshA;
        }

        else if (BA_StepNum_ == 0 && ((ParentNode_.m_ActionTypArr[0] == BA_ActionTyp_RightLaneChangePrepare) || (ParentNode_.m_ActionTypArr[0] == BA_ActionTyp_LeftLaneChangePrepare)) && TargetBehaviorChk(typ5, BA_LatAvlblBehavior) && (ParentPassage_.action_type == BA_ActionTyp_LaneFollow))
        {
            BAChgWork_S4_ = true;
            BAChgCost_ = BAChgCost_ + ADC::common::k_BA_BNAChgCost_S4_pnshB;
        }

        // else if(Step_Num_ == 0)
        // {
        //     BAChgWork_S4_ = false;
        //     BAChgCost_ = BAChgCost_ +  k_BA_BNAChgCostCommon;      // S2-S4 only add one Common list,  it will do at S4 Fcn for StepNum = 0 cond, else ADD ZERO

        // }
        else
        {
            BAChgWork_S4_ = false;
            BAChgCost_ = BAChgCost_ + 0;
        }
    }
    else
    {
        BAChgWork_S4_ = false;
        BAChgCost_ = BAChgCost_ + 0;
    }
}

void BehaviorNodesAdd::BehaviorChgCostS2_S4ALLFail()
{
    if (BAChgWork_S2_ == false && BAChgWork_S3_ == false && BAChgWork_S4_ == false && BA_StepNum_ == 0)
    {
        BAS2_S4Allfail_ = true;
        BAChgCost_ = BAChgCost_ + k_BA_BNAChgCostCommon;
    }

    else
    {
        BAS2_S4Allfail_ = false;
        BAChgCost_ = BAChgCost_ + 0;
    }
}

bool BehaviorNodesAdd::TargetBehaviorChk(BA::BA_ActionTyp Goal, vector<BA::BA_ActionTyp> BA_LatAvlblBehavior)
{
    bool chk = false;
    for (int i = 0; i < BA_LatAvlblBehavior.size(); i++)
    {
        if (Goal == BA_LatAvlblBehavior[i])
        {
            chk = true;
        }
    }
    return chk;
}

void BehaviorNodesAdd::BehaviorChgGuideCost(BA::behaviorNodeGlobal ParentNode_,
                                            Passage ParentPassage_,
                                            BA::Input_BA Input_BA,
                                            BA::behaviorNodeGlobal &NewNode_)
{
    // LCP iner Encrg&Punsh
    if (k_BA_BNABAChgCostLCPInerOn)
    {
        if ((BA_StepNum_ == 0) && (BA_OptAction_LastCycle_ == BA_ActionTyp_LeftLaneChangePrepare || BA_OptAction_LastCycle_ == BA_ActionTyp_RightLaneChangePrepare) && (BA_OptAction_LastCycle_ == ParentPassage_.action_type))
        {
            if (BA_OptTargetObjID_LastCycle_ != ParentPassage_.longt_info.target_object_id && LCPFrtBckChk(BA_OptLCPCls_LastCycle_) != LCPFrtBckChk(ParentPassage_.lcp_cls))
            {
                BAChgCost_ = BAChgCost_ + 2 * k_BA_BNACommonChgCost_pnshC;
            }
            else if ((BA_OptLCPCls_LastCycle_ == 4 || BA_OptLCPCls_LastCycle_ == 8) && (ParentPassage_.lcp_cls == 3 || ParentPassage_.lcp_cls == 7))
            {
                // encrg bck_lcp more 0808
                BAChgCost_ = BAChgCost_ + k_BA_BNAChgCostSteady_EncrgB;
            }
            else if ((BA_OptLCPCls_LastCycle_ == 2 || BA_OptLCPCls_LastCycle_ == 5) && (ParentPassage_.lcp_cls == 1 || ParentPassage_.lcp_cls == 6))
            {
                if (ParentPassage_.longt_info.target_object_id == 0) //||(ParentPassage_.longt_info.red_border_distance < 2))
                {
                    BAChgCost_ = BAChgCost_ + k_BA_BNAChgCostSteady_EncrgC;
                }
                else
                {
                    BAChgCost_ = BAChgCost_ + k_BA_BNAChgCostSteady_EncrgA;
                }
            }
            else
            {
                BAChgCost_ = BAChgCost_ + 0;
            }
        }
    }

    // LCP Frt&SpdLimit Punsh
    // Punsh Frt_LCP when SpdLmt On
    if (k_BA_BNABAChgCostFrtLCPSpdLmOn)
    {
        DLOG(INFO) << "LCPLm_ID = " << float(NewNode_.m_SpdLmtID) << std::endl;
        DLOG(INFO) << "LCPLm_Dist = " << float(NewNode_.m_SpdLmtDst) << std::endl;
        DLOG(INFO) << "LCPLm_FRT = " << int(LCPFrtBckChk(ParentPassage_.lcp_cls)) << std::endl;
        if ((BA_StepNum_ == 0) && (BA_OptAction_LastCycle_ == BA_ActionTyp_LaneFollow) && (ParentPassage_.action_type == BA_ActionTyp_RightLaneChangePrepare))
        {
            if (NewNode_.m_SpdLmtID != uint8_t(0) && NewNode_.m_SpdLmtDst < k_BA_BNABAChgCostFrtLCPSpdLmDistThr && LCPFrtBckChk(ParentPassage_.lcp_cls))
            {
                BAChgCost_ = BAChgCost_ + 5 * k_BA_BNACommonChgCost_pnshC;
                DLOG(INFO) << "LCPSpdLmOn = " << std::endl;
            }
        }
    }

    // ILC Feature
    // enum FeatureTag : int  {    FeatureTag_NotAvlbl = 0,    FeatureTag_LineFllw = 1,    FeatureTag_CarFllw = 2,    FeatureTag_LineDvt = 3,    FeatureTag_LineMiss = 4  };
    if (k_BA_BNABAChgCostILCOn)
    {
        if ((BA_StepNum_ == 0) && (BA_OptAction_LastCycle_ == BA_ActionTyp_LaneFollow) && (ParentPassage_.action_type == BA_ActionTyp_LaneFollow) && (ParentPassage_.feature_label == 3))
        {
            BAChgCost_ = BAChgCost_ + k_BA_BNABAChgCostILCEncrg;
        }
        else
        {
            BAChgCost_ = BAChgCost_ + k_BA_BNABAChgCostILCPnsh;
        }

        if (((BA_StepNum_ == 0) && (ParentPassage_.action_type == BA_ActionTyp_RightLaneChangePrepare || ParentPassage_.action_type == BA_ActionTyp_LeftLaneChangePrepare)))
        {
            if (ParentPassage_.feature_label == 3)
            {
                BAChgCost_ = BAChgCost_ - 0.01;
            }
        } // encrg LCP with ILC
    }
}

bool BehaviorNodesAdd::LCPFrtBckChk(uint8_t LCPCls)
{
    if (LCPCls == 1 || LCPCls == 2 || LCPCls == 5 || LCPCls == 6)
    {
        return true;
    }
    else
    {
        return false;
    }
}

void BehaviorNodesAdd::PDStepCostEstimate(PD::BehaviorNodeLocal BehaviorNodeLocal,
                                          Passage ParentPassage)
{
    // initialize
    BAPDCost_ = 0;
    // BADelimit Cost
    PDRuleDelimit(BehaviorNodeLocal);
    // PDNormal Cost
    PDNormalization(BehaviorNodeLocal, ParentPassage);
    if (k_BA_BNAPDCostIN) // PDCost Work Buttion
    {
        // PD Cost Step
        BAPDCost_ = BADelimitCost_ + PDNormalCost_;
        // just for Ob
        if (ParentPassage.action_type == BA_ActionTyp::BA_ActionTyp_LaneFollow)
        {
            DLOG(INFO) << "LF_BADelimitCost is: " << BADelimitCost_ << std::endl;
            DLOG(INFO) << "LF_BAPDCost_ is: " << BAPDCost_ << std::endl;
        }
        else if ((ParentPassage.action_type == BA_ActionTyp::BA_ActionTyp_LeftLaneChangePrepare) || (ParentPassage.action_type == BA_ActionTyp::BA_ActionTyp_RightLaneChangePrepare))
        {
            DLOG(INFO) << "LCP_BADelimitCost is: " << BADelimitCost_ << std::endl;
            DLOG(INFO) << "LCP_BAPDCost_ is: " << BAPDCost_ << std::endl;
        }
        else if ((ParentPassage.action_type == BA_ActionTyp::BA_ActionTyp_LeftLaneChange) || (ParentPassage.action_type == BA_ActionTyp::BA_ActionTyp_RightLaneChange))
        {
            DLOG(INFO) << "LC_BADelimitCost is: " << BADelimitCost_ << std::endl;
            DLOG(INFO) << "LC_BAPDCost_ is: " << BAPDCost_ << std::endl;
        }
        else
        {
            DLOG(INFO) << "STANDBY " << std::endl;
        }
    }
}

void BehaviorNodesAdd::LayerCostGen()
{
    LayerCost_ = 0; // initialize

    LayerCost_ = (min(Gain_share_, float(1.0))) * Gain_decay_ * (((BAChgCost_ + BAALCCourageCost_) * k_BA_BNAEachLyrChgCostWeight) + (BAPDCost_ * (1 - k_BA_BNAEachLyrChgCostWeight)));
}

void BehaviorNodesAdd::PDRuleDelimit(PD::BehaviorNodeLocal BehaviorNodeLocal)
//  0314 chg for PD Cost Test;
{
    BADelimitCost_ = 0;
    if (k_BA_BNAPDCostDelimitOff)
    {
        BADelimitCost_ = 0;
    }
    else
    {
        // LegalPunishment Delimit
        bool chk1 = 0;
        if (!BehaviorNodeLocal.m_risk_evaluation.m_legal_pnshmt_list.empty())
        {
            for (int i = 0; i < BehaviorNodeLocal.m_risk_evaluation.m_legal_pnshmt_list.size(); i++)
            {
                if (BehaviorNodeLocal.m_risk_evaluation.m_legal_pnshmt_list[i].m_pnshmnt == 1)
                {
                    chk1 = 1;
                }
            }

            if (chk1)
            {
                BADelimitCost_ = BADelimitCost_ + ADC::common::k_BA_BNADelimitValue;
            }
        }

        if (!BehaviorNodeLocal.m_risk_evaluation.m_comfort_list.empty())
        {
            float LgtAccelMax = BehaviorNodeLocal.m_risk_evaluation.m_comfort_list[0].m_cft_long_acc_max;
            float LatAccelMax = BehaviorNodeLocal.m_risk_evaluation.m_comfort_list[0].m_cft_lat_acc_max;
            float LgtJerkMax = BehaviorNodeLocal.m_risk_evaluation.m_comfort_list[0].m_cft_long_jerk_max;
            float LatJerkMax = BehaviorNodeLocal.m_risk_evaluation.m_comfort_list[0].m_cft_lat_jerk_max;

            if (LgtAccelMax >= ADC::common::k_BA_BNADelimitLgtAccelISO)
            {
                BADelimitCost_ = BADelimitCost_ + ADC::common::k_BA_BNADelimitValue;
            }
            if (LatAccelMax >= ADC::common::k_BA_BNADelimitLatAccelThr)
            {
                BADelimitCost_ = BADelimitCost_ + ADC::common::k_BA_BNADelimitValue;
            }
        } // Comfort Delimit

        if (!BehaviorNodeLocal.m_risk_evaluation.m_stnobstc_risk_list.empty())
        {
            float BdrLatMinDis = BehaviorNodeLocal.m_risk_evaluation.m_stnobstc_risk_list[0].m_bdrlat_min_distance;
            if (BdrLatMinDis != 0 && BdrLatMinDis <= ADC::common::k_BA_BNAStnObjLatDistMin)
            {
                BADelimitCost_ = BADelimitCost_ + ADC::common::k_BA_BNADelimitValue;
            }
        } // StnObstcRisk Delimit
        if (BehaviorNodeLocal.m_node_clsn_sts == true)
        {
            BADelimitCost_ = BADelimitCost_ + k_BA_BNADelimitClsnGain * ADC::common::k_BA_BNADelimitValue;
        } // ClsnRisk Delimit
    }
}

void BehaviorNodesAdd::PDNormalization(PD::BehaviorNodeLocal BehaviorNodeLocal,
                                       Passage ParentPassage)
{
    ReminderObjID_ = 0;
    ReminderObjLst_.clear();
    PDNormalPartArr_.clear();
    MathFcn::LinearIntrPltn<float> IntrPltn;

    // average spd  & terminal spd    Efficient
    float SpdMax_ = ParentPassage.longt_info.set_speed;
    vector<float> LgtSpdDynRgn;
    LgtSpdDynRgn.push_back(0.0);
    LgtSpdDynRgn.push_back(SpdMax_);

    float AvgSpdPart = IntrPltn.InterPolation(LgtSpdDynRgn, k_BA_BNAQuantizationCommon_re,
                                              float(BehaviorNodeLocal.m_risk_evaluation.m_trffc_effcy_list[0].m_effcy_avgspd));
    float EndSpdPart = IntrPltn.InterPolation(LgtSpdDynRgn, k_BA_BNAQuantizationCommon_re,
                                              float(BehaviorNodeLocal.m_risk_evaluation.m_trffc_effcy_list[0].m_effcy_endspd));

    float SpdEffFactor = k_BA_BNANormalSpdEffAvgWgt * AvgSpdPart + (1 - k_BA_BNANormalSpdEffAvgWgt) * EndSpdPart;
    // Push 1 Eff
    PDNormalPartArr_.push_back(SpdEffFactor);

    // Comfrt
    //  Lgt Comfort
    float LgtStartAccel = 0; // just for temp code,  will be replace by pd input
    float LgtAccelMax = 0;
    float LgtAccelMin = 0;
    float LgtAccelDltMax = 0;
    float LgtAccelDltMin = 0;
    float LgtAccelPart = 0;

    float LgtJerkMax = 0;
    float LgtJerkPart = 0;

    if (LgtAccelMax > 0 && LgtAccelMin > 0)
    {
        LgtAccelPart = 0;
    }
    else if (LgtAccelMax > 0 && LgtAccelMin < 0)
    {
        LgtAccelPart = 0;
    }
    else if (LgtAccelMin < 0 && LgtAccelMax < 0)
    {
        LgtAccelPart = 0;
    }
    else
    {
        LgtAccelPart = 0;
    }

    //  Lat Comfort
    float LatAccelAbsMax = 0;
    float LatJerkAbsMax = 0;
    float LatAccelPart = 0;
    float LatJerkPart = 0;

    float ComfrtFactor = k_BA_BNANormalCfrtLgtAccelWgt * LgtAccelPart + k_BA_BNANormalCfrtLgtJrkWgt * LgtJerkPart + k_BA_BNANormalCfrtLatAccelWgt * LatAccelPart + k_BA_BNANormalCfrtLatJrkWgt * LatJerkPart;
    // Push 2 Cmfrt
    PDNormalPartArr_.push_back(ComfrtFactor);

    // risk
    float RiskFactor = 0;
    // get interest obj
    if (ParentPassage.action_type == BA_ActionTyp::BA_ActionTyp_LaneFollow)
    {
        float LF_Obj1TTC = 999.0F;
        float LF_Obj1LgtDist = 999.0F;
        float LF_Obj1TD = 999.0F;
        if (!ParentPassage.interest_obj_id_list.empty())
        {
            LF_Obj1TTC = PDLabelObjTTC(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[0]);
            LF_Obj1LgtDist = PDLabelObjLgtMinDist(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[0]);
            LF_Obj1TD = PDLabelObjTD(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[0]);
        }
        // just for OB
        // DLOG(INFO) << "LF_Obj1TTC is: " << LF_Obj1TTC << std::endl;
        // DLOG(INFO) << "LF_Obj1TD is: " << LF_Obj1TD << std::endl;
        // DLOG(INFO) << "LF_Obj1LgtDist is: " << LF_Obj1LgtDist << std::endl;
        if ((abs(LF_Obj1TTC) < k_BA_BNANormalRskTTCLFDgrThr) || (abs(LF_Obj1LgtDist) < k_BA_BNANormalRskLFDistThr || (abs(LF_Obj1TD) < k_BA_BNANormalRskLFTmDistThr)))
        {
            RiskFactor = k_BA_BNANormalRskTTCLFUnDgrCost;
        }
        else
        {
            RiskFactor = k_BA_BNANormalRskTTCLFUnDgrCost;
        }
    }
    else if (ParentPassage.action_type == BA_ActionTyp::BA_ActionTyp_LeftLaneChange || ParentPassage.action_type == BA_ActionTyp::BA_ActionTyp_RightLaneChange)
    {
        float LC_Obj1TTC = 999.0F;
        float LC_Obj2TTC = 999.0F;
        float LC_Obj3TTC = 999.0F;
        float LC_Obj1LgtDist = 999.0F;
        float LC_Obj2LgtDist = 999.0F;
        float LC_Obj3LgtDist = 999.0F;
        float LC_Obj1TD = 999.0F;
        float LC_Obj2TD = 999.0F;
        float LC_Obj3TD = 999.0F;
        if (ParentPassage.interest_obj_id_list.size() >= 3)
        {
            LC_Obj1TTC = abs(PDLabelObjTTC(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[0]));
            LC_Obj2TTC = abs(PDLabelObjTTC(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[1]));
            LC_Obj3TTC = abs(PDLabelObjTTC(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[2]));
            LC_Obj1LgtDist = abs(PDLabelObjLgtMinDist(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[0]));
            LC_Obj2LgtDist = abs(PDLabelObjLgtMinDist(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[1]));
            LC_Obj3LgtDist = abs(PDLabelObjLgtMinDist(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[2]));
            LC_Obj1TD = abs(PDLabelObjTD(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[0]));
            LC_Obj2TD = abs(PDLabelObjTD(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[1]));
            LC_Obj3TD = abs(PDLabelObjTD(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[2]));
            // Obj4 do not be consided in NorRsk
        }
        float LC_ObjTTCMin = min(LC_Obj1TTC, (min(LC_Obj2TTC, LC_Obj3TTC)));
        float LC_ObjLgtDistMin = min(LC_Obj1LgtDist, (min(LC_Obj2LgtDist, LC_Obj3LgtDist)));
        float LC_ObjTDMin = min(LC_Obj1TD, (min(LC_Obj2TD, LC_Obj3TD)));
        if (LC_ObjTTCMin >= k_BA_BNANormalRskTTCNORThr)
        {
            if ((LC_ObjLgtDistMin < k_BA_BNANormalRskLCDistThr_NOR) || ((LC_ObjTDMin < k_BA_BNANormalRskLCTmDistThr_NOR)))
            {
                RiskFactor = k_BA_BNANormalRskTTCDgrCost;
                if (LC_ObjLgtDistMin == LC_Obj1LgtDist && ParentPassage.interest_obj_id_list[0] != 0)
                {
                    ReminderObjLst_.push_back(ParentPassage.interest_obj_id_list[0]);
                }
                else if (LC_ObjLgtDistMin == LC_Obj2LgtDist && ParentPassage.interest_obj_id_list[1] != 0)
                {
                    ReminderObjLst_.push_back(ParentPassage.interest_obj_id_list[1]);
                }
                else if (LC_ObjLgtDistMin == LC_Obj3LgtDist && ParentPassage.interest_obj_id_list[2] != 0)
                {
                    ReminderObjLst_.push_back(ParentPassage.interest_obj_id_list[2]);
                }
            }
            else
            {
                RiskFactor = k_BA_BNANormalRskTTCUnDgrCost;
            }
        }
        else
        {
            if ((LC_ObjTTCMin < k_BA_BNANormalRskTTCDgrThr) || (LC_ObjLgtDistMin < k_BA_BNANormalRskLCDistThr) || ((LC_ObjTDMin < k_BA_BNANormalRskLCTmDistThr)))
            {
                RiskFactor = k_BA_BNANormalRskTTCDgrCost;
                if (LC_ObjLgtDistMin == LC_Obj1LgtDist && ParentPassage.interest_obj_id_list[0] != 0)
                {
                    ReminderObjLst_.push_back(ParentPassage.interest_obj_id_list[0]);
                }
                else if (LC_ObjLgtDistMin == LC_Obj2LgtDist && ParentPassage.interest_obj_id_list[1] != 0)
                {
                    ReminderObjLst_.push_back(ParentPassage.interest_obj_id_list[1]);
                }
                else if (LC_ObjLgtDistMin == LC_Obj3LgtDist && ParentPassage.interest_obj_id_list[2] != 0)
                {
                    ReminderObjLst_.push_back(ParentPassage.interest_obj_id_list[2]);
                }
            }
            else
            {
                RiskFactor = k_BA_BNANormalRskTTCUnDgrCost;
            }
        }
    }
    else if (ParentPassage.action_type == BA_ActionTyp::BA_ActionTyp_LeftLaneChangePrepare || ParentPassage.action_type == BA_ActionTyp::BA_ActionTyp_RightLaneChangePrepare)
    {
        float LCP_Obj1TTC = 999.0F;
        float LCP_Obj2TTC = 999.0F;
        float LCP_Obj3TTC = 999.0F;
        float LCP_Obj4TTC = 999.0F;
        float LCP_Obj1LgtDist = 999.0F;
        float LCP_Obj2LgtDist = 999.0F;
        float LCP_Obj3LgtDist = 999.0F;
        float LCP_Obj4LgtDist = 999.0F;
        float LCP_Obj1TD = 999.0F;
        float LCP_Obj2TD = 999.0F;
        float LCP_Obj3TD = 999.0F;
        float LCP_Obj4TD = 999.0F;
        if (ParentPassage.interest_obj_id_list.size() >= 5)
        {
            if (ParentPassage.lcp_cls == 1 || ParentPassage.lcp_cls == 2 || ParentPassage.lcp_cls == 5 || ParentPassage.lcp_cls == 6)
            {
                LCP_Obj1TTC = abs(PDLabelObjTTC(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[0]));
                LCP_Obj2TTC = abs(PDLabelObjTTC(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[1]));
                // LCP_Obj2TTC = 999;
                LCP_Obj3TTC = abs(PDLabelObjTTC(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[2]));
                LCP_Obj4TTC = abs(PDLabelObjTTC(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[3]));
                LCP_Obj1LgtDist = abs(PDLabelObjLgtMinDist(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[0]));
                LCP_Obj2LgtDist = abs(PDLabelObjLgtMinDist(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[1]));
                LCP_Obj3LgtDist = abs(PDLabelObjLgtMinDist(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[2]));
                LCP_Obj4LgtDist = abs(PDLabelObjLgtMinDist(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[3]));
                LCP_Obj1TD = abs(PDLabelObjTD(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[0]));
                LCP_Obj2TD = abs(PDLabelObjTD(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[1]));
                LCP_Obj3TD = abs(PDLabelObjTD(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[2]));
                LCP_Obj4TD = abs(PDLabelObjTD(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[3]));
            }
            else
            {
                LCP_Obj1TTC = abs(PDLabelObjTTC(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[0]));
                LCP_Obj2TTC = abs(PDLabelObjTTC(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[1]));
                LCP_Obj3TTC = abs(PDLabelObjTTC(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[2]));
                LCP_Obj4TTC = abs(PDLabelObjTTC(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[4]));
                LCP_Obj1LgtDist = abs(PDLabelObjLgtMinDist(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[0]));
                LCP_Obj2LgtDist = abs(PDLabelObjLgtMinDist(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[1]));
                LCP_Obj3LgtDist = abs(PDLabelObjLgtMinDist(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[2]));
                LCP_Obj4LgtDist = abs(PDLabelObjLgtMinDist(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[4]));
                LCP_Obj1TD = abs(PDLabelObjTD(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[0]));
                LCP_Obj2TD = abs(PDLabelObjTD(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[1]));
                LCP_Obj3TD = abs(PDLabelObjTD(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[2]));
                LCP_Obj4TD = abs(PDLabelObjTD(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[4]));
            }
            // just for OB
            // DLOG(INFO) << "LCP_Obj1TTC is: " << LCP_Obj1TTC << std::endl;
            // DLOG(INFO) << "LCP_Obj1LgtDist is: " << LCP_Obj1LgtDist << std::endl;
            // DLOG(INFO) << "LCP_Obj2TTC is: " << LCP_Obj2TTC << std::endl;
            // DLOG(INFO) << "LCP_Obj2LgtDist is: " << LCP_Obj2LgtDist << std::endl;
            // DLOG(INFO) << "LCP_Obj3TTC is: " << LCP_Obj3TTC << std::endl;
            // DLOG(INFO) << "LCP_Obj3LgtDist is: " << LCP_Obj3LgtDist << std::endl;
            // DLOG(INFO) << "LCP_Obj4TTC is: " << LCP_Obj4TTC << std::endl;
            // DLOG(INFO) << "LCP_Obj4LgtDist is: " << LCP_Obj4LgtDist << std::endl;

            float LCP_ObjTTCMin = min(min(LCP_Obj1TTC, (min(LCP_Obj2TTC, LCP_Obj3TTC))), LCP_Obj4TTC);
            float LCP_ObjLgtDistMin = min(min(LCP_Obj1LgtDist, (min(LCP_Obj2LgtDist, LCP_Obj3LgtDist))), LCP_Obj4LgtDist);
            float LCP_ObjTDMin = min(min(LCP_Obj1TD, (min(LCP_Obj2TD, LCP_Obj3TD))), LCP_Obj4TD);
            if (LCP_ObjTTCMin >= k_BA_BNANormalRskTTCNORThr)
            {
                if ((LCP_ObjLgtDistMin < k_BA_BNANormalRskLCDistThr_NOR) || ((LCP_ObjTDMin < k_BA_BNANormalRskLCTmDistThr_NOR)))
                {
                    RiskFactor = k_BA_BNANormalRskTTCDgrCost;
                }
                else
                {
                    RiskFactor = k_BA_BNANormalRskTTCUnDgrCost;
                }
            }
            else
            {
                if ((LCP_ObjTTCMin < k_BA_BNANormalRskTTCDgrThr) || (LCP_ObjLgtDistMin < k_BA_BNANormalRskLCDistThr) || ((LCP_ObjTDMin < k_BA_BNANormalRskLCTmDistThr)))
                {
                    RiskFactor = k_BA_BNANormalRskTTCDgrCost;
                }
                else
                {
                    RiskFactor = k_BA_BNANormalRskTTCUnDgrCost;
                }
            }
        }
        else
        {
            RiskFactor = k_BA_BNANormalRskTTCUnDgrCost;
        }
    }
    else
    {
        RiskFactor = k_BA_BNANormalRskTTCUnDgrCost;
    }
    // OFF Button FOR NormalCost's Risking Part
    if (k_BA_BNAPDCostNorRskOff)
    {
        RiskFactor = 0;
    }
    // just for Ob
    // if (ParentPassage.action_type == BA_ActionTyp::BA_ActionTyp_LaneFollow)
    // {
    // 	DLOG(INFO) << "LF_RiskFactor is: " << RiskFactor << std::endl;
    // }
    // else if ((ParentPassage.action_type == BA_ActionTyp::BA_ActionTyp_LeftLaneChangePrepare) || (ParentPassage.action_type == BA_ActionTyp::BA_ActionTyp_RightLaneChangePrepare))
    // {
    // 	DLOG(INFO) << "LCP_RiskFactor is: " << RiskFactor << std::endl;
    // }
    // else
    // {
    // 	DLOG(INFO) << "STANDBY "<< std::endl;
    // }

    // Uninteresting Obj Risk
    // Separate NorRsk & UnexpectObjRsk 0714
    float UnexpectRiskFactor = 0;
    if (k_BA_BNAUnexpectObjPnshOn && ((ParentPassage.action_type == BA_ActionTyp::BA_ActionTyp_LeftLaneChange) || (ParentPassage.action_type == BA_ActionTyp::BA_ActionTyp_RightLaneChange)))
    {
        if (!BehaviorNodeLocal.m_risk_evaluation.m_dynamicobj_risk_list.empty())
        {
            if (k_BA_BNAPDCostUnexpectRskOn_EgoBck)
            {
                if (ParentPassage.interest_obj_id_list[3] != 0)
                {
                    DLOG(INFO) << "Unexpect Danger Obj ID is: " << int(ParentPassage.interest_obj_id_list[3]) << std::endl;
                    // EgoBckObj
                    bool TTCRes = false;
                    bool TDRes = false;
                    bool DistRes = false;
                    if (abs(PDLabelObjTTC(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[3])) >= k_BA_BNAUnexpectObjTTCThr_NOR_EgoBck)
                    {
                        TTCRes = false;
                        TDRes = abs(PDLabelObjTD(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[3])) < k_BA_BNAUnexpectObjTDThr_NOR_EgoBck;
                        DistRes = abs(PDLabelObjLgtMinDist(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[3])) < k_BA_BNAUnexpectObjDistThr_NOR_EgoBck;
                    }
                    else
                    {
                        TTCRes = abs(PDLabelObjTTC(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[3])) < k_BA_BNAUnexpectObjTTCThr;
                        TDRes = abs(PDLabelObjTD(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[3])) < k_BA_BNAUnexpectObjTDThr;
                        DistRes = abs(PDLabelObjLgtMinDist(BehaviorNodeLocal, ParentPassage.interest_obj_id_list[3])) < k_BA_BNAUnexpectObjDisThr;
                    }

                    if (TTCRes || TDRes || DistRes)
                    {
                        UnexpectRiskFactor = k_BA_BNAUnexpectObjPnshCost;
                        ReminderObjLst_.push_back(ParentPassage.interest_obj_id_list[3]);
                        // 	DLOG(INFO) << "Unexpect Danger Obj ID is: " << int(iter -> m_obj_id)<< std::endl;
                        // iter = BehaviorNodeLocal.m_risk_evaluation.m_dynamicobj_risk_list.end();
                    }
                }
            }
            if (k_BA_BNAPDCostUnexpectRskOn_ALL)
            {
                int InterestObjNum = int(ParentPassage.interest_obj_id_list.size());
                auto iter = BehaviorNodeLocal.m_risk_evaluation.m_dynamicobj_risk_list.begin();
                for (; iter != BehaviorNodeLocal.m_risk_evaluation.m_dynamicobj_risk_list.end(); iter++)
                {
                    bool ObjInclude = false;
                    for (int i = 0; i < InterestObjNum; i++)
                    {
                        if (iter->m_obj_id == ParentPassage.interest_obj_id_list[i] || iter->m_obj_id == 0)
                        {
                            ObjInclude = true;
                            break;
                        }
                    }
                    if (!ObjInclude)
                    {
                        bool TTCRes = false;
                        bool TDRes = false;
                        bool DistRes = false;
                        if (abs(PDLabelObjTTC(BehaviorNodeLocal, iter->m_obj_id)) >= k_BA_BNANormalRskTTCNORThr)
                        {
                            TTCRes = false;
                            TDRes = abs(PDLabelObjTD(BehaviorNodeLocal, iter->m_obj_id)) < k_BA_BNAUnexpectObjTDThr_NOR;
                            DistRes = abs(PDLabelObjLgtMinDist(BehaviorNodeLocal, iter->m_obj_id)) < k_BA_BNAUnexpectObjDistThr_NOR;
                        }
                        else
                        {
                            TTCRes = abs(PDLabelObjTTC(BehaviorNodeLocal, iter->m_obj_id)) < k_BA_BNAUnexpectObjTTCThr;
                            TDRes = abs(PDLabelObjTD(BehaviorNodeLocal, iter->m_obj_id)) < k_BA_BNAUnexpectObjTDThr;
                            DistRes = abs(PDLabelObjLgtMinDist(BehaviorNodeLocal, iter->m_obj_id)) < k_BA_BNAUnexpectObjDisThr;
                        }
                        if (TTCRes || TDRes || DistRes)
                        {
                            UnexpectRiskFactor = k_BA_BNAUnexpectObjPnshCost;
                            if (iter->m_obj_id != 0)
                            {
                                ReminderObjLst_.push_back(iter->m_obj_id);
                            }

                            // 	DLOG(INFO) << "Unexpect Danger Obj ID is: " << int(iter -> m_obj_id)<< std::endl;
                            // iter = BehaviorNodeLocal.m_risk_evaluation.m_dynamicobj_risk_list.end();
                            break;
                        }
                    }
                }
            }
        }
    }

    // ReminderObj Judge 0927
    if (ReminderObjLst_.empty())
    {
        ReminderObjID_ = 0;
    }
    else
    {
        ReminderObjID_ = ReminderObjLst_.front();
        // lclabel > lcegobck > unexpect
    }

    // Push 3 Rsk
    PDNormalPartArr_.push_back(RiskFactor);
    PDNormalPartArr_.push_back(UnexpectRiskFactor);
    // Calc Normal PD Cost
    PDNormalCost_ = k_BA_BNANormalPDNormalFctry_Eff * PDNormalPartArr_[0] + k_BA_BNANormalPDNormalFctry_Cmfrt * PDNormalPartArr_[1] + k_BA_BNANormalPDNormalFctry_Rsk * PDNormalPartArr_[2] + PDNormalPartArr_[3];
    // PDNormalPartArr_[3] is unexpectObjPnshCost 0714
    // just for Ob
    if (ParentPassage.action_type == BA_ActionTyp::BA_ActionTyp_LaneFollow)
    {
        DLOG(INFO) << "LF_PDNormalCost is: " << PDNormalCost_ << std::endl;
    }
    else if ((ParentPassage.action_type == BA_ActionTyp::BA_ActionTyp_LeftLaneChangePrepare) || (ParentPassage.action_type == BA_ActionTyp::BA_ActionTyp_RightLaneChangePrepare))
    {
        DLOG(INFO) << "LCP_PDNormalCost is: " << PDNormalCost_ << std::endl;
    }
    else if ((ParentPassage.action_type == BA_ActionTyp::BA_ActionTyp_LeftLaneChange) || (ParentPassage.action_type == BA_ActionTyp::BA_ActionTyp_RightLaneChange))
    {
        DLOG(INFO) << "LC_PDNormalCost is: " << PDNormalCost_ << std::endl;
    }
    else
    {
        DLOG(INFO) << "STANDBY " << std::endl;
    }
}

float BehaviorNodesAdd::PDLabelObjTTC(PD::BehaviorNodeLocal BehaviorNodeLocal,
                                      uint32_t InterestObjID)
{
    if ((!BehaviorNodeLocal.m_risk_evaluation.m_dynamicobj_risk_list.empty()) && (InterestObjID != 0))
    {
        int ObjNum = int(BehaviorNodeLocal.m_risk_evaluation.m_dynamicobj_risk_list.size());
        float Chk_ObjTTC = 999;
        for (int i = 0; i < ObjNum; i++)
        {
            if (InterestObjID == BehaviorNodeLocal.m_risk_evaluation.m_dynamicobj_risk_list[i].m_obj_id)
            {
                auto Chk_ObjTTCAry = BehaviorNodeLocal.m_risk_evaluation.m_dynamicobj_risk_list[i].m_obj_long_clsn_ttc;
                if (!Chk_ObjTTCAry.empty())
                {
                    for (int j = 0; j < Chk_ObjTTCAry.size(); j++)
                    {
                        if (abs(Chk_ObjTTC) > abs(Chk_ObjTTCAry[j]) && Chk_ObjTTCAry[j] != 999 && Chk_ObjTTCAry[j] != 0)
                        {
                            Chk_ObjTTC = Chk_ObjTTCAry[j];
                        }
                    }
                    return Chk_ObjTTC;
                }
                else
                {
                    return 999.0F;
                }
            }
        }
        return 999.0F;
    }
    else
    {
        return 999.0F;
    }
}

float BehaviorNodesAdd::PDLabelObjLgtMinDist(PD::BehaviorNodeLocal BehaviorNodeLocal,
                                             uint32_t InterestObjID)
{
    if ((!BehaviorNodeLocal.m_risk_evaluation.m_dynamicobj_risk_list.empty()) && (InterestObjID != 0))
    {
        int ObjNum = int(BehaviorNodeLocal.m_risk_evaluation.m_dynamicobj_risk_list.size());
        for (int i = 0; i < ObjNum; i++)
        {
            if (InterestObjID == BehaviorNodeLocal.m_risk_evaluation.m_dynamicobj_risk_list[i].m_obj_id)
            {
                return BehaviorNodeLocal.m_risk_evaluation.m_dynamicobj_risk_list[i].m_obj_long_min_distance;
            }
        }
        return 999.0F;
    }
    else
    {
        return 999.0F;
    }
}

float BehaviorNodesAdd::PDLabelObjTD(PD::BehaviorNodeLocal BehaviorNodeLocal,
                                     uint32_t InterestObjID)
{
    if ((!BehaviorNodeLocal.m_risk_evaluation.m_dynamicobj_risk_list.empty()) && (InterestObjID != 0))
    {
        int ObjNum = int(BehaviorNodeLocal.m_risk_evaluation.m_dynamicobj_risk_list.size());
        float Chk_ObjTD = 999;
        for (int i = 0; i < ObjNum; i++)
        {
            if (InterestObjID == BehaviorNodeLocal.m_risk_evaluation.m_dynamicobj_risk_list[i].m_obj_id)
            {
                auto Chk_ObjTDAry = BehaviorNodeLocal.m_risk_evaluation.m_dynamicobj_risk_list[i].m_obj_long_clsn_td;
                if (!Chk_ObjTDAry.empty())
                {
                    for (int j = 0; j < Chk_ObjTDAry.size(); j++)
                    {
                        if (abs(Chk_ObjTD) > abs(Chk_ObjTDAry[j]) && Chk_ObjTDAry[j] != 999 && Chk_ObjTDAry[j] != 0)
                        {
                            Chk_ObjTD = Chk_ObjTDAry[j];
                        }
                    }
                    return Chk_ObjTD;
                }
                else
                {
                    return 999.0F;
                }
            }
        }
        return 999.0F;
    }
    else
    {
        return 999.0F;
    }
}
