#include "RoadModelUpdate.h"
#include "glog/logging.h"

#define PRINT(x) #x << " = " << x //用于打印信息

// float P_RMU_t_CycleTime = 0.1;                       // unit:s
float P_RMU_t_CycleTime = BEHAVIORANALYZERCYCLE / 1000.0; // unit:s modify to constant value
float P_RMU_t_LaneChangeWaitingTimeLimit = 15;            // unit:s original: 6
float P_RMU_t_LeftChange2LeftChangeInterval = 6;          // unit:s original: 5
float P_RMU_t_LeftChange2RightChangeInterval = 6;         // unit:s original: 5
float P_RMU_t_RightChange2RightChangeInterval = 6;        // unit:s original: 5
float P_RMU_t_RightChange2LeftChangeInterval = 6;         // unit:s original: 5
float P_RMU_t_LeftPrepare2RightPrepareInterval = 3;       // unit:s
float P_RMU_t_RightPrepare2LeftPrepareInterval = 3;       // unit:s
float P_RMU_t_LeftPrepare2RightChangeInterval = 3;        // unit:s
float P_RMU_t_RightPrepare2LeftChangeInterval = 3;        // unit:s

float P_RMU_t_LaneFollowActionDuration = 2;    // unit:s
float P_RMU_t_LaneChangeActionDuration = 6;    // unit:s
float P_RMU_t_ChangePrepareActionDuration = 2; // unit:s
float P_RMU_t_DefaultActionDuration = 2;       // unit:s

const int BA_StepNumInit = 0;

using namespace BA;
using namespace std;
using DM::DirectionIndicatorDuration;

//对RoadModelUpdate类里面的私有成员变量进行初始化，并调用Process函数
//对m_BA_RoadModelVector里面的节点根据行为节点发散规则进行发散
void RoadModelUpdate::Init(int BA_StepNum, vector<RoadInfo> *roadModelVector,
                           vector<BA::behaviorNodeGlobal> *behaviorNodeGlobalVector,
                           DM::DirectionIndicatorDuration dircnIndDur,
                           vector<BIP::HistoryBehavior> &BehaviorStackVector,
                           BA::LFCmfrtAvlblBhvr SM_LFComfortAvlblBehavior,
                           BA::LCAvlblBhvr SM_LCAvlblBehavior,
                           BA::LCPAvlblBhvr SM_LCPAvlblBehavior,
                           bool BA_LaneChangeAbolitionAvlbl, bool BA_VehInLnCenter,
                           float drvrSetSpd, float longtSpd,
                           vector<BA::BA_ActionTyp> &SupprActLst,
                           vector<DM::Passageob> &passageObLst,
                           bool BA_VehClsToLnCntr,
                           BA::BA_ActionTyp lastActionTyp2SM,
                           const BA::SeldDistLvl &drvrSetDstLvl,
                           const BA::AutoLCAvlblSts &autoLnChgAvlbl,
                           const BA::SignSpeedLimit &signSpdLmtInfo01,
                           const BA::SignSpeedLimit &signSpdLmtInfo02,
                           const bool &lastCycleSpdLmtFlg,
                           const BA::SpdLmtTyp &lastCycleSpdLmtTyp,
                           const uint8_t &lastCycleSpdLmtID,
                           const uint8_t &lcIntervalLvl,
                           const bool &lcLnIDnotChg,
                           BA::DriveMd &driveMode,
                           BA::ScenarioTyp &scenarioTyp)
{

  m_BA_StepNum = BA_StepNum;

  m_BA_RoadModelVector = roadModelVector;

  m_BA_BehaviorNodeGlobalVector = behaviorNodeGlobalVector;

  m_BA_DirectionIndicatorDuration = dircnIndDur;

  m_BA_HistoryBehaviorStackVector = BehaviorStackVector;

  m_BA_LFComfortAvlblBehavior = SM_LFComfortAvlblBehavior;

  m_BA_LCAvlblBehavior = SM_LCAvlblBehavior;

  m_BA_LCPAvlblBehavior = SM_LCPAvlblBehavior;

  m_BA_LaneChangeAbolitionAvlbl = BA_LaneChangeAbolitionAvlbl;

  m_BA_VehInLnCenterbl = BA_VehInLnCenter;

  m_DrvrSetSpd = drvrSetSpd;

  m_VehLongtSpd = longtSpd;

  m_BA_SuppressedActionList = SupprActLst;

  m_BA_VehClsToLnCntr = BA_VehClsToLnCntr;

  m_BA_LastActionTyp2SM = lastActionTyp2SM;

  m_BA_DrvrSetDstLvl = drvrSetDstLvl;

  m_BA_AutoLnChgAvlbl = autoLnChgAvlbl;

  m_BA_SignSpdLmtInfo01 = signSpdLmtInfo01;

  m_BA_SignSpdLmtInfo02 = signSpdLmtInfo02;

  m_BA_LastCycleSpdLmtFlg = lastCycleSpdLmtFlg;

  m_BA_LastCycleSpdLmtTyp = lastCycleSpdLmtTyp;

  m_BA_LastCycleSpdLmtID = lastCycleSpdLmtID;

  m_BA_DynLCIntervalLvl = lcIntervalLvl;

  m_BA_LcLnIDnotChg = lcLnIDnotChg;

  m_BA_DriveMd = driveMode;

  m_BA_ScenarioTyp = scenarioTyp;

  Process(passageObLst);
}

//处理包含三重循环，第一层：对RoadModelUpdate类里的m_BA_RoadModelVector内的每一个节点的
//RoadModel进行判断，获得对应节点道路模型的可用行为序列。第二层：对可用行为序列内的每一个行为，
//从节点道路模型中取得对应的横向主辅Corridor。第三层：将主辅Corridor传递给纵向目标筛选，相关
//逻辑会给每个Corridor添加上对应的纵向目标生成Passage，后续发给PD模块进行评估。
void RoadModelUpdate::Process(vector<DM::Passageob> &passageObLst)
{
  vector<BA_ActionTyp> _CurNodeLatAvlblBehavior{};

  vector<BA_ActionTyp> _CurNodeActionTypArr{};

  vector<float> _CurNodeDurationTimeArr{};

  BA::LongtAvlblBhvr _LongtAvlblBehavior = BA::LongtAvlblBhvr_ACC_Base;

  //根据节点ID从behaviorNodeGlobal中查找出对应的m_ActionTypArr和m_DurationTimeArr
  for (auto &BA_RoadModel : *m_BA_RoadModelVector)
  {
    auto BehaviorNodeGlobal_itr =
        find_if(m_BA_BehaviorNodeGlobalVector->begin(),
                m_BA_BehaviorNodeGlobalVector->end(),
                [&](BA::behaviorNodeGlobal &behavior_node)
                {
                  if (behavior_node.m_NodeID.m_X == BA_RoadModel.road_id.x &&
                      behavior_node.m_NodeID.m_Y == BA_RoadModel.road_id.y)
                  {
                    _CurNodeActionTypArr = behavior_node.m_ActionTypArr;

                    _CurNodeDurationTimeArr = behavior_node.m_DurationTimeArr;

                    return true;
                  }
                  else
                  {
                    return false;
                  }
                });
    //正常找到的话，调用LateralAvailableBehaviorProcess函数，获得对应节点道路模型的可用行为序列
    if (!m_BA_RoadModelVector->empty() &&
        BehaviorNodeGlobal_itr != m_BA_BehaviorNodeGlobalVector->end())
    {
      LateralAvailableBehaviorProcess(
          m_BA_StepNum, m_BA_HistoryBehaviorStackVector, _CurNodeActionTypArr,
          _CurNodeDurationTimeArr, m_BA_DirectionIndicatorDuration,
          m_BA_LFComfortAvlblBehavior, m_BA_LCAvlblBehavior,
          m_BA_LCPAvlblBehavior, m_BA_LaneChangeAbolitionAvlbl,
          m_BA_VehInLnCenterbl, _CurNodeLatAvlblBehavior,
          _LongtAvlblBehavior, m_BA_VehClsToLnCntr,
          this->m_BA_AutoLnChgAvlbl, this->m_BA_DynLCIntervalLvl);
      DLOG(INFO) << "Is vehicle in center: " << int(m_BA_VehInLnCenterbl) << std::endl;
      DLOG(INFO) << "Is vehicle close to lane center: " << int(m_BA_VehClsToLnCntr) << std::endl;
      DLOG(INFO) << "Is lane change abolition available: " << int(m_BA_LaneChangeAbolitionAvlbl) << std::endl;
      DLOG(INFO) << "Lc lane id not changed: " << int(m_BA_LcLnIDnotChg) << std::endl;
      int i = 0;
#ifdef GLOG
      for (auto curLatAvlblBhvr : _CurNodeLatAvlblBehavior)
      {
        i++;
        DLOG(INFO) << i << " bhvr is " << int(curLatAvlblBhvr) << std::endl;
      }
#endif

      ActionSuppressProcess(_CurNodeLatAvlblBehavior, this->m_BA_SuppressedActionList);

      //对可用行为序列内的每一个行为，从节点道路模型中取得对应的横向主辅Corridor。
      for (auto &Action_i : _CurNodeLatAvlblBehavior)
      {
        this->Corridors2Lon.clear();

        SelectCorridors(m_BA_StepNum, BA_RoadModel, Action_i,
                        this->Corridors2Lon, this->AuxiliaryCorridors, m_BA_LastActionTyp2SM);
        DLOG(INFO) << "ACTION TYPE: " << int(Action_i) << std::endl;
        DLOG(INFO) << "IS corridor2Lon empty: " << int(this->Corridors2Lon.empty()) << std::endl;

        //将主辅Corridor传递给纵向目标筛选，相关逻辑会给每个Corridor添加上对应
        //的纵向目标生成Passage，后续发给PD模块进行评估。
        OutputInfoAdd(Action_i, this->Corridors2Lon, BA_RoadModel,
                      BehaviorNodeGlobal_itr, /* this->AuxiliaryCorridors, */ passageObLst);
      }
    }
  }

  // reset Index_ when process() has finished this round
  this->Index_ = 0;
}

void RoadModelUpdate::ActionSuppressProcess(vector<BA::BA_ActionTyp> &CurNodeActionTypArr,
                                            vector<BA::BA_ActionTyp> &SupprActLst)
{
  if (!SupprActLst.empty())
  {
    for (auto &Action : SupprActLst)
    {
      auto it = find(CurNodeActionTypArr.begin(), CurNodeActionTypArr.end(), Action);
      if (it != CurNodeActionTypArr.end())
      {
        CurNodeActionTypArr.erase(it);
      }
    }
  }
  return;
}

//将历史行为序列和预测行为序列、历史行为序列时间和预测行为序列时间进行融合，存进一个vector
//后续用于在需要考虑历史行为的场景下，对不同的行为之间用规则进行约束
void PastandFutureActionDurationMerge(
    int BA_StepNum, const vector<BIP::HistoryBehavior> &BehaviorStackVector,
    const vector<BA_ActionTyp> &ActionTypArr,
    const vector<float> &DurationTimeArr,
    vector<BA_ActionTyp> &MergedActionTypArr,
    vector<float> &MergedDurationTimeArr)
{
  MergedActionTypArr = ActionTypArr;

  MergedDurationTimeArr = DurationTimeArr;

  for (int i = 0; i < BehaviorStackVector.size(); i++)
  {
    MergedActionTypArr.insert(MergedActionTypArr.begin(),
                              BehaviorStackVector.at(i).Action);

    MergedDurationTimeArr.insert(MergedDurationTimeArr.begin(),
                                 BehaviorStackVector.at(i).Duration);
  }

  return;
}

//主要用于判断某个行为在行为序列中是否满足时间的约束条件，
//当ConstActionTyp行为在行为序列ActionTypArr内发生的时间
//大于ConstraintLimitTime，则返回true，反之则返回false
bool BehaviorSequenceConstraint(vector<BA_ActionTyp> &ActionTypArr,
                                vector<float> &DurationTimeArr,
                                BA_ActionTyp ConstActionTyp,
                                float ConstraintLimitTime)
{
  bool action_break_condition_bl = false;

  bool time_break_condition_bl = false;

  int VctSize = ActionTypArr.size();

  float total_time = float(0);

  assert(ActionTypArr.size() == DurationTimeArr.size());

  for (int i = VctSize - 1; i >= 0; i--)
  {
    action_break_condition_bl = (ActionTypArr.at(i) == ConstActionTyp);

    time_break_condition_bl = (total_time >= ConstraintLimitTime);

    if (action_break_condition_bl || time_break_condition_bl)
    {
      break;
    }
    else
    {
      total_time += DurationTimeArr.at(i);
    }
  }

  if (action_break_condition_bl)
  {
    if (time_break_condition_bl)
    {
      return false;
    }
    else
    {
      return true;
    }
  }
  else
  {
    return false;
  }
}
//行为节点发散规则的核心逻辑代码，具体细节参考行为节点发散规则文档
void LateralAvailableBehaviorProcess(
    int BA_StepNum, vector<BIP::HistoryBehavior> BehaviorStackVector,
    vector<BA_ActionTyp> ActionTypArr, vector<float> DurationTimeArr,
    DirectionIndicatorDuration BA_DirectionIndicatorInfo,
    BA::LFCmfrtAvlblBhvr SM_LFComfortAvlblBehavior,
    BA::LCAvlblBhvr SM_LCAvlblBehavior, BA::LCPAvlblBhvr SM_LCPAvlblBehavior,
    bool BA_LaneChangeAbolitionAvlbl, bool BA_VehInLnCenter,
    vector<BA_ActionTyp> &BA_LatAvlblBehavior_Restricted,
    BA::LongtAvlblBhvr &BA_LongtAvlblBehavior,
    bool BA_IsVehClsToLnCntr,
    const BA::AutoLCAvlblSts &autoLnChgAvlbl,
    const uint8_t &lcIntervalLvl)
{
  BA_LongtAvlblBehavior = LongtAvlblBhvr_ACC_Base;

  BA_LatAvlblBehavior_Restricted.clear();

  bool lanefollowbl_LongtLat =
      (SM_LFComfortAvlblBehavior == LFCmfrtAvlblBhvr_LongtLat_HO ||
       SM_LFComfortAvlblBehavior == LFCmfrtAvlblBhvr_LongtLat_HOInclILC ||
       SM_LFComfortAvlblBehavior == LFCmfrtAvlblBhvr_LongtLat_HF ||
       SM_LFComfortAvlblBehavior == LFCmfrtAvlblBhvr_LatStandAlone);
  // SM_LFComfortAvlblBehavior == LFCmfrtAvlblBhvr_LatStandAlone added 20220327, longitudinal override

  bool LeftLaneChangebl =
      (SM_LCAvlblBehavior == LCAvlblBhvr_LeftLaneChange_C ||
       SM_LCAvlblBehavior == LCAvlblBhvr_LaneChange_Auto ||
       SM_LCAvlblBehavior == LCAvlblBhvr_LaneChange_SemiAuto ||
       (SM_LCAvlblBehavior == LCAvlblBhvr_LeftLaneChange_Auto) && ((autoLnChgAvlbl == BA::AutoLCAvlblSts_BothAvl) || (autoLnChgAvlbl == BA::AutoLCAvlblSts_LftAvl)));

  bool LeftLaneChangeAbolitionbl =
      (SM_LCAvlblBehavior == LCAvlblBhvr_LeftLaneChangeAbolition_C);

  bool LeftLaneChangePrepare_bl =
      (SM_LCPAvlblBehavior == LCPAvlblBhvr_LeftLaneChangePrepare ||
       (SM_LCPAvlblBehavior == LCPAvlblBhvr_LaneChangePrepare_Auto) && ((autoLnChgAvlbl == BA::AutoLCAvlblSts_BothAvl) || (autoLnChgAvlbl == BA::AutoLCAvlblSts_LftAvl)));

  bool RightLaneChangebl =
      (SM_LCAvlblBehavior == LCAvlblBhvr_RightLaneChange_C ||
       SM_LCAvlblBehavior == LCAvlblBhvr_LaneChange_Auto ||
       SM_LCAvlblBehavior == LCAvlblBhvr_LaneChange_SemiAuto ||
       (SM_LCAvlblBehavior == LCAvlblBhvr_RightLaneChange_Auto) && ((autoLnChgAvlbl == BA::AutoLCAvlblSts_BothAvl) || (autoLnChgAvlbl == BA::AutoLCAvlblSts_RghtAvl)));

  bool RightLaneChangeAbolitionbl =
      (SM_LCAvlblBehavior == LCAvlblBhvr_RightLaneChangeAbolition_C);

  bool RightLaneChangePrepare_bl =
      (SM_LCPAvlblBehavior == LCPAvlblBhvr_RightLaneChangePrepare ||
       SM_LCPAvlblBehavior == LCPAvlblBhvr_LaneChangePrepare_Auto) &&
      ((autoLnChgAvlbl == BA::AutoLCAvlblSts_BothAvl) || (autoLnChgAvlbl == BA::AutoLCAvlblSts_RghtAvl));

  bool AutoLaneChangePrepare_bl =
      ((SM_LCPAvlblBehavior == LCPAvlblBhvr_LaneChangePrepare_Auto) && ((autoLnChgAvlbl == BA::AutoLCAvlblSts_BothAvl)));

  /* static BA::LCAvlblBhvr lastLCAvlblBhvr = BA::LCAvlblBhvr_None;
  if (lastLCAvlblBhvr == BA::LCAvlblBhvr_LaneChange_SemiAuto && SM_LCAvlblBehavior != BA::LCAvlblBhvr_LaneChange_SemiAuto)
  {
    if (!BehaviorStackVector.empty())
    {
      BehaviorStackVector.at(0).Action = BA::BA_ActionTyp_LaneFollow;
    }
  } */

  if (lanefollowbl_LongtLat)
  {
    BA_ActionTyp CurrentActionProc = BehaviorStackVector.at(0).Action;
    DLOG(INFO) << "CurrentActionProc: " << int(CurrentActionProc) << std::endl;

    switch (CurrentActionProc)
    {
    case BA_ActionTyp_LeftLaneChange:

      if (BA_VehInLnCenter == false)
      {
        if (LeftLaneChangebl)
        {
          if (BA_LaneChangeAbolitionAvlbl == false)
          {
            if (BA_StepNum == BA_StepNumInit)
            {
              if (BehaviorStackVector[0].Action ==
                      BA_ActionTyp_LeftLaneChange &&
                  /* BehaviorStackVector[0].Duration <
                        P_RMU_t_LaneChangeActionDuration */
                  BA_IsVehClsToLnCntr == false)
              {
                BA_LatAvlblBehavior_Restricted.push_back(
                    BA_ActionTyp_LeftLaneChange);
              }
              else
              {
                BA_LatAvlblBehavior_Restricted.push_back(
                    BA_ActionTyp_LaneFollow);
              }
            }
            else
            {
              BA_LatAvlblBehavior_Restricted.push_back(
                  BA_ActionTyp_LaneFollow);
            }
          }
          else
          {
            if (BA_StepNum == BA_StepNumInit)
            {
              BA_LatAvlblBehavior_Restricted.push_back(
                  BA_ActionTyp_LeftLaneChange);

              BA_LatAvlblBehavior_Restricted.push_back(
                  BA_ActionTyp_LeftLaneChangeAbolition);
            }
            else
            {
              BA_LatAvlblBehavior_Restricted.push_back(
                  BA_ActionTyp_LaneFollow);
            }
          }
        }
        else if (LeftLaneChangeAbolitionbl)
        {
          if (BA_LaneChangeAbolitionAvlbl == false)
          {
            if (BA_StepNum == BA_StepNumInit)
            {
              if (BehaviorStackVector[0].Action ==
                      BA_ActionTyp_LeftLaneChange &&
                  /* BehaviorStackVector[0].Duration <
                        P_RMU_t_LaneChangeActionDuration */
                  BA_IsVehClsToLnCntr == false)
              {
                BA_LatAvlblBehavior_Restricted.push_back(
                    BA_ActionTyp_LeftLaneChange);
              }
              else
              {
                BA_LatAvlblBehavior_Restricted.push_back(
                    BA_ActionTyp_LaneFollow);
              }
            }
            else
            {
              BA_LatAvlblBehavior_Restricted.push_back(
                  BA_ActionTyp_LaneFollow);
            }
          }
          else
          {
            if (BA_StepNum == BA_StepNumInit)
            {
              BA_LatAvlblBehavior_Restricted.push_back(
                  BA_ActionTyp_LeftLaneChangeAbolition);
            }
            else
            {
              BA_LatAvlblBehavior_Restricted.push_back(
                  BA_ActionTyp_LaneFollow);
            }
          }
        }
        else
        {
          BA_LatAvlblBehavior_Restricted.push_back(BA_ActionTyp_LaneFollow);
        }
      }
      else
      {
        if (BA_LaneChangeAbolitionAvlbl == false)
        {
          BA_LatAvlblBehavior_Restricted.push_back(BA_ActionTyp_LaneFollow);
        }
        else
        {
          if (BA_StepNum == BA_StepNumInit)
          {
            BA_LatAvlblBehavior_Restricted.push_back(
                BA_ActionTyp_LeftLaneChange);

            BA_LatAvlblBehavior_Restricted.push_back(
                BA_ActionTyp_LeftLaneChangeAbolition);
          }
          else
          {
            BA_LatAvlblBehavior_Restricted.push_back(BA_ActionTyp_LaneFollow);
          }
        }
        // BA_LatAvlblBehavior_Restricted.push_back(BA_ActionTyp_LaneFollow); // comment 0425, it is actually not necessarily
      }
      break;

    case BA_ActionTyp_RightLaneChange:

      if (BA_VehInLnCenter == false)
      {
        if (RightLaneChangebl)
        {
          if (BA_LaneChangeAbolitionAvlbl == false)
          {
            if (BA_StepNum == BA_StepNumInit)
            {
              if (BehaviorStackVector[0].Action ==
                      BA_ActionTyp_RightLaneChange &&
                  /* BehaviorStackVector[0].Duration <
                        P_RMU_t_LaneChangeActionDuration */
                  BA_IsVehClsToLnCntr == false)
              {
                BA_LatAvlblBehavior_Restricted.push_back(
                    BA_ActionTyp_RightLaneChange);
              }
              else
              {
                BA_LatAvlblBehavior_Restricted.push_back(
                    BA_ActionTyp_LaneFollow);
              }
            }
            else
            {
              BA_LatAvlblBehavior_Restricted.push_back(
                  BA_ActionTyp_LaneFollow);
            }
          }
          else
          {
            if (BA_StepNum == BA_StepNumInit)
            {
              BA_LatAvlblBehavior_Restricted.push_back(
                  BA_ActionTyp_RightLaneChange);

              BA_LatAvlblBehavior_Restricted.push_back(
                  BA_ActionTyp_RightLaneChangeAbolition);
            }
            else
            {
              BA_LatAvlblBehavior_Restricted.push_back(
                  BA_ActionTyp_LaneFollow);
            }
          }
        }
        else if (RightLaneChangeAbolitionbl)
        {
          if (BA_LaneChangeAbolitionAvlbl == false)
          {
            if (BA_StepNum == BA_StepNumInit)
            {
              if (BehaviorStackVector[0].Action ==
                      BA_ActionTyp_RightLaneChange &&
                  /* BehaviorStackVector[0].Duration <
                        P_RMU_t_LaneChangeActionDuration */
                  BA_IsVehClsToLnCntr == false)
              {
                BA_LatAvlblBehavior_Restricted.push_back(
                    BA_ActionTyp_RightLaneChange);
              }
              else
              {
                BA_LatAvlblBehavior_Restricted.push_back(
                    BA_ActionTyp_LaneFollow);
              }
            }
            else
            {
              BA_LatAvlblBehavior_Restricted.push_back(
                  BA_ActionTyp_LaneFollow);
            }
          }
          else
          {
            if (BA_StepNum == BA_StepNumInit)
            {
              BA_LatAvlblBehavior_Restricted.push_back(
                  BA_ActionTyp_RightLaneChangeAbolition);
            }
            else
            {
              BA_LatAvlblBehavior_Restricted.push_back(
                  BA_ActionTyp_LaneFollow);
            }
          }
        }
        else
        {
          BA_LatAvlblBehavior_Restricted.push_back(BA_ActionTyp_LaneFollow);
        }
      }
      else
      {
        if (BA_LaneChangeAbolitionAvlbl == false)
        {
          BA_LatAvlblBehavior_Restricted.push_back(BA_ActionTyp_LaneFollow);
        }
        else
        {
          if (BA_StepNum == BA_StepNumInit)
          {
            BA_LatAvlblBehavior_Restricted.push_back(
                BA_ActionTyp_RightLaneChange);

            BA_LatAvlblBehavior_Restricted.push_back(
                BA_ActionTyp_RightLaneChangeAbolition);
          }
          else
          {
            BA_LatAvlblBehavior_Restricted.push_back(BA_ActionTyp_LaneFollow);
          }
        }
        if (BA_LaneChangeAbolitionAvlbl == false)
        {
          BA_LatAvlblBehavior_Restricted.push_back(BA_ActionTyp_LaneFollow);
        }
        else
        {
          if (BA_StepNum == BA_StepNumInit)
          {
            BA_LatAvlblBehavior_Restricted.push_back(
                BA_ActionTyp_RightLaneChange);

            BA_LatAvlblBehavior_Restricted.push_back(
                BA_ActionTyp_RightLaneChangeAbolition);
          }
          else
          {
            BA_LatAvlblBehavior_Restricted.push_back(BA_ActionTyp_LaneFollow);
          }
        }
      }
      break;

    case BA_ActionTyp_LeftLaneChangeAbolition:

      if (BA_VehInLnCenter == false)
      {
        if (LeftLaneChangeAbolitionbl)
        {
          if (BA_StepNum == BA_StepNumInit)
          {
            BA_LatAvlblBehavior_Restricted.push_back(
                BA_ActionTyp_LeftLaneChangeAbolition);
          }
          else
          {
            BA_LatAvlblBehavior_Restricted.push_back(BA_ActionTyp_LaneFollow);
          }
        }
        else
        {
          BA_LatAvlblBehavior_Restricted.push_back(BA_ActionTyp_LaneFollow);
        }
      }
      else
      {
        BA_LatAvlblBehavior_Restricted.push_back(BA_ActionTyp_LaneFollow);
      }
      break;
    case BA_ActionTyp_RightLaneChangeAbolition:

      if (BA_VehInLnCenter == false)
      {
        if (RightLaneChangeAbolitionbl)
        {
          if (BA_StepNum == BA_StepNumInit)
          {
            BA_LatAvlblBehavior_Restricted.push_back(
                BA_ActionTyp_RightLaneChangeAbolition);
          }
          else
          {
            BA_LatAvlblBehavior_Restricted.push_back(BA_ActionTyp_LaneFollow);
          }
        }
        else
        {
          BA_LatAvlblBehavior_Restricted.push_back(BA_ActionTyp_LaneFollow);
        }
      }
      else
      {
        BA_LatAvlblBehavior_Restricted.push_back(BA_ActionTyp_LaneFollow);
      }
      break;

    default:

      vector<BA_ActionTyp> MergedActionTypArr(ActionTypArr);

      vector<float> MergedDurationTimeArr(DurationTimeArr);

      if (SM_LCAvlblBehavior == LCAvlblBhvr_LeftLaneChange_C)
      {
        BA_LatAvlblBehavior_Restricted.push_back(BA_ActionTyp_LaneFollow);

        float timesum = 0;

        vector<BA_ActionTyp>::iterator it =
            find(ActionTypArr.begin(), ActionTypArr.end(),
                 BA_ActionTyp_LeftLaneChange);

        bool LeftLaneActionNotExist = (it == ActionTypArr.end());

        if (LeftLaneActionNotExist)
        {
          timesum = accumulate(
              DurationTimeArr.begin(), DurationTimeArr.end(),
              BA_DirectionIndicatorInfo.m_left_direction_indicator_duration);

          float LaneChangeWaitingRemainTime =
              P_RMU_t_LaneChangeWaitingTimeLimit - timesum;

          bool LaneChangeRemainTimeBool = (LaneChangeWaitingRemainTime >= 0);

          if (LaneChangeRemainTimeBool)
          {
            BA_LatAvlblBehavior_Restricted.push_back(
                BA_ActionTyp_LeftLaneChange);
          }

          bool LaneChangePrepareRemainTimeBool =
              ((LaneChangeWaitingRemainTime -
                P_RMU_t_ChangePrepareActionDuration) >= 0);

          if (LaneChangePrepareRemainTimeBool && LeftLaneChangePrepare_bl)
          {
            BA_LatAvlblBehavior_Restricted.push_back(
                BA_ActionTyp_LeftLaneChangePrepare);
          }
        }
      }

      else if (SM_LCAvlblBehavior == LCAvlblBhvr_LeftLaneChange_Auto)
      {
        float LaneChangeWaitingRemainTime =
            P_RMU_t_LaneChangeWaitingTimeLimit -
            BA_DirectionIndicatorInfo.m_left_direction_indicator_duration;
        bool LaneChangeRemainTimeBool = (LaneChangeWaitingRemainTime >= 0);

        if (LaneChangeRemainTimeBool)
        {
          if (BA_StepNum == BA_StepNumInit)
          {
            BA_LatAvlblBehavior_Restricted.push_back(BA_ActionTyp_LaneFollow);

            BA_LatAvlblBehavior_Restricted.push_back(
                BA_ActionTyp_LeftLaneChange);

            if (LeftLaneChangePrepare_bl)
            {
              float LaneChangeWaitingRemainTimeAfterPrepare =
                  LaneChangeWaitingRemainTime -
                  P_RMU_t_ChangePrepareActionDuration;

              if (LaneChangeWaitingRemainTimeAfterPrepare >= 0)
              {
                BA_LatAvlblBehavior_Restricted.push_back(
                    BA_ActionTyp_LeftLaneChangePrepare);
              }
            }
          }
          else
          {
            LatralBehaviorSequenceTimeConstraint(
                AutoLaneChangePrepare_bl, ActionTypArr, DurationTimeArr,
                BA_LatAvlblBehavior_Restricted, lcIntervalLvl);
          }
        }
        else
        {
          if (BA_StepNum == BA_StepNumInit)
          {
            BA_LatAvlblBehavior_Restricted.push_back(BA_ActionTyp_LaneFollow);
          }
          else
          {
            BA_LatAvlblBehavior_Restricted.push_back(BA_ActionTyp_LaneFollow);
          }
        }
      }

      else if (SM_LCAvlblBehavior == LCAvlblBhvr_RightLaneChange_C)
      {
        BA_LatAvlblBehavior_Restricted.push_back(BA_ActionTyp_LaneFollow);

        float timesum = 0;

        vector<BA_ActionTyp>::iterator it =
            find(ActionTypArr.begin(), ActionTypArr.end(),
                 BA_ActionTyp_RightLaneChange);

        bool RightLaneActionNotExist = (it == ActionTypArr.end());

        if (RightLaneActionNotExist)
        {
          timesum = accumulate(
              DurationTimeArr.begin(), DurationTimeArr.end(),
              BA_DirectionIndicatorInfo.m_right_direction_indicator_duration);

          float LaneChangeWaitingRemainTime =
              P_RMU_t_LaneChangeWaitingTimeLimit - timesum;

          bool LaneChangeRemainTimeBool = (LaneChangeWaitingRemainTime >= 0);
          if (LaneChangeRemainTimeBool)
          {
            BA_LatAvlblBehavior_Restricted.push_back(
                BA_ActionTyp_RightLaneChange);
          }

          bool LaneChangePrepareRemainTimeBool =
              ((LaneChangeWaitingRemainTime -
                P_RMU_t_ChangePrepareActionDuration) >= 0);

          if (LaneChangePrepareRemainTimeBool && RightLaneChangePrepare_bl)
          {
            BA_LatAvlblBehavior_Restricted.push_back(
                BA_ActionTyp_RightLaneChangePrepare);
          }
        }
      }

      else if (SM_LCAvlblBehavior == LCAvlblBhvr_RightLaneChange_Auto)
      {
        float LaneChangeWaitingRemainTime =
            P_RMU_t_LaneChangeWaitingTimeLimit -
            BA_DirectionIndicatorInfo.m_right_direction_indicator_duration;

        bool LaneChangeRemainTimeBool = (LaneChangeWaitingRemainTime >= 0);

        if (LaneChangeRemainTimeBool)
        {
          if (BA_StepNum == BA_StepNumInit)
          {
            BA_LatAvlblBehavior_Restricted.push_back(BA_ActionTyp_LaneFollow);

            BA_LatAvlblBehavior_Restricted.push_back(
                BA_ActionTyp_RightLaneChange);

            if (RightLaneChangePrepare_bl) // 20220704: change from AutoLaneChangePrepare_bl
            {
              float LaneChangeWaitingRemainTimeAfterPrepare =
                  LaneChangeWaitingRemainTime -
                  P_RMU_t_ChangePrepareActionDuration;
              // DLOG(INFO) << "LaneChangeWaitingRemainTime: " << LaneChangeWaitingRemainTime << std::endl;
              // DLOG(INFO) << "LaneChangeWaitingRemainTimeAfterPrepare: " << LaneChangeWaitingRemainTimeAfterPrepare << std::endl;
              if (LaneChangeWaitingRemainTimeAfterPrepare >= 0)
              {
                BA_LatAvlblBehavior_Restricted.push_back(
                    BA_ActionTyp_RightLaneChangePrepare);
              }
            }
            // DLOG(INFO) << "RightLaneChange_Auto BA_LatAvlblBehavior_Restricted size: " << BA_LatAvlblBehavior_Restricted.size() << std::endl;
          }
          else
          {
            LatralBehaviorSequenceTimeConstraint(
                AutoLaneChangePrepare_bl, ActionTypArr, DurationTimeArr,
                BA_LatAvlblBehavior_Restricted, lcIntervalLvl);
          }
        }
        else
        {
          if (BA_StepNum == BA_StepNumInit)
          {
            BA_LatAvlblBehavior_Restricted.push_back(BA_ActionTyp_LaneFollow);
          }
          else
          {
            BA_LatAvlblBehavior_Restricted.push_back(BA_ActionTyp_LaneFollow);
          }
        }
      }

      else if ((SM_LCAvlblBehavior == LCAvlblBhvr_LaneChange_Auto || SM_LCAvlblBehavior == LCAvlblBhvr_LaneChange_SemiAuto))
      {
        PastandFutureActionDurationMerge(
            BA_StepNum, BehaviorStackVector, ActionTypArr, DurationTimeArr,
            MergedActionTypArr, MergedDurationTimeArr);
        /* int actionCnt = 0;
        for (auto action : ActionTypArr)
        {
          actionCnt++;
          DLOG(INFO) << actionCnt << "action type = " << int(action) << std::endl; 
        }
        int mrgActionCnt = 0;
        for (auto mrgAction : MergedActionTypArr)
        {
          mrgActionCnt++;
          DLOG(INFO) << mrgActionCnt << "merge action type = " << int(mrgAction) << std::endl; 
        }
        DLOG(INFO) << "~~~~~~~~~" << std::endl; */
        LatralBehaviorSequenceTimeConstraint(
            AutoLaneChangePrepare_bl, MergedActionTypArr,
            MergedDurationTimeArr, BA_LatAvlblBehavior_Restricted,
            lcIntervalLvl);
      }

      else
      {
        BA_LatAvlblBehavior_Restricted.push_back(BA_ActionTyp_LaneFollow);
      }

      break;
    }
  }
  else if (SM_LFComfortAvlblBehavior == LFCmfrtAvlblBhvr_None)
  {
    BA_LatAvlblBehavior_Restricted.clear();
  }
  else if (SM_LFComfortAvlblBehavior == LFCmfrtAvlblBhvr_LongtStandAlone)
  {
    BA_LatAvlblBehavior_Restricted.push_back(BA_ActionTyp_LaneFollow);
  }
  else if (SM_LFComfortAvlblBehavior == LFCmfrtAvlblBhvr_LatStandAlone)
  {
    BA_LatAvlblBehavior_Restricted.push_back(BA_ActionTyp_LaneFollow);
  }
  else
  {
    BA_LatAvlblBehavior_Restricted.clear();
  }

  if (!BA_LatAvlblBehavior_Restricted.empty())
  {
    sort(BA_LatAvlblBehavior_Restricted.begin(),
         BA_LatAvlblBehavior_Restricted.end());

    BA_LatAvlblBehavior_Restricted.erase(
        unique(BA_LatAvlblBehavior_Restricted.begin(),
               BA_LatAvlblBehavior_Restricted.end()),
        BA_LatAvlblBehavior_Restricted.end());
  }
  // 20221019:
  // SET current LCAvlblBhvr to lastLCAvlblBhvr
  // lastLCAvlblBhvr = SM_LCAvlblBehavior;
}
//该模块主要用于实现如下逻辑，将满足约束条件的行为添加进BA_LatAvlblBehavior_Restricted容器内
/* SM变道输入为LCAvlblBhvr_LaneChange_Auto时：
  SM_LCPAvlblBehavior为LCPAvlblBhvr_LaneChangePrepare_Auto时：
    行为列表包含BA_ActionTyp_LaneFollow行为；
    行为列表中候补行为包含BA_ActionTyp_RightLaneChangePrepare、BA_ActionTyp_LeftLaneChangePrepare、
    BA_ActionTyp_RightLaneChange、BA_ActionTyp_LeftLaneChange行为，不同的行为之间应该满足如下规则约束：
      LeftLaneChange(先)和LeftLaneChange（后）行为的时间间隔应大于等于P_RMU_t_LeftChange2LeftChangeInterval；
      LeftLaneChange(先)和RightLaneChange（后）行为的时间间隔应大于等于P_RMU_t_LeftChange2RightChangeInterval；
      RightLaneChange(先)和RightLaneChange（后）行为的时间间隔应大于等于P_RMU_t_RightChange2RightChangeInterval；
      RightLaneChange(先)和LeftLaneChange（后）行为的时间间隔应大于等于P_RMU_t_RightChange2LeftChangeInterval；
      LeftLaneChangePrepare(先)和RightLaneChangePrepare（后）行为的时间间隔应大于等于P_RMU_t_LeftPrepare2RightPrepareInterval；
      RightLaneChangePrepare(先)和LeftLaneChangePrepare（后）行为的时间间隔应大于等于P_RMU_t_RightPrepare2LeftPrepareInterval；
      LeftLaneChangePrepare(先)和RightLaneChange（后）行为的时间间隔应大于等于P_RMU_t_LeftPrepare2RightChangeInterval；
      RightLaneChangePrepare(先)和LeftLaneChange（后）行为的时间间隔应大于等于P_RMU_t_RightPrepare2LeftChangeInterval；
      注：如上规则需考虑历史行为序列，例如：历史发生过变道行为，当前若想变道则需满足一定时间间隔约束；
  SM_LCPAvlblBehavior为其他时：
    行为列表包含BA_ActionTyp_LaneFollow行为；
    行为列表中候补行为包含BA_ActionTyp_RightLaneChange、BA_ActionTyp_LeftLaneChange行为，不同的行为之间应该满足如下规则约束：
      LeftLaneChange(先)和LeftLaneChange（后）行为的时间间隔应大于等于P_RMU_t_LeftChange2LeftChangeInterval；
      LeftLaneChange(先)和RightLaneChange（后）行为的时间间隔应大于等于P_RMU_t_LeftChange2RightChangeInterval；
      RightLaneChange(先)和RightLaneChange（后）行为的时间间隔应大于等于P_RMU_t_RightChange2RightChangeInterval；
      RightLaneChange(先)和LeftLaneChange（后）行为的时间间隔应大于等于P_RMU_t_RightChange2LeftChangeInterval；
      注：如上规则需考虑历史行为序列，例如：历史发生过变道行为，当前若想变道则需满足一定时间间隔约束； */
void LatralBehaviorSequenceTimeConstraint(
    bool AutoLaneChangePrepare_bl, vector<BA_ActionTyp> &ActionTypArr,
    vector<float> &DurationTimeArr,
    vector<BA_ActionTyp> &BA_LatAvlblBehavior_Restricted,
    const uint8_t &dynLCIntervalLvl)
{
  // 20220419: add dynamic lane change interval switch
  float lCIntervalL2L = 0.0f;
  float lCIntervalL2R = 0.0f;
  float lCIntervalR2R = 0.0f;
  float lCIntervalR2L = 0.0f;
  switch (dynLCIntervalLvl)
  {
  case 0:
    lCIntervalL2L = P_RMU_t_LeftChange2LeftChangeInterval;
    lCIntervalL2R = P_RMU_t_LeftChange2RightChangeInterval;
    lCIntervalR2R = P_RMU_t_RightChange2RightChangeInterval;
    lCIntervalR2L = P_RMU_t_RightChange2LeftChangeInterval;
    break;
  case 1:
    lCIntervalL2L = ADC::common::k_BA_LCIntervalLvl01;
    lCIntervalL2R = ADC::common::k_BA_LCIntervalLvl01;
    lCIntervalR2R = ADC::common::k_BA_LCIntervalLvl01;
    lCIntervalR2L = ADC::common::k_BA_LCIntervalLvl01;
  }
  bool LeftChange2LeftChangeConstraint = BehaviorSequenceConstraint(
      ActionTypArr, DurationTimeArr, BA_ActionTyp_LeftLaneChange,
      lCIntervalL2L);

  bool LeftChange2RightChangeConstraint = BehaviorSequenceConstraint(
      ActionTypArr, DurationTimeArr, BA_ActionTyp_LeftLaneChange,
      lCIntervalL2R);

  bool RightChange2RightChangeConstraint = BehaviorSequenceConstraint(
      ActionTypArr, DurationTimeArr, BA_ActionTyp_RightLaneChange,
      lCIntervalR2R);

  bool RightChange2LeftChangeConstraint = BehaviorSequenceConstraint(
      ActionTypArr, DurationTimeArr, BA_ActionTyp_RightLaneChange,
      lCIntervalR2L);

  if (AutoLaneChangePrepare_bl)
  {
    bool LeftPrepare2RightPrepareConstraint = BehaviorSequenceConstraint(
        ActionTypArr, DurationTimeArr, BA_ActionTyp_LeftLaneChangePrepare,
        P_RMU_t_LeftPrepare2RightPrepareInterval);

    bool RightPrepare2LeftPrepareConstraint = BehaviorSequenceConstraint(
        ActionTypArr, DurationTimeArr, BA_ActionTyp_RightLaneChangePrepare,
        P_RMU_t_RightPrepare2LeftPrepareInterval);

    bool LeftPrepare2RightChangeConstraint = BehaviorSequenceConstraint(
        ActionTypArr, DurationTimeArr, BA_ActionTyp_LeftLaneChangePrepare,
        P_RMU_t_LeftPrepare2RightChangeInterval);

    bool RightPrepare2LeftChangeConstraint = BehaviorSequenceConstraint(
        ActionTypArr, DurationTimeArr, BA_ActionTyp_RightLaneChangePrepare,
        P_RMU_t_RightPrepare2LeftChangeInterval);

    BA_LatAvlblBehavior_Restricted.push_back(BA_ActionTyp_LaneFollow);

    /* if (!LeftPrepare2RightPrepareConstraint)
    {
      BA_LatAvlblBehavior_Restricted.push_back(
          BA_ActionTyp_RightLaneChangePrepare);
    }

    if (!RightPrepare2LeftPrepareConstraint)
    {
      BA_LatAvlblBehavior_Restricted.push_back(
          BA_ActionTyp_LeftLaneChangePrepare);
    }

    if (!LeftPrepare2RightChangeConstraint &&
        !LeftChange2RightChangeConstraint &&
        !RightChange2RightChangeConstraint)
    {
      BA_LatAvlblBehavior_Restricted.push_back(BA_ActionTyp_RightLaneChange);
    }

    if (!RightPrepare2LeftChangeConstraint &&
        !RightChange2LeftChangeConstraint && !LeftChange2LeftChangeConstraint)
    {
      BA_LatAvlblBehavior_Restricted.push_back(BA_ActionTyp_LeftLaneChange);
    } */

    // 20220616: allow LCP when LC is available
    if (!LeftPrepare2RightChangeConstraint &&
        !LeftChange2RightChangeConstraint &&
        !RightChange2RightChangeConstraint)
    {
      BA_LatAvlblBehavior_Restricted.push_back(BA_ActionTyp_RightLaneChange);
      if (!LeftPrepare2RightPrepareConstraint)
      {
        BA_LatAvlblBehavior_Restricted.push_back(
            BA_ActionTyp_RightLaneChangePrepare);
      }
    }

    if (!RightPrepare2LeftChangeConstraint &&
        !RightChange2LeftChangeConstraint && !LeftChange2LeftChangeConstraint)
    {
      BA_LatAvlblBehavior_Restricted.push_back(BA_ActionTyp_LeftLaneChange);
      if (!RightPrepare2LeftPrepareConstraint)
      {
        BA_LatAvlblBehavior_Restricted.push_back(
            BA_ActionTyp_LeftLaneChangePrepare);
      }
    }
  }
  else
  {
    /* DLOG(INFO) << "RightChange2LeftChangeConstraint: " << int(RightChange2LeftChangeConstraint) << std::endl;
    DLOG(INFO) << "LeftChange2LeftChangeConstraint: " << int(LeftChange2LeftChangeConstraint) << std::endl;
    DLOG(INFO) << "LeftChange2RightChangeConstraint: " << int(LeftChange2RightChangeConstraint) << std::endl;
    DLOG(INFO) << "RightChange2RightChangeConstraint: " << int(RightChange2RightChangeConstraint) << std::endl; */
    BA_LatAvlblBehavior_Restricted.push_back(BA_ActionTyp_LaneFollow);

    if (!RightChange2LeftChangeConstraint && !LeftChange2LeftChangeConstraint)
    {
      BA_LatAvlblBehavior_Restricted.push_back(BA_ActionTyp_LeftLaneChange);
    }

    if (!LeftChange2RightChangeConstraint && !RightChange2RightChangeConstraint)
    {
      BA_LatAvlblBehavior_Restricted.push_back(BA_ActionTyp_RightLaneChange);
    }
  }

  return;
}

//根据车道ID LaneIdSearch从道路模型中查找到对应Lane的迭代器iterator
std::vector<Lane>::const_iterator GetLaneIterator(
    const RoadInfo &CurNodeRoadInfo, uint8_t LaneIdSearch)
{
  auto it = find_if(CurNodeRoadInfo.lane_list.begin(),
                    CurNodeRoadInfo.lane_list.end(),
                    [&](const Lane &lane)
                    {
                      if (lane.start_id == LaneIdSearch)
                      {
                        return true;
                      }
                      else
                      {
                        return false;
                      }
                    });

  return it;
}

//根据当前行为，从节点道路模型中获取对应的横向主辅Corridor
void GetLaneOfAction(int StepNum, BA_ActionTyp NodeLatBehavior,
                     uint8_t CurrentlaneId,
                     RoadInfo &CurNodeRoadInfo,
                     bool VehInLnCenter, bool BA_LaneChangeAbolitionAvlbl,
                     uint8_t &TargetlaneId, uint8_t &AuxiliarylaneId,
                     bool &TargetlaneFindResult, bool &AuxiliarylaneFindResult,
                     bool &LCLnIDnotChgFlg)
{
  static uint8_t _history_line_id = CurrentlaneId;

  bool result = false;

  //当车辆居中时，用CurrentlaneId更新_history_line_id
  if (VehInLnCenter)
  {
    _history_line_id = CurrentlaneId;
  }

  //根据当前车道ID即CurrentLaneId，从道路模型CurNodeRoadInfo中获取相临左/右车道的ID
  //并将结果传递给NeighborLaneId，return查找的结果，找到时返回true，反之亦然
  //RightOrLeftSide为1时表示获取右侧，为0时表示获取左侧
  auto GetNeighborLaneID = [&](const RoadInfo &CurNodeRoadInfo,
                               uint8_t CurrentLaneId, uint8_t &NeighborLaneId,
                               bool RightOrLeftSide)
  {
    bool result = false;

    std::vector<Lane>::const_iterator it;

    it = GetLaneIterator(CurNodeRoadInfo, CurrentLaneId);

    if (!CurNodeRoadInfo.lane_list.empty() &&
        it != CurNodeRoadInfo.lane_list.end())
    {
      std::vector<NeighborLane> NeighborLaneList;

      if (!RightOrLeftSide)
      {
        NeighborLaneList = it->left_neighbor_lane;
      }
      else
      {
        NeighborLaneList = it->right_neighbor_lane;
      }

      if (!NeighborLaneList.empty())
      {
        // 20220330: select the first one by default
        // NeighborLaneId = NeighborLaneList.at(0).lane_id;
        // 20220417: default neightbor lane id is set to 0
        NeighborLaneId = uint8_t(0);
        // 20220330: select the lane with start offset is less than 0
        /* for (auto neighborLn_ : NeighborLaneList)
        {
          if (neighborLn_.start_range < 0)
          {
            NeighborLaneId = neighborLn_.lane_id; // replace default lane_id
            break;
          }
        } */
        float min_st_rng = 1000.0f;
        uint8_t min_st_rng_ln_id = uint8_t(0);
        for (auto neighborLn_ : NeighborLaneList)
        {
          // get the neighbor lane with samllest start range
          if (neighborLn_.start_range < min_st_rng)
          {
            min_st_rng_ln_id = neighborLn_.lane_id;
            min_st_rng = neighborLn_.start_range;
          }
        }
        if (min_st_rng < ADC::common::k_BA_MinNgbrLnStRng)
        {
          NeighborLaneId = min_st_rng_ln_id;
        }

        if (NeighborLaneId == uint8_t(0))
        {
          // there is no neighbor lane with negative start offset
          result = false;
        }
        else
        {
          result = true;
        }
      }
      else
      {
        NeighborLaneId = 0;
        result = false;
      }
    }
    else
    {
      NeighborLaneId = CurrentlaneId;

      result = false;
    }

    return result;
  };

  //StepNum为初始步时，要根据当前车辆正在执行的行为以及车辆是否跨线来选取对应的主辅Lane ID。
  //当当前行为为BA_ActionTyp_LeftLaneChange时，当变道取消可用（未跨线），主Lane为当前车道左侧Lane，
  //辅Lane为当前车道；当变道取消不可用时（已跨线），主Lane为当前车道，辅Lane为当前车道右侧Lane，
  //BA_ActionTyp_RightLaneChange行为同理。
  //当当前行为为BA_ActionTyp_LeftLaneChangePrepare时，主Lane为当前车道，辅Lane为当前车道左侧Lane，
  //BA_ActionTyp_LeftLaneChangeAbolition、及右侧对应Prepare及Abolition行为同理
  //当当前行为为BA_ActionTyp_LaneFollow时，主Lane为当前车道，辅Lane为当前车道，
  //BA_ActionTyp_ILC行为同理。

  //StepNum不为初始步时，同样需要根据当前车辆正在执行的行为以及车辆是否跨线来选取对应的主辅Lane ID。
  //逻辑与上述逻辑相似，区别在于LeftLaneChange行为，不用判断变道取消是否可用（是否跨线），都默认未跨线
  if (StepNum == BA_StepNumInit)
  {
    switch (NodeLatBehavior)
    {
    case BA_ActionTyp_LeftLaneChange:
      // if (BA_LaneChangeAbolitionAvlbl)
      if (LCLnIDnotChgFlg)
      {
        TargetlaneFindResult = GetNeighborLaneID(CurNodeRoadInfo, CurrentlaneId,
                                                 TargetlaneId, 0);

        AuxiliarylaneId = CurrentlaneId;
        AuxiliarylaneFindResult = true;
      }
      else
      {
        TargetlaneId = CurrentlaneId;
        TargetlaneFindResult = true;

        AuxiliarylaneFindResult = GetNeighborLaneID(CurNodeRoadInfo, CurrentlaneId,
                                                    AuxiliarylaneId, 1);
      }
      break;
    case BA_ActionTyp_RightLaneChange:
      // if (BA_LaneChangeAbolitionAvlbl)
      if (LCLnIDnotChgFlg)
      {
        TargetlaneFindResult = GetNeighborLaneID(CurNodeRoadInfo, CurrentlaneId,
                                                 TargetlaneId, 1);

        AuxiliarylaneId = CurrentlaneId;
        AuxiliarylaneFindResult = true;
      }
      else
      {
        TargetlaneId = CurrentlaneId;
        TargetlaneFindResult = true;

        AuxiliarylaneFindResult = GetNeighborLaneID(CurNodeRoadInfo, CurrentlaneId,
                                                    AuxiliarylaneId, 0);
      }
      break;
    case BA_ActionTyp_LeftLaneChangePrepare:
    case BA_ActionTyp_LeftLaneChangeAbolition:
      // if (BA_LaneChangeAbolitionAvlbl)
      if (LCLnIDnotChgFlg)
      {
        AuxiliarylaneFindResult = GetNeighborLaneID(CurNodeRoadInfo, CurrentlaneId,
                                                    AuxiliarylaneId, 0);

        TargetlaneId = CurrentlaneId;
        TargetlaneFindResult = true;
      }
      else
      {
        AuxiliarylaneId = CurrentlaneId;
        AuxiliarylaneFindResult = true;

        TargetlaneFindResult = GetNeighborLaneID(CurNodeRoadInfo, CurrentlaneId,
                                                 TargetlaneId, 1);
      }
      break;
    case BA_ActionTyp_RightLaneChangePrepare:
    case BA_ActionTyp_RightLaneChangeAbolition:
      // if (BA_LaneChangeAbolitionAvlbl)
      if (LCLnIDnotChgFlg)
      {
        AuxiliarylaneFindResult = GetNeighborLaneID(CurNodeRoadInfo, CurrentlaneId,
                                                    AuxiliarylaneId, 1);

        TargetlaneId = CurrentlaneId;
        TargetlaneFindResult = true;
      }
      else
      {
        AuxiliarylaneId = CurrentlaneId;
        AuxiliarylaneFindResult = true;

        TargetlaneFindResult = GetNeighborLaneID(CurNodeRoadInfo, CurrentlaneId,
                                                 TargetlaneId, 0);
      }
      break;
    case BA_ActionTyp_LaneFollow:
    case BA_ActionTyp_ILC:
      TargetlaneId = CurrentlaneId;
      TargetlaneFindResult = true;

      AuxiliarylaneId = CurrentlaneId;
      AuxiliarylaneFindResult = false;
      break;
    default:
      TargetlaneId = _history_line_id;
      TargetlaneFindResult = false;

      AuxiliarylaneId = CurrentlaneId;
      AuxiliarylaneFindResult = false;
      break;
    }
  }
  else
  {
    switch (NodeLatBehavior)
    {
    case BA_ActionTyp_LeftLaneChange:
      TargetlaneFindResult =
          GetNeighborLaneID(CurNodeRoadInfo, CurrentlaneId, TargetlaneId, 0);

      AuxiliarylaneId = CurrentlaneId;
      AuxiliarylaneFindResult = true;
      break;
    case BA_ActionTyp_RightLaneChange:
      result =
          GetNeighborLaneID(CurNodeRoadInfo, CurrentlaneId, TargetlaneId, 1);

      AuxiliarylaneId = CurrentlaneId;
      AuxiliarylaneFindResult = true;
      break;
    case BA_ActionTyp_LeftLaneChangePrepare:
    case BA_ActionTyp_LeftLaneChangeAbolition:
      // if (BA_LaneChangeAbolitionAvlbl)
      if (LCLnIDnotChgFlg)
      {
        AuxiliarylaneFindResult = GetNeighborLaneID(CurNodeRoadInfo, CurrentlaneId,
                                                    AuxiliarylaneId, 0);

        TargetlaneId = CurrentlaneId;
        TargetlaneFindResult = true;
      }
      else
      {
        AuxiliarylaneId = CurrentlaneId;
        AuxiliarylaneFindResult = true;

        TargetlaneFindResult = GetNeighborLaneID(CurNodeRoadInfo, CurrentlaneId,
                                                 TargetlaneId, 1);
      }
      break;
    case BA_ActionTyp_RightLaneChangePrepare:
    case BA_ActionTyp_RightLaneChangeAbolition:
      // if (BA_LaneChangeAbolitionAvlbl)
      if (LCLnIDnotChgFlg)
      {
        AuxiliarylaneFindResult = GetNeighborLaneID(CurNodeRoadInfo, CurrentlaneId,
                                                    AuxiliarylaneId, 1);

        TargetlaneId = CurrentlaneId;
        TargetlaneFindResult = true;
      }
      else
      {
        AuxiliarylaneId = CurrentlaneId;
        AuxiliarylaneFindResult = true;

        TargetlaneFindResult = GetNeighborLaneID(CurNodeRoadInfo, CurrentlaneId,
                                                 TargetlaneId, 0);
      }
      break;
    case BA_ActionTyp_LaneFollow:
    case BA_ActionTyp_ILC:
      TargetlaneId = CurrentlaneId;
      TargetlaneFindResult = true;

      AuxiliarylaneId = CurrentlaneId;
      AuxiliarylaneFindResult = false;
      break;
    default:
      TargetlaneId = _history_line_id;
      TargetlaneFindResult = false;

      AuxiliarylaneId = CurrentlaneId;
      AuxiliarylaneFindResult = false;
      break;
    }
  }
}

//根据action调用GetLaneOfAction函数，获得对应的主辅Lane，再选择对应的主辅Corridor
//辅Corridor的选取原则是根据辅助Lane的ID，从中查找第一个feature_tag为LaneFollow的Corridor
//主Corridor的选取原则，首先调用JudgeTrafficRules函数，判断当前Lane是否满足交通规则，
//（该部分逻辑根据需求已注释）
//当m_BA_LFComfortAvlblBehavior为LongtLat_HOInclILC时，将feature_tag为ILC的Corridor
//添加进主_Corridor2Lon；当corridor_list里包含feature_tag为CarFollow的Corridor，
//不包含feature_tag为LaneFollow的Corridor时，将该corridor添加进主_Corridor2Lon；
//feature_tag为LaneFollow的Corridor，都会被添加进主_Corridor2Lon。
//相应Corridor在添加进主_Corridor2Lon前，会调用JudgeODD函数判断该Corridor是否满足相应的
//ODD约束，然后再考虑是否添加进去（该部分逻辑根据需求已注释）
void RoadModelUpdate::SelectCorridors(
    int StepNum, RoadInfo &CurNodeRoadInfo,
    BA::BA_ActionTyp _Action_i,
    std::vector<Corridor> &_Corridor2Lon,
    std::vector<Corridor> &AuxiliaryCorridor,
    const BA::BA_ActionTyp &_LastActionTyp2SM)
{
  // This module needs to be modified to cooperate wiht RM's Macro definition
  const uint8_t _C_LaneFollow_Corridor = 1;
  const uint8_t _C_CarFollow_Corridor = 2;
  const uint8_t _C_ILC_Corridor = 3;
  const uint8_t _C_LaneExtend_Corridor = 5;

  // keep current lane id stable, when last action is lane change abolition
  // in case that ego lane id changed during lane change abolition
  uint8_t _current_lane_id = uint8_t(0);
  static uint8_t internCurLnID = uint8_t(0);
  if (_LastActionTyp2SM == BA::BA_ActionTyp_LeftLaneChangeAbolition || _LastActionTyp2SM == BA::BA_ActionTyp_RightLaneChangeAbolition)
  {
    _current_lane_id = internCurLnID;
    // 20220507: if original lane is not found in current road model, then replace it with ego lane id
    auto it_curLn = GetLaneIterator(CurNodeRoadInfo, _current_lane_id);
    if (it_curLn == CurNodeRoadInfo.lane_list.end())
    {
      // current lane id is not found
      _current_lane_id = CurNodeRoadInfo.car_position.ego_lane_id;
    }
  }
  else
  {
    _current_lane_id = CurNodeRoadInfo.car_position.ego_lane_id;
    internCurLnID = _current_lane_id;
  }
  // std::cout << "_current_lane_id: " << int(_current_lane_id) << std::endl;
  // std::cout << "ego_lane_id: " << int(CurNodeRoadInfo.car_position.ego_lane_id) << std::endl;

  uint8_t _target_lane_id = CurNodeRoadInfo.car_position.ego_lane_id;

  uint8_t _auxiliary_lane_id = CurNodeRoadInfo.car_position.ego_lane_id;

  std::vector<Lane>::const_iterator target_lane_iterator;
  std::vector<Lane>::const_iterator auxiliary_lane_iterator;

  bool target_lane_find_result = false;
  bool auxiliary_lane_find_result = false;

  bool _ILC_Corridor_bl =
      (m_BA_LFComfortAvlblBehavior == LFCmfrtAvlblBhvr_LongtLat_HOInclILC);

  GetLaneOfAction(
      StepNum, _Action_i, _current_lane_id, CurNodeRoadInfo,
      m_BA_VehInLnCenterbl, m_BA_LaneChangeAbolitionAvlbl,
      _target_lane_id, _auxiliary_lane_id,
      target_lane_find_result, auxiliary_lane_find_result,
      this->m_BA_LcLnIDnotChg);

  DLOG(INFO) << "target lane id = " << int(_target_lane_id) << std::endl;
  DLOG(INFO) << "target lane find res = " << int(target_lane_find_result) << std::endl;
  DLOG(INFO) << "auxiliar lane id = " << int(_auxiliary_lane_id) << std::endl;
  DLOG(INFO) << "auxiliar lane find res = " << int(auxiliary_lane_find_result) << std::endl;

  auxiliary_lane_iterator = GetLaneIterator(CurNodeRoadInfo, _auxiliary_lane_id);

  if (auxiliary_lane_find_result && auxiliary_lane_iterator != CurNodeRoadInfo.lane_list.end())
  {
    auto &_lane = *auxiliary_lane_iterator;
    bool LaneFollow_Corridor_find_result = false;

    if (!_lane.corridor_list.empty())
    {
      auto it =
          find_if(_lane.corridor_list.begin(), _lane.corridor_list.end(),
                  [](const Corridor &corridor)
                  {
                    if (corridor.feature_tag == _C_LaneFollow_Corridor)
                    {
                      return true;
                    }
                    else
                    {
                      return false;
                    }
                  });

      if (!_lane.corridor_list.empty() && it != _lane.corridor_list.end())
      {
        LaneFollow_Corridor_find_result = true;
        AuxiliaryCorridor.clear();
        AuxiliaryCorridor.push_back(*it);
      }
      else
      {
        LaneFollow_Corridor_find_result = false;
        AuxiliaryCorridor.clear();
      }
    }
  }

  /*
  updater in 20220801
  0=Unknown
  1=RegularLane
  2=DrivaleShoulderLane 
  3=SlowLane
  4=CarpoolLane
  5=EmergencyLane
  6=BothDirectionLane
  7=BusLane
  8=DrivableParkingLane
  9=AcclerationLane
  10=DecelerationLane
  11=DIVERSION
  12=EmergencyParkingBar
  13=VirtualLane
  14=Non-motorizedVehicleLane
  15=VIRTUAL_EXTEND_LANE
  16=ENTRY
  17=EXIT,
  18=ONRAMP,
  19=OFFRAMP,
  20=CONNECTINGRAMP,
  21=EMERGENCY_PARKING_STRIP,
  22=Normal
  Others=Reserved*/
  _Corridor2Lon.clear();

  target_lane_iterator = GetLaneIterator(CurNodeRoadInfo, _target_lane_id);

  //根据需求去除应急车道
  bool _NotEmergencyLane = false;

  if (target_lane_iterator != CurNodeRoadInfo.lane_list.end() &&
      _Action_i != BA_ActionTyp_None)
  {
    if (target_lane_iterator->lane_type != 5 && target_lane_iterator->lane_type != 21 && target_lane_iterator->lane_type != 11)
    {
      _NotEmergencyLane = true;
    }
    else
    {
      _NotEmergencyLane = false;
    }
  }

  if (target_lane_find_result && _NotEmergencyLane)
  {
    auto &_lane = *target_lane_iterator;

    bool _Result_JudgeTrafficRules = true;
    //因为当前根据PD模块需求，不需要对当前行为进行交通规则判定，故将_Result_JudgeTrafficRules置true，
    //并对相关代码进行注释。
    // _Result_JudgeTrafficRules =
    //     JudgeTrafficRules(StepNum, CurNodeRoadInfo, _lane, _Action_i);

    if (_Result_JudgeTrafficRules)
    {
      bool LaneFollow_Corridor_find_result = false;

      if (!_lane.corridor_list.empty())
      {
        auto it =
            find_if(_lane.corridor_list.begin(), _lane.corridor_list.end(),
                    [](const Corridor &corridor)
                    {
                      if (corridor.feature_tag == _C_LaneFollow_Corridor)
                      {
                        return true;
                      }
                      else
                      {
                        return false;
                      }
                    });

        if (!_lane.corridor_list.empty() && it != _lane.corridor_list.end())
        {
          LaneFollow_Corridor_find_result = true;
        }
        else
        {
          LaneFollow_Corridor_find_result = false;
        }
      }

      bool CarFollow_Corridor_find_result = false;

      if (ADC::common::k_BA_InclLaneExtCrrd == true && !_lane.corridor_list.empty())
      {
        auto it =
            find_if(_lane.corridor_list.begin(), _lane.corridor_list.end(),
                    [](const Corridor &corridor)
                    {
                      if (corridor.feature_tag == _C_CarFollow_Corridor)
                      {
                        return true;
                      }
                      else
                      {
                        return false;
                      }
                    });

        if (!_lane.corridor_list.empty() && it != _lane.corridor_list.end())
        {
          CarFollow_Corridor_find_result = true;
        }
        else
        {
          CarFollow_Corridor_find_result = false;
        }
      }

      bool _push_back_trigger = false;

      bool _Result_JudgeODD = true;

      DLOG(INFO) << "corridor list size in lane_: " << _lane.corridor_list.size() << std::endl;

      uint32_t LFCrrdID = uint32_t(0);
      // get line follow corridor id
      for (auto _corridor : _lane.corridor_list)
      {
        if (_corridor.feature_tag == _C_LaneFollow_Corridor)
        {
          LFCrrdID = _corridor.corridor_id;
          break;
        }
      }
      DLOG(INFO) << "LF Corridor id = " << int(LFCrrdID) << std::endl;

      for (auto corridor_i : _lane.corridor_list)
      {
        _push_back_trigger = false;
        DLOG(INFO) << "---corridor feature tag: " << int(corridor_i.feature_tag) << std::endl;
        DLOG(INFO) << " --corridor id: " << int(corridor_i.corridor_id) << std::endl;
        DLOG(INFO) << " --_ILC_Corridor_bl: " << int(_ILC_Corridor_bl) << std::endl;
        // 20220512:
        // only the corridor with ILC feature tag and its corridor id equals to x multiplying 10000 plus deviation level, x denotes line follow corridor id
        switch (corridor_i.feature_tag)
        {
        case _C_ILC_Corridor:
          if (_ILC_Corridor_bl)
          {
            // ONLY ILC corridor with 0.2m deviation will be selected
            if (LFCrrdID != uint32_t(0))
            {
              uint32_t trgtILCCrrdID = LFCrrdID * 10000 + 2;
              // DLOG(INFO) << "Target ILC Corridor id = " << int(trgtILCCrrdID) << std::endl;
              if (corridor_i.corridor_id == trgtILCCrrdID)
              {
                // DLOG(INFO) << "enter" << std::endl;
                _push_back_trigger = true;
              }
              else
              {
                _push_back_trigger = false;
              }
            }
            else
            {
              _push_back_trigger = false;
            }
          }
          else
          {
            _push_back_trigger = false;
          }
          break;
        case _C_LaneFollow_Corridor:
          _push_back_trigger = true;
          break;
        case _C_CarFollow_Corridor:
          if (!LaneFollow_Corridor_find_result)
            _push_back_trigger = true;
          else
            _push_back_trigger = false;
          break;
        case _C_LaneExtend_Corridor:
          if (!CarFollow_Corridor_find_result && !LaneFollow_Corridor_find_result)
          {
            _push_back_trigger = true;
          }
          else
          {
            _push_back_trigger = false;
          }
          break;
        default:
          _push_back_trigger = false;
          break;
        }

        if (_push_back_trigger == true)
        {
          //根据当前需求，不需要对相应的corridor进行ODD判定，故_Result_JudgeODD置true，
          //并对ODD判定的代码进行了注释
          // if (corridor_i.feature_tag != _C_CarFollow_Corridor) {
          //   _Result_JudgeODD =
          //       JudgeODD(StepNum, CurNodeRoadInfo, corridor_i, _Action_i);
          // }

          if (_Result_JudgeODD)
          {
            _Corridor2Lon.push_back(corridor_i);
          }
        }
      }
    }
    else
    {
      _Corridor2Lon.clear();
    }

    return;
  }
}
//对Corridor进行ODD判断，初始步不进行判断，具体逻辑由SM模块实现，后续步骤进行简要的
//ODD判断，判断Corridor的drivable_width是否满足debounce约束、当行为为LaneChange
//时判断start_range、c2是否满足一定条件，具体根据需求来实现判断逻辑（可参考SM的相关逻辑）
bool RoadModelUpdate::JudgeODD(int StepNum,
                               RoadInfo &CurNodeRoadInfo,
                               Corridor &Corridor,
                               BA::BA_ActionTyp _Action_i)
{
  if (StepNum != BA_StepNumInit)
  {
    bool _result = false;

    float _drivable_width = Corridor.drivable_width;

    bool _drivable_width_judge =
        (_drivable_width <= 4.8 && _drivable_width >= 2.8);

    if (_Action_i == BA_ActionTyp_LeftLaneChange ||
        _Action_i == BA_ActionTyp_RightLaneChange)
    {
      LineGeometry _LineGeometry = Corridor.reference_line;

      bool _line_param_judge = (_LineGeometry.start_range <= 5 &&
                                _LineGeometry.c2 <= ((1 / 100) / 2));

      if (_drivable_width_judge && _line_param_judge)
      {
        _result = true;
      }
      else
      {
        _result = false;
      }
    }
    else
    {
      if (_drivable_width_judge)
      {
        _result = true;
      }
      else
      {
        _result = false;
      }
    }

    return _result;
  }
  else
  {
    return true;
  }
}

//车道的交通规则判断逻辑，根据具体的变道行为，选择对应的lane_marker，判断对应的lane_marker_id
//能否在lane_marker_list中找打以及变道过程中会压实线
bool RoadModelUpdate::JudgeTrafficRules(int StepNum,
                                        RoadInfo &CurNodeRoadInfo,
                                        const Lane &Lane,
                                        BA::BA_ActionTyp _Action_i)
{
  // JudgeTrafficRules() is focused on lane line type, the cost of
  // traffic rules disobey will be calced by pd, so it is bypassed
  // here
  const int function_switch = 1;

  if (function_switch)
  {
    /* if (StepNum != BA_StepNumInit)
    {
      uint32_t _target_lane_boundary_id = 0;
      bool _targer_lanemarker_result = false;

      bool _result = false;

      if (_Action_i == BA_ActionTyp_LeftLaneChange)
      {
        if (!Lane.right_lane_marker_id_list.empty())
        {
          _target_lane_boundary_id = Lane.right_lane_marker_id_list.at(0);
          _targer_lanemarker_result = true;
        }
      }
      else if (_Action_i == BA_ActionTyp_RightLaneChange)
      {
        if (!Lane.left_lane_marker_id_list.empty())
          _target_lane_boundary_id = Lane.left_lane_marker_id_list.at(0);
        _targer_lanemarker_result = true;
      }
      else
      {
        _result = true;
        _targer_lanemarker_result = false;
      }

      if (_targer_lanemarker_result)
      {
        // this module may need to be modified to coorperate with PD
        auto it = find_if(
            CurNodeRoadInfo.lane_marker_list.at(0).front_lane_boundary_list.begin(),
            CurNodeRoadInfo.lane_marker_list.at(0).front_lane_boundary_list.end(),
            [&_target_lane_boundary_id](
                mpc_drv_msgs::msg::LaneBoundary &lane_boundary)
            {
              bool TrafficRulesResult =
                  !(lane_boundary.line_type ==
                        mpc_drv_msgs::msg::LaneBoundary::LINETYPE_DASHED ||
                    lane_boundary.line_type ==
                        mpc_drv_msgs::msg::LaneBoundary::LINETYPE_DOUBLEDASHEDDASHED ||
                    lane_boundary.line_type ==
                        mpc_drv_msgs::msg::LaneBoundary::LINETYPE_DOUBLEDASHEDSOLID);

              if (lane_boundary.line_boundary_id == _target_lane_boundary_id &&
                  TrafficRulesResult)
                return true;
              else
                return false;
            });

        if (!CurNodeRoadInfo.lane_marker_list.at(0).front_lane_boundary_list.empty() &&
            it != CurNodeRoadInfo.lane_marker_list.at(0).front_lane_boundary_list.end())
        {
          _result = true;
        }
        else
        {
          _result = false;
        }
      }
      else
      {
        _result = false;
      }

      return _result;
    }
    else
    {
      return true;
    }
  }
  else
  { */
    return true;
  }
}

//更新行为的duration时间，不同的行为有着不同duration的标定值，
//当StepNum为初始步时,且行为为LaneChange行为，duration时间则为
//标定参数减去LaneChange行为的持续时间，并进行最大值最小值限幅，
//最后调用my_LgtCrdorGen.LgtCrdorGenProcess函数对所有的Corridors添加纵向目标。
void RoadModelUpdate::OutputInfoAdd(
    BA::BA_ActionTyp Action_i, vector<Corridor> &Corridors2Lon,
    RoadInfo &RoadModel,
    std::vector<BA::behaviorNodeGlobal>::iterator &BehaviorNodeGlobalIterator,
    /* std::vector<Corridor> &AuxiliaryCorridors, */
    vector<DM::Passageob> &passageObLst)
{
  //暂时根据Action类型来确定during时间
  if (m_BA_StepNum == BA_StepNumInit)
  {
    if (Action_i == BA::BA_ActionTyp_LaneFollow)
    {
      duration = P_RMU_t_LaneFollowActionDuration;
    }
    else if (Action_i == BA::BA_ActionTyp_LeftLaneChange ||
             Action_i == BA::BA_ActionTyp_RightLaneChange)
    {
      if (m_BA_HistoryBehaviorStackVector.at(0).Action == Action_i)
      {
        duration = P_RMU_t_LaneChangeActionDuration -
                   m_BA_HistoryBehaviorStackVector.at(0).Duration;

        duration = max(float(2), duration); // 0503: min dur tm of LC 0-->2

        duration = min(P_RMU_t_LaneChangeActionDuration, duration);
      }
      else
      {
        duration = P_RMU_t_LaneChangeActionDuration;
      }
    }
    else if (Action_i == BA::BA_ActionTyp_LeftLaneChangePrepare ||
             Action_i == BA::BA_ActionTyp_RightLaneChangePrepare)
    {
      duration = P_RMU_t_ChangePrepareActionDuration;
    }
    else
    {
      duration = P_RMU_t_DefaultActionDuration;
    }
  }
  else
  {
    if (Action_i == BA::BA_ActionTyp_LaneFollow)
    {
      duration = P_RMU_t_LaneFollowActionDuration;
    }
    else if (Action_i == BA::BA_ActionTyp_LeftLaneChange ||
             Action_i == BA::BA_ActionTyp_RightLaneChange)
    {
      duration = P_RMU_t_LaneChangeActionDuration;
    }
    else if (Action_i == BA::BA_ActionTyp_LeftLaneChangePrepare ||
             Action_i == BA::BA_ActionTyp_RightLaneChangePrepare)
    {
      duration = P_RMU_t_ChangePrepareActionDuration;
    }
    else
    {
      duration = P_RMU_t_DefaultActionDuration;
    }
  }
  DLOG(INFO) << "Corridor2Lon size = " << Corridors2Lon.size() << std::endl;
  LgtFcn::LgtCrdorGen my_LgtCrdorGen;
  if (ADC::common::k_RMU_LCPUseILCCorridor)
  {
    const uint8_t _C_LaneFollow_Corridor = 1;
    const uint8_t _C_CarFollow_Corridor = 2;
    const uint8_t _C_ILC_Corridor = 3;
    const uint8_t _C_LaneExtend_Corridor = 5;

    const auto iter_ilc = find_if(Corridors2Lon.begin(), Corridors2Lon.end(), [](const Corridor &crd)
                                  { return crd.feature_tag == _C_ILC_Corridor; });
    const auto iter_lanefollow = find_if(Corridors2Lon.begin(), Corridors2Lon.end(), [](const Corridor &crd)
                                         { return crd.feature_tag == _C_LaneFollow_Corridor; });

    if ((iter_ilc != Corridors2Lon.end() &&
         iter_lanefollow != Corridors2Lon.end()) &&
        (Action_i == BA::BA_ActionTyp_LeftLaneChangePrepare ||
         Action_i == BA::BA_ActionTyp_RightLaneChangePrepare))
    {

      Corridor ilc_coridor = *iter_ilc;
      ilc_coridor.object_in_corridor_list = iter_lanefollow->object_in_corridor_list;
      my_LgtCrdorGen.LgtCrdorGenProcess(
          duration, Action_i, ilc_coridor, m_DrvrSetSpd, this->Index_, RoadModel,
          m_BA_StepNum, m_VehLongtSpd, m_BA_LaneChangeAbolitionAvlbl,
          BehaviorNodeGlobalIterator, this->AuxiliaryCorridors, passageObLst, this->m_BA_DrvrSetDstLvl,
          this->m_BA_SignSpdLmtInfo01, this->m_BA_SignSpdLmtInfo02, this->m_BA_LastCycleSpdLmtFlg, this->m_BA_LastCycleSpdLmtTyp, this->m_BA_LastCycleSpdLmtID,
          this->m_BA_DriveMd,this->m_BA_ScenarioTyp);
    }  
    else
    {
      for (auto corridor_i : Corridors2Lon)
      {

        my_LgtCrdorGen.LgtCrdorGenProcess(
            duration, Action_i, corridor_i, m_DrvrSetSpd, this->Index_, RoadModel,
            m_BA_StepNum, m_VehLongtSpd, m_BA_LaneChangeAbolitionAvlbl,
            BehaviorNodeGlobalIterator, this->AuxiliaryCorridors, passageObLst, this->m_BA_DrvrSetDstLvl,
            this->m_BA_SignSpdLmtInfo01, this->m_BA_SignSpdLmtInfo02, this->m_BA_LastCycleSpdLmtFlg, this->m_BA_LastCycleSpdLmtTyp, this->m_BA_LastCycleSpdLmtID,
            this->m_BA_DriveMd,this->m_BA_ScenarioTyp);
      }
    }
  }
  else
  {
    for (auto corridor_i : Corridors2Lon)
    {

      // print corridor info below
      DLOG(INFO) << "### action type: " << int(Action_i) << std::endl;
      DLOG(INFO) << "### corridor info: " << std::endl;
      DLOG(INFO) << "  corridor id = " << int(corridor_i.corridor_id) << std::endl;
      int objCnt = 0;
#ifdef GLOG
      for (auto obj : corridor_i.object_in_corridor_list)
      {
        objCnt++;
        DLOG(INFO) << "  " << objCnt << " obj id = " << int(obj.object_id) << std::endl;
        DLOG(INFO) << "  " << objCnt << " obj rel x = " << obj.relative_center_x << std::endl;
        DLOG(INFO) << "  " << objCnt << " obj rel y = " << obj.relative_center_y << std::endl;
      }
#endif

      my_LgtCrdorGen.LgtCrdorGenProcess(
          duration, Action_i, corridor_i, m_DrvrSetSpd, this->Index_, RoadModel,
          m_BA_StepNum, m_VehLongtSpd, m_BA_LaneChangeAbolitionAvlbl,
          BehaviorNodeGlobalIterator, this->AuxiliaryCorridors, passageObLst, this->m_BA_DrvrSetDstLvl,
          this->m_BA_SignSpdLmtInfo01, this->m_BA_SignSpdLmtInfo02, this->m_BA_LastCycleSpdLmtFlg, this->m_BA_LastCycleSpdLmtTyp, this->m_BA_LastCycleSpdLmtID,
          this->m_BA_DriveMd,this->m_BA_ScenarioTyp);
    }
  }
}
