// #ifndef ROAD_MODEL_UPDATE_H_
// #define ROAD_MODEL_UPDATE_H_
#pragma once
#include <vector>
// #include "ContractHeader_common/CommonStructType.h"
#include "../include/CtApBehaviorAnalysis_/BA_Enum_List.h"
#include "../include/CtApBehaviorAnalysis_/common_BA.h"
#include "longitudinal_corridor_generate.h"
// #include "mpc_drv_msgs/msg/direction_indicator_duration.hpp"
// #include "mpc_drv_msgs/msg/road_info.hpp"
#include <numeric>
#include "../include/CtApBehaviorAnalysis_/behaviorNodeGlobal.h"
#include "BA_BaseInfoProcess.h"
#include "BA_CurrentBehaviorJudgement.h"
#include "assert.h"
// #include "mpc_drv_msgs/msg/corridor.hpp"
// #include "mpc_drv_msgs/msg/passageob.hpp"
#include "../commonfunc.h"
#include "../include/CtApBehaviorAnalysis_/interface/dm_internal_interface.h"
// #include "../include/CtApBehaviorAnalysis_/interface/rm_road_info.h"
#include "dpc/impl_type_roadinfo.h"


extern float P_RMU_t_CycleTime;                         // unit:s
extern float P_RMU_t_LaneChangeWaitingTimeLimit;        // unit:s
extern float P_RMU_t_LeftChange2LeftChangeInterval;     // unit:s
extern float P_RMU_t_LeftChange2RightChangeInterval;    // unit:s
extern float P_RMU_t_RightChange2RightChangeInterval;   // unit:s
extern float P_RMU_t_RightChange2LeftChangeInterval;    // unit:s
extern float P_RMU_t_LeftPrepare2RightPrepareInterval;  // unit:s
extern float P_RMU_t_RightPrepare2LeftPrepareInterval;  // unit:s
extern float P_RMU_t_LeftPrepare2RightChangeInterval;   // unit:s
extern float P_RMU_t_RightPrepare2LeftChangeInterval;   // unit:s

extern float P_RMU_t_LaneFollowActionDuration;     // unit:s
extern float P_RMU_t_LaneChangeActionDuration;     // unit:s
extern float P_RMU_t_ChangePrepareActionDuration;  // unit:s
extern float P_RMU_t_DefaultActionDuration;        // unit:s

void PastandFutureActionDurationMerge(
    int BA_StepNum, const vector<BIP::HistoryBehavior> &BehaviorStackVector,
    const vector<BA::BA_ActionTyp> &ActionTypArr,
    const vector<float> &DurationTimeArr,
    vector<BA::BA_ActionTyp> &MergedActionTypArr,
    vector<float> &MergedDurationTimeArr);

bool BehaviorSequenceConstraint(vector<BA::BA_ActionTyp> &ActionTypArr,
                                vector<float> &DurationTimeArr,
                                BA::BA_ActionTyp ConstActionTyp,
                                float ConstraintLimitTime);

void LateralAvailableBehaviorProcess(
    int BA_StepNum, vector<BIP::HistoryBehavior> BehaviorStackVector,
    vector<BA::BA_ActionTyp> ActionTypArr, vector<float> DurationTimeArr,
    DM::DirectionIndicatorDuration BA_DirectionIndicatorInfo,
    BA::LFCmfrtAvlblBhvr SM_LFComfortAvlblBehavior,
    BA::LCAvlblBhvr SM_LCAvlblBehavior, BA::LCPAvlblBhvr SM_LCPAvlblBehavior,
    bool BA_LaneChangeAbolitionAvlbl, bool BA_VehInLnCenter,
    vector<BA::BA_ActionTyp> &BA_LatAvlblBehavior_Restricted,
    BA::LongtAvlblBhvr &BA_LongtAvlblBehavior,
    bool BA_IsVehClsToLnCntr,
    const BA::AutoLCAvlblSts &autoLnChgAvlbl,
    const uint8_t &lcIntervalLvl);

void LatralBehaviorSequenceTimeConstraint(
    bool AutoLaneChangePrepare_bl, vector<BA::BA_ActionTyp> &ActionTypArr,
    vector<float> &DurationTimeArr,
    vector<BA::BA_ActionTyp> &BA_LatAvlblBehavior_Restricted,
    const uint8_t &dynLCIntervalLvl);

class RoadModelUpdate {
 public:
  RoadModelUpdate() = default;

  void Init(int BA_StepNum, vector<RoadInfo> *roadModelVector,
            vector<BA::behaviorNodeGlobal> *behaviorNodeGlobalVector,
            DM::DirectionIndicatorDuration dircnIndDur,
            vector<BIP::HistoryBehavior> &BehaviorStackVector,
            BA::LFCmfrtAvlblBhvr SM_LFComfortAvlblBehavior,
            BA::LCAvlblBhvr SM_LCAvlblBehavior,
            BA::LCPAvlblBhvr SM_LCPAvlblBehavior,
            bool BA_LaneChangeAbolitionAvlbl, bool BA_VehInLnCenter,
            float drvrSetSpd, float longtSpd,
            vector<BA::BA_ActionTyp> &SupprActLst,
            vector<DM::Passageob> &passageObLst,
            bool BA_VehClsToLnCntr,
            BA::BA_ActionTyp lastActionTyp2SM,
            const BA::SeldDistLvl &drvrSetDstLvl,
            const BA::AutoLCAvlblSts &autoLnChgAvlbl,
            const BA::SignSpeedLimit &signSpdLmtInfo01,
            const BA::SignSpeedLimit &signSpdLmtInfo02,
            const bool &lastCycleSpdLmtFlg,
            const BA::SpdLmtTyp &lastCycleSpdLmtTyp,
            const uint8_t &lastCycleSpdLmtID,
            const uint8_t &lcIntervalLvl,
            const bool &lcLnIDnotChg,
            BA::DriveMd &driveMode,
            BA::ScenarioTyp &scenarioTyp);

  void Process(vector<DM::Passageob> &passageObLst);

 private:
  //横向Corridors生成
  void LateralCorridorGenerator();

  //根据action选择对应的Corridors
  void SelectCorridors(int StepNum, RoadInfo &CurNodeRoadInfo,
                       BA::BA_ActionTyp _Action_i,
                       std::vector<Corridor> &_Corridor2Lon,
                       std::vector<Corridor> &AuxiliaryCorridor,
                       const BA::BA_ActionTyp &_LastActionTyp2SM);

  //根据action排序规则判断action是否合理
  // void JudgeAction();

  //根据交通规则及物理约束判断是否合理
  bool JudgeTrafficRules(int StepNum, RoadInfo &CurNodeRoadInfo,
                         const Lane &Lane,
                         BA::BA_ActionTyp _Action_i);

  //根据交通规则及物理约束判断是否合理
  bool JudgeODD(int StepNum, RoadInfo &CurNodeRoadInfo,
                Corridor &Corridor, BA::BA_ActionTyp _Action_i);

  //输出时增加持续时间和编号等信息  纵向的corridor的生成可以作为该函数的子函数
  void OutputInfoAdd(BA::BA_ActionTyp Action_i,
                     vector<Corridor> &Corridors2Lon,
                     RoadInfo &RoadModel,
                     std::vector<BA::behaviorNodeGlobal>::iterator
                         &BehaviorNodeGlobalIterator,
                     /* std::vector<Corridor> &AuxiliaryCorridors, */
                     vector<DM::Passageob> &passageObLst);

  void ActionSuppressProcess(vector<BA::BA_ActionTyp> &CurNodeActionTypArr,  
                            vector<BA::BA_ActionTyp> &SupprActLst);                   

  std::vector<Corridor>
      Corridors2Lon;  //给下游Lon Corridor generation的corridors

  std::vector<Corridor>
      AuxiliaryCorridors;  //给下游Lon Corridor generation的corridors
  float duration = 0;  //持续时间

  int Index_ = 0;  //用于Passage ID (层数, index)生成

  int m_BA_StepNum = 0;  //当前行为节点是第几层

  std::vector<RoadInfo> *m_BA_RoadModelVector = NULL;

  std::vector<BA::behaviorNodeGlobal> *m_BA_BehaviorNodeGlobalVector = NULL;

  DM::DirectionIndicatorDuration m_BA_DirectionIndicatorDuration;

  vector<BIP::HistoryBehavior> m_BA_HistoryBehaviorStackVector;

  BA::LFCmfrtAvlblBhvr m_BA_LFComfortAvlblBehavior;

  BA::LCAvlblBhvr m_BA_LCAvlblBehavior;

  BA::LCPAvlblBhvr m_BA_LCPAvlblBehavior;

  bool m_BA_LaneChangeAbolitionAvlbl;

  bool m_BA_VehInLnCenterbl;

  float m_DrvrSetSpd = 0;  //设定速度,用于发给下游Lon Corridor

  float m_VehLongtSpd = 0;

  std::vector<BA::BA_ActionTyp> m_BA_SuppressedActionList;

  bool m_BA_VehClsToLnCntr;

  BA::BA_ActionTyp m_BA_LastActionTyp2SM;

  BA::SeldDistLvl m_BA_DrvrSetDstLvl;

  BA::AutoLCAvlblSts m_BA_AutoLnChgAvlbl = BA::AutoLCAvlblSts_BothAvl;

  BA::SignSpeedLimit m_BA_SignSpdLmtInfo01;

  BA::SignSpeedLimit m_BA_SignSpdLmtInfo02;

  bool m_BA_LastCycleSpdLmtFlg;

  BA::SpdLmtTyp m_BA_LastCycleSpdLmtTyp;

  uint8_t m_BA_LastCycleSpdLmtID;

  uint8_t m_BA_DynLCIntervalLvl;

  bool m_BA_LcLnIDnotChg;

  BA::DriveMd m_BA_DriveMd;

  BA::ScenarioTyp m_BA_ScenarioTyp;
};

// #endif