#include "corridor_generation.h"
#include "../include/CtApBehaviorAnalysis_/behaviorNodeGlobal.h"
#include "glog/logging.h"

namespace ADC
{
    namespace planning
    {
        CorridorGeneration::CorridorGeneration(RoadInfo &RM_RoadModel,
                                               uint32_t BA_OptCorridorID,
                                               BA::BA_ActionTyp BA_OptAction,
                                               float VSE_VehSpd,
                                               uint8_t BA_TrgtLaneStartID,
                                               uint8_t BA_TrgtLaneEndID,
                                               bool BA_IsLCAbltAvlbl,
                                               BA::Index BA_OptPassageID,
                                               BA::BA_ActionTyp BA_OptAction_LastCycle,
                                               std::vector<Passage> BA_FirstPassageLst,
                                               const BA::BA_ActionTyp &BA_Action4TRPLat,
                                               const uint32_t &BA_Corridor4TRPLat)
        {
            this->roadModel = &RM_RoadModel;
            this->optCorridorID = BA_OptCorridorID;
            this->optAction = BA_OptAction;
            this->vehSpd = VSE_VehSpd;
            this->trgtLaneStartID = BA_TrgtLaneStartID;
            this->trgtLaneEndID = BA_TrgtLaneEndID;
            this->isLCAbltAvlbl = BA_IsLCAbltAvlbl;
            this->optPassageID = BA_OptPassageID;
            this->optAction_LastCycle = BA_OptAction_LastCycle;
            this->passageList = BA_FirstPassageLst;
            this->optAction4TRPLat = BA_Action4TRPLat;
            this->optCorridorID4TRPLat = BA_Corridor4TRPLat;
        }

        float CorridorGeneration::CalcDesCrrdrLen(float vehicleSpeed, float crrdrRngEnd)
        {
            float rawDesCrrdrLen = c_BA_ms2kmh * vehicleSpeed;
            rawDesCrrdrLen = min(rawDesCrrdrLen, k_BA_CrrdrLenUpThr);
            rawDesCrrdrLen = max(rawDesCrrdrLen, k_BA_CrrdrLenLowThr);
            rawDesCrrdrLen = min(rawDesCrrdrLen, crrdrRngEnd);
            return rawDesCrrdrLen;
        }

        float CorridorGeneration::CalcLatDvtn(float dist, LineGeometry line)
        {
            return line.c0 + line.c1 * dist + line.c2 * dist * dist + line.c3 * dist * dist * dist;
        }

        bool CorridorGeneration::FindLane(RoadInfo &roadModel,
                                          const uint8_t laneStartID,
                                          const uint8_t laneEndID,
                                          Lane &lane)
        {
            uint8_t lane_num = roadModel.lane_list.size();
            bool foundFlg = false;
            for (uint8_t i = 0; i < lane_num; i++)
            {
                if (roadModel.lane_list[i].start_id == laneStartID && roadModel.lane_list[i].end_id == laneEndID)
                {
                    lane = roadModel.lane_list[i];
                    foundFlg = true;
                    break;
                }
            }
            return foundFlg;
        }

        bool CorridorGeneration::FindLane(RoadInfo &roadModel,
                                          const uint8_t laneID,
                                          std::vector<Lane> &laneLst)
        {
            uint8_t lane_num = roadModel.lane_list.size();
            for (uint8_t i = 0; i < lane_num; i++)
            {
                if (roadModel.lane_list[i].start_id == laneID)
                {
                    laneLst.emplace_back(roadModel.lane_list[i]);
                }
            }
            if (!laneLst.empty())
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        bool CorridorGeneration::FindCorridor(Lane &lane,
                                              const uint32_t corridorID,
                                              Corridor &corridorObj)
        {
            uint8_t corridor_num = lane.corridor_list.size();
            bool foundFlg = false;
            for (uint8_t i = 0; i < corridor_num; i++)
            {
                if (lane.corridor_list[i].corridor_id == corridorID)
                {
                    corridorObj = lane.corridor_list[i];
                    foundFlg = true;
                    break;
                }
            }
            return foundFlg;
        }

        bool CorridorGeneration::FindPassage(const RoadInfo &roadModel,
                                             const BA::Index passageID,
                                             Passage &passageObj)
        {
            uint8_t passage_num = roadModel.passage_list.size();
            bool foundFlg = false;
            for (uint8_t i = 0; i < passage_num; i++)
            {
                if (passageID.m_Y == roadModel.passage_list[i].passage_id.y)
                {
                    passageObj = roadModel.passage_list[i];
                    foundFlg = true;
                    break;
                }
            }
            return foundFlg;
        }

        bool CorridorGeneration::FindPassage(const std::vector<Passage> &passageLst,
                                             const BA::Index passageID,
                                             Passage &passageObj)
        {
            uint8_t passage_num = passageLst.size();
            bool foundFlg = false;
            for (uint8_t i = 0; i < passage_num; i++)
            {
                if (passageID.m_Y == passageLst[i].passage_id.y)
                {
                    passageObj = passageLst[i];
                    foundFlg = true;
                    break;
                }
            }
            return foundFlg;
        }

        bool CorridorGeneration::FindLaneBdry(const std::vector<LaneMarker> &laneMrkrLst,
                                              const BA::BoundaryIndex &laneBdryIndx,
                                              LaneBoundary &laneBdryObj)
        {
            int laneMrkrNum = laneMrkrLst.size();
            bool foundFlg = false;
            for (int k = 0; k < laneMrkrNum; k++)
            {
                if (laneMrkrLst.at(k).id == laneBdryIndx.m_LaneMarkerID)
                {
                    int laneBdryNum = laneMrkrLst.at(k).front_lane_boundary_list.size();        
                    for (int i = 0; i < laneBdryNum; i++)
                    {
                        if (laneBdryIndx.m_LaneBoundaryID == laneMrkrLst.at(k).front_lane_boundary_list.at(i).line_boundary_id)
                        {
                            laneBdryObj = laneMrkrLst.at(k).front_lane_boundary_list.at(i);
                            foundFlg = true;
                            break;
                        }
                    }
                }
                if (foundFlg == true)
                {
                    break;
                }
            }
            return foundFlg;
        }

        bool CorridorGeneration::GenerateEndRngLst(const RoadInfo &roadModelObj,
                                                   const std::vector<BA::BoundaryIndex> &bdryIndxIDLst,
                                                   std::vector<float> &endRngLst)
        {
            LaneBoundary laneBdryObjTmp;
            for (int i = 0; i < bdryIndxIDLst.size(); i++)
            {
                if (FindLaneBdry(roadModelObj.lane_marker_list, bdryIndxIDLst.at(i), laneBdryObjTmp))
                {
                    endRngLst.emplace_back(laneBdryObjTmp.line_geometry.end_range);
                }
            }
            if (bdryIndxIDLst.size() != endRngLst.size())
            {
                DLOG(WARNING) << "certain boundary id is not found." << std::endl;
                return false;
            }
            return true;
        }

        uint8_t CorridorGeneration::GetPosInVec(std::vector<float> &rawVector, float value)
        {
            if (value < rawVector[0])
            {
                return 0;
            }

            if (value >= rawVector[rawVector.size() - 1])
            {
                // std::cout << "value is out of range!" << std::endl;
                return (rawVector.size() - 1);
            }
            // bool isPosFound = false;
            for (int i = 0; i < rawVector.size() - 1; i++)
            {
                if (value >= rawVector[i] && value < rawVector[i + 1])
                {
                    // isPosFound = true;
                    return (i + 1);
                }
            }
            std::cout << "The position has not been found in raw vector!" << std::endl;
            return 0;
        }


        void CorridorGeneration::UpdateOrgLaneID()
        {
            // when optAction changes from not laneChange to laneChange, update original lane ID which is applied for lane change case
            if ((this->optAction == BA::BA_ActionTyp_LeftLaneChange && this->optAction_LastCycle != BA::BA_ActionTyp_LeftLaneChange) || (this->optAction == BA::BA_ActionTyp_RightLaneChange && this->optAction_LastCycle != BA::BA_ActionTyp_RightLaneChange))
            {
                this->orgLaneStID = this->roadModel->car_position.ego_lane_id;
                this->orgLaneEndID = this->roadModel->car_position.ego_lane_id;
            }
        }

        void CorridorGeneration::GenerateLatCrrdr(DM::LateralBlueCorridor &BA_LatBlueCrrdr,
                                                  DM::LateralBorder &BA_LatYllwBrdr,
                                                  DM::LateralBorder &BA_LatRedBrdr)
        {
            RoadInfo rawRoadModel = *this->roadModel;
            uint8_t egoLaneID = rawRoadModel.car_position.ego_lane_id;
            uint8_t targetLaneStID = this->trgtLaneStartID;
            uint8_t targetLaneEndID = this->trgtLaneEndID;
            std::vector<Lane> laneLst;
            // Lane lane;
            Corridor optCorridorObj;
            Lane trgtlaneObj;
            bool isCorridorFound = false;

            for (int i = 0; i < rawRoadModel.lane_list.size(); i++)
            {
                Lane laneObjTmp = rawRoadModel.lane_list[i];
                isCorridorFound = FindCorridor(laneObjTmp, this->optCorridorID4TRPLat, optCorridorObj); // 20220729: replace optCorridorID with optCorridorID4TRPLat
                DLOG(INFO) << "isCorridorFound: " << int(isCorridorFound) << std::endl;
                DLOG(INFO) << "this->optCorridorID: " << int64_t(this->optCorridorID) << std::endl;
                DLOG(INFO) << "this->optCorridorID4TRPLat: " << int64_t(this->optCorridorID4TRPLat) << std::endl;
                // DLOG(INFO) << "corridorID in road model: " << int64_t(rawRoadModel.lane_list[0].corridor_list[0].corridor_id) << std::endl;
                if (isCorridorFound)
                {
                    trgtlaneObj = laneObjTmp;
                    break;
                }
            }
            if (!isCorridorFound)
            {
                DLOG(WARNING) << "optimal corridor obj has not been found!" << std::endl;
            }
            else
            {
                // blue corridor generation below
                BA_LatBlueCrrdr.m_reference_line = optCorridorObj.reference_line;
                BA_LatBlueCrrdr.m_width = optCorridorObj.drivable_width;
                float corridorRngEnd = optCorridorObj.reference_line.end_range;
                float desCrrdrLen = CalcDesCrrdrLen(this->vehSpd, corridorRngEnd); // desired corridor length

                // yellow and red border generation below
                // set a switch flag to decide whether to calculate latYllwBrdr and redBrdr or not
                bool toCalcYllwRedBrdrFlg = false;
                if (toCalcYllwRedBrdrFlg == true)
                {
                    if (this->optAction4TRPLat == BA::BA_ActionTyp_LaneFollow || 
                        this->optAction4TRPLat == BA::BA_ActionTyp_LeftLaneChangePrepare || 
                        this->optAction4TRPLat == BA::BA_ActionTyp_RightLaneChangePrepare || 
                        this->optAction4TRPLat == BA::BA_ActionTyp_LeftLaneChangeAbolition || 
                        this->optAction4TRPLat == BA::BA_ActionTyp_RightLaneChangeAbolition)
                    {
                        LineGeometry yllwBrdrLineLft = optCorridorObj.reference_line;
                        LineGeometry yllwBrdrLineRght = yllwBrdrLineLft;
                        yllwBrdrLineLft.c0 = yllwBrdrLineLft.c0 + optCorridorObj.drivable_width / 2;   // left is positive
                        yllwBrdrLineRght.c0 = yllwBrdrLineRght.c0 - optCorridorObj.drivable_width / 2; // right is negative
                                                                                                       // yellow border generation below
                        for (int k = 0; k < k_BA_SamplePointNumber; k++)
                        {
                            float x = desCrrdrLen * k / (k_BA_SamplePointNumber - 1);
                            float y_yllw_l = CalcLatDvtn(x, yllwBrdrLineLft);
                            // BA::SLPoint pntPair = {x, y_l};
                            BA_LatYllwBrdr.m_left_border[k].s = x;
                            BA_LatYllwBrdr.m_left_border[k].l = y_yllw_l;
                            float y_yllw_r = CalcLatDvtn(x, yllwBrdrLineRght);
                            BA_LatYllwBrdr.m_right_border[k].s = x;
                            BA_LatYllwBrdr.m_right_border[k].l = y_yllw_r;
                        }
                        // red border generation below
                        // left red border generation below
                        // generate boundary index list of left side of target lane
                        std::vector<BA::BoundaryIndex> left_boundary_indx_list;
                        for (auto mrkrID : trgtlaneObj.left_lane_marker_id_list)
                        {
                            // get marker obj
                            for (auto tmpMrkrObj : rawRoadModel.lane_marker_list)
                            {
                                if (tmpMrkrObj.id == mrkrID)
                                {
                                    BA::BoundaryIndex tmpBdryIndx = {0,0};
                                    tmpBdryIndx.m_LaneMarkerID = mrkrID;
                                    // get lane boundary obj
                                    for (auto tmpLnBdryObj : tmpMrkrObj.front_lane_boundary_list)
                                    {
                                        tmpBdryIndx.m_LaneBoundaryID = tmpLnBdryObj.line_boundary_id;
                                        left_boundary_indx_list.emplace_back(tmpBdryIndx);
                                    }
                                }
                            }
                        }
                        std::vector<float> leftEndRngLst;
                        if (!GenerateEndRngLst(rawRoadModel, left_boundary_indx_list, leftEndRngLst))
                        {
                            DLOG(WARNING) << "Generate left end range list failed!" << std::endl;
                        }

                        for (int l = 0; l < k_BA_SamplePointNumber; l++)
                        {
                            float x = desCrrdrLen * l / (k_BA_SamplePointNumber - 1);
                            if (!leftEndRngLst.empty())
                            {
                                uint8_t pos = GetPosInVec(leftEndRngLst, x);
                                BA::BoundaryIndex laneBdryIndx = left_boundary_indx_list[pos];
                                LaneBoundary laneBdryObj;
                                if (!FindLaneBdry(rawRoadModel.lane_marker_list, laneBdryIndx, laneBdryObj))
                                {
                                    DLOG(WARNING) << "Lane boundary has not been found!" << std::endl;
                                    //exit(1);
                                }
                                float y_red_l = CalcLatDvtn(x, laneBdryObj.line_geometry);
                                BA_LatRedBrdr.m_left_border[l].s = x;
                                BA_LatRedBrdr.m_left_border[l].l = y_red_l;
                            }
                        }
                        // right red border generation below
                        // generate boundary index list of right side of target lane
                        std::vector<BA::BoundaryIndex> right_boundary_indx_list;
                        for (auto mrkrID : trgtlaneObj.right_lane_marker_id_list)
                        {
                            // get marker obj
                            for (auto tmpMrkrObj : rawRoadModel.lane_marker_list)
                            {
                                if (tmpMrkrObj.id == mrkrID)
                                {
                                    BA::BoundaryIndex tmpBdryIndx = {0,0};
                                    tmpBdryIndx.m_LaneMarkerID = mrkrID;
                                    // get lane boundary obj
                                    for (auto tmpLnBdryObj : tmpMrkrObj.front_lane_boundary_list)
                                    {
                                        tmpBdryIndx.m_LaneBoundaryID = tmpLnBdryObj.line_boundary_id;
                                        right_boundary_indx_list.emplace_back(tmpBdryIndx);
                                    }
                                }
                            }
                        }
                        std::vector<float> rightEndRngLst;
                        if (!GenerateEndRngLst(rawRoadModel, right_boundary_indx_list, rightEndRngLst))
                        {
                            DLOG(WARNING) << "Generate right end range list failed!" << std::endl;
                        }

                        for (int l = 0; l < k_BA_SamplePointNumber; l++)
                        {
                            float x = desCrrdrLen * l / (k_BA_SamplePointNumber - 1);
                            if (!rightEndRngLst.empty())
                            {
                                uint8_t pos = GetPosInVec(rightEndRngLst, x);
                                BA::BoundaryIndex laneBdryIndx = right_boundary_indx_list[pos];
                                LaneBoundary laneBdryObj;
                                if (!FindLaneBdry(rawRoadModel.lane_marker_list, laneBdryIndx, laneBdryObj))
                                {
                                    DLOG(WARNING) << "Lane boundary has not been found!" << std::endl;
                                }
                                float y_red_r = CalcLatDvtn(x, laneBdryObj.line_geometry);
                                BA_LatRedBrdr.m_right_border[l].s = x;
                                BA_LatRedBrdr.m_right_border[l].l = y_red_r;
                            }
                        }
                    }
                    else if (this->optAction4TRPLat == BA::BA_ActionTyp_LeftLaneChange) // optimal action is left lane change
                    {
                        LineGeometry yllwBrdrLineLft;
                        LineGeometry yllwBrdrLineRght;

                        // right yellow border varies by isLCAbltAvlbl, while left yellow border and red borders do not

                        Lane orgLaneObj;
                        if (!FindLane(rawRoadModel, this->orgLaneStID, this->orgLaneEndID, orgLaneObj))
                        {
                            DLOG(WARNING) << "original lane has not been found!" << std::endl;
                        }
                        Corridor orgLaneCrrdrObj;
                        for (int m = 0; m < orgLaneObj.corridor_list.size(); m++)
                        {
                            if (orgLaneObj.corridor_list[m].feature_tag == 0) // assuming feature_tag = 0 denotes follow lane lines
                            {
                                orgLaneCrrdrObj = orgLaneObj.corridor_list[m];
                            }
                        }

                        if (this->isLCAbltAvlbl) // before crossing lane line
                        {
                            // right yellow border line generation below
                            yllwBrdrLineRght = orgLaneCrrdrObj.reference_line;
                            yllwBrdrLineRght.c0 = yllwBrdrLineRght.c0 - orgLaneCrrdrObj.drivable_width / 2; // right is negative
                        }
                        else // after crossing lane line
                        {
                            // right yellow border line generation below
                            yllwBrdrLineRght = orgLaneCrrdrObj.reference_line;
                        }

                        // left yellow border line generation below
                        for (int m = 0; m < trgtlaneObj.corridor_list.size(); m++)
                        {
                            if (trgtlaneObj.corridor_list[m].feature_tag == 0) // assuming feature_tag = 0 denotes follow lane lines
                            {
                                Corridor trgtLaneCrrdrObj = trgtlaneObj.corridor_list[m];
                                yllwBrdrLineLft = trgtLaneCrrdrObj.reference_line;
                                yllwBrdrLineLft.c0 = yllwBrdrLineLft.c0 + trgtLaneCrrdrObj.drivable_width / 2; // left is positive
                                break;
                            }
                        }
                        // yellow border generation below
                        // TODO: whether code below can be integrated
                        for (int k = 0; k < k_BA_SamplePointNumber; k++)
                        {
                            float x = desCrrdrLen * k / (k_BA_SamplePointNumber - 1);
                            float y_yllw_l = CalcLatDvtn(x, yllwBrdrLineLft);
                            BA_LatYllwBrdr.m_left_border[k].s = x;
                            BA_LatYllwBrdr.m_left_border[k].l = y_yllw_l;
                            float y_yllw_r = CalcLatDvtn(x, yllwBrdrLineRght);
                            BA_LatYllwBrdr.m_right_border[k].s = x;
                            BA_LatYllwBrdr.m_right_border[k].l = y_yllw_r;
                        }

                        // red borders generation starts here
                        // left red border generation below
                        // generate boundary index list of left side of target lane
                        std::vector<BA::BoundaryIndex> left_boundary_indx_list;
                        for (auto mrkrID : trgtlaneObj.left_lane_marker_id_list)
                        {
                            // get marker obj
                            for (auto tmpMrkrObj : rawRoadModel.lane_marker_list)
                            {
                                if (tmpMrkrObj.id == mrkrID)
                                {
                                    BA::BoundaryIndex tmpBdryIndx = {0,0};
                                    tmpBdryIndx.m_LaneMarkerID = mrkrID;
                                    // get lane boundary obj
                                    for (auto tmpLnBdryObj : tmpMrkrObj.front_lane_boundary_list)
                                    {
                                        tmpBdryIndx.m_LaneBoundaryID = tmpLnBdryObj.line_boundary_id;
                                        left_boundary_indx_list.emplace_back(tmpBdryIndx);
                                    }
                                }
                            }
                        }
                        std::vector<float> leftEndRngLst;
                        if (!GenerateEndRngLst(rawRoadModel, left_boundary_indx_list, leftEndRngLst))
                        {
                            DLOG(WARNING) << "Generate left end range list failed!" << std::endl;
                        }

                        for (int l = 0; l < k_BA_SamplePointNumber; l++)
                        {
                            float x = desCrrdrLen * l / (k_BA_SamplePointNumber - 1);
                            if (!leftEndRngLst.empty())
                            {
                                uint8_t pos = GetPosInVec(leftEndRngLst, x);
                                BA::BoundaryIndex laneBdryIndx = left_boundary_indx_list[pos];
                                LaneBoundary laneBdryObj;
                                if (!FindLaneBdry(rawRoadModel.lane_marker_list, laneBdryIndx, laneBdryObj))
                                {
                                    DLOG(WARNING) << "Lane boundary has not been found!" << std::endl;
                                    //exit(1);
                                }
                                float y_red_l = CalcLatDvtn(x, laneBdryObj.line_geometry);
                                BA_LatRedBrdr.m_left_border[l].s = x;
                                BA_LatRedBrdr.m_left_border[l].l = y_red_l;
                            }
                        }

                        // right red border generation below
                        // generate boundary index list of right side of ego lane
                        std::vector<BA::BoundaryIndex> right_boundary_indx_list;
                        for (auto mrkrID : orgLaneObj.right_lane_marker_id_list)
                        {
                            // get marker obj
                            for (auto tmpMrkrObj : rawRoadModel.lane_marker_list)
                            {
                                if (tmpMrkrObj.id == mrkrID)
                                {
                                    BA::BoundaryIndex tmpBdryIndx = {0,0};
                                    tmpBdryIndx.m_LaneMarkerID = mrkrID;
                                    // get lane boundary obj
                                    for (auto tmpLnBdryObj : tmpMrkrObj.front_lane_boundary_list)
                                    {
                                        tmpBdryIndx.m_LaneBoundaryID = tmpLnBdryObj.line_boundary_id;
                                        right_boundary_indx_list.emplace_back(tmpBdryIndx);
                                    }
                                }
                            }
                        }
                        std::vector<float> rightEndRngLst;
                        if (!GenerateEndRngLst(rawRoadModel, right_boundary_indx_list, rightEndRngLst))
                        {
                            DLOG(WARNING) << "Generate right end range list failed!" << std::endl;
                        }

                        for (int l = 0; l < k_BA_SamplePointNumber; l++)
                        {
                            float x = desCrrdrLen * l / (k_BA_SamplePointNumber - 1);
                            if (!rightEndRngLst.empty())
                            {
                                uint8_t pos = GetPosInVec(rightEndRngLst, x);
                                BA::BoundaryIndex laneBdryIndx = right_boundary_indx_list[pos];
                                LaneBoundary laneBdryObj;
                                if (!FindLaneBdry(rawRoadModel.lane_marker_list, laneBdryIndx, laneBdryObj))
                                {
                                    DLOG(WARNING) << "Lane boundary has not been found!" << std::endl;
                                }
                                float y_red_r = CalcLatDvtn(x, laneBdryObj.line_geometry);
                                BA_LatRedBrdr.m_right_border[l].s = x;
                                BA_LatRedBrdr.m_right_border[l].l = y_red_r;
                            }
                        }
                    }
                    else if (this->optAction4TRPLat == BA::BA_ActionTyp_RightLaneChange) // optimal action is right lane change
                    {
                        LineGeometry yllwBrdrLineLft;
                        LineGeometry yllwBrdrLineRght;

                        // left yellow border varies by isLCAbltAvlbl, while right yellow border and red borders do not

                        Lane orgLaneObj;
                        if (!FindLane(rawRoadModel, this->orgLaneStID, this->orgLaneEndID, orgLaneObj))
                        {
                            DLOG(WARNING) << "original lane has not been found!" << std::endl;
                        }
                        Corridor orgLaneCrrdrObj;
                        for (int m = 0; m < orgLaneObj.corridor_list.size(); m++)
                        {
                            if (orgLaneObj.corridor_list[m].feature_tag == 0) // assuming feature_tag = 0 denotes follow lane lines
                            {
                                orgLaneCrrdrObj = orgLaneObj.corridor_list[m];
                            }
                        }

                        if (this->isLCAbltAvlbl) // before crossing lane line
                        {
                            // left yellow border line generation below
                            yllwBrdrLineLft = orgLaneCrrdrObj.reference_line;
                            yllwBrdrLineLft.c0 = yllwBrdrLineLft.c0 + orgLaneCrrdrObj.drivable_width / 2; // left is positive
                        }
                        else // after crossing lane line
                        {
                            // left yellow border line generation below
                            yllwBrdrLineLft = orgLaneCrrdrObj.reference_line;
                        }

                        // right yellow border line generation below
                        for (int m = 0; m < trgtlaneObj.corridor_list.size(); m++)
                        {
                            if (trgtlaneObj.corridor_list[m].feature_tag == 0) // assuming feature_tag = 0 denotes follow lane lines
                            {
                                Corridor trgtLaneCrrdrObj = trgtlaneObj.corridor_list[m];
                                yllwBrdrLineRght = trgtLaneCrrdrObj.reference_line;
                                yllwBrdrLineRght.c0 = yllwBrdrLineRght.c0 - trgtLaneCrrdrObj.drivable_width / 2; // right is positive
                                break;
                            }
                        }
                        // yellow border generation below
                        // TODO: whether code below can be integrated
                        for (int k = 0; k < k_BA_SamplePointNumber; k++)
                        {
                            float x = desCrrdrLen * k / (k_BA_SamplePointNumber - 1);
                            float y_yllw_l = CalcLatDvtn(x, yllwBrdrLineLft);
                            BA_LatYllwBrdr.m_left_border[k].s = x;
                            BA_LatYllwBrdr.m_left_border[k].l = y_yllw_l;
                            float y_yllw_r = CalcLatDvtn(x, yllwBrdrLineRght);
                            BA_LatYllwBrdr.m_right_border[k].s = x;
                            BA_LatYllwBrdr.m_right_border[k].l = y_yllw_r;
                        }

                        // red borders generation starts here
                        // left red border generation below
                        // generate boundary index list of left side of original lane
                        std::vector<BA::BoundaryIndex> left_boundary_indx_list;
                        for (auto mrkrID : orgLaneObj.left_lane_marker_id_list)
                        {
                            // get marker obj
                            for (auto tmpMrkrObj : rawRoadModel.lane_marker_list)
                            {
                                if (tmpMrkrObj.id == mrkrID)
                                {
                                    BA::BoundaryIndex tmpBdryIndx = {0,0};
                                    tmpBdryIndx.m_LaneMarkerID = mrkrID;
                                    // get lane boundary obj
                                    for (auto tmpLnBdryObj : tmpMrkrObj.front_lane_boundary_list)
                                    {
                                        tmpBdryIndx.m_LaneBoundaryID = tmpLnBdryObj.line_boundary_id;
                                        left_boundary_indx_list.emplace_back(tmpBdryIndx);
                                    }
                                }
                            }
                        }
                        std::vector<float> leftEndRngLst;
                        if (!GenerateEndRngLst(rawRoadModel, left_boundary_indx_list, leftEndRngLst))
                        {
                            DLOG(WARNING) << "Generate left end range list failed!" << std::endl;
                        }

                        for (int l = 0; l < k_BA_SamplePointNumber; l++)
                        {
                            float x = desCrrdrLen * l / (k_BA_SamplePointNumber - 1);
                            if (!leftEndRngLst.empty())
                            {
                                uint8_t pos = GetPosInVec(leftEndRngLst, x);
                                BA::BoundaryIndex laneBdryIndx = left_boundary_indx_list[pos];
                                LaneBoundary laneBdryObj;
                                if (!FindLaneBdry(rawRoadModel.lane_marker_list, laneBdryIndx, laneBdryObj))
                                {
                                    DLOG(WARNING) << "Lane boundary has not been found!" << std::endl;
                                    //exit(1);
                                }
                                float y_red_l = CalcLatDvtn(x, laneBdryObj.line_geometry);
                                BA_LatRedBrdr.m_left_border[l].s = x;
                                BA_LatRedBrdr.m_left_border[l].l = y_red_l;
                            }
                        }

                        // right red border generation below
                        // generate boundary index list of right side of ego lane
                        std::vector<BA::BoundaryIndex> right_boundary_indx_list;
                        for (auto mrkrID : trgtlaneObj.right_lane_marker_id_list)
                        {
                            // get marker obj
                            for (auto tmpMrkrObj : rawRoadModel.lane_marker_list)
                            {
                                if (tmpMrkrObj.id == mrkrID)
                                {
                                    BA::BoundaryIndex tmpBdryIndx = {0,0};
                                    tmpBdryIndx.m_LaneMarkerID = mrkrID;
                                    // get lane boundary obj
                                    for (auto tmpLnBdryObj : tmpMrkrObj.front_lane_boundary_list)
                                    {
                                        tmpBdryIndx.m_LaneBoundaryID = tmpLnBdryObj.line_boundary_id;
                                        right_boundary_indx_list.emplace_back(tmpBdryIndx);
                                    }
                                }
                            }
                        }
                        std::vector<float> rightEndRngLst;
                        if (!GenerateEndRngLst(rawRoadModel, right_boundary_indx_list, rightEndRngLst))
                        {
                            DLOG(WARNING) << "Generate right end range list failed!" << std::endl;
                        }

                        for (int l = 0; l < k_BA_SamplePointNumber; l++)
                        {
                            float x = desCrrdrLen * l / (k_BA_SamplePointNumber - 1);
                            if (!rightEndRngLst.empty())
                            {
                                uint8_t pos = GetPosInVec(rightEndRngLst, x);
                                BA::BoundaryIndex laneBdryIndx = right_boundary_indx_list[pos];
                                LaneBoundary laneBdryObj;
                                if (!FindLaneBdry(rawRoadModel.lane_marker_list, laneBdryIndx, laneBdryObj))
                                {
                                    DLOG(WARNING) << "Lane boundary has not been found!" << std::endl;
                                }
                                float y_red_r = CalcLatDvtn(x, laneBdryObj.line_geometry);
                                BA_LatRedBrdr.m_right_border[l].s = x;
                                BA_LatRedBrdr.m_right_border[l].l = y_red_r;
                            }
                        }
                    }
                }
            }
        }

        void CorridorGeneration::GenerateLgtCrrdr(DM::LongitudinalCorridor &BA_LgtBlueCrrdr,
                                                  DM::LongitudinalCorridor &BA_LgtYllwBrdr,
                                                  DM::LongitudinalCorridor &BA_LgtRedBrdr,
                                                  DM::SpeedLimit &BA_LgtSpdLmt)
        {
            Passage optPassageObj;
            bool isPassageFound = false;
            // DLOG(INFO) << "Optimal passage id x: " << int(this->optPassageID.m_X) << std::endl;
            // DLOG(INFO) << "Optimal passage id y: " << int(this->optPassageID.m_Y) << std::endl;
            if (!FindPassage(this->passageList, this->optPassageID, optPassageObj))
            {
                DLOG(WARNING) << "Passage has not been found!" << std::endl;
            }
            LongitudinalInfo lgtInfo = optPassageObj.longt_info;
            // longitudinal blue corridor generation below
            BA_LgtBlueCrrdr.m_corridor_distance = lgtInfo.desired_distance;
            BA_LgtBlueCrrdr.m_corridor_speed = lgtInfo.desired_speed;
            // longitudinal yellow border generation below
            BA_LgtYllwBrdr.m_corridor_distance = lgtInfo.yellow_border_distance;
            BA_LgtYllwBrdr.m_corridor_speed = lgtInfo.desired_speed;
            // longitudinal red border generation below
            BA_LgtRedBrdr.m_corridor_distance = lgtInfo.red_border_distance;
            BA_LgtRedBrdr.m_corridor_speed = lgtInfo.desired_speed;
            // speed limit below
            BA_LgtSpdLmt.m_related_distance = lgtInfo.set_distance;
            BA_LgtSpdLmt.m_target_speed = lgtInfo.set_speed;
        }
    } // namespace planning
} // namespace ADC