#pragma once
#include "../commonfunc.h"
#include "../include/CtApBehaviorAnalysis_/common_BA.h"
#include "../include/CtApBehaviorAnalysis_/behaviorNodeGlobal.h"
// #include "mpc_drv_msgs/msg/vehicle_state_output.hpp"
// #include "mpc_drv_msgs/msg/lateral_blue_corridor.hpp"
// #include "mpc_drv_msgs/msg/lateral_border.hpp"
// #include "mpc_drv_msgs/msg/longitudinal_corridor.hpp"
// #include "mpc_drv_msgs/msg/speed_limit.hpp"
// #include "mpc_drv_msgs/msg/lane.hpp"
// #include "mpc_drv_msgs/msg/corridor.hpp"
// #include "mpc_drv_msgs/msg/road_info.hpp"
// #include "mpc_drv_msgs/msg/lane_boundary.hpp"
// #include "mpc_drv_msgs/msg/line_geometry.hpp"
#include "../forcePublic.h"
#include "../include/CtApBehaviorAnalysis_/interface/dm_internal_interface.h"
#include "../include/CtApBehaviorAnalysis_/interface/ved_dpc_interface.h"
// #include "../include/CtApBehaviorAnalysis_/interface/rm_road_info.h"
#include "dpc/impl_type_roadinfo.h"

namespace ADC
{
    namespace planning
    {
        using namespace ADC::common;

        class CorridorGeneration
        {
        public:
            /**
            *class constructor.
            *@param  RM_RoadModel the road model from RM module.
            *@param  BA_OptCorridorID the optimal corridor ID from BA INTERNAL.
            *@param  BA_OptAction the optimal action from BA INTERNAL.
            *@param  VSE_VehSpd the vehicle speed.
            *@param  BA_TrgtLaneStartID the start ID of target lane.
            *@param  BA_TrgtLaneEndID  the end ID of target lane.
            *@param  BA_IsLCAbltAvlbl  the flag to denote whether lane change abolition is available or not.
            *@param  BA_OptPassageID the optimal passage ID
            *@param  BA_OptAction_LastCycle the optimal action of last cycle
            */
            CorridorGeneration(RoadInfo &RM_RoadModel,
                               uint32_t BA_OptCorridorID,
                               BA::BA_ActionTyp BA_OptAction,
                               float VSE_VehSpd,
                               uint8_t BA_TrgtLaneStartID,
                               uint8_t BA_TrgtLaneEndID,
                               bool BA_IsLCAbltAvlbl,
                               BA::Index BA_OptPassageID,
                               BA::BA_ActionTyp BA_OptAction_LastCycle,
                               std::vector<Passage> BA_FirstPassageLst,
                               const BA::BA_ActionTyp &BA_Action4TRPLat,
                               const uint32_t &BA_Corridor4TRPLat);

            void GenerateLatCrrdr(DM::LateralBlueCorridor &BA_LatBlueCrrdr,
                                  DM::LateralBorder &BA_LatYllwBrdr,
                                  DM::LateralBorder &BA_LatRedBrdr);

            void GenerateLgtCrrdr(DM::LongitudinalCorridor &BA_LgtBlueCrrdr,
                                  DM::LongitudinalCorridor &BA_LgtYllwBrdr,
                                  DM::LongitudinalCorridor &BA_LgtRedBrdr,
                                  DM::SpeedLimit &BA_LgtSpdLmt);

            DM::LateralBlueCorridor latBlueCrrdr;
            DM::LateralBorder latYllwBrdr;
            DM::LateralBorder latRedBrdr;
            DM::LongitudinalCorridor lgtBlueCrrdr;
            DM::LongitudinalCorridor lgtYllwBrdr;
            DM::LongitudinalCorridor lgtRedBrdr;
            DM::SpeedLimit lgtSpdLmt;

        private:
            float CalcDesCrrdrLen(float vehicleSpeed, float crrdrRngEnd);  // calculate desired corridor length according to vehicle speed
            float CalcLatDvtn(float dist, LineGeometry line); // output lateral deviation along a specific line
            bool FindLane(RoadInfo &roadModel,
                          const uint8_t laneStartID,
                          const uint8_t laneEndID,
                          Lane &lane); // get lane obj according to laneStartID and laneEndID
            bool FindLane(RoadInfo &roadModel,
                          const uint8_t laneID,
                          std::vector<Lane> &laneLst); // get lane objs according to lane ID
            bool FindCorridor(Lane &lane,
                              const uint32_t corridorID,
                              Corridor &corridorObj);           // get corridor obj according to corridor ID
            uint8_t GetPosInVec(std::vector<float> &rawVector, float value); // get the value's position in a sorted vector
            bool GenerateEndRngLst(const RoadInfo &roadModelObj,
                                   const std::vector<BA::BoundaryIndex> &bdryIndxIDLst,
                                   std::vector<float> &endRngLst); // generate a list filled with end range of different boundaries
            bool FindLaneBdry(const std::vector<LaneMarker> &laneMrkrLst,
                              const BA::BoundaryIndex &laneBdryIndx,
                              LaneBoundary &laneBdryObj); // get lane boundary obj according to lane boundary ID
            bool FindPassage(const RoadInfo &roadModel,
                             const BA::Index passageID,
                             Passage &passageObj);
            bool FindPassage(const std::vector<Passage> &passageLst,
                             const BA::Index passageID,
                             Passage &passageObj);

            void UpdateOrgLaneID(); // update original lane ID

            RoadInfo *roadModel;
            uint32_t optCorridorID;
            BA::BA_ActionTyp optAction;
            uint32_t optCorridorID4TRPLat;
            BA::BA_ActionTyp optAction4TRPLat;
            float vehSpd;
            uint8_t trgtLaneStartID;
            uint8_t trgtLaneEndID;
            bool isLCAbltAvlbl;
            BA::Index optPassageID;
            std::vector<Passage> passageList;

            uint8_t orgLaneStID = uint8_t(0);
            uint8_t orgLaneEndID = uint8_t(0);
            BA::BA_ActionTyp optAction_LastCycle; // store optAction in last cycle
        };

    } // namespace planning
} // namespace ADC
