#pragma once
#include <iostream>
#include "../../include/CtApBehaviorAnalysis_/common_BA.h"
// #include "mpc_drv_msgs/msg/car_position.hpp"
#include "../commonfunc.h"
#include "../forcePublic.h"
#include "algorithm"
// #include "mpc_drv_msgs/msg/control_waypoints.hpp"
// #include "mpc_drv_msgs/msg/road_info.hpp"
// #include "mpc_drv_msgs/msg/lane.hpp"
#include "../../include/CtApBehaviorAnalysis_/BA_Enum_List.h"
// #include "../../include/CtApBehaviorAnalysis_/interface/rm_road_info.h"
// #include "../../include/CtApBehaviorAnalysis_/interface/slam_decision.h"
#include "dpc/impl_type_roadinfo.h"
#include "dpc/impl_type_controlwaypoints.h"


class GlobalCostCalculation
{
public:
    void Init(const BA::GlobalInfoContainer &globalInfoContainer,
              const BA::HV_Info &hostVehInfo,
              const RoadInfo &roadModel,
              const std::vector<BA::BA_ActionTyp> &actionTypArr); // initialization
    void CalculateGlobalCost(const ControlWaypoints &ehr_msg,
                             const RoadInfo &rm_msg,
                             const CarPosition &carPos,
                             float egoVehSpd,
                             bool vldWayPntDtc,
                             bool isEffcLCIntGnrt,
                             BA::DriveMd drvMd,
                             const bool &spdCondStsfyFlg,
                             const bool &isEgoOnRghtSndLn); // judge whether lane change intention is generated
                             
    bool UpdtHistVehInfo(const CarPosition &carPos, const float &vehSpd, const bool &isHistVehInfoUpdt); // update history vehicle info, put outside of main cycle, run each 100ms

public:
    float m_GlobalCost = 0.0f;
    float m_GlobalPathFllwCost = 0.0f;
    float m_LgTermFastLnCost = 0.0f;
    float m_LgTermSlowLnCost = 0.0f;
    float m_VehFlowInLnCost = 0.0f; // not assigned
    float m_LaneEndCost = 0.0f;
    float m_TrffcEffcCost = 0.0f;
    float m_MrgRdLnCost = 0.0f;
    float m_TrgtLnCost = 0.0f;
    float m_RghtScndLnCost = 0.0f;
    BA::GlobalInfoContainer m_GlobalInfoContainer;
    BA::HV_Info m_CurHostVehInfo;
    float m_RoadSpdLmt = 80.0f;
    BA::HV_HistInfo m_HistHostVehInfo;
    float m_CycleTimer = 0;
    std::vector<BA::BA_ActionTyp> m_ActionTypArr;

private:
    void CalcGlobalPathFllwCost();                       // calculate global path follow cost
    void CalcGlobalPathFllwCost_B();                     // calculate global path follow cost plan B
    void CalcGlobalPathFllwCost_C();                     // calculate global path follow cost plan C for multi-step sequence
    void CalcGlobalPathFllwCost_InclLCP();               // calculate global path follow cost covering LCP action
    void CalcLgtTermFastLnCost(const uint8_t &fastLnID); // calculate long-term staying in fast lane cost
    void CalcLgtTermFastLnCost_B(const uint8_t &fastLnID, const RoadInfo &rm_msg); // calculate long-term staying in fast lane cost plan B
    // void CalcLgtTermFastLnCost_InclLCP(const uint8_t &fastLnID); // calculate long-term staying in fast lane cost covering LCP action
    void CalcLgtTermSlowLnCost(const uint8_t &slowLnID); // calculate long-term staying in slowest lane cost, namely the lane on the right side
    void CalcLgtTermSlowLnCost_B(const uint8_t &slowLnID); // calculate long-term staying in slowest lane cost, namely the lane on the right side plan B
    void CalcLgtTermSlowLnCost_InclLCP(const uint8_t &slowLnID, const ControlWaypoints &ehr_msg); // calculate long-term staying in slowest lane cost covering LCP action
    void CalcLnEndCost(const ControlWaypoints &ehr_msg,
                       const CarPosition &carPos,
                       const float &vehSpd); // calculate lane end cost
    void CalcLnEndCost_B(const ControlWaypoints &ehr_msg,
                         const CarPosition &carPos,
                         const float &vehSpd);                                          // calculate lane end cost plan B
    void CalcLnEndCost_C(const ControlWaypoints &ehr_msg,
                         const CarPosition &carPos,
                         const float &vehSpd);                                          // calculate lane end cost plan C
    void CalcLnEndCost_InclLCP(const ControlWaypoints &ehr_msg,
                               const CarPosition &carPos,
                               const float &vehSpd);                                    // calculate lane end cost covering LCP action
    void CalcMrgRdLnCost_InclLCP(const ControlWaypoints &ehr_msg,
                                 const CarPosition &carPos,
                                 const float &vehSpd,
                                 const BA::DriveMd &driveMode);                         // calculate merge road lane cost covering LCP action
    void CalcTrgtLnCost(const ControlWaypoints &ehr_msg,
                        const CarPosition &carPos,
                        const float &vehSpd);                                          // calculate target lane cost
    void RevCrdSysOrgPnt(float dstToWayPnt);                                            // change m_CurHostVehInfo's original point from ego vehicle to waypoint
    bool GetLnChgPnts(uint8_t laneID, float &desLnChgPntDst, float &latestLnChgPntDst); // get desired and latest lane change point distance
    void UpdtHistVehInfoDist(float vehSpd, BA::HV_HistInfo &histInfo);                                             // update m_DistanceAlongS in history vehicle info
    bool CalcSmplPntTimeStamp();                                                        // determine whether it is time to set the sampling time stamp or not
    bool IsSmplPnt4LnEndPosi(const std::vector<TransitLaneStruct> &transitLnLst,
                             const uint8_t &laneLoc,
                             float &distAlgS,
                             const float &vehSpd,
                             float &remainDist);                       // judge whether a sample point should be included for lane end cost calculation
    void CalcTrffcEffcyCost(const bool &isEffcLCIntGnrt, const bool &spdCondStsfy); // calculate traffic efficiency cost
    void CalcRghtScndLnCost(const bool &isOnRghtScndLn); // calculate car on the right second lane cost
};