#include "global_info_process.h"
#include <algorithm>
#include <functional>
#include <cmath>
#include <map>
#include "glog/logging.h"

void GlobalInfoProcess::initGlobalInfo(ControlWaypoints &ehr_msg, const RoadInfo &roadModel, float vehSpd)
{
    // set flag about whether to rebuild transit lane list
    bool toRebuildTransitLnLst = true;
    BA::RouteRltvPos routeRltvPos = BA::RouteRltvPos_Middle;
    int minLaneGapCWP = 100;
    int minLaneGapRWP = 100;
    if (ehr_msg.control_waypoints_list.at(0).control_waypoint_id != 300 && (ehr_msg.reminder_waypoint.at(0).reminder_waypoint_id == 0 || ehr_msg.reminder_waypoint.at(0).reminder_waypoint_offset > ehr_msg.control_waypoints_list.at(0).control_waypoint_offset)) // only actual control waypoint exists
    {
        this->globalInfoContainer.m_GlobalInfoSrc = BA::GlobalInfoSrc_OnlyActCtrlWayPoint;
        this->globalInfoContainer.m_WayPointID = ehr_msg.control_waypoints_list.at(0).control_waypoint_id;
        this->globalInfoContainer.m_DistToWayPoint = ehr_msg.control_waypoints_list.at(0).control_waypoint_offset;
        ControlWayPointStruct firstCtrlWayPnt_;
        if (GetFirstCtrlWP(ehr_msg, firstCtrlWayPnt_))
        {
            this->globalInfoContainer.m_RawTgtLaneLst_CWP = firstCtrlWayPnt_.control_waypoint_raw_lane_id_list;
            this->globalInfoContainer.m_TgtRoadTyp_CWP = BA::TargetRoadTyp(firstCtrlWayPnt_.control_waypoint_target_road_type);
            this->globalInfoContainer.m_WpExtendLaneLst = firstCtrlWayPnt_.control_wp_extendlane_list;
            this->globalInfoContainer.m_SpeedLimit = firstCtrlWayPnt_.control_waypoint_speed_limit;
            if (!GetCnrLaneID(firstCtrlWayPnt_, this->globalInfoContainer.m_LftCrnrLaneID, this->globalInfoContainer.m_RghtCrnrLaneID, ehr_msg.reminder_waypoint.front(), roadModel.car_position.ego_lane_id))
            {
                DLOG(WARNING) << "Getting corner lane id failed!" << std::endl;
                DLOG(INFO) << "22222" << std::endl;
            }
            routeRltvPos = GetRouteRltvPos(this->globalInfoContainer.m_RawTgtLaneLst_CWP, this->globalInfoContainer.m_LftCrnrLaneID, this->globalInfoContainer.m_RghtCrnrLaneID);
            this->globalInfoContainer.m_RouteRltvPos = routeRltvPos;
            bool isTgtLnLstMdf = MdfTgtLnLst(firstCtrlWayPnt_.control_waypoint_lane_id_list, routeRltvPos, ehr_msg, roadModel);
            DLOG(INFO) << "isTgtLnLstMdf = " << int(isTgtLnLstMdf) <<std::endl;
            minLaneGapCWP = CalcLaneGap(roadModel.car_position.ego_lane_id, 
                                        firstCtrlWayPnt_.control_wp_extendlane_list, 
                                        firstCtrlWayPnt_.control_waypoint_lane_id_list,
                                        this->globalInfoContainer.m_MinLnGapTgtLaneLst);
            DLOG(INFO) << "minLaneGapCWP = " << minLaneGapCWP << std::endl;
            this->globalInfoContainer.m_GlobalLaneChgDrct = DtrmLCDrcn(minLaneGapCWP);
            this->globalInfoContainer.m_GlobalLaneChgNum = std::abs(minLaneGapCWP);
            this->globalInfoContainer.m_TgtLaneLst_CWP = firstCtrlWayPnt_.control_waypoint_lane_id_list;

            // rebuild transit lane list when necessary
            rebuildTransLnList(ehr_msg, toRebuildTransitLnLst);
            // init m_GlobalLaneInfoArr, ID is from CWP target lane ID list
            for (int i = 0; i < firstCtrlWayPnt_.control_wp_extendlane_list.size(); i++)
            {
                BA::GlobalLaneInfo globalLnInfo_;
                if (InitGlobalLnInfo(firstCtrlWayPnt_, ehr_msg.transit_lanes_list, firstCtrlWayPnt_.control_wp_extendlane_list[i].lane_id, globalLnInfo_, vehSpd))
                {
                    this->globalInfoContainer.m_GlobalLaneInfoArr.emplace_back(globalLnInfo_);
                }
            }
        }
        else
        {
            DLOG(WARNING) << "Getting First Control Waypoint failed!" << std::endl;
        }
    }
    else if (ehr_msg.control_waypoints_list.at(0).control_waypoint_id == 300 && 
             (ehr_msg.reminder_waypoint.at(0).reminder_waypoint_id == 0 || 
              ehr_msg.reminder_waypoint.at(0).reminder_waypoint_offset > ehr_msg.control_waypoints_list.at(0).control_waypoint_offset ||
              ehr_msg.reminder_waypoint.at(0).reminder_waypoint_target_road_type == BA::TargetRoadTyp_SubMrgRd)) // only virtual control waypoint exists
    {
        // 20221005: exclude submrgrd
        this->globalInfoContainer.m_GlobalInfoSrc = BA::GlobalInfoSrc_OnlyVirtCtrlWayPoint;
        this->globalInfoContainer.m_WayPointID = ehr_msg.control_waypoints_list.at(0).control_waypoint_id;
        this->globalInfoContainer.m_DistToWayPoint = ehr_msg.control_waypoints_list.at(0).control_waypoint_offset;
        ControlWayPointStruct firstCtrlWayPnt_;
        if (GetFirstCtrlWP(ehr_msg, firstCtrlWayPnt_))
        {
            
            this->globalInfoContainer.m_RawTgtLaneLst_CWP = firstCtrlWayPnt_.control_waypoint_raw_lane_id_list;
            this->globalInfoContainer.m_TgtRoadTyp_CWP = BA::TargetRoadTyp(firstCtrlWayPnt_.control_waypoint_target_road_type);
            this->globalInfoContainer.m_WpExtendLaneLst = firstCtrlWayPnt_.control_wp_extendlane_list;
            this->globalInfoContainer.m_SpeedLimit = firstCtrlWayPnt_.control_waypoint_speed_limit;
            if (!GetCnrLaneID(firstCtrlWayPnt_, this->globalInfoContainer.m_LftCrnrLaneID, this->globalInfoContainer.m_RghtCrnrLaneID, ehr_msg.reminder_waypoint.front(), roadModel.car_position.ego_lane_id))
            {
                DLOG(WARNING) << "Getting corner lane id failed!" << std::endl;
            }
            routeRltvPos = GetRouteRltvPos(this->globalInfoContainer.m_RawTgtLaneLst_CWP, this->globalInfoContainer.m_LftCrnrLaneID, this->globalInfoContainer.m_RghtCrnrLaneID);
            this->globalInfoContainer.m_RouteRltvPos = routeRltvPos;
            minLaneGapCWP = CalcLaneGap(roadModel.car_position.ego_lane_id, 
                                        firstCtrlWayPnt_.control_wp_extendlane_list, 
                                        firstCtrlWayPnt_.control_waypoint_lane_id_list,
                                        this->globalInfoContainer.m_MinLnGapTgtLaneLst);
            this->globalInfoContainer.m_GlobalLaneChgDrct = DtrmLCDrcn(minLaneGapCWP);
            this->globalInfoContainer.m_GlobalLaneChgNum = std::abs(minLaneGapCWP);
            this->globalInfoContainer.m_TgtLaneLst_CWP = firstCtrlWayPnt_.control_waypoint_lane_id_list;

            // rebuild transit lane list when necessary
            rebuildTransLnList(ehr_msg, toRebuildTransitLnLst);
            // init m_GlobalLaneInfoArr, ID is from CWP target lane ID list
            for (int i = 0; i < firstCtrlWayPnt_.control_wp_extendlane_list.size(); i++)
            {
                BA::GlobalLaneInfo globalLnInfo_;
                if (InitGlobalLnInfo(firstCtrlWayPnt_, ehr_msg.transit_lanes_list, firstCtrlWayPnt_.control_wp_extendlane_list[i].lane_id, globalLnInfo_, vehSpd))
                {
                    this->globalInfoContainer.m_GlobalLaneInfoArr.emplace_back(globalLnInfo_);
                }
            }
        }
        else
        {
            DLOG(WARNING) << "Getting First Control Waypoint failed!" << std::endl;
        }
    }
    else if ((ehr_msg.control_waypoints_list.at(0).control_waypoint_id == 300 && ehr_msg.control_waypoints_list.at(1).control_waypoint_id == 0) && ehr_msg.reminder_waypoint.at(0).reminder_waypoint_id != 0) // only reminder waypoint and virtual control waypoints exist
    {
        // if minimum lane gap for RWP is not zero, then initialization is based on first reminder waypoint
        // or initialization is based on first control waypoint
        minLaneGapRWP = CalcLaneGap(roadModel.car_position.ego_lane_id, 
                                    ehr_msg.reminder_waypoint.front().reminder_wp_extendlane_list, 
                                    ehr_msg.reminder_waypoint.front().reminder_waypoint_lane_id_list,
                                    this->globalInfoContainer.m_MinLnGapTgtLaneLst);
        if (abs(minLaneGapRWP) > 0 && minLaneGapRWP != 100)
        {
            this->globalInfoContainer.m_GlobalInfoSrc = BA::GlobalInfoSrc_OnlyRmdWayPoint;
            this->globalInfoContainer.m_WayPointID = ehr_msg.reminder_waypoint.at(0).reminder_waypoint_id;
            this->globalInfoContainer.m_DistToWayPoint = ehr_msg.reminder_waypoint.at(0).reminder_waypoint_offset;
            this->globalInfoContainer.m_TgtLaneLst_RWP = ehr_msg.reminder_waypoint.front().reminder_waypoint_lane_id_list;
            this->globalInfoContainer.m_TgtRoadTyp_RWP = BA::TargetRoadTyp(ehr_msg.reminder_waypoint.front().reminder_waypoint_target_road_type);
            this->globalInfoContainer.m_WpExtendLaneLst = ehr_msg.reminder_waypoint.front().reminder_wp_extendlane_list;
            this->globalInfoContainer.m_SpeedLimit = ehr_msg.reminder_waypoint.front().reminder_waypoint_speed_limit;
            if (!GetCnrLaneID(ehr_msg.reminder_waypoint.front(), this->globalInfoContainer.m_LftCrnrLaneID, this->globalInfoContainer.m_RghtCrnrLaneID))
            {
                DLOG(WARNING) << "Getting corner lane id failed!" << std::endl;
            }
            routeRltvPos = GetRouteRltvPos(this->globalInfoContainer.m_TgtLaneLst_RWP, this->globalInfoContainer.m_LftCrnrLaneID, this->globalInfoContainer.m_RghtCrnrLaneID);
            this->globalInfoContainer.m_RouteRltvPos = routeRltvPos;
            this->globalInfoContainer.m_GlobalLaneChgDrct = DtrmLCDrcn(minLaneGapRWP);
            this->globalInfoContainer.m_GlobalLaneChgNum = std::abs(minLaneGapRWP);
            
            // rebuild transit lane list when necessary
            rebuildTransLnList(ehr_msg, toRebuildTransitLnLst);
            // if reminder waypoint has influence, fill reminder waypoint info inside
            // init m_GlobalLaneInfoArr, ID is from RWP target lane ID list
            for (int i = 0; i < ehr_msg.reminder_waypoint.front().reminder_wp_extendlane_list.size(); i++)
            {
                BA::GlobalLaneInfo globalLnInfo_;
                if (InitGlobalLnInfo(ehr_msg.reminder_waypoint.front(), ehr_msg.transit_lanes_list, ehr_msg.reminder_waypoint.front().reminder_wp_extendlane_list[i].lane_id, globalLnInfo_, vehSpd))
                {
                    this->globalInfoContainer.m_GlobalLaneInfoArr.emplace_back(globalLnInfo_);
                }
            }
        }
        else
        {
            this->globalInfoContainer.m_GlobalInfoSrc = BA::GlobalInfoSrc_OnlyVirtCtrlWayPoint;
            ControlWayPointStruct firstCtrlWayPnt_;
            if (GetFirstCtrlWP(ehr_msg, firstCtrlWayPnt_))
            {
                this->globalInfoContainer.m_WayPointID = firstCtrlWayPnt_.control_waypoint_id;
                this->globalInfoContainer.m_DistToWayPoint = firstCtrlWayPnt_.control_waypoint_offset;
                this->globalInfoContainer.m_RawTgtLaneLst_CWP = firstCtrlWayPnt_.control_waypoint_raw_lane_id_list;
                this->globalInfoContainer.m_TgtRoadTyp_CWP = BA::TargetRoadTyp(firstCtrlWayPnt_.control_waypoint_target_road_type);
                this->globalInfoContainer.m_WpExtendLaneLst = firstCtrlWayPnt_.control_wp_extendlane_list;
                this->globalInfoContainer.m_SpeedLimit = firstCtrlWayPnt_.control_waypoint_speed_limit;
                if (!GetCnrLaneID(firstCtrlWayPnt_, this->globalInfoContainer.m_LftCrnrLaneID, this->globalInfoContainer.m_RghtCrnrLaneID, ehr_msg.reminder_waypoint.front(), roadModel.car_position.ego_lane_id))
                {
                    DLOG(WARNING) << "Getting corner lane id failed!" << std::endl;
                }
                routeRltvPos = GetRouteRltvPos(this->globalInfoContainer.m_RawTgtLaneLst_CWP, this->globalInfoContainer.m_LftCrnrLaneID, this->globalInfoContainer.m_RghtCrnrLaneID);
                this->globalInfoContainer.m_RouteRltvPos = routeRltvPos;
                minLaneGapCWP = CalcLaneGap(roadModel.car_position.ego_lane_id, 
                                            firstCtrlWayPnt_.control_wp_extendlane_list, 
                                            firstCtrlWayPnt_.control_waypoint_lane_id_list,
                                            this->globalInfoContainer.m_MinLnGapTgtLaneLst);
                this->globalInfoContainer.m_GlobalLaneChgDrct = DtrmLCDrcn(minLaneGapCWP);
                this->globalInfoContainer.m_GlobalLaneChgNum = std::abs(minLaneGapCWP);
                this->globalInfoContainer.m_TgtLaneLst_CWP = firstCtrlWayPnt_.control_waypoint_lane_id_list;
                // rebuild transit lane list when necessary
                rebuildTransLnList(ehr_msg, toRebuildTransitLnLst);
                // if reminder waypoint has not influenced, fill control waypoint info inside
                // init m_GlobalLaneInfoArr, ID is from CWP target lane ID list
                for (int i = 0; i < firstCtrlWayPnt_.control_wp_extendlane_list.size(); i++)
                {
                    BA::GlobalLaneInfo globalLnInfo_;
                    if (InitGlobalLnInfo(firstCtrlWayPnt_, ehr_msg.transit_lanes_list, firstCtrlWayPnt_.control_wp_extendlane_list[i].lane_id, globalLnInfo_, vehSpd))
                    {
                        this->globalInfoContainer.m_GlobalLaneInfoArr.emplace_back(globalLnInfo_);
                    }
                }
            }
            else
            {
                DLOG(WARNING) << "Getting First Control Waypoint failed!" << std::endl;
            }
        }
    }
    else if (ehr_msg.control_waypoints_list.at(0).control_waypoint_id != 300 && ehr_msg.reminder_waypoint.at(0).reminder_waypoint_id != 0) // both actual control and reminder waypoints exist
    {
        // this->globalInfoContainer.m_GlobalInfoSrc = BA::GlobalInfoSrc_BothWayPoint;
        // if minimum lane gap for RWP is not zero, then initialization is based on first reminder waypoint
        // or initialization is based on first control waypoint
        int minLaneGapRWP = CalcLaneGap(roadModel.car_position.ego_lane_id, 
                                        ehr_msg.reminder_waypoint.front().reminder_wp_extendlane_list, 
                                        ehr_msg.reminder_waypoint.front().reminder_waypoint_lane_id_list,
                                        this->globalInfoContainer.m_MinLnGapTgtLaneLst);
        if (abs(minLaneGapRWP) > 0 && minLaneGapRWP != 100)
        {
            this->globalInfoContainer.m_GlobalInfoSrc = BA::GlobalInfoSrc_OnlyRmdWayPoint;
            this->globalInfoContainer.m_WayPointID = ehr_msg.reminder_waypoint.at(0).reminder_waypoint_id;
            this->globalInfoContainer.m_DistToWayPoint = ehr_msg.reminder_waypoint.at(0).reminder_waypoint_offset;
            this->globalInfoContainer.m_TgtLaneLst_RWP = ehr_msg.reminder_waypoint.front().reminder_waypoint_lane_id_list;
            this->globalInfoContainer.m_TgtRoadTyp_RWP = BA::TargetRoadTyp(ehr_msg.reminder_waypoint.front().reminder_waypoint_target_road_type);
            this->globalInfoContainer.m_WpExtendLaneLst = ehr_msg.reminder_waypoint.front().reminder_wp_extendlane_list;
            this->globalInfoContainer.m_SpeedLimit = ehr_msg.reminder_waypoint.front().reminder_waypoint_speed_limit;
            if (!GetCnrLaneID(ehr_msg.reminder_waypoint.front(), this->globalInfoContainer.m_LftCrnrLaneID, this->globalInfoContainer.m_RghtCrnrLaneID))
            {
                DLOG(WARNING) << "Getting corner lane id failed!" << std::endl;
            }
            routeRltvPos = GetRouteRltvPos(this->globalInfoContainer.m_TgtLaneLst_RWP, this->globalInfoContainer.m_LftCrnrLaneID, this->globalInfoContainer.m_RghtCrnrLaneID);
            this->globalInfoContainer.m_RouteRltvPos = routeRltvPos;
            this->globalInfoContainer.m_GlobalLaneChgDrct = DtrmLCDrcn(minLaneGapRWP);
            this->globalInfoContainer.m_GlobalLaneChgNum = std::abs(minLaneGapRWP);
            // rebuild transit lane list when necessary
            rebuildTransLnList(ehr_msg, toRebuildTransitLnLst);
            // if reminder waypoint has influence, fill reminder waypoint info inside
            // init m_GlobalLaneInfoArr, ID is from RWP target lane ID list
            for (int i = 0; i < ehr_msg.reminder_waypoint.front().reminder_wp_extendlane_list.size(); i++)
            {
                BA::GlobalLaneInfo globalLnInfo_;
                if (InitGlobalLnInfo(ehr_msg.reminder_waypoint.front(), ehr_msg.transit_lanes_list, ehr_msg.reminder_waypoint.front().reminder_wp_extendlane_list[i].lane_id, globalLnInfo_, vehSpd))
                {
                    this->globalInfoContainer.m_GlobalLaneInfoArr.emplace_back(globalLnInfo_);
                }
            }
        }
        else
        {
            this->globalInfoContainer.m_GlobalInfoSrc = BA::GlobalInfoSrc_OnlyActCtrlWayPoint;
            ControlWayPointStruct firstCtrlWayPnt_;
            if (GetFirstCtrlWP(ehr_msg, firstCtrlWayPnt_))
            {
                this->globalInfoContainer.m_WayPointID = firstCtrlWayPnt_.control_waypoint_id;
                this->globalInfoContainer.m_DistToWayPoint = firstCtrlWayPnt_.control_waypoint_offset;
                this->globalInfoContainer.m_RawTgtLaneLst_CWP = firstCtrlWayPnt_.control_waypoint_raw_lane_id_list;
                this->globalInfoContainer.m_TgtRoadTyp_CWP = BA::TargetRoadTyp(firstCtrlWayPnt_.control_waypoint_target_road_type);
                this->globalInfoContainer.m_WpExtendLaneLst = firstCtrlWayPnt_.control_wp_extendlane_list;
                this->globalInfoContainer.m_SpeedLimit = firstCtrlWayPnt_.control_waypoint_speed_limit;
                if (!GetCnrLaneID(firstCtrlWayPnt_, this->globalInfoContainer.m_LftCrnrLaneID, this->globalInfoContainer.m_RghtCrnrLaneID, ehr_msg.reminder_waypoint.front(), roadModel.car_position.ego_lane_id))
                {
                    DLOG(WARNING) << "Getting corner lane id failed!" << std::endl;
                    DLOG(INFO) << "33333" << std::endl;
                }
                routeRltvPos = GetRouteRltvPos(this->globalInfoContainer.m_RawTgtLaneLst_CWP, this->globalInfoContainer.m_LftCrnrLaneID, this->globalInfoContainer.m_RghtCrnrLaneID);
                this->globalInfoContainer.m_RouteRltvPos = routeRltvPos;
                bool isTgtLnLstMdf = MdfTgtLnLst(firstCtrlWayPnt_.control_waypoint_lane_id_list, routeRltvPos, ehr_msg, roadModel);
                minLaneGapCWP = CalcLaneGap(roadModel.car_position.ego_lane_id, 
                                            firstCtrlWayPnt_.control_wp_extendlane_list, 
                                            firstCtrlWayPnt_.control_waypoint_lane_id_list,
                                            this->globalInfoContainer.m_MinLnGapTgtLaneLst);
                this->globalInfoContainer.m_GlobalLaneChgDrct = DtrmLCDrcn(minLaneGapCWP);
                this->globalInfoContainer.m_GlobalLaneChgNum = std::abs(minLaneGapCWP);
                this->globalInfoContainer.m_TgtLaneLst_CWP = firstCtrlWayPnt_.control_waypoint_lane_id_list;
                // rebuild transit lane list when necessary
                rebuildTransLnList(ehr_msg, toRebuildTransitLnLst);
                // if reminder waypoint has not influenced, fill control waypoint info inside
                // init m_GlobalLaneInfoArr, ID is from CWP target lane ID list
                for (int i = 0; i < firstCtrlWayPnt_.control_wp_extendlane_list.size(); i++)
                {
                    BA::GlobalLaneInfo globalLnInfo_;
                    if (InitGlobalLnInfo(firstCtrlWayPnt_, ehr_msg.transit_lanes_list, firstCtrlWayPnt_.control_wp_extendlane_list[i].lane_id, globalLnInfo_, vehSpd))
                    {
                        this->globalInfoContainer.m_GlobalLaneInfoArr.emplace_back(globalLnInfo_);
                    }
                }
            }
            else
            {
                DLOG(WARNING) << "Getting First Control Waypoint failed!" << std::endl;
            }
        }
    }
    else // no valid waypoint exists
    {
        this->globalInfoContainer.m_GlobalInfoSrc = BA::GlobalInfoSrc_NoVldWayPoint;
        this->globalInfoContainer.m_WayPointID = uint8_t(0);
    }
}

int GlobalInfoProcess::CalcLaneGap(const uint8_t &egoLaneID, 
                                   const std::vector<WPExtendLaneStruct> &wpExtendLaneLst, 
                                   const std::vector<uint8_t> &trgtLaneLst,
                                   std::vector<uint8_t> &minLnGapLaneLst)
{
    minLnGapLaneLst.clear();
    if (!trgtLaneLst.empty())
    {
        int maxLaneGap = 0;
        std::vector<int> laneGapLst, laneGapAbsLst;
        for (int i = 0; i < trgtLaneLst.size(); i++)
        {
            int tmpLaneGap = CalcGapBtwLns(wpExtendLaneLst, egoLaneID, trgtLaneLst.at(i));
            // DLOG(INFO) << "i = " << i << std::endl;
            // DLOG(INFO) << "tmpLaneGap = " << tmpLaneGap << std::endl;
            laneGapLst.emplace_back(tmpLaneGap);
            laneGapAbsLst.emplace_back(abs(tmpLaneGap));
        }
        std::vector<int>::iterator min_iter = min_element(begin(laneGapAbsLst), end(laneGapAbsLst));
        int min_LaneGap = *min_iter;
        int min_LaneGap_Loc = distance(begin(laneGapAbsLst), min_iter);
        DLOG(INFO) << "min_LaneGap_Loc = " << min_LaneGap_Loc << std::endl;
        uint8_t min_LaneGap_TgtLn = trgtLaneLst[min_LaneGap_Loc];
        minLnGapLaneLst.emplace_back(min_LaneGap_TgtLn);
        DLOG(INFO) << "target lane id with min_LaneGap = " << int(min_LaneGap_TgtLn) << std::endl;
        return laneGapLst[min_LaneGap_Loc];
    }
    else
    {
        DLOG(WARNING) << "target lane list is empty!!" << std::endl;
        return 100;
    }
}

BA::LaneChgDrct GlobalInfoProcess::DtrmLCDrcn(int minLaneGap)
{
    if (minLaneGap > 0)
    {
        return BA::LaneChgDrct_Right;
    }
    else if (minLaneGap < 0)
    {
        return BA::LaneChgDrct_Left;
    }
    else
    {
        return BA::LaneChgDrct_None;
    }
}

bool GlobalInfoProcess::GetFirstCtrlWP(const ControlWaypoints ehr_msg, ControlWayPointStruct &firstCtrlWayPnt)
{
    if (ehr_msg.control_waypoints_list.at(0).control_waypoint_id != 0)
    {
        if (ehr_msg.control_waypoints_list.at(1).control_waypoint_id != 0)
        {
            if (ehr_msg.control_waypoints_list.at(0).control_waypoint_offset <= ehr_msg.control_waypoints_list.at(1).control_waypoint_offset)
            {
                firstCtrlWayPnt = ehr_msg.control_waypoints_list.at(0);
            }
            else
            {
                firstCtrlWayPnt = ehr_msg.control_waypoints_list.at(1);
            }
        }
        else
        {
            firstCtrlWayPnt = ehr_msg.control_waypoints_list.at(0);
        }
        return true;
    }
    else
    {
        DLOG(WARNING) << "control waypoints list is empty!" << std::endl;
        return false;
    }
}

bool GlobalInfoProcess::GetCnrLaneID(const ControlWayPointStruct &ctrlWayPnt,
                                     uint8_t &lftCrnrLaneID,
                                     uint8_t &rghtCrnrLaneID,
                                     const RemindWayPointStruct &rmdWayPnt,
                                     const uint8_t &egoLaneID)
{
    // 20220712: remove extendLanes leading to non-target ramp
    std::vector<uint8_t> subSpltLnIDLst;
    std::vector<uint8_t> subMrgLnIDLst;
    subSpltLnIDLst.clear();
    subMrgLnIDLst.clear();
    /* int ctrlTrgtLnNum = ctrlWayPnt.control_waypoint_raw_lane_id_list.size();
    int rmdTrgtLnNum = 0;
    for (auto it_rmdWpLn = rmdWayPnt.reminder_wp_extendlane_list.begin(); it_rmdWpLn != rmdWayPnt.reminder_wp_extendlane_list.end(); it_rmdWpLn++)
    {
        auto it_rmdLn = std::find(rmdWayPnt.reminder_waypoint_lane_id_list.begin(), rmdWayPnt.reminder_waypoint_lane_id_list.end(), it_rmdWpLn->lane_id);
        if(it_rmdLn == rmdWayPnt.reminder_waypoint_lane_id_list.end() && it_rmdWpLn->is_emergency_lane == uint8_t(0))
        {
            rmdTrgtLnNum++;
        }
    }
    DLOG(INFO) << "ctrlTrgtLnNum = " << ctrlTrgtLnNum << std::endl;
    DLOG(INFO) << "rmdTrgtLnNum = " << rmdTrgtLnNum << std::endl; */
    bool isRmdNonTrgtLnInCtrlRawTrgtLnLst = false;
   
    if (rmdWayPnt.reminder_waypoint_target_road_type != BA::TargetRoadTyp_NotAvlbl && 
        (!rmdWayPnt.reminder_wp_extendlane_list.empty()) &&
        rmdWayPnt.reminder_waypoint_offset < ctrlWayPnt.control_waypoint_offset && 
        (!rmdWayPnt.reminder_waypoint_lane_id_list.empty()))
    {
        if ((ctrlWayPnt.control_waypoint_target_road_type == BA::TargetRoadTyp_ICRd ||
            ctrlWayPnt.control_waypoint_target_road_type == BA::TargetRoadTyp_JCTRd ||
            ctrlWayPnt.control_waypoint_target_road_type == BA::TargetRoadTyp_SpltRd) &&
            rmdWayPnt.reminder_waypoint_target_road_type == BA::TargetRoadTyp_SubMrgRd)
        {
            // 20221013
            // check whether certain non-target lane of rmd wp belongs to raw target lane list
            for (auto it_rmdWpExtLn = rmdWayPnt.reminder_wp_extendlane_list.begin(); it_rmdWpExtLn != rmdWayPnt.reminder_wp_extendlane_list.end(); it_rmdWpExtLn++)
            {
                auto itr = std::find(rmdWayPnt.reminder_waypoint_lane_id_list.begin(), rmdWayPnt.reminder_waypoint_lane_id_list.end(), it_rmdWpExtLn->lane_id);
                if (itr == rmdWayPnt.reminder_waypoint_lane_id_list.end())
                {
                    subMrgLnIDLst.emplace_back(it_rmdWpExtLn->lane_id);
                }
            }
            for (auto subMrgLnID : subMrgLnIDLst)
            {
                auto it_subMrgLnInCtrlTrgtLnLst = std::find(ctrlWayPnt.control_waypoint_raw_lane_id_list.begin(), ctrlWayPnt.control_waypoint_raw_lane_id_list.end(), subMrgLnID);
                if (it_subMrgLnInCtrlTrgtLnLst != ctrlWayPnt.control_waypoint_raw_lane_id_list.end())
                {
                    isRmdNonTrgtLnInCtrlRawTrgtLnLst = true;
                }
            }
        }
        for (auto it_rmdWpExtLn = rmdWayPnt.reminder_wp_extendlane_list.begin(); it_rmdWpExtLn != rmdWayPnt.reminder_wp_extendlane_list.end(); it_rmdWpExtLn++)
        {
            auto iter = std::find(rmdWayPnt.reminder_waypoint_lane_id_list.begin(), rmdWayPnt.reminder_waypoint_lane_id_list.end(), it_rmdWpExtLn->lane_id);
            // 20221009 added
            // exclude ego lane and lane with less end offset than reminder waypoint offset
            // 20221012 added
            // if ctrl wp type is JCTRd or ICRd or SpltRd and raw target lane number of ctrl wp is equal to non-target lane number of rmd wp(excluding emergency lane)
            // then exclude all non-target lanes in rmd wp
            if (iter == rmdWayPnt.reminder_waypoint_lane_id_list.end() && 
                it_rmdWpExtLn->lane_id != egoLaneID && 
                it_rmdWpExtLn->current_extendlane_endoffset >= rmdWayPnt.reminder_waypoint_offset)
            {
                if ((ctrlWayPnt.control_waypoint_target_road_type == BA::TargetRoadTyp_ICRd ||
                    ctrlWayPnt.control_waypoint_target_road_type == BA::TargetRoadTyp_JCTRd ||
                    ctrlWayPnt.control_waypoint_target_road_type == BA::TargetRoadTyp_SpltRd) &&
                    rmdWayPnt.reminder_waypoint_target_road_type == BA::TargetRoadTyp_SubMrgRd)
                {
                    if (isRmdNonTrgtLnInCtrlRawTrgtLnLst == false)
                    {
                        subSpltLnIDLst.emplace_back(it_rmdWpExtLn->lane_id);
                    }
                }
                else
                {
                    subSpltLnIDLst.emplace_back(it_rmdWpExtLn->lane_id);
                }
            }
        }
    }
    DLOG(INFO) << " isRmdNonTrgtLnInCtrlRawTrgtLnLst: " << int(isRmdNonTrgtLnInCtrlRawTrgtLnLst) << std::endl;
    DLOG(INFO) << " subSpltLnIDLst below: " << std::endl;
    #ifdef GLOG
    for (auto it : subSpltLnIDLst)
    {
        DLOG(INFO) << int(it) << std::endl;
    }
    #endif
    std::vector<WPExtendLaneStruct> extendLaneLst = ctrlWayPnt.control_wp_extendlane_list;
    DLOG(INFO) << "original ctrl extendLaneLst size = " << extendLaneLst.size() << std::endl;
    // 20220727 remove unnecessary extend lane which is not drivable
    /* std::vector<uint8_t> emrgLnIDLst;
    std::vector<float> emrgLnOffsetLst;
    std::vector<float> emrgLnEndOffsetLst;
    for (auto extLn : extendLaneLst)
    {
        if (extLn.is_emergency_lane == uint8_t(1))
        {
            emrgLnIDLst.emplace_back(extLn.lane_id);
            emrgLnOffsetLst.emplace_back(extLn.current_extendlane_offset);
            emrgLnEndOffsetLst.emplace_back(extLn.current_extendlane_endoffset);
        }
    }
    if (emrgLnIDLst.size() > 1 && emrgLnEndOffsetLst.size() > 1 && emrgLnOffsetLst.size() > 1)
    {
        // erase unnecessary extend lane in different case
        uint8_t saveExtLnID = uint8_t(0);
        if (this->globalInfoContainer.m_TgtRoadTyp_CWP == BA::TargetRoadTyp_ICRd || 
            this->globalInfoContainer.m_TgtRoadTyp_CWP == BA::TargetRoadTyp_JCTRd || 
            this->globalInfoContainer.m_TgtRoadTyp_CWP == BA::TargetRoadTyp_RmpDvdRd || 
            this->globalInfoContainer.m_TgtRoadTyp_RWP == BA::TargetRoadTyp_SubSpltRd ||
            this->globalInfoContainer.m_TgtRoadTyp_CWP == BA::TargetRoadTyp_SpltRd)
        {
            // keep extend lane with max end offset
            int max_num = std::max_element(emrgLnEndOffsetLst.begin(), emrgLnEndOffsetLst.end()) - emrgLnEndOffsetLst.begin();
            saveExtLnID = emrgLnIDLst.at(max_num);
        }
        else if (this->globalInfoContainer.m_TgtRoadTyp_CWP == BA::TargetRoadTyp_RmpMrgCARd ||
                 this->globalInfoContainer.m_TgtRoadTyp_RWP == BA::TargetRoadTyp_SubMrgRd)
        {
            // keep extend lane with min offset
            int min_num = std::min_element(emrgLnOffsetLst.begin(), emrgLnOffsetLst.end()) - emrgLnOffsetLst.begin();
            saveExtLnID = emrgLnIDLst.at(min_num);
        }
        if (saveExtLnID != uint8_t(0))
        {
            // remove it from emrgLnIDLst which will be erased from extandLaneLst
            for (auto it_ermgLn = emrgLnIDLst.begin(); it_ermgLn != emrgLnIDLst.end();)
            {
                if (saveExtLnID == *it_ermgLn)
                {
                   it_ermgLn = emrgLnIDLst.erase(it_ermgLn); 
                }
                else
                {
                   it_ermgLn++;
                }
            }
            for (auto it_extLn = extendLaneLst.begin(); it_extLn != extendLaneLst.end();)
            {
                auto iter = std::find(emrgLnIDLst.begin(), emrgLnIDLst.end(), it_extLn->lane_id);
                if (iter != emrgLnIDLst.end())
                {
                    it_extLn = extendLaneLst.erase(it_extLn);
                }
                else
                {
                    it_extLn++;
                }
            }
        } 
    } */
    // DLOG(INFO) << " filtered ctrl extendLaneLst size = " << extendLaneLst.size() << std::endl;
    // 20220712
    if (!subSpltLnIDLst.empty())
    {
        for (auto it_extLn = extendLaneLst.begin(); it_extLn != extendLaneLst.end();)
        {
            auto iter = std::find(subSpltLnIDLst.begin(), subSpltLnIDLst.end(), it_extLn->lane_id);
            if (iter != subSpltLnIDLst.end())
            {
                // remove the extLn
                it_extLn = extendLaneLst.erase(it_extLn);
            }
            else
            {
                it_extLn++;
            }
        }
    }
    DLOG(INFO) << "processed ctrl extendLaneLst size = " << extendLaneLst.size() << std::endl;
    int sizeOfCWPLnLst = extendLaneLst.size();
    if (sizeOfCWPLnLst == 0)
    {
        DLOG(WARNING) << "ExtendLaneList of control waypoint is empty!" << std::endl;
        return false;
    }
    else
    {
        if (sizeOfCWPLnLst == 1)
        {
            lftCrnrLaneID = extendLaneLst.front().lane_id;
            rghtCrnrLaneID = extendLaneLst.front().lane_id;
            return true;
        }
        else
        {
            bool lftCrnrLnFound = false;
            bool rghtCrnrLnFound = false;
            for (int i = 0; i < sizeOfCWPLnLst; i++)
            {
                if (!lftCrnrLnFound && extendLaneLst.at(i).left_extend_lane.empty())
                {
                    lftCrnrLaneID = extendLaneLst.at(i).lane_id;
                    if (extendLaneLst.at(i).is_emergency_lane == 1)
                    {
                        // if actual left corner lane is emergency lane
                        // then the lane with the greatest right adjacent_extendlane_offset is the left corner lane among normal lanes
                        std::vector<AdjacentExtendLaneStruct> adjRghtLnLst = extendLaneLst.at(i).right_extend_lane;
                        if (!adjRghtLnLst.empty())
                        {
                            if (adjRghtLnLst.size() == 1)
                            {
                                lftCrnrLaneID = adjRghtLnLst.front().adjacent_extendlane_id;
                            }
                            else
                            {
                                if (this->globalInfoContainer.m_TgtRoadTyp_CWP == BA::TargetRoadTyp_ICRd || 
                                    this->globalInfoContainer.m_TgtRoadTyp_CWP == BA::TargetRoadTyp_JCTRd || 
                                    this->globalInfoContainer.m_TgtRoadTyp_CWP == BA::TargetRoadTyp_RmpDvdRd || 
                                    this->globalInfoContainer.m_TgtRoadTyp_RWP == BA::TargetRoadTyp_SubSpltRd ||
                                    this->globalInfoContainer.m_TgtRoadTyp_CWP == BA::TargetRoadTyp_SpltRd)
                                {
                                    // split structure
                                    // find out the the lane with greatest left adjacent_extendlane_offset
                                    float maxStOffset = -3000.0;
                                    lftCrnrLaneID = 0;
                                    for (auto adjRghtLn : adjRghtLnLst)
                                    {
                                        WPExtendLaneStruct tmpWpExtendLnObj;
                                        if (FindWPExtendObj(this->globalInfoContainer.m_WpExtendLaneLst, adjRghtLn.adjacent_extendlane_id, tmpWpExtendLnObj) == true)
                                        {
                                            if (maxStOffset < adjRghtLn.adjacent_extendlane_offset && tmpWpExtendLnObj.current_extendlane_endoffset >= this->globalInfoContainer.m_DistToWayPoint)
                                            {
                                                lftCrnrLaneID = adjRghtLn.adjacent_extendlane_id;
                                                maxStOffset = adjRghtLn.adjacent_extendlane_offset;
                                            }
                                        }
                                    }
                                }
                                else if (this->globalInfoContainer.m_TgtRoadTyp_CWP == BA::TargetRoadTyp_RmpMrgCARd || this->globalInfoContainer.m_TgtRoadTyp_RWP == BA::TargetRoadTyp_SubMrgRd)
                                {
                                    // merge structure
                                    // find out the the lane with least left AdjacentExtendLaneEndOffset
                                    float minEndOffset = 3000.0;
                                    lftCrnrLaneID = 0;
                                    for (auto adjRghtLn : adjRghtLnLst)
                                    {
                                        WPExtendLaneStruct tmpWpExtendLnObj;
                                        if (FindWPExtendObj(this->globalInfoContainer.m_WpExtendLaneLst, adjRghtLn.adjacent_extendlane_id, tmpWpExtendLnObj) == true)
                                        {
                                            if (minEndOffset > adjRghtLn.adjacent_extendlane_endoffset /* && tmpWpExtendLnObj.current_extendlane_offset <= 0 */)
                                            {
                                                lftCrnrLaneID = adjRghtLn.adjacent_extendlane_id;
                                                minEndOffset = adjRghtLn.adjacent_extendlane_endoffset;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        else
                        {
                            DLOG(WARNING) << "adjacent right lane list is empty!" << std::endl;
                        }
                    }
                    lftCrnrLnFound = true;
                }
                if (!rghtCrnrLnFound && extendLaneLst.at(i).right_extend_lane.empty())
                {
                    rghtCrnrLaneID = extendLaneLst.at(i).lane_id;
                    if (extendLaneLst.at(i).is_emergency_lane == 1)
                    {
                        // if actual right corner lane is emergency lane
                        // then the lane with the greatest left adjacent_extendlane_offset is the right corner lane among normal lanes
                        std::vector<AdjacentExtendLaneStruct> adjLftLnLst = extendLaneLst.at(i).left_extend_lane;
                        if (!adjLftLnLst.empty())
                        {
                            // based on target road type
                            if (adjLftLnLst.size() == 1)
                            {
                                rghtCrnrLaneID = adjLftLnLst.front().adjacent_extendlane_id;
                            }
                            else
                            {

                                if (this->globalInfoContainer.m_TgtRoadTyp_CWP == BA::TargetRoadTyp_ICRd ||
                                    this->globalInfoContainer.m_TgtRoadTyp_CWP == BA::TargetRoadTyp_JCTRd ||
                                    this->globalInfoContainer.m_TgtRoadTyp_CWP == BA::TargetRoadTyp_RmpDvdRd ||
                                    this->globalInfoContainer.m_TgtRoadTyp_RWP == BA::TargetRoadTyp_SubSpltRd ||
                                    this->globalInfoContainer.m_TgtRoadTyp_CWP == BA::TargetRoadTyp_SpltRd)
                                {
                                    // split structure
                                    // find out the the lane with greatest left adjacent_extendlane_offset
                                    float maxStOffset = -3000.0;
                                    rghtCrnrLaneID = 0;
                                    for (auto adjLftLn : adjLftLnLst)
                                    {
                                        WPExtendLaneStruct tmpWpExtendLnObj;
                                        if (FindWPExtendObj(this->globalInfoContainer.m_WpExtendLaneLst, adjLftLn.adjacent_extendlane_id, tmpWpExtendLnObj) == true)
                                        {
                                            if (maxStOffset < adjLftLn.adjacent_extendlane_offset && tmpWpExtendLnObj.current_extendlane_endoffset >= this->globalInfoContainer.m_DistToWayPoint)
                                            {
                                                rghtCrnrLaneID = adjLftLn.adjacent_extendlane_id;
                                                maxStOffset = adjLftLn.adjacent_extendlane_offset;
                                            }
                                        }
                                    }
                                }
                                else if (this->globalInfoContainer.m_TgtRoadTyp_CWP == BA::TargetRoadTyp_RmpMrgCARd || this->globalInfoContainer.m_TgtRoadTyp_RWP == BA::TargetRoadTyp_SubMrgRd)
                                {
                                    // merge structure
                                    // find out the the lane with least left AdjacentExtendLaneEndOffset
                                    float minEndOffset = 3000.0;
                                    rghtCrnrLaneID = 0;
                                    for (auto adjLftLn : adjLftLnLst)
                                    {
                                        WPExtendLaneStruct tmpWpExtendLnObj;
                                        if (FindWPExtendObj(this->globalInfoContainer.m_WpExtendLaneLst, adjLftLn.adjacent_extendlane_id, tmpWpExtendLnObj) == true)
                                        {
                                            if (minEndOffset > adjLftLn.adjacent_extendlane_endoffset /* && tmpWpExtendLnObj.current_extendlane_offset <= 0 */)
                                            {
                                                rghtCrnrLaneID = adjLftLn.adjacent_extendlane_id;
                                                minEndOffset = adjLftLn.adjacent_extendlane_endoffset;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        else
                        {
                            DLOG(WARNING) << "adjacent left lane list is empty!" << std::endl;
                        }
                    }
                    rghtCrnrLnFound = true;
                }
            }
            if (rghtCrnrLnFound == false && (!ctrlWayPnt.control_waypoint_lane_id_list.empty()))
            {
                rghtCrnrLaneID = ctrlWayPnt.control_waypoint_lane_id_list.front();
                rghtCrnrLnFound = true;
            }
            // 20221013: set right corner lane id compulsorily
            if ((ctrlWayPnt.control_waypoint_target_road_type == BA::TargetRoadTyp_ICRd ||
                ctrlWayPnt.control_waypoint_target_road_type == BA::TargetRoadTyp_JCTRd ||
                ctrlWayPnt.control_waypoint_target_road_type == BA::TargetRoadTyp_SpltRd) &&
                rmdWayPnt.reminder_waypoint_target_road_type == BA::TargetRoadTyp_SubMrgRd &&
                isRmdNonTrgtLnInCtrlRawTrgtLnLst == true)
            {
                if (!ctrlWayPnt.control_waypoint_lane_id_list.empty())
                {
                    rghtCrnrLaneID = ctrlWayPnt.control_waypoint_lane_id_list.front();
                }
            }
            return (lftCrnrLnFound && rghtCrnrLnFound);
        }
    }
}

bool GlobalInfoProcess::GetCnrLaneID(const RemindWayPointStruct &rmdWayPnt,
                                     uint8_t &lftCrnrLaneID,
                                     uint8_t &rghtCrnrLaneID)
{
    std::vector<WPExtendLaneStruct> extendLaneLst = rmdWayPnt.reminder_wp_extendlane_list;
    int sizeOfRWPLnLst = extendLaneLst.size();
    if (sizeOfRWPLnLst == 0)
    {
        DLOG(WARNING) << "ExtendLaneList of reminder waypoint is empty!" << std::endl;
        return false;
    }
    else
    {
        if (sizeOfRWPLnLst == 1)
        {
            lftCrnrLaneID = extendLaneLst.front().lane_id;
            rghtCrnrLaneID = extendLaneLst.front().lane_id;
        }
        else
        {
            bool lftCrnrLnFound = false;
            bool rghtCrnrLnFound = false;
            for (int i = 0; i < sizeOfRWPLnLst; i++)
            {
                if (!lftCrnrLnFound && extendLaneLst.at(i).left_extend_lane.empty())
                {
                    lftCrnrLaneID = extendLaneLst.at(i).lane_id;
                    // 20221016: if found corner lane id is emergency lane, then try to search the corner lane again
                    if (extendLaneLst.at(i).is_emergency_lane == 1)
                    {
                        // if actual left corner lane is emergency lane
                        // then the lane with the greatest right adjacent_extendlane_offset is the left corner lane among normal lanes
                        std::vector<AdjacentExtendLaneStruct> adjRghtLnLst = extendLaneLst.at(i).right_extend_lane;
                        if (!adjRghtLnLst.empty())
                        {
                            if (adjRghtLnLst.size() == 1)
                            {
                                lftCrnrLaneID = adjRghtLnLst.front().adjacent_extendlane_id;
                            }
                            else
                            {
                                if (this->globalInfoContainer.m_TgtRoadTyp_RWP == BA::TargetRoadTyp_SubSpltRd)
                                {
                                    // split structure
                                    // find out the the lane with greatest left adjacent_extendlane_offset
                                    float maxStOffset = -3000.0;
                                    lftCrnrLaneID = 0;
                                    for (auto adjRghtLn : adjRghtLnLst)
                                    {
                                        WPExtendLaneStruct tmpWpExtendLnObj;
                                        if (FindWPExtendObj(this->globalInfoContainer.m_WpExtendLaneLst, adjRghtLn.adjacent_extendlane_id, tmpWpExtendLnObj) == true)
                                        {
                                            if (maxStOffset < adjRghtLn.adjacent_extendlane_offset && tmpWpExtendLnObj.current_extendlane_endoffset >= this->globalInfoContainer.m_DistToWayPoint)
                                            {
                                                lftCrnrLaneID = adjRghtLn.adjacent_extendlane_id;
                                                maxStOffset = adjRghtLn.adjacent_extendlane_offset;
                                            }
                                        }
                                    }
                                }
                                else if (this->globalInfoContainer.m_TgtRoadTyp_RWP == BA::TargetRoadTyp_SubMrgRd)
                                {
                                    // merge structure
                                    // find out the the lane with least left AdjacentExtendLaneEndOffset
                                    float minEndOffset = 3000.0;
                                    lftCrnrLaneID = 0;
                                    for (auto adjRghtLn : adjRghtLnLst)
                                    {
                                        WPExtendLaneStruct tmpWpExtendLnObj;
                                        if (FindWPExtendObj(this->globalInfoContainer.m_WpExtendLaneLst, adjRghtLn.adjacent_extendlane_id, tmpWpExtendLnObj) == true)
                                        {
                                            if (minEndOffset > adjRghtLn.adjacent_extendlane_endoffset /* && tmpWpExtendLnObj.current_extendlane_offset <= 0 */)
                                            {
                                                lftCrnrLaneID = adjRghtLn.adjacent_extendlane_id;
                                                minEndOffset = adjRghtLn.adjacent_extendlane_endoffset;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        else
                        {
                            DLOG(WARNING) << "adjacent right lane list is empty!" << std::endl;
                        }
                    }
                    lftCrnrLnFound = true;
                }
                if (!rghtCrnrLnFound && extendLaneLst.at(i).right_extend_lane.empty())
                {
                    rghtCrnrLaneID = extendLaneLst.at(i).lane_id;
                    if (extendLaneLst.at(i).is_emergency_lane == 1)
                    {
                        // if actual right corner lane is emergency lane
                        // then the lane with the smallest left adjacent_extendlane_offset is the right corner lane among normal lanes
                        std::vector<AdjacentExtendLaneStruct> adjLftLnLst = extendLaneLst.at(i).left_extend_lane;
                        if (!adjLftLnLst.empty())
                        {
                            // based on target road type
                            if (adjLftLnLst.size() == 1)
                            {
                                rghtCrnrLaneID = adjLftLnLst.front().adjacent_extendlane_id;
                            }
                            else
                            {
                                if (this->globalInfoContainer.m_TgtRoadTyp_RWP == BA::TargetRoadTyp_SubSpltRd)
                                {
                                    // split structure
                                    // find out the the lane with greatest left adjacent_extendlane_offset
                                    float maxStOffset = -3000.0;
                                    rghtCrnrLaneID = 0;
                                    for (auto adjLftLn : adjLftLnLst)
                                    {
                                        WPExtendLaneStruct tmpWpExtendLnObj;
                                        if (FindWPExtendObj(this->globalInfoContainer.m_WpExtendLaneLst, adjLftLn.adjacent_extendlane_id, tmpWpExtendLnObj) == true)
                                        {
                                            if (maxStOffset < adjLftLn.adjacent_extendlane_offset && tmpWpExtendLnObj.current_extendlane_endoffset >= this->globalInfoContainer.m_DistToWayPoint)
                                            {
                                                rghtCrnrLaneID = adjLftLn.adjacent_extendlane_id;
                                                maxStOffset = adjLftLn.adjacent_extendlane_offset;
                                            }
                                        }
                                    }
                                }
                                else if (this->globalInfoContainer.m_TgtRoadTyp_RWP == BA::TargetRoadTyp_SubMrgRd)
                                {
                                    // merge structure
                                    // find out the the lane with least left AdjacentExtendLaneEndOffset
                                    float minEndOffset = 3000.0;
                                    rghtCrnrLaneID = 0;
                                    for (auto adjLftLn : adjLftLnLst)
                                    {
                                        WPExtendLaneStruct tmpWpExtendLnObj;
                                        if (FindWPExtendObj(this->globalInfoContainer.m_WpExtendLaneLst, adjLftLn.adjacent_extendlane_id, tmpWpExtendLnObj) == true)
                                        {
                                            if (minEndOffset > adjLftLn.adjacent_extendlane_endoffset /* && tmpWpExtendLnObj.current_extendlane_offset <= 0 */)
                                            {
                                                rghtCrnrLaneID = adjLftLn.adjacent_extendlane_id;
                                                minEndOffset = adjLftLn.adjacent_extendlane_endoffset;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        else
                        {
                            DLOG(WARNING) << "adjacent left lane list is empty!" << std::endl;
                        }
                    }
                    rghtCrnrLnFound = true;
                }
            }
        }
        return true;
    }
}

bool GlobalInfoProcess::InitGlobalLnInfo(const ControlWayPointStruct ctrlWayPnt,
                                         const std::vector<TransitLaneStruct> transitLnArr,
                                         const uint8_t laneID,
                                         BA::GlobalLaneInfo &globalLnInfo,
                                         float vehSpd)
{
    globalLnInfo.m_LaneID = laneID;
    int cwpExLaneSize = ctrlWayPnt.control_wp_extendlane_list.size();
    for (int i = 0; i < cwpExLaneSize; i++)
    {
        if (ctrlWayPnt.control_wp_extendlane_list[i].lane_id == laneID)
        {
            // last lane change point revision, add reaction time for driver after TOR is sent
            // lane_latest_lc_point_offset_to_wp is distance between ego vehicle and the point
            // m_LastLaneChangePoint is distance between the point and waypoint //
            // globalLnInfo.m_LastLaneChangePoint.emplace_back(ctrlWayPnt.control_waypoint_offset - ctrlWayPnt.control_wp_extendlane_list[i].lane_latest_lc_point_offset_to_wp + vehSpd * ADC::common::k_BA_LastLCPntTH);
            // 20220227
            // lane_latest_lc_point_offset_to_wp is the distance between waypoint and the point
            // m_LastLaneChangePoint is the distance between the point and waypoint
            // 20220511: last lane change point offset of target lanes should be 0
            // 20220804: if neighbor lane in lane change direction is start ahead
            // then modify last lane change point
            auto it_LnID = std::find(ctrlWayPnt.control_waypoint_lane_id_list.begin(), ctrlWayPnt.control_waypoint_lane_id_list.end(), laneID);
            if (it_LnID != ctrlWayPnt.control_waypoint_lane_id_list.end())
            {
                // laneID belongs to target lane list
                globalLnInfo.m_LastLaneChangePoint.emplace_back(ctrlWayPnt.control_wp_extendlane_list[i].lane_latest_lc_point_offset_to_wp); // it should be 0
                globalLnInfo.m_Priority = ctrlWayPnt.control_wp_extendlane_list[i].lane_priority;
            }
            else
            {
                if (ctrlWayPnt.control_waypoint_target_road_type == BA::TargetRoadTyp_ICRd ||
                    ctrlWayPnt.control_waypoint_target_road_type == BA::TargetRoadTyp_JCTRd ||
                    ctrlWayPnt.control_waypoint_target_road_type == BA::TargetRoadTyp_SpltRd ||
                    ctrlWayPnt.control_waypoint_target_road_type == BA::TargetRoadTyp_RmpDvdRd)
                {
                    float lastLCPntDst = std::max(ctrlWayPnt.control_wp_extendlane_list[i].lane_latest_lc_point_offset_to_wp + vehSpd * ADC::common::k_BA_LastLCPntTH, ADC::common::k_BA_MinLastLCPntDst);
                    float revLastLCPntDst = 0.0f;
                    auto iter = std::find_if(this->globalInfoContainer.m_WpExtendLaneLst.begin(), this->globalInfoContainer.m_WpExtendLaneLst.end(),
                                             [laneID](const WPExtendLaneStruct &wpLn_)
                                             {
                                                return (laneID == wpLn_.lane_id);
                                             });
                    if (iter != this->globalInfoContainer.m_WpExtendLaneLst.end())
                    {
                        if (this->globalInfoContainer.m_GlobalLaneChgDrct == BA::LaneChgDrct_Right)
                        {
                            // check right neighbor lane start range
                            uint8_t minRghtLnID = uint8_t(0);
                            float minRghtStDst = 1000.0f;
                            if (!iter->right_extend_lane.empty())
                            {
                                // find the nearest right neighbor lane
                                for (auto extLn : iter->right_extend_lane)
                                {
                                    if (minRghtStDst > extLn.adjacent_extendlane_offset)
                                    {
                                        minRghtStDst = extLn.adjacent_extendlane_offset;
                                        minRghtLnID = extLn.adjacent_extendlane_id;
                                    }
                                }
                                if (minRghtLnID != uint8_t(0))
                                {
                                    if (minRghtStDst > 0.0f)
                                    {
                                        revLastLCPntDst = (this->globalInfoContainer.m_DistToWayPoint - minRghtStDst);
                                        DLOG(INFO) << "right raw revLastLCPntDst = " << revLastLCPntDst << std::endl; 
                                        if (revLastLCPntDst < lastLCPntDst)
                                        {
                                            lastLCPntDst = revLastLCPntDst;
                                            DLOG(INFO) << "lastLCPntDst = " << lastLCPntDst << std::endl;
                                            DLOG(INFO) << "laneID = " << int(laneID) << std::endl; 
                                        }
                                    }
                                }
                            }
                        }
                        else if (this->globalInfoContainer.m_GlobalLaneChgDrct == BA::LaneChgDrct_Left)
                        {
                            // check left neighbor lane start range
                            uint8_t minLftLnID = uint8_t(0);
                            float minLftStDst = 1000.0f;
                            if (!iter->left_extend_lane.empty())
                            {
                                // find the nearest right neighbor lane
                                for (auto extLn : iter->left_extend_lane)
                                {
                                    if (minLftStDst > extLn.adjacent_extendlane_offset)
                                    {
                                        minLftStDst = extLn.adjacent_extendlane_offset;
                                        minLftLnID = extLn.adjacent_extendlane_id;
                                    }
                                }
                                if (minLftLnID != uint8_t(0))
                                {
                                    if (minLftStDst > 0.0f)
                                    {
                                        revLastLCPntDst = (this->globalInfoContainer.m_DistToWayPoint - minLftStDst);
                                        DLOG(INFO) << "left raw revLastLCPntDst = " << revLastLCPntDst << std::endl; 
                                        if (revLastLCPntDst < lastLCPntDst)
                                        {
                                            lastLCPntDst = revLastLCPntDst;
                                            DLOG(INFO) << "lastLCPntDst = " << lastLCPntDst << std::endl;
                                            DLOG(INFO) << "laneID = " << int(laneID) << std::endl; 
                                        }
                                    }
                                }
                            }
                        }
                    }
                    globalLnInfo.m_LastLaneChangePoint.emplace_back(lastLCPntDst);
                }
                else
                {
                    globalLnInfo.m_LastLaneChangePoint.emplace_back(ctrlWayPnt.control_wp_extendlane_list[i].lane_latest_lc_point_offset_to_wp + vehSpd * ADC::common::k_BA_LastLCPntTH);
                }
                globalLnInfo.m_Priority = ctrlWayPnt.control_wp_extendlane_list[i].lane_priority;
            }
            break;
        }
    }
    if (globalLnInfo.m_LastLaneChangePoint.empty())
    {
        DLOG(WARNING) << "laneID has not been found!" << std::endl;
        return false;
    }

    int transitLnSize = transitLnArr.size();
    for (int j = 0; j < transitLnSize; j++)
    {
        if (transitLnArr[j].transit_lane_id == laneID)
        {
            // there may be various transitInfo
            // transit_lane_offset is distance between ego vehicle and transition point
            // m_TransitInInfo or m_TransitOutInfo is distance between waypoint and transition point
            BA::LaneTransitInfo laneTransitInfo;
            laneTransitInfo.m_ConnectedLaneID = transitLnArr[j].transit_connected_lane_id;
            laneTransitInfo.m_TransitTyp = BA::TransitionTyp(transitLnArr[j].transit_type);
            if (laneTransitInfo.m_TransitTyp == BA::TransitionTyp_RghtExitBegin || laneTransitInfo.m_TransitTyp == BA::TransitionTyp_LftExitBegin || laneTransitInfo.m_TransitTyp == BA::TransitionTyp_LaneAddLft || laneTransitInfo.m_TransitTyp == BA::TransitionTyp_LaneAddRght)
            {
                globalLnInfo.m_TransitInInfo.insert(std::make_pair(ctrlWayPnt.control_waypoint_offset - transitLnArr[j].transit_lane_offset, laneTransitInfo));
            }
            else if (laneTransitInfo.m_TransitTyp == BA::TransitionTyp_LaneEnd || laneTransitInfo.m_TransitTyp == BA::TransitionTyp_LftEnterEnd || laneTransitInfo.m_TransitTyp == BA::TransitionTyp_RghtEnterEnd)
            {
                globalLnInfo.m_TransitOutInfo.insert(std::make_pair(ctrlWayPnt.control_waypoint_offset - transitLnArr[j].transit_lane_offset, laneTransitInfo));
            }
        }
        else if (transitLnArr[j].transit_connected_lane_id == laneID)
        {
            BA::LaneTransitInfo laneTransitInfo;
            laneTransitInfo.m_ConnectedLaneID = transitLnArr[j].transit_lane_id;
            laneTransitInfo.m_TransitTyp = BA::TransitionTyp(transitLnArr[j].transit_type);
            if (laneTransitInfo.m_TransitTyp == BA::TransitionTyp_RghtExitBegin || laneTransitInfo.m_TransitTyp == BA::TransitionTyp_LftExitBegin || laneTransitInfo.m_TransitTyp == BA::TransitionTyp_LaneAddLft || laneTransitInfo.m_TransitTyp == BA::TransitionTyp_LaneAddRght)
            {
                globalLnInfo.m_TransitOutInfo.insert(std::make_pair(ctrlWayPnt.control_waypoint_offset - transitLnArr[j].transit_lane_offset, laneTransitInfo));
            }
            else if (laneTransitInfo.m_TransitTyp == BA::TransitionTyp_LaneEnd || laneTransitInfo.m_TransitTyp == BA::TransitionTyp_LftEnterEnd || laneTransitInfo.m_TransitTyp == BA::TransitionTyp_RghtEnterEnd)
            {
                globalLnInfo.m_TransitInInfo.insert(std::make_pair(ctrlWayPnt.control_waypoint_offset - transitLnArr[j].transit_lane_offset, laneTransitInfo));
            }
        }
    }
    return true;
}

bool GlobalInfoProcess::InitGlobalLnInfo(const RemindWayPointStruct rmdWayPnt,
                                         const std::vector<TransitLaneStruct> transitLnArr,
                                         const uint8_t laneID,
                                         BA::GlobalLaneInfo &globalLnInfo,
                                         float vehSpd)
{
    globalLnInfo.m_LaneID = laneID;
    int cwpExLaneSize = rmdWayPnt.reminder_wp_extendlane_list.size();
    for (int i = 0; i < cwpExLaneSize; i++)
    {
        if (rmdWayPnt.reminder_wp_extendlane_list[i].lane_id == laneID)
        {
            // last lane change point revision, add reaction time for driver after TOR is sent
            // lane_latest_lc_point_offset_to_wp is distance between ego vehicle and the point
            // m_LastLaneChangePoint is distance between the point and waypoint //
            // globalLnInfo.m_LastLaneChangePoint.emplace_back(rmdWayPnt.reminder_waypoint_offset - rmdWayPnt.reminder_wp_extendlane_list[i].lane_latest_lc_point_offset_to_wp + vehSpd * ADC::common::k_BA_LastLCPntTH);
            // 20220227
            // lane_latest_lc_point_offset_to_wp is the distance between waypoint and the point
            // m_LastLaneChangePoint is the distance between the point and waypoint
            // 20220511: last lane change point offset of target lanes should be 0
            auto it_LnID = std::find(rmdWayPnt.reminder_waypoint_lane_id_list.begin(), rmdWayPnt.reminder_waypoint_lane_id_list.end(), laneID);
            if (it_LnID != rmdWayPnt.reminder_waypoint_lane_id_list.end())
            {
                // laneID belongs to target lane list
                globalLnInfo.m_LastLaneChangePoint.emplace_back(rmdWayPnt.reminder_wp_extendlane_list[i].lane_latest_lc_point_offset_to_wp); // it should be 0
                globalLnInfo.m_Priority = rmdWayPnt.reminder_wp_extendlane_list[i].lane_priority;
            }
            else
            {
                globalLnInfo.m_LastLaneChangePoint.emplace_back(std::max(rmdWayPnt.reminder_wp_extendlane_list[i].lane_latest_lc_point_offset_to_wp + vehSpd * ADC::common::k_BA_LastLCPntTH, ADC::common::k_BA_MinLastLCPntDst));
                globalLnInfo.m_Priority = rmdWayPnt.reminder_wp_extendlane_list[i].lane_priority;
            }
            break;
        }
    }
    if (globalLnInfo.m_LastLaneChangePoint.empty())
    {
        DLOG(WARNING) << "laneID has not been found!" << std::endl;
        return false;
    }

    int transitLnSize = transitLnArr.size();
    for (int j = 0; j < transitLnSize; j++)
    {
        // 20220601: those transit lanes in original transit lane list will be filtered
        if (transitLnArr[j].transit_connected_lane_id != uint8_t(0))
        {
            if (transitLnArr[j].transit_lane_id == laneID)
            {
                // there may be various transitInfo
                BA::LaneTransitInfo laneTransitInfo;
                laneTransitInfo.m_ConnectedLaneID = transitLnArr[j].transit_connected_lane_id;
                laneTransitInfo.m_TransitTyp = BA::TransitionTyp(transitLnArr[j].transit_type);
                if (laneTransitInfo.m_TransitTyp == BA::TransitionTyp_RghtExitBegin || laneTransitInfo.m_TransitTyp == BA::TransitionTyp_LftExitBegin || laneTransitInfo.m_TransitTyp == BA::TransitionTyp_LaneAddLft || laneTransitInfo.m_TransitTyp == BA::TransitionTyp_LaneAddRght)
                {
                    globalLnInfo.m_TransitInInfo.insert(std::make_pair(rmdWayPnt.reminder_waypoint_offset - transitLnArr[j].transit_lane_offset, laneTransitInfo));
                }
                else if (laneTransitInfo.m_TransitTyp == BA::TransitionTyp_LaneEnd || laneTransitInfo.m_TransitTyp == BA::TransitionTyp_LftEnterEnd || laneTransitInfo.m_TransitTyp == BA::TransitionTyp_RghtEnterEnd)
                {
                    globalLnInfo.m_TransitOutInfo.insert(std::make_pair(rmdWayPnt.reminder_waypoint_offset - transitLnArr[j].transit_lane_offset, laneTransitInfo));
                }
            }
            else if (transitLnArr[j].transit_connected_lane_id == laneID)
            {
                BA::LaneTransitInfo laneTransitInfo;
                laneTransitInfo.m_ConnectedLaneID = transitLnArr[j].transit_lane_id;
                laneTransitInfo.m_TransitTyp = BA::TransitionTyp(transitLnArr[j].transit_type);
                if (laneTransitInfo.m_TransitTyp == BA::TransitionTyp_RghtExitBegin || laneTransitInfo.m_TransitTyp == BA::TransitionTyp_LftExitBegin || laneTransitInfo.m_TransitTyp == BA::TransitionTyp_LaneAddLft || laneTransitInfo.m_TransitTyp == BA::TransitionTyp_LaneAddRght)
                {
                    globalLnInfo.m_TransitOutInfo.insert(std::make_pair(rmdWayPnt.reminder_waypoint_offset - transitLnArr[j].transit_lane_offset, laneTransitInfo));
                }
                else if (laneTransitInfo.m_TransitTyp == BA::TransitionTyp_LaneEnd || laneTransitInfo.m_TransitTyp == BA::TransitionTyp_LftEnterEnd || laneTransitInfo.m_TransitTyp == BA::TransitionTyp_RghtEnterEnd)
                {
                    globalLnInfo.m_TransitInInfo.insert(std::make_pair(rmdWayPnt.reminder_waypoint_offset - transitLnArr[j].transit_lane_offset, laneTransitInfo));
                }
            }
        }
    }
    return true;
}

void GlobalInfoProcess::processGlobalInfo(float vehSpd, bool isAvgVehFlowDenseHigh, const BA::RouteRltvPos &routeRltvPos)
{
    // judge whether m_TgtLaneLst_CWP is empty
    // if yes, RWP related info are applied, or CWP related info are applied
    std::vector<uint8_t> rawTgtLaneLst;
    std::vector<uint8_t> tgtLaneLst;
    if (!this->globalInfoContainer.m_TgtLaneLst_CWP.empty())
    {
        rawTgtLaneLst.assign(this->globalInfoContainer.m_RawTgtLaneLst_CWP.begin(), this->globalInfoContainer.m_RawTgtLaneLst_CWP.end());
        tgtLaneLst.assign(this->globalInfoContainer.m_TgtLaneLst_CWP.begin(), this->globalInfoContainer.m_TgtLaneLst_CWP.end());
    }
    else
    {
        rawTgtLaneLst.assign(this->globalInfoContainer.m_TgtLaneLst_RWP.begin(), this->globalInfoContainer.m_TgtLaneLst_RWP.end());
        tgtLaneLst.assign(this->globalInfoContainer.m_TgtLaneLst_RWP.begin(), this->globalInfoContainer.m_TgtLaneLst_RWP.end());
    }
    uint8_t nrstLaneID = GetNearestLaneID(routeRltvPos, tgtLaneLst, this->globalInfoContainer.m_WpExtendLaneLst); // replace rawTgtLaneLst with tgtLaneLst
    // 20221017 clear m_MinLnGapTgtLaneLst after getting nearest lane id
    this->globalInfoContainer.m_MinLnGapTgtLaneLst.clear();
    DLOG(INFO) << "nearest lane id = " << int(nrstLaneID) << std::endl;
    // uint8_t nrstLaneID = GetNearestLaneID(routeRltvPos, rawTgtLaneLst, this->globalInfoContainer.m_WpExtendLaneLst);
    // if nrstLaneID == 0, try to find the lane nearest to target lane
    // if nrstLaneID != 0, ignore this step
    if (nrstLaneID == 0)
    {
        if (routeRltvPos == BA::RouteRltvPos_Left)
        {
            uint8_t tgtLaneID = 0;
            if (this->globalInfoContainer.m_GlobalInfoSrc == BA::GlobalInfoSrc_OnlyActCtrlWayPoint || this->globalInfoContainer.m_GlobalInfoSrc == BA::GlobalInfoSrc_OnlyVirtCtrlWayPoint)
            {
                if (!this->globalInfoContainer.m_TgtLaneLst_CWP.empty())
                {
                    tgtLaneID = this->globalInfoContainer.m_TgtLaneLst_CWP.at(0); // temperarily 20220222
                }
                WPExtendLaneStruct tgtWpExtendLnObj;
                if (FindWPExtendObj(this->globalInfoContainer.m_WpExtendLaneLst, tgtLaneID, tgtWpExtendLnObj))
                {
                    if (!tgtWpExtendLnObj.right_extend_lane.empty())
                    {
                        nrstLaneID = tgtWpExtendLnObj.right_extend_lane.at(0).adjacent_extendlane_id;
                    }
                }
            }
            else if (this->globalInfoContainer.m_GlobalInfoSrc == BA::GlobalInfoSrc_OnlyRmdWayPoint)
            {
                if (!this->globalInfoContainer.m_TgtLaneLst_RWP.empty())
                {
                    tgtLaneID = this->globalInfoContainer.m_TgtLaneLst_RWP.at(0); // temperarily 20220222
                }
                WPExtendLaneStruct tgtWpExtendLnObj;
                if (FindWPExtendObj(this->globalInfoContainer.m_WpExtendLaneLst, tgtLaneID, tgtWpExtendLnObj))
                {
                    if (!tgtWpExtendLnObj.right_extend_lane.empty())
                    {
                        nrstLaneID = tgtWpExtendLnObj.right_extend_lane.at(0).adjacent_extendlane_id;
                    }
                }
            }
        }
        else if (routeRltvPos == BA::RouteRltvPos_Right)
        {
            uint8_t tgtLaneID = 0;
            if (this->globalInfoContainer.m_GlobalInfoSrc == BA::GlobalInfoSrc_OnlyActCtrlWayPoint || this->globalInfoContainer.m_GlobalInfoSrc == BA::GlobalInfoSrc_OnlyVirtCtrlWayPoint)
            {
                if (!this->globalInfoContainer.m_TgtLaneLst_CWP.empty())
                {
                    tgtLaneID = this->globalInfoContainer.m_TgtLaneLst_CWP.at(0); // temperarily 20220222
                }
                WPExtendLaneStruct tgtWpExtendLnObj;
                if (FindWPExtendObj(this->globalInfoContainer.m_WpExtendLaneLst, tgtLaneID, tgtWpExtendLnObj))
                {
                    if (!tgtWpExtendLnObj.left_extend_lane.empty())
                    {
                        nrstLaneID = tgtWpExtendLnObj.left_extend_lane.at(0).adjacent_extendlane_id;
                    }
                }
            }
            else if (this->globalInfoContainer.m_GlobalInfoSrc == BA::GlobalInfoSrc_OnlyRmdWayPoint)
            {
                if (!this->globalInfoContainer.m_TgtLaneLst_RWP.empty())
                {
                    tgtLaneID = this->globalInfoContainer.m_TgtLaneLst_RWP.at(0); // temperarily 20220222
                }
                WPExtendLaneStruct tgtWpExtendLnObj;
                if (FindWPExtendObj(this->globalInfoContainer.m_WpExtendLaneLst, tgtLaneID, tgtWpExtendLnObj))
                {
                    if (!tgtWpExtendLnObj.left_extend_lane.empty())
                    {
                        nrstLaneID = tgtWpExtendLnObj.left_extend_lane.at(0).adjacent_extendlane_id;
                    }
                }
            }
        }
    }
    // get emergency lane IDs
    std::vector<uint8_t> emergencyLaneIDList;
    for (auto extdLane : this->globalInfoContainer.m_WpExtendLaneLst)
    {
        if (extdLane.is_emergency_lane == 1)
        {
            emergencyLaneIDList.emplace_back(extdLane.lane_id);
        }
    }

    // 20220316
    // if target lane number is equal to non-emergency extendLane number, namely there is no non-target lane
    // then global info process ends here
    int nonEmrgExtLnNum = this->globalInfoContainer.m_WpExtendLaneLst.size() - emergencyLaneIDList.size();
    if (nonEmrgExtLnNum == this->globalInfoContainer.m_TgtLaneLst_CWP.size() && this->globalInfoContainer.m_GlobalInfoSrc == BA::GlobalInfoSrc_OnlyVirtCtrlWayPoint)
    {
        return;
    }
    else
    {
        // get number of non-target lanes
        int notTgtLnNum = 0;
        int indx = 0;
        bool isNrstLnToTgtLn = false;
        bool out2TopLoop = false;
        float tmpAdjDesLnChgPnt = 0;
        WPExtendLaneStruct tmpNotTgtWpExtendLnObj;
        // add desired lane change points
        // relative position of route has influence on calculation order
        if (routeRltvPos == BA::RouteRltvPos_Right)
        {
            // right corner lane id is in rawTgtLaneLst, namely navigation route is on the right side
            // start from left corner lane
            WPExtendLaneStruct tmpWpExtendLnObj;
            if (FindWPExtendObj(this->globalInfoContainer.m_WpExtendLaneLst, this->globalInfoContainer.m_LftCrnrLaneID, tmpWpExtendLnObj))
            {
                // left corner lane object has been found
                // here is good to go to find notTgtLnNum
                // notTgtLnNum = notTgtLnNum + 1;
                for (int k = 0; k < this->globalInfoContainer.m_WpExtendLaneLst.size(); k++)
                {
                    if (tmpWpExtendLnObj.lane_id == nrstLaneID)
                    {
                        notTgtLnNum++;
                        out2TopLoop = true;
                    }
                    else
                    {
                        if (!tmpWpExtendLnObj.right_extend_lane.empty())
                        {
                            notTgtLnNum++;
                            for (int j = 0; j < tmpWpExtendLnObj.right_extend_lane.size(); j++)
                            {
                                if (tmpWpExtendLnObj.right_extend_lane.at(j).adjacent_extendlane_id == nrstLaneID)
                                {
                                    out2TopLoop = true;
                                    notTgtLnNum = notTgtLnNum + 1;
                                    break;
                                }
                            }
                        }
                        // update tmpWpExtendLnObj to adjacentLaneID
                        int tmpRghtExtendLaneNum = tmpWpExtendLnObj.right_extend_lane.size();
                        for (int j = 0; j < tmpRghtExtendLaneNum; j++)
                        {
                            if (find(emergencyLaneIDList.begin(), emergencyLaneIDList.end(), tmpWpExtendLnObj.right_extend_lane.at(j).adjacent_extendlane_id) == emergencyLaneIDList.end())
                            {
                                uint8_t adjacentLaneID = tmpWpExtendLnObj.right_extend_lane.at(j).adjacent_extendlane_id;
                                if (FindWPExtendObj(this->globalInfoContainer.m_WpExtendLaneLst, adjacentLaneID, tmpWpExtendLnObj))
                                {
                                    // std::cout << "adjacent lane object found." << std::endl;
                                    break;
                                }
                            }
                        }
                    }
                    if (out2TopLoop == true)
                    {
                        break;
                    }
                }
            }
            // 20220818: notTgtLnNum expands
            if (notTgtLnNum > 0)
            {
                notTgtLnNum = notTgtLnNum + 2;
            }
            DLOG(INFO) << "notTgtLnNum = " << notTgtLnNum << std::endl;

            if (notTgtLnNum < 1)
            {
                DLOG(WARNING) << "Not target lane number is abnormal!" << std::endl;
                return;
            }
            // float tmpAdjDesLnChgPnt = 0;
            // WPExtendLaneStruct tmpNotTgtWpExtendLnObj;
            if (FindWPExtendObj(this->globalInfoContainer.m_WpExtendLaneLst, nrstLaneID, tmpNotTgtWpExtendLnObj))
            {
                uint8_t laneID = 0;
                for (int i = 0; i < notTgtLnNum; i++)
                {

                    /* if (!tmpNotTgtWpExtendLnObj.left_extend_lane.empty())
                {
                    laneID = tmpNotTgtWpExtendLnObj.left_extend_lane.at(0).adjacent_extendlane_id;
                }
                else
                {
                    break;
                } */

                    if (i == 0)
                    {
                        laneID = nrstLaneID;
                        isNrstLnToTgtLn = true;
                    }
                    else
                    {
                        isNrstLnToTgtLn = false;
                    }

                    if (FindTgtGlblLnIndx(this->globalInfoContainer.m_GlobalLaneInfoArr, laneID, indx))
                    {
                        // int timeHeadway = GetDesLCTimeHeadway(i + 1);
                        int timeHeadway = GetDesLCTimeHeadway_B(i + 1, isAvgVehFlowDenseHigh); // 20220706
                        float deslLCPntLowThr = GetDesLCPntLowThr(i + 1);
                        float adjDesLnChgPnt = 0;
                        if (i == 0)
                        {
                            adjDesLnChgPnt = this->globalInfoContainer.m_GlobalLaneInfoArr[indx].m_LastLaneChangePoint.front();
                        }
                        else
                        {
                            adjDesLnChgPnt = tmpAdjDesLnChgPnt;
                        }
                        if (!AddDesLaneChgPnt(this->globalInfoContainer.m_GlobalLaneInfoArr[indx], timeHeadway, deslLCPntLowThr, vehSpd, adjDesLnChgPnt, isNrstLnToTgtLn))
                        {
                            DLOG(WARNING) << "Add desired lane change point at " << indx << "global lane info failed!" << std::endl;
                        }
                        // update tmpAdjDesLnChgPnt for next loop
                        tmpAdjDesLnChgPnt = this->globalInfoContainer.m_GlobalLaneInfoArr[indx].m_DesiredLaneChangePoint.front();
                        // renew tmpNotTgtWpExtendLnObj for next loop
                        if (!FindWPExtendObj(this->globalInfoContainer.m_WpExtendLaneLst, laneID, tmpNotTgtWpExtendLnObj))
                        {
                            DLOG(WARNING) << "lane " << int(laneID) << "has not been found in m_WpExtendLaneLst!" << std::endl;
                        }
                    }

                    if (!tmpNotTgtWpExtendLnObj.left_extend_lane.empty())
                    {
                        laneID = tmpNotTgtWpExtendLnObj.left_extend_lane.at(0).adjacent_extendlane_id;
                    }
                    else
                    {
                        break;
                    }
                }
            }
        }
        else if (routeRltvPos == BA::RouteRltvPos_Left)
        {
            // left corner lane id is in rawTgtLaneLst, namely navigation route is on the left side
            // start from right corner lane
            WPExtendLaneStruct tmpWpExtendLnObj;
            if (FindWPExtendObj(this->globalInfoContainer.m_WpExtendLaneLst, this->globalInfoContainer.m_RghtCrnrLaneID, tmpWpExtendLnObj))
            {
                // right corner lane object has been found
                // here is good to go to find notTgtLnNum
                for (int k = 0; k < this->globalInfoContainer.m_WpExtendLaneLst.size(); k++)
                {
                    if (tmpWpExtendLnObj.lane_id == nrstLaneID)
                    {
                        notTgtLnNum++;
                        out2TopLoop = true;
                        DLOG(INFO) << "out2TopLoop = " << int(out2TopLoop) << std::endl;
                    }
                    else
                    {
                        if (!tmpWpExtendLnObj.left_extend_lane.empty())
                        {
                            notTgtLnNum++;
                            for (int j = 0; j < tmpWpExtendLnObj.left_extend_lane.size(); j++)
                            {
                                if (tmpWpExtendLnObj.left_extend_lane.at(j).adjacent_extendlane_id == nrstLaneID)
                                {
                                    out2TopLoop = true;
                                    notTgtLnNum = notTgtLnNum + 1;
                                    break;
                                }
                            }
                        }
                        // update tmpWpExtendLnObj to adjacentLaneID
                        int tmpRghtExtendLaneNum = tmpWpExtendLnObj.left_extend_lane.size();
                        for (int j = 0; j < tmpRghtExtendLaneNum; j++)
                        {
                            if (find(emergencyLaneIDList.begin(), emergencyLaneIDList.end(), tmpWpExtendLnObj.left_extend_lane.at(j).adjacent_extendlane_id) == emergencyLaneIDList.end())
                            {
                                uint8_t adjacentLaneID = tmpWpExtendLnObj.left_extend_lane.at(j).adjacent_extendlane_id;
                                if (FindWPExtendObj(this->globalInfoContainer.m_WpExtendLaneLst, adjacentLaneID, tmpWpExtendLnObj))
                                {
                                    break;
                                }
                            }
                        }
                    }
                    if (out2TopLoop == true)
                    {
                        break;
                    }
                }
            }
            // 20220918: notTgtLnNum expands
            if (notTgtLnNum > 0)
            {
                notTgtLnNum = notTgtLnNum + 2;
            }
            DLOG(INFO) << "notTgtLnNum = " << notTgtLnNum << std::endl;
            if (notTgtLnNum < 1)
            {
                DLOG(WARNING) << "Not target lane number is abnormal!" << std::endl;
                return;
            }
            // float tmpAdjDesLnChgPnt = 0;
            // WPExtendLaneStruct tmpNotTgtWpExtendLnObj;
            if (FindWPExtendObj(this->globalInfoContainer.m_WpExtendLaneLst, nrstLaneID, tmpNotTgtWpExtendLnObj))
            {
                uint8_t laneID = 0;
                float tmpLaneOffset = 1000.0f;
                for (int i = 0; i < notTgtLnNum; i++)
                {
                    if (i == 0)
                    {
                        laneID = nrstLaneID;
                        isNrstLnToTgtLn = true;
                    }
                    else
                    {
                        isNrstLnToTgtLn = false;
                    }

                    if (FindTgtGlblLnIndx(this->globalInfoContainer.m_GlobalLaneInfoArr, laneID, indx))
                    {
                        // int timeHeadway = GetDesLCTimeHeadway(i + 1);
                        int timeHeadway = GetDesLCTimeHeadway_B(i + 1, isAvgVehFlowDenseHigh); // 20220706
                        float deslLCPntLowThr = GetDesLCPntLowThr(i + 1);
                        float adjDesLnChgPnt = 0;
                        if (i == 0)
                        {
                            adjDesLnChgPnt = this->globalInfoContainer.m_GlobalLaneInfoArr[indx].m_LastLaneChangePoint.front();
                        }
                        else
                        {
                            adjDesLnChgPnt = tmpAdjDesLnChgPnt;
                        }
                        if (!AddDesLaneChgPnt(this->globalInfoContainer.m_GlobalLaneInfoArr[indx], timeHeadway, deslLCPntLowThr, vehSpd, adjDesLnChgPnt, isNrstLnToTgtLn))
                        {
                            DLOG(WARNING) << "Add desired lane change point at " << indx << "global lane info failed!" << std::endl;
                        }
                        // update tmpAdjDesLnChgPnt for next loop
                        tmpAdjDesLnChgPnt = this->globalInfoContainer.m_GlobalLaneInfoArr[indx].m_DesiredLaneChangePoint.front();
                        // renew tmpNotTgtWpExtendLnObj for next loop
                        if (!FindWPExtendObj(this->globalInfoContainer.m_WpExtendLaneLst, laneID, tmpNotTgtWpExtendLnObj))
                        {
                            DLOG(WARNING) << "lane " << int(laneID) << "has not been found in m_WpExtendLaneLst!" << std::endl;
                        }
                    }

                    if (!tmpNotTgtWpExtendLnObj.right_extend_lane.empty())
                    {
                        // 20220918:
                        // add protection logic for iterated lane id
                        laneID = tmpNotTgtWpExtendLnObj.right_extend_lane.at(0).adjacent_extendlane_id;
                        tmpLaneOffset = tmpNotTgtWpExtendLnObj.right_extend_lane.at(0).adjacent_extendlane_offset;
                        for (auto rghtExtLane : tmpNotTgtWpExtendLnObj.right_extend_lane)
                        {
                            if (rghtExtLane.adjacent_extendlane_offset < tmpLaneOffset)
                            {
                                laneID = rghtExtLane.adjacent_extendlane_id;
                                tmpLaneOffset = rghtExtLane.adjacent_extendlane_offset;
                            }
                        }
                    }
                    else
                    {
                        break;
                    }
                }
            }
        }
        else
        {
            // right and left corner lane ids are both not in rawTgtLaneLst, namely navigation route is in the middle
            // 20221017 added
            if (FindWPExtendObj(this->globalInfoContainer.m_WpExtendLaneLst, nrstLaneID, tmpNotTgtWpExtendLnObj))
            {
                uint8_t laneID = 0;
                float tmpLaneOffset = 1000.0f;
                if (this->globalInfoContainer.m_GlobalLaneChgDrct == BA::LaneChgDrct_Left)
                {
                    for (int i = 0; i < this->globalInfoContainer.m_GlobalLaneChgNum + 1; i++)
                    {
                        if (i == 0)
                        {
                            laneID = nrstLaneID;
                            isNrstLnToTgtLn = true;
                        }
                        else
                        {
                            isNrstLnToTgtLn = false;
                        }
                        if (FindTgtGlblLnIndx(this->globalInfoContainer.m_GlobalLaneInfoArr, laneID, indx))
                        {
                            // int timeHeadway = GetDesLCTimeHeadway(i + 1);
                            int timeHeadway = GetDesLCTimeHeadway_B(i + 1, isAvgVehFlowDenseHigh); // 20220706
                            float deslLCPntLowThr = GetDesLCPntLowThr(i + 1);
                            float adjDesLnChgPnt = 0;
                            if (i == 0)
                            {
                                adjDesLnChgPnt = this->globalInfoContainer.m_GlobalLaneInfoArr[indx].m_LastLaneChangePoint.front();
                            }
                            else
                            {
                                adjDesLnChgPnt = tmpAdjDesLnChgPnt;
                            }
                            if (!AddDesLaneChgPnt(this->globalInfoContainer.m_GlobalLaneInfoArr[indx], timeHeadway, deslLCPntLowThr, vehSpd, adjDesLnChgPnt, isNrstLnToTgtLn))
                            {
                                DLOG(WARNING) << "Add desired lane change point at " << indx << "global lane info failed!" << std::endl;
                            }
                            // update tmpAdjDesLnChgPnt for next loop
                            tmpAdjDesLnChgPnt = this->globalInfoContainer.m_GlobalLaneInfoArr[indx].m_DesiredLaneChangePoint.front();
                            // renew tmpNotTgtWpExtendLnObj for next loop
                            if (!FindWPExtendObj(this->globalInfoContainer.m_WpExtendLaneLst, laneID, tmpNotTgtWpExtendLnObj))
                            {
                                DLOG(WARNING) << "lane " << int(laneID) << "has not been found in m_WpExtendLaneLst!" << std::endl;
                            }
                        }
                        if (!tmpNotTgtWpExtendLnObj.right_extend_lane.empty())
                        {
                            // 20220918:
                            // add protection logic for iterated lane id
                            laneID = tmpNotTgtWpExtendLnObj.right_extend_lane.at(0).adjacent_extendlane_id;
                            tmpLaneOffset = tmpNotTgtWpExtendLnObj.right_extend_lane.at(0).adjacent_extendlane_offset;
                            for (auto rghtExtLane : tmpNotTgtWpExtendLnObj.right_extend_lane)
                            {
                                if (rghtExtLane.adjacent_extendlane_offset < tmpLaneOffset)
                                {
                                    laneID = rghtExtLane.adjacent_extendlane_id;
                                    tmpLaneOffset = rghtExtLane.adjacent_extendlane_offset;
                                }
                            }
                        }
                        else
                        {
                            break;
                        }
                    }   
                }
                else if (this->globalInfoContainer.m_GlobalLaneChgDrct == BA::LaneChgDrct_Right)
                {
                    for (int i = 0; i < this->globalInfoContainer.m_GlobalLaneChgNum + 1; i++)
                    {
                        if (i == 0)
                        {
                            laneID = nrstLaneID;
                            isNrstLnToTgtLn = true;
                        }
                        else
                        {
                            isNrstLnToTgtLn = false;
                        }
                        if (FindTgtGlblLnIndx(this->globalInfoContainer.m_GlobalLaneInfoArr, laneID, indx))
                        {
                            // int timeHeadway = GetDesLCTimeHeadway(i + 1);
                            int timeHeadway = GetDesLCTimeHeadway_B(i + 1, isAvgVehFlowDenseHigh); // 20220706
                            float deslLCPntLowThr = GetDesLCPntLowThr(i + 1);
                            float adjDesLnChgPnt = 0;
                            if (i == 0)
                            {
                                adjDesLnChgPnt = this->globalInfoContainer.m_GlobalLaneInfoArr[indx].m_LastLaneChangePoint.front();
                            }
                            else
                            {
                                adjDesLnChgPnt = tmpAdjDesLnChgPnt;
                            }
                            if (!AddDesLaneChgPnt(this->globalInfoContainer.m_GlobalLaneInfoArr[indx], timeHeadway, deslLCPntLowThr, vehSpd, adjDesLnChgPnt, isNrstLnToTgtLn))
                            {
                                DLOG(WARNING) << "Add desired lane change point at " << indx << "global lane info failed!" << std::endl;
                            }
                            // update tmpAdjDesLnChgPnt for next loop
                            tmpAdjDesLnChgPnt = this->globalInfoContainer.m_GlobalLaneInfoArr[indx].m_DesiredLaneChangePoint.front();
                            // renew tmpNotTgtWpExtendLnObj for next loop
                            if (!FindWPExtendObj(this->globalInfoContainer.m_WpExtendLaneLst, laneID, tmpNotTgtWpExtendLnObj))
                            {
                                DLOG(WARNING) << "lane " << int(laneID) << "has not been found in m_WpExtendLaneLst!" << std::endl;
                            }
                        }
                        if (!tmpNotTgtWpExtendLnObj.left_extend_lane.empty())
                        {
                            // 20220918:
                            // add protection logic for iterated lane id
                            laneID = tmpNotTgtWpExtendLnObj.left_extend_lane.at(0).adjacent_extendlane_id;
                            tmpLaneOffset = tmpNotTgtWpExtendLnObj.left_extend_lane.at(0).adjacent_extendlane_offset;
                            for (auto lftExtLane : tmpNotTgtWpExtendLnObj.left_extend_lane)
                            {
                                if (lftExtLane.adjacent_extendlane_offset < tmpLaneOffset)
                                {
                                    laneID = lftExtLane.adjacent_extendlane_id;
                                    tmpLaneOffset = lftExtLane.adjacent_extendlane_offset;
                                }
                            }
                        }
                        else
                        {
                            break;
                        }
                    }   
                }  
            }
        }
    }
}

uint8_t GlobalInfoProcess::GetNearestLaneID(BA::RouteRltvPos rtRltvPos, 
                                            std::vector<uint8_t> rawTgtLaneLst,
                                            const std::vector<WPExtendLaneStruct> &wpExtendLaneLst)
{
    if (rawTgtLaneLst.empty())
    {
        DLOG(WARNING) << "Raw target lane list is empty." << std::endl;
        return 0;
    }
    else
    {
        uint8_t nrstLaneID = 0;
        bool isNrstLnFound = false;
        if (rtRltvPos == BA::RouteRltvPos_Left)
        {
            // in rawTgtLaneLst, search each lane's right neighbor lane
            // if its all right neighbor lanes are not included in rawtgtLaneLst, then it is the lane nearest to rawTgtLaneLst
            for (int i = 0; i < rawTgtLaneLst.size(); i++)
            {
                WPExtendLaneStruct tmpWpExtendLnObj;
                if (FindWPExtendObj(wpExtendLaneLst, rawTgtLaneLst.at(i), tmpWpExtendLnObj))
                {
                    // uint8_t nrstLaneID = 0;
                    if (!tmpWpExtendLnObj.right_extend_lane.empty())
                    {
                        if (tmpWpExtendLnObj.right_extend_lane.size() == 1)
                        {
                            // if temp wpExtendLn's right lane list size is 1, then check whether the only lane is emergency lane or not
                            // IF positive, the temp wpExtendLn is the nearest lane
                            // ELSE, keep going
                            // 20220303
                            WPExtendLaneStruct wpExtendLnObj;
                            if (FindWPExtendObj(wpExtendLaneLst, tmpWpExtendLnObj.right_extend_lane.front().adjacent_extendlane_id, wpExtendLnObj))
                            {
                                if (wpExtendLnObj.is_emergency_lane == 1)
                                {
                                    return tmpWpExtendLnObj.lane_id;
                                }
                            }
                            else
                            {
                                DLOG(WARNING) << "get nearest lane failed!" << std::endl;
                                return 0;
                            }
                        }
                        for (int j = 0; j < tmpWpExtendLnObj.right_extend_lane.size(); j++)
                        {
                            if (find(rawTgtLaneLst.begin(), rawTgtLaneLst.end(), tmpWpExtendLnObj.right_extend_lane.at(j).adjacent_extendlane_id) != rawTgtLaneLst.end())
                            {
                                isNrstLnFound = false;
                                break;
                            }
                            else
                            {
                                isNrstLnFound = true;
                            }
                        }
                        if (isNrstLnFound == true)
                        {
                            if (!tmpWpExtendLnObj.right_extend_lane.empty())
                            {
                                // distinguished by waypoint type or waypoint source
                                AdjacentExtendLaneStruct tmpRightExtLane;
                                float tmpRightExtLaneStOffset = 0;
                                if (this->globalInfoContainer.m_TgtRoadTyp_CWP == BA::TargetRoadTyp_JCTRd ||
                                    this->globalInfoContainer.m_TgtRoadTyp_CWP == BA::TargetRoadTyp_ICRd ||
                                    this->globalInfoContainer.m_TgtRoadTyp_CWP == BA::TargetRoadTyp_RmpDvdRd ||
                                    this->globalInfoContainer.m_TgtRoadTyp_CWP == BA::TargetRoadTyp_SpltRd)
                                {
                                    tmpRightExtLane = tmpWpExtendLnObj.right_extend_lane.at(0);
                                    tmpRightExtLaneStOffset = tmpRightExtLane.adjacent_extendlane_offset;
                                    for (auto rightExtendLane : tmpWpExtendLnObj.right_extend_lane)
                                    {
                                        WPExtendLaneStruct rghtWpExtendLnObj;
                                        if (!FindWPExtendObj(wpExtendLaneLst, rightExtendLane.adjacent_extendlane_id, rghtWpExtendLnObj))
                                        {
                                            continue;
                                        }

                                        if (rghtWpExtendLnObj.is_emergency_lane == 0 && rightExtendLane.adjacent_extendlane_offset > tmpRightExtLaneStOffset)
                                        {
                                            tmpRightExtLane = rightExtendLane;
                                            tmpRightExtLaneStOffset = tmpRightExtLane.adjacent_extendlane_offset;
                                        }
                                    }
                                }
                                else
                                {
                                    tmpRightExtLane = tmpWpExtendLnObj.right_extend_lane.at(0);
                                    tmpRightExtLaneStOffset = tmpRightExtLane.adjacent_extendlane_endoffset;
                                    for (auto rightExtendLane : tmpWpExtendLnObj.right_extend_lane)
                                    {
                                        WPExtendLaneStruct rghtWpExtendLnObj;
                                        if (!FindWPExtendObj(wpExtendLaneLst, rightExtendLane.adjacent_extendlane_id, rghtWpExtendLnObj))
                                        {
                                            continue;
                                        }

                                        if (rghtWpExtendLnObj.is_emergency_lane == 0 && rightExtendLane.adjacent_extendlane_endoffset < tmpRightExtLaneStOffset)
                                        {
                                            tmpRightExtLane = rightExtendLane;
                                            tmpRightExtLaneStOffset = tmpRightExtLane.adjacent_extendlane_endoffset;
                                        }
                                    }
                                }
                                nrstLaneID = tmpRightExtLane.adjacent_extendlane_id;
                            }
                            else
                            {
                                DLOG(WARNING) << "get nearest lane failed!" << std::endl;
                            }
                            return nrstLaneID;
                        }
                    }
                    else
                    {
                        // if temp extendLane' right lane is empty,
                        // set nrstLaneID to the temp extendLane's id
                        nrstLaneID = tmpWpExtendLnObj.lane_id;
                        return nrstLaneID;
                    }
                }
            }
            DLOG(WARNING) << "get nearest lane failed!" << std::endl;
            return 0;
        }
        else if (rtRltvPos == BA::RouteRltvPos_Right)
        {
            // in rawTgtLaneLst, search each lane's left neighbor lane
            // if its all left neighbor lanes are not included in rawtgtLaneLst, then it is the lane nearest to rawTgtLaneLst
            for (int i = 0; i < rawTgtLaneLst.size(); i++)
            {
                WPExtendLaneStruct tmpWpExtendLnObj;
                if (FindWPExtendObj(wpExtendLaneLst, rawTgtLaneLst.at(i), tmpWpExtendLnObj))
                {
                    if (!tmpWpExtendLnObj.left_extend_lane.empty())
                    {
                        if (tmpWpExtendLnObj.left_extend_lane.size() == 1)
                        {
                            // if temp wpExtendLn's left lane list size is 1, then check whether the only lane is emergency lane or not
                            // IF positive, the temp wpExtendLn is the nearest lane
                            // ELSE, keep going
                            // 20220303
                            WPExtendLaneStruct wpExtendLnObj;
                            if (FindWPExtendObj(wpExtendLaneLst, tmpWpExtendLnObj.left_extend_lane.front().adjacent_extendlane_id, wpExtendLnObj))
                            {
                                if (wpExtendLnObj.is_emergency_lane == 1)
                                {
                                    return tmpWpExtendLnObj.lane_id;
                                }
                            }
                            else
                            {
                                DLOG(WARNING) << "get nearest lane failed!" << std::endl;
                                return 0;
                            }
                        }
                        for (int j = 0; j < tmpWpExtendLnObj.left_extend_lane.size(); j++)
                        {
                            if (find(rawTgtLaneLst.begin(), rawTgtLaneLst.end(), tmpWpExtendLnObj.left_extend_lane.at(j).adjacent_extendlane_id) != rawTgtLaneLst.end())
                            {
                                isNrstLnFound = false;
                                break;
                            }
                            else
                            {
                                isNrstLnFound = true;
                            }
                        }
                        if (isNrstLnFound == true)
                        {
                            if (isNrstLnFound == true)
                            {
                                // uint8_t nrstLaneID = 0;
                                if (!tmpWpExtendLnObj.left_extend_lane.empty())
                                {
                                    AdjacentExtendLaneStruct tmpLeftExtLane;
                                    float tmpLeftExtLaneStOffset = 0;
                                    if (this->globalInfoContainer.m_TgtRoadTyp_CWP == BA::TargetRoadTyp_JCTRd ||
                                        this->globalInfoContainer.m_TgtRoadTyp_CWP == BA::TargetRoadTyp_ICRd || 
                                        this->globalInfoContainer.m_TgtRoadTyp_CWP == BA::TargetRoadTyp_RmpDvdRd ||
                                        this->globalInfoContainer.m_TgtRoadTyp_CWP == BA::TargetRoadTyp_SpltRd)
                                    {
                                        tmpLeftExtLane = tmpWpExtendLnObj.left_extend_lane.at(0);
                                        tmpLeftExtLaneStOffset = tmpLeftExtLane.adjacent_extendlane_offset;
                                        for (auto leftExtendLane : tmpWpExtendLnObj.left_extend_lane)
                                        {
                                            WPExtendLaneStruct lftWpExtendLnObj;
                                            if (!FindWPExtendObj(wpExtendLaneLst, leftExtendLane.adjacent_extendlane_id, lftWpExtendLnObj))
                                            {
                                                continue;
                                            }

                                            if (lftWpExtendLnObj.is_emergency_lane == 0 && leftExtendLane.adjacent_extendlane_offset > tmpLeftExtLaneStOffset)
                                            {
                                                tmpLeftExtLane = leftExtendLane;
                                                tmpLeftExtLaneStOffset = tmpLeftExtLane.adjacent_extendlane_offset;
                                            }
                                        }
                                    }
                                    else
                                    {
                                        tmpLeftExtLane = tmpWpExtendLnObj.left_extend_lane.at(0);
                                        tmpLeftExtLaneStOffset = tmpLeftExtLane.adjacent_extendlane_endoffset;
                                        for (auto leftExtendLane : tmpWpExtendLnObj.left_extend_lane)
                                        {
                                            WPExtendLaneStruct lftWpExtendLnObj;
                                            if (!FindWPExtendObj(wpExtendLaneLst, leftExtendLane.adjacent_extendlane_id, lftWpExtendLnObj))
                                            {
                                                continue;
                                            }

                                            if (lftWpExtendLnObj.is_emergency_lane == 0 && leftExtendLane.adjacent_extendlane_endoffset < tmpLeftExtLaneStOffset)
                                            {
                                                tmpLeftExtLane = leftExtendLane;
                                                tmpLeftExtLaneStOffset = tmpLeftExtLane.adjacent_extendlane_endoffset;
                                            }
                                        }
                                    }
                                    nrstLaneID = tmpLeftExtLane.adjacent_extendlane_id;
                                }
                                else
                                {
                                    DLOG(WARNING) << "get nearest lane failed!" << std::endl;
                                }
                                return nrstLaneID;
                            }
                        }
                    }
                }
            }
            DLOG(WARNING) << "get nearest lane failed!" << std::endl;
            return 0;
        }
        // 20221017 add procedure for middle route relatvie position
        else if (rtRltvPos == BA::RouteRltvPos_Middle)
        {
            DLOG(INFO) << "flag_00" << std::endl;
            DLOG(INFO) << "m_MinLnGapTgtLaneLst size: " << this->globalInfoContainer.m_MinLnGapTgtLaneLst.size() << std::endl;
            if (this->globalInfoContainer.m_GlobalLaneChgNum != uint8_t(0) && 
                this->globalInfoContainer.m_GlobalLaneChgDrct != BA::LaneChgDrct_None &&
                !this->globalInfoContainer.m_MinLnGapTgtLaneLst.empty())
            {
                uint8_t minLnGapTrgtLnID = this->globalInfoContainer.m_MinLnGapTgtLaneLst.front();
                auto iter_MinLnGapTrgtLn = std::find_if(this->globalInfoContainer.m_WpExtendLaneLst.begin(), this->globalInfoContainer.m_WpExtendLaneLst.end(),
                                                        [minLnGapTrgtLnID](const WPExtendLaneStruct &wpLn_)
                                                        {
                                                            return (minLnGapTrgtLnID == wpLn_.lane_id);
                                                        });
                if (iter_MinLnGapTrgtLn != this->globalInfoContainer.m_WpExtendLaneLst.end())
                {
                    if (this->globalInfoContainer.m_GlobalLaneChgDrct == BA::LaneChgDrct_Left)
                    {
                        if (!iter_MinLnGapTrgtLn->right_extend_lane.empty())
                        {
                            AdjacentExtendLaneStruct tmpRightExtLane;
                            float tmpRightExtLaneStOffset = 0;
                            if (this->globalInfoContainer.m_TgtRoadTyp_CWP == BA::TargetRoadTyp_JCTRd ||
                                this->globalInfoContainer.m_TgtRoadTyp_CWP == BA::TargetRoadTyp_ICRd ||
                                this->globalInfoContainer.m_TgtRoadTyp_CWP == BA::TargetRoadTyp_RmpDvdRd ||
                                this->globalInfoContainer.m_TgtRoadTyp_CWP == BA::TargetRoadTyp_SpltRd)
                            {
                                tmpRightExtLane = iter_MinLnGapTrgtLn->right_extend_lane.at(0);
                                tmpRightExtLaneStOffset = tmpRightExtLane.adjacent_extendlane_offset;
                                for (auto rightExtendLane : iter_MinLnGapTrgtLn->right_extend_lane)
                                {
                                    WPExtendLaneStruct rghtWpExtendLnObj;
                                    if (!FindWPExtendObj(wpExtendLaneLst, rightExtendLane.adjacent_extendlane_id, rghtWpExtendLnObj))
                                    {
                                        continue;
                                    }

                                    if (rghtWpExtendLnObj.is_emergency_lane == 0 && rightExtendLane.adjacent_extendlane_offset > tmpRightExtLaneStOffset)
                                    {
                                        tmpRightExtLane = rightExtendLane;
                                        tmpRightExtLaneStOffset = tmpRightExtLane.adjacent_extendlane_offset;
                                    }
                                }
                            }
                            else
                            {
                                tmpRightExtLane = iter_MinLnGapTrgtLn->right_extend_lane.at(0);
                                tmpRightExtLaneStOffset = tmpRightExtLane.adjacent_extendlane_endoffset;
                                for (auto rightExtendLane : iter_MinLnGapTrgtLn->right_extend_lane)
                                {
                                    WPExtendLaneStruct rghtWpExtendLnObj;
                                    if (!FindWPExtendObj(wpExtendLaneLst, rightExtendLane.adjacent_extendlane_id, rghtWpExtendLnObj))
                                    {
                                        continue;
                                    }
                                    if (rghtWpExtendLnObj.is_emergency_lane == 0 && rightExtendLane.adjacent_extendlane_endoffset < tmpRightExtLaneStOffset)
                                    {
                                        tmpRightExtLane = rightExtendLane;
                                        tmpRightExtLaneStOffset = tmpRightExtLane.adjacent_extendlane_endoffset;
                                    }
                                }
                            }
                            nrstLaneID = tmpRightExtLane.adjacent_extendlane_id;
                        }
                        else
                        {
                            DLOG(WARNING) << "get nearest lane failed!" << std::endl;
                        }
                        return nrstLaneID;
                    }
                    else if (this->globalInfoContainer.m_GlobalLaneChgDrct == BA::LaneChgDrct_Right)
                    {
                        if (!iter_MinLnGapTrgtLn->left_extend_lane.empty())
                        {
                            AdjacentExtendLaneStruct tmpLeftExtLane;
                            float tmpLeftExtLaneStOffset = 0;
                            if (this->globalInfoContainer.m_TgtRoadTyp_CWP == BA::TargetRoadTyp_JCTRd ||
                                this->globalInfoContainer.m_TgtRoadTyp_CWP == BA::TargetRoadTyp_ICRd || 
                                this->globalInfoContainer.m_TgtRoadTyp_CWP == BA::TargetRoadTyp_RmpDvdRd ||
                                this->globalInfoContainer.m_TgtRoadTyp_CWP == BA::TargetRoadTyp_SpltRd)
                            {
                                tmpLeftExtLane = iter_MinLnGapTrgtLn->left_extend_lane.at(0);
                                tmpLeftExtLaneStOffset = tmpLeftExtLane.adjacent_extendlane_offset;
                                for (auto leftExtendLane : iter_MinLnGapTrgtLn->left_extend_lane)
                                {
                                    WPExtendLaneStruct lftWpExtendLnObj;
                                    if (!FindWPExtendObj(wpExtendLaneLst, leftExtendLane.adjacent_extendlane_id, lftWpExtendLnObj))
                                    {
                                        continue;
                                    }
                                    if (lftWpExtendLnObj.is_emergency_lane == 0 && leftExtendLane.adjacent_extendlane_offset > tmpLeftExtLaneStOffset)
                                    {
                                        tmpLeftExtLane = leftExtendLane;
                                        tmpLeftExtLaneStOffset = tmpLeftExtLane.adjacent_extendlane_offset;
                                    }
                                }
                            }
                            else
                            {
                                tmpLeftExtLane = iter_MinLnGapTrgtLn->left_extend_lane.at(0);
                                tmpLeftExtLaneStOffset = tmpLeftExtLane.adjacent_extendlane_endoffset;
                                for (auto leftExtendLane : iter_MinLnGapTrgtLn->left_extend_lane)
                                {
                                    WPExtendLaneStruct lftWpExtendLnObj;
                                    if (!FindWPExtendObj(wpExtendLaneLst, leftExtendLane.adjacent_extendlane_id, lftWpExtendLnObj))
                                    {
                                        continue;
                                    }
                                    if (lftWpExtendLnObj.is_emergency_lane == 0 && leftExtendLane.adjacent_extendlane_endoffset < tmpLeftExtLaneStOffset)
                                    {
                                        tmpLeftExtLane = leftExtendLane;
                                        tmpLeftExtLaneStOffset = tmpLeftExtLane.adjacent_extendlane_endoffset;
                                    }
                                }
                            }
                            nrstLaneID = tmpLeftExtLane.adjacent_extendlane_id;
                        }
                        else
                        {
                            DLOG(WARNING) << "get nearest lane failed!" << std::endl;
                        }
                        return nrstLaneID;
                    }
                    else
                    {
                        return 0;
                    }
                }
                else
                {
                    return 0;
                }
            }
            else
            {
                return 0;
            }
        }
        else
        {
            return 0; // lane change is not necessary
        }
    }
}

bool GlobalInfoProcess::AddDesLaneChgPnt(BA::GlobalLaneInfo &globalLnInfo, float timeHeadway, float minLnChgPntDst, float vehSpd, float adjDesLnChgPnt, bool isNrstLnToTgtLn)
{
    DLOG(INFO) << " timeHeadway = " << timeHeadway << std::endl;
    DLOG(INFO) << " minLnChgPntDst = " << minLnChgPntDst << std::endl;
    DLOG(INFO) << " adjDesLnChgPnt = " << adjDesLnChgPnt << std::endl;
    float rawDesLnChgPntDst = std::max(globalLnInfo.m_LastLaneChangePoint.front(), adjDesLnChgPnt) + vehSpd * timeHeadway;
    float revDesLnChgPntDst = std::max(rawDesLnChgPntDst, minLnChgPntDst);
    DLOG(INFO) << " revDesLnChgPntDst = " << revDesLnChgPntDst << std::endl;
    // consider nearest lane's start offset
    float nrstLnStOffset = 0;
    uint8_t laneID = globalLnInfo.m_LaneID;
    WPExtendLaneStruct nrstLnWpExtendLnObj;
    if (FindWPExtendObj(this->globalInfoContainer.m_WpExtendLaneLst, laneID, nrstLnWpExtendLnObj))
    {
        nrstLnStOffset = nrstLnWpExtendLnObj.current_extendlane_offset;
    }
    float nrstDstToWP = this->globalInfoContainer.m_DistToWayPoint - nrstLnStOffset;

    // comment globalLnInfo.m_TransitOutInfo judgement
    /* if (globalLnInfo.m_TransitOutInfo.empty())
    {
        std::cout << "No transit out info!" << std::endl;
    }
    else
    { */

    // for the lane nearest to raw target lane lsit, consider split lane transition
    if (isNrstLnToTgtLn)
    {
        // find the nearest TransitOut point to ego vehicle and in front of ego vehicle
        // and based on that to revise desired lane change point
        // take Target road type into account, only getting into ramp is considered
        if (this->globalInfoContainer.m_TgtRoadTyp_CWP == BA::TargetRoadTyp_ICRd || 
            this->globalInfoContainer.m_TgtRoadTyp_CWP == BA::TargetRoadTyp_JCTRd || 
            this->globalInfoContainer.m_TgtRoadTyp_CWP == BA::TargetRoadTyp_RmpDvdRd ||
            this->globalInfoContainer.m_TgtRoadTyp_CWP == BA::TargetRoadTyp_SpltRd)
        {
            std::map<float, BA::LaneTransitInfo>::iterator iter = globalLnInfo.m_TransitOutInfo.begin();
            float tmpDstSplt = revDesLnChgPntDst; // replace tmpDstSplt with revDesLnChgPntDst instead of iter->first 20220222
            for (; iter != globalLnInfo.m_TransitOutInfo.end(); iter++)
            {
                if (iter->first < tmpDstSplt && iter->first > 0 && 
                    ( iter->second.m_TransitTyp == BA::TransitionTyp_LaneAddLft ||
                      iter->second.m_TransitTyp == BA::TransitionTyp_LaneAddRght ))
                {
                    tmpDstSplt = iter->first;
                    DLOG(INFO) << "*****" << std::endl;
                    DLOG(INFO) << "transit offset: " << int(iter->first) << std::endl; 
                    DLOG(INFO) << "m_ConnectedLaneID: " << int(iter->second.m_ConnectedLaneID) << std::endl;
                    DLOG(INFO) << "m_TransitTyp: " << int(iter->second.m_TransitTyp) << std::endl; 
                }
            }
            DLOG(INFO) << " tmpDstSplt = " << tmpDstSplt << std::endl;
            DLOG(INFO) << " nrstDstToWP = " << nrstDstToWP << std::endl;
            revDesLnChgPntDst = std::min(revDesLnChgPntDst, tmpDstSplt);
            revDesLnChgPntDst = std::min(revDesLnChgPntDst, nrstDstToWP);
        }
    }
    // for the lane with lane end transition type, consider end lane transition
    std::map<float, BA::LaneTransitInfo>::iterator it = globalLnInfo.m_TransitOutInfo.begin();
    float tmpDstEnd = 0;
    for (; it != globalLnInfo.m_TransitOutInfo.end(); it++)
    {
        if (it->second.m_TransitTyp == BA::TransitionTyp_LaneEnd)
        {
            tmpDstEnd = it->first;
        }
    }
    revDesLnChgPntDst = std::max(revDesLnChgPntDst, tmpDstEnd);

    globalLnInfo.m_DesiredLaneChangePoint.emplace_back(revDesLnChgPntDst);

    DLOG(INFO) << "global lane info id = " << int(globalLnInfo.m_LaneID) << std::endl;
    #ifdef GLOG
    for (auto desLCPnt : globalLnInfo.m_DesiredLaneChangePoint)
    {
        DLOG(INFO) << "desired lane change point = " << desLCPnt << std::endl;
    }
    #endif
    return true;
}

bool GlobalInfoProcess::FindTgtGlblLnIndx(std::vector<BA::GlobalLaneInfo> globalLnInfoArr, uint8_t laneID, int &index)
{
    if (globalLnInfoArr.empty())
    {
        DLOG(WARNING) << "globalLnInfoArr is empty!" << std::endl;
        return false;
    }
    else
    {
        bool flag = false;
        for (int k = 0; k <= globalLnInfoArr.size(); k++)
        {
            if (globalLnInfoArr[k].m_LaneID == laneID)
            {
                index = k;
                flag = true;
                break;
            }
        }
        if (flag)
        {
            return true;
        }
        else
        {
            DLOG(WARNING) << "lane ID is not found!" << std::endl;
            return false;
        }
    }
}

float GlobalInfoProcess::GetDesLCTimeHeadway(int lnChgNum)
{
    switch (lnChgNum)
    {
    case 1:
        return ADC::common::k_BA_DesLCPntTH1st;
        break;
    case 2:
        return ADC::common::k_BA_DesLCPntTH2nd;
        break;
    case 3:
        return ADC::common::k_BA_DesLCPntTH3rd;
        break;
    case 4:
        return ADC::common::k_BA_DesLCPntTH4th;
        break;
    default:
        return ADC::common::k_BA_DesLCPntTH4th;
    }
}

float GlobalInfoProcess::GetDesLCTimeHeadway_B(int lnChgNum, bool isAvgVehFlowHigh)
{
    if (isAvgVehFlowHigh == false)
    {
        switch (lnChgNum)
        {
        case 1:
            return ADC::common::k_BA_DesLCPntTH1st;
            break;
        case 2:
            return ADC::common::k_BA_DesLCPntTH2nd;
            break;
        case 3:
            return ADC::common::k_BA_DesLCPntTH3rd;
            break;
        case 4:
            return ADC::common::k_BA_DesLCPntTH4th;
            break;
        default:
            return ADC::common::k_BA_DesLCPntTH4th;
        }
    }
    else
    {
        switch (lnChgNum)
        {
        case 1:
            return ADC::common::k_BA_DesLCPntTH1st_B;
            break;
        case 2:
            return ADC::common::k_BA_DesLCPntTH2nd_B;
            break;
        case 3:
            return ADC::common::k_BA_DesLCPntTH3rd_B;
            break;
        case 4:
            return ADC::common::k_BA_DesLCPntTH4th_B;
            break;
        default:
            return ADC::common::k_BA_DesLCPntTH4th_B;
        }
    }
}

float GlobalInfoProcess::GetDesLCPntLowThr(int lnChgNum)
{
    switch (lnChgNum)
    {
    case 1:
        return ADC::common::k_BA_DesLCPntLowThr1st;
        break;
    case 2:
        return ADC::common::k_BA_DesLCPntLowThr2nd;
        break;
    case 3:
        return ADC::common::k_BA_DesLCPntLowThr3rd;
        break;
    case 4:
        return ADC::common::k_BA_DesLCPntLowThr4th;
        break;
    default:
        return ADC::common::k_BA_DesLCPntLowThr4th;
    }
}

BA::RouteRltvPos GlobalInfoProcess::GetRouteRltvPos(std::vector<uint8_t> rawTgtLaneLst, uint8_t lftCrnrLaneID, uint8_t rghtCrnrLaneID)
{
    if (std::find(rawTgtLaneLst.begin(), rawTgtLaneLst.end(), lftCrnrLaneID) != rawTgtLaneLst.end())
    {
        // left corner lane id is in rawTgtLaneLst, namely navigation route is on the left side
        return BA::RouteRltvPos_Left;
    }    
    else if (std::find(rawTgtLaneLst.begin(), rawTgtLaneLst.end(), rghtCrnrLaneID) != rawTgtLaneLst.end())
    {
        // right corner lane id is in rawTgtLaneLst, namely navigation route is on the right side
        return BA::RouteRltvPos_Right;
    }
    else
    {
        // right and left corner lane ids are both not in rawTgtLaneLst, namely navigation route is in the middle
        return BA::RouteRltvPos_Middle;
    }
}

int GlobalInfoProcess::CalcGapBtwLns(const std::vector<WPExtendLaneStruct> &wpExtendLaneLst, uint8_t laneID, uint8_t tgtLaneID)
{
    uint8_t tmpLaneID = tgtLaneID;
    float tmpLaneOffset = 1000.0f;
    WPExtendLaneStruct tmpWpExtendLnObj;
    int laneGapNum = 0;
    bool calcGapBtwLnsSccd = false;
    // from target lane to certain lane, searching left side
    for (int j = 0; j < wpExtendLaneLst.size(); j++)
    {
        if (FindWPExtendObj(wpExtendLaneLst, tmpLaneID, tmpWpExtendLnObj))
        {
            if (tmpWpExtendLnObj.lane_id == laneID)
            {
                calcGapBtwLnsSccd = true;
                // std::cout << "CalcGapBtwLns finished normally" << std::endl;
                break;
            }
            else
            {
                if (!tmpWpExtendLnObj.left_extend_lane.empty())
                {
                    // get first left extend lane by default, and update tmpLaneID
                    // 20220918 select lane with less offset as tmpLane
                    tmpLaneID = tmpWpExtendLnObj.left_extend_lane.at(0).adjacent_extendlane_id;
                    tmpLaneOffset = tmpWpExtendLnObj.left_extend_lane.at(0).adjacent_extendlane_offset;
                    for (auto lftExtLane : tmpWpExtendLnObj.left_extend_lane)
                    {
                        if (lftExtLane.adjacent_extendlane_offset < tmpLaneOffset)
                        {
                            tmpLaneID = lftExtLane.adjacent_extendlane_id;
                            tmpLaneOffset = lftExtLane.adjacent_extendlane_offset;
                        }
                    }
                    laneGapNum = laneGapNum + 1;
                }
                else
                {
                    // 20220929: set lane gap number value to 100
                    laneGapNum = 100;
                    break;
                }
            }
            if (calcGapBtwLnsSccd == true)
            {
                break;
            }
        }
        else
        {
            DLOG(WARNING) << "tmpLaneID has not been found in wpExtendLaneLst!" << std::endl;
            break;
        }
    }
    DLOG(INFO) << "laneGapNum01 = " << laneGapNum << std::endl;
    // laneID maybe on the right side of target lane ID
    if (!calcGapBtwLnsSccd)
    {
        DLOG(INFO) << "enter09" << std::endl;
        // from target lane to certain lane, searching right side
        // reset laneGapNum = 0
        tmpLaneID = tgtLaneID;
        laneGapNum = 0;
        for (int j = 0; j < wpExtendLaneLst.size(); j++)
        {
            if (FindWPExtendObj(wpExtendLaneLst, tmpLaneID, tmpWpExtendLnObj))
            {
                if (tmpWpExtendLnObj.lane_id == laneID)
                {
                    calcGapBtwLnsSccd = true;
                    break;
                }
                else
                {
                    if (!tmpWpExtendLnObj.right_extend_lane.empty())
                    {
                        // get first right extend lane by default, and update tmpLaneID
                        // tmpLaneID = tmpWpExtendLnObj.right_extend_lane.at(0).adjacent_extendlane_id;
                        // 20220328: get the first one which is not an emergency lane as tmpLaneID
                        // it is a temp solution
                        // 20221009 select lane with less offset as tmpLane
                        WPExtendLaneStruct adjExtLnObj;
                        tmpLaneID = tmpWpExtendLnObj.right_extend_lane.at(0).adjacent_extendlane_id;
                        tmpLaneOffset = tmpWpExtendLnObj.right_extend_lane.at(0).adjacent_extendlane_offset;
                        for (auto adjExtLn : tmpWpExtendLnObj.right_extend_lane)
                        {
                            /* if (FindWPExtendObj(wpExtendLaneLst, adjExtLn.adjacent_extendlane_id, adjExtLnObj))
                            {
                                if (adjExtLnObj.is_emergency_lane == 0)
                                {
                                    tmpLaneID = adjExtLnObj.lane_id;
                                    break;
                                }
                            }  */ 
                            if (adjExtLn.adjacent_extendlane_offset < tmpLaneOffset)
                            {
                                tmpLaneID = adjExtLn.adjacent_extendlane_id;
                                tmpLaneOffset = adjExtLn.adjacent_extendlane_offset;
                            }
                        }
                        laneGapNum = laneGapNum - 1;
                    }
                    else
                    {
                        // 20220929: set lane gap number value to 100
                        laneGapNum = 100;
                        break;
                    }
                }
            }
            else
            {
                DLOG(WARNING) << "tmpLaneID has not been found in wpExtendLaneLst!" << std::endl;
                break;
            }
        }
        DLOG(INFO) << "laneGapNum02 = " << laneGapNum << std::endl;
    }
    // laneID has still not been found
    // vehicle should change lanes at least two times with different directions
    if (!calcGapBtwLnsSccd)
    {
        laneGapNum = 100; // 20220929: 0 --> 100
    }
    return laneGapNum;
}

bool GlobalInfoProcess::FindWPExtendObj(const std::vector<WPExtendLaneStruct> &wpExtendLaneLst, uint8_t laneID, WPExtendLaneStruct &wpExtendLnObj)
{
    for (int i = 0; i < wpExtendLaneLst.size(); i++)
    {
        if (wpExtendLaneLst.at(i).lane_id == laneID)
        {
            wpExtendLnObj = wpExtendLaneLst.at(i);
            return true;
        }
    }
    DLOG(WARNING) << "target wpExtendLanelist has not been found!" << std::endl;
    return false;
}

bool GlobalInfoProcess::rebuildTransLnList(ControlWaypoints &ehr_msg, bool toRebuildTransitLnLst)
{
    if (toRebuildTransitLnLst == true)
    {
        // it is necessary to rebuild transit lane list
        // ehr_msg.transit_lanes_list.clear(); // 20220601: uncomment this line when necessary
        for (auto extendLane : this->globalInfoContainer.m_WpExtendLaneLst)
        {
            // check split transit lane
            // 20220720: check whether extendLane is emergency lane
            WPExtendLaneStruct extLnObj;
            bool isExtLnEmerg = false;
            if (FindWPExtendObj(this->globalInfoContainer.m_WpExtendLaneLst, extendLane.lane_id, extLnObj))
            {
                if (extLnObj.is_emergency_lane == uint8_t(1))
                {
                    isExtLnEmerg = true;
                }
            }
            if (extendLane.current_extendlane_offset > 0 && isExtLnEmerg == false)
            {
                // the start point of this extend lane is ahead of ego veh position
                bool transitLaneCondPosi = false;
                TransitLaneStruct transitLaneSplt;
                transitLaneSplt.transit_lane_id = extendLane.lane_id;
                transitLaneSplt.transit_lane_offset = extendLane.current_extendlane_offset;
                if (extendLane.right_extend_lane.empty())
                {
                    // its transit type is right split
                    // select the lane in the left adjacent lane list with the smallest start offset as connected lane
                    if (!extendLane.left_extend_lane.empty())
                    {
                        transitLaneCondPosi = true;
                        float minStOffset = extendLane.left_extend_lane.front().adjacent_extendlane_offset;
                        uint8_t minStOffsetLnID = extendLane.left_extend_lane.front().adjacent_extendlane_id;
                        for (auto leftExtendLane : extendLane.left_extend_lane)
                        {
                            if (leftExtendLane.adjacent_extendlane_offset < minStOffset)
                            {
                                minStOffset = leftExtendLane.adjacent_extendlane_offset;
                                minStOffsetLnID = leftExtendLane.adjacent_extendlane_id;
                            }
                        }
                        transitLaneSplt.transit_connected_lane_id = minStOffsetLnID;
                        transitLaneSplt.transit_type = BA::TransitionTyp_LaneAddRght;
                    }
                    else
                    {
                        transitLaneCondPosi = false;
                    }
                }
                else if (!extendLane.right_extend_lane.empty())
                {
                    // check whether the lane in the right adjacent lane list with the smallest start offset is emergency lane
                    // or the lane above has greater start offset than current extend lane
                    float minStOffset = extendLane.right_extend_lane.front().adjacent_extendlane_offset;
                    uint8_t minStOffsetLnID = extendLane.right_extend_lane.front().adjacent_extendlane_id;
                    for (auto rightExtendLane : extendLane.right_extend_lane)
                    {
                        if (rightExtendLane.adjacent_extendlane_offset < minStOffset)
                        {
                            minStOffset = rightExtendLane.adjacent_extendlane_offset;
                            minStOffsetLnID = rightExtendLane.adjacent_extendlane_id;
                        }
                    }
                    auto it_lane = std::find_if(this->globalInfoContainer.m_WpExtendLaneLst.begin(), this->globalInfoContainer.m_WpExtendLaneLst.end(),
                                                [minStOffsetLnID](const WPExtendLaneStruct &wpExtendLane_)
                                                {
                                                    return (wpExtendLane_.lane_id == minStOffsetLnID);
                                                });
                    if (it_lane != this->globalInfoContainer.m_WpExtendLaneLst.end())
                    {
                        if (it_lane->is_emergency_lane == uint8_t(1) || it_lane->current_extendlane_offset > extendLane.current_extendlane_offset)
                        {
                            // its transit type is right split
                            // select the lane in the left adjacent lane list with the smallest start offset as connected lane
                            if (!extendLane.left_extend_lane.empty())
                            {
                                transitLaneCondPosi = true;
                                float minStOffset = extendLane.left_extend_lane.front().adjacent_extendlane_offset;
                                uint8_t minStOffsetLnID = extendLane.left_extend_lane.front().adjacent_extendlane_id;
                                for (auto leftExtendLane : extendLane.left_extend_lane)
                                {
                                    if (leftExtendLane.adjacent_extendlane_offset < minStOffset)
                                    {
                                        minStOffset = leftExtendLane.adjacent_extendlane_offset;
                                        minStOffsetLnID = leftExtendLane.adjacent_extendlane_id;
                                    }
                                }
                                transitLaneSplt.transit_connected_lane_id = minStOffsetLnID;
                                transitLaneSplt.transit_type = BA::TransitionTyp_LaneAddRght;
                            }
                            else
                            {
                                transitLaneCondPosi = false;
                            }
                        }
                        else
                        {
                            if (extendLane.left_extend_lane.empty())
                            {
                                // its transit type is left split
                                // select the lane in the right adjacent lane list with the smallest start offset as connected lane
                                if (!extendLane.right_extend_lane.empty())
                                {
                                    transitLaneCondPosi = true;
                                    float minStOffset = extendLane.right_extend_lane.front().adjacent_extendlane_offset;
                                    uint8_t minStOffsetLnID = extendLane.right_extend_lane.front().adjacent_extendlane_id;
                                    for (auto rightExtendLane : extendLane.right_extend_lane)
                                    {
                                        if (rightExtendLane.adjacent_extendlane_offset < minStOffset)
                                        {
                                            minStOffset = rightExtendLane.adjacent_extendlane_offset;
                                            minStOffsetLnID = rightExtendLane.adjacent_extendlane_id;
                                        }
                                    }
                                    transitLaneSplt.transit_connected_lane_id = minStOffsetLnID;
                                    transitLaneSplt.transit_type = BA::TransitionTyp_LaneAddLft;
                                }
                                else
                                {
                                    transitLaneCondPosi = false;
                                }
                            }
                            else if (!extendLane.left_extend_lane.empty())
                            {
                                // check whether the lane in the left adjacent lane list with the smallest start offset is emergency lane
                                float minStOffset = extendLane.left_extend_lane.front().adjacent_extendlane_offset;
                                uint8_t minStOffsetLnID = extendLane.left_extend_lane.front().adjacent_extendlane_id;
                                for (auto leftExtendLane : extendLane.left_extend_lane)
                                {
                                    if (leftExtendLane.adjacent_extendlane_offset < minStOffset)
                                    {
                                        minStOffset = leftExtendLane.adjacent_extendlane_offset;
                                        minStOffsetLnID = leftExtendLane.adjacent_extendlane_id;
                                    }
                                }
                                auto it_lane = std::find_if(this->globalInfoContainer.m_WpExtendLaneLst.begin(), this->globalInfoContainer.m_WpExtendLaneLst.end(),
                                                            [minStOffsetLnID](const WPExtendLaneStruct &wpExtendLane_)
                                                            {
                                                                return (wpExtendLane_.lane_id == minStOffsetLnID);
                                                            });
                                if (it_lane != this->globalInfoContainer.m_WpExtendLaneLst.end())
                                {
                                    if (it_lane->is_emergency_lane == uint8_t(1) || it_lane->current_extendlane_offset > extendLane.current_extendlane_offset)
                                    {
                                        // its transit type is left split
                                        // select the lane in the right adjacent lane list with the smallest start offset as connected lane
                                        if (!extendLane.right_extend_lane.empty())
                                        {
                                            transitLaneCondPosi = true;
                                            float minStOffset = extendLane.right_extend_lane.front().adjacent_extendlane_offset;
                                            uint8_t minStOffsetLnID = extendLane.right_extend_lane.front().adjacent_extendlane_id;
                                            for (auto rightExtendLane : extendLane.right_extend_lane)
                                            {
                                                if (rightExtendLane.adjacent_extendlane_offset < minStOffset)
                                                {
                                                    minStOffset = rightExtendLane.adjacent_extendlane_offset;
                                                    minStOffsetLnID = rightExtendLane.adjacent_extendlane_id;
                                                }
                                            }
                                            transitLaneSplt.transit_connected_lane_id = minStOffsetLnID;
                                            transitLaneSplt.transit_type = BA::TransitionTyp_LaneAddLft;
                                        }
                                        else
                                        {
                                            transitLaneCondPosi = false;
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
                if (transitLaneCondPosi == true)
                {
                    ehr_msg.transit_lanes_list.emplace_back(transitLaneSplt);
                }
            }

            // check merge transit lane
            /* DLOG(INFO) << "rebuild extend lane list" << std::endl;
            DLOG(INFO) << "extend lane id = " << int(extendLane.lane_id) << std::endl;
            DLOG(INFO) << "extend lane current_extendlane_endoffset = " << extendLane.current_extendlane_endoffset << std::endl;
            DLOG(INFO) << "extend lane current_extendlane_offset = " << extendLane.current_extendlane_offset << std::endl;
            DLOG(INFO) << "m_DistToWayPoint = " << this->globalInfoContainer.m_DistToWayPoint << std::endl; */
            if (extendLane.current_extendlane_endoffset < this->globalInfoContainer.m_DistToWayPoint - 0.1f)
            {
                // the end point of this extend lane is in front of the waypoint
                // DLOG(INFO) << "ENTER" << std::endl;
                bool transitLaneCondPosi = false;
                TransitLaneStruct transitLaneMrg;
                transitLaneMrg.transit_lane_id = extendLane.lane_id;
                transitLaneMrg.transit_lane_offset = extendLane.current_extendlane_endoffset;
                if (extendLane.right_extend_lane.empty())
                {
                    // its transit type is left merge
                    // select the lane in the left adjacent lane list with the greatest end offset as connected lane
                    if (!extendLane.left_extend_lane.empty())
                    {
                        transitLaneCondPosi = true;
                        float maxEndOffset = extendLane.left_extend_lane.front().adjacent_extendlane_endoffset;
                        uint8_t maxEndOffsetLnID = extendLane.left_extend_lane.front().adjacent_extendlane_id;
                        for (auto leftExtendLane : extendLane.left_extend_lane)
                        {
                            if (leftExtendLane.adjacent_extendlane_endoffset > maxEndOffset)
                            {
                                maxEndOffset = leftExtendLane.adjacent_extendlane_endoffset;
                                maxEndOffsetLnID = leftExtendLane.adjacent_extendlane_id;
                            }
                        }
                        transitLaneMrg.transit_connected_lane_id = maxEndOffsetLnID;
                        transitLaneMrg.transit_type = BA::TransitionTyp_LaneEnd;
                    }
                    else
                    {
                        transitLaneCondPosi = false;
                    }
                }
                else if (extendLane.left_extend_lane.empty())
                {
                    // its transit type is right merge
                    // select the lane in the right adjacent lane list with the greatest end offset as connected lane
                    if (!extendLane.right_extend_lane.empty())
                    {
                        transitLaneCondPosi = true;
                        float maxEndOffset = extendLane.right_extend_lane.front().adjacent_extendlane_endoffset;
                        uint8_t maxEndOffsetLnID = extendLane.right_extend_lane.front().adjacent_extendlane_id;
                        for (auto rightExtendLane : extendLane.right_extend_lane)
                        {
                            if (rightExtendLane.adjacent_extendlane_endoffset > maxEndOffset)
                            {
                                maxEndOffset = rightExtendLane.adjacent_extendlane_endoffset;
                                maxEndOffsetLnID = rightExtendLane.adjacent_extendlane_id;
                            }
                        }
                        transitLaneMrg.transit_connected_lane_id = maxEndOffsetLnID;
                        transitLaneMrg.transit_type = BA::TransitionTyp_LaneEnd;
                    }
                    else
                    {
                        transitLaneCondPosi = false;
                    }
                }
                else if (!extendLane.right_extend_lane.empty())
                {
                    // check whether the lane in the right adjacent lane list with the greatest end offset is emergency lane
                    // or the lane above has less end offset than current extend lane
                    float maxEndOffset = extendLane.right_extend_lane.front().adjacent_extendlane_endoffset;
                    uint8_t maxEndOffsetLnID = extendLane.right_extend_lane.front().adjacent_extendlane_id;
                    for (auto rightExtendLane : extendLane.right_extend_lane)
                    {
                        if (rightExtendLane.adjacent_extendlane_endoffset > maxEndOffset)
                        {
                            maxEndOffset = rightExtendLane.adjacent_extendlane_endoffset;
                            maxEndOffsetLnID = rightExtendLane.adjacent_extendlane_id;
                        }
                    }
                    auto it_lane = std::find_if(this->globalInfoContainer.m_WpExtendLaneLst.begin(), this->globalInfoContainer.m_WpExtendLaneLst.end(),
                                                [maxEndOffsetLnID](const WPExtendLaneStruct &wpExtendLane_)
                                                {
                                                    return (wpExtendLane_.lane_id == maxEndOffsetLnID);
                                                });
                    if (it_lane != this->globalInfoContainer.m_WpExtendLaneLst.end())
                    {
                        if (it_lane->is_emergency_lane == uint8_t(1) || it_lane->current_extendlane_endoffset < extendLane.current_extendlane_endoffset)
                        {
                            // its transit type is left merge
                            // select the lane in the left adjacent lane list with the greatest end offset as connected lane
                            if (!extendLane.left_extend_lane.empty())
                            {
                                transitLaneCondPosi = true;
                                float maxEndOffset = extendLane.left_extend_lane.front().adjacent_extendlane_endoffset;
                                uint8_t maxEndOffsetLnID = extendLane.left_extend_lane.front().adjacent_extendlane_id;
                                for (auto leftExtendLane : extendLane.left_extend_lane)
                                {
                                    if (leftExtendLane.adjacent_extendlane_endoffset > maxEndOffset)
                                    {
                                        maxEndOffset = leftExtendLane.adjacent_extendlane_endoffset;
                                        maxEndOffsetLnID = leftExtendLane.adjacent_extendlane_id;
                                    }
                                }
                                transitLaneMrg.transit_connected_lane_id = maxEndOffsetLnID;
                                transitLaneMrg.transit_type = BA::TransitionTyp_LaneEnd;
                            }
                            else
                            {
                                transitLaneCondPosi = false;
                            }
                        }
                        else
                        {
                            transitLaneCondPosi = true;
                            transitLaneMrg.transit_connected_lane_id = it_lane->lane_id;
                            transitLaneMrg.transit_type = BA::TransitionTyp_LaneEnd;
                        }
                    }
                }
                if (transitLaneCondPosi == true)
                {
                    ehr_msg.transit_lanes_list.emplace_back(transitLaneMrg);
                }
            }
        }
    }
    return true;
}

bool GlobalInfoProcess::MdfTgtLnLst(std::vector<uint8_t> &ctrlLnLst, 
                                    const BA::RouteRltvPos &rtRltvPos, 
                                    const ControlWaypoints &ehr_msg,
                                    const RoadInfo &road_model)
{
    // 20220802: 
    // if target lane list size is equal to 2 and 
    // ctrl waypoint is split type road
    // then remove the inside target lane
    // 20220803:
    // add case recognition condition
    // 20220829:
    // add road model input, if ego lane type is deceleration lane or exit or on ramp or off ramp or connecting ramp
    // then bypass this process
    // 20220901:
    // modify ego lane type to only exit and deceleration lane
    bool isEgoLnSpcLnTyp = false;
    uint8_t ego_lane_id = road_model.car_position.ego_lane_id;
    auto it_ego_ln = std::find_if(road_model.lane_list.begin(), road_model.lane_list.end(),
                                  [ego_lane_id](const Lane &lane_)
                                  {
                                      return (ego_lane_id == lane_.start_id);
                                  });
    if (it_ego_ln != road_model.lane_list.end())
    {
        if (it_ego_ln->lane_type == 10 || 
            it_ego_ln->lane_type == 17 || 
            /* it_ego_ln->lane_type == 18 || */
            it_ego_ln->lane_type == 19/*  || 
            it_ego_ln->lane_type == 20 */)
        {
            isEgoLnSpcLnTyp = true;
        }
    }
    uint8_t rmvTgtLn = uint8_t(0);
    uint8_t unRmvTgtLn = uint8_t(0);
    if (ctrlLnLst.size() == 2 && 
        ehr_msg.control_waypoints_list.size() == 2 &&
        isEgoLnSpcLnTyp == false &&
        (
         (this->globalInfoContainer.m_TgtRoadTyp_CWP == BA::TargetRoadTyp_ICRd ||
          this->globalInfoContainer.m_TgtRoadTyp_CWP == BA::TargetRoadTyp_JCTRd ||
          this->globalInfoContainer.m_TgtRoadTyp_CWP == BA::TargetRoadTyp_SpltRd ||
          this->globalInfoContainer.m_TgtRoadTyp_CWP == BA::TargetRoadTyp_RmpDvdRd) && 
          ((ehr_msg.control_waypoints_list.at(1).control_waypoint_target_road_type == BA::TargetRoadTyp_RmpDvdRd && 
            ehr_msg.control_waypoints_list.at(1).control_waypoint_offset - this->globalInfoContainer.m_DistToWayPoint > 400.0f) ||
            ehr_msg.control_waypoints_list.at(1).control_waypoint_target_road_type == BA::TargetRoadTyp_RmpMrgCARd ||
           /*  ehr_msg.control_waypoints_list.at(1).control_waypoint_target_road_type == BA::TargetRoadTyp_NotAvlbl || */
            /* ehr_msg.control_waypoints_list.at(1).control_waypoint_target_road_type == BA::TargetRoadTyp_MrgRd || */
            ehr_msg.control_waypoints_list.at(1).control_waypoint_target_road_type == BA::TargetRoadTyp_JCTRd))
        )
    {
        // determine which one is on the outside
        uint8_t tgtLn01 = ctrlLnLst.front();
        uint8_t tgtLn02 = ctrlLnLst.at(1);
        if (rtRltvPos == BA::RouteRltvPos_Right)
        {
            // find the target lane on the left side
            // check whether tgtLn01's right neighbor lane includes tgtLn02
            auto it_tgtLn01 = std::find_if(this->globalInfoContainer.m_WpExtendLaneLst.begin(), this->globalInfoContainer.m_WpExtendLaneLst.end(),
                                           [tgtLn01](const WPExtendLaneStruct &wpLn_)
                                           {
                                               return (tgtLn01 == wpLn_.lane_id);
                                           });
            if (it_tgtLn01 != this->globalInfoContainer.m_WpExtendLaneLst.end())
            {
                if (!it_tgtLn01->right_extend_lane.empty())
                {
                    auto it_tgtLn01InWpExt = std::find_if(it_tgtLn01->right_extend_lane.begin(), it_tgtLn01->right_extend_lane.end(),
                                                          [tgtLn02](const AdjacentExtendLaneStruct &ngbrWpLn_)
                                                          {
                                                              return (tgtLn02 == ngbrWpLn_.adjacent_extendlane_id);
                                                          });
                    if (it_tgtLn01InWpExt != it_tgtLn01->right_extend_lane.end())
                    {
                        // tgtLn02 is included in tgtLn01's right neighbor lane list
                        rmvTgtLn = tgtLn01;
                        unRmvTgtLn = tgtLn02;
                    }
                    else
                    {
                        rmvTgtLn = tgtLn02;
                        unRmvTgtLn = tgtLn01;
                    }
                }
            }
        }
        else if (rtRltvPos == BA::RouteRltvPos_Left)
        {
            // find the target lane on the right side
            // check whether tgtLn01's left neighbor lane includes tgtLn02
            auto it_tgtLn01 = std::find_if(this->globalInfoContainer.m_WpExtendLaneLst.begin(), this->globalInfoContainer.m_WpExtendLaneLst.end(),
                                           [tgtLn01](const WPExtendLaneStruct &wpLn_)
                                           {
                                               return (tgtLn01 == wpLn_.lane_id);
                                           });
            if (it_tgtLn01 != this->globalInfoContainer.m_WpExtendLaneLst.end())
            {
                if (!it_tgtLn01->left_extend_lane.empty())
                {
                    auto it_tgtLn01InWpExt = std::find_if(it_tgtLn01->left_extend_lane.begin(), it_tgtLn01->left_extend_lane.end(),
                                                          [tgtLn02](const AdjacentExtendLaneStruct &ngbrWpLn_)
                                                          {
                                                              return (tgtLn02 == ngbrWpLn_.adjacent_extendlane_id);
                                                          });
                    if (it_tgtLn01InWpExt != it_tgtLn01->left_extend_lane.end())
                    {
                        // tgtLn02 is included in tgtLn01's left neighbor lane list
                        rmvTgtLn = tgtLn01;
                        unRmvTgtLn = tgtLn02;
                    }
                    else
                    {
                        rmvTgtLn = tgtLn02;
                        unRmvTgtLn = tgtLn01;
                    }
                }
            }
            DLOG(INFO) << "rmvTgtLn = " << int(rmvTgtLn) << std::endl;
        }
        // 20220811: check whether outside lane is a new lane or not
        auto it_unRmvTgtLn = std::find_if(this->globalInfoContainer.m_WpExtendLaneLst.begin(), this->globalInfoContainer.m_WpExtendLaneLst.end(),
                                        [unRmvTgtLn](const WPExtendLaneStruct &wpLn_)
                                        {
                                            return (unRmvTgtLn == wpLn_.lane_id);
                                        });
        if (it_unRmvTgtLn != this->globalInfoContainer.m_WpExtendLaneLst.end())
        {
            if (it_unRmvTgtLn->current_extendlane_offset > 0.0f)
            {
                // if the lane to be removed starts ahead, the reverse target lane list
                rmvTgtLn = uint8_t(0);
            }
        }

        if (rmvTgtLn != uint8_t(0))
        {
            for (auto it_tgtLne = ctrlLnLst.begin(); it_tgtLne != ctrlLnLst.end();)
            {
                if (*it_tgtLne == rmvTgtLn)
                {
                    ctrlLnLst.erase(it_tgtLne);
                }
                else
                {
                    it_tgtLne++;
                }
            }
        }
        return true;
    }
    else
    {
        return false;
    }
}