#pragma once
#include <iostream>
#include "../../include/CtApBehaviorAnalysis_/common_BA.h"
// #include "mpc_drv_msgs/msg/control_waypoints.hpp"
// #include "mpc_drv_msgs/msg/road_info.hpp"
#include "../commonfunc.h"
#include "../forcePublic.h"
#include "algorithm"
// #include "../../include/CtApBehaviorAnalysis_/interface/slam_decision.h"
// #include "../../include/CtApBehaviorAnalysis_/interface/rm_road_info.h"
#include "dpc/impl_type_roadinfo.h"
#include "dpc/impl_type_controlwaypoints.h"
#include "dpc/impl_type_controlwaypointstruct.h"
#include "dpc/impl_type_remindwaypointstruct.h"

using namespace dpc;
class GlobalInfoProcess
{
public:
    void initGlobalInfo(ControlWaypoints &ehr_msg, const RoadInfo &roadModel, float vehSpd);
    void processGlobalInfo(float vehSpd, bool isAvgVehFlowDenseHigh, const BA::RouteRltvPos &routeRltvPos);
    bool FindWPExtendObj(const std::vector<WPExtendLaneStruct> &wpExtendLaneLst,
                         uint8_t laneID,
                         WPExtendLaneStruct &wpExtendLnObj); // find wpextendlane Obj by lane ID

public:
    BA::GlobalInfoContainer globalInfoContainer;

private:
    int CalcLaneGap(const uint8_t &egoLaneID, 
                    const std::vector<WPExtendLaneStruct> &wpExtendLaneLst, 
                    const std::vector<uint8_t> &trgtLaneLst,
                    std::vector<uint8_t> &minLnGapLaneLst);                                                                   // calculate minimum lane gap
    BA::LaneChgDrct DtrmLCDrcn(int minLaneGap);                                                                               // determine lane change direction by sign of minLaneGap
    bool GetFirstCtrlWP(const ControlWaypoints ehr_msg, ControlWayPointStruct &firstCtrlWayPnt); // get first control waypoint
    bool GetCnrLaneID(const ControlWayPointStruct &ctrlWayPnt,
                      uint8_t &lftCrnrLaneID,
                      uint8_t &rghtCrnrLaneID,
                      const RemindWayPointStruct &rmdWayPnt,
                      const uint8_t &egoLaneID); // get corner lane id for control waypoint
    bool GetCnrLaneID(const RemindWayPointStruct &rmdWayPnt,
                      uint8_t &lftCrnrLaneID,
                      uint8_t &rghtCrnrLaneID); // get corner lane id for reminder waypoint
    bool InitGlobalLnInfo(const ControlWayPointStruct ctrlWayPnt,
                          const std::vector<TransitLaneStruct> transitLnArr,
                          const uint8_t laneID,
                          BA::GlobalLaneInfo &globalLnInfo,
                          float vehSpd); // init single global lane info for control waypoint
    bool InitGlobalLnInfo(const RemindWayPointStruct rmdWayPnt,
                          const std::vector<TransitLaneStruct> transitLnArr,
                          const uint8_t laneID,
                          BA::GlobalLaneInfo &globalLnInfo,
                          float vehSpd); // init single global lane info for reminder waypoint
    uint8_t GetNearestLaneID(BA::RouteRltvPos rtRltvPos,
                             std::vector<uint8_t> rawTgtLaneLst,
                             const std::vector<WPExtendLaneStruct> &wpExtendLaneLst); // get the id of the lane which is nearest to raw target lane list
    bool AddDesLaneChgPnt(BA::GlobalLaneInfo &globalLnInfo,
                          float timeHeadway,
                          float minLnChgPntDst,
                          float vehSpd,
                          float adjDesLnChgPnt,
                          bool isNrstLnToTgtLn); // assign desired lane change points to globalLnInfo
    bool FindTgtGlblLnIndx(std::vector<BA::GlobalLaneInfo> globalLnInfoArr,
                           uint8_t laneID,
                           int &index);      // find target lane id' index in globalLnInfoArr
    float GetDesLCTimeHeadway(int lnChgNum); // get desired lane change time headway by lane change number
    float GetDesLCTimeHeadway_B(int lnChgNum, bool isAvgVehFlowHigh); // get desired lane change time headway by lane change number plan B, considering average vehicle flow dense
    float GetDesLCPntLowThr(int lnChgNum);   // get desired lane change points' low threshold by lane change number
    BA::RouteRltvPos GetRouteRltvPos(std::vector<uint8_t> rawTgtLaneLst,
                                     uint8_t lftCrnrLaneID,
                                     uint8_t rghtCrnrLaneID); // determine route's relative position
    int CalcGapBtwLns(const std::vector<WPExtendLaneStruct> &wpExtendLaneLst,
                      uint8_t laneID,
                      uint8_t tgtLaneID); // calculate lane gap from laneID to tgtLaneID
    bool rebuildTransLnList(ControlWaypoints &ehr_msg, bool toRebuildTransitLnLst); // based on extendLaneList, rebuild transit lane list, which should have been done by EHP
    bool MdfTgtLnLst(std::vector<uint8_t> &ctrlLnLst, const BA::RouteRltvPos &rtRltvPos, const ControlWaypoints &ehr_msg, const RoadInfo &road_model);
};
