#pragma once

#include <memory>
#include <vector>
#include <array>
#include <algorithm>
#include <math.h>

// #include "mpc_drv_msgs/msg/road_info.hpp"
// #include "mpc_drv_msgs/msg/corridor.hpp"
#include "../include/CtApBehaviorAnalysis_/BA_Enum_List.h"
#include "../include/CtApBehaviorAnalysis_/input_BA.h"
#include "../source/commonfunc.h"
#include "../../third_party/linear_interpolation.h"
#include "../include/CtApBehaviorAnalysis_/behaviorNodeGlobal.h"
// #include "mpc_drv_msgs/msg/passageob.hpp"
// #include "../include/CtApBehaviorAnalysis_/interface/rm_road_info.h"
#include "../include/CtApBehaviorAnalysis_/interface/dm_internal_interface.h"
#include "dpc/impl_type_roadinfo.h"

namespace LgtFcn
{
    class LgtCrdorGen
    {
    public:
        void Init(const float DurationTm,
                  const BA::BA_ActionTyp ActionType,
                  const Corridor CorridorInfo,
                  const float DrvrSetSpd,
                  const int BA_StepNum,
                  const float longtVelocity,
                  const bool m_BA_LaneChangeAbolitionAvlbl,
                  const vector<Corridor> Corridors2LonSub,
                  const BA::SeldDistLvl &drvrSetDstLvl,
                  const BA::SignSpeedLimit &signSpdLmtInfo01,
                  const BA::SignSpeedLimit &signSpdLmtInfo02,
                  const bool &lastCycleSpdLmtFlg,
                  const BA::SpdLmtTyp &lastCycleSpdLmtTyp,
                  const uint8_t &lastCycleSpdLmtID,
                  BA::DriveMd &driveMode,
                  BA::ScenarioTyp &ScenarioType); // get durationTm&Action&Corridor for each corridor cycle

        void LgtCrdorGenProcess(float duration,
                                BA::BA_ActionTyp Action,
                                Corridor Cr,
                                const float DrvrSetSpd,
                                int &Index_,
                                RoadInfo &roadModel,
                                int BA_StepNum,
                                float longtVelocity,
                                const bool m_BA_LaneChangeAbolitionAvlbl,
                                std::vector<BA::behaviorNodeGlobal>::iterator BNGItr,
                                std::vector<Corridor> Corridors2LonSub,
                                vector<DM::Passageob> &passageObLst,
                                const BA::SeldDistLvl &drvrSetDstLvl,
                                const BA::SignSpeedLimit &signSpdLmtInfo01,
                                const BA::SignSpeedLimit &signSpdLmtInfo02,
                                const bool &lastCycleSpdLmtFlg,
                                const BA::SpdLmtTyp &lastCycleSpdLmtTyp,
                                const uint8_t &lastCycleSpdLmtID,
                                BA::DriveMd &driveMode,
                                BA::ScenarioTyp &ScenarioType); // serivce of RoadModelUpdate
        enum AutoLaneChngSetResp
        {
            eALCSR_Norequest = 0,
            eALCSR_Level1 = 1,  //Conservative
            eALCSR_Level2 = 2,  //Default
            eALCSR_Level3 = 3   //Aggressvie
        };
    private:
        // Basical
        float L_ActionTm;
        BA::BA_ActionTyp L_ActionType;
        int L_ObjNum;
        Corridor L_Corridor;
        Corridor L_CorridorSub;
        float L_DrvrSetSpd;
        uint32_t L_TempObjID = 0;
        float L_TempObjDist = 1000;
        float L_TempObjSpd = 100;
        float L_EgoLaneTau;
        float L_EgoLaneD0;
        float L_SideLaneTau;
        float L_SideLaneD0;
        // float L_YellowGain = 0.8; // move to commonfcn 0324
        // float L_RedGain = 0.6;    // move to commonfcn 0324
        float L_Blue;
        float L_Yellow;
        float L_Red;
        float L_DesireDist;
        float L_VEgo;
        float L_DesireSpd;
        float L_SpdLimit;
        bool L_EgoOvrLne; // chk ego vehicle over lne, if ture, Ego do not
        float L_DrvrDistLvl;
        //    float L_EgoLenth = 4.9; // move to commonfcn 0324
        AutoLaneChngSetResp L_ALCSR = eALCSR_Norequest; //0720
        BA::DriveMd L_DriveMode = BA::DriveMd_Normal; //0730 merge ALCSR with DriveMode
        // For SpdLimit
        BA::SignSpeedLimit L_SignSpdLmtInfo01;
        BA::SignSpeedLimit L_SignSpdLmtInfo02;
        bool L_LastCycleSpdLmtFlg;
        BA::SpdLmtTyp L_LastCycleSpdLmtTyp;
        uint8_t L_LastCycleSpdLmtID;
        BA::ScenarioTyp L_ScnrTyp;
        

        // For LC Space
        bool FrtEst_ = true;
        bool BckEst_ = false;
        // float AlcDis_TarLane_ = 0;
        // float reAlcDis_TarLane_ = 0;
        float AlcDis_EgoLane_ = 0;
        float reAlcDis_EgoLane_ = 0;
        float AlcDis_TarLane_frt_ = 0;
        float reAlcDis_TarLane_frt_ = 0;
        float AlcDis_TarLane_bck_ = 0;
        float reAlcDis_TarLane_bck_ = 0;
        float LC_TarFrtObjDistNrst_ = -1;
        float LC_TarBckObjDistNrst_ = -1;
        //  LC Space Decrease
        // float ALCDisDecrease = 2.5;    // add 0311    move to commonfcn 0323
        // For LCP Space
        float iAlcDis_TarLane_ = 0;
        float reiAlcDis_TarLane_ = 0;

        // For PassageGen Chk
        bool LC_PassageGenFlg_ = false;
        bool LCP_Frt_PassageGenFlg_ = false;
        bool LCP_Bck_PassageGenFlg_ = false;

        // Inherit LabelObj For JudgeFcn
        // FOR LC
        bool LC_EgoFrtObjExstFlg_ = false;
        uint32_t LC_EgoFrtObjID_ = 0;
        float LC_EgoFrtObjDist_ = 0;
        float LC_EgoFrtObjSpd_ = 0;
        float LC_EgoFrtObjLen_ = 0;
        uint8_t LC_EgoFrtObjCls_ = 0;
        float LC_EgoFrtObjAcc_ = 0;

        bool LC_EgoBckObjExstFlg_ = false; // 0714 use for ego bck obj Label for Unexpect cls 1 obj chk
        uint32_t LC_EgoBckObjID_ = 0;

        bool LC_TarFrtObjExstFlg_ = false;
        uint32_t LC_TarFrtObjID_ = 0;
        float LC_TarFrtObjDist_ = 0;
        float LC_TarFrtObjSpd_ = 0;
        float LC_TarFrtObjLen_ = 0;
        uint8_t LC_TarFrtObjCls_ = 0;
        float LC_TarFrtObjAcc_ = 0;

        bool LC_TarBckObjExstFlg_ = false;
        uint32_t LC_TarBckObjID_ = 0;
        float LC_TarBckObjDist_ = 0;
        float LC_TarBckObjSpd_ = 0;
        float LC_TarBckObjLen_ = 0;
        uint8_t LC_TarBckObjCls_ = 0;
        float LC_TarBckObjAcc_ = 0;

        // FOR LCP
        bool LCP_EgoFrtObjExstFlg_ = false;
        uint32_t LCP_EgoFrtObjID_ = 0;
        float LCP_EgoFrtObjDist_ = 0;
        float LCP_EgoFrtObjSpd_ = 0;
        float LCP_EgoFrtObjLen_ = 0;
        uint8_t LCP_EgoFrtObjCls_ = 0;
        float LCP_EgoFrtObjACC_ = 0;

        bool LCP_EgoBckObjExstFlg_ = false;
        uint32_t LCP_EgoBckObjID_ = 0;
        float LCP_EgoBckObjDist_ = 0;
        float LCP_EgoBckObjSpd_ = 0;
        float LCP_EgoBckObjLen_ = 0;
        uint8_t LCP_EgoBckObjCls_ = 0;
        float LCP_EgoBckObjAcc_ = 0;

        bool LCP_NrObjExstFlg_ = false;
        uint32_t LCP_NrObjID_ = 0;
        float LCP_NrObjDist_ = 0;
        float LCP_NrObjSpd_ = 0;
        float LCP_NrObjLen_ = 0;
        uint8_t LCP_NrObjCls_ = 0;
        float LCP_NrObjAcc_ = 0;

        bool LCP_NrFrtObjExstFlg_ = false;
        uint32_t LCP_NrFrtObjID_ = 0;
        float LCP_NrFrtObjDist_ = 0;
        float LCP_NrFrtObjSpd_ = 0;
        float LCP_NrFrtObjLen_ = 0;
        uint8_t LCP_NrFrtObjCls_ = 0;
        float LCP_NrFrtObjAcc_ = 0;

        bool LCP_NrBckObjExstFlg_ = false;
        uint32_t LCP_NrBckObjID_ = 0;
        float LCP_NrBckObjDist_ = 0;
        float LCP_NrBckObjSpd_ = 0;
        float LCP_NrBckObjLen_ = 0;
        uint8_t LCP_NrBckObjCls_ = 0;
        float LCP_NrBckObjAcc_ = 0;

        // For LabelFcn
        bool FrtObjExstFlg_;
        bool BckObjExstFlg_;
        bool NrObjExstFlg_;
        bool NrFrtObjExstFlg_;
        bool NrBckObjExstFlg_;
        std::vector<ObjectLocation, std::allocator<ObjectLocation>>::iterator PrtFrt_;
        std::vector<ObjectLocation, std::allocator<ObjectLocation>>::iterator PrtBck_;
        std::vector<ObjectLocation, std::allocator<ObjectLocation>>::iterator PrtNr_;
        std::vector<ObjectLocation, std::allocator<ObjectLocation>>::iterator PrtNrFrt_;
        std::vector<ObjectLocation, std::allocator<ObjectLocation>>::iterator PrtNrBck_;

        // observe LC space not satisfy flag
        bool LneChgRes0_ = 0;
        bool LneChgRes1_ = 0;
        bool LneChgRes2_ = 0;
        bool LneChgRes3_ = 0;
        bool PassageGenFlg_ = false;

        int L_BA_StepNum; // Judge first Passagelist save for TRP Control;
        std::vector<Passage> L_PassageFirstStep;
        std::vector<uint32_t> L_InterestObjIDList;
        std::vector<uint32_t> L_ReminderObjIDList;        
        uint32_t CommonObjID_ = 0;
        // for LCP Space
        float FrtLcpDis_TarLane_EgoFrt_Raw_ = 0;
        float FrtLcpDis_TarLane_EgoBck_Raw_ = 0;
        float BckLcpDis_TarLane_EgoFrt_Raw_ = 0;
        float BckLcpDis_TarLane_EgoBck_Raw_ = 0;
        float FrtLcpDis_TarLane_EgoFrt_ = 0;
        float FrtLcpDis_TarLane_EgoBck_ = 0;
        float BckLcpDis_TarLane_EgoFrt_ = 0;
        float BckLcpDis_TarLane_EgoBck_ = 0;
        float LcpEgoLaneSftyDist_frt = 2; // add 0623  dynamic lcp ego sfy distance
                                       //        float LCP_DisDebounce = 1.2;   move to commonfcn 0323
        float LcpEgoLaneSftyDist_bck = 2; //1117 split frt&bck ,using different value, reduce frtlcp lock.
        bool Frt_LneChgPrRes1_ = 0;
        bool Frt_LneChgPrRes2_ = 0;
        bool Frt_LneChgPrRes3_ = 0;
        bool Frt_LneChgPrRes4_ = 0;
        bool Frt_LneChgPrRes5_ = 0; // large spderr between ego and nr,do not work 0525
        bool Bck_LneChgPrRes1_ = 0;
        bool Bck_LneChgPrRes2_ = 0;
        bool Bck_LneChgPrRes3_ = 0;
        bool Bck_LneChgPrRes4_ = 0;
        bool Bck_LneChgPrRes5_ = 0; // small NrObjSpd do not work 0520
        // float LCP_EgoAccelSpd = 5.55;   // 15kph move to commonfcn 0324
        // float LCP_EgoDeAccelSpd = 4.16; // 20kph move to commonfcn 0324
        // for senario label
        uint8_t LC_Cls_ = 0;
        uint8_t LCP_Cls_frt_ = 0;
        uint8_t LCP_Cls_bck_ = 0;
        // for LCP Cond Value Ob
        float LCP_Frt_MovDist_ = 0;
        float LCP_Frt_EgoLneSpace_ = 0;
        float LCP_Frt_EstSpace_ = 0;
        float LCP_Frt_NdSpace_ = 0;
        float LCP_Bck_MovDist_ = 0;
        float LCP_Bck_EgoLneSpace_ = 0;
        float LCP_Bck_EstSpace_ = 0;
        float LCP_Bck_NdSpace_ = 0;
        float LCP_Frt_EgoArvEstTm_ = 0;
        float LCP_Frt_EgoRegionDist_ = 0;
        float LCP_Frt_EgoArvEstTmDynThr_ = 5; // 0614 add
        float LCP_Bck_EgoArvEstTm_ = 0;
        float LCP_Bck_EgoRegionDist_ = 0;
        float LCP_Bck_EgoArvEstTmDynThr_ = 5;

        // for Compress Gain Ob
        float CmprsGain_lc_tar_frt_ = 1.0;
        float CmprsGain_lc_tar_bck_ = 1.0;
        float CmprsGain_lcp_frt_frt_ = 1.0;
        float CmprsGain_lcp_frt_bck_ = 1.0;
        float CmprsGain_lcp_bck_frt_ = 1.0;
        float CmprsGain_lcp_bck_bck_ = 1.0;
        enum BA::LCPSts Frt_LCPState_ = BA::LCPSts_Normal;
        enum BA::LCPSts Bck_LCPState_ = BA::LCPSts_Normal;

        // void VSEInfoUpdate(const float &longtVelocity);
        void LgtInfoCalc();    // Integrate LgtControl needed info, include TmGap,SfDist,DsrDist,Borders,DsrSpd,SetSpd;     merge 0426
        void RefGainCalc();    // TimeGap&safeDist Gain calc
        void DesireSVCalc();   // DesireDist Basic Calc
        void RefDistLineGen(); // Yellow/Blue/Red Border Gen
        void DesSpdCalc();     // calculate desired speed, namely following vehicle speed

        void SetSpdSelect();          // select setspd&curvtespd&LneSpdLmt for setspd output
        void LneSpdLmtOperateJudge(); // Judge LneSpdLmt do or not

        void EachCridorObjJudge(int &index_,
                                RoadInfo &roadModel,
                                int BA_StepNum,
                                std::vector<BA::behaviorNodeGlobal>::iterator BNGItr,
                                vector<DM::Passageob> &passageObLst); // Locate Obj with Corridor &Interest Obj Num
        void UpdatePassageInfo(int &index_, int BA_StepNum, Passage &L_Passage);
        void UpdatePassageObInfo(int &index_, int BA_StepNum, DM::Passageob &PassageOb);
        void IndexSend2LatFcn();
        // void LgtInfoGenWork();  // cover by PassageGenPush()
        void PassageGenPush(int &index_, int BA_StepNum, RoadInfo &roadModel);                                                       // include lgtInfoGen & PassageGen & PassagePush
        void PassageOBGenPush(int &index_, int BA_StepNum, RoadInfo &roadModel, vector<DM::Passageob> &passageObLst); // OB all passage include fail gen;
        void DoubleObjLabel(Corridor &Corridor);
        void TripleCrdObjLabel(Corridor &Corridor);
        float EstDistCalc(float EstSpd, bool FrtEst);
        bool ObjConFlictChk(std::vector<BA::behaviorNodeGlobal>::iterator BNGItr, RoadInfo &roadModel); // Chk InterestObj Wehter have the same ID or not, if Conflict return ture; 0614
        bool CommonResChk(std::vector<BA::behaviorNodeGlobal>::iterator BNGItr, RoadInfo &roadModel);   // Manage Common Fail Condition for LC&LCP Manner , if Fail return ture; 0614
        float DistCmPresGainCalc(float EgoSpd, float ReferObjSpd,std::vector<BA::behaviorNodeGlobal>::iterator BNGItr, bool EgoFrtEst); // LC_Tar & FrtLCP_BckDis & Bck_LCP_FrtDis Compress Logic which take two car spd err into consideration. 0701
        bool BckObjSpdErrChk();                                                    // LC & LCP IF TarBck&Ego Spd Err to large , Quit Manner.
        bool GetRefLneMrk(RoadInfo &RoadModel, float &TgtC0);
        float LCAbltTolrGain(RoadInfo &RoadModel);
        bool BckLCPNrObjChk();
        bool LCPActionLongTm(std::vector<BA::behaviorNodeGlobal>::iterator BNGItr);

        float LCPMergeInGain(float EgoSpd, float ReferObjSpd,std::vector<BA::behaviorNodeGlobal>::iterator BNGItr, bool EgoFrtEst);
    };

} // namespace LgtFcn
