#pragma once

#include <memory>
#include <vector>
#include <array>
#include <algorithm>

using namespace std;

namespace MathFcn
{
    //  input breakpoint must be strictly monotonically increasing
    template <class T1>
    class LinearIntrPltn
    {

    public:
        // private:
        vector<T1> y_;
        vector<T1> x_;
        T1 xA_point;
        T1 xB_point;
        T1 yA_point;
        T1 yB_point;
        T1 xMax_point;
        T1 xMin_point;
        T1 Value;
        T1 Point_;

        vector<T1> z_;
        T1 zAA_;
        T1 zAB_;
        T1 zBA_;
        T1 zBB_;
        T1 yMax_point;
        T1 yMin_point;
        T1 xPoint_;
        T1 yPoint_;

    public:
        bool BrkPntChk(vector<T1> ChkAry);

        bool init(vector<T1> x, vector<T1> y, T1 Point);
        T1 InterPolation(vector<T1> x, vector<T1> y, T1 Point);

        bool init(vector<T1> x, vector<T1> y, vector<T1> z, T1 xPoint, T1 yPoint);
        T1 InterPolation(vector<T1> x, vector<T1> y, vector<T1> z, T1 xPoint, T1 yPoint);
    };

    template <class T1>
    bool LinearIntrPltn<T1>::BrkPntChk(vector<T1> ChkAry)
    {
        int Num = int(ChkAry.size());
        for (int i = 0; i < Num - 1; i++)
        {
            if (ChkAry[i] >= ChkAry[i + 1])
            {
                return false;
                break;
            }
        }
        return true;
    }

    template <class T1>
    T1 LinearIntrPltn<T1>::InterPolation(vector<T1> x, vector<T1> y, T1 Point)
    {
        if (init(x, y, Point))
        {
            if (Point_ <= xMin_point)
            {
                Value = y_.at(0);
            }

            if (Point >= xMax_point)
            {
                Value = y_.back();
            }

            if (Point_ > xMin_point && Point_ < xMax_point)
            {
                for (int i = 0; i < x_.size() - 1; i++)
                {
                    if (Point_ >= x_[i])
                    {
                        xA_point = x_[i];
                        xB_point = x_[i + 1];
                        yA_point = y_[i];
                        yB_point = y_[i + 1];
                        // break;
                    }
                }
                Value = yA_point + (Point_ - xA_point) * ((yB_point - yA_point) / (xB_point - xA_point));
            }
            return Value;
        }
        else
        {
            return T1(1);
        }
    }

    template <class T1>
    bool LinearIntrPltn<T1>::init(vector<T1> x, vector<T1> y, T1 Point)
    {
        x_.clear();
        y_.clear();
        if (x.size() == y.size() && BrkPntChk(x))
        {
            for (int i = 0; i < x.size(); i++) // push x_
            {
                float x_temp = x[i];
                x_.push_back(x_temp);
            }
            for (int i = 0; i < y.size(); i++) // push y_
            {
                float y_temp = y[i];
                y_.push_back(y_temp);
            }
            Point_ = Point; // push Point
            xMin_point = x_[0];
            xMax_point = x_.back();

            return true;
        }
        else
        {
            return false;
        }
    }

    template <class T1>
    bool LinearIntrPltn<T1>::init(vector<T1> x, vector<T1> y, vector<T1> z, T1 xPoint, T1 yPoint)
    {
        x_.clear();
        y_.clear();
        z_.clear();
        if (z.size() == x.size() * y.size() && BrkPntChk(x) && BrkPntChk(y))
        {
            for (int i = 0; i < x.size(); i++) // push x_
            {
                float x_temp = x[i];
                x_.push_back(x_temp);
            }
            for (int i = 0; i < y.size(); i++) // push y_
            {
                float y_temp = y[i];
                y_.push_back(y_temp);
            }
            for (int i = 0; i < z.size(); i++) // push z_
            {
                float z_temp = z[i];
                z_.push_back(z_temp);
            }
            xPoint_ = xPoint;
            yPoint_ = yPoint;
            xMin_point = x_[0];
            xMax_point = x_.back();
            yMin_point = y_[0];
            yMax_point = y_.back();
            return true;
        }
        else
        {
            return false;
        }
    }

    template <class T1>
    T1 LinearIntrPltn<T1>::InterPolation(vector<T1> x, vector<T1> y, vector<T1> z, T1 xPoint, T1 yPoint)
    {
        bool chk = init(x, y, z, xPoint, yPoint);
        // std::cout << " chk  = : " << int(chk) << std::endl;
        if (chk)
        {
            // input coordinate included simpliy
            if (xPoint_ < xMin_point)
            {
                xPoint_ = xMin_point + T1(0.001);
            }
            else if (xPoint_ > xMax_point)
            {
                xPoint_ = xMax_point - T1(0.001);
            }

            if (yPoint_ < yMin_point)
            {
                yPoint_ = yMin_point + T1(0.001);
            }
            else if (yPoint_ > yMax_point)
            {
                yPoint_ = yMax_point - T1(0.001);
            }

            // get four point coordinate & value;
            int xlb = 0;
            int ylb = 0;
            int xsiz = int(x_.size());
            for (int i = 0; i < x_.size() - 1; i++)
            {
                if (xPoint_ >= x_[i])
                {
                    xA_point = x_[i];
                    xB_point = x_[i + 1];
                    xlb = i;
                }
            }
            for (int i = 0; i < y_.size() - 1; i++)
            {
                if (yPoint_ >= y_[i])
                {
                    yA_point = y_[i];
                    yB_point = y_[i + 1];
                    ylb = i;
                }
            }

            zAA_ = z_[xlb + ylb * xsiz];
            zAB_ = z_[xlb + (ylb + 1) * xsiz];
            zBA_ = z_[(xlb + 1) + ylb * xsiz];
            zBB_ = z_[(xlb + 1) + (ylb + 1) * xsiz];

            // matrix calc for Value
            T1 A1 = (xB_point - xPoint_) * zAA_ + (xPoint_ - xA_point) * zBA_;
            T1 A2 = (xB_point - xPoint_) * zAB_ + (xPoint_ - xA_point) * zBB_;
            T1 B = (xB_point - xA_point) * (yB_point - yA_point);
            Value = (A1 * (yB_point - yPoint_) + A2 * (yPoint_ - yA_point)) / B;
            return Value;
        }
        else
        {
            return T1(1);
        }
    }
}