#include "funnel.h"

namespace ACC_Target_Selection{

    Funnel::Funnel(const Param_Funnel &param_conf, const Logger &logger):
            param_conf_(param_conf),
            logger_(logger),
            outerleft_(zero_float32_t),
            outerright_(zero_float32_t),
            innerleft_(zero_float32_t),
            innerright_(zero_float32_t),
            laneoffset_(zero_float32_t),
            lanewidth_(),
            lanewidthsaturated_(),
            preparedloolkupparam_(static_cast<size_t>(0u), zero_float32_t),
            calculatelowconffunnel_(boolean_t_false) {
        for (basetype::objectid_t ii = zero_objectid_t; ii < c_ObjectIndexMax; ii++) {
            lanewidth_.at(ii) = zero_float32_t;
            lanewidthsaturated_.at(ii) = zero_float32_t;
        }
    }

    Funnel::Funnel(const Funnel& r) :
            param_conf_(r.param_conf_),
            logger_(r.logger_),
            outerleft_(r.outerleft_),
            outerright_(r.outerright_),
            innerleft_(r.innerleft_),
            innerright_(r.innerright_),
            laneoffset_(r.laneoffset_),
            lanewidth_(r.lanewidth_),
            lanewidthsaturated_(r.lanewidthsaturated_),
            preparedloolkupparam_(r.preparedloolkupparam_),
            calculatelowconffunnel_(r.calculatelowconffunnel_) {
    }

    Funnel::~Funnel() {
    }

    void Funnel::updatelanewidth(const basetype::float32_t &lanewidth, const basetype::objectid_t &idn0) {
        basetype::float32_t sampletime{c_SampleTime};
        lanewidth_.at(idn0) = Util::limitchange(lanewidth,
                                                lanewidth_.at(idn0),
                                                static_cast<basetype::float32_t>(20.0f) * sampletime,
                                                static_cast<basetype::float32_t>(0.25f) * sampletime);
        
        lanewidthsaturated_.at(idn0) = Util::saturate(lanewidth_.at(idn0),
                                                      static_cast<basetype::float32_t>(4.0f),
                                                      static_cast<basetype::float32_t>(3.2f));
        
        // log lanewidth
        logger_.loglanewidth(lanewidthsaturated_.at(idn0), idn0);
    }

    void Funnel::prepare(const basetype::float32_t &egospd,
                         const basetype::boolean_t &leftindicator,
                         const basetype::boolean_t &rightindicator,
                         const basetype::float32_t &laneoffset) {
        PreLoolkUpParam preloolkupparam{};
        Util::preloolkup(param_conf_.funneloffsetspeed,
                         egospd,
                         boolean_t_false,
                         preloolkupparam);
        
        basetype::float32_t normal{Util::lookuprun(param_conf_.funneloffsetnormal,
                                                   preloolkupparam)};
        basetype::float32_t bigger{Util::lookuprun(param_conf_.funneloffsetovertaking,
                                                   preloolkupparam)};
        basetype::float32_t smaller{Util::lookuprun(param_conf_.funneloffsetopposite,
                                                    preloolkupparam)};
        basetype::float32_t biggerinner{Util::lookuprun(param_conf_.innerfunneloffsetovertaking,
                                                        preloolkupparam)};
        basetype::float32_t smallerinner{Util::lookuprun(param_conf_.innerfunneloffsetopposite,
                                                         preloolkupparam)};
        
        basetype::float32_t outerleft{zero_float32_t};
        basetype::float32_t outerright{zero_float32_t};
        basetype::float32_t innerleft{zero_float32_t};
        basetype::float32_t inneright{zero_float32_t};
        if (leftindicator == boolean_t_true) {
            outerleft     = bigger;
            innerleft     = biggerinner;
        }
        else if (rightindicator == boolean_t_true) {
            outerleft = smaller;
            innerleft = smallerinner;
        }
        else{
            outerleft = normal;
            innerleft = zero_float32_t;
        }
        if (rightindicator ==boolean_t_true) {
            outerright = bigger;
            inneright = biggerinner;
        }
        else if (leftindicator == boolean_t_true) {
            outerright = smaller;
            inneright = smallerinner;
        }
        else{
            outerright = normal;
            inneright = zero_float32_t;
        }

        basetype::float32_t upperlimit{c_SampleTime * param_conf_.funnelgrowthrate};
        basetype::float32_t lowerlimit{c_SampleTime * param_conf_.funnelshrinkrate * (-1.0f)};

        outerleft_ = Util::limitchange(outerleft, outerleft_, upperlimit, lowerlimit);
        outerright_ = Util::limitchange(outerright, outerright_, upperlimit, lowerlimit);
        
        upperlimit = c_SampleTime * param_conf_.innerfunnelgrowthrate;
        lowerlimit = c_SampleTime * param_conf_.innerfunnelshrinkrate * (-1.0f);

        innerleft_ = Util::limitchange(innerleft, innerleft_, upperlimit, lowerlimit);
        innerright_ = Util::limitchange(inneright, innerright_, upperlimit, lowerlimit);

        // log outerleft_ outerright_ innerleft_ innerright_
        logger_.loginnerleftfunneloffset(innerleft_);
        logger_.loginnerrightfunneloffset(innerright_);
        logger_.logouterleftfunneloffset(outerleft_);
        logger_.logouterrightfunneloffset(outerright_);

        laneoffset_ = laneoffset;

        //log laneoffset
        logger_.loglaneoffset(laneoffset_);
    }

    void Funnel::prepareobject(const basetype::boolean_t &isrearobject,
                               const basetype::float32_t &objlgtdis,
                               const basetype::objectid_t &idn0,
                               const basetype::float32_t &lanewidth) {
        Funnel::updatelanewidth(lanewidth, idn0);

        if (isrearobject == boolean_t_false) {
            Util::preloolkup(param_conf_.funnelbreakpoints,
                             objlgtdis,
                             boolean_t_false,
                             preparedloolkupparam_);
        } else {
            Util::preloolkup(param_conf_.funnelrearbreakpoints,
                             objlgtdis,
                             boolean_t_false,
                             preparedloolkupparam_);
        }
        
        if ((objlgtdis >= param_conf_.maxrangehighquality) ||
            (objlgtdis <= param_conf_.minrangehighquality)) {
            calculatelowconffunnel_ = boolean_t_true;
        } else {
            calculatelowconffunnel_ = boolean_t_false;
        }
    }

    FunnelWidthParam Funnel::getinner(const basetype::boolean_t &isrearobject, const basetype::objectid_t &idn0) {
        FunnelWidthParam res{};
        basetype::float32_t innerfunnel{zero_float32_t};
        if (isrearobject == boolean_t_false) {
            innerfunnel = lanewidthsaturated_.at(idn0) * 
                        Util::lookuprun(param_conf_.innerfunnelwidth,
                                        preparedloolkupparam_);
        } else {
            innerfunnel = lanewidthsaturated_.at(idn0) * 
                        Util::lookuprun(param_conf_.innerrearfunnelwidth,
                                        preparedloolkupparam_);
        }
        basetype::float32_t halffunnel{zero_float32_t};
        halffunnel = innerfunnel * static_cast<basetype::float32_t>(0.5f);
        res.rightwidth = static_cast<basetype::float32_t>(-1.0f) *(innerright_ + halffunnel) + laneoffset_*0.5f;

        if (calculatelowconffunnel_ == boolean_t_true) {
            basetype::float32_t outfunnel;
            if (isrearobject == boolean_t_false) {
                outfunnel = lanewidthsaturated_.at(idn0) *
                                Util::lookuprun(param_conf_.outerfunnelwidth,
                                                preparedloolkupparam_);
            } else {
                outfunnel = lanewidthsaturated_.at(idn0) *
                                Util::lookuprun(param_conf_.outerrearfunnelwidth,
                                                preparedloolkupparam_);
            }
            res.leftwidth = outfunnel * (param_conf_.adjacentlanemultifactorinner) * static_cast<basetype::float32_t>(0.5f);
            res.rightwidth = res.rightwidth * param_conf_.rightinnerlowconfscaling;
        }
        else{
            res.leftwidth = innerleft_ + halffunnel + laneoffset_*0.5f;
        }

        return res;
    }

    FunnelWidthParam Funnel::getouter(const basetype::boolean_t &isrearobject, const basetype::objectid_t &idn0) {
        FunnelWidthParam res{};

        basetype::float32_t outerfunnel{zero_float32_t};
        if (isrearobject == boolean_t_false) {
            outerfunnel = lanewidthsaturated_.at(idn0) * 
                            Util::lookuprun(param_conf_.outerfunnelwidth,
                                            preparedloolkupparam_);
        } else{
            outerfunnel = lanewidthsaturated_.at(idn0) * 
                            Util::lookuprun(param_conf_.outerrearfunnelwidth,
                                            preparedloolkupparam_);
        }
        basetype::float32_t halffunnel{zero_float32_t};
        halffunnel = outerfunnel * static_cast<basetype::float32_t>(0.5f);
        res.rightwidth = static_cast<basetype::float32_t>(-1.0f) *(outerright_ + halffunnel) + laneoffset_ * 0.5f;

        if (calculatelowconffunnel_ == boolean_t_true) {
            res.leftwidth = outerfunnel * (param_conf_.adjacentlanemultifactorinner) *
                    (param_conf_.adjacentlanemultifactorouter) *
                    static_cast<basetype::float32_t>(0.5f);
            res.rightwidth = (res.rightwidth) * param_conf_.rightouterlowconfscaling;
        }
        else{
            res.leftwidth = outerleft_ + halffunnel + laneoffset_ * 0.5f;
        }

        return res;
    }

    FunnelWidthParam Funnel::getwideinner(const basetype::boolean_t &isrearobject, const basetype::objectid_t &idn0) {
        FunnelWidthParam res{};
        basetype::float32_t innerfunnel{zero_float32_t};
        if (isrearobject == boolean_t_false) {
            innerfunnel = lanewidthsaturated_.at(idn0) * 
                            Util::lookuprun(param_conf_.innerfunnelwidth,
                                            preparedloolkupparam_);
        } else {
            innerfunnel = lanewidthsaturated_.at(idn0) * 
                            Util::lookuprun(param_conf_.innerrearfunnelwidth,
                                            preparedloolkupparam_);
        }
        
        res.leftwidth = innerfunnel * (param_conf_.adjacentlanemultifactorinner) * static_cast<basetype::float32_t>(0.5f);
        res.rightwidth = res.leftwidth * (static_cast<basetype::float32_t>(-1.0f));

        return res;
    }

    FunnelWidthParam Funnel::getwideouter(const basetype::boolean_t &isrearobject, const basetype::objectid_t &idn0) {
        FunnelWidthParam res{};
        basetype::float32_t outerfunnel{zero_float32_t};
        if (isrearobject == boolean_t_false) {
            outerfunnel = lanewidthsaturated_.at(idn0) * 
                            Util::lookuprun(param_conf_.outerfunnelwidth,
                                            preparedloolkupparam_);
        } else {
            outerfunnel = lanewidthsaturated_.at(idn0) * 
                            Util::lookuprun(param_conf_.outerrearfunnelwidth,
                                            preparedloolkupparam_);
        }
        
        res.leftwidth = outerfunnel * (param_conf_.adjacentlanemultifactorinner) * 
                (param_conf_.adjacentlanemultifactorouter) *
                static_cast<basetype::float32_t>(0.5f);
        res.rightwidth = res.leftwidth * (static_cast<basetype::float32_t>(-1.0f));

        return res;
    }
}