#ifndef ACC_TARGET_SELECTION_FUNNEL_H
#define ACC_TARGET_SELECTION_FUNNEL_H

#include <array>
#include <memory>

#include "basetype.h"
#include "param.h"
#include "logger.h"
#include "atsutil.h"

namespace ACC_Target_Selection{
    struct FunnelWidthParam {
        basetype::float32_t leftwidth;
        basetype::float32_t rightwidth;
        explicit FunnelWidthParam() :
                leftwidth(zero_float32_t),
                rightwidth(zero_float32_t) {
        }
        FunnelWidthParam(const FunnelWidthParam &r) :
                leftwidth(r.leftwidth),
                rightwidth(r.rightwidth) {
        }
        explicit FunnelWidthParam(const basetype::float32_t &leftwidth_,
                                  const basetype::float32_t &rightwidth_):
                leftwidth(leftwidth_),
                rightwidth(rightwidth_) {
        }
        FunnelWidthParam& operator=(const FunnelWidthParam& rhs) {
            leftwidth = rhs.leftwidth;
            rightwidth = rhs.rightwidth;
            return *this;
        }
    };
    
    class Funnel {
    public:
        const Param_Funnel &param_conf_;
        const Logger &logger_;

        explicit Funnel(const Param_Funnel &param_conf, const Logger &logger);
        Funnel(const Funnel& r);
        ~Funnel();

        void prepare(const basetype::float32_t &egospd,
                     const basetype::boolean_t &leftindicator,
                     const basetype::boolean_t &rightindicator,
                     const basetype::float32_t &laneoffset);

        void prepareobject(const basetype::boolean_t &isrearobject,
                           const basetype::float32_t &objlgtdis,
                           const basetype::objectid_t &idn0,
                           const basetype::float32_t &lanewidth);

        FunnelWidthParam getinner(const basetype::boolean_t &isrearobject, const basetype::objectid_t &idn0);
        
        FunnelWidthParam getouter(const basetype::boolean_t &isrearobject, const basetype::objectid_t &idn0);
        
        FunnelWidthParam getwideinner(const basetype::boolean_t &isrearobject, const basetype::objectid_t &idn0);

        FunnelWidthParam getwideouter(const basetype::boolean_t &isrearobject, const basetype::objectid_t &idn0);
    private:
        basetype::float32_t outerleft_;
        basetype::float32_t outerright_;
        basetype::float32_t innerleft_;
        basetype::float32_t innerright_;
        basetype::float32_t laneoffset_;
        std::array<basetype::float32_t, c_ObjectSize> lanewidth_;
        std::array<basetype::float32_t, c_ObjectSize> lanewidthsaturated_;
        PreLoolkUpParam preparedloolkupparam_;
        basetype::boolean_t calculatelowconffunnel_;

        void updatelanewidth(const basetype::float32_t &lanewidth, const basetype::objectid_t &idn0);
    };

}



#endif