#include "egoinfohandler.h"
#include "atsutil.h"

namespace ACC_Target_Selection {
EgoInfoHandler::EgoInfoHandler() :
        leftindicatortimer(zero_uint16_t),
        rightindicatortimer(zero_uint16_t),
        preleftindicator(zero_uint8_t),
        prerightindicator(zero_uint8_t),
        prestandstillfactor(one_float32_t),
        preindicator(IndicatorStatus::None),
        indicatorwithoffdelay(IndicatorStatus::None),
        indicatortimer(zero_uint16_t),
        egovehinfo_(EgoVehicleInformation_Default),
        age_(zero_uint16_t) {
}

EgoInfoHandler::EgoInfoHandler(const EgoInfoHandler &r):
        leftindicatortimer(r.leftindicatortimer),
        rightindicatortimer(r.rightindicatortimer),
        preleftindicator(r.preleftindicator),
        prerightindicator(r.prerightindicator),
        prestandstillfactor(r.prestandstillfactor),
        preindicator(r.preindicator),
        indicatorwithoffdelay(r.indicatorwithoffdelay),
        indicatortimer(r.indicatortimer),
        egovehinfo_(r.egovehinfo_),
        age_(r.age_) {
}

EgoInfoHandler::~EgoInfoHandler() {}
void EgoInfoHandler::updateegoinfo(
    const int64_t &now_timestamp,
    const std::shared_ptr<vehicle_msgs::msg::ApSas10msPdu00> &SAS_10ms_PDU00,
    const std::shared_ptr<vehicle_msgs::msg::ApBcm50msPdu02> &BCM_50ms_PDU02,
    const std::shared_ptr<vehicle_msgs::msg::ApIbs20msPdu18> &IBS_20ms_PDU18,
    const std::shared_ptr<vehicle_msgs::msg::ApIbs20msPdu15> &IBS_20ms_PDU15,
    const std::shared_ptr<vehicle_msgs::msg::ApIbs20msPdu08> &IBS_20ms_PDU08) {
  EgoInfoHandler::update_steeringwheelangle(SAS_10ms_PDU00);

  EgoInfoHandler::update_indicator(BCM_50ms_PDU02);

  EgoInfoHandler::update_standstillstatus(IBS_20ms_PDU18);

  EgoInfoHandler::update_vehspd(IBS_20ms_PDU15);

  EgoInfoHandler::update_yawrate(IBS_20ms_PDU08);

  EgoInfoHandler::update_lgtaccel(IBS_20ms_PDU08);

  EgoInfoHandler::update_lataccel(IBS_20ms_PDU08);
}

void EgoInfoHandler::update_steeringwheelangle(
    const std::shared_ptr<vehicle_msgs::msg::ApSas10msPdu00> &SAS_10ms_PDU00) {
  basetype::float32_t steeringangle{egovehinfo_.SteeringAngle};

  if (SAS_10ms_PDU00->istrgwhlangv == 0u) {
    steeringangle =
        Util::safetydiv(SAS_10ms_PDU00->istrgwhlang * 3.1415926f, 180.0f);
    if (steeringangle > 0.15f) {
      steeringangle = steeringangle - 0.15f;
    } else if (steeringangle < -0.15f) {
      steeringangle = steeringangle + 0.15f;
    } else {
      steeringangle = zero_float32_t;
    }
    egovehinfo_.SteeringAngle = steeringangle;
  } else {
    egovehinfo_.SteeringAngle = steeringangle;
  }
}

void EgoInfoHandler::update_indicator(
    const std::shared_ptr<vehicle_msgs::msg::ApBcm50msPdu02> &BCM_50ms_PDU02) {
  preindicator = egovehinfo_.Indicator;
  if ((BCM_50ms_PDU02->ildircnio == 1u) && (BCM_50ms_PDU02->irdircnio == 1u)) {
    if (preleftindicator == 1u) {
      egovehinfo_.Indicator = IndicatorStatus::Right;
      leftindicatortimer = zero_uint16_t;
      rightindicatortimer = zero_uint16_t;
    } else if (prerightindicator == 1u) {
      egovehinfo_.Indicator = IndicatorStatus::Left;
      leftindicatortimer = zero_uint16_t;
      rightindicatortimer = zero_uint16_t;
    }
  } else if (BCM_50ms_PDU02->ildircnio == 1u) {
    egovehinfo_.Indicator = IndicatorStatus::Left;
    leftindicatortimer = zero_uint16_t;
    rightindicatortimer = zero_uint16_t;
  } else if (BCM_50ms_PDU02->irdircnio == 1u) {
    egovehinfo_.Indicator = IndicatorStatus::Right;
    leftindicatortimer = zero_uint16_t;
    rightindicatortimer = zero_uint16_t;
  } else {
    if (egovehinfo_.Indicator == IndicatorStatus::Left) {
      leftindicatortimer =
          leftindicatortimer + one_uint16_t;
      if (leftindicatortimer > 16u) {
        egovehinfo_.Indicator = IndicatorStatus::None;
        leftindicatortimer = zero_uint16_t;
      }

    } else if (egovehinfo_.Indicator ==
               IndicatorStatus::Right) {
      rightindicatortimer =
          rightindicatortimer + one_uint16_t;
      if (rightindicatortimer > 16u) {
        egovehinfo_.Indicator = IndicatorStatus::None;
        rightindicatortimer = zero_uint16_t;
      }
    } else {
      leftindicatortimer = zero_uint16_t;
      rightindicatortimer = zero_uint16_t;
    }
  }

  preleftindicator = BCM_50ms_PDU02->ildircnio;
  prerightindicator = BCM_50ms_PDU02->irdircnio;
}

void EgoInfoHandler::update_standstillstatus(
    const std::shared_ptr<vehicle_msgs::msg::ApIbs20msPdu18> &IBS_20ms_PDU18) {
  basetype::float32_t res{};
  if (IBS_20ms_PDU18->ivehsdslsts == 1u) {
    res = zero_float32_t;
  } else if (IBS_20ms_PDU18->ivehsdslsts == 0u) {
    res = one_float32_t;
  } else {
    res = prestandstillfactor;
  }
  prestandstillfactor = res;
}

void EgoInfoHandler::update_vehspd(
    const std::shared_ptr<vehicle_msgs::msg::ApIbs20msPdu15> &IBS_20ms_PDU15) {
  if (IBS_20ms_PDU15->ivehspdavgv == 0u) {
    egovehinfo_.SpdLgt =
        Util::safetydiv(IBS_20ms_PDU15->ivehspdavg * 5.0f, 18.0f) *
        prestandstillfactor;
    egovehinfo_.SpdLat = zero_float32_t;
  } else {
  }
}

void EgoInfoHandler::update_yawrate(
    const std::shared_ptr<vehicle_msgs::msg::ApIbs20msPdu08> &IBS_20ms_PDU08) {
  basetype::float32_t yawrate{};
  if (IBS_20ms_PDU08->ivehdynyawratev == 0u) {
    yawrate =
        Util::safetydiv(IBS_20ms_PDU08->ivehdynyawrate * 3.1415926f, 180.0f);
  } else {
    yawrate = egovehinfo_.YawRate;
  }

  basetype::float32_t yawratelimited{};
  if ((egovehinfo_.SteeringAngle < eps6_float32_t) &&
      ((egovehinfo_.SteeringAngle > -eps6_float32_t))) {
    yawratelimited = zero_float32_t;
  } else {
    yawratelimited = yawrate;
  }

  if ((preindicator > IndicatorStatus::None) &&
      (egovehinfo_.Indicator == IndicatorStatus::None) &&
      (indicatortimer > zero_uint16_t)) {
    indicatortimer = 16u;
    indicatorwithoffdelay = preindicator;
  } else {
    if ((indicatortimer > zero_uint16_t) &&
        (egovehinfo_.Indicator == IndicatorStatus::None)) {
      indicatortimer = indicatortimer - one_uint16_t;
    } else {
      indicatorwithoffdelay = egovehinfo_.Indicator;
      indicatortimer = zero_uint16_t;
    }
  }

  if ((indicatortimer > zero_uint16_t) &&
      (indicatorwithoffdelay > IndicatorStatus::None)) {
    egovehinfo_.YawRate = egovehinfo_.YawRate -
        egovehinfo_.YawRate * 0.08f;
  } else {
    egovehinfo_.YawRate = yawratelimited * 0.08f + egovehinfo_.YawRate -
        egovehinfo_.YawRate * 0.08f;
  }
  egovehinfo_.YawRateRaw = egovehinfo_.YawRate;
}

void EgoInfoHandler::update_lgtaccel(
    const std::shared_ptr<vehicle_msgs::msg::ApIbs20msPdu08> &IBS_20ms_PDU08) {
  if (IBS_20ms_PDU08->ivselongtaccv == 0u) {
    egovehinfo_.AccelLgt = IBS_20ms_PDU08->ivselongtacc;
    egovehinfo_.AccelLgtRaw = egovehinfo_.AccelLgt;
  }
}

void EgoInfoHandler::update_lataccel(
    const std::shared_ptr<vehicle_msgs::msg::ApIbs20msPdu08> &IBS_20ms_PDU08) {
  egovehinfo_.AccelLatRaw = IBS_20ms_PDU08->ivselatacc;
}
} // namespace ACC_Target_Selection