#ifndef ACC_TARGET_SELECTION_EGOINFOHANDLER_H
#define ACC_TARGET_SELECTION_EGOINFOHANDLER_H

#include <array>
#include <memory>
#include <mutex>

#include "egovehicleinformation.h"
#include "vehicle_msgs/msg/ap_bcm50ms_pdu02.hpp"
#include "vehicle_msgs/msg/ap_ibs20ms_pdu08.hpp"
#include "vehicle_msgs/msg/ap_ibs20ms_pdu15.hpp"
#include "vehicle_msgs/msg/ap_ibs20ms_pdu18.hpp"
#include "vehicle_msgs/msg/ap_sas10ms_pdu00.hpp"

namespace ACC_Target_Selection {
class EgoInfoHandler {
public:
  explicit EgoInfoHandler();
  EgoInfoHandler(const EgoInfoHandler &r);
  ~EgoInfoHandler();

  EgoVehicleInformation egovehinfo_;

  uint16_t age_;

  void updateegoinfo(const int64_t &now_timestamp,
                     const std::shared_ptr<vehicle_msgs::msg::ApSas10msPdu00> &SAS_10ms_PDU00,
                     const std::shared_ptr<vehicle_msgs::msg::ApBcm50msPdu02> &BCM_50ms_PDU02,
                     const std::shared_ptr<vehicle_msgs::msg::ApIbs20msPdu18> &IBS_20ms_PDU18,
                     const std::shared_ptr<vehicle_msgs::msg::ApIbs20msPdu15> &IBS_20ms_PDU15,
                     const std::shared_ptr<vehicle_msgs::msg::ApIbs20msPdu08> &IBS_20ms_PDU08);

  void update_steeringwheelangle(const std::shared_ptr<vehicle_msgs::msg::ApSas10msPdu00> &SAS_10ms_PDU00);

  void update_indicator(const std::shared_ptr<vehicle_msgs::msg::ApBcm50msPdu02> &BCM_50ms_PDU02);

  void update_standstillstatus(const std::shared_ptr<vehicle_msgs::msg::ApIbs20msPdu18> &IBS_20ms_PDU18);

  void update_vehspd(const std::shared_ptr<vehicle_msgs::msg::ApIbs20msPdu15> &IBS_20ms_PDU15);

  void update_yawrate(const std::shared_ptr<vehicle_msgs::msg::ApIbs20msPdu08> &IBS_20ms_PDU08);

  void update_lgtaccel(const std::shared_ptr<vehicle_msgs::msg::ApIbs20msPdu08> &IBS_20ms_PDU08);

  void update_lataccel(const std::shared_ptr<vehicle_msgs::msg::ApIbs20msPdu08> &IBS_20ms_PDU08);
private:
  basetype::uint16_t leftindicatortimer;
  basetype::uint16_t rightindicatortimer;
  basetype::uint8_t preleftindicator;
  basetype::uint8_t prerightindicator;
  basetype::float32_t prestandstillfactor;
  IndicatorStatus preindicator;
  IndicatorStatus indicatorwithoffdelay;
  basetype::uint16_t indicatortimer;
};

} // namespace ACC_Target_Selection

#endif