#include "laneinfohandler.h"
#include "atsutil.h"
#include <cmath>
namespace ACC_Target_Selection{
    LaneInfoHandler::LaneInfoHandler() :
            laneinfo_(),
            age_(zero_uint16_t),
            validnum_ego_lane_(zero_uint8_t) {
    }

    LaneInfoHandler::LaneInfoHandler(const LaneInfoHandler& r) :
            laneinfo_(r.laneinfo_),
            age_(r.age_),
            validnum_ego_lane_(r.validnum_ego_lane_) {
    }

    LaneInfoHandler::~LaneInfoHandler() {
    }

    void LaneInfoHandler::updatelane(const int64_t &now_timestamp,
                                     const std::shared_ptr<cem_interfaces::msg::TrafficRoadStruct> &trafficroadrosmsg) {
        basetype::uint32_t leftline_id{zero_uint32_t};
        //basetype::float32_t leftline_existprob  = zero_float32_t;
        basetype::float32_t leftline_dy{zero_float32_t};
        basetype::uint32_t rightline_id{zero_uint32_t};
        //basetype::float32_t rightline_existprob = zero_float32_t;
        basetype::float32_t rightline_dy{zero_float32_t};

        basetype::uint8_t validnum{zero_uint8_t};
        // lane list existance
        basetype::boolean_t ego_lane_exsit{boolean_t_false};
        basetype::boolean_t isfound{boolean_t_false};
        basetype::float32_t ego_offset{zero_float32_t};

        for (size_t ii = 0; ii < trafficroadrosmsg->lanes_list.size(); ii++) {
            if (trafficroadrosmsg->lanes_list.at(ii).position == 1) {
                if (trafficroadrosmsg->lanes_list.at(ii).front_segment_lane_elements_list.size() > 0) {
                    ego_offset = trafficroadrosmsg->lanes_list.at(ii).front_segment_lane_elements_list.at(0).center_line_geometry.curve.c0;
                    if (validnum_ego_lane_ > 4) {
                        ego_lane_exsit = boolean_t_true;
                    } else {
                        validnum_ego_lane_++;
                    }
                    isfound = boolean_t_true;
                }
                break;
            }
        }

        if (isfound == boolean_t_false) {
            validnum_ego_lane_ = zero_uint8_t;
        } else {
        } 
        
        // left line
        if (trafficroadrosmsg->lane_markers_list.size() > static_cast<size_t>(0)) {
            if (trafficroadrosmsg->lane_markers_list[0].front_segment_lane_marker_elements_list.size() > static_cast<size_t>(0) &&
                (trafficroadrosmsg->lane_markers_list[0].front_segment_lane_marker_elements_list[0].geometry.is_curve_valid > uint8_t(0u)) &&
                (ego_lane_exsit == boolean_t_true) &&
                ((trafficroadrosmsg->lane_markers_list[0].front_segment_lane_marker_elements_list[0].geometry.measurement_state == 1) ||
                 (trafficroadrosmsg->lane_markers_list[0].front_segment_lane_marker_elements_list[0].geometry.measurement_state == 2) ||
                 (trafficroadrosmsg->lane_markers_list[0].front_segment_lane_marker_elements_list[0].geometry.measurement_state == 3))) {
                leftline_id           = one_uint32_t;
                //leftline_existprob    = one_float32_t;
                leftline_dy           = trafficroadrosmsg->lane_markers_list[0].front_segment_lane_marker_elements_list[0].geometry.curve.c0;
                validnum = validnum + one_uint8_t;
            }
        }

        // right line
        if (trafficroadrosmsg->lane_markers_list.size() > static_cast<size_t>(1)) {
            if (trafficroadrosmsg->lane_markers_list[1].front_segment_lane_marker_elements_list.size() > static_cast<size_t>(0) &&
                (trafficroadrosmsg->lane_markers_list[1].front_segment_lane_marker_elements_list[0].geometry.is_curve_valid > uint8_t(0u)) &&
                (ego_lane_exsit == boolean_t_true) &&
                ((trafficroadrosmsg->lane_markers_list[1].front_segment_lane_marker_elements_list[0].geometry.measurement_state == 1) ||
                 (trafficroadrosmsg->lane_markers_list[1].front_segment_lane_marker_elements_list[0].geometry.measurement_state == 2) ||
                 (trafficroadrosmsg->lane_markers_list[1].front_segment_lane_marker_elements_list[0].geometry.measurement_state == 3))) {
                rightline_id        = one_uint32_t;
                //rightline_existprob = one_float32_t;
                rightline_dy        = trafficroadrosmsg->lane_markers_list[1].front_segment_lane_marker_elements_list[0].geometry.curve.c0;
                validnum = validnum + one_uint8_t;
            }
        }

        
        if (validnum > one_uint8_t) {
            if ((ego_offset <= leftline_dy) &&
                (ego_offset >= rightline_dy)) {
                laneinfo_.Valid= boolean_t_true;
            } else {
                laneinfo_.Valid= boolean_t_false;
                validnum = zero_uint8_t;
                //validnum_ego_lane_ = zero_uint8_t;
                leftline_dy = zero_float32_t;
                rightline_dy = zero_float32_t;
            }
        } else if ((validnum > zero_uint8_t) &&
                 (laneinfo_.Valid == boolean_t_true)) {
            laneinfo_.Valid= boolean_t_true;
        } else {
             laneinfo_.Valid= boolean_t_false;
        }

        switch (validnum) {
        case (static_cast<basetype::uint8_t>(0u)):
            laneinfo_.LaneWidth = static_cast<basetype::float32_t>(3.6f);
            laneinfo_.LateralOffset = zero_float32_t;
            break;
        case (static_cast<basetype::uint8_t>(1u)):
            if (laneinfo_.Valid == boolean_t_false) {
                laneinfo_.LaneWidth = static_cast<basetype::float32_t>(3.6f);
                laneinfo_.LateralOffset = zero_float32_t;
            } else if (leftline_id > zero_uint32_t) {
                laneinfo_.LateralOffset = laneinfo_.LaneWidth * 0.5f - leftline_dy;
                laneinfo_.LateralOffset = Util::min(Util::max(laneinfo_.LateralOffset, -2.2f),
                                                    2.2f);
            } else if (rightline_id > zero_uint32_t) {
                laneinfo_.LateralOffset = -laneinfo_.LaneWidth * 0.5f - rightline_dy;
                laneinfo_.LateralOffset = Util::min(Util::max(laneinfo_.LateralOffset, -2.2f),
                                                    2.2f);
            } else {
                laneinfo_.LaneWidth = static_cast<basetype::float32_t>(3.6f);
                laneinfo_.LateralOffset = zero_float32_t;
            }
            break;
        case (static_cast<basetype::uint8_t>(2u)):
            if ((leftline_dy - rightline_dy) > static_cast<basetype::float32_t>(4.2f)) {
                 laneinfo_.LaneWidth = static_cast<basetype::float32_t>(4.2f);
            } else if ((leftline_dy - rightline_dy) < static_cast<basetype::float32_t>(2.8f)) {
                laneinfo_.LaneWidth = static_cast<basetype::float32_t>(2.8f);
            } else {
                laneinfo_.LaneWidth = (leftline_dy - rightline_dy);
            }
            laneinfo_.LateralOffset = Util::min(Util::max(laneinfo_.LaneWidth * 0.5f - leftline_dy, -2.2f),
                                                2.2f);
            break;
        default:
            laneinfo_.LaneWidth = static_cast<basetype::float32_t>(3.6f);
            laneinfo_.LateralOffset = zero_float32_t;
            break;
        } 
        
    }
} // namespace ACC_Target_Selection