#include "objectinfohandler.h"
#include "atsutil.h"

#include <bitset>
#include <iostream>

namespace ACC_Target_Selection {
ObjectInfoHandler::ObjectInfoHandler():
        age_(zero_uint16_t),
        status_(ObjectsStatus::Normal),
        objectinfo_() {
    basetype::objectid_t obj_idn{zero_objectid_t};
    for (auto &obj : objectinfo_) {
        obj_idn++;
        obj = Object_Default;
        obj.Ppty.Id = obj_idn;
    }
    
}

ObjectInfoHandler::ObjectInfoHandler(const ObjectInfoHandler &r):
        age_(r.age_),
        status_(r.status_),
        objectinfo_(r.objectinfo_) {
}

ObjectInfoHandler::~ObjectInfoHandler() {}

void ObjectInfoHandler::initobjectgroups() {
  basetype::objectid_t obj_idn{zero_objectid_t};
    for (auto &obj : objectinfo_) {
        obj_idn++;
        obj = Object_Default;
        obj.Ppty.Id = obj_idn;
    }
  //ObjectInfoHandler::objectinfo_[1].Est.DisLgt = -30.0f;
  //ObjectInfoHandler::objectinfo_[1].Est.DisLat = 3.0f;
  //ObjectInfoHandler::objectinfo_[1].Est.Spd = 10.0f;
  //ObjectInfoHandler::objectinfo_[1].Est.SpdLgt = 10.0f;
  //ObjectInfoHandler::objectinfo_[1].Info.DisLatFromLaneCenter = 0.0;
  //ObjectInfoHandler::objectinfo_[1].Info.MotionHistoryType = ObjectMotionHistoryType::Receeding;
  //ObjectInfoHandler::objectinfo_[1].Info.MotionType = ObjectMotionType::Receeding;
  //ObjectInfoHandler::objectinfo_[1].Info.Type = ObjectType::Car;
  //ObjectInfoHandler::objectinfo_[1].Info.Width = 0.0f;
  //ObjectInfoHandler::objectinfo_[1].Ppty.FusionSource = ObjectFusionSource::Fused;
  //ObjectInfoHandler::objectinfo_[1].Ppty.TrackStatus = ObjectTrackStatus::Update;
  //ObjectInfoHandler::objectinfo_[1].Ppty.DisLatFromLaneCenterQf = ObjectDisLatFromLaneCenterQf::UnReliable;
}

basetype::boolean_t ObjectInfoHandler::checkstatus(const int64_t &now_timestamp,
                                                   const std::shared_ptr<cem_interfaces::msg::TrafficParticipantStruct> &trafficparticipantrosmg) {
    // std::cout <<(trafficparticipantrosmg->cem_header.time_stamp)<<std::endl;
    // std::cout <<(trafficparticipantrosmg->cem_header.time_stamp + 40 * 1000000 * 2  +  1)<<std::endl;
    // std::cout <<now_timestamp<<std::endl;
    // std::cout <<trafficparticipantrosmg->cem_header.time_stamp - 40 * 1000000 * 2  -  1<<std::endl;
    
    basetype::boolean_t IsOk{boolean_t_true};
    uint64_t frame{0};
    if (trafficparticipantrosmg->object_list.size() > 10) {
        frame = 4;
    } else {
        frame = 2;
    }

    if ((now_timestamp < (trafficparticipantrosmg->cem_header.time_stamp + 40 * 1000000 * frame  +  1)) &&
        (now_timestamp > (trafficparticipantrosmg->cem_header.time_stamp - 40 * 1000000 * frame  -  1))) {
        if (age_ >= 1 &&
            (status_ == ObjectsStatus::Lost || status_ == ObjectsStatus::Recovring)) {
            age_ --;
            status_ = ObjectsStatus::Recovring;
            // std::cout<< "traffic participant is recovering"<<std::endl;
            ObjectInfoHandler::initobjectgroups();
            IsOk = boolean_t_false;
            return IsOk;
        } else {
            if (age_ >= 1) {
                age_ --;
                status_ = ObjectsStatus::QuickRecovring;
                // std::cout<< "traffic participant is quickly recovering"<<std::endl;
            } else {
                status_ = ObjectsStatus::Normal;
                // std::cout<< "traffic participant is normal"<<std::endl;
            }
        }
        ObjectInfoHandler::initobjectgroups();
    } else {
        ObjectInfoHandler::initobjectgroups();
        if (age_ < 5) {
            age_ ++;
            status_ = ObjectsStatus::Losing;
            // std::cout<< "traffic participant is losing"<<std::endl;
        } else {
            status_ = ObjectsStatus::Lost;
            //std::cout<< "traffic participant is lost"<<std::endl;
            IsOk = boolean_t_false;
            ObjectInfoHandler::initobjectgroups();
            return IsOk;
        }
    }

    if (trafficparticipantrosmg->cem_header.status != 0) {
        std::bitset<64> temp(trafficparticipantrosmg->cem_header.status);
        if (temp[0]) {
            // std::cout<< "cem_header status error"<<std::endl;
            ObjectInfoHandler::initobjectgroups();
            return IsOk;
        }
    }
    return IsOk;
}

void ObjectInfoHandler::updateobjectgroups(const int64_t &now_timestamp,
        const std::shared_ptr<cem_interfaces::msg::TrafficParticipantStruct> &trafficparticipantrosmg) {
    basetype::boolean_t IsOk{boolean_t_false};
    IsOk = ObjectInfoHandler::checkstatus(now_timestamp, trafficparticipantrosmg);
    
    if (boolean_t_true != IsOk &&
        boolean_t_true == usecomcheck) {
    } else {
        for (auto& obj : trafficparticipantrosmg->object_list) {
            if ((obj.object_id > (250u)) ||
                (obj.object_id == zero_objectid_t)) {
                continue;
            }
            basetype::objectid_t index0{obj.object_id - one_objectid_t};
            objectinfo_.at(index0).Est.Accel = obj.absolute_acceleration_x;
            objectinfo_.at(index0).Est.AccelLat = obj.absolute_acceleration_y;
            objectinfo_.at(index0).Est.Curvature = zero_float32_t;
            ObjectInfoHandler::generate_position(obj.reference_point_type,
                                                 obj.relative_position_x,
                                                 obj.relative_position_y,
                                                 obj.size_length,
                                                 obj.size_width,
                                                 index0);
            objectinfo_.at(index0).Est.HeadingAngle = obj.relative_orientation;
            objectinfo_.at(index0).Est.Spd = obj.absolute_velocity_x;
            objectinfo_.at(index0).Est.SpdLat = obj.absolute_velocity_y;
            objectinfo_.at(index0).Est.SpdLgt = obj.absolute_velocity_x;
            objectinfo_.at(index0).Est.SpdLgt = obj.absolute_velocity_x;
            objectinfo_.at(index0).Info.DisLatFromLaneCenter = obj.offset_to_host_lane;
            // ObjectInfoHandler::objectinfo_[index0].Info.Height = zero_float32_t;
            // ObjectInfoHandler::objectinfo_[index0].Info.MotionHistoryType =
            // ObjectMotionHistoryType::Unknown;
            // ObjectInfoHandler::objectinfo_[index0].Info.MotionType =
            // ObjectMotionType::Unknown; ObjectInfoHandler::objectinfo_[index0].Info.Type
            // = zero_float32_t;
            ObjectInfoHandler::generate_type(obj.classification,
                                             obj.absolute_velocity_x,
                                             obj.size_width,
                                             obj.size_height,
                                             obj.size_length,
                                             index0);
            ObjectInfoHandler::generate_motiontype(obj.moving_direction, obj.motion_property, obj.absolute_velocity_x, index0);
            // ObjectInfoHandler::objectinfo_[index0].Info.Width = zero_float32_t;
            objectinfo_.at(index0).Ppty.AccelStdDev = obj.absolute_acceleration_x;
            objectinfo_.at(index0).Ppty.CameraId = zero_uint32_t;
            // ObjectInfoHandler::objectinfo_[index0].Ppty.DisLatFromLaneCenterQf =
            // ObjectDisLatFromLaneCenterQf::UnReliable;
            ObjectInfoHandler::generate_DstLatFromCentreLineQly(obj.offset_to_host_lane_flag, index0, obj.relative_position_y);
            objectinfo_.at(index0).Ppty.DisLatStdDev = obj.std_of_relative_position_y;
            objectinfo_.at(index0).Ppty.DisLgtStdDev = obj.std_of_relative_position_x;
            // ObjectInfoHandler::objectinfo[index]_.Ppty.FusionSource =
            // ObjectFusionSource::None;
            ObjectInfoHandler::generate_fusionsource(obj.contributing_sensors, index0);
            objectinfo_.at(index0).Ppty.HeadingAngleStdDev = obj.std_of_relative_orientation;
            objectinfo_.at(index0).Ppty.Id = index0 + one_objectid_t;
            objectinfo_.at(index0).Ppty.RadarId = zero_uint32_t;
            objectinfo_.at(index0).Ppty.SpdStdDev = obj.std_of_absolute_velocity_x;
            // ObjectInfoHandler::objectinfo_[index0].Ppty.TrackStatus =
            // ObjectTrackStatus::Invalid;
            ObjectInfoHandler::generate_TrackStatus(obj.measurement_state, index0);
              
            if (objectinfo_.at(index0).Info.Type == ObjectType::Pedestrain) {
                objectinfo_.at(index0).Est.HeadingAngle = zero_float32_t;
                objectinfo_.at(index0).Est.SpdLat = zero_float32_t;
                objectinfo_.at(index0).Info.MotionHistoryType = ObjectMotionHistoryType::Stationary;
                objectinfo_.at(index0).Info.MotionType = ObjectMotionType::Stationary;
            }
        }
    }
}

void ObjectInfoHandler::updateobjectgroups(const int64_t &now_timestamp,
    const mpc_drv_msgs::msg::RoadInfo &roadinfo) {
  if (roadinfo.lane_list.size() > 0) {
    if (roadinfo.lane_list[0].corridor_list.size() > 0) {
      if (roadinfo.lane_list[0].corridor_list[0].object_in_corridor_list.size() > 0) {
        for (size_t i = 0; i < (roadinfo.lane_list[0].corridor_list[0].object_in_corridor_list.size()); i++) {
          uint32_t idn0{roadinfo.lane_list[0].corridor_list[0].object_in_corridor_list[i].object_id};
          if ((idn0 <= static_cast<uint32_t>(c_ObjectIndexMax)) &&
              (idn0 >= static_cast<uint32_t>(c_ObjectIndexMin))) {
            objectinfo_.at(idn0).Ppty.DisLatFromLaneCenterQf = ObjectDisLatFromLaneCenterQf::HigherReliable;
            objectinfo_.at(idn0).Info.DisLatFromLaneCenter =
                roadinfo.lane_list[0].corridor_list[0].object_in_corridor_list[i].reference_point2corridor_frenet.l;
          }
        }
      }

      if (roadinfo.lane_list[0].corridor_list[0].relevant_object_corridor_list.size() > 0) {
        for (size_t i = 0; i < (roadinfo.lane_list[0].corridor_list[0].relevant_object_corridor_list.size()); i++) {
          uint32_t idn0{roadinfo.lane_list[0].corridor_list[0].relevant_object_corridor_list[i].object_id};
          if ((idn0 <= static_cast<uint32_t>(c_ObjectIndexMax)) &&
              (idn0 >= static_cast<uint32_t>(c_ObjectIndexMin))) {
            objectinfo_.at(idn0).Ppty.DisLatFromLaneCenterQf = ObjectDisLatFromLaneCenterQf::HigherReliable;
            objectinfo_.at(idn0).Info.DisLatFromLaneCenter = roadinfo.lane_list[0].corridor_list[0].relevant_object_corridor_list[i].reference_point2corridor_frenet.l;
          }
        }
      }
    }
  }
}

void ObjectInfoHandler::generate_DstLatFromCentreLineQly(
    const basetype::uint8_t &OffsetToHostLaneFlag, const basetype::objectid_t &idn0, const basetype::float32_t &latdis) {
  if (OffsetToHostLaneFlag == one_uint8_t) {
    if (Util::abs(latdis) < 9.0f) {
      objectinfo_.at(idn0).Ppty.DisLatFromLaneCenterQf = ObjectDisLatFromLaneCenterQf::HigherReliable;
    } else {
      objectinfo_.at(idn0).Ppty.DisLatFromLaneCenterQf = ObjectDisLatFromLaneCenterQf::UnReliable;
    }
  } else {
    objectinfo_.at(idn0).Ppty.DisLatFromLaneCenterQf = ObjectDisLatFromLaneCenterQf::UnReliable;
  }
}

void ObjectInfoHandler::generate_fusionsource(
    const basetype::uint32_t &ContributingSensors, const basetype::objectid_t &idn0) {
  std::bitset<32> contributingsensors(ContributingSensors);
  if (contributingsensors[0] && contributingsensors[1]) {
    objectinfo_.at(idn0).Ppty.FusionSource = ObjectFusionSource::Fused;
  } else if (ContributingSensors == 1) {
    objectinfo_.at(idn0).Ppty.FusionSource = ObjectFusionSource::CameraOnly;
  } else if (contributingsensors[1]) {
    objectinfo_.at(idn0).Ppty.FusionSource = ObjectFusionSource::RadarOnly;
  } else if (contributingsensors[0]) {
    objectinfo_.at(idn0).Ppty.FusionSource = ObjectFusionSource::Fused;
  } else {
    objectinfo_.at(idn0).Ppty.FusionSource = ObjectFusionSource::None;
  }
}

void ObjectInfoHandler::generate_TrackStatus(const basetype::uint8_t &MeasurementState,
                                             const basetype::objectid_t &idn0) {
  switch (MeasurementState) {
  case 0u:
    objectinfo_.at(idn0).Ppty.TrackStatus = ObjectTrackStatus::Invalid;
    break;
  case 1u:
    objectinfo_.at(idn0).Ppty.TrackStatus = ObjectTrackStatus::New;
    break;
  case 2u:
    objectinfo_.at(idn0).Ppty.TrackStatus = ObjectTrackStatus::Update;
    break;
  case 3u:
    objectinfo_.at(idn0).Ppty.TrackStatus = ObjectTrackStatus::Predict;
    break;
  case 4u:
    objectinfo_.at(idn0).Ppty.TrackStatus = ObjectTrackStatus::Invalid;
    break;
  default:
    break;
  }
}

void ObjectInfoHandler::generate_type(const basetype::uint8_t &Classification,
                                      const basetype::float32_t &vx,
                                      const basetype::float32_t &width,
                                      const basetype::float32_t &height,
                                      const basetype::float32_t &length,
                                      const basetype::objectid_t &idn0) {
    switch (Classification) {
    case 0u:
        objectinfo_.at(idn0).Info.Type_raw = 0u;
        if (vx > 1.7f) {
          objectinfo_.at(idn0).Info.Type = ObjectType::UnknownVehicleClass;
          objectinfo_.at(idn0).Info.Width = width;
          objectinfo_.at(idn0).Info.Height = height;
          objectinfo_.at(idn0).Info.Length = length;
        } else {
          objectinfo_.at(idn0).Info.Type = ObjectType::Unknown;
          objectinfo_.at(idn0).Info.Width = width;
          objectinfo_.at(idn0).Info.Height = height;
          objectinfo_.at(idn0).Info.Length = length;
        }
        break;
    case 1u:
    case 16u:
    case 17u:
    case 18u:
    case 19u:
    case 20u:
        objectinfo_.at(idn0).Info.Type_raw = 1u;
        objectinfo_.at(idn0).Info.Type = ObjectType::Car;
        objectinfo_.at(idn0).Info.Width = width;
        objectinfo_.at(idn0).Info.Height = height;
        objectinfo_.at(idn0).Info.Length = length;
        break;
    case 2u:
    case 3u:
    case 6u:
    case 12u:
        objectinfo_.at(idn0).Info.Type_raw = 2u;
        objectinfo_.at(idn0).Info.Type = ObjectType::Truck;
        objectinfo_.at(idn0).Info.Width = width;
        objectinfo_.at(idn0).Info.Height = height;
        objectinfo_.at(idn0).Info.Length = length;
        break;
    case 4u:
    case 8u:
    case 11u:
        objectinfo_.at(idn0).Info.Type_raw = 4u;
        objectinfo_.at(idn0).Info.Type = ObjectType::Bicycle;
        objectinfo_.at(idn0).Info.Width = width;
        objectinfo_.at(idn0).Info.Height = height;
        objectinfo_.at(idn0).Info.Length = length;
        break;
    case 5u:
        objectinfo_.at(idn0).Info.Type_raw = 5u;
        objectinfo_.at(idn0).Info.Type = ObjectType::Pedestrain;
        objectinfo_.at(idn0).Info.Width = width;
        objectinfo_.at(idn0).Info.Height = height;
        objectinfo_.at(idn0).Info.Length = length;
        break;
    case 10u:
        objectinfo_.at(idn0).Info.Type_raw = 0u;
        objectinfo_.at(idn0).Info.Type = ObjectType::UnknownVehicleClass;
        objectinfo_.at(idn0).Info.Width = width;
        objectinfo_.at(idn0).Info.Height = height;
        objectinfo_.at(idn0).Info.Length = length;
        break;
    case 21u:
    case 22u:
        objectinfo_.at(idn0).Info.Type_raw = 4u;
        objectinfo_.at(idn0).Info.Type = ObjectType::Motorcycle;
        objectinfo_.at(idn0).Info.Width = width;
        objectinfo_.at(idn0).Info.Height = height;
        objectinfo_.at(idn0).Info.Length = length;
        break;
    default:
        objectinfo_.at(idn0).Info.Type_raw = 0u;
        objectinfo_.at(idn0).Info.Type = ObjectType::Unknown;
        objectinfo_.at(idn0).Info.Width = 0.4f;
        objectinfo_.at(idn0).Info.Height = 0.0f;
        objectinfo_.at(idn0).Info.Length = 0.0f;
        break;
    }
}

void ObjectInfoHandler::generate_motiontype(const basetype::uint8_t &MovingDirection,
                                            const basetype::uint8_t &MotionProperty,
                                            const basetype::float32_t &vx,
                                            const basetype::objectid_t &idn0) {
  switch (MotionProperty) {
  case 0u: // MtnProp0_Unknown
    objectinfo_.at(idn0).Info.MotionType = ObjectMotionType::Unknown;
    objectinfo_.at(idn0).Info.MotionHistoryType = ObjectMotionHistoryType::Unknown;
    break;
  case 1u: // MtnProp0_Stationary
    objectinfo_.at(idn0).Info.MotionType = ObjectMotionType::Stationary;
    if ((objectinfo_.at(idn0).Est.HeadingAngle < 1.396f) &&
        (objectinfo_.at(idn0).Est.HeadingAngle > -1.396f)) {
          objectinfo_.at(idn0).Info.MotionHistoryType = ObjectMotionHistoryType::Stationary;
    } else if ((objectinfo_.at(idn0).Est.HeadingAngle > 1.744f) ||
               (objectinfo_.at(idn0).Est.HeadingAngle < -1.744f)) {
          objectinfo_.at(idn0).Info.MotionHistoryType = ObjectMotionHistoryType::Oncoming;
    } else {
          objectinfo_.at(idn0).Info.MotionHistoryType = ObjectMotionHistoryType::Crossing;
    }
    break;
  case 2u: // MtnProp0_Moving
  case 4u: // MtnProp0_MovingSlowly
    if (MovingDirection == 1u) {
        objectinfo_.at(idn0).Info.MotionType = ObjectMotionType::Receeding;
        objectinfo_.at(idn0).Info.MotionHistoryType = ObjectMotionHistoryType::Receeding;
    } else if (MovingDirection == 2u) {
        objectinfo_.at(idn0).Info.MotionType = ObjectMotionType::Oncoming;
        objectinfo_.at(idn0).Info.MotionHistoryType = ObjectMotionHistoryType::Oncoming;
    } else if (MovingDirection == 3u) {
        objectinfo_.at(idn0).Info.MotionType = ObjectMotionType::Crossing;
        objectinfo_.at(idn0).Info.MotionHistoryType = ObjectMotionHistoryType::Crossing;
    } else {
        if (Util::abs(vx) < 1.3f) {
            objectinfo_.at(idn0).Info.MotionType = ObjectMotionType::Stationary;
            if ((objectinfo_.at(idn0).Est.HeadingAngle < 1.396f) &&
                (objectinfo_.at(idn0).Est.HeadingAngle > -1.396f)) {
              objectinfo_.at(idn0).Info.MotionHistoryType = ObjectMotionHistoryType::Stationary;
            } else if ((objectinfo_.at(idn0).Est.HeadingAngle > 1.744f) ||
                       (objectinfo_.at(idn0).Est.HeadingAngle < -1.744f)) {
                        objectinfo_.at(idn0).Info.MotionHistoryType = ObjectMotionHistoryType::Oncoming;
            } else {
                objectinfo_.at(idn0).Info.MotionHistoryType = ObjectMotionHistoryType::Crossing;
            }
        } else if (vx >= 1.3f) {
            objectinfo_.at(idn0).Info.MotionType = ObjectMotionType::Receeding;
            objectinfo_.at(idn0).Info.MotionHistoryType = ObjectMotionHistoryType::Receeding;
        } else {
            if (vx < -1.7f) {
                objectinfo_.at(idn0).Info.MotionType = ObjectMotionType::Oncoming;
                objectinfo_.at(idn0).Info.MotionHistoryType = ObjectMotionHistoryType::Oncoming;
            } else {
                objectinfo_.at(idn0).Info.MotionType = ObjectMotionType::Unknown;
                objectinfo_.at(idn0).Info.MotionHistoryType = ObjectMotionHistoryType::Unknown;
            }
        }
    }
    break;
  case 3u: // MtnProp0_Stopped
    if ((objectinfo_.at(idn0).Est.SpdLat < 1.2f) &&
        (objectinfo_.at(idn0).Est.SpdLat > -1.2f)) {
        objectinfo_.at(idn0).Info.MotionType = ObjectMotionType::Stationary;
        switch (MovingDirection) {
        case 0u:// MtnDir0_Unknown
            if ((objectinfo_.at(idn0).Est.HeadingAngle < 1.396f) &&
                (objectinfo_.at(idn0).Est.HeadingAngle > -1.396f)) {
                objectinfo_.at(idn0).Info.MotionHistoryType = ObjectMotionHistoryType::Stationary;
            } else if ((objectinfo_.at(idn0).Est.HeadingAngle > 1.744f) ||
                       (objectinfo_.at(idn0).Est.HeadingAngle < -1.744f)) {
                objectinfo_.at(idn0).Info.MotionHistoryType = ObjectMotionHistoryType::Oncoming;
            } else {
                objectinfo_.at(idn0).Info.MotionHistoryType = ObjectMotionHistoryType::Crossing;
            }
            break;
        case 1u: // MtnDir0_Receding
            objectinfo_.at(idn0).Info.MotionHistoryType = ObjectMotionHistoryType::Receeding;
            break;
        case 2u: // MtnDir0_Oncoming
            objectinfo_.at(idn0).Info.MotionHistoryType = ObjectMotionHistoryType::Oncoming;
            break;
        case 3u: // MtnDir0_Crossing
            objectinfo_.at(idn0).Info.MotionHistoryType = ObjectMotionHistoryType::Crossing;
            break;
        default: // MtnDir0_Unknown
            objectinfo_.at(idn0).Info.MotionHistoryType = ObjectMotionHistoryType::Receeding;
            break;
        }
    } else {
      objectinfo_.at(idn0).Info.MotionType = ObjectMotionType::Crossing;
      objectinfo_.at(idn0).Info.MotionHistoryType = ObjectMotionHistoryType::Crossing;
    }
    break;
  default: // MtnProp0_Unknown
    objectinfo_.at(idn0).Info.MotionType = ObjectMotionType::Unknown;
    objectinfo_.at(idn0).Info.MotionHistoryType = ObjectMotionHistoryType::Unknown;
    break;
  }
}

void ObjectInfoHandler::generate_position(const basetype::uint8_t &Referencepoint,
                                          const basetype::float32_t &lgtdis,
                                          const basetype::float32_t &latdis,
                                          const basetype::float32_t &length,
                                          const basetype::float32_t &width,
                                          const basetype::objectid_t &idn0) {
    switch (Referencepoint) {
    case 0: // front left;
        objectinfo_.at(idn0).Est.DisLgt = lgtdis - 0.95f * length;
        objectinfo_.at(idn0).Est.DisLat = latdis - 0.95f * width * 0.5f; 
        break;
    case 1: // front center;
        objectinfo_.at(idn0).Est.DisLgt = lgtdis - 0.95f * length;
        objectinfo_.at(idn0).Est.DisLat = latdis; 
        break;
    case 2: // front right;
        objectinfo_.at(idn0).Est.DisLgt = lgtdis - 0.95f * length;
        objectinfo_.at(idn0).Est.DisLat = latdis + 0.95f * width * 0.5f; 
        break;
    case 3: // right center;
        objectinfo_.at(idn0).Est.DisLgt = lgtdis - 0.95f * length * 0.5f;
        objectinfo_.at(idn0).Est.DisLat = latdis + 0.95f * width * 0.5f;
        break;
    case 7: // left center;
        objectinfo_.at(idn0).Est.DisLgt = lgtdis - 0.95f * length * 0.5f;
        objectinfo_.at(idn0).Est.DisLat = latdis - 0.95f * width * 0.5f; 
        break;
    case 8: // center;
        objectinfo_.at(idn0).Est.DisLgt = lgtdis - 0.95f * length * 0.5f;
        objectinfo_.at(idn0).Est.DisLat = latdis; 
        break;
    case 4: // rear right;
        objectinfo_.at(idn0).Est.DisLgt = lgtdis;
        objectinfo_.at(idn0).Est.DisLat = latdis + 0.95f * width * 0.5f; 
        break;
    case 5: // rear center;
        objectinfo_.at(idn0).Est.DisLgt = lgtdis;
        objectinfo_.at(idn0).Est.DisLat = latdis; 
        break;
    case 6: // rear left;
        objectinfo_.at(idn0).Est.DisLgt = lgtdis;
        objectinfo_.at(idn0).Est.DisLat = latdis - 0.95f * width * 0.5f; 
        break;
    default:
        objectinfo_.at(idn0).Est.DisLgt = lgtdis;
        objectinfo_.at(idn0).Est.DisLat = latdis; 
        break;
    }
}

} // namespace ACC_Target_Selection