#ifndef ACC_TARGET_SELECTION_OBJECTINFOHANDLER_H
#define ACC_TARGET_SELECTION_OBJECTINFOHANDLER_H

#include <array>
#include <memory>
#include <mutex>

#include "cem_interfaces/msg/traffic_participant_struct.hpp"
#include "mpc_drv_msgs/msg/road_info.hpp"
#include "object.h"
#include "param.h"

namespace ACC_Target_Selection {
class ObjectInfoHandler {
enum class ObjectsStatus : basetype::int16_t{
    Error,
    Lost,
    Losing,
    Recovring,
    QuickRecovring,
    Normal
};
public:
    ObjectInfoHandler();

    ObjectInfoHandler(const ObjectInfoHandler &r);

    ~ObjectInfoHandler();

    std::array<Object, c_ObjectSize> objectinfo_;

    uint16_t age_;

    ObjectsStatus status_;
  
    void initobjectgroups();

    void updateobjectgroups(const int64_t &now_timestamp,
                            const std::shared_ptr<cem_interfaces::msg::TrafficParticipantStruct> &trafficparticipantrosmg);

    void updateobjectgroups(const int64_t &now_timestamp,
                            const mpc_drv_msgs::msg::RoadInfo &roadinfo);

    void generate_DstLatFromCentreLineQly(const basetype::uint8_t &OffsetToHostLaneFlag,
                                          const basetype::objectid_t &idn0,
                                          const basetype::float32_t &latdis);

    void generate_fusionsource(const basetype::uint32_t &ContributingSensors,
                               const basetype::objectid_t &idn0);

    void generate_TrackStatus(const basetype::uint8_t &MeasurementState,
                              const basetype::objectid_t &idn0);

    void generate_type(const basetype::uint8_t &Classification,
                       const basetype::float32_t &vx,
                       const basetype::float32_t &width,
                       const basetype::float32_t &height,
                       const basetype::float32_t &length,
                       const basetype::objectid_t &idn0);

    void generate_motiontype(const basetype::uint8_t &MovingDirection,
                             const basetype::uint8_t &MotionProperty,
                             const basetype::float32_t &vx,
                             const basetype::objectid_t &idn0);
    void generate_position(const basetype::uint8_t &Referencepoint,
                           const basetype::float32_t &lgtdis,
                           const basetype::float32_t &latdis,
                           const basetype::float32_t &length,
                           const basetype::float32_t &width,
                           const basetype::objectid_t &idn0);
    basetype::boolean_t checkstatus(const int64_t &now_timestamp,
                                    const std::shared_ptr<cem_interfaces::msg::TrafficParticipantStruct> &trafficparticipantrosmg);
};

} // namespace ACC_Target_Selection

#endif