#include "objectresulthandler.h"
#include "atsutil.h"
#include <cmath>

namespace ACC_Target_Selection {

ObjectResultHandler::ObjectResultHandler():
        dis2frontbump_(3.752f),
        dis2rearbump_(1.5f),
        logger_(nullptr),
        default_iv_(),
        default_riv_() {
    default_iv_.is_tgtnew = false;
    default_iv_.d_tgtlongtdist = 250.0f;
    default_iv_.d_tgtlatdist = 0.0f;
    default_iv_.v_tgtlongtspd = 0.0f;
    default_iv_.v_tgtlatspd = 0.0f;
    default_iv_.v_tgtrltspd = 0.0f;
    default_iv_.is_tgtprsnt = false;
    default_iv_.t_tgtdurtm = 0.0f;
    default_iv_.indx_tgtid = 0u;
    default_iv_.a_tgtlongtaccel = 0.0f;
    default_iv_.a_tgtlataccel = 0.0f;
    default_iv_.indx_tgttyp = 0u;
    default_iv_.is_tgtlnechgflag = false;

    default_riv_.is_tgtnew = false;
    default_riv_.d_tgtlongtdist = -250.0f;
    default_riv_.d_tgtlatdist = 0.0f;
    default_riv_.v_tgtlongtspd = 0.0f;
    default_riv_.v_tgtlatspd = 0.0f;
    default_riv_.v_tgtrltspd = 0.0f;
    default_riv_.is_tgtprsnt = false;
    default_riv_.t_tgtdurtm = 0.0f;
    default_riv_.indx_tgtid = 0u;
    default_riv_.a_tgtlongtaccel = 0.0f;
    default_riv_.a_tgtlataccel = 0.0f;
    default_riv_.indx_tgttyp = 0u;
    default_riv_.is_tgtlnechgflag = false;
}

ObjectResultHandler::ObjectResultHandler(const ObjectResultHandler &r):
        dis2frontbump_(r.dis2frontbump_),
        dis2rearbump_(r.dis2rearbump_),
        logger_(r.logger_),
        default_iv_(),
        default_riv_() {
    default_iv_.is_tgtnew = false;
    default_iv_.d_tgtlongtdist = 250.0f;
    default_iv_.d_tgtlatdist = 0.0f;
    default_iv_.v_tgtlongtspd = 0.0f;
    default_iv_.v_tgtlatspd = 0.0f;
    default_iv_.v_tgtrltspd = 0.0f;
    default_iv_.is_tgtprsnt = false;
    default_iv_.t_tgtdurtm = 0.0f;
    default_iv_.indx_tgtid = 0u;
    default_iv_.a_tgtlongtaccel = 0.0f;
    default_iv_.a_tgtlataccel = 0.0f;
    default_iv_.indx_tgttyp = 0u;
    default_iv_.is_tgtlnechgflag = false;

    default_riv_.is_tgtnew = false;
    default_riv_.d_tgtlongtdist = -250.0f;
    default_riv_.d_tgtlatdist = 0.0f;
    default_riv_.v_tgtlongtspd = 0.0f;
    default_riv_.v_tgtlatspd = 0.0f;
    default_riv_.v_tgtrltspd = 0.0f;
    default_riv_.is_tgtprsnt = false;
    default_riv_.t_tgtdurtm = 0.0f;
    default_riv_.indx_tgtid = 0u;
    default_riv_.a_tgtlongtaccel = 0.0f;
    default_riv_.a_tgtlataccel = 0.0f;
    default_riv_.indx_tgttyp = 0u;
    default_riv_.is_tgtlnechgflag = false;
}

ObjectResultHandler::~ObjectResultHandler() {}

void ObjectResultHandler::setlogger(Logger *logger) {
    ObjectResultHandler::logger_ = logger;
}

void ObjectResultHandler::assigniv(mpc_drv_msgs::msg::TargetsInfo &ats_targetsinfo) {
    basetype::objectid_t main_id{logger_->Getinfocreator().element.iv};
    basetype::float32_t spd{logger_->Getinfoprovider().egovehinfo.SpdLgt};
    if (main_id > zero_objectid_t) {
        ats_targetsinfo.iv.is_tgtnew =
            (main_id != static_cast<basetype::objectid_t>(ats_targetsinfo.iv.indx_tgtid)
                 ? true : false);
        ats_targetsinfo.iv.indx_tgtid = main_id;
        ats_targetsinfo.iv.d_tgtlongtdist =
            fmax((logger_->Getinfoprovider().objects.at(main_id - one_objectid_t).Est.DisLgt - dis2frontbump_),
                  0.0f);
        ats_targetsinfo.iv.d_tgtlatdist =
            logger_->Getinfoprovider().objects.at(main_id - one_objectid_t).Est.DisLat;
        ats_targetsinfo.iv.v_tgtlongtspd =
            logger_->Getinfoprovider().objects.at(main_id - one_objectid_t).Est.SpdLgt;
        ats_targetsinfo.iv.v_tgtlatspd =
            logger_->Getinfoprovider().objects.at(main_id - one_objectid_t).Est.SpdLat;

        ats_targetsinfo.iv.v_tgtrltspd = ats_targetsinfo.iv.v_tgtlongtspd - spd;
        ats_targetsinfo.iv.is_tgtprsnt = true;
        ats_targetsinfo.iv.t_tgtdurtm =
            (logger_->Getinfocreator().array.maturitycounter.at(main_id - one_objectid_t)) * 0.04f;
        ats_targetsinfo.iv.a_tgtlongtaccel = 
            logger_->Getinfoprovider().objects.at(main_id - one_objectid_t).Est.AccelLgt;
        ats_targetsinfo.iv.a_tgtlataccel =
            logger_->Getinfoprovider().objects.at(main_id - one_objectid_t).Est.AccelLat;
        ats_targetsinfo.iv.indx_tgttyp = 1u;
        ats_targetsinfo.iv.is_tgtlnechgflag = false;
    } else {
        ats_targetsinfo.iv = default_iv_;
    }
}

void ObjectResultHandler::assigniiv(mpc_drv_msgs::msg::TargetsInfo &ats_targetsinfo) {
    basetype::objectid_t main2_id{logger_->Getinfocreator().element.iiv};
    basetype::float32_t spd{logger_->Getinfoprovider().egovehinfo.SpdLgt};
    if (main2_id > zero_objectid_t) {
        ats_targetsinfo.iiv.is_tgtnew =
            (main2_id != static_cast<basetype::objectid_t>(ats_targetsinfo.iiv.indx_tgtid)
                 ? true : false);
        ats_targetsinfo.iiv.indx_tgtid = main2_id;
        ats_targetsinfo.iiv.d_tgtlongtdist =
            fmax((logger_->Getinfoprovider().objects.at(main2_id - one_objectid_t).Est.DisLgt - dis2frontbump_),
                 0.0f);
        ats_targetsinfo.iiv.d_tgtlatdist =
            logger_->Getinfoprovider().objects.at(main2_id - one_objectid_t).Est.DisLat;
        ats_targetsinfo.iiv.v_tgtlongtspd =
            logger_->Getinfoprovider().objects.at(main2_id - one_objectid_t).Est.SpdLgt;
        ats_targetsinfo.iiv.v_tgtlatspd =
            logger_->Getinfoprovider().objects.at(main2_id - one_objectid_t).Est.SpdLat;
        ats_targetsinfo.iiv.v_tgtrltspd = ats_targetsinfo.iiv.v_tgtlongtspd - spd;
        ats_targetsinfo.iiv.is_tgtprsnt = true;
        ats_targetsinfo.iiv.t_tgtdurtm =
            (logger_->Getinfocreator().array.maturitycounter.at(main2_id - one_objectid_t)) * 0.04f;
        ats_targetsinfo.iiv.a_tgtlongtaccel =
            logger_->Getinfoprovider().objects.at(main2_id - one_objectid_t).Est.AccelLgt;
        ats_targetsinfo.iiv.a_tgtlataccel =
            logger_->Getinfoprovider().objects.at(main2_id - one_objectid_t).Est.AccelLat;
        ats_targetsinfo.iiv.indx_tgttyp = 1u;
        ats_targetsinfo.iiv.is_tgtlnechgflag = false;
    } else {
        ats_targetsinfo.iiv = default_iv_;
    }
}

void ObjectResultHandler::assignflniv(mpc_drv_msgs::msg::TargetsInfo &ats_targetsinfo) {
    basetype::objectid_t left_id{logger_->Getinfocreator().element.fliv};
    basetype::objectid_t cutin_id{logger_->Getinfocreator().element.cutin};
    basetype::float32_t spd{logger_->Getinfoprovider().egovehinfo.SpdLgt};
    if (left_id > zero_objectid_t) {
        ats_targetsinfo.flniv.is_tgtnew =
            (left_id != static_cast<basetype::objectid_t>(ats_targetsinfo.flniv.indx_tgtid)
                 ? true : false);
        ats_targetsinfo.flniv.indx_tgtid = left_id;
        ats_targetsinfo.flniv.d_tgtlongtdist =
            fmax((logger_->Getinfoprovider().objects.at(left_id - one_objectid_t).Est.DisLgt - dis2frontbump_),
                 0.0f);
        ats_targetsinfo.flniv.d_tgtlatdist =
            logger_->Getinfoprovider().objects.at(left_id - one_objectid_t).Est.DisLat;
        ats_targetsinfo.flniv.v_tgtlongtspd =
            logger_->Getinfoprovider().objects.at(left_id - one_objectid_t).Est.SpdLgt;
        ats_targetsinfo.flniv.v_tgtlatspd =
            logger_->Getinfoprovider().objects.at(left_id - one_objectid_t).Est.SpdLat;
        ats_targetsinfo.flniv.v_tgtrltspd = ats_targetsinfo.flniv.v_tgtlongtspd - spd;
        ats_targetsinfo.flniv.is_tgtprsnt = true;
        ats_targetsinfo.flniv.t_tgtdurtm =
            (logger_->Getinfocreator().array.maturitycounter.at(left_id - one_objectid_t)) * 0.04f;
        ats_targetsinfo.flniv.a_tgtlongtaccel =
            logger_->Getinfoprovider().objects.at(left_id - one_objectid_t).Est.AccelLgt;
        ats_targetsinfo.flniv.a_tgtlataccel =
            logger_->Getinfoprovider().objects.at(left_id - one_objectid_t).Est.AccelLat;
        ats_targetsinfo.flniv.indx_tgttyp = 1u;
        ats_targetsinfo.flniv.is_tgtlnechgflag = ((cutin_id == left_id) ? true : false);
    } else {
        ats_targetsinfo.flniv = default_iv_;
  }
}

void ObjectResultHandler::assignflniiv(mpc_drv_msgs::msg::TargetsInfo &ats_targetsinfo) {
    basetype::objectid_t left2_id{logger_->Getinfocreator().element.fliiv};
    basetype::objectid_t cutin_id{logger_->Getinfocreator().element.cutin};
    basetype::float32_t spd{logger_->Getinfoprovider().egovehinfo.SpdLgt};
    if (left2_id > zero_objectid_t) {
        ats_targetsinfo.flniiv.is_tgtnew =
            (left2_id != static_cast<basetype::objectid_t>(ats_targetsinfo.flniiv.indx_tgtid)
                 ? true : false);
        ats_targetsinfo.flniiv.indx_tgtid = left2_id;
        ats_targetsinfo.flniiv.d_tgtlongtdist =
            fmax((logger_->Getinfoprovider().objects.at(left2_id - one_objectid_t).Est.DisLgt - dis2frontbump_),
                 0.0f);
        ats_targetsinfo.flniiv.d_tgtlatdist =
            logger_->Getinfoprovider().objects.at(left2_id - one_objectid_t).Est.DisLat;
        ats_targetsinfo.flniiv.v_tgtlongtspd =
            logger_->Getinfoprovider().objects.at(left2_id - one_objectid_t).Est.SpdLgt;
        ats_targetsinfo.flniiv.v_tgtlatspd =
            logger_->Getinfoprovider().objects.at(left2_id - one_objectid_t).Est.SpdLat;
        ats_targetsinfo.flniiv.v_tgtrltspd = ats_targetsinfo.flniiv.v_tgtlongtspd - spd;
        ats_targetsinfo.flniiv.is_tgtprsnt = true;
        ats_targetsinfo.flniiv.t_tgtdurtm =
            (logger_->Getinfocreator().array.maturitycounter.at(left2_id - one_objectid_t)) * 0.04f;
        ats_targetsinfo.flniiv.a_tgtlongtaccel =
            logger_->Getinfoprovider().objects.at(left2_id - one_objectid_t).Est.AccelLgt;
        ats_targetsinfo.flniiv.a_tgtlataccel =
            logger_->Getinfoprovider().objects.at(left2_id - one_objectid_t).Est.AccelLat;
        ats_targetsinfo.flniiv.indx_tgttyp = 1u;
        ats_targetsinfo.flniiv.is_tgtlnechgflag = ((cutin_id == left2_id) ? true : false);
    } else {
        ats_targetsinfo.flniiv = default_iv_;
    }
}

void ObjectResultHandler::assignslniv(mpc_drv_msgs::msg::TargetsInfo &ats_targetsinfo) {
    basetype::objectid_t right_id{logger_->Getinfocreator().element.sliv};
    basetype::objectid_t cutin_id{logger_->Getinfocreator().element.cutin};
    basetype::float32_t spd{logger_->Getinfoprovider().egovehinfo.SpdLgt};

    if (right_id > zero_objectid_t) {
        ats_targetsinfo.slniv.is_tgtnew =
            (right_id != static_cast<basetype::objectid_t>(ats_targetsinfo.slniv.indx_tgtid)
                 ? true : false);
        ats_targetsinfo.slniv.indx_tgtid = right_id;
        ats_targetsinfo.slniv.d_tgtlongtdist =
            fmax((logger_->Getinfoprovider().objects.at(right_id - one_objectid_t).Est.DisLgt - dis2frontbump_),
                 0.0f);
        ats_targetsinfo.slniv.d_tgtlatdist =
            logger_->Getinfoprovider().objects.at(right_id - one_objectid_t).Est.DisLat;
        ats_targetsinfo.slniv.v_tgtlongtspd =
            logger_->Getinfoprovider().objects.at(right_id - one_objectid_t).Est.SpdLgt;
        ats_targetsinfo.slniv.v_tgtlatspd =
            logger_->Getinfoprovider().objects.at(right_id - one_objectid_t).Est.SpdLat;
        ats_targetsinfo.slniv.v_tgtrltspd = ats_targetsinfo.slniv.v_tgtlongtspd - spd;
        ats_targetsinfo.slniv.is_tgtprsnt = true;
        ats_targetsinfo.slniv.t_tgtdurtm =
            (logger_->Getinfocreator().array.maturitycounter.at(right_id - one_objectid_t)) * 0.04f;
        ats_targetsinfo.slniv.a_tgtlongtaccel =
            logger_->Getinfoprovider().objects.at(right_id - one_objectid_t).Est.AccelLgt;
        ats_targetsinfo.slniv.a_tgtlataccel =
            logger_->Getinfoprovider().objects.at(right_id - one_objectid_t).Est.AccelLat;
        ats_targetsinfo.slniv.indx_tgttyp = 1u;
        ats_targetsinfo.slniv.is_tgtlnechgflag = ((cutin_id == right_id) ? true : false);
    } else {
        ats_targetsinfo.slniv = default_iv_;
    }
}

void ObjectResultHandler::assignslniiv(mpc_drv_msgs::msg::TargetsInfo &ats_targetsinfo) {
    basetype::objectid_t right2_id{logger_->Getinfocreator().element.sliiv};
    basetype::objectid_t cutin_id{logger_->Getinfocreator().element.cutin};
    basetype::float32_t spd{logger_->Getinfoprovider().egovehinfo.SpdLgt};

    if (right2_id > zero_objectid_t) {
        ats_targetsinfo.slniiv.is_tgtnew =
            (right2_id != static_cast<basetype::objectid_t>(ats_targetsinfo.slniiv.indx_tgtid)
                 ? true : false);
        ats_targetsinfo.slniiv.indx_tgtid = right2_id;
        ats_targetsinfo.slniiv.d_tgtlongtdist =
            fmax((logger_->Getinfoprovider().objects.at(right2_id - one_objectid_t).Est.DisLgt - dis2frontbump_),
                 0.0f);
        ats_targetsinfo.slniiv.d_tgtlatdist =
            logger_->Getinfoprovider().objects.at(right2_id - one_objectid_t).Est.DisLat;
        ats_targetsinfo.slniiv.v_tgtlongtspd =
            logger_->Getinfoprovider().objects.at(right2_id - one_objectid_t).Est.SpdLgt;
        ats_targetsinfo.slniiv.v_tgtlatspd =
            logger_->Getinfoprovider().objects.at(right2_id - one_objectid_t).Est.SpdLat;
        ats_targetsinfo.slniiv.v_tgtrltspd = ats_targetsinfo.slniiv.v_tgtlongtspd - spd;
        ats_targetsinfo.slniiv.is_tgtprsnt = true;
        ats_targetsinfo.slniiv.t_tgtdurtm =
            (logger_->Getinfocreator().array.maturitycounter.at(right2_id - one_objectid_t)) * 0.04f;
        ats_targetsinfo.slniiv.a_tgtlongtaccel =
            logger_->Getinfoprovider().objects.at(right2_id - one_objectid_t).Est.AccelLgt;
        ats_targetsinfo.slniiv.a_tgtlataccel =
            logger_->Getinfoprovider().objects.at(right2_id - one_objectid_t).Est.AccelLat;
        ats_targetsinfo.slniiv.indx_tgttyp = 1u;
        ats_targetsinfo.slniiv.is_tgtlnechgflag = ((cutin_id == right2_id) ? true : false);
    } else {
        ats_targetsinfo.slniiv = default_iv_;
    }
}

void ObjectResultHandler::assignriv(mpc_drv_msgs::msg::TargetsInfo &ats_targetsinfo) {
    basetype::objectid_t rmain_id{logger_->Getinfocreator().element.riv};
    basetype::float32_t spd{logger_->Getinfoprovider().egovehinfo.SpdLgt};
    if (rmain_id > zero_objectid_t) {
        ats_targetsinfo.riv.is_tgtnew =
            (rmain_id != static_cast<basetype::objectid_t>(ats_targetsinfo.riv.indx_tgtid)
                 ? true : false);
        ats_targetsinfo.riv.indx_tgtid = rmain_id;
        ats_targetsinfo.riv.d_tgtlongtdist =
            fmin((logger_->Getinfoprovider().objects.at(rmain_id - one_objectid_t).Est.DisLgt) - dis2rearbump_,
                 dis2frontbump_);
        ats_targetsinfo.riv.d_tgtlatdist =
            logger_->Getinfoprovider().objects.at(rmain_id - one_objectid_t).Est.DisLat;
        ats_targetsinfo.riv.v_tgtlongtspd =
            logger_->Getinfoprovider().objects.at(rmain_id - one_objectid_t).Est.SpdLgt;
        ats_targetsinfo.riv.v_tgtlatspd =
            logger_->Getinfoprovider().objects.at(rmain_id - one_objectid_t).Est.SpdLat;
        ats_targetsinfo.riv.v_tgtrltspd = ats_targetsinfo.riv.v_tgtlongtspd - spd;
        ats_targetsinfo.riv.is_tgtprsnt = true;
        ats_targetsinfo.riv.t_tgtdurtm =
            (logger_->Getinfocreator().array.maturitycounter.at(rmain_id - one_objectid_t)) * 0.04f;
        ats_targetsinfo.riv.a_tgtlongtaccel =
            logger_->Getinfoprovider().objects.at(rmain_id - one_objectid_t).Est.AccelLgt;
        ats_targetsinfo.riv.a_tgtlataccel =
            logger_->Getinfoprovider().objects.at(rmain_id - one_objectid_t).Est.AccelLat;
        ats_targetsinfo.riv.indx_tgttyp = 1u;
        ats_targetsinfo.riv.is_tgtlnechgflag = false;
    } else {
        ats_targetsinfo.riv = default_riv_;
    }
}

void ObjectResultHandler::assignflnriv(mpc_drv_msgs::msg::TargetsInfo &ats_targetsinfo) {
    basetype::objectid_t rleft_id = logger_->Getinfocreator().element.rfliv;
    basetype::objectid_t rcutin_id = logger_->Getinfocreator().element.rcutin;
    basetype::float32_t spd = logger_->Getinfoprovider().egovehinfo.SpdLgt;
    if (rleft_id > zero_objectid_t) {
        ats_targetsinfo.flnriv.is_tgtnew =
            (rleft_id != static_cast<basetype::objectid_t>(ats_targetsinfo.flnriv.indx_tgtid)
                 ? true : false);
        ats_targetsinfo.flnriv.indx_tgtid = rleft_id;
        ats_targetsinfo.flnriv.d_tgtlongtdist =
            fmin((logger_->Getinfoprovider().objects.at(rleft_id - one_objectid_t).Est.DisLgt) - dis2rearbump_,
                dis2frontbump_);
        ats_targetsinfo.flnriv.d_tgtlatdist =
            logger_->Getinfoprovider().objects.at(rleft_id - one_objectid_t).Est.DisLat;
        ats_targetsinfo.flnriv.v_tgtlongtspd =
            logger_->Getinfoprovider().objects.at(rleft_id - one_objectid_t).Est.SpdLgt;
        ats_targetsinfo.flnriv.v_tgtlatspd =
            logger_->Getinfoprovider().objects.at(rleft_id - one_objectid_t).Est.SpdLat;
        ats_targetsinfo.flnriv.v_tgtrltspd = ats_targetsinfo.flnriv.v_tgtlongtspd - spd;
        ats_targetsinfo.flnriv.is_tgtprsnt = true;
        ats_targetsinfo.flnriv.t_tgtdurtm =
            (logger_->Getinfocreator().array.maturitycounter.at(rleft_id - one_objectid_t)) * 0.04f;
        ats_targetsinfo.flnriv.a_tgtlongtaccel =
            logger_->Getinfoprovider().objects.at(rleft_id - one_objectid_t).Est.AccelLgt;
        ats_targetsinfo.flnriv.a_tgtlataccel =
            logger_->Getinfoprovider().objects.at(rleft_id - one_objectid_t).Est.AccelLat;
        ats_targetsinfo.flnriv.indx_tgttyp = 1u;
        ats_targetsinfo.flnriv.is_tgtlnechgflag = ((rcutin_id == rleft_id) ? true : false);
    } else {
        ats_targetsinfo.flnriv = default_riv_;
    }
}

void ObjectResultHandler::assignslnriv(mpc_drv_msgs::msg::TargetsInfo &ats_targetsinfo) {
    basetype::objectid_t rright_id{logger_->Getinfocreator().element.rsliv};
    basetype::objectid_t rcutin_id{logger_->Getinfocreator().element.rcutin};
    basetype::float32_t spd{logger_->Getinfoprovider().egovehinfo.SpdLgt};
    if (rright_id > zero_objectid_t) {
    ats_targetsinfo.slnriv.is_tgtnew =
        (rright_id != static_cast<basetype::objectid_t>(ats_targetsinfo.slnriv.indx_tgtid)
             ? true : false);
    ats_targetsinfo.slnriv.indx_tgtid = rright_id - one_objectid_t;
    ats_targetsinfo.slnriv.d_tgtlongtdist =
        fmin((logger_->Getinfoprovider().objects.at(rright_id - one_objectid_t).Est.DisLgt) - dis2rearbump_,
            dis2frontbump_);
    ats_targetsinfo.slnriv.d_tgtlatdist =
        logger_->Getinfoprovider().objects.at(rright_id - one_objectid_t).Est.DisLat;
    ats_targetsinfo.slnriv.v_tgtlongtspd =
        logger_->Getinfoprovider().objects.at(rright_id - one_objectid_t).Est.SpdLgt;
    ats_targetsinfo.slnriv.v_tgtlatspd =
        logger_->Getinfoprovider().objects.at(rright_id - one_objectid_t).Est.SpdLat;
    ats_targetsinfo.slnriv.v_tgtrltspd = ats_targetsinfo.slnriv.v_tgtlongtspd - spd;
    ats_targetsinfo.slnriv.is_tgtprsnt = one_uint8_t;
    ats_targetsinfo.slnriv.t_tgtdurtm =
        (logger_->Getinfocreator().array.maturitycounter.at(rright_id - one_objectid_t)) * 0.04f;
    ats_targetsinfo.slnriv.a_tgtlongtaccel =
        logger_->Getinfoprovider().objects.at(rright_id - one_objectid_t).Est.AccelLgt;
    ats_targetsinfo.slnriv.a_tgtlataccel =
        logger_->Getinfoprovider().objects.at(rright_id - one_objectid_t).Est.AccelLat;
    ats_targetsinfo.slnriv.indx_tgttyp = 1u;
    ats_targetsinfo.slnriv.is_tgtlnechgflag = ((rcutin_id == rright_id) ? true : false);
    } else {
      ats_targetsinfo.slnriv = default_riv_;
    }
}

void ObjectResultHandler::assignresult(
    mpc_drv_msgs::msg::TargetsInfo &ats_targetsinfo) {
        ObjectResultHandler::assigniv(ats_targetsinfo);
        ObjectResultHandler::assigniiv(ats_targetsinfo);
        ObjectResultHandler::assignflniv(ats_targetsinfo);
        ObjectResultHandler::assignflniiv(ats_targetsinfo);
        ObjectResultHandler::assignslniv(ats_targetsinfo);
        ObjectResultHandler::assignslniiv(ats_targetsinfo);
        ObjectResultHandler::assignriv(ats_targetsinfo);
        ObjectResultHandler::assignflnriv(ats_targetsinfo);
        ObjectResultHandler::assignslnriv(ats_targetsinfo);
}

void ObjectResultHandler::assignresult(mpc_drv_msgs::msg::Corridor &ats_corridor) {
  InfoCreator infocreator{logger_->Getinfocreator()};
  InfoProvider infoprovider{logger_->Getinfoprovider()};
  ats_corridor.corridor_id = 0u;
  ats_corridor.disenable_reason = 0u;
  ats_corridor.drivable_width = 0.0f;
  ats_corridor.feature_tag = 0u;
  ats_corridor.ilc_object_target = 0u;
  ats_corridor.reference_line.c0 = 0.0f;
  ats_corridor.reference_line.c1 = 0.0f;
  ats_corridor.reference_line.c2 = 0.0f;
  ats_corridor.reference_line.c3 = 0.0f;
  ats_corridor.reference_line.start_range = 0.0f;
  ats_corridor.reference_line.end_range = 0.0f;
  ats_corridor.object_in_corridor_list.clear();
  ats_corridor.relevant_object_corridor_list.clear();
  basetype::uint8_t in_corridor_size{0u};
  basetype::uint8_t out_corridor_size{0u};
  for (basetype::objectid_t i = zero_objectid_t; i < c_ObjectSize; i++) {
    if (infocreator.array.valid.at(i) == boolean_t_true && infoprovider.objects.at(i).Info.Type != ObjectType::Pedestrain) {
      if (infocreator.array.withinfunnel.at(i) == boolean_t_true ||
          (infocreator.element.cutin > zero_objectid_t &&
           (infocreator.element.cutin - one_objectid_t == i))) {
            mpc_drv_msgs::msg::ObjectLocation objectlocation{};
            objectlocation.object_id = i + one_objectid_t;
            objectlocation.reference_point2corridor_frenet.l =
                infocreator.array.offset.at(i);
            objectlocation.reference_point2corridor_frenet.dl =
                infocreator.array.offsetderiv.at(i);
            objectlocation.reference_point2corridor_frenet.dl_valid_time = 0.0f;
            objectlocation.reference_point2corridor_frenet.ddl = 0.0f;
            objectlocation.reference_point2corridor_frenet.ddl_valid_time = 0.0f;
            objectlocation.reference_point2corridor_frenet.s =
                infoprovider.objects.at(i).Est.DisLgt;
            objectlocation.reference_point2corridor_frenet.ds =
                infoprovider.objects.at(i).Est.Spd;
            objectlocation.reference_point2corridor_frenet.dds =
                infoprovider.objects.at(i).Est.Accel;
            objectlocation.reference_point2corridor_frenet.dds_valid_time = 0.0f;
            ats_corridor.object_in_corridor_list.push_back(objectlocation);
            in_corridor_size++;
      } else {
        if (infocreator.array.withinwidefunnel.at(i) == boolean_t_true) {
            mpc_drv_msgs::msg::ObjectLocation objectlocation{};
            objectlocation.object_id = i + one_objectid_t;
            objectlocation.reference_point2corridor_frenet.l =
                infocreator.array.offset.at(i);
            objectlocation.reference_point2corridor_frenet.dl =
                infocreator.array.offsetderiv.at(i);
            objectlocation.reference_point2corridor_frenet.dl_valid_time = 0.0f;
            objectlocation.reference_point2corridor_frenet.ddl = 0.0f;
            objectlocation.reference_point2corridor_frenet.ddl_valid_time = 0.0f;
            objectlocation.reference_point2corridor_frenet.s =
                infoprovider.objects.at(i).Est.DisLgt;
            objectlocation.reference_point2corridor_frenet.ds =
                infoprovider.objects.at(i).Est.Spd;
            objectlocation.reference_point2corridor_frenet.dds =
                infoprovider.objects.at(i).Est.Accel;
            objectlocation.reference_point2corridor_frenet.dds_valid_time = 0.0f;
            ats_corridor.relevant_object_corridor_list.push_back(objectlocation);
            out_corridor_size++;
        }
      }
    }
  }
}

 void ObjectResultHandler::assignresult(uint32_t &pedestrain_id, mpc_drv_msgs::msg::TargetsInfo &ats_targetsinfo) {
    pedestrain_id = logger_->Getinfocreator().element.pedestrain;

    basetype::objectid_t sliv{logger_->Getinfocreator().element.sliv};
    basetype::objectid_t fliv{logger_->Getinfocreator().element.fliv};
    basetype::boolean_t enable_sliv{boolean_t_false};
    basetype::boolean_t enable_fliv{boolean_t_false};
    
    if (sliv > zero_objectid_t) {
        if (logger_->Getinfoprovider().objects.at(sliv - one_objectid_t).Info.Type == ObjectType::Bicycle ||
            logger_->Getinfoprovider().objects.at(sliv - one_objectid_t).Info.Type == ObjectType::Motorcycle) {
            if ((logger_->Getinfoprovider().objects.at(sliv - one_objectid_t).Est.DisLat < -2.5f) &&
                ((logger_->Getinfoprovider().objects.at(sliv - one_objectid_t).Est.DisLgt < 50.0f) &&
                 (logger_->Getinfoprovider().objects.at(sliv - one_objectid_t).Est.DisLgt < Util::max(logger_->Getinfoprovider().egovehinfo.SpdLgt, 2.7f) * 3.0f))) {
                enable_sliv = boolean_t_true;
            }
        }
    }
    
    if (fliv > zero_objectid_t) {
        if (logger_->Getinfoprovider().objects.at(fliv - one_objectid_t).Info.Type == ObjectType::Bicycle ||
            logger_->Getinfoprovider().objects.at(fliv - one_objectid_t).Info.Type == ObjectType::Motorcycle) {
            if ((logger_->Getinfoprovider().objects.at(fliv - one_objectid_t).Est.DisLat < 2.5f) &&
                ((logger_->Getinfoprovider().objects.at(fliv - one_objectid_t).Est.DisLgt < 50.0f) &&
                 (logger_->Getinfoprovider().objects.at(fliv - one_objectid_t).Est.DisLgt < Util::max(logger_->Getinfoprovider().egovehinfo.SpdLgt, 2.7f) * 3.0f))) {
                enable_fliv = boolean_t_true;
            }
        }
    }

    if (logger_->Getinfocreator().element.pedestrain == zero_objectid_t) {
        if (enable_sliv == boolean_t_true && enable_fliv == boolean_t_true) {
            if (logger_->Getinfoprovider().objects.at(sliv - one_objectid_t).Est.DisLgt <= 
                logger_->Getinfoprovider().objects.at(fliv - one_objectid_t).Est.DisLgt) {
                if (ats_targetsinfo.slniv.d_tgtlongtdist < ats_targetsinfo.iv.d_tgtlongtdist) {
                    //pedestrain_id = sliv;
                    ats_targetsinfo.slniv.indx_tgttyp = logger_->Getinfoprovider().objects.at(sliv - one_objectid_t).Info.Type_raw;
                }
            } else {
                if (ats_targetsinfo.flniv.d_tgtlongtdist < ats_targetsinfo.iv.d_tgtlongtdist) {
                    //pedestrain_id = fliv;
                    ats_targetsinfo.flniv.indx_tgttyp = logger_->Getinfoprovider().objects.at(fliv - one_objectid_t).Info.Type_raw;
                }
            }
        } else if (enable_sliv == boolean_t_true) {
            if (ats_targetsinfo.slniv.d_tgtlongtdist < ats_targetsinfo.iv.d_tgtlongtdist) {
                //pedestrain_id = sliv;
                ats_targetsinfo.slniv.indx_tgttyp = logger_->Getinfoprovider().objects.at(sliv - one_objectid_t).Info.Type_raw;
            }
        } else if (enable_fliv == boolean_t_true) {
            if (ats_targetsinfo.flniv.d_tgtlongtdist < ats_targetsinfo.iv.d_tgtlongtdist) {
                //pedestrain_id = fliv;
                ats_targetsinfo.flniv.indx_tgttyp = logger_->Getinfoprovider().objects.at(fliv - one_objectid_t).Info.Type_raw;
            }
        }
    }
}
} // namespace ACC_Target_Selection