#include "logger.h"

namespace ACC_Target_Selection{
    Logger::Logger() :
            InfoProvider_(),
            InfoCreator_() {
        for (basetype::objectid_t i = zero_objectid_t; i < c_ObjectSize; i++) {
            InfoProvider_.objects.at(i).Est.Accel                    = zero_float32_t;
            InfoProvider_.objects.at(i).Est.AccelLat                 = zero_float32_t;
            InfoProvider_.objects.at(i).Est.AccelLgt                 = zero_float32_t;
            InfoProvider_.objects.at(i).Est.Curvature                = zero_float32_t;
            InfoProvider_.objects.at(i).Est.DisLat                   = zero_float32_t;
            InfoProvider_.objects.at(i).Est.DisLgt                   = zero_float32_t;
            InfoProvider_.objects.at(i).Est.HeadingAngle             = zero_float32_t;
            InfoProvider_.objects.at(i).Est.Spd                      = zero_float32_t;
            InfoProvider_.objects.at(i).Est.SpdLat                   = zero_float32_t;
            InfoProvider_.objects.at(i).Est.SpdLgt                   = zero_float32_t;

            InfoProvider_.objects.at(i).Info.DisLatFromLaneCenter    = zero_float32_t;
            InfoProvider_.objects.at(i).Info.Height                  = zero_float32_t;
            InfoProvider_.objects.at(i).Info.MotionHistoryType       = ObjectMotionHistoryType::Unknown;
            InfoProvider_.objects.at(i).Info.MotionType              = ObjectMotionType::Unknown;
            InfoProvider_.objects.at(i).Info.Type                    = ObjectType::Unknown;
            InfoProvider_.objects.at(i).Info.Width                   = zero_float32_t;

            InfoProvider_.objects.at(i).Ppty.AccelStdDev             = zero_float32_t;
            InfoProvider_.objects.at(i).Ppty.CameraId                = zero_uint32_t;
            InfoProvider_.objects.at(i).Ppty.DisLatFromLaneCenterQf  = ObjectDisLatFromLaneCenterQf::UnReliable;
            InfoProvider_.objects.at(i).Ppty.DisLatStdDev            = zero_float32_t;
            InfoProvider_.objects.at(i).Ppty.DisLgtStdDev            = zero_float32_t;
            InfoProvider_.objects.at(i).Ppty.FusionSource            = ObjectFusionSource::None;
            InfoProvider_.objects.at(i).Ppty.HeadingAngleStdDev      = zero_float32_t;
            InfoProvider_.objects.at(i).Ppty.Id                      = zero_objectid_t;
            InfoProvider_.objects.at(i).Ppty.RadarId                 = zero_uint32_t;
            InfoProvider_.objects.at(i).Ppty.SpdStdDev               = zero_float32_t;
            InfoProvider_.objects.at(i).Ppty.TrackStatus             = ObjectTrackStatus::Invalid;

            InfoCreator_.array.c1.at(i)                              = zero_float32_t;
            InfoCreator_.array.coastedcounter.at(i)                  = zero_uint8_t;
            InfoCreator_.array.coastedok.at(i)                       = boolean_t_false;
            InfoCreator_.array.highqualityoffsetfromlanecenter.at(i) = boolean_t_false;
            InfoCreator_.array.isvalidmovingtarget.at(i)             = boolean_t_false;
            InfoCreator_.array.isvalidstationarytarget.at(i)         = boolean_t_false;
            InfoCreator_.array.jumpcounter.at(i)                     = zero_uint8_t;
            InfoCreator_.array.jumpok.at(i)                          = boolean_t_false;
            InfoCreator_.array.lanewidth.at(i)                       = zero_float32_t;
            InfoCreator_.array.innerleftfunnel.at(i)                 = zero_float32_t;
            InfoCreator_.array.outerleftfunnel.at(i)                 = zero_float32_t;
            InfoCreator_.array.maturitycounter.at(i)                 = zero_uint8_t;
            InfoCreator_.array.maturityok.at(i)                      = boolean_t_false;
            InfoCreator_.array.offset.at(i)                          = zero_float32_t;
            InfoCreator_.array.offsetderiv.at(i)                     = zero_float32_t;
            InfoCreator_.array.offsetraw.at(i)                       = zero_float32_t;
            InfoCreator_.array.qualityok.at(i)                       = boolean_t_false;
            InfoCreator_.array.innerrightfunnel.at(i)                = zero_float32_t;
            InfoCreator_.array.outerrightfunnel.at(i)                = zero_float32_t;
            InfoCreator_.array.useoffsetfromlanecenter.at(i)         = boolean_t_false;
            InfoCreator_.array.useprediction.at(i)                   = boolean_t_false;
            InfoCreator_.array.valid.at(i)                           = boolean_t_false;
            InfoCreator_.array.withinfunnel.at(i)                    = boolean_t_false;
            InfoCreator_.array.withinwidefunnel.at(i)                = boolean_t_false;
        }
        
    }
    
    Logger::Logger(const Logger& r) {
        Logger::InfoProvider_ = r.InfoProvider_;
        Logger::InfoCreator_ = r.InfoCreator_;
    }

    Logger::~Logger() {
    }

    InfoProvider& Logger::Getinfoprovider() const {
        return Logger::InfoProvider_;
    }

    InfoCreator& Logger::Getinfocreator() const {
        return Logger::InfoCreator_;
    }

    void Logger::logegovehinfo(const EgoVehicleInformation &egovehinfo) const {
        Logger::InfoProvider_.egovehinfo = egovehinfo;
    }

    void Logger::logroadperty(const RoadProperty &roadperty) const {
        Logger::InfoProvider_.roadperty = roadperty;
    }

    void Logger::logobjects(const std::array<Object, c_ObjectSize> &objects) const {
        for (basetype::objectid_t i = zero_objectid_t; i < c_ObjectSize; i++) {
            Logger::InfoProvider_.objects.at(i) = objects.at(i);
        }
    }

    void Logger::logpedestrain(const basetype::objectid_t &pedestrain) const {
        Logger::InfoCreator_.element.pedestrain = pedestrain;
    }

    void Logger::logiv(const basetype::objectid_t &iv) const {
        Logger::InfoCreator_.element.iv = iv;
    }

    void Logger::logiiv(const basetype::objectid_t &iiv) const {
        Logger::InfoCreator_.element.iiv = iiv;
    }

    void Logger::logfliv(const basetype::objectid_t &fliv) const {
        Logger::InfoCreator_.element.fliv = fliv;
    }

    void Logger::logcutin(const basetype::objectid_t &cutin) const {
        Logger::InfoCreator_.element.cutin = cutin;
    }

    void Logger::logsliv(const basetype::objectid_t &sliv) const {
        Logger::InfoCreator_.element.sliv = sliv;
    }

    void Logger::logfliiv(const basetype::objectid_t &fliiv) const {
        Logger::InfoCreator_.element.fliiv = fliiv;
    }

    void Logger::logsliiv(const basetype::objectid_t &sliiv) const {
        Logger::InfoCreator_.element.sliiv = sliiv;
    }

    void Logger::logriv(const basetype::objectid_t &riv) const {
        Logger::InfoCreator_.element.riv = riv;
    }

    void Logger::logriiv(const basetype::objectid_t &riiv) const {
        Logger::InfoCreator_.element.riiv = riiv;
    }

    void Logger::logrfliv(const basetype::objectid_t &rfliv) const {
        Logger::InfoCreator_.element.rfliv = rfliv;
    }

    void Logger::logrcutin(const basetype::objectid_t &rcutin) const {
        Logger::InfoCreator_.element.rcutin = rcutin;
    }

    void Logger::logrsliv(const basetype::objectid_t &rsliv) const {
        Logger::InfoCreator_.element.rsliv = rsliv;
    }

    void Logger::logrfliiv(const basetype::objectid_t &rfliiv) const {
        Logger::InfoCreator_.element.rfliiv = rfliiv;
    }

    void Logger::logrsliiv(const basetype::objectid_t &rsliiv) const {
        Logger::InfoCreator_.element.rsliiv = rsliiv;
    }

    void Logger::logivquality(const basetype::int8_t &ivquality) const {
        Logger::InfoCreator_.element.ivquality = ivquality;
    }

    void Logger::logc0(const basetype::float32_t &c0) const {
        Logger::InfoCreator_.element.c0 = c0;
    }

    void Logger::loginnerleftfunneloffset(const basetype::float32_t &innerleftfunneloffset) const {
        Logger::InfoCreator_.element.innerleftfunneloffset = innerleftfunneloffset;
    }

    void Logger::loginnerrightfunneloffset(const basetype::float32_t &innerrightfunneloffset) const {
        Logger::InfoCreator_.element.innerrightfunneloffset = innerrightfunneloffset;
    }

    void Logger::logouterleftfunneloffset(const basetype::float32_t &outerleftfunneloffset) const {
        Logger::InfoCreator_.element.outerleftfunneloffset = outerleftfunneloffset;
    }

    void Logger::logouterrightfunneloffset(const basetype::float32_t &outerrightfunneloffset) const {
        Logger::InfoCreator_.element.outerrightfunneloffset = outerrightfunneloffset;
    }

    void Logger::loglaneoffset(const basetype::float32_t &laneoffset) const {
        Logger::InfoCreator_.element.laneoffset = laneoffset;
    }

    void Logger::loghighqualityoffsetfromlanecenter(const basetype::boolean_t &highqualityoffsetfromlanecenter, const basetype::objectid_t &idn0) const {
        Logger::InfoCreator_.array.highqualityoffsetfromlanecenter.at(idn0) = highqualityoffsetfromlanecenter;
    }

    void Logger::loguseoffsetfromlanecenter(const basetype::boolean_t &useoffsetfromlanecenter, const basetype::objectid_t &idn0) const {
        Logger::InfoCreator_.array.useoffsetfromlanecenter.at(idn0) = useoffsetfromlanecenter;
    }

    void Logger::logoffset(const basetype::float32_t &offset, const basetype::objectid_t &idn0) const {
        Logger::InfoCreator_.array.offset.at(idn0) = offset;
    }

    void Logger::logoffsetraw(const basetype::float32_t &offsetraw, const basetype::objectid_t &idn0) const {
        Logger::InfoCreator_.array.offsetraw.at(idn0) = offsetraw;
    }

    void Logger::logoffsetderiv(const basetype::float32_t &offsetderiv, const basetype::objectid_t &idn0) const {
        Logger::InfoCreator_.array.offsetderiv.at(idn0) = offsetderiv;
    }

    void Logger::logc1(const basetype::float32_t &c1, const basetype::objectid_t &idn0) const {
        Logger::InfoCreator_.array.c1.at(idn0) = c1;
    }

    void Logger::logwithinfunnel(const basetype::boolean_t &withinfunnel, const basetype::objectid_t &idn0) const {
        Logger::InfoCreator_.array.withinfunnel.at(idn0) = withinfunnel;
    }

    void Logger::logwithinwidefunnel(const basetype::boolean_t &withinwidefunnel, const basetype::objectid_t &idn0) const {
        Logger::InfoCreator_.array.withinwidefunnel.at(idn0) = withinwidefunnel;
    }

    void Logger::loguseprediction(const basetype::boolean_t &useprediction, const basetype::objectid_t &idn0) const {
        Logger::InfoCreator_.array.useprediction.at(idn0) = useprediction;
    }

    void Logger::loginnerleftfunnel(const basetype::float32_t &leftfunnel, const basetype::objectid_t &idn0) const {
        Logger::InfoCreator_.array.innerleftfunnel.at(idn0) = leftfunnel;
    }

    void Logger::loginnerrightfunnel(const basetype::float32_t &rightfunnel, const basetype::objectid_t &idn0) const {
        Logger::InfoCreator_.array.innerrightfunnel.at(idn0) = rightfunnel;
    }

    void Logger::logouterleftfunnel(const basetype::float32_t &leftfunnel, const basetype::objectid_t &idn0) const {
        Logger::InfoCreator_.array.outerleftfunnel.at(idn0) = leftfunnel;
    }

    void Logger::logouterrightfunnel(const basetype::float32_t &rightfunnel, const basetype::objectid_t &idn0) const {
        Logger::InfoCreator_.array.outerrightfunnel.at(idn0) = rightfunnel;
    }

    void Logger::loglanewidth(const basetype::float32_t &lanewidth, const basetype::objectid_t &idn0) const {
        Logger::InfoCreator_.array.lanewidth.at(idn0) = lanewidth;
    }

    void Logger::logisrearobject(const basetype::boolean_t &isrearobject, const basetype::objectid_t &idn0) const {
        Logger::InfoCreator_.array.isrearobject.at(idn0) = isrearobject;
    }

    void Logger::logvalid(const basetype::boolean_t &valid, const basetype::objectid_t &idn0) const {
        Logger::InfoCreator_.array.valid.at(idn0) = valid;
    }

    void Logger::logisvalidstationarytarget(const basetype::boolean_t &isvalidstationarytarget, const basetype::objectid_t &idn0) const {
        Logger::InfoCreator_.array.isvalidstationarytarget.at(idn0) = isvalidstationarytarget;
    }
    
    void Logger::logisvalidmovingtarget(const basetype::boolean_t &isvalidmovingtarget, const basetype::objectid_t &idn0) const {
        Logger::InfoCreator_.array.isvalidmovingtarget.at(idn0) = isvalidmovingtarget;
    }

    void Logger::logmaturityok(const basetype::boolean_t &maturityok, const basetype::objectid_t &idn0) const {
        Logger::InfoCreator_.array.maturityok.at(idn0) = maturityok;
    }

    void Logger::logjumpok(const basetype::boolean_t &jumpok, const basetype::objectid_t &idn0) const {
        Logger::InfoCreator_.array.jumpok.at(idn0) = jumpok;
    }

    void Logger::logcoastedok(const basetype::boolean_t &coastedok, const basetype::objectid_t &idn0) const {
        Logger::InfoCreator_.array.coastedok.at(idn0) = coastedok;
    }

    void Logger::logqualityok(const basetype::boolean_t &qualityok, const basetype::objectid_t &idn0) const {
        Logger::InfoCreator_.array.qualityok.at(idn0) = qualityok;
    }

    void Logger::logmaturitycounters(const basetype::uint8_t &maturitycounter, const basetype::objectid_t &idn0) const {
        Logger::InfoCreator_.array.maturitycounter.at(idn0) = maturitycounter;
    }

    void Logger::logjumpcounter(const basetype::uint8_t &jumpcounter, const basetype::objectid_t &idn0) const {
        Logger::InfoCreator_.array.jumpcounter.at(idn0) = jumpcounter;
    }

    void Logger::logcoastedcounter(const basetype::uint8_t &coastedcounter, const basetype::objectid_t &idn0) const {
        Logger::InfoCreator_.array.coastedcounter.at(idn0) = coastedcounter;
    }

} // namespace ACC_Target_Selection
