#ifndef ACC_TARGET_SELECTION_LOGGER_H
#define ACC_TARGET_SELECTION_LOGGER_H

#include <array>
#include <memory>

#include "param.h"
#include "egovehicleinformation.h"
#include "roadproperty.h"
#include "object.h"

namespace ACC_Target_Selection{

    struct InfoProvider {
        EgoVehicleInformation egovehinfo;
        RoadProperty roadperty;
        std::array<Object, c_ObjectSize> objects;
        explicit InfoProvider() :
                egovehinfo(),
                roadperty(),
                objects() {
        }
        InfoProvider(const InfoProvider &r) :
                egovehinfo(r.egovehinfo),
                roadperty(r.roadperty),
                objects(r.objects) {
        }
        explicit InfoProvider(const EgoVehicleInformation &egovehinfo_,
                              const RoadProperty &roadperty_,
                              const std::array<Object, c_ObjectSize> &objects_):
                egovehinfo(egovehinfo_),
                roadperty(roadperty_),
                objects(objects_) {
        }
        InfoProvider& operator=(const InfoProvider& rhs) {
            egovehinfo = rhs.egovehinfo;
            roadperty = rhs.roadperty;
            objects = rhs.objects;
            return *this;
        }
    };

    struct DebugElement {
        basetype::objectid_t pedestrain;
        basetype::objectid_t iv;
        basetype::objectid_t iiv;
        basetype::objectid_t fliv;
        basetype::objectid_t sliv;
        basetype::objectid_t cutin;
        basetype::objectid_t fliiv;
        basetype::objectid_t sliiv;
        basetype::objectid_t riv;
        basetype::objectid_t riiv;
        basetype::objectid_t rfliv;
        basetype::objectid_t rsliv;
        basetype::objectid_t rcutin;
        basetype::objectid_t rfliiv;
        basetype::objectid_t rsliiv;
        basetype::int8_t ivquality;
        basetype::float32_t c0;
        basetype::float32_t innerleftfunneloffset;
        basetype::float32_t innerrightfunneloffset;
        basetype::float32_t outerleftfunneloffset;
        basetype::float32_t outerrightfunneloffset;
        basetype::float32_t laneoffset;
        explicit DebugElement() :
                pedestrain(zero_objectid_t),
                iv(zero_objectid_t),
                iiv(zero_objectid_t),
                fliv(zero_objectid_t),
                sliv(zero_objectid_t),
                cutin(zero_objectid_t),
                fliiv(zero_objectid_t),
                sliiv(zero_objectid_t),
                riv(zero_objectid_t),
                riiv(zero_objectid_t),
                rfliv(zero_objectid_t),
                rsliv(zero_objectid_t),
                rcutin(zero_objectid_t),
                rfliiv(zero_objectid_t),
                rsliiv(zero_objectid_t),
                ivquality(zero_int8_t),
                c0(zero_float32_t),
                innerleftfunneloffset(zero_float32_t),
                innerrightfunneloffset(zero_float32_t),
                outerleftfunneloffset(zero_float32_t),
                outerrightfunneloffset(zero_float32_t),
                laneoffset(zero_float32_t) {
        }
        DebugElement(const DebugElement &r) :
                pedestrain(r.pedestrain),
                iv(r.iv),
                iiv(r.iiv),
                fliv(r.fliv),
                sliv(r.sliv),
                cutin(r.cutin),
                fliiv(r.fliiv),
                sliiv(r.sliiv),
                riv(r.riv),
                riiv(r.riiv),
                rfliv(r.rfliv),
                rsliv(r.rsliv),
                rcutin(r.rcutin),
                rfliiv(r.rfliiv),
                rsliiv(r.rsliiv),
                ivquality(r.ivquality),
                c0(r.c0),
                innerleftfunneloffset(r.innerleftfunneloffset),
                innerrightfunneloffset(r.innerrightfunneloffset),
                outerleftfunneloffset(r.outerleftfunneloffset),
                outerrightfunneloffset(r.outerrightfunneloffset),
                laneoffset(r.laneoffset)  {
        }
        explicit DebugElement(const basetype::objectid_t &pedestrain_,
                              const basetype::objectid_t &iv_,
                              const basetype::objectid_t &iiv_,
                              const basetype::objectid_t &fliv_,
                              const basetype::objectid_t &sliv_,
                              const basetype::objectid_t &cutin_,
                              const basetype::objectid_t &fliiv_,
                              const basetype::objectid_t &sliiv_,
                              const basetype::objectid_t &riv_,
                              const basetype::objectid_t &riiv_,
                              const basetype::objectid_t &rfliv_,
                              const basetype::objectid_t &rsliv_,
                              const basetype::objectid_t &rcutin_,
                              const basetype::objectid_t &rfliiv_,
                              const basetype::objectid_t &rsliiv_,
                              const basetype::int8_t &ivquality_,
                              const basetype::float32_t &c0_,
                              const basetype::float32_t &innerleftfunneloffset_,
                              const basetype::float32_t &innerrightfunneloffset_,
                              const basetype::float32_t &outerleftfunneloffset_,
                              const basetype::float32_t &outerrightfunneloffset_,
                              const basetype::float32_t &laneoffset_):
                pedestrain(pedestrain_),
                iv(iv_),
                iiv(iiv_),
                fliv(fliv_),
                sliv(sliv_),
                cutin(cutin_),
                fliiv(fliiv_),
                sliiv(sliiv_),
                riv(riv_),
                riiv(riiv_),
                rfliv(rfliv_),
                rsliv(rsliv_),
                rcutin(rcutin_),
                rfliiv(rfliiv_),
                rsliiv(rsliiv_),
                ivquality(ivquality_),
                c0(c0_),
                innerleftfunneloffset(innerleftfunneloffset_),
                innerrightfunneloffset(innerrightfunneloffset_),
                outerleftfunneloffset(outerleftfunneloffset_),
                outerrightfunneloffset(outerrightfunneloffset_),
                laneoffset(laneoffset_) {
        }
        DebugElement& operator=(const DebugElement& rhs) {
            pedestrain = rhs.pedestrain;
            iv = rhs.iv;
            iiv = rhs.iiv;
            fliv = rhs.fliv;
            sliv = rhs.sliv;
            cutin = rhs.cutin;
            fliiv = rhs.fliiv;
            sliiv = rhs.sliiv;
            riv = rhs.riv;
            riiv = rhs.riiv;
            rfliv = rhs.rfliv;
            rsliv = rhs.rsliv;
            rcutin = rhs.rcutin;
            rfliiv = rhs.rfliiv;
            rsliiv = rhs.rsliiv;
            ivquality = rhs.ivquality;
            c0 = rhs.c0;
            innerleftfunneloffset = rhs.innerleftfunneloffset;
            innerrightfunneloffset = rhs.innerrightfunneloffset;
            outerleftfunneloffset = rhs.outerleftfunneloffset;
            outerrightfunneloffset = rhs.outerrightfunneloffset;
            laneoffset = rhs.laneoffset;
            return *this;
        }
    };

    struct DebugArray {
        std::array<basetype::boolean_t, c_ObjectSize> highqualityoffsetfromlanecenter;
        std::array<basetype::boolean_t, c_ObjectSize> useoffsetfromlanecenter;
        std::array<basetype::float32_t, c_ObjectSize> offset;
        std::array<basetype::float32_t, c_ObjectSize> offsetraw;
        std::array<basetype::float32_t, c_ObjectSize> offsetderiv;
        std::array<basetype::float32_t, c_ObjectSize> c1;
        std::array<basetype::boolean_t, c_ObjectSize> withinfunnel;
        std::array<basetype::boolean_t, c_ObjectSize> withinwidefunnel;
        std::array<basetype::boolean_t, c_ObjectSize> useprediction;
        std::array<basetype::float32_t, c_ObjectSize> innerleftfunnel;
        std::array<basetype::float32_t, c_ObjectSize> innerrightfunnel;
        std::array<basetype::float32_t, c_ObjectSize> outerleftfunnel;
        std::array<basetype::float32_t, c_ObjectSize> outerrightfunnel;
        std::array<basetype::float32_t, c_ObjectSize> lanewidth;
        std::array<basetype::boolean_t, c_ObjectSize> isrearobject;
        std::array<basetype::boolean_t, c_ObjectSize> valid;
        std::array<basetype::boolean_t, c_ObjectSize> isvalidstationarytarget;
        std::array<basetype::boolean_t, c_ObjectSize> isvalidmovingtarget;
        std::array<basetype::boolean_t, c_ObjectSize> maturityok;
        std::array<basetype::boolean_t, c_ObjectSize> jumpok;
        std::array<basetype::boolean_t, c_ObjectSize> coastedok;
        std::array<basetype::boolean_t, c_ObjectSize> qualityok;
        std::array<basetype::uint8_t, c_ObjectSize> maturitycounter;
        std::array<basetype::uint8_t, c_ObjectSize> jumpcounter;
        std::array<basetype::uint8_t, c_ObjectSize> coastedcounter;
        explicit DebugArray():
                highqualityoffsetfromlanecenter(),
                useoffsetfromlanecenter(),
                offset(),
                offsetraw(),
                offsetderiv(),
                c1(),
                withinfunnel(),
                withinwidefunnel(),
                useprediction(),
                innerleftfunnel(),
                innerrightfunnel(),
                outerleftfunnel(),
                outerrightfunnel(),
                lanewidth(),
                isrearobject(),
                valid(),
                isvalidstationarytarget(),
                isvalidmovingtarget(),
                maturityok(),
                jumpok(),
                coastedok(),
                qualityok(),
                maturitycounter(),
                jumpcounter(),
                coastedcounter() {
            for (basetype::objectid_t ii = zero_objectid_t; ii < c_ObjectSize; ii++) {
                highqualityoffsetfromlanecenter.at(ii) = boolean_t_false;
                useoffsetfromlanecenter.at(ii) = boolean_t_false;
                offset.at(ii) = zero_float32_t;
                offsetraw.at(ii) = zero_float32_t;
                offsetderiv.at(ii) = zero_float32_t;
                c1.at(ii) = zero_float32_t;
                withinfunnel.at(ii) = boolean_t_false;
                withinwidefunnel.at(ii) = boolean_t_false;
                useprediction.at(ii) = boolean_t_false;
                innerleftfunnel.at(ii) = zero_float32_t;
                innerrightfunnel.at(ii) = zero_float32_t;
                outerleftfunnel.at(ii) = zero_float32_t;
                outerrightfunnel.at(ii) = zero_float32_t;
                lanewidth.at(ii) = zero_float32_t;
                isrearobject.at(ii) = boolean_t_false;
                valid.at(ii) = boolean_t_false;
                isvalidstationarytarget.at(ii) = boolean_t_false;
                isvalidmovingtarget.at(ii) = boolean_t_false;
                maturityok.at(ii) = boolean_t_false;
                jumpok.at(ii) = boolean_t_false;
                coastedok.at(ii) = boolean_t_false;
                qualityok.at(ii) = boolean_t_false;
                maturitycounter.at(ii) = zero_uint8_t;
                jumpcounter.at(ii) = zero_uint8_t;
                coastedcounter.at(ii) = zero_uint8_t;
            }       
        }
        DebugArray(const DebugArray &r) :
                highqualityoffsetfromlanecenter(r.highqualityoffsetfromlanecenter),
                useoffsetfromlanecenter(r.useoffsetfromlanecenter),
                offset(r.offset),
                offsetraw(r.offsetraw),
                offsetderiv(r.offsetderiv),
                c1(r.c1),
                withinfunnel(r.withinfunnel),
                withinwidefunnel(r.withinwidefunnel),
                useprediction(r.useprediction),
                innerleftfunnel(r.innerleftfunnel),
                innerrightfunnel(r.innerrightfunnel),
                outerleftfunnel(r.outerleftfunnel),
                outerrightfunnel(r.outerrightfunnel),
                lanewidth(r.lanewidth),
                isrearobject(r.isrearobject),
                valid(r.valid),
                isvalidstationarytarget(r.isvalidstationarytarget),
                isvalidmovingtarget(r.isvalidmovingtarget),
                maturityok(r.maturityok),
                jumpok(r.jumpok),
                coastedok(r.coastedok),
                qualityok(r.qualityok),
                maturitycounter(r.maturitycounter),
                jumpcounter(r.jumpcounter),
                coastedcounter(r.coastedcounter) {
        }
        explicit DebugArray(const std::array<basetype::boolean_t, c_ObjectSize> &highqualityoffsetfromlanecenter_,
                            const std::array<basetype::boolean_t, c_ObjectSize> &useoffsetfromlanecenter_,
                            const std::array<basetype::float32_t, c_ObjectSize> &offset_,
                            const std::array<basetype::float32_t, c_ObjectSize> &offsetraw_,
                            const std::array<basetype::float32_t, c_ObjectSize> &offsetderiv_,
                            const std::array<basetype::float32_t, c_ObjectSize> &c1_,
                            const std::array<basetype::boolean_t, c_ObjectSize> &withinfunnel_,
                            const std::array<basetype::boolean_t, c_ObjectSize> &withinwidefunnel_,
                            const std::array<basetype::boolean_t, c_ObjectSize> &useprediction_,
                            const std::array<basetype::float32_t, c_ObjectSize> &innerleftfunnel_,
                            const std::array<basetype::float32_t, c_ObjectSize> &innerrightfunnel_,
                            const std::array<basetype::float32_t, c_ObjectSize> &outerleftfunnel_,
                            const std::array<basetype::float32_t, c_ObjectSize> &outerrightfunnel_,
                            const std::array<basetype::float32_t, c_ObjectSize> &lanewidth_,
                            const std::array<basetype::boolean_t, c_ObjectSize> &isrearobject_,
                            const std::array<basetype::boolean_t, c_ObjectSize> &valid_,
                            const std::array<basetype::boolean_t, c_ObjectSize> &isvalidstationarytarget_,
                            const std::array<basetype::boolean_t, c_ObjectSize> &isvalidmovingtarget_,
                            const std::array<basetype::boolean_t, c_ObjectSize> &maturityok_,
                            const std::array<basetype::boolean_t, c_ObjectSize> &jumpok_,
                            const std::array<basetype::boolean_t, c_ObjectSize> &coastedok_,
                            const std::array<basetype::boolean_t, c_ObjectSize> &qualityok_,
                            const std::array<basetype::uint8_t, c_ObjectSize> &maturitycounter_,
                            const std::array<basetype::uint8_t, c_ObjectSize> &jumpcounter_,
                            const std::array<basetype::uint8_t, c_ObjectSize> &coastedcounter_):
                    highqualityoffsetfromlanecenter(highqualityoffsetfromlanecenter_),
                    useoffsetfromlanecenter(useoffsetfromlanecenter_),
                    offset(offset_),
                    offsetraw(offsetraw_),
                    offsetderiv(offsetderiv_),
                    c1(c1_),
                    withinfunnel(withinfunnel_),
                    withinwidefunnel(withinwidefunnel_),
                    useprediction(useprediction_),
                    innerleftfunnel(innerleftfunnel_),
                    innerrightfunnel(innerrightfunnel_),
                    outerleftfunnel(outerleftfunnel_),
                    outerrightfunnel(outerrightfunnel_),
                    lanewidth(lanewidth_),
                    isrearobject(isrearobject_),
                    valid(valid_),
                    isvalidstationarytarget(isvalidstationarytarget_),
                    isvalidmovingtarget(isvalidmovingtarget_),
                    maturityok(maturityok_),
                    jumpok(jumpok_),
                    coastedok(coastedok_),
                    qualityok(qualityok_),
                    maturitycounter(maturitycounter_),
                    jumpcounter(jumpcounter_),
                    coastedcounter(coastedcounter_) {
        }
        DebugArray& operator=(const DebugArray& rhs) {
            highqualityoffsetfromlanecenter = rhs.highqualityoffsetfromlanecenter;
            useoffsetfromlanecenter = rhs.useoffsetfromlanecenter;
            offset = rhs.offset;
            offsetraw = rhs.offsetraw;
            offsetderiv = rhs.offsetderiv;
            c1 = rhs.c1;
            withinfunnel = rhs.withinfunnel;
            withinwidefunnel = rhs.withinwidefunnel;
            useprediction = rhs.useprediction;
            innerleftfunnel = rhs.innerleftfunnel;
            innerrightfunnel = rhs.innerrightfunnel;
            outerleftfunnel = rhs.outerleftfunnel;
            outerrightfunnel = rhs.outerrightfunnel;
            lanewidth = rhs.lanewidth;
            isrearobject = rhs.isrearobject;
            valid = rhs.valid;
            isvalidstationarytarget = rhs.isvalidstationarytarget;
            isvalidmovingtarget = rhs.isvalidmovingtarget;
            maturityok = rhs.maturityok;
            jumpok = rhs.jumpok;
            coastedok = rhs.coastedok;
            qualityok = rhs.qualityok;
            maturitycounter = rhs.maturitycounter;
            jumpcounter = rhs.jumpcounter;
            coastedcounter = rhs.coastedcounter;
            return *this;
        }
    };

    struct InfoCreator {
        DebugElement element;
        DebugArray array;
        explicit InfoCreator() :
                element(),
                array() {
        }
        InfoCreator(const InfoCreator &r) :
                element(r.element),
                array(r.array) {
        }
        explicit InfoCreator(const DebugElement &element_,
                              const DebugArray &array_) :
                element(element_),
                array(array_) {
        }
        InfoCreator& operator=(const InfoCreator& rhs) {
            element = rhs.element;
            array = rhs.array;
            return *this;
        }
    };

    class Logger {
    public:
        explicit Logger();
        Logger(const Logger& r);
        ~Logger();

        InfoProvider& Getinfoprovider() const;
        InfoCreator& Getinfocreator() const;

        void logegovehinfo(const EgoVehicleInformation &egovehinfo) const;
        void logroadperty(const RoadProperty &roadperty) const;
        void logobjects(const std::array<Object, c_ObjectSize> &objects) const;
        void logpedestrain(const basetype::objectid_t &pedestrain) const;
        void logiv(const basetype::objectid_t &iv) const;
        void logiiv(const basetype::objectid_t &iiv) const;
        void logfliv(const basetype::objectid_t &fliv) const;
        void logcutin(const basetype::objectid_t &cutin) const;
        void logsliv(const basetype::objectid_t &sliv) const;
        void logfliiv(const basetype::objectid_t &fliiv) const;
        void logsliiv(const basetype::objectid_t &sliiv) const;
        void logriv(const basetype::objectid_t &riv) const;
        void logriiv(const basetype::objectid_t &riiv) const;
        void logrfliv(const basetype::objectid_t &rfliv) const;
        void logrcutin(const basetype::objectid_t &rcutin) const;
        void logrsliv(const basetype::objectid_t &rsliv) const;
        void logrfliiv(const basetype::objectid_t &rfliiv) const;
        void logrsliiv(const basetype::objectid_t &rsliiv) const;
        void logivquality(const basetype::int8_t &ivquality) const;
        void logc0(const basetype::float32_t &c0) const;
        void loginnerleftfunneloffset(const basetype::float32_t &innerleftfunneloffset) const;
        void loginnerrightfunneloffset(const basetype::float32_t &innerrightfunneloffset) const;
        void logouterleftfunneloffset(const basetype::float32_t &outerleftfunneloffset) const;
        void logouterrightfunneloffset(const basetype::float32_t &outerrightfunneloffset) const;
        void loglaneoffset(const basetype::float32_t &laneoffset) const;

        void loghighqualityoffsetfromlanecenter(const basetype::boolean_t &highqualityoffsetfromlanecenter, const basetype::objectid_t &idn0) const;
        void loguseoffsetfromlanecenter(const basetype::boolean_t &useoffsetfromlanecenter, const basetype::objectid_t &idn0) const;
        void logoffset(const basetype::float32_t &offset, const basetype::objectid_t &idn0) const;
        void logoffsetraw(const basetype::float32_t &offsetraw, const basetype::objectid_t &idn0) const;
        void logoffsetderiv(const basetype::float32_t &offsetderiv, const basetype::objectid_t &idn0) const;
        void logc1(const basetype::float32_t &c1, const basetype::objectid_t &idn0) const;
        void logwithinfunnel(const basetype::boolean_t &withinfunnel, const basetype::objectid_t &idn0) const;
        void logwithinwidefunnel(const basetype::boolean_t &withinwidefunnel, const basetype::objectid_t &idn0) const;
        void loguseprediction(const basetype::boolean_t &useprediction, const basetype::objectid_t &idn0) const;
        void loginnerleftfunnel(const basetype::float32_t &leftfunnel, const basetype::objectid_t &idn0) const;
        void loginnerrightfunnel(const basetype::float32_t &rightfunnel, const basetype::objectid_t &idn0) const;
        void logouterleftfunnel(const basetype::float32_t &leftfunnel, const basetype::objectid_t &idn0) const;
        void logouterrightfunnel(const basetype::float32_t &rightfunnel, const basetype::objectid_t &idn0) const;
        void loglanewidth(const basetype::float32_t &lanewidth, const basetype::objectid_t &idn0) const;
        void logisrearobject(const basetype::boolean_t &isrearobject, const basetype::objectid_t &idn0) const;
        void logvalid(const basetype::boolean_t &valid, const basetype::objectid_t &idn0) const;
        void logisvalidstationarytarget(const basetype::boolean_t &isvalidstationarytarget, const basetype::objectid_t &idn0) const;
        void logisvalidmovingtarget(const basetype::boolean_t &isvalidmovingtarget, const basetype::objectid_t &idn0) const;
        void logmaturityok(const basetype::boolean_t &maturityok, const basetype::objectid_t &idn0) const;
        void logjumpok(const basetype::boolean_t &jumpok, const basetype::objectid_t &idn0) const;
        void logcoastedok(const basetype::boolean_t &coastedok, const basetype::objectid_t &idn0) const;
        void logqualityok(const basetype::boolean_t &qualityok, const basetype::objectid_t &idn0) const;
        void logmaturitycounters(const basetype::uint8_t &maturitycounter, const basetype::objectid_t &idn0) const;
        void logjumpcounter(const basetype::uint8_t &jumpcounter, const basetype::objectid_t &idn0) const;
        void logcoastedcounter(const basetype::uint8_t &coastedcounter, const basetype::objectid_t &idn0) const;
    private:
        mutable InfoProvider InfoProvider_;
        mutable InfoCreator InfoCreator_;
    };
} // namespace ACC_Target_Selection

#endif