#include "ats_node.h"

namespace ACC_Target_Selection {
Node::Node(/*rclcpp::Node::SharedPtr ros2node*/) :
        // ros2node_(ros2node),
        EgoInfoHandler_(),
        ObjectInfoHandler_(),
        Runner_(),
        ObjectResultHandler_(),
        counters_(),
        running_(false) {
  Node::Init(); 
}

Node::~Node() { Node::Stop(); }
void Node::Init() {
  ObjectResultHandler_.setlogger(&(Runner_.logger_));
}

void Node::Run(const int64_t &now_timestamp,
    const std::shared_ptr<vehicle_msgs::msg::ApSas10msPdu00> &SAS_10ms_PDU00,
    const std::shared_ptr<vehicle_msgs::msg::ApBcm50msPdu02> &BCM_50ms_PDU02,
    const std::shared_ptr<vehicle_msgs::msg::ApIbs20msPdu18> &IBS_20ms_PDU18,
    const std::shared_ptr<vehicle_msgs::msg::ApIbs20msPdu15> &IBS_20ms_PDU15,
    const std::shared_ptr<vehicle_msgs::msg::ApIbs20msPdu08> &IBS_20ms_PDU08,
    const std::shared_ptr<cem_interfaces::msg::TrafficParticipantStruct> &trafficparticipantrosmg,
    const std::shared_ptr<cem_interfaces::msg::TrafficRoadStruct> &trafficroadrosmsg,
    const mpc_drv_msgs::msg::RoadInfo &roadinfo,
    mpc_drv_msgs::msg::Corridor &ats_corridor,
    mpc_drv_msgs::msg::TargetsInfo &ats_targetsinfo,
    uint32_t &pedestrain_id) {
  Node::Start(now_timestamp, SAS_10ms_PDU00, BCM_50ms_PDU02, IBS_20ms_PDU18, IBS_20ms_PDU15,
              IBS_20ms_PDU08, trafficparticipantrosmg, trafficroadrosmsg,
              roadinfo, ats_corridor, ats_targetsinfo,
              pedestrain_id);
  Node::running_ = true;
}

// void Node::Run(const int64_t &now_timestamp,
//     const vehicle_msgs::msg::ApSas10msPdu00::SharedPtr &SAS_10ms_PDU00,
//     const vehicle_msgs::msg::ApBcm50msPdu02::SharedPtr &BCM_50ms_PDU02,
//     const vehicle_msgs::msg::ApIbs20msPdu18::SharedPtr &IBS_20ms_PDU18,
//     const vehicle_msgs::msg::ApIbs20msPdu15::SharedPtr &IBS_20ms_PDU15,
//     const vehicle_msgs::msg::ApIbs20msPdu08::SharedPtr &IBS_20ms_PDU08,
//     const cem_interfaces::msg::TrafficParticipantStruct::SharedPtr &trafficparticipantrosmg,
//     const cem_interfaces::msg::TrafficRoadStruct::SharedPtr &trafficroadrosmsg,
//     const mpc_drv_msgs::msg::RoadInfo &roadinfo,
//     mpc_drv_msgs::msg::Corridor &ats_corridor,
//     mpc_drv_msgs::msg::TargetsInfo &ats_targetsinfo,
//     uint32_t &pedestrain_id) {
//   Node::Start(now_timestamp, SAS_10ms_PDU00, BCM_50ms_PDU02, IBS_20ms_PDU18, IBS_20ms_PDU15,
//               IBS_20ms_PDU08, trafficparticipantrosmg, trafficroadrosmsg,
//               roadinfo, ats_corridor, ats_targetsinfo,
//               pedestrain_id);
//   Node::running_ = true;
// }

void Node::Run(const int64_t &now_timestamp,
    const rm::LocalView &local_view,
    const mpc_drv_msgs::msg::RoadInfo &roadinfo,
    mpc_drv_msgs::msg::Corridor &ats_corridor,
    mpc_drv_msgs::msg::TargetsInfo &ats_targetsinfo,
    uint32_t &pedestrain_id) {
  Node::Start(now_timestamp,
              local_view.sas_10ms_pdu00, local_view.bcm_50ms_pdu02,
              local_view.ibs_20ms_pdu18, local_view.ibs_20ms_pdu15,
              local_view.ibs_20ms_pdu08, local_view.traffic_participant,
              local_view.traffic_road,
              roadinfo, ats_corridor, ats_targetsinfo,
              pedestrain_id);
  Node::running_ = true;
}

// void Node::Start(const int64_t &now_timestamp,
//     const vehicle_msgs::msg::ApSas10msPdu00::SharedPtr &SAS_10ms_PDU00,
//     const vehicle_msgs::msg::ApBcm50msPdu02::SharedPtr &BCM_50ms_PDU02,
//     const vehicle_msgs::msg::ApIbs20msPdu18::SharedPtr &IBS_20ms_PDU18,
//     const vehicle_msgs::msg::ApIbs20msPdu15::SharedPtr &IBS_20ms_PDU15,
//     const vehicle_msgs::msg::ApIbs20msPdu08::SharedPtr &IBS_20ms_PDU08,
//     const cem_interfaces::msg::TrafficParticipantStruct::SharedPtr &trafficparticipantrosmg,
//     const cem_interfaces::msg::TrafficRoadStruct::SharedPtr &trafficroadrosmsg,
//     const mpc_drv_msgs::msg::RoadInfo &roadinfo,
//     mpc_drv_msgs::msg::Corridor &ats_corridor,
//     mpc_drv_msgs::msg::TargetsInfo &ats_targetsinfo,
//     uint32_t &pedestrain_id) {

//   // Node::mainThread_ =
//   // std::make_unique<std::thread>(std::bind(&Node::MainProcess, this));
//   Node::MainProcess(now_timestamp, SAS_10ms_PDU00, BCM_50ms_PDU02, IBS_20ms_PDU18,
//                     IBS_20ms_PDU15, IBS_20ms_PDU08, trafficparticipantrosmg,
//                     trafficroadrosmsg, roadinfo, ats_corridor, ats_targetsinfo,
//                     pedestrain_id);
// }

void Node::Start(const int64_t &now_timestamp,
    const std::shared_ptr<vehicle_msgs::msg::ApSas10msPdu00> &SAS_10ms_PDU00,
    const std::shared_ptr<vehicle_msgs::msg::ApBcm50msPdu02> &BCM_50ms_PDU02,
    const std::shared_ptr<vehicle_msgs::msg::ApIbs20msPdu18> &IBS_20ms_PDU18,
    const std::shared_ptr<vehicle_msgs::msg::ApIbs20msPdu15> &IBS_20ms_PDU15,
    const std::shared_ptr<vehicle_msgs::msg::ApIbs20msPdu08> &IBS_20ms_PDU08,
    const std::shared_ptr<cem_interfaces::msg::TrafficParticipantStruct> &trafficparticipantrosmg,
    const std::shared_ptr<cem_interfaces::msg::TrafficRoadStruct> &trafficroadrosmsg,
    const mpc_drv_msgs::msg::RoadInfo &roadinfo,
    mpc_drv_msgs::msg::Corridor &ats_corridor,
    mpc_drv_msgs::msg::TargetsInfo &ats_targetsinfo,
    uint32_t &pedestrain_id) {

  // Node::mainThread_ =
  // std::make_unique<std::thread>(std::bind(&Node::MainProcess, this));
  Node::MainProcess(now_timestamp, SAS_10ms_PDU00, BCM_50ms_PDU02, IBS_20ms_PDU18,
                    IBS_20ms_PDU15, IBS_20ms_PDU08, trafficparticipantrosmg,
                    trafficroadrosmsg, roadinfo, ats_corridor, ats_targetsinfo,
                    pedestrain_id);
}

void Node::Stop() {
  Node::running_ = false;
  // if (Node::mainThread_->joinable()) {
  //    Node::mainThread_->join();
  //}
  // Node::mainThread_ = nullptr;
}

void Node::MainProcess(
    const int64_t &now_timestamp,
    const std::shared_ptr<vehicle_msgs::msg::ApSas10msPdu00> &SAS_10ms_PDU00,
    const std::shared_ptr<vehicle_msgs::msg::ApBcm50msPdu02> &BCM_50ms_PDU02,
    const std::shared_ptr<vehicle_msgs::msg::ApIbs20msPdu18> &IBS_20ms_PDU18,
    const std::shared_ptr<vehicle_msgs::msg::ApIbs20msPdu15> &IBS_20ms_PDU15,
    const std::shared_ptr<vehicle_msgs::msg::ApIbs20msPdu08> &IBS_20ms_PDU08,
    const std::shared_ptr<cem_interfaces::msg::TrafficParticipantStruct> &trafficparticipantrosmg,
    const std::shared_ptr<cem_interfaces::msg::TrafficRoadStruct> &trafficroadrosmsg,
    const mpc_drv_msgs::msg::RoadInfo &roadinfo,
    mpc_drv_msgs::msg::Corridor &ats_corridor,
    mpc_drv_msgs::msg::TargetsInfo &ats_targetsinfo,
    uint32_t &pedestrain_id) {
  // while (Node::running_ = true) {
  EgoInfoHandler_.updateegoinfo(now_timestamp, SAS_10ms_PDU00, BCM_50ms_PDU02,
                                       IBS_20ms_PDU18, IBS_20ms_PDU15, IBS_20ms_PDU08);
  LaneInfoHandler_.updatelane(now_timestamp, trafficroadrosmsg);
  ObjectInfoHandler_.updateobjectgroups(now_timestamp, trafficparticipantrosmg);
  //Node::ObjectInfoHandler_->updateobjectgroups(now_timestamp, roadinfo);
  
  //Node::ObjectInfoHandler_->objectinfo_[1].Est.DisLgt = -30.0f;
  //Node::ObjectInfoHandler_->objectinfo_[1].Est.DisLat = 3.0f;
  //Node::ObjectInfoHandler_->objectinfo_[1].Est.Spd = 10.0f;
  //Node::ObjectInfoHandler_->objectinfo_[1].Est.SpdLgt = 10.0f;
  //Node::ObjectInfoHandler_->objectinfo_[1].Info.DisLatFromLaneCenter = 0.0;
  //Node::ObjectInfoHandler_->objectinfo_[1].Info.MotionHistoryType = ObjectMotionHistoryType::Receeding;
  //Node::ObjectInfoHandler_->objectinfo_[1].Info.MotionType = ObjectMotionType::Receeding;
  //Node::ObjectInfoHandler_->objectinfo_[1].Info.Type = ObjectType::Car;
  //Node::ObjectInfoHandler_->objectinfo_[1].Info.Width = 0.0f;
  //Node::ObjectInfoHandler_->objectinfo_[1].Ppty.FusionSource = ObjectFusionSource::Fused;
  //Node::ObjectInfoHandler_->objectinfo_[1].Ppty.TrackStatus = ObjectTrackStatus::Update;
  //Node::ObjectInfoHandler_->objectinfo_[1].Ppty.DisLatFromLaneCenterQf = ObjectDisLatFromLaneCenterQf::UnReliable;
  Node::Runner_.step((EgoInfoHandler_.egovehinfo_),
                      (ObjectInfoHandler_.objectinfo_),
                      (LaneInfoHandler_.laneinfo_));

  Node::assignresult(ats_corridor, ats_targetsinfo, pedestrain_id);
  //}
  // std::cout<< "IV: "<<int(ats_targetsinfo->IV.indx_TgtID)<<std::endl;
  // std::cout<< "Dx: "<<(ats_targetsinfo->IV.d_TgtLongtDist)<<std::endl;
  // std::cout<< "Dy: "<<(ats_targetsinfo->IV.d_TgtLatDist)<<std::endl;
  // std::cout<< "spd: "<<(ats_targetsinfo->IV.v_TgtLongtSpd)<<std::endl;
  // std::cout<< "counters_: "<<int(ats_targetsinfo->CycleCounter)<<std::endl;
}

void Node::assignresult(mpc_drv_msgs::msg::Corridor &ats_corridor,
                        mpc_drv_msgs::msg::TargetsInfo &ats_targetsinfo,
                        uint32_t &pedestrain_id) {
  Node::accumulate();
  ObjectResultHandler_.assignresult(ats_targetsinfo);
  ObjectResultHandler_.assignresult(ats_corridor);
  ObjectResultHandler_.assignresult(pedestrain_id, ats_targetsinfo);
  ats_targetsinfo.cyclecounter = Node::counters_;
  ats_targetsinfo.timestamp = 0;
}

void Node::accumulate() {
  if (Node::running_) {
    Node::counters_++;
  } else {
    Node::counters_ = zero_uint16_t;
  }
  Node::counters_ = Node::counters_ % 64;
}

} // namespace ACC_Target_Selection
