#ifndef ACC_TARGET_SELECTION_NODE_H
#define ACC_TARGET_SELECTION_NODE_H

#include <atomic>
#include <memory>
#include <thread>

// #include "rclcpp/rclcpp.hpp"
#include "egoinfohandler.h"
#include "laneinfohandler.h"
#include "logger.h"
#include "objectinfohandler.h"
#include "objectresulthandler.h"
#include "runner.h"
#include "selector.h"
#include "cem_interfaces/msg/traffic_participant_struct.hpp"
#include "cem_interfaces/msg/traffic_road_struct.hpp"
#include "vehicle_msgs/msg/ap_bcm50ms_pdu02.hpp"
#include "vehicle_msgs/msg/ap_ibs20ms_pdu08.hpp"
#include "vehicle_msgs/msg/ap_ibs20ms_pdu15.hpp"
#include "vehicle_msgs/msg/ap_ibs20ms_pdu18.hpp"
#include "vehicle_msgs/msg/ap_sas10ms_pdu00.hpp"
#include "local_view.h"

namespace ACC_Target_Selection {
class Selector;
class Logger;
class Runner;
class EgoInfoHandler;
class ObjectInfoHandler;
class LaneInfoHandler;
class ObjectResultHandler;

class Node {
public:
  Node(/*rclcpp::Node::SharedPtr ros2node*/);
  ~Node();
  // std::shared_ptr<rclcpp::Node> ros2node_;
  basetype::uint16_t counters_;
  void Init();
  void
  Run(const int64_t &now_timestamp,
      const std::shared_ptr<vehicle_msgs::msg::ApSas10msPdu00> &SAS_10ms_PDU00,
      const std::shared_ptr<vehicle_msgs::msg::ApBcm50msPdu02> &BCM_50ms_PDU02,
      const std::shared_ptr<vehicle_msgs::msg::ApIbs20msPdu18> &IBS_20ms_PDU18,
      const std::shared_ptr<vehicle_msgs::msg::ApIbs20msPdu15> &IBS_20ms_PDU15,
      const std::shared_ptr<vehicle_msgs::msg::ApIbs20msPdu08> &IBS_20ms_PDU08,
      const std::shared_ptr<cem_interfaces::msg::TrafficParticipantStruct> &trafficparticipantrosmg,
      const std::shared_ptr<cem_interfaces::msg::TrafficRoadStruct> &trafficroadrosmsg,
      const mpc_drv_msgs::msg::RoadInfo &roadinfo,
      mpc_drv_msgs::msg::Corridor &ats_corridor,
      mpc_drv_msgs::msg::TargetsInfo &ats_targetsinfo,
      uint32_t &pedestrain_id);
  // Run(const int64_t &now_timestamp,
  //     const vehicle_msgs::msg::ApSas10msPdu00::SharedPtr &SAS_10ms_PDU00,
  //     const vehicle_msgs::msg::ApBcm50msPdu02::SharedPtr &BCM_50ms_PDU02,
  //     const vehicle_msgs::msg::ApIbs20msPdu18::SharedPtr &IBS_20ms_PDU18,
  //     const vehicle_msgs::msg::ApIbs20msPdu15::SharedPtr &IBS_20ms_PDU15,
  //     const vehicle_msgs::msg::ApIbs20msPdu08::SharedPtr &IBS_20ms_PDU08,
  //     const cem_interfaces::msg::TrafficParticipantStruct::SharedPtr &trafficparticipantrosmg,
  //     const cem_interfaces::msg::TrafficRoadStruct::SharedPtr &trafficroadrosmsg,
  //     const mpc_drv_msgs::msg::RoadInfo &roadinfo,
  //     mpc_drv_msgs::msg::Corridor &ats_corridor,
  //     mpc_drv_msgs::msg::TargetsInfo &ats_targetsinfo,
  //     uint32_t &pedestrain_id);
  
  void Run(const int64_t &now_timestamp,
           const rm::LocalView &local_view,
           const mpc_drv_msgs::msg::RoadInfo &roadinfo,
           mpc_drv_msgs::msg::Corridor &ats_corridor,
           mpc_drv_msgs::msg::TargetsInfo &ats_targetsinfo,
           uint32_t &pedestrain_id);

  void MainProcess(
      const int64_t &now_timestamp,
      const std::shared_ptr<vehicle_msgs::msg::ApSas10msPdu00> &SAS_10ms_PDU00,
      const std::shared_ptr<vehicle_msgs::msg::ApBcm50msPdu02> &BCM_50ms_PDU02,
      const std::shared_ptr<vehicle_msgs::msg::ApIbs20msPdu18> &IBS_20ms_PDU18,
      const std::shared_ptr<vehicle_msgs::msg::ApIbs20msPdu15> &IBS_20ms_PDU15,
      const std::shared_ptr<vehicle_msgs::msg::ApIbs20msPdu08> &IBS_20ms_PDU08,
      const std::shared_ptr<cem_interfaces::msg::TrafficParticipantStruct> &trafficparticipantrosmg,
      const std::shared_ptr<cem_interfaces::msg::TrafficRoadStruct> &trafficroadrosmsg,
      const mpc_drv_msgs::msg::RoadInfo &roadinfo,
      mpc_drv_msgs::msg::Corridor &ats_corridor,
      mpc_drv_msgs::msg::TargetsInfo &ats_targetsinfo,
      uint32_t &pedestrain_id);

  void assignresult(mpc_drv_msgs::msg::Corridor &ats_corridor,
                    mpc_drv_msgs::msg::TargetsInfo &ats_targetsinfo,
                    uint32_t &pedestrain_id);

  void assigncorridor(mpc_drv_msgs::msg::Corridor &ats_corridor);

  void accumulate();
private:
void
  Start(const int64_t &now_timestamp,
        const std::shared_ptr<vehicle_msgs::msg::ApSas10msPdu00> &SAS_10ms_PDU00,
        const std::shared_ptr<vehicle_msgs::msg::ApBcm50msPdu02> &BCM_50ms_PDU02,
        const std::shared_ptr<vehicle_msgs::msg::ApIbs20msPdu18> &IBS_20ms_PDU18,
        const std::shared_ptr<vehicle_msgs::msg::ApIbs20msPdu15> &IBS_20ms_PDU15,
        const std::shared_ptr<vehicle_msgs::msg::ApIbs20msPdu08> &IBS_20ms_PDU08,
        const std::shared_ptr<cem_interfaces::msg::TrafficParticipantStruct> &trafficparticipantrosmg,
        const std::shared_ptr<cem_interfaces::msg::TrafficRoadStruct> &trafficroadrosmsg,
        const mpc_drv_msgs::msg::RoadInfo &roadinfo,
        mpc_drv_msgs::msg::Corridor &ats_corridor,
        mpc_drv_msgs::msg::TargetsInfo &ats_targetsinfo,
        uint32_t &pedestrain_id);
  // void
  // Start(const int64_t &now_timestamp,
  //       const vehicle_msgs::msg::ApSas10msPdu00::SharedPtr &SAS_10ms_PDU00,
  //       const vehicle_msgs::msg::ApBcm50msPdu02::SharedPtr &BCM_50ms_PDU02,
  //       const vehicle_msgs::msg::ApIbs20msPdu18::SharedPtr &IBS_20ms_PDU18,
  //       const vehicle_msgs::msg::ApIbs20msPdu15::SharedPtr &IBS_20ms_PDU15,
  //       const vehicle_msgs::msg::ApIbs20msPdu08::SharedPtr &IBS_20ms_PDU08,
  //       const cem_interfaces::msg::TrafficParticipantStruct::SharedPtr &trafficparticipantrosmg,
  //       const cem_interfaces::msg::TrafficRoadStruct::SharedPtr &trafficroadrosmsg,
  //       const mpc_drv_msgs::msg::RoadInfo &roadinfo,
  //       mpc_drv_msgs::msg::Corridor &ats_corridor,
  //       mpc_drv_msgs::msg::TargetsInfo &ats_targetsinfo,
  //       uint32_t &pedestrain_id);
  void Stop();
  std::atomic_bool running_{false};
  // std::unique_ptr<std::thread> mainThread_;
  Runner Runner_;
  EgoInfoHandler EgoInfoHandler_;
  ObjectInfoHandler ObjectInfoHandler_;
  LaneInfoHandler LaneInfoHandler_;
  ObjectResultHandler ObjectResultHandler_;
};

} // namespace ACC_Target_Selection

#endif