#include "offsetcalculator.h"

#include <cmath>

#include "atsutil.h"

namespace ACC_Target_Selection{
    OffsetCalculator::OffsetCalculator(const Param_OffsetCalculator &param_conf, const Logger &logger):
        param_conf_(param_conf),
        logger_(logger),
        offset_(zero_float32_t),
        offsetderiv_(zero_float32_t),
        highqualityoffsetfromlanecenter_(boolean_t_false) {
        for (basetype::objectid_t i = zero_objectid_t; i < c_ObjectSize; i++) {
            maturitycounter_.at(i)    = zero_uint8_t;
            offsetfilterstates_.at(i) = zero_float32_t;
            savedoffsetderiv_.at(i)   = zero_float32_t;
            savedoffset_.at(i)        = zero_float32_t;
            savedstatus_.at(i)        = boolean_t_false;
        }      
    }

    OffsetCalculator::OffsetCalculator(const OffsetCalculator& r) :
            param_conf_(r.param_conf_),
            logger_(r.logger_),
            offset_(r.offset_),
            offsetderiv_(r.offsetderiv_),
            highqualityoffsetfromlanecenter_(r.highqualityoffsetfromlanecenter_),
            maturitycounter_(r.maturitycounter_),
            offsetfilterstates_(r.offsetfilterstates_),
            savedoffsetderiv_(r.savedoffsetderiv_),
            savedoffset_(r.savedoffset_),
            savedstatus_(r.savedstatus_) {
    }

    OffsetCalculator::~OffsetCalculator() {
    }

    void OffsetCalculator::checkuselanecenteroffset(const ObjectDisLatFromLaneCenterQf &quality,
                                                    const basetype::boolean_t &isoffsetfromlanecenterusable,
                                                    const basetype::float32_t &timegap,
                                                    const basetype::boolean_t &useyawinlanechange,
                                                    basetype::boolean_t &useoffsetfromlanecenter,
                                                    basetype::boolean_t &highqualityoffsetfromlanecenter) {
        useoffsetfromlanecenter = boolean_t_false;
        highqualityoffsetfromlanecenter = boolean_t_false;

        if ((isoffsetfromlanecenterusable == boolean_t_true) &&
            (param_conf_.enableuseoffsetfromlanecenter == boolean_t_true) &&
            (quality > ObjectDisLatFromLaneCenterQf::LowReliable)) {
            if ((Util::abs(timegap) > param_conf_.mintimegapforusingdistancefromlanecenter) &&
                (useyawinlanechange == boolean_t_false)) {
                useoffsetfromlanecenter = boolean_t_true;
                if (quality > ObjectDisLatFromLaneCenterQf::LowReliable) {
                    highqualityoffsetfromlanecenter = boolean_t_true;
                }
            }
        }
    }

    void OffsetCalculator::updatematurity(const basetype::objectid_t &idn0) {
        if (maturitycounter_.at(idn0) < param_conf_.objectminmaturity) {
            maturitycounter_.at(idn0) = maturitycounter_.at(idn0) + one_uint8_t;
        }
    }

     basetype::boolean_t OffsetCalculator::ismaturity(const basetype::objectid_t &idn0) {
         basetype::boolean_t res;
         if (maturitycounter_.at(idn0) >= param_conf_.objectminmaturity) {
            res = boolean_t_true;
         }
         else{
             res = boolean_t_false;
         }

         return res;
     }

    C1Struct OffsetCalculator::calculatec1(const Object &obj,
                                           const basetype::objectid_t &idn0,
                                           const basetype::boolean_t &isrearobject,
                                           const basetype::float32_t &spd,
                                           const basetype::float32_t &c0) {
        C1Struct res{};
        basetype::float32_t angle{zero_float32_t};
        basetype::float32_t anglelimited{zero_float32_t};
        basetype::float32_t posnlgt{zero_float32_t};
        basetype::float32_t c1{zero_float32_t};

        if (isrearobject == boolean_t_false) {
            angle = obj.Est.HeadingAngle;
            posnlgt = obj.Est.DisLgt;
            anglelimited = Util::saturate(angle,
                                          param_conf_.maxheadingangleforyawplus,
                                          param_conf_.minheadingangleforyawplus);
            c1 = Util::safetydiv((2.0f * (std::tan(anglelimited) - 2.0 * c0 *posnlgt)),
                                                posnlgt * posnlgt, eps4_float32_t);
            
            basetype::float32_t assumnedmaxhostyaw{Util::lookup(param_conf_.assumedmaxhostyaw,
                                                                param_conf_.assumedmaxhostyawbreakpoints,
                                                                spd,
                                                                boolean_t_false) * pi / 180.0f};
            basetype::float32_t time{Util::abs(Util::safetydiv(angle,
                                                               assumnedmaxhostyaw, eps4_float32_t))};
            basetype::float32_t timegap{Util::safetydiv(posnlgt, spd, eps3_float32_t)};

            basetype::float32_t timefraction{Util::safetydiv(time, timegap, eps3_float32_t)};

            basetype::float32_t timefactor{Util::lookup(param_conf_.timefactor,
                                                        param_conf_.timefactorbreakpoints,
                                                        timefraction,
                                                        boolean_t_false)};
            basetype::float32_t speedfactor{Util::lookup(param_conf_.speedfactor,
                                                         param_conf_.speedfactorbreakpoints,
                                                         spd,
                                                         boolean_t_false)};

            basetype::float32_t timegapfactor{Util::lookup(param_conf_.mintimegap,
                                                           param_conf_.mintimegapbreakpoints,
                                                           timegap,
                                                           boolean_t_false)};

            basetype::float32_t targetdistancefactor{Util::lookup(param_conf_.mintatrdis,
                                                                  param_conf_.mintatrdisbreakpoints,
                                                                  posnlgt,
                                                                  boolean_t_false)};
            basetype::float32_t targetspeedfactor{Util::lookup(param_conf_.mintatrdis,
                                                               param_conf_.mintatrdisbreakpoints,
                                                               obj.Est.SpdLgt,
                                                               boolean_t_false)};

            basetype::float32_t targetanglestdfactor{Util::lookup(param_conf_.mintatranglestd,
                                                                  param_conf_.mintatranglestdbreakpoints,
                                                                  obj.Ppty.HeadingAngleStdDev,
                                                                  boolean_t_false)};

            res.c1factor = timefactor * speedfactor * timegapfactor * targetdistancefactor * targetspeedfactor * targetanglestdfactor;
            res.c1 = c1 * res.c1factor / 6.0f;
        } else {
            angle = -obj.Est.HeadingAngle;

            basetype::float32_t tarspeed{obj.Est.SpdLgt};

            anglelimited = Util::saturate(angle,
                                          param_conf_.maxheadingangleforyawplus,
                                          param_conf_.minheadingangleforyawplus);

            basetype::float32_t assumnedmaxhostyaw{Util::lookup(param_conf_.assumedmaxhostyaw,
                                                                param_conf_.assumedmaxhostyawbreakpoints,
                                                                tarspeed,
                                                                boolean_t_false) * pi / 180.0f};

            basetype::float32_t time{Util::abs(Util::safetydiv(angle,
                                                               assumnedmaxhostyaw, eps4_float32_t))};

            posnlgt = -obj.Est.DisLgt;
            
            c1 = Util::safetydiv((2.0f * (std::tan(anglelimited) - 2.0 * 0.0 *posnlgt)),
                                                posnlgt * posnlgt, eps4_float32_t);
 
            basetype::float32_t timegap{Util::safetydiv(posnlgt, tarspeed, eps3_float32_t)};

            basetype::float32_t timefraction{Util::safetydiv(time, timegap, eps3_float32_t)};

            basetype::float32_t timefactor{Util::lookup(param_conf_.timefactor,
                                                        param_conf_.timefactorbreakpoints,
                                                        timefraction,
                                                        boolean_t_false)};
            
            basetype::float32_t speedfactor{Util::lookup(param_conf_.speedfactor,
                                                         param_conf_.speedfactorbreakpoints,
                                                         tarspeed,
                                                         boolean_t_false)};

            basetype::float32_t timegapfactor{Util::lookup(param_conf_.mintimegap,
                                                           param_conf_.mintimegapbreakpoints,
                                                           timegap,
                                                           boolean_t_false)};

            basetype::float32_t targetdistancefactor{Util::lookup(param_conf_.mintatrdis,
                                                                  param_conf_.mintatrdisbreakpoints,
                                                                  posnlgt,
                                                                  boolean_t_false)};

            basetype::float32_t targetspeedfactor{Util::lookup(param_conf_.mintatrdis,
                                                               param_conf_.mintatrdisbreakpoints,
                                                               spd,
                                                               boolean_t_false)};

            res.c1factor = timefactor * speedfactor * timegapfactor * targetdistancefactor * targetspeedfactor;
            res.c1 = c1 * res.c1factor / 6.0f;
        }
        
        return res;
    }

    basetype::float32_t OffsetCalculator::calculateyawplusoffset(const Object &obj,
                                                                 const basetype::objectid_t &idn0,
                                                                 const basetype::boolean_t &isrearobject,
                                                                 const basetype::float32_t &c0,
                                                                 const basetype::float32_t &c1,
                                                                 const basetype::float32_t &c1factor) {
        basetype::float32_t res{zero_float32_t};

        basetype::float32_t anglelimited{zero_float32_t};
        basetype::float32_t anglefactor{zero_float32_t};
        basetype::float32_t x2{zero_float32_t};
        basetype::float32_t x3{zero_float32_t};
        basetype::float32_t newc1{zero_float32_t};
        if (isrearobject == boolean_t_false) {
            anglelimited = Util::saturate(obj.Est.HeadingAngle,
                                          param_conf_.maxheadingangleforyawplus,
                                          param_conf_.minheadingangleforyawplus);
            
            anglefactor = Util::lookup(param_conf_.headinganglefactor,
                                       param_conf_.headinganglefactorbreakpoints,
                                       obj.Est.DisLgt,
                                       boolean_t_false);
            anglelimited = anglelimited * anglefactor;
            x2 = obj.Est.DisLgt * obj.Est.DisLgt;
            x3 = x2 * obj.Est.DisLgt;
            newc1 = Util::safetydiv((std::tan(anglelimited) - 2.0f * c0 * obj.Est.DisLgt), 
                                    (3.0f * x2), eps3_float32_t) * c1factor;
            res = obj.Est.DisLat - c0 * x2 - newc1 * x3;
        } else {
            anglelimited = Util::saturate(-(obj.Est.HeadingAngle),
                                          param_conf_.maxheadingangleforyawplus,
                                          param_conf_.minheadingangleforyawplus);
            basetype::float32_t dis;
            dis = -obj.Est.DisLgt;
            anglefactor = Util::lookup(param_conf_.headinganglefactor,
                                       param_conf_.headinganglefactorbreakpoints,
                                       dis,
                                       boolean_t_false);
            anglelimited = anglelimited * anglefactor;
            x2 = dis * dis;
            x3 = x2 * dis;
            newc1 = Util::safetydiv((std::tan(anglelimited) - 2.0f * 0.0f * obj.Est.DisLgt), 
                                    (3.0f * x2), eps3_float32_t) * c1factor;
            res = obj.Est.DisLat - 0.0f * x2 - newc1 * x3;
        }
        // log c1
        logger_.logc1(newc1, idn0);

        return res;
    }

    basetype::float32_t OffsetCalculator::calculatefilterconstant(const basetype::float32_t dislgt) {
        basetype::float32_t res;
        basetype::float32_t curvature = Util::lookup(param_conf_.cutofffreqyawplus,
                                                     param_conf_.cutofffreqyawplusbreakpoints,
                                                     dislgt,
                                                     boolean_t_false);
        
        res = one_float32_t + (Util::safetydiv(one_float32_t, 2.0f * curvature * c_SampleTime * pi, eps5_float32_t));
        res = Util::safetydiv(one_float32_t, res, eps5_float32_t);
        
        return res;
    }

    basetype::float32_t OffsetCalculator::calculateoffsetderiv(const Object &obj,
                                                               const basetype::objectid_t idn0,
                                                               const basetype::float32_t offsetraw) {
        basetype::float32_t res;

        res = Util::saturate((offsetraw - savedoffset_.at(idn0))/c_SampleTime,
                             param_conf_.maxoffsetderiv,
                             param_conf_.minoffsetderiv);

        if (savedstatus_.at(idn0) == boolean_t_true) {
            res = Util::filter(res,
                               savedoffsetderiv_[idn0],
                               param_conf_.filtercontantoffsetderiv);
            savedoffsetderiv_[idn0] = res;
        } else {
            res = zero_float32_t;
            savedoffsetderiv_[idn0] = res;
        }

        if (obj.Ppty.TrackStatus > ObjectTrackStatus::UpdateNew) {
            savedstatus_[idn0] = boolean_t_true;
        }
        else{
            savedstatus_[idn0] = boolean_t_true;
        }
        
        savedoffset_[idn0] = offsetraw;
        
        return res;
    }
    void OffsetCalculator::adjustoffset(const Object &obj,
                                        const basetype::objectid_t &idn0,
                                        const basetype::boolean_t &isrearobject,
                                        const basetype::boolean_t &isoffsetfromlanecentervalid) {
        basetype::float32_t res;
        if ((isoffsetfromlanecentervalid == boolean_t_true) &&
            (obj.Info.Type != ObjectType::Unknown) &&
            (obj.Info.Type != ObjectType::UnknownVehicleClass) &&
            (obj.Info.Type != ObjectType::Motorcycle) &&
            (obj.Ppty.TrackStatus != ObjectTrackStatus::Invalid) &&
            (!(obj.Info.Type == ObjectType::Truck && obj.Ppty.FusionSource == ObjectFusionSource::RadarOnly)) &&
            (param_conf_.useobjectwidth == boolean_t_true) &&
            (isrearobject == boolean_t_false)) {
            basetype::float32_t posvalue = Util::lookup(param_conf_.objectwidth,
                                                        param_conf_.objectwidthbreakpoints,
                                                        obj.Est.DisLgt,
                                                        boolean_t_false);

            basetype::float32_t posfac = Util::lookup(param_conf_.objectwidthfactor,
                                                      param_conf_.objectwidthbreakpoints,
                                                      obj.Est.DisLgt,
                                                      boolean_t_false);

            basetype::int8_t signfac = Util::signfunc(OffsetCalculator::offset_, eps3_float32_t);
            if (signfac >= zero_int8_t) {
                res = Util::saturate(obj.Info.Width,
                                     static_cast<basetype::float32_t>(2.6f),
                                     static_cast<basetype::float32_t>(1.0f)) *
                                     posfac * (-posvalue);
            } else {
                res = Util::saturate(obj.Info.Width,
                                     static_cast<basetype::float32_t>(2.6f),
                                     static_cast<basetype::float32_t>(1.0f)) *
                                     posfac * (posvalue);
            }
            
            

            OffsetCalculator::offset_ = OffsetCalculator::offset_ + res;
        }
    }

    void OffsetCalculator::calculate(const Object &obj,
                                     const basetype::boolean_t &isrearobject,
                                     const basetype::boolean_t &isoffsetfromlanecenterusable,
                                     const basetype::boolean_t &isoffsetfromlanecentervalid,
                                     const basetype::float32_t &laneoffset,
                                     const EgoVehicleInformation &egovehinfo,
                                     const basetype::float32_t &c0,
                                     const basetype::boolean_t &useyawinlanechange) {
        basetype::objectid_t index{obj.Ppty.Id - one_objectid_t};
        OffsetCalculator::updatematurity(index);

        basetype::float32_t timegap{Util::safetydiv(obj.Est.DisLgt, egovehinfo.SpdLgt, eps3_float32_t)};

        basetype::float32_t filterconstant{OffsetCalculator::calculatefilterconstant(obj.Est.DisLgt)};

        C1Struct c1{OffsetCalculator::calculatec1(obj, index, isrearobject, egovehinfo.SpdLgt, c0)};

        basetype::boolean_t useoffsetfromlanecenter{boolean_t_true};

        OffsetCalculator::checkuselanecenteroffset(obj.Ppty.DisLatFromLaneCenterQf,
                                                   isoffsetfromlanecenterusable,
                                                   timegap,
                                                   useyawinlanechange,
                                                   useoffsetfromlanecenter,
                                                   OffsetCalculator::highqualityoffsetfromlanecenter_);
        
        // log useoffsetfromlanecenter
        logger_.loguseoffsetfromlanecenter(useoffsetfromlanecenter, index);

        basetype::float32_t offsetraw;
        if (useoffsetfromlanecenter == boolean_t_true) {
            offset_ = obj.Info.DisLatFromLaneCenter - laneoffset;
            offsetraw = offset_;
        } else {
            offset_ = OffsetCalculator::calculateyawplusoffset(obj,
                                                               index,
                                                               isrearobject,
                                                               c0,
                                                               zero_float32_t,
                                                               c1.c1factor);
            offsetraw = offset_;

            if (maturitycounter_.at(index) > one_uint8_t) {
                offset_ = Util::filter(offset_,
                                       OffsetCalculator::offsetfilterstates_[index],
                                       filterconstant);
            }
            
        }
        offsetfilterstates_.at(index) = offset_;

        offsetderiv_ = OffsetCalculator::calculateoffsetderiv(obj,index,offsetraw);

        // log offsetraw
        logger_.logoffsetraw(offsetraw, index);

        OffsetCalculator::adjustoffset(obj, index, isrearobject, isoffsetfromlanecentervalid);

        // log offset offsetderiv
        logger_.logoffset(offset_, index);
    }

    void OffsetCalculator::reset(const basetype::objectid_t &idn0) {
        maturitycounter_.at(idn0) = zero_uint8_t;
        savedoffset_.at(idn0) = zero_float32_t;
        savedoffsetderiv_.at(idn0) = zero_float32_t;
    }

    basetype::float32_t OffsetCalculator::getoffset() {
        return offset_;
    }

    basetype::float32_t OffsetCalculator::getoffsetderiv() {
        return offsetderiv_;
    }

    basetype::boolean_t OffsetCalculator::gethighqualityoffsetfromlanecenter() {
        return highqualityoffsetfromlanecenter_;
    }

}