#ifndef ACC_TARGET_SELECTION_OFFSETCALCULATOR_H
#define ACC_TARGET_SELECTION_OFFSETCALCULATOR_H

#include <array>
#include <memory>

#include "basetype.h"
#include "param.h"

#include "logger.h"
#include "egovehicleinformation.h"
#include "road.h"
#include "object.h"

namespace ACC_Target_Selection {
    struct C1Struct {
        basetype::float32_t c1;
        basetype::float32_t c1factor;
        explicit C1Struct() :
            c1(zero_float32_t),
            c1factor(zero_float32_t) {
        }
        C1Struct(const C1Struct &r) :
                c1(r.c1),
                c1factor(r.c1factor) {
        }
        explicit C1Struct(const basetype::float32_t &c1_,
                          const basetype::float32_t &c1factor_):
                c1(c1_),
                c1factor(c1factor_) {
        }
        C1Struct& operator=(const C1Struct& rhs) {
            c1 = rhs.c1;
            c1factor = rhs.c1factor;
            return *this;
        }
    };
    
    class OffsetCalculator {
    public:
        const Param_OffsetCalculator &param_conf_;
        const Logger &logger_;

        explicit OffsetCalculator(const Param_OffsetCalculator &param_conf, const Logger &logger);
        OffsetCalculator(const OffsetCalculator& r);
        ~OffsetCalculator();

        void calculate(const Object &obj,
                       const basetype::boolean_t &isrearobject,
                       const basetype::boolean_t &isoffsetfromlanecenterusable,
                       const basetype::boolean_t &isoffsetfromlanecentervalid,
                       const basetype::float32_t &laneoffset,
                       const EgoVehicleInformation &egovehinfo,
                       const basetype::float32_t &c0,
                       const basetype::boolean_t &useyawinlanechange);

        void reset(const basetype::objectid_t &idn0);

        basetype::float32_t getoffset();

        basetype::float32_t getoffsetderiv();

        basetype::boolean_t gethighqualityoffsetfromlanecenter();
    private:
        std::array<basetype::uint8_t, c_ObjectSize> maturitycounter_;
        std::array<basetype::float32_t, c_ObjectSize> offsetfilterstates_;
        std::array<basetype::float32_t, c_ObjectSize> savedoffsetderiv_;
        std::array<basetype::float32_t, c_ObjectSize> savedoffset_;
        std::array<basetype::boolean_t, c_ObjectSize> savedstatus_;
        basetype::float32_t offset_;
        basetype::float32_t offsetderiv_;
        basetype::boolean_t highqualityoffsetfromlanecenter_;
        
        void checkuselanecenteroffset(const ObjectDisLatFromLaneCenterQf &quality,
                                      const basetype::boolean_t &isoffsetfromlanecenterusable,
                                      const basetype::float32_t &timegap,
                                      const basetype::boolean_t &useyawinlanechange,
                                      basetype::boolean_t &useoffsetfromlanecenter,
                                      basetype::boolean_t &highqualityoffsetfromlanecenter);

        void updatematurity(const basetype::objectid_t &idn0);

        basetype::boolean_t ismaturity(const basetype::objectid_t &idn0);

        C1Struct calculatec1(const Object &obj,
                             const basetype::objectid_t &idn0,
                             const basetype::boolean_t &isrearobject,
                             const basetype::float32_t &spd,
                             const basetype::float32_t &c0);

        basetype::float32_t calculateyawplusoffset(const Object &obj,
                                                   const basetype::objectid_t &idn0,
                                                   const basetype::boolean_t &isrearobject,
                                                   const basetype::float32_t &c0,
                                                   const basetype::float32_t &c1,
                                                   const basetype::float32_t &c1factor);
        
        basetype::float32_t calculatefilterconstant(const basetype::float32_t dislgt);

        basetype::float32_t calculateoffsetderiv(const Object &obj,
                                                 const basetype::objectid_t idn0,
                                                 const basetype::float32_t offsetraw);

        void adjustoffset(const Object &obj,
                          const basetype::objectid_t &idn0,
                          const basetype::boolean_t &isrearobject,
                          const basetype::boolean_t &isoffsetfromlanecentervalid);

    };

} // namespace ACC_Target_Selection

#endif