#ifndef ACC_TARGET_SELECTION_BASETYPE_H
#define ACC_TARGET_SELECTION_BASETYPE_H

namespace basetype{
    
/* Logical type definitions */
#if (!defined(__cplusplus))
#  ifndef false
#   define false                       (0U)
#  endif

#  ifndef true
#   define true                        (1U)
#  endif
#endif

/*=======================================================================*
 * Target hardware information
 *   Device type: Intel->x86-64 (Windows64)
 *   Number of bits:     char:   8    short:   16    int:  32
 *                       long:  32    long long:  64
 *                       native word size:  64
 *   Byte ordering: LittleEndian
 *   Signed integer division rounds to: Zero
 *   Shift right on a signed integer as arithmetic shift: on
 *=======================================================================*/
#ifdef PORTABLE_WORDSIZES              /* PORTABLE_WORDSIZES defined */

/*=======================================================================*
 * Host information
 *   Number of bits:     char:   8    short:   16    int:  32
 *                       long:  64
 *                       long long:  64
 *                       native word size:  64
 *=======================================================================*/

/*=======================================================================*
 * Fixed width word size data types:                                     *
 *   int8_t, int16_t, int32_t     - signed 8, 16, or 32 bit integers     *
 *   uint8_t, uint16_t, uint32_t  - unsigned 8, 16, or 32 bit integers   *
 *   real32_t, real64_t           - 32 and 64 bit floating point numbers *
 *=======================================================================*/
typedef signed char int8_t;
typedef unsigned char uint8_t;
typedef short int16_t;
typedef unsigned short uint16_t;
typedef int int32_t;
typedef unsigned int uint32_t;
typedef float float32_t;
typedef double float64_t;

/*===========================================================================*
 * Generic type definitions: boolean_t, char_t, byte_t, int_t, uint_t,       *
 *                           real_t, time_t, ulong_t, ulonglong_t.           *
 *===========================================================================*/
typedef double real_t;
typedef double time_t;
typedef unsigned char boolean_t;
typedef int int_t;
typedef unsigned int uint_t;
typedef unsigned int ulong_t;
typedef unsigned long ulonglong_t;
typedef char char_t;
typedef unsigned char uchar_t;
typedef char_t byte_t;
typedef long int64_t;
typedef unsigned long uint64_t;

#else                                  /* PORTABLE_WORDSIZES not defined */

/*=======================================================================*
 * Fixed width word size data types:                                     *
 *   int8_T, int16_T, int32_T     - signed 8, 16, or 32 bit integers     *
 *   uint8_T, uint16_T, uint32_T  - unsigned 8, 16, or 32 bit integers   *
 *   real32_T, real64_T           - 32 and 64 bit floating point numbers *
 *=======================================================================*/
typedef signed char int8_t;
typedef unsigned char uint8_t;
typedef short int16_t;
typedef unsigned short uint16_t;
typedef int int32_t;
typedef unsigned int uint32_t;
typedef long long int64_t;
typedef unsigned long long uint64_t;
typedef float float32_t;
typedef double float64_t;

/*===========================================================================*
 * Generic type definitions: boolean_T, char_T, byte_T, int_T, uint_T,       *
 *                           real_T, time_T, ulong_T, ulonglong_T.           *
 *===========================================================================*/
typedef double real_t;
typedef double time_t;
typedef unsigned char boolean_t;
typedef int int_t;
typedef unsigned int uint_t;
typedef unsigned long ulong_t;
typedef unsigned long long ulonglong_t;
typedef char char_t;
typedef unsigned char uchar_t;
typedef char_t byte_t;


typedef unsigned int objectid_t;
#endif                                 /* PORTABLE_WORDSIZES */
} // namespace basetype

#endif