#include "road.h"

#include "atsutil.h"
namespace ACC_Target_Selection{
    Road::Road(const Param_Road &param_conf, const Logger  &logger):
            param_conf_(param_conf),
            logger_(logger),
            offset_(zero_float32_t),
            landwidth_(zero_float32_t),
            isoffsetfromlanecenterusable_(boolean_t_false),
            isoffsetfromlanecentervalid_(boolean_t_false) {
    }

    Road::Road(const Road& r):
            param_conf_(r.param_conf_),
            logger_(r.logger_),
            offset_(r.offset_),
            landwidth_(r.landwidth_),
            isoffsetfromlanecenterusable_(r.isoffsetfromlanecenterusable_),
            isoffsetfromlanecentervalid_(r.isoffsetfromlanecentervalid_) {
    }

    Road::~Road() {
    }

    basetype::boolean_t Road::checkoffsetlat() {
        basetype::boolean_t res{boolean_t_false};
        if (Util::abs(offset_) < param_conf_.maxdistancefromlanecenter) {
            res = boolean_t_true;
        } else {
            res = boolean_t_false;
        }
        
        return res;
    }

    basetype::float32_t Road::getlaneoffset() {
        basetype::float32_t res{zero_float32_t};
        if ((isoffsetfromlanecenterusable_ == boolean_t_true) &&
            (Road::checkoffsetlat() == boolean_t_true) &&
            (param_conf_.useadjustfunnelwithposinlane == boolean_t_true)) {
            res = offset_;
        } else {
            res = zero_float32_t;
        }
        return res;
    }

    basetype::float32_t Road::getoffset() {
        return offset_;
    }

    basetype::float32_t Road::getlanewidth(const basetype::float32_t &lgtdis) {
        basetype::float32_t res{param_conf_.defaultlanewidth};
        if ((isoffsetfromlanecenterusable_) &&
            (Road::checkoffsetlat() == boolean_t_true) &&
            (param_conf_.uselanewidthforfunnel == boolean_t_true) &&
            (lgtdis > param_conf_.mindistanceforusinglanewidth)) {
            res = landwidth_;
        } else {
            res = param_conf_.defaultlanewidth;
        }

        return res;
    }

    basetype::boolean_t Road::getisoffsetfromlanecenterusable() {
        return isoffsetfromlanecenterusable_;
    }

    basetype::boolean_t Road::getisoffsetfromlanecentervalid() {
        basetype::boolean_t res = Road::isoffsetfromlanecentervalid_;
        return res;
    }

    void Road::set(const RoadProperty &roadproperty, const EgoVehicleInformation &egovehinfo) {
        isoffsetfromlanecentervalid_ = roadproperty.Valid;
        if ((egovehinfo.SpdLgt > param_conf_.minspeedforusingdistancefromlanecenter) &&
            (isoffsetfromlanecentervalid_ == boolean_t_true)) {
            isoffsetfromlanecenterusable_ = boolean_t_true;
        } else {
            isoffsetfromlanecenterusable_ = boolean_t_false;
        }

        landwidth_ = roadproperty.LaneWidth;
        offset_ = roadproperty.LateralOffset;
        
    }

}
