#ifndef ACC_TARGET_SELECTION_SELECTOR_H
#define ACC_TARGET_SELECTION_SELECTOR_H

#include <array>
#include <memory>

#include "basetype.h"
#include "param.h"

#include "c0calculator.h"
#include "validator.h"
#include "offsetcalculator.h"
#include "funnel.h"
#include "road.h"
#include "logger.h"
#include "objectlanestatus.h"
#include "lanechangetype.h"
#include "atsutil.h"

namespace ACC_Target_Selection {
    class BooleanSignal;
    class Selector {
    public:
        const Param_Selector &param_conf_;
        const Logger &logger_;
        explicit Selector(const Param_Selection &param_conf, const Logger &logger);
        Selector(const Selector& r);
        ~Selector();

        void update(const std::array<Object, c_ObjectSize> &objects,
                    const EgoVehicleInformation &egovehinfo,
                    const RoadProperty &roadproperty,
                    const basetype::boolean_t &leftindicator,
                    const basetype::boolean_t &rightindicator,
                    const LaneChangeType &lanechange);
        
        void reset(const basetype::objectid_t idn0);

        basetype::objectid_t getiv();

        basetype::objectid_t getiiv();

        basetype::objectid_t getfliv();

        basetype::objectid_t getsliv();

        basetype::objectid_t getfliiv();

        basetype::objectid_t getsliiv();

        basetype::objectid_t getcutin();

        basetype::float32_t getivoffset();

        basetype::float32_t getiivoffset();

        basetype::float32_t getflivoffset();

        basetype::float32_t getslivoffset();

        basetype::float32_t getfliivoffset();

        basetype::float32_t getsliivoffset();

        basetype::float32_t getcutinoffset();

        basetype::objectid_t getriv();

        basetype::objectid_t getriiv();

        basetype::objectid_t getrfliv();

        basetype::objectid_t getrsliv();

        basetype::objectid_t getrfliiv();

        basetype::objectid_t getrsliiv();

        basetype::objectid_t getrcutin();

        basetype::float32_t getrivoffset();

        basetype::float32_t getriivoffset();

        basetype::float32_t getrflivoffset();

        basetype::float32_t getrslivoffset();

        basetype::float32_t getrfliivoffset();

        basetype::float32_t getrsliivoffset();

        basetype::float32_t getrcutinoffset();

        basetype::int8_t getivquality();

        ObjectLaneStatus getivcuttingoutsts();

        basetype::boolean_t getivcuttingout();

    private:
        C0Calculator c0calculator_;
        Validator validator_;
        OffsetCalculator offsetcalculator_;
        Funnel funnel_;
        Road road_;

        basetype::objectid_t pedestrain_;
        basetype::objectid_t iv_;
        basetype::objectid_t iiv_;
        basetype::objectid_t cutin_;
        basetype::objectid_t fliv_;
        basetype::objectid_t sliv_;
        basetype::objectid_t fliiv_;
        basetype::objectid_t sliiv_;
        basetype::objectid_t riv_;
        basetype::objectid_t riiv_;
        basetype::objectid_t rcutin_;
        basetype::objectid_t rfliv_;
        basetype::objectid_t rsliv_;
        basetype::objectid_t rfliiv_;
        basetype::objectid_t rsliiv_;
        basetype::int8_t ivquality_;
        basetype::boolean_t ivcutingout_;
        ObjectLaneStatus ivcutoutstatus_;
        basetype::float32_t ivoffset_;
        basetype::float32_t iivoffset_;
        basetype::float32_t cutinoffset_;
        basetype::float32_t flivoffset_;
        basetype::float32_t slivoffset_;
        basetype::float32_t fliivoffset_;
        basetype::float32_t sliivoffset_;
        basetype::float32_t rivoffset_;
        basetype::float32_t riivoffset_;
        basetype::float32_t rcutinoffset_;
        basetype::float32_t rflivoffset_;
        basetype::float32_t rslivoffset_;
        basetype::float32_t rfliivoffset_;
        basetype::float32_t rsliivoffset_;
        std::array<basetype::uint8_t, c_ObjectSize> inpathcounter_;
        std::array<basetype::uint8_t, c_ObjectSize> inwidepathcounter_;
        std::array<basetype::uint8_t, c_ObjectSize> cutinpathcounters_;
        std::array<basetype::uint8_t, c_ObjectSize> cutinholdcounters_;
        basetype::float32_t predictionmaxdistance_;
        Util::BooleanSignal yawplusafterindicatortimer_;
        Util::BooleanSignal yawplusafterlanechangetimer_;

        basetype::boolean_t checkpredicition(const Object &obj);

        basetype::boolean_t checkcutout(const basetype::objectid_t &idn0,
                                        const basetype::boolean_t &isrearobject,
                                        const basetype::float32_t &dislgt,
                                        const basetype::float32_t &spd,
                                        const basetype::float32_t &offset,
                                        const basetype::float32_t &offsetderiv);

        basetype::boolean_t checkinpath(const basetype::objectid_t &idn0,
                                        const basetype::boolean_t &isrearobject,
                                        const basetype::float32_t &offset,
                                        FunnelWidthParam &funnelwidthparam);

        basetype::boolean_t checkinwidepath(const basetype::objectid_t &idn0,
                                            const basetype::boolean_t &isrearobject,
                                            const basetype::float32_t &offset,
                                            FunnelWidthParam &funnelwidthparam);

        basetype::boolean_t checkcutin(const basetype::uint8_t &idn0,
                                       const basetype::boolean_t &isrearobject,
                                       const basetype::boolean_t &useprediction,
                                       const basetype::float32_t &predictedoffset);

        void resetassignment();
        
        basetype::boolean_t isrearobject(const Object &obj);

    };
    
}
#endif