#include "atsutil.h"
#include <iostream>

namespace ACC_Target_Selection {
namespace Util {
BooleanSignal::BooleanSignal(basetype::boolean_t val, basetype::float32_t sampletime) :
        sampletime_(sampletime),
        val_(val),
        timer_(sampletime * 150000.0f) {
}

BooleanSignal::BooleanSignal(const BooleanSignal& r):
        sampletime_(r.sampletime_),
        val_(r.val_),
        timer_(r.timer_) {
}

BooleanSignal::~BooleanSignal() {
}

void BooleanSignal::reset(basetype::boolean_t val) {
    BooleanSignal::val_ = val;
    BooleanSignal::timer_ = BooleanSignal::sampletime_ * 150000.0f;
}

void BooleanSignal::update(basetype::boolean_t val) {
    if (val == BooleanSignal::val_) {
        if (BooleanSignal::timer_ < BooleanSignal::sampletime_ * 150000.0f) {
            BooleanSignal::timer_ = BooleanSignal::timer_ + BooleanSignal::sampletime_;
        } else{
        }
    } else {
        BooleanSignal::timer_ = zero_float32_t;
    }

    BooleanSignal::val_ = val;
}

basetype::boolean_t BooleanSignal::isrising() {
    basetype::boolean_t res{boolean_t_false};
    if ((BooleanSignal::val_ == boolean_t_true) &&
        (BooleanSignal::timer_ < BooleanSignal::sampletime_)) {
        res = boolean_t_true;
    } else{
        res = boolean_t_false;
    }
    
    return res;
}

basetype::boolean_t BooleanSignal::isfalling() {
    basetype::boolean_t res{boolean_t_false};
    if ((BooleanSignal::val_ == boolean_t_false) &&
        (BooleanSignal::timer_ < BooleanSignal::sampletime_)) {
        res = boolean_t_true;
    } else {
        res = boolean_t_false;
    }
    
    return res;
}

basetype::boolean_t BooleanSignal::value() {
    return BooleanSignal::val_;
}

basetype::boolean_t BooleanSignal::previousvalue() {
    basetype::boolean_t res{boolean_t_false};
    if ((BooleanSignal::val_ = boolean_t_true) &&
        (BooleanSignal::isrising() == boolean_t_false)) {
        res = boolean_t_true;
    } else if (BooleanSignal::isfalling() == boolean_t_true) {
        res = boolean_t_true;
    } else {
        res = boolean_t_false;
    }
    
    return res;
}

basetype::boolean_t BooleanSignal::hold(basetype::boolean_t holdvalue, basetype::float32_t holdtime) {
    basetype::boolean_t res{boolean_t_false};
    if ((BooleanSignal::val_ == holdvalue) ||
        (BooleanSignal::timer_ < (holdtime -static_cast<basetype::float32_t>(0.5f)*BooleanSignal::sampletime_))) {
        res = holdvalue;
    } else {
        if (holdvalue == boolean_t_true) {
            res = boolean_t_false;
        }
        else{
            res = boolean_t_true;
        }
    }

    return res;
}

basetype::boolean_t BooleanSignal::delay(basetype::boolean_t delayvalue, basetype::float32_t delaytime) {
    basetype::boolean_t res;
    if ((BooleanSignal::val_ == delayvalue) &&
        (BooleanSignal::timer_ >= delaytime)) {
        res = delayvalue;
    } else {
        if (delayvalue == boolean_t_true) {
            res = boolean_t_false;
        } else {
            res = boolean_t_true;
        }
    }
    
    return res;
}

basetype::float32_t safedivide(const basetype::float32_t &nominator,
                               const basetype::float32_t &denominator,
                               const basetype::float32_t &threshold) {
    basetype::float32_t res{zero_float32_t};
    if ((denominator < threshold) &&
        (denominator > -threshold)) {
        res = nominator / threshold;
        if (nominator < zero_float32_t) {
            res = -res;
        } else {
        }
    } else {
        res = nominator / denominator;
    }

    return res;
}

basetype::float32_t filter(const basetype::float32_t &value,
                           const basetype::float32_t &previousvalue,
                           const basetype::float32_t &tsovertau) {
    return (value * tsovertau + previousvalue * (one_float32_t - tsovertau));
}

basetype::float32_t min(const basetype::float32_t &value1,
                        const basetype::float32_t &value2) {
    return ((value1 < value2) ? value1 : value2);
}

basetype::float64_t min(const basetype::float64_t &value1,
                        const basetype::float64_t &value2) {
    return ((value1 < value2) ? value1 : value2);
}

basetype::uint8_t min(const basetype::uint8_t &value1,
                      const basetype::uint8_t &value2) {
    return ((value1 < value2) ? value1 : value2);
}

basetype::uint16_t min(const basetype::uint16_t &value1,
                       const basetype::uint16_t &value2) {
    return ((value1 < value2) ? value1 : value2);
}

basetype::uint32_t min(const basetype::uint32_t &value1,
                       const basetype::uint32_t &value2) {
    return ((value1 < value2) ? value1 : value2);
}

basetype::uint64_t min(const basetype::uint64_t &value1,
                       const basetype::uint64_t &value2) {
    return ((value1 < value2) ? value1 : value2);
}

basetype::int8_t min(const basetype::int8_t &value1,
                     const basetype::int8_t &value2) {
    return ((value1 < value2) ? value1 : value2);
}

basetype::int16_t min(const basetype::int16_t &value1,
                      const basetype::int16_t &value2) {
    return ((value1 < value2) ? value1 : value2);
}

basetype::int32_t min(const basetype::int32_t &value1,
                      const basetype::int32_t &value2) {
    return ((value1 < value2) ? value1 : value2);
}

basetype::int64_t min(const basetype::int64_t &value1,
                      const basetype::int64_t &value2) {
    return ((value1 < value2) ? value1 : value2);
}

basetype::float32_t max(const basetype::float32_t &value1,
                        const basetype::float32_t &value2) {
    return ((value1 > value2) ? value1 : value2);
}

basetype::float64_t max(basetype::float64_t &value1,
                        basetype::float64_t &value2) {
    return ((value1 > value2) ? value1 : value2);
}

basetype::uint8_t max(basetype::uint8_t &value1,
                      basetype::uint8_t &value2) {
    return ((value1 > value2) ? value1 : value2);
}

basetype::uint16_t max(basetype::uint16_t &value1,
                       basetype::uint16_t &value2) {
    return ((value1 > value2) ? value1 : value2);
}

basetype::uint32_t max(basetype::uint32_t &value1,
                       basetype::uint32_t &value2) {
    return ((value1 > value2) ? value1 : value2);
}

basetype::uint64_t max(basetype::uint64_t &value1,
                       basetype::uint64_t &value2) {
    return ((value1 > value2) ? value1 : value2);
}

basetype::int8_t max(basetype::int8_t &value1,
                     basetype::int8_t &value2) {
    return ((value1 > value2) ? value1 : value2);
}

basetype::int16_t max(const basetype::int16_t &value1,
                      const basetype::int16_t &value2) {
    return ((value1 > value2) ? value1 : value2);
}

basetype::int32_t max(const basetype::int32_t &value1,
                      const basetype::int32_t &value2) {
    return ((value1 > value2) ? value1 : value2);
}

basetype::int64_t max(const basetype::int64_t &value1,
                      const basetype::int64_t &value2) {
    return ((value1 > value2) ? value1 : value2);
}

basetype::float32_t abs(const basetype::float32_t &value) {
    basetype::float32_t res{value};
    if (value < zero_float32_t) {
        res = -res;
    } else {
    }
    return res;
}

basetype::float64_t abs(const basetype::float64_t &value) {
    basetype::float64_t res{value};
    if (value < zero_float64_t) {
        res = -res;
    } else {
    }
    return res;
}

basetype::int8_t abs(const basetype::int8_t &value) {
    basetype::int8_t res{value};
    if (value < zero_int8_t) {
        res = -res;
    } else {
    }
    return res;
}

basetype::int16_t abs(const basetype::int16_t &value) {
    basetype::int16_t res{value};
    if (value < zero_int16_t) {
        res = -res;
    } else {
    }
    return res;
}

basetype::int32_t abs(const basetype::int32_t &value) {
    basetype::int32_t res{value};
    if (value < zero_int32_t) {
        res = -res;
    } else {
    }
    return res;
}

basetype::int64_t abs(const basetype::int64_t &value) {
    basetype::int32_t res{value};
    if (value < zero_int64_t) {
        res = -res;
    } else {
    }
    return res;
}

basetype::int8_t signfunc(const basetype::float32_t &value,
                          const basetype::float32_t &threshold) {
    basetype::int8_t res{zero_int8_t};
    if (value > threshold) {
        res = one_int8_t;
    }
    else if (value < (threshold * -1.0f)) {
        res =  -one_int8_t;
    } else {
        res = zero_int8_t;
    }
    
    return res;
}
    
basetype::int8_t signfunc(const basetype::float32_t &value) {
    basetype::float32_t threshold{eps4_float32_t};
    basetype::int8_t res{zero_int8_t};
    res = signfunc(value, threshold);
    
    return res;
}

basetype::float32_t safetydiv(const basetype::float32_t &nominator,
                              const basetype::float32_t &denominator,
                              const basetype::float32_t &threshold) {
    basetype::float32_t res{zero_float32_t};
    basetype::int8_t signfac{Util::signfunc(denominator, threshold)};
    if (signfac == zero_int8_t) {
        res = nominator / threshold;
        res = res * static_cast<basetype::float32_t>(signfac);
    } else {
       res = nominator / denominator; 
    }
    
    return res;
}

basetype::float32_t lookup(const std::vector<basetype::float32_t> &data,
                           const std::vector<basetype::float32_t> &breakpoints,
                           const basetype::float32_t &value,
                           const basetype::boolean_t &extrapolate) {
    //static_assert((data->size() == breakpoints->size() && data->size() > static_cast<std::size_t>(2u)), "data and breakpoints are not same size");
    basetype::float32_t res{zero_float32_t};
    PreLoolkUpParam preloolkupparam{};
    preloolkup(breakpoints, value, extrapolate, preloolkupparam);
    res = lookuprun(data, preloolkupparam);
    return res;
}

basetype::uint8_t lookup_u8_float(const std::vector<basetype::uint8_t> &data,
                                  const std::vector<basetype::float32_t> &breakpoints,
                                  const basetype::float32_t &value,
                                  const basetype::boolean_t &extrapolate) {
    //static_assert((data->size() == breakpoints->size() && data->size() > static_cast<std::size_t>(2u)), "data and breakpoints are not same size");
    basetype::uint8_t res{zero_uint8_t};
    PreLoolkUpParam preloolkupparam{};
    preloolkup_u8_float(breakpoints, value, extrapolate, preloolkupparam);
    res = lookuprun_u8_float(data, preloolkupparam);
    return res;
}

void preloolkup(const std::vector<basetype::float32_t> &breakpoints,
                const basetype::float32_t &value,
                const basetype::boolean_t &extrapolate,
                PreLoolkUpParam &preloolkupparam) {
    std::size_t maxsize {breakpoints.size()};
    if (maxsize <= 1) {
        preloolkupparam.interval = static_cast<std::size_t>(0u);
        preloolkupparam.fraction = zero_float32_t;
    } else {
        if (value <= breakpoints.at(0)) {
            preloolkupparam.interval = static_cast<std::size_t>(0u);
        }
        else if (value >= breakpoints.at(maxsize - 1)) {
            preloolkupparam.interval = maxsize - static_cast<std::size_t>(2u);
        }
        else{
            std::size_t upperidx{maxsize - 1};
            std::size_t loweridx{static_cast<std::size_t>(0u)};
            std::size_t middleidx{loweridx};

            while ((upperidx - static_cast<std::size_t>(1u)) > loweridx) {
                middleidx = (upperidx + loweridx) >> 1;

                if (value <= breakpoints.at(middleidx)) {
                    upperidx = middleidx;
                } else {
                    loweridx = middleidx;
                }
            }

            preloolkupparam.interval = loweridx;
        }

        if ((value <= breakpoints.at(0)) && (extrapolate == boolean_t_false)) {
            preloolkupparam.fraction = 0.0f;
        } else if ((value >= breakpoints.at(maxsize - 1)) && (extrapolate == boolean_t_false)) {
            preloolkupparam.fraction = 1.0f;
        } else {
            preloolkupparam.fraction = safetydiv((value - breakpoints.at(preloolkupparam.interval)),
                                                 (breakpoints.at(preloolkupparam.interval + 1) - breakpoints.at(preloolkupparam.interval)));
        }
    }  
}

void preloolkup_u8_float(const std::vector<basetype::float32_t> &breakpoints,
                         const basetype::float32_t &value,
                         const basetype::boolean_t &extrapolate,
                         PreLoolkUpParam &preloolkupparam) {
    std::size_t maxsize{breakpoints.size()};
    if (maxsize <= 1) {
        preloolkupparam.interval = static_cast<std::size_t>(0u);
        preloolkupparam.fraction = zero_float32_t;
    } else {
        if (value <= breakpoints.at(0)) {
            preloolkupparam.interval = static_cast<std::size_t>(0u);
        } else if (value >= breakpoints.at(maxsize - 1)) {
            preloolkupparam.interval = maxsize - static_cast<std::size_t>(2u);
        } else {
            std::size_t upperidx{maxsize - 1};
            std::size_t loweridx{static_cast<std::size_t>(0u)};
            std::size_t middleidx{loweridx};

            while ((upperidx - static_cast<std::size_t>(1u)) > loweridx) {
                middleidx = (upperidx + loweridx) >> 1;

                if (value <= breakpoints.at(middleidx)) {
                    upperidx = middleidx;
                } else {
                    loweridx = middleidx;
                }
            }

            preloolkupparam.interval = loweridx;
        }

        if ((value <= breakpoints.at(0)) && (extrapolate == boolean_t_false)) {
            preloolkupparam.fraction = 0.0f;
        } else if ((value >= breakpoints.at(maxsize - 1)) && (extrapolate == boolean_t_false)) {
            preloolkupparam.fraction = 1.0f;
        } else {
            preloolkupparam.fraction = safetydiv((value - breakpoints.at(preloolkupparam.interval)),
                                                 (breakpoints.at(preloolkupparam.interval + 1) - breakpoints.at(preloolkupparam.interval)));
        }
    }
       
}
    
basetype::float32_t lookuprun(const std::vector<basetype::float32_t> &data,
                              const PreLoolkUpParam &preloolkupparam) {
    basetype::float32_t res{zero_float32_t};
    std::size_t idx1{preloolkupparam.interval};
    std::size_t idx2{idx1 + static_cast<std::size_t>(1)};
    res = data.at(idx1) + (data.at(idx2) - data.at(idx1)) * (preloolkupparam.fraction);
    return res;
}

basetype::uint8_t lookuprun_u8_float(const std::vector<basetype::uint8_t> &data, const PreLoolkUpParam &preloolkupparam) {
    basetype::uint8_t res{zero_uint8_t};
    std::size_t idx1{preloolkupparam.interval};
    std::size_t idx2{idx1 + static_cast<std::size_t>(1)};
    basetype::float32_t val1{static_cast<basetype::float32_t>(data.at(idx2))};
    basetype::float32_t val2{static_cast<basetype::float32_t>(data.at(idx1))};
    res = data.at(idx1) + static_cast<basetype::uint8_t>((val1 - val2)) * (preloolkupparam.fraction);
    return res;
}

basetype::uint8_t limitchange(const basetype::uint8_t &newvalue, 
                              const basetype::uint8_t &oldvalue,
                              const basetype::uint8_t &upperlimit,
                              const basetype::uint8_t &lowerlimit) {
    basetype::uint8_t res{newvalue};
    basetype::uint8_t uppervalue{oldvalue + upperlimit};
    basetype::uint8_t lowervalue{oldvalue - lowerlimit};

    if (newvalue > uppervalue) {
        res = uppervalue;
    } else if (newvalue < lowervalue) {
        res = lowervalue;
    } else {
        res = newvalue;
    }
    return res;
}

basetype::float32_t limitchange(const basetype::float32_t &newvalue, 
                                const basetype::float32_t &oldvalue,
                                const basetype::float32_t &upperlimit,
                                const basetype::float32_t &lowerlimit) {
    basetype::float32_t res{newvalue};
    basetype::float32_t uppervalue{oldvalue + upperlimit};
    basetype::float32_t lowervalue{oldvalue - lowerlimit};
    
    if (newvalue > uppervalue) {
        res = uppervalue;
    } else if (newvalue < lowervalue) {
        res = lowervalue;
    } else {
        res = newvalue;
    }
    return res;
}

basetype::uint8_t saturate(const basetype::uint8_t &value,
                           const basetype::uint8_t &upperlimit,
                           const basetype::uint8_t &lowerlimit) {
    basetype::uint8_t res{value};
    if (value > upperlimit) {
        res = upperlimit;
    } else if (value < lowerlimit) {
        res = lowerlimit;
    } else {
        res = value;
    }
    return res;
}

basetype::float32_t saturate(const basetype::float32_t &value,
                             const basetype::float32_t &upperlimit,
                             const basetype::float32_t &lowerlimit) {
    basetype::float32_t res{value};
    if (value > upperlimit) {
        res = upperlimit;
    } else if(value < lowerlimit) {
        res = lowerlimit;
    } else {
        res = value;
    }
    return res;
}

} // namespace Util
} // namespace ACC_Target_Selection