#ifndef ACC_TARGET_SELECTION_ATSUTIL_H
#define ACC_TARGET_SELECTION_ATSUTIL_H

#include "basetype.h"
#include "simpleconstant.h"

#include <vector>

namespace ACC_Target_Selection {
struct PreLoolkUpParam {
    std::size_t interval;
    basetype::float32_t fraction;
    explicit PreLoolkUpParam():
            interval(static_cast<std::size_t>(0)),
            fraction(zero_float32_t) {
    }
    PreLoolkUpParam(const PreLoolkUpParam &r):
            interval(r.interval),
            fraction(r.fraction) {
    }
    explicit PreLoolkUpParam(const std::size_t &interval_,
                             const basetype::float32_t  &fraction_):
        interval(interval_),
        fraction(fraction_) {
    }
    PreLoolkUpParam& operator=(const PreLoolkUpParam& rhs) {
        interval = rhs.interval;
        fraction = rhs.fraction;
        return *this;
    }
};

namespace Util {
class BooleanSignal {
public:
    BooleanSignal();
    BooleanSignal(const BooleanSignal& r);
    BooleanSignal(basetype::boolean_t val, basetype::float32_t sampletime);
    ~BooleanSignal();

    void reset(basetype::boolean_t val);

    void update(basetype::boolean_t val);

    basetype::boolean_t isrising();

    basetype::boolean_t isfalling();

    basetype::boolean_t value();

    basetype::boolean_t previousvalue();

    basetype::boolean_t hold(basetype::boolean_t holdvalue, basetype::float32_t holdtime);

    basetype::boolean_t delay(basetype::boolean_t delayvalue, basetype::float32_t delaytime);
    
private:
    basetype::float32_t sampletime_;
    basetype::float32_t timer_;
    basetype::boolean_t val_;
};
    
extern basetype::float32_t safedivide(const basetype::float32_t &nominator,
                                      const basetype::float32_t &denominator,
                                      const basetype::float32_t &threshold = eps4_float32_t);

extern basetype::float32_t filter(const basetype::float32_t &value,
                                  const basetype::float32_t &previousvalue,
                                  const basetype::float32_t &tsovertau);

extern basetype::float32_t min(const basetype::float32_t &value1,
                               const basetype::float32_t &value2);

extern basetype::float64_t min(const basetype::float64_t &value1,
                               const basetype::float64_t &value2);

extern basetype::uint8_t min(const basetype::uint8_t &value1,
                             const basetype::uint8_t &value2);

extern basetype::uint16_t min(const basetype::uint16_t &value1,
                              const basetype::uint16_t &value2);

extern basetype::uint32_t min(const basetype::uint32_t &value1,
                              const basetype::uint32_t &value2);

extern basetype::uint64_t min(const basetype::uint64_t &value1,
                              const basetype::uint64_t &value2);

extern basetype::int8_t min(const basetype::int8_t &value1,
                            const basetype::int8_t &value2);

extern basetype::int16_t min(const basetype::int16_t value1,
                             const basetype::int16_t &value2);

extern basetype::int32_t min(const basetype::int32_t &value1,
                             const basetype::int32_t &value2);

extern basetype::int64_t min(const basetype::int64_t &value1,
                             const basetype::int64_t &value2);

extern basetype::float32_t max(const basetype::float32_t &value1,
                               const basetype::float32_t &value2);

extern basetype::float64_t max(const basetype::float64_t &value1,
                               const basetype::float64_t &value2);

extern basetype::uint8_t max(const basetype::uint8_t &value1,
                             const basetype::uint8_t &value2);

extern basetype::uint16_t max(const basetype::uint16_t &value1,
                              const basetype::uint16_t &value2);

extern basetype::uint32_t max(const basetype::uint32_t &value1,
                              const basetype::uint32_t &value2);

extern basetype::uint64_t max(const basetype::uint64_t &value1,
                              const basetype::uint64_t &value2);

extern basetype::int8_t max(const basetype::int8_t &value1,
                            const basetype::int8_t &value2);

extern basetype::int16_t max(const basetype::int16_t &value1,
                             const basetype::int16_t &value2);

extern basetype::int32_t max(const basetype::int32_t &value1,
                             const basetype::int32_t &value2);

extern basetype::int64_t max(const basetype::int64_t &value1,
                             const basetype::int64_t &value2);

extern basetype::float32_t abs(const basetype::float32_t &value);

extern basetype::float64_t abs(const basetype::float64_t &value);

extern basetype::int8_t abs(const basetype::int8_t &value);

extern basetype::int16_t abs(const basetype::int16_t &value);

extern basetype::int32_t abs(const basetype::int32_t &value);

extern basetype::int64_t abs(const basetype::int64_t &value);

extern basetype::int8_t signfunc(const basetype::float32_t &value,
                                 const basetype::float32_t &threshold);
        
extern basetype::int8_t signfunc(const basetype::float32_t &value);

extern basetype::float32_t safetydiv(const basetype::float32_t &nominator,
                                     const basetype::float32_t &denominator,
                                     const basetype::float32_t &threshold = eps4_float32_t);

extern basetype::float32_t lookup(const std::vector<basetype::float32_t> &data,
                                  const std::vector<basetype::float32_t> &breakpoints,
                                  const basetype::float32_t &value,
                                  const basetype::boolean_t &extrapolate);
        
extern basetype::uint8_t lookup_u8_float(const std::vector<basetype::uint8_t> &data,
                                         const std::vector<basetype::float32_t> &breakpoints,
                                         const basetype::float32_t &value,
                                         const basetype::boolean_t &extrapolate);
    
extern void preloolkup(const std::vector<basetype::float32_t> &breakpoints,
                       const basetype::float32_t &value,
                       const basetype::boolean_t &extrapolate,
                       PreLoolkUpParam &preloolkupparam);

extern void preloolkup_u8_float(const std::vector<basetype::float32_t> &breakpoints,
                                const basetype::float32_t &value,
                                const basetype::boolean_t &extrapolate,
                                PreLoolkUpParam &preloolkupparam);

extern basetype::float32_t lookuprun(const std::vector<basetype::float32_t> &data,
                                     const PreLoolkUpParam &preloolkupparam);

extern basetype::uint8_t lookuprun_u8_float(const std::vector<basetype::uint8_t> &data,
                                            const PreLoolkUpParam &preloolkupparam);

extern basetype::uint8_t limitchange(const basetype::uint8_t &newvalue, 
                                     const basetype::uint8_t &oldvalue,
                                     const basetype::uint8_t &upperlimit,
                                     const basetype::uint8_t &lowerlimit);

extern basetype::float32_t limitchange(const basetype::float32_t &newvalue, 
                                       const basetype::float32_t &oldvalue,
                                       const basetype::float32_t &upperlimit,
                                       const basetype::float32_t &lowerlimit);
        
extern basetype::uint8_t saturate(const basetype::uint8_t &value,
                                  const basetype::uint8_t &upperlimit,
                                  const basetype::uint8_t &lowerlimit);

extern basetype::float32_t saturate(const basetype::float32_t &value,
                                    const basetype::float32_t &upperlimit,
                                    const basetype::float32_t &lowerlimit);
} // namespace Util
} // namespace ACC_Target_Selection

#endif