#ifndef ACC_TARGET_SELECTION_VALIDATOR_H
#define ACC_TARGET_SELECTION_VALIDATOR_H

#include <array>
#include <memory>

#include "basetype.h"
#include "param.h"
#include "logger.h"
#include "objecttrackstatus.h"
#include "objectfusionsource.h"
#include "object.h"
#include "egovehicleinformation.h"

namespace ACC_Target_Selection {
    
    class Validator {
    public:
        const Param_Validator &param_conf_;
        const Logger &logger_;
        explicit Validator(const Param_Validator &param_conf, const Logger &logger);
        Validator(const Validator& r);
        ~Validator();

        void setprevtgta(const basetype::objectid_t &tgtaidnm, const basetype::objectid_t &rtgtaidn);
        basetype::boolean_t checkvalid(const Object &obj, const EgoVehicleInformation &egovehinfo, const basetype::boolean_t &isrearobject);

    private:
        std::array<basetype::uint8_t, c_ObjectSize> maturitycounter_;
        std::array<basetype::uint8_t, c_ObjectSize> coastcounter_;
        std::array<basetype::uint8_t, c_ObjectSize> jumpcounter_;
        std::array<basetype::uint8_t, c_ObjectSize> qualitycounter_;
        std::array<basetype::float32_t, c_ObjectSize> lgtpositionmemory_;
        std::array<basetype::float32_t, c_ObjectSize> latpositionmemory_;
        std::array<basetype::float32_t, c_ObjectSize> spdmemory_;
        basetype::objectid_t pretgtaid_;
        basetype::float32_t pretgtaspd_;
        basetype::float32_t pretgtalgtposition_;
        basetype::float32_t pretgtalatposition_;
        basetype::objectid_t prereartgtaid_;
        basetype::float32_t prereartgtaspd_;
        basetype::float32_t prereartgtalgtposition_;
        basetype::float32_t prereartgtalatposition_;

        basetype::boolean_t updatecoastcounter(const ObjectTrackStatus &trackstatus, const basetype::objectid_t &idn0);

        basetype::boolean_t updatejumpmaturity(const basetype::float32_t &poslgt,
                                               const basetype::float32_t &poslat,
                                               const basetype::float32_t &spd,
                                               const basetype::objectid_t &idn0);

        basetype::boolean_t updatematurity(const basetype::boolean_t &detected,
                                           const basetype::float32_t &spd,
                                           const basetype::float32_t &poslgt,
                                           const ObjectFusionSource &fusionsource,
                                           const basetype::float32_t &angle,
                                           const basetype::objectid_t &idn);

        basetype::boolean_t checkvalidpre(const basetype::boolean_t &maturityok,
                                          const basetype::boolean_t &jumpok,
                                          const basetype::boolean_t &qualityok,
                                          const basetype::boolean_t &coastedok,
                                          const basetype::boolean_t &isrearobject,
                                          const ObjectTrackStatus &trackstatus,
                                          const basetype::float32_t &objlgtdis,
                                          const basetype::float32_t &objdisLgtstddev,
                                          const basetype::float32_t &objdisLatstddev,
                                          const basetype::float32_t &objaccelstddev,
                                          const basetype::float32_t &objheadinganglestddev);

        basetype::boolean_t checkisvalidtypeandstationary(const basetype::objectid_t &idn,
                                                          const ObjectType &type,
                                                          const ObjectMotionType &motiontype,
                                                          const ObjectMotionHistoryType &motionhistorytype,
                                                          const basetype::float32_t &egospd);

        basetype::boolean_t enableuseofanyobject(const basetype::boolean_t &isrearobject,
                                                 const basetype::float32_t &lgtdis,
                                                 const ObjectMotionType &motiontype,
                                                 const ObjectMotionHistoryType &motionhistorytype,
                                                 const basetype::float32_t &egospd);

        basetype::boolean_t checkisvalidmoving(const basetype::objectid_t &idn,
                                               const ObjectType &type,
                                               const ObjectMotionType &motiontype,
                                               const ObjectMotionHistoryType &motionhistorytype,
                                               const basetype::float32_t &objspd,
                                               const basetype::float32_t &egospd);

        basetype::boolean_t checkisvalidcameramotorcycle(const basetype::boolean_t &isrearobject,
                                                         const ObjectType &type,
                                                         const basetype::float32_t &objlgtdis,
                                                         const ObjectMotionType &motiontype,
                                                         const ObjectTrackStatus &trackstatus,
                                                         const ObjectFusionSource &fusionsource,
                                                         const basetype::boolean_t &jumpok,
                                                         const basetype::boolean_t &maturityok);

        basetype::boolean_t checkisvalidcameracar(const basetype::objectid_t &idn0,
                                                  const basetype::boolean_t &isrearobject,
                                                  const ObjectType &type,
                                                  const basetype::float32_t &objlgtdis,
                                                  const basetype::float32_t &objdisLgtstddev,
                                                  const ObjectMotionType &motiontype,
                                                  const ObjectTrackStatus &trackstatus,
                                                  const ObjectFusionSource &fusionsource,
                                                  const basetype::boolean_t &jumpok,
                                                  const basetype::boolean_t &maturityok,
                                                  const basetype::float32_t &egospd);
        void reset(const basetype::objectid_t &idn0);
        basetype::boolean_t checkisvalidpedestrain(const Object &obj, const EgoVehicleInformation &egovehinfo, const basetype::boolean_t &isrearobject, const basetype::boolean_t &validpre);

    };
    
} // namespace ACC_Target_Selection

#endif