#include "ats_output_switch.h"


namespace rm {
SINGLETON_STATIC(ATSOutputSwitch);

void ATSOutputSwitch::RunAtsOutputSwitch(
    dpc::RoadInfo &rm_road_info,
    dpc::RMSMMessage &rm_sm_msg,
    dpc::TargetsInfo &rm_targets_info,
    dpc::Corridor &ats_corridor,
    dpc::TargetsInfo &ats_targetsinfo,
    dpc::TargetsInfo &targets_info, rm::Frame *frame,
    uint32_t ats_pedestrian_iv) {
  bool ats_switch_decider_result = frame->ats_switch_flag();
  uint8_t idx_counter = 0;
  uint32_t ego_lane_id = rm_road_info.car_position.ego_lane_id;
  bool ego_lane_miss;
  bool ego_corridor_list_empty;
  auto ego_lane_it = std::find_if(
      rm_road_info.lane_list.begin(), rm_road_info.lane_list.end(),
      [=](dpc::Lane lane) { return lane.end_id == ego_lane_id; });
  if (ego_lane_it != rm_road_info.lane_list.end()) {
    // ego lane exists
    ego_lane_miss = false;
    ego_corridor_list_empty = ego_lane_it->corridor_list.empty();
  } else {
    // ego lane not exist
    ego_lane_miss = true;
    ego_corridor_list_empty = true;
  }
  bool ego_lane_miss_delayed = ego_lane_miss_delay_.RunDelay(ego_lane_miss);
  bool ego_corridor_list_empty_delay =
      ego_corridor_list_empty_delay_.RunDelay(ego_corridor_list_empty);
  if (ego_lane_miss) {
    // if ego lane is missing
    dpc::Lane virtual_ego_lane;
    // if
    // (IntersectionVirtualLaneGenerator::Instance()->GenerateVitrualCenterLine(
    //         road_info_provider, ats_targetsinfo)) {
    //   rm::Corridor intersection_corridor;
    //   IntersectionVirtualLaneGenerator::Instance()->GenIntersectionCorridor(
    //       intersection_corridor);
    //   dpc::Corridor cor;
    //   output::Corridor2Corridor(intersection_corridor, cor);
    //   virtual_ego_lane.corridor_list.emplace_back(std::move(cor));
    //   output::CorridorInfo2SMCorridorInfo(idx_counter, rm_sm_msg,
    //                                       intersection_corridor.corridor_id(),
    //                                       intersection_corridor.feature_tag());
    //   virtual_ego_lane.end_id =
    //       std::floor(intersection_corridor.corridor_id() / 10);
    //   virtual_ego_lane.start_id = virtual_ego_lane.end_id;
    //   virtual_ego_lane.lane_type =
    //   static_cast<uint8_t>(LaneType::VIRTUAL_LANE);
    //   rm_road_info.car_position.ego_lane_id = virtual_ego_lane.start_id;
    // } else {
    virtual_ego_lane.end_id = 0;
    virtual_ego_lane.start_id = 0;
    virtual_ego_lane.lane_type = static_cast<uint8_t>(LaneType::VIRTUAL_LANE);
    ats_corridor.feature_tag =
        static_cast<uint8_t>(CorridorFeatureTag::LINEMISS);
    ats_corridor.corridor_id = 500;
    output::CorridorInfo2SMCorridorInfo(idx_counter, rm_sm_msg, 500U,
                                        CorridorFeatureTag::LINEMISS);
    virtual_ego_lane.corridor_list.push_back(ats_corridor);
    // }
    if (ats_targetsinfo.iv.is_tgtprsnt &&
        (frame->road_info_provider()->mutable_obstacles()->count(
             ats_targetsinfo.iv.indx_tgtid) == 1)) {
      rm::Obstacle *ats_obstacle =
          &(frame->road_info_provider()->mutable_obstacles()->at(
              ats_targetsinfo.iv.indx_tgtid));
      if (ats_obstacle->Generate_virtual_corridor()) {
        dpc::Corridor cor;
        output::Corridor2Corridor(ats_obstacle->virtual_corridor(), cor);
        virtual_ego_lane.corridor_list.emplace_back(std::move(cor));
        output::CorridorInfo2SMCorridorInfo(
            idx_counter, rm_sm_msg,
            ats_obstacle->virtual_corridor().corridor_id(),
            ats_obstacle->virtual_corridor().feature_tag());
      }
    }
    rm_road_info.lane_list.push_back(virtual_ego_lane);
    // default output for sm ego lane info
    rm_sm_msg.ego_lane.at(0).lane_merge_distance = 1000.0;
  } else if (ego_corridor_list_empty) {
    // if ego lane exist, but has no corridor, either because the lane is too
    // short or the corridor is too narrow if
    // (IntersectionVirtualLaneGenerator::Instance()->GenerateVitrualCenterLine(
    //         road_info_provider, ats_targetsinfo)) {
    //   rm::Corridor intersection_corridor;
    //   IntersectionVirtualLaneGenerator::Instance()->GenIntersectionCorridor(
    //       intersection_corridor);
    //   dpc::Corridor cor;
    //   output::Corridor2Corridor(intersection_corridor, cor);
    //   ego_lane_it->corridor_list.emplace_back(std::move(cor));
    //   output::CorridorInfo2SMCorridorInfo(idx_counter, rm_sm_msg,
    //                                       intersection_corridor.corridor_id(),
    //                                       intersection_corridor.feature_tag());
    // } else {
    ats_corridor.feature_tag =
        static_cast<uint8_t>(CorridorFeatureTag::LINEMISS);
    ats_corridor.corridor_id = 500;
    output::CorridorInfo2SMCorridorInfo(idx_counter, rm_sm_msg, 500U,
                                        CorridorFeatureTag::LINEMISS);
    ego_lane_it->corridor_list.push_back(ats_corridor);
    // }
    // DLOG(INFO) << PRINT(ats_corridor.object_in_corridor_list.size());
    if (ats_targetsinfo.iv.is_tgtprsnt &&
        (frame->road_info_provider()->mutable_obstacles()->count(
             ats_targetsinfo.iv.indx_tgtid) == 1)) {
      rm::Obstacle *ats_obstacle =
          &(frame->road_info_provider()->mutable_obstacles()->at(
              ats_targetsinfo.iv.indx_tgtid));
      auto ego_lane_info_ptr =
          frame->FindLaneInfoPtrFromLanePosition(LanePosition::EGO_LANE);
      if (ats_obstacle->Generate_virtual_corridor(
              ego_lane_info_ptr->mutable_boundary())) {
        dpc::Corridor cor;
        output::Corridor2Corridor(ats_obstacle->virtual_corridor(), cor);
        ego_lane_it->corridor_list.emplace_back(std::move(cor));
        output::CorridorInfo2SMCorridorInfo(
            idx_counter, rm_sm_msg,
            ats_obstacle->virtual_corridor().corridor_id(),
            ats_obstacle->virtual_corridor().feature_tag());
      }
    }
  } else {
    // do nothing
  }

  if (ego_lane_miss_delayed) {
    // no lane exists delayed
    targets_info = ats_targetsinfo;
    rm_sm_msg.rm_dsblrsn.at(1) = ats_pedestrian_iv;
  } else {
    if (ego_corridor_list_empty_delay) {
      // delayed
      rm_sm_msg.rm_dsblrsn.at(1) = ats_pedestrian_iv;
      targets_info = ats_targetsinfo;
      SetRMTSResWithATSRes(frame, ats_targetsinfo);
    } else {
      targets_info = rm_targets_info;
    }
  }

  if (frame->ats_switch_flag()) {
    // only copy target selection result
    targets_info.iv = ats_targetsinfo.iv;
    targets_info.iiv = ats_targetsinfo.iiv;
    targets_info.riv = ats_targetsinfo.riv;
    targets_info.flniv = ats_targetsinfo.flniv;
    targets_info.flniiv = ats_targetsinfo.flniiv;
    targets_info.flnriv = ats_targetsinfo.flnriv;
    targets_info.slniv = ats_targetsinfo.slniv;
    targets_info.slniiv = ats_targetsinfo.slniiv;
    targets_info.slnriv = ats_targetsinfo.slnriv;
    SetRMTSResWithATSRes(frame, ats_targetsinfo);
  }

  targets_info.iv.is_tgtnew = EvaluateTargetNew(targetsinfo_pre_.iv.indx_tgtid,
                                                targets_info.iv.indx_tgtid);
  targets_info.iiv.is_tgtnew = EvaluateTargetNew(
      targetsinfo_pre_.iiv.indx_tgtid, targets_info.iiv.indx_tgtid);
  targets_info.flniiv.is_tgtnew = EvaluateTargetNew(
      targetsinfo_pre_.flniiv.indx_tgtid, targets_info.flniiv.indx_tgtid);
  targets_info.flniv.is_tgtnew = EvaluateTargetNew(
      targetsinfo_pre_.flniv.indx_tgtid, targets_info.flniv.indx_tgtid);
  targets_info.flnriv.is_tgtnew = EvaluateTargetNew(
      targetsinfo_pre_.flnriv.indx_tgtid, targets_info.flnriv.indx_tgtid);
  targets_info.riv.is_tgtnew = EvaluateTargetNew(
      targetsinfo_pre_.riv.indx_tgtid, targets_info.riv.indx_tgtid);
  targets_info.slniiv.is_tgtnew = EvaluateTargetNew(
      targetsinfo_pre_.slniiv.indx_tgtid, targets_info.slniiv.indx_tgtid);
  targets_info.slniv.is_tgtnew = EvaluateTargetNew(
      targetsinfo_pre_.slniv.indx_tgtid, targets_info.slniv.indx_tgtid);
  targets_info.slnriv.is_tgtnew = EvaluateTargetNew(
      targetsinfo_pre_.slnriv.indx_tgtid, targets_info.slnriv.indx_tgtid);

  targets_info.cyclecounter = ats_targetsinfo.cyclecounter;
  targetsinfo_pre_ = targets_info;
}

void ATSOutputSwitch::SetRMTSResWithATSRes(
    rm::Frame *frame, dpc::TargetsInfo &ats_targetsinfo) {
  if (ats_targetsinfo.iv.is_tgtprsnt &&
      (frame->road_info_provider()->mutable_obstacles()->count(
           ats_targetsinfo.iv.indx_tgtid) == 1)) {
    // DLOG(INFO) << PRINT(ats_targetsinfo.iv.indx_tgtid);
    frame->road_info_provider()
        ->mutable_obstacles()
        ->at(ats_targetsinfo.iv.indx_tgtid)
        .set_trplontype(rm::TargetType::IV);
  }
  if (ats_targetsinfo.iiv.is_tgtprsnt &&
      (frame->road_info_provider()->mutable_obstacles()->count(
          ats_targetsinfo.iiv.indx_tgtid))) {
    // DLOG(INFO) << PRINT(ats_targetsinfo.iiv.indx_tgtid);
    frame->road_info_provider()
        ->mutable_obstacles()
        ->at(ats_targetsinfo.iiv.indx_tgtid)
        .set_trplontype(rm::TargetType::IIV);
  }
  return;
}

bool ATSOutputSwitch::EvaluateTargetNew(uint32_t &id_pre, uint32_t &id_new) {
  bool result = false;
  if ((id_pre != id_new) && (id_new > 0)) {
    result = true;
  } else {
    result = false;
  }
  return result;
}
} // namespace rm
