#include "adapter_output.h"

namespace rm {
namespace output {
void PolynomLine2LineGeometry(const PolynomLine &polynomLine,
                              dpc::LineGeometry &line_geometry) {
  line_geometry.c0 = polynomLine.Coef(0);
  line_geometry.c1 = polynomLine.Coef(1);
  line_geometry.c2 = polynomLine.Coef(2);
  line_geometry.c3 = polynomLine.Coef(3);
  line_geometry.start_range = polynomLine.start_range();
  line_geometry.end_range = polynomLine.end_range();
};

void LaneMarkerSegment2LaneBoundary(
    const LaneMarkerSegment &lane_marker_segment,
    dpc::LaneBoundary &lane_boundary) {
  lane_boundary.line_boundary_id = lane_marker_segment.lane_marker_segment_id();
  lane_boundary.arc_length = lane_marker_segment.arc_length();
  PolynomLine2LineGeometry(lane_marker_segment.geometry().curve(),
                           lane_boundary.line_geometry);
  // lane_boundary.line_color = uint8_t(lane_marker_segment.color());
  // lane_boundary.line_type = uint8_t(lane_marker_segment.type());
  for (const auto &sample_point : lane_marker_segment.sampling_point_list()) {
    dpc::SamplingPoint pubs_sampling_point;
    SamplingPoint2SamplingPoint(sample_point, pubs_sampling_point);
    lane_boundary.sampling_point_list.emplace_back(
        std::move(pubs_sampling_point));
  }
}

void LineSegAttribute2LineColor(
    const LaneMarkerSegmentAttribute &line_seg_attribute,
    dpc::LineColor &line_color) {
  line_color.linecolor = static_cast<uint8_t>(line_seg_attribute.line_color());
  line_color.start_offset = line_seg_attribute.start_offset();
  line_color.end_offset = line_seg_attribute.end_offset();
}

void LineSegAttribute2LineType(
    const LaneMarkerSegmentAttribute &line_seg_attribute,
    dpc::LineType &line_type) {
  line_type.linetype = static_cast<uint8_t>(line_seg_attribute.line_type());
  line_type.start_offset = line_seg_attribute.start_offset();
  line_type.end_offset = line_seg_attribute.end_offset();
}

void LaneMarkerSegAttributeReassignment(
    dpc::LaneMarker &pubs_lane_marker,
    const std::vector<rm::LaneMarkerSegmentAttribute>
        &lane_marker_segment_attribute) {
  bool first_run = true;
  dpc::LineColor line_color;
  dpc::LineType line_type;
  for (int idx = 0; idx < lane_marker_segment_attribute.size(); idx++) {
    if (first_run) {
      LineSegAttribute2LineColor(lane_marker_segment_attribute.at(idx),
                                 line_color);
      LineSegAttribute2LineType(lane_marker_segment_attribute.at(idx),
                                line_type);
      if (lane_marker_segment_attribute.size() == 1) {
        // FIXME temp solution
        line_color.start_offset = std::max(-200.0F, line_color.start_offset);
        line_color.end_offset = std::min(1000.0F, line_color.end_offset);
        line_type.start_offset = std::max(-200.0F, line_type.start_offset);
        line_type.end_offset = std::min(1000.0F, line_type.end_offset);
        pubs_lane_marker.line_color_attribute_list.push_back(line_color);
        pubs_lane_marker.line_type_attribute_list.push_back(line_type);
      }
      first_run = false;
      continue;
    }
    if (line_color.linecolor ==
        static_cast<uint8_t>(
            lane_marker_segment_attribute.at(idx).line_color())) {
      line_color.end_offset =
          lane_marker_segment_attribute.at(idx).end_offset();
      if (idx == lane_marker_segment_attribute.size() - 1) {
        pubs_lane_marker.line_color_attribute_list.push_back(line_color);
      }
    } else {
      pubs_lane_marker.line_color_attribute_list.push_back(line_color);
      LineSegAttribute2LineColor(lane_marker_segment_attribute.at(idx),
                                 line_color);
      if (idx == lane_marker_segment_attribute.size() - 1) {
        pubs_lane_marker.line_color_attribute_list.push_back(line_color);
      }
    }
    if (line_type.linetype ==
        static_cast<uint8_t>(
            lane_marker_segment_attribute.at(idx).line_type())) {
      line_type.end_offset = lane_marker_segment_attribute.at(idx).end_offset();
      if (idx == lane_marker_segment_attribute.size() - 1) {
        pubs_lane_marker.line_type_attribute_list.push_back(line_type);
      }
    } else {
      pubs_lane_marker.line_type_attribute_list.push_back(line_type);
      LineSegAttribute2LineType(lane_marker_segment_attribute.at(idx),
                                line_type);
      if (idx == lane_marker_segment_attribute.size() - 1) {
        pubs_lane_marker.line_type_attribute_list.push_back(line_type);
      }
    }
  }
}

void SamplingPoint2SamplingPoint(
    const SamplingPoint &sample_point,
    dpc::SamplingPoint &pubs_sampling_point) {
  pubs_sampling_point.x = sample_point.x;
  pubs_sampling_point.y = sample_point.y;
  pubs_sampling_point.curve = sample_point.curve;
  pubs_sampling_point.valid = true;
}

void SamplingPoint2TrpLonSamplingPoint(
    const SamplingPoint &sample_point,
    dpc::SamplingPoint &pubs_sampling_point) {
  pubs_sampling_point.x = sample_point.offset;
  pubs_sampling_point.y = 0.0F;
  pubs_sampling_point.curve = sample_point.curve;
  pubs_sampling_point.valid = true;
}

void PolynomLine2SamplingPoint(
    const PolynomLine &line,
    dpc::SamplingPoint &pubs_sampling_point, double x) {
  pubs_sampling_point.x = x;
  pubs_sampling_point.y = line.Evaluate(0, x);
  pubs_sampling_point.curve = std::abs(line.Evaluate(2, x));
  pubs_sampling_point.valid = true;
}

void LaneMarker2LaneMarker(const LaneMarker &lane_marker,
                           dpc::LaneMarker &pubs_lane_marker) {
  pubs_lane_marker.id = lane_marker.lane_marker_id();
  pubs_lane_marker.is_start_out_of_range = lane_marker.is_start_out_of_range();
  pubs_lane_marker.is_end_out_of_range = lane_marker.is_end_out_of_range();
  for (const auto &lane_marker_segment :
       lane_marker.front_lane_marker_segment()) {
    if (lane_marker_segment.is_sampling_point_valid() ||
        lane_marker_segment.geometry().valid()) {
      dpc::LaneBoundary front_lane_boundary;
      LaneMarkerSegment2LaneBoundary(lane_marker_segment, front_lane_boundary);
      pubs_lane_marker.front_lane_boundary_list.emplace_back(
          std::move(front_lane_boundary));
    }
  }
  for (const auto &lane_marker_segment :
       lane_marker.back_lane_marker_segment()) {
    if (lane_marker_segment.is_sampling_point_valid() ||
        lane_marker_segment.geometry().valid()) {
      dpc::LaneBoundary back_lane_boundary;
      LaneMarkerSegment2LaneBoundary(lane_marker_segment, back_lane_boundary);
      pubs_lane_marker.back_lane_boundary_list.emplace_back(
          std::move(back_lane_boundary));
    }
  }
  LaneMarkerSegAttributeReassignment(
      pubs_lane_marker, lane_marker.lane_marker_segment_attribute());
};

void ObjLocation2ObjLocation(const ObjectLocation object_location,
                             dpc::ObjectLocation &ret) {
  ret.object_id = object_location.object_id;
  ret.reference_point2corridor_frenet.s =
      object_location.reference_point2corridor_frenet.s();
  ret.reference_point2corridor_frenet.l =
      object_location.reference_point2corridor_frenet.l();
  ret.reference_point2corridor_frenet.ds =
      object_location.reference_point2corridor_frenet.ds();
  ret.reference_point2corridor_frenet.dl =
      object_location.reference_point2corridor_frenet.dl();
  ret.reference_point2corridor_frenet.dds =
      object_location.reference_point2corridor_frenet.dds();
  ret.reference_point2corridor_frenet.ddl =
      object_location.reference_point2corridor_frenet.ddl();
  ret.reference_point2corridor_frenet.dl_valid_time =
      object_location.reference_point2corridor_frenet.dl_valid_time();
  ret.reference_point2corridor_frenet.ddl_valid_time =
      object_location.reference_point2corridor_frenet.ddl_valid_time();
  ret.reference_point2corridor_frenet.dds_valid_time =
      object_location.reference_point2corridor_frenet.dds_valid_time();
  ret.relative_center_x = object_location.relative_center_x;
  ret.relative_center_y = object_location.relative_center_y;
  ret.size_length = object_location.size_length;
  ret.size_width = object_location.size_width;
  ret.classification = static_cast<uint8_t>(object_location.classification);
}

void Corridor2Corridor(const Corridor &corridor,
                       dpc::Corridor &cor) {
  cor.corridor_id = corridor.corridor_id();
  cor.drivable_width = corridor.drivable_width();
  cor.ilc_object_target = corridor.ilc_target_id();
  PolynomLine2LineGeometry(corridor.center_line(), cor.reference_line);
  cor.feature_tag = static_cast<uint8_t>(corridor.feature_tag());
  for (auto it = corridor.object_location().begin();
       it != corridor.object_location().end(); it++) {
    dpc::ObjectLocation object_location;
    ObjLocation2ObjLocation(*it, object_location);

    if (it->in_corridor == true &&
        it->classification != ObstacleType::PEDESTRIAN) {
      cor.object_in_corridor_list.emplace_back(std::move(object_location));
    } else {
      cor.relevant_object_corridor_list.emplace_back(
          std::move(object_location));
    }
  }
  std::sort(cor.object_in_corridor_list.begin(),
            cor.object_in_corridor_list.end(),
            [](const dpc::ObjectLocation &obj_1,
               const dpc::ObjectLocation &obj_2) {
              return obj_1.reference_point2corridor_frenet.s >
                     obj_2.reference_point2corridor_frenet.s;
            });
}

void DefaultCorridor2Corridor(dpc::Corridor &cor) {
  cor.corridor_id = 500U;
  cor.drivable_width = 0.0;
  cor.ilc_object_target = 0U;
  PolynomLine defalut_ref_line = PolynomLine({0.0, 0.0, 0.0, 0.0}, 0.0, 0.0);
  PolynomLine2LineGeometry(defalut_ref_line, cor.reference_line);
  cor.feature_tag = static_cast<uint8_t>(rm::CorridorFeatureTag::LINEMISS);
}

void Obstacle2Object(const Obstacle &obstacle,
                     dpc::Object &object) {
  object.object_id = obstacle.id();
  object.object_exist_probability = obstacle.exist_probability();
  object.classification = uint8_t(obstacle.classification());
  object.relative_center_x = obstacle.center().x();
  object.relative_center_y = obstacle.center().y();
  object.absolute_velocity_x = obstacle.speed().x();
  object.absolute_velocity_y = obstacle.speed().y();
  object.relative_position_x = obstacle.position().x();
  object.relative_position_y = obstacle.position().y();
  object.absolute_acceleration_x = obstacle.acceleration().x();
  object.absolute_acceleration_y = obstacle.acceleration().y();
  // object.absolute_yawrate = obstacle.yaw_rate();
  object.absolute_yawrate = obstacle.heading() * rm::RAD2DEG;
  object.size_length = obstacle.length();
  object.size_width = obstacle.width();
}

void FrontObstacle2Target(const Obstacle &obstacle,
                          dpc::TargetInfo &target) {
  double dx{0.0}, dy{0.0};
  ObstacleReferencePointAdapter(dx, dy, obstacle);
  target.a_tgtlataccel = obstacle.acceleration().y();
  target.a_tgtlongtaccel = obstacle.acceleration().x();
  target.d_tgtlatdist = dy;
  target.d_tgtlongtdist = std::max(dx, 0.0);
  target.is_tgtnew =
      (obstacle.prv_trplontype() == obstacle.trplontype()) ? false : true;
  target.indx_tgtid = obstacle.id();
  target.indx_tgttyp = obstacle.is_vulnerable_critical_obstacle()
                           ? static_cast<uint8_t>(obstacle.classification())
                           : 1U;
  target.is_tgtlnechgflag = obstacle.lane_change_intention();
  target.is_tgtprsnt = true;
  target.t_tgtdurtm = std::min(obstacle.existence_duration(), 5.0F);
  target.v_tgtlatspd = obstacle.speed().y();
  target.v_tgtlongtspd = obstacle.speed().x();
  target.v_tgtrltspd = obstacle.speed().x() -
                       VehicleStateProvider::Instance()
                           ->lon_velocity(); // longitudinal relative speed
}

void RearObstacle2Target(const Obstacle &obstacle,
                         dpc::TargetInfo &target) {
  double dx{0.0}, dy{0.0};
  ObstacleReferencePointAdapter(dx, dy, obstacle);
  target.a_tgtlataccel = obstacle.acceleration().y();
  target.a_tgtlongtaccel = obstacle.acceleration().x();
  target.d_tgtlatdist = dy;
  target.d_tgtlongtdist = std::min(dx, 0.0);
  target.is_tgtnew =
      (obstacle.prv_trplontype() == obstacle.trplontype()) ? false : true;
  target.indx_tgtid = obstacle.id();
  target.indx_tgttyp = static_cast<uint8_t>(obstacle.classification());
  target.is_tgtlnechgflag = false;
  target.is_tgtprsnt = true;
  target.t_tgtdurtm = std::min(obstacle.existence_duration(), 5.0F);
  target.v_tgtlatspd = obstacle.speed().y();
  target.v_tgtlongtspd = obstacle.speed().x();
  target.v_tgtrltspd = obstacle.speed().x() -
                       VehicleStateProvider::Instance()
                           ->lon_velocity(); // longitudinal relative speed
}

void DefaultFrontTarget(dpc::TargetInfo &target) {
  target.a_tgtlataccel = 0.0;
  target.a_tgtlongtaccel = 0.0;
  target.d_tgtlatdist = 0.0;
  target.d_tgtlongtdist = 250.0;
  target.is_tgtnew = false;
  target.indx_tgtid = 0;
  target.indx_tgttyp = 0;
  target.is_tgtlnechgflag = false;
  target.is_tgtprsnt = false;
  target.t_tgtdurtm = 0.0;
  target.v_tgtlatspd = 0.0;
  target.v_tgtlongtspd = 0.0;
  target.v_tgtrltspd = 0.0;
}

void DefaultRearTarget(dpc::TargetInfo &target) {
  target.a_tgtlataccel = 0.0;
  target.a_tgtlongtaccel = 0.0;
  target.d_tgtlatdist = 0.0;
  target.d_tgtlongtdist = -250.0;
  target.is_tgtnew = false;
  target.indx_tgtid = 0;
  target.indx_tgttyp = 0;
  target.is_tgtlnechgflag = false;
  target.is_tgtprsnt = false;
  target.t_tgtdurtm = 0.0;
  target.v_tgtlatspd = 0.0;
  target.v_tgtlongtspd = 0.0;
  target.v_tgtrltspd = 0.0;
}

void EHRSpeedLimit2SMSpeedLimit(dpc::RMSMMessage *ptr_rm_sm_msg,
                                const rm::ControlWayPoints *control_waypoints,
                                const rm::Lane *lane) {
  if (control_waypoints->is_on_ramp() && lane && lane->lane_type() != LaneType::EXIT) {
    // ptr_rm_sm_msg->control_way_point_speed_limit01 =
    //     control_waypoints->speed_limit().speed_limit().at(0).speed_limit;
    ptr_rm_sm_msg->control_way_point_speed_limit01 = std::max(static_cast<uint8_t>(60),
     static_cast<uint8_t>(lane->speed_limit().first));
    ptr_rm_sm_msg->control_way_point_offset01 = 0.0F;
    ptr_rm_sm_msg->spd_lmt_id01 = 255U;
    ptr_rm_sm_msg->control_way_point_speed_limit02 = 130U;
    ptr_rm_sm_msg->control_way_point_offset02 = 1000.0F;
    ptr_rm_sm_msg->spd_lmt_id02 = 0U;
  } else {
    ptr_rm_sm_msg->control_way_point_speed_limit01 =
        control_waypoints->speed_limit().speed_limit().at(0).speed_limit;
    ptr_rm_sm_msg->control_way_point_offset01 =
        control_waypoints->speed_limit().speed_limit().at(0).offset;
    ptr_rm_sm_msg->spd_lmt_id01 =
        control_waypoints->speed_limit().speed_limit().at(0).id;
    ptr_rm_sm_msg->control_way_point_speed_limit02 =
        control_waypoints->speed_limit().speed_limit().at(1).speed_limit;
    ptr_rm_sm_msg->control_way_point_offset02 =
        control_waypoints->speed_limit().speed_limit().at(1).offset;
    ptr_rm_sm_msg->spd_lmt_id02 =
        control_waypoints->speed_limit().speed_limit().at(1).id;
  }
}

void Lane2LaneInfo(rm::LaneInfo *lane_info_ptr,
                   dpc::LaneInfo &lane_info) {
  if (lane_info_ptr) {
    lane_info.lane_id = lane_info_ptr->mutable_lane()->lane_id();
    lane_info.max_speed = lane_info_ptr->mutable_lane()->speed_limit().first;
    lane_info.min_speed = lane_info_ptr->mutable_lane()->speed_limit().second;
    lane_info.start_offset = lane_info_ptr->mutable_lane()->start_offset();
    lane_info.end_offset = lane_info_ptr->mutable_lane()->end_offset();
    lane_info.lane_type =
        static_cast<uint8_t>(lane_info_ptr->mutable_lane()->lane_type());
    lane_info.lane_merge_distance =
        lane_info_ptr->mutable_lane()->lane_merge_distance();
    ReferenceLineGeometeryAdapter(lane_info, lane_info_ptr);
  }
}

void ILCTargetList2RMILCTarget(dpc::RMSMMessage *ptr_rm_sm_msg,
                               const std::vector<uint32_t> &ilc_target_list) {
  for (int i = 0; i < std::min(static_cast<uint32_t>(ilc_target_list.size()),
                               static_cast<uint32_t>(10));
       i++) {
    ptr_rm_sm_msg->rm_ilc_target.at(i) = ilc_target_list.at(i);
  }
}

void RoadEdge2RoadEdge(const RoadEdge &road_edge,
                       dpc::RoadEdge &pubs_road_edge) {
  pubs_road_edge.road_edge_id = road_edge.road_edge_id();
  for (const auto &road_edge_seg : road_edge.road_edge_segment_list()) {
    dpc::RoadEdgeSegment pubs_road_edge_seg;
    RoadEdgeSeg2RoadEdgeSeg(road_edge_seg, pubs_road_edge_seg);
    pubs_road_edge.road_edge_segment_list.emplace_back(
        std::move(pubs_road_edge_seg));
  }
}

void RoadEdgeSeg2RoadEdgeSeg(
    const RoadEdgeSegment &road_edge_seg,
    dpc::RoadEdgeSegment &pubs_road_edge_seg) {
  pubs_road_edge_seg.id = road_edge_seg.road_edge_segment_id();
  pubs_road_edge_seg.contributing_sensor =
      static_cast<uint16_t>(road_edge_seg.sensor_source());
  pubs_road_edge_seg.height_above_ground = road_edge_seg.height_above_ground();
  pubs_road_edge_seg.type = static_cast<uint8_t>(road_edge_seg.type());
  pubs_road_edge_seg.is_end_out_of_view = road_edge_seg.end_out_of_view();
  pubs_road_edge_seg.is_start_out_of_view = road_edge_seg.start_out_of_view();
  PolynomLine2LineGeometry(road_edge_seg.geometry().curve(),
                           pubs_road_edge_seg.line_geometry);
}

uint8_t RoadEdgeType2TRPRoadEdgeType(rm::RoadEdgeType road_edge_type) {
  switch (road_edge_type) {
  case rm::RoadEdgeType::UNKNOWN:
    return 0;
  case rm::RoadEdgeType::FLAT:
    return 1;
  case rm::RoadEdgeType::ELEVATED:
    return 2;
  case rm::RoadEdgeType::CURB:
    return 3;
  case rm::RoadEdgeType::CONE_AND_POLE:
    return 4;
  case rm::RoadEdgeType::PARKED_CARS:
    return 5;
  default:
    return 0;
  }
}

void CorridorInfo2SMCorridorInfo(uint8_t &idx,
                                 dpc::RMSMMessage &rm_sm_msg,
                                 uint32_t id, rm::CorridorFeatureTag tag) {
  if (idx < 10U) {
    rm_sm_msg.rm_crrdr_id.at(idx) = id;
    rm_sm_msg.rm_featuretag.at(idx) = static_cast<uint8_t>(tag);
    idx++;
  } else {
    return;
  }
}

void ObstacleReferencePointAdapter(double &dx, double &dy,
                                   const Obstacle &object) {
  switch (object.reference_point_type()) {
  case rm::ReferencePointType::FRONTLEFT:
    dy = object.position().y() - 0.5 * 0.95 * object.width();
    dx = object.position().x() - 0.95 * object.length() - rm::DISREARAXEL2FRONT;
    break;
  case rm::ReferencePointType::FRONTCENTER:
    dy = object.position().y();
    dx = object.position().x() - 0.95 * object.length() - rm::DISREARAXEL2FRONT;
    break;
  case rm::ReferencePointType::FRONTRIGHT:
    dy = object.position().y() + 0.5 * 0.95 * object.width();
    dx = object.position().x() - 0.95 * object.length() - rm::DISREARAXEL2FRONT;
    break;
  case rm::ReferencePointType::RIGHTCENTER:
    dy = object.position().y() + 0.5 * 0.95 * object.width();
    dx = object.position().x() - 0.5 * 0.95 * object.length() -
         rm::DISREARAXEL2FRONT;
    break;
  case rm::ReferencePointType::REARRIGHT:
    dy = object.position().y() + 0.5 * 0.95 * object.width();
    dx = object.position().x() - rm::DISREARAXEL2FRONT;
    break;
  case rm::ReferencePointType::REARCENTER:
    dy = object.position().y();
    dx = object.position().x() - rm::DISREARAXEL2FRONT;
    break;
  case rm::ReferencePointType::REARLEFT:
    dy = object.position().y() - 0.5 * 0.95 * object.width();
    dx = object.position().x() - rm::DISREARAXEL2FRONT;
    break;
  case rm::ReferencePointType::LEFTCENTER:
    dy = object.position().y() - 0.5 * 0.95 * object.width();
    dx = object.position().x() - 0.5 * 0.95 * object.length() -
         rm::DISREARAXEL2FRONT;
    break;
  case rm::ReferencePointType::CENTER:
    dy = object.position().y();
    dx = object.position().x() - 0.5 * 0.95 * object.length() -
         rm::DISREARAXEL2FRONT;
    break;
  }
}

void ODDInfoAdapter(dpc::LaneInfo &sm_lane_info,
                    rm::WPExtendLane *extend_lane) {
  if (!extend_lane) {
    return;
  }
  auto odd_info_ptr = extend_lane->mutable_odd_info();
  uint8_t num = std::min(5, static_cast<int>(odd_info_ptr->size()));
  for (uint8_t i = 0; i < num; ++i) {
    sm_lane_info.odd_info.at(i).valid = true;
    sm_lane_info.odd_info.at(i).lane_id = odd_info_ptr->at(i).lane_id;
    sm_lane_info.odd_info.at(i).type = odd_info_ptr->at(i).type;
    sm_lane_info.odd_info.at(i).sub_type = odd_info_ptr->at(i).sub_type;
    sm_lane_info.odd_info.at(i).start_offset = odd_info_ptr->at(i).start_offset;
    sm_lane_info.odd_info.at(i).end_offset = odd_info_ptr->at(i).end_offset;
  }
}

void ReferenceLineGeometeryAdapter(dpc::LaneInfo &sm_lane_info,
                                   rm::LaneInfo *lane_info) {
  uint8_t idx = 0;
  for (auto &cor : *lane_info->mutable_corridor()) {
    PolynomLine *center_line_ptr = cor.mutable_center_line();
    sm_lane_info.reference_line.at(idx).c0 = center_line_ptr->Coef(0);
    sm_lane_info.reference_line.at(idx).c1 = center_line_ptr->Coef(1);
    sm_lane_info.reference_line.at(idx).c2 = center_line_ptr->Coef(2);
    sm_lane_info.reference_line.at(idx).c3 = center_line_ptr->Coef(3);
    sm_lane_info.reference_line.at(idx).start_range =
        center_line_ptr->start_range();
    sm_lane_info.reference_line.at(idx).end_range =
        center_line_ptr->end_range();
    idx++;
    if (idx > 3)
      break;
  }
  // if (lane_info->mutable_corridor()->size() > 0) {
  //   PolynomLine reference_line =
  //       lane_info->mutable_corridor()->at(0).center_line();
  //   sm_lane_info.c0_0 = reference_line.Coef(0);
  //   sm_lane_info.c1_0 = reference_line.Coef(1);
  //   sm_lane_info.c2_0 = reference_line.Coef(2);
  //   sm_lane_info.c3_0 = reference_line.Coef(3);
  // }
  // if (lane_info->mutable_corridor()->size() > 1) {
  //   PolynomLine reference_line =
  //       lane_info->mutable_corridor()->at(1).center_line();
  //   sm_lane_info.c0_1 = reference_line.Coef(0);
  //   sm_lane_info.c1_1 = reference_line.Coef(1);
  //   sm_lane_info.c2_1 = reference_line.Coef(2);
  //   sm_lane_info.c3_1 = reference_line.Coef(3);
  // }
}

void MaxSpeedLimitAdapter(dpc::Lane &lane,
                          const rm::Lane &rm_lane) {
  dpc::LaneControl lane_control;
  if (rm_lane.lane_segment_attribute().empty()) {
    lane_control.value = 0.0F;
    lane_control.start_range = 0.0F;
    lane_control.end_range = 0.0F;
  } else {
    auto it = std::find_if(rm_lane.lane_segment_attribute().begin(),
                           rm_lane.lane_segment_attribute().end(),
                           [](const LaneSegmentAttribute &seg) {
                             return (0.0 < seg.end_offset() &&
                                     0.0 >= seg.start_offset());
                           });
    if (it != rm_lane.lane_segment_attribute().end()) {
      lane_control.value = it->max_speed_limit();
      lane_control.start_range = it->start_offset();
      lane_control.end_range = it->end_offset();
    } else {
      lane_control.value =
          rm_lane.lane_segment_attribute().back().max_speed_limit();
      lane_control.start_range =
          rm_lane.lane_segment_attribute().back().start_offset();
      lane_control.end_range =
          rm_lane.lane_segment_attribute().back().end_offset();
    }
  }
  lane.max_speed_limit.emplace_back(lane_control);
}

void SampingPointTrpLongAdapter(
    std::array<dpc::SamplingPoint, 25> &sampling_points_output,
    std::vector<LaneMarkerSegment> *lane_marker_segs, uint8_t desired_lane_id,
    const Lane &special_ego_lane) {

  if (desired_lane_id == 1) {
    // near ramp
    uint16_t sp_idx{0U}, seg_idx{0U};
    uint16_t num_of_segments = special_ego_lane.front_lane_segment().size();
    uint16_t num_output_sampling_points = sampling_points_output.size();
    while (sp_idx < num_output_sampling_points) {
      if (special_ego_lane.front_lane_segment()
              .at(seg_idx)
              .is_sampling_point_valid()) {
        const std::vector<SamplingPoint> &seg_sampling_points_list =
            special_ego_lane.front_lane_segment()
                .at(seg_idx)
                .sampling_point_list();
        for (uint16_t i = 0U; i < seg_sampling_points_list.size();) {
          if (seg_sampling_points_list.at(i).offset >
              GetRMConfig().trplon_curv_sampling_point_start_offset) {
            SamplingPoint2TrpLonSamplingPoint(
                seg_sampling_points_list.at(i),
                sampling_points_output.at(sp_idx));
            ++sp_idx;
            if (sp_idx >= num_output_sampling_points)
              break;
          }
          i += GetRMConfig().trplon_curv_sampling_point_gap;
        }
      }
      ++seg_idx;
      if (seg_idx >= num_of_segments)
        break;
    }
    return;
  } else if (lane_marker_segs->size() > 1) {
    // other case at least 2 segs
    for (std::vector<LaneMarkerSegment>::const_iterator iter =
             lane_marker_segs->begin() + 1;
         iter != lane_marker_segs->end();) {
      if (iter->lane_marker_segment_id() != 0U) {
        // current seg is from hdm
        if (iter->is_sampling_point_valid()) {
          // if the sampling points of second seg is valid
          uint8_t idx = 0U;
          for (uint8_t i = 0; i < iter->sampling_point_list().size(); i += 2) {
            // take the sampling point every 2 meters
            if (idx < 25) {
              SamplingPoint2SamplingPoint(iter->sampling_point_list().at(i),
                                          sampling_points_output.at(idx));
              idx++;
            } else {
              break;
            }
          }
          for (int i = 1; i < idx - 1; i++) {
            sampling_points_output.at(i).curve =
                rm::common::math::DiscretePointCurvatureCalculator(
                    sampling_points_output.at(i - 1),
                    sampling_points_output.at(i),
                    sampling_points_output.at(i + 1));
            if (i == 1) {
              sampling_points_output.at(i - 1).curve =
                  sampling_points_output.at(i).curve;
            }
            if (i == idx - 2) {
              sampling_points_output.at(i + 1).curve =
                  sampling_points_output.at(i).curve;
            }
          }
        } else {
          // do nothing
        }
        break;
      } else {
        // current seg is bezier transition, go to next seg
        iter++;
      }
    }
  }
}

rm::MeasurementState
MeasurementStateRemap(rm::MeasurementState original_meas_state,
                      rm::LineType current_ln_type) {
  if (original_meas_state != rm::MeasurementState::VIRTUAL) {
    return original_meas_state;
  } else {
    if (current_ln_type != rm::LineType::VIRTUAL &&
        current_ln_type != rm::LineType::COMP) {
      return rm::MeasurementState::MEASURED;
    } else {
      return original_meas_state;
    }
  }
}

} // namespace output
} // namespace rm