#include <string.h>

#include "lh1_ep21_p_can2_v12.h"

static inline uint8_t unpack_right_shift_u8(
    uint8_t value,
    uint8_t shift,
    uint8_t mask)
{
    return (uint8_t)((uint8_t)(value & mask) >> shift);
}

static inline uint16_t unpack_left_shift_u16(
    uint8_t value,
    uint8_t shift,
    uint8_t mask)
{
    return (uint16_t)((uint16_t)(value & mask) << shift);
}

static inline uint16_t unpack_right_shift_u16(
    uint8_t value,
    uint8_t shift,
    uint8_t mask)
{
    return (uint16_t)((uint16_t)(value & mask) >> shift);
}

int lh1_ep21_p_can2_v12_sas_hsc2_fr_p00_unpack(
    struct lh1_ep21_p_can2_v12_sas_hsc2_fr_p00_t *dst_p,
    const uint8_t *src_p,
    size_t size)
{
    if (size < 8u) {
        return (-EINVAL);
    }

    dst_p->strg_whl_ang_vhsc2 = unpack_right_shift_u8(src_p[0], 7u, 0x80u);
    dst_p->strg_whl_ang_snsr_cal_sts_hsc2 = unpack_right_shift_u8(src_p[0], 5u, 0x60u);
    dst_p->strg_whl_ang_snsr_inid_hsc2 = unpack_right_shift_u8(src_p[0], 2u, 0x04u);
    dst_p->strg_whl_ang_snsr_mult_capb_hsc2 = unpack_right_shift_u8(src_p[0], 1u, 0x02u);
    dst_p->strg_whl_ang_snsr_flt_hsc2 = unpack_right_shift_u8(src_p[0], 0u, 0x01u);
    dst_p->strg_whl_ang_hsc2 = unpack_left_shift_u16(src_p[1], 8u, 0xffu);
    dst_p->strg_whl_ang_hsc2 |= unpack_right_shift_u16(src_p[2], 0u, 0xffu);
    dst_p->strg_whl_ang_alv_rchsc2 = unpack_right_shift_u8(src_p[3], 4u, 0xf0u);
    dst_p->strg_whl_ang_grd_hsc2 = unpack_left_shift_u16(src_p[3], 8u, 0x0fu);
    dst_p->strg_whl_ang_grd_hsc2 |= unpack_right_shift_u16(src_p[4], 0u, 0xffu);
    dst_p->strg_whl_ang_extd_pvhsc2 = unpack_left_shift_u16(src_p[5], 8u, 0xffu);
    dst_p->strg_whl_ang_extd_pvhsc2 |= unpack_right_shift_u16(src_p[6], 0u, 0xffu);
    dst_p->strg_whl_ang_snsr_chksm_hsc2 = unpack_right_shift_u8(src_p[7], 0u, 0xffu);

    return (0);
}

int lh1_ep21_p_can2_v12_scs_hsc2_fr_p05_unpack(
    struct lh1_ep21_p_can2_v12_scs_hsc2_fr_p05_t *dst_p,
    const uint8_t *src_p,
    size_t size)
{
    uint16_t veh_dyn_yaw_rate_hsc2;

    if (size < 8u) {
        return (-EINVAL);
    }

    dst_p->virtual_mstr_cylnd_prs_sts_hsc2 = unpack_right_shift_u8(src_p[0], 2u, 0x0cu);
    dst_p->virtual_mstr_cylnd_prs_hsc2 = unpack_left_shift_u16(src_p[0], 8u, 0x03u);
    dst_p->virtual_mstr_cylnd_prs_hsc2 |= unpack_right_shift_u16(src_p[1], 0u, 0xffu);
    dst_p->brk_sys_tr_gear_req_typ_hsc2 = unpack_right_shift_u8(src_p[2], 4u, 0x30u);
    dst_p->brk_sys_tr_gear_req_gear_hsc2 = unpack_right_shift_u8(src_p[2], 0u, 0x0fu);
    dst_p->absahsc2 = unpack_right_shift_u8(src_p[3], 6u, 0x40u);
    dst_p->absfhsc2 = unpack_right_shift_u8(src_p[3], 5u, 0x20u);
    dst_p->tcsahsc2 = unpack_right_shift_u8(src_p[3], 4u, 0x10u);
    dst_p->tcs_enbd_hsc2 = unpack_right_shift_u8(src_p[3], 3u, 0x08u);
    dst_p->en_drag_toq_rducn_ahsc2 = unpack_right_shift_u8(src_p[3], 1u, 0x02u);
    dst_p->vse_sys_ahsc2 = unpack_right_shift_u8(src_p[3], 0u, 0x01u);
    dst_p->veh_dyn_yaw_rate_vhsc2 = unpack_right_shift_u8(src_p[4], 4u, 0x10u);
    veh_dyn_yaw_rate_hsc2 = unpack_left_shift_u16(src_p[4], 8u, 0x0fu);
    veh_dyn_yaw_rate_hsc2 |= unpack_right_shift_u16(src_p[5], 0u, 0xffu);

    if ((veh_dyn_yaw_rate_hsc2 & (1u << 11)) != 0u) {
        veh_dyn_yaw_rate_hsc2 |= 0xf000u;
    }

    dst_p->veh_dyn_yaw_rate_hsc2 = (int16_t)veh_dyn_yaw_rate_hsc2;
    dst_p->absarchsc2 = unpack_right_shift_u8(src_p[6], 0u, 0x0fu);
    dst_p->absa_chksm_hsc2 = unpack_right_shift_u8(src_p[7], 0u, 0xffu);

    return (0);
}

int lh1_ep21_p_can2_v12_scs_hsc2_fr_p24_unpack(
    struct lh1_ep21_p_can2_v12_scs_hsc2_fr_p24_t *dst_p,
    const uint8_t *src_p,
    size_t size)
{
    if (size < 8u) {
        return (-EINVAL);
    }

    dst_p->veh_sdsl_sts_hsc2 = unpack_right_shift_u8(src_p[0], 6u, 0xc0u);
    dst_p->ch_acc_acc_req_resp_hsc2 = unpack_right_shift_u8(src_p[0], 3u, 0x38u);
    dst_p->ch_aeb_hyd_brk_ast_req_resp_hsc2 = unpack_right_shift_u8(src_p[0], 0u, 0x07u);
    dst_p->ch_acc_req_flr_sts_hsc2 = unpack_right_shift_u8(src_p[1], 6u, 0xc0u);
    dst_p->ch_aeb_prfl_req_resp_hsc2 = unpack_right_shift_u8(src_p[1], 3u, 0x38u);
    dst_p->ch_accaeb_brk_jerk_req_resp_hsc2 = unpack_right_shift_u8(src_p[1], 0u, 0x07u);
    dst_p->ch_aeb_dcl_req_resp_hsc2 = unpack_right_shift_u8(src_p[2], 5u, 0xe0u);
    dst_p->ch_accaeb_alv_rlng_ctr_radar_hsc2 = unpack_right_shift_u8(src_p[2], 1u, 0x1eu);
    dst_p->veh_trgt_longt_acc_hsc2 = unpack_right_shift_u8(src_p[3], 0u, 0xffu);
    dst_p->ch_accaeb_chksm_radar_hsc2 = unpack_right_shift_u8(src_p[7], 0u, 0xffu);

    return (0);
}

int lh1_ep21_p_can2_v12_scs_hsc2_fr_p19_unpack(
    struct lh1_ep21_p_can2_v12_scs_hsc2_fr_p19_t *dst_p,
    const uint8_t *src_p,
    size_t size)
{
    if (size < 8u) {
        return (-EINVAL);
    }

    dst_p->veh_spd_avg_pvhsc2 = unpack_left_shift_u16(src_p[0], 8u, 0xffu);
    dst_p->veh_spd_avg_pvhsc2 |= unpack_right_shift_u16(src_p[1], 0u, 0xffu);
    dst_p->veh_spd_avg_vhsc2 = unpack_right_shift_u8(src_p[2], 7u, 0x80u);
    dst_p->veh_spd_avg_hsc2 = unpack_left_shift_u16(src_p[2], 8u, 0x7fu);
    dst_p->veh_spd_avg_hsc2 |= unpack_right_shift_u16(src_p[3], 0u, 0xffu);
    dst_p->veh_spd_avg_alv_rchsc2 = unpack_right_shift_u8(src_p[4], 4u, 0xf0u);
    dst_p->veh_spd_avg_sts_hsc2 = unpack_right_shift_u8(src_p[5], 6u, 0xc0u);
    dst_p->max_estd_whl_prs_hsc2 = unpack_right_shift_u8(src_p[6], 0u, 0xffu);
    dst_p->veh_spd_avg_chksm_hsc2 = unpack_right_shift_u8(src_p[7], 0u, 0xffu);

    return (0);
}

int lh1_ep21_p_can2_v12_gw_hsc2_bcm_fr_p04_unpack(
    struct lh1_ep21_p_can2_v12_gw_hsc2_bcm_fr_p04_t *dst_p,
    const uint8_t *src_p,
    size_t size)
{
    if (size < 8u) {
        return (-EINVAL);
    }

    dst_p->ldspc_open_sts_hsc2 = unpack_right_shift_u8(src_p[0], 6u, 0xc0u);
    dst_p->rr_door_open_sts_hsc2 = unpack_right_shift_u8(src_p[0], 4u, 0x30u);
    dst_p->dircn_ind_lamp_sw_sts_hsc2 = unpack_right_shift_u8(src_p[0], 2u, 0x0cu);
    dst_p->drvr_door_open_sts_hsc2 = unpack_right_shift_u8(src_p[0], 0u, 0x03u);
    dst_p->main_beam_lght_on_hsc2 = unpack_right_shift_u8(src_p[1], 7u, 0x80u);
    dst_p->frt_fog_lght_on_hsc2 = unpack_right_shift_u8(src_p[1], 6u, 0x40u);
    dst_p->dipd_beam_lght_on_hsc2 = unpack_right_shift_u8(src_p[1], 5u, 0x20u);
    dst_p->l_dircn_iohsc2 = unpack_right_shift_u8(src_p[1], 3u, 0x08u);
    dst_p->l_dircn_ind_lght_fhsc2 = unpack_right_shift_u8(src_p[1], 2u, 0x04u);
    dst_p->r_dircn_iohsc2 = unpack_right_shift_u8(src_p[1], 1u, 0x02u);
    dst_p->rr_fog_lght_on_hsc2 = unpack_right_shift_u8(src_p[1], 0u, 0x01u);
    dst_p->bnt_open_sts_hsc2 = unpack_right_shift_u8(src_p[5], 6u, 0xc0u);
    dst_p->dsp_meas_sys_hsc2 = unpack_right_shift_u8(src_p[5], 4u, 0x10u);
    dst_p->frt_psng_door_open_sts_hsc2 = unpack_right_shift_u8(src_p[5], 2u, 0x0cu);
    dst_p->rl_door_open_sts_hsc2 = unpack_right_shift_u8(src_p[5], 0u, 0x03u);
    dst_p->frt_wshr_pump_ahsc2 = unpack_right_shift_u8(src_p[6], 7u, 0x80u);
    dst_p->trlr_hitch_sw_ahsc2 = unpack_right_shift_u8(src_p[6], 6u, 0x40u);
    dst_p->frt_wiper_park_pos_ahsc2 = unpack_right_shift_u8(src_p[6], 4u, 0x10u);
    dst_p->key_det_indx_hsc2 = unpack_right_shift_u8(src_p[6], 1u, 0x0eu);
    dst_p->r_dircn_ind_lght_fhsc2 = unpack_right_shift_u8(src_p[6], 0u, 0x01u);
    dst_p->main_beam_sw_pos_sts_hsc2 = unpack_right_shift_u8(src_p[7], 6u, 0xc0u);
    dst_p->ambt_lght_lvl_hsc2 = unpack_right_shift_u8(src_p[7], 2u, 0x0cu);
    dst_p->veh_side_lght_sts_hsc2 = unpack_right_shift_u8(src_p[7], 0u, 0x03u);

    return (0);
}

int lh1_ep21_p_can2_v12_scs_hsc2_fr_p08_unpack(
    struct lh1_ep21_p_can2_v12_scs_hsc2_fr_p08_t *dst_p,
    const uint8_t *src_p,
    size_t size)
{
    uint16_t latd_acc_snsr_val_hsc2;
    uint16_t longd_acc_snsr_val_hsc2;

    if (size < 8u) {
        return (-EINVAL);
    }

    longd_acc_snsr_val_hsc2 = unpack_left_shift_u16(src_p[0], 8u, 0xffu);
    longd_acc_snsr_val_hsc2 |= unpack_right_shift_u16(src_p[1], 0u, 0xffu);
    dst_p->longd_acc_snsr_val_hsc2 = (int16_t)longd_acc_snsr_val_hsc2;
    latd_acc_snsr_val_hsc2 = unpack_left_shift_u16(src_p[2], 8u, 0xffu);
    latd_acc_snsr_val_hsc2 |= unpack_right_shift_u16(src_p[3], 0u, 0xffu);
    dst_p->latd_acc_snsr_val_hsc2 = (int16_t)latd_acc_snsr_val_hsc2;
    dst_p->longd_acc_snsr_val_vhsc2 = unpack_right_shift_u8(src_p[4], 7u, 0x80u);
    dst_p->longd_latd_acc_snsr_val_archsc2 = unpack_right_shift_u8(src_p[6], 6u, 0xc0u);
    dst_p->longd_latd_acc_snsr_chksm_hsc2 = unpack_left_shift_u16(src_p[6], 8u, 0x07u);
    dst_p->longd_latd_acc_snsr_chksm_hsc2 |= unpack_right_shift_u16(src_p[7], 0u, 0xffu);

    return (0);
}

double lh1_ep21_p_can2_v12_sas_hsc2_fr_p00_strg_whl_ang_hsc2_decode(uint16_t value)
{
    return (((double)value * 0.0625) + -2048.0);
}

double lh1_ep21_p_can2_v12_scs_hsc2_fr_p05_veh_dyn_yaw_rate_hsc2_decode(int16_t value)
{
    return ((double)value * 0.0625);
}

double lh1_ep21_p_can2_v12_scs_hsc2_fr_p08_latd_acc_snsr_val_hsc2_decode(int16_t value)
{
    return ((double)value * 0.000122);
}

double lh1_ep21_p_can2_v12_scs_hsc2_fr_p08_longd_acc_snsr_val_hsc2_decode(int16_t value)
{
    return ((double)value * 0.00012207);
}

double lh1_ep21_p_can2_v12_scs_hsc2_fr_p19_veh_spd_avg_hsc2_decode(uint16_t value)
{
    return ((double)value * 0.015625);
}