#include "discrete_point_curvature_calculator.h"

namespace rm {
namespace common {
namespace math {

float DiscretePointCurvatureCalculator(Vec2d point_1, Vec2d point_2,
                                       Vec2d point_3) {
//  float x1{point_1.x()}, y1{point_1.y()}, x2{point_2.x()}, y2{point_2.y()},
//      x3{point_3.x()}, y3{point_3.y()};
  float x1 = static_cast<float>(point_1.x());
  float y1 = static_cast<float>(point_1.y());
  float x2 = static_cast<float>(point_2.x());
  float y2 = static_cast<float>(point_2.y());
  float x3 = static_cast<float>(point_3.x());
  float y3 = static_cast<float>(point_3.y());
  float a, b, c, d, e, f, g, X, Y, R;
  e = 2 * (x2 - x1);
  f = 2 * (y2 - y1);
  g = x2 * x2 - x1 * x1 + y2 * y2 - y1 * y1;
  a = 2 * (x3 - x2);
  b = 2 * (y3 - y2);
  c = x3 * x3 - x2 * x2 + y3 * y3 - y2 * y2;
  X = (g * b - c * f) / (e * b - a * f);
  Y = (a * g - c * e) / (a * f - b * e);
  R = std::sqrt((X - x1) * (X - x1) + (Y - y1) * (Y - y1));
  return std::min(1 / R, 0.1F);
}

float DiscretePointCurvatureCalculator(
    dpc::SamplingPoint point_1,
    dpc::SamplingPoint point_2,
    dpc::SamplingPoint point_3) {
  float x1{point_1.x}, y1{point_1.y}, x2{point_2.x}, y2{point_2.y},
      x3{point_3.x}, y3{point_3.y};
  float a, b, c, d, e, f, g, X, Y, R;
  e = 2 * (x2 - x1);
  f = 2 * (y2 - y1);
  g = x2 * x2 - x1 * x1 + y2 * y2 - y1 * y1;
  a = 2 * (x3 - x2);
  b = 2 * (y3 - y2);
  c = x3 * x3 - x2 * x2 + y3 * y3 - y2 * y2;
  X = (g * b - c * f) / (e * b - a * f);
  Y = (a * g - c * e) / (a * f - b * e);
  R = std::sqrt((X - x1) * (X - x1) + (Y - y1) * (Y - y1));
  return std::min(1 / R, 0.1F);
}
} // namespace math
} // namespace common
} // namespace rm
