/*
 * @Author: your name
 * @Date: 2021-12-16 08:18:41
 * @LastEditTime: 2021-12-16 11:11:44
 * @LastEditors: your name
 * @Description: 打开koroFileHeader查看配置 进行设置:
 * https://github.com/OBKoro1/koro1FileHeader/wiki/%E9%85%8D%E7%BD%AE
 * @FilePath:
 * /EHR/home/igs/roadmodel/src/core/source/common/math/extend_kalman_filter_object.cc
 */
#include "extend_kalman_filter_object.h"
#include <array>
// state_vec_ = (x y v a phi phi_dot)
namespace rm {
namespace common {
namespace math {

const double ExtendKalmanFilterObject::t_ = 0.04;
Eigen::Matrix<double, 6, 6> ExtendKalmanFilterObject::coord_transf_ =
    Eigen::Matrix<double, 6, 6>::Zero();
Eigen::Matrix<double, 6, 1> ExtendKalmanFilterObject::coord_offset_ =
    Eigen::Matrix<double, 6, 1>::Zero();

ExtendKalmanFilterObject::ExtendKalmanFilterObject() {
//   process_cov_ = Eigen::Matrix<double, 6, 6>::Zero();
//   meas_cov_ = Eigen::Matrix<double, 6, 6>::Zero();
  Eigen::Matrix<double, 1, 6> process_cov_vec, meas_cov_vec;
  process_cov_vec << 0.04, 0.08, 0.2, 0.025, 0.03, 1e-8;
  process_cov_ = process_cov_vec.asDiagonal();
  meas_cov_vec << 0.04, 0.04, 0.2, 0.025, 0.03, 1e-8;
  meas_cov_ = meas_cov_vec.asDiagonal();
//   process_cov_ = 0.1 * Eigen::Matrix<double, 6, 6>::Identity();
//   meas_cov_ = 0.3 * Eigen::Matrix<double, 6, 6>::Identity();
  jacob_process_ =
      Eigen::Matrix<double, 6, 6>::Identity(); // DO NOT CHANGE THIS VALUE
}

ExtendKalmanFilterObject::~ExtendKalmanFilterObject() {}

void ExtendKalmanFilterObject::Init(const std::vector<double> &meas) {
  // cout << observer_->getDynamicConfig()->road_info_provider_config->int_param
  // << endl;
  state_vec_ << meas[0], meas[1], meas[2], meas[3], meas[4], meas[5];
  state_cov_ = 0.01 * Eigen::Matrix<double, 6, 6>::Identity();
}

void ExtendKalmanFilterObject::Set_CoordinateTransformMat(const double dx,
                                                          const double dangle) {
  coord_transf_(0, 0) = cos(dangle);
  coord_transf_(0, 1) = sin(dangle);
  coord_transf_(1, 0) = -sin(dangle);
  coord_transf_(1, 1) = cos(dangle);
  coord_offset_(0) = -dx;
  coord_offset_(4) = -dangle;
}

void ExtendKalmanFilterObject::CorrdinateTransform() {
  state_vec_ =
      coord_transf_ * state_vec_ + coord_offset_; // state in new coordinate
  state_cov_ = coord_transf_ * state_cov_ *
               state_cov_.transpose(); // state covariance in new coordinate
}

// void ExtendKalmanFilterObject::UpdateParam(const std::shared_ptr<Observer>
// &observer)
// {
//     // process_cov_(0, 0) =
//     observer->getDynamicConfig()->road_info_provider_config->proc_x_cov;
//     // process_cov_(1, 1) =
//     observer->getDynamicConfig()->road_info_provider_config->proc_y_cov;
//     // process_cov_(2, 2) =
//     observer->getDynamicConfig()->road_info_provider_config->proc_v_cov;
//     // process_cov_(3, 3) =
//     observer->getDynamicConfig()->road_info_provider_config->proc_a_cov;
//     // process_cov_(4, 4) =
//     observer->getDynamicConfig()->road_info_provider_config->proc_yaw_cov;
//     // process_cov_(5, 5) =
//     observer->getDynamicConfig()->road_info_provider_config->proc_yawrate_cov;
//     // meas_cov_(0, 0) =
//     observer->getDynamicConfig()->road_info_provider_config->meas_x_cov;
//     // meas_cov_(1, 1) =
//     observer->getDynamicConfig()->road_info_provider_config->meas_y_cov;
//     // meas_cov_(2, 2) =
//     observer->getDynamicConfig()->road_info_provider_config->meas_v_cov;
//     // meas_cov_(3, 3) =
//     observer->getDynamicConfig()->road_info_provider_config->meas_a_cov;
//     // meas_cov_(4, 4) =
//     observer->getDynamicConfig()->road_info_provider_config->meas_yaw_cov;
//     // meas_cov_(5, 5) =
//     observer->getDynamicConfig()->road_info_provider_config->meas_yawrate_cov;
// }

void ExtendKalmanFilterObject::Predict() {
  CorrdinateTransform();
  jacob_process_(0, 2) = cos(state_vec_(4)) * t_;
  jacob_process_(0, 4) = -state_vec_(2) * t_ * sin(state_vec_(4));
  jacob_process_(1, 2) = sin(state_vec_(4)) * t_;
  jacob_process_(1, 4) = state_vec_(2) * t_ * cos(state_vec_(4));
  jacob_process_(2, 3) = t_;
  jacob_process_(4, 5) = t_;

  state_vec_(0) = state_vec_(0) + cos(state_vec_(4)) * state_vec_(2) * t_; // x
  state_vec_(1) = state_vec_(1) + sin(state_vec_(4)) * state_vec_(2) * t_; // y
  state_vec_(2) = state_vec_(2) + state_vec_(3) * t_;                      // v
  // state_vec_(3) = state_vec_(3)// a
  state_vec_(4) = state_vec_(4) + state_vec_(5) * t_; // phi
  // state_vec_(5) = // phi_dot

  state_cov_ =
      jacob_process_ * state_cov_ * jacob_process_.transpose() + process_cov_;

  filter_status_ = FilterStatus::PREDICT;
}

bool ExtendKalmanFilterObject::Correct(const std::vector<double> &meas) {
  Eigen::Matrix<double, 6, 6> temp;
  Eigen::Matrix<double, 6, 1> meas_vec;
  meas_vec << meas[0], meas[1], meas[2], meas[3], meas[4], meas[5];
  temp = state_cov_ + meas_cov_; // jacobian of measurement is identity
  Eigen::FullPivLU<Eigen::Matrix<double, 6, 6>> LU(temp);
  if (LU.isInvertible()) {
    kalman_gain_ = state_cov_ * temp.inverse();
    state_vec_ = state_vec_ + kalman_gain_ * (meas_vec - state_vec_);
    state_cov_ =
        (Eigen::Matrix<double, 6, 6>::Identity() - kalman_gain_) * state_cov_;
    filter_status_ = FilterStatus::UPDATE;
    return true;
  } else {
    // std::cout << "error" << std::endl;
    filter_status_ = FilterStatus::INVALID;
    return false;
  }
}

void ExtendKalmanFilterObject::GetState(std::array<double, 6> &filtered_state) {
  for (int i = 0; i < 6; i++) {
    filtered_state.at(i) = state_vec_(i);
//    filtered_state[i] = state_vec_(i);

  }
}

} // namespace math
} // namespace common
} // namespace rm
