/*
 * @Author: your name
 * @Date: 2021-09-16 18:10:55
 * @LastEditTime: 2021-11-02 15:16:05
 * @LastEditors: Please set LastEditors
 * @Description: In User Settings Edit
 * @FilePath: /NOP_EHR/media/igs/xnm/ros/RM/roadmodel/modules/common/math/polynom_line.h
 */

/**
 * @file
 **/

#pragma once

#include <array>
#include <string>
#include <vector>
#include <list>
#include "polynomial_curve1d.h"
#include "reference_point.h"
#include "macros.h"
#include "vec2d.h"
#include "safe_divide.h"


namespace rm {
  using  common::math::Vec2d;
// TODO(xnm) 需要添加 及 点到三次多项式的距离和点函数  
class PolynomLine : public PolynomialCurve1d {
 public:
  PolynomLine() {
      start_range_ = 0.0;
      end_range_ = 50.0;
  }
  virtual ~PolynomLine() = default;
  
  PolynomLine(std::array<double, 4> coef , double start_range ,double end_range);


  void DerivedFromQuarticCurve(const PolynomialCurve1d& other);

  double Evaluate(const std::uint32_t order, const double p) const override;

  double ParamLength() const override { return param_; }
  std::string ToString() const override;

  double Coef(const size_t order) const override;
  double EndCondition()const {return end_range_;}
  size_t Order() const override { return 3; }

  double GetKappa(double x);
  //根据曲线方程和离散间隔step  离散曲线生成点信息
  std::vector<common::math::ReferencePoint> DiscreteCurved(double end_range, double unit_x);
  std::vector<common::math::ReferencePoint> DiscreteCurved(double unit_x);

  //平移三次多项式 得到新的三次多项式dy>0时 向左平移  dy<0时 向右平移
  PolynomLine MoveCurved(double dy, bool is_front_marker);

  PolynomLine MoveCurved(double dy);
  
  double start_range()const {return start_range_;}
  double end_range()const {return end_range_;}

  void set_start_range(const double start_range){start_range_ = start_range;}
  void set_end_range(const double end_range){end_range_ = end_range;}
  void set_coef(const double c0,const double c1,const double c2,const double c3){
    coef_[0] = c0;
    coef_[1] = c1;
    coef_[2] = c2;
    coef_[3] = c3;
  }
  
  friend std::ostream& operator<<(std::ostream &os, const PolynomLine&  line){
    os<<"\n"
      <<"start_range_ = "<<line.start_range_<<"\n"
      <<"end_range_ = "<<line.end_range_<<"\n"
      <<"coef_ = "<<line.Coef(0)<<" "<<line.Coef(1)<<" "<<line.Coef(2)<<" "<<line.Coef(3);
    return os;
  }
  double ComputeArc(double x);
  double DistanceToCurve(Vec2d point, double x);
  double ComputeArcLengthbySimpson(double start_point, double end_point); // approximate the arc length using simpson method
  double ComputeArcLengthbySimpson(); // approximate the arc length using simpson method form start range to end range
  double CalcMinCartDist2AnotherLine(const PolynomLine& line);

 private:
  void ComputeCoefficients(const double x0, const double dx0, const double ddx0,
                           const double x1, const double param);
  std::array<double, 4> coef_ = {{0.0, 0.0, 0.0, 0.0}};
  double start_range_ = 0.0;
  double end_range_ = 0.0;
};

}  // namespace rm
