#include "state_machine/state_machine.h"
#include <array>
#include "glog/logging.h"

namespace rm {
namespace state_machine {

StateMachine::StateMachine() {}

void StateMachine::SetInitializeState(const int state_id) {
  std::unique_lock<std::mutex> lck(state_mtx_);
  auto it = states_->find(state_id);
  if (it != states_->end()) {
    current_state_ = it->second;
    // DLOG(INFO) << "set initilized state: " << current_state_->Id();
  }
}

bool StateMachine::ChangeState(const int &change_flag) {
  std::unique_lock<std::mutex> lck(state_mtx_);
  if (current_state_ == nullptr) {
    DLOG(ERROR) << "state not initialied";
    return false;
  }
  auto next_state_ptr = current_state_->ChangeState(change_flag);
  if (next_state_ptr != nullptr) {
    change_flag_ = change_flag;
    current_state_ = next_state_ptr;
    return true;
  } else {
    if (change_flag_ != change_flag) {
      change_flag_ = change_flag;
      DLOG(WARNING) << "change state error: " << change_flag
                    << "prev: " << current_state_->Id() << " " << change_flag_;
    }
    return false;
  }
}

void StateMachine::LoadStateMachine(
    const std::vector<std::array<int, 3>> &transition) {
  std::unique_lock<std::mutex> lck(state_mtx_);
  for (const auto &it : transition) {
    if (states_->find(it[0]) == states_->end()) {
      StatePtr state = std::make_shared<State>(it[0]);
      states_->insert(std::make_pair(it[0], state));
    }
    if (states_->find(it[2]) == states_->end()) {
      StatePtr state = std::make_shared<State>(it[2]);
      states_->insert(std::make_pair(it[2], state));
    }
    auto it_from = states_->find(it[0]);
    auto it_to = states_->find(it[2]);
    if (it_from == states_->end() || it_to == states_->end()) {
      DLOG(ERROR) << "load transition error! from_state: " << it[0]
                  << "changeflag: " << it[1] << "to_state: " << it[2];
    } else {
      it_from->second->AddTransition(it[1], it_to->second);
    }
  }
}

}  // namespace state_machine
}  // namespace rm
