#pragma once

#include <atomic>
#include <map>
#include <memory>
// #include <rclcpp/rclcpp.hpp>
#include <string>

namespace rm {

struct RMConfig {
private:
  RMConfig() {}
  RMConfig(const RMConfig &) = delete;
  RMConfig &operator=(const RMConfig &) = delete;

public:
  static RMConfig *Instance() {
    static RMConfig instance;
    return &instance;
  }

public:
  std::string node{"roadmodel"};
  std::string topic_rm2sm{"mpc_interface_drv_rm_p0040ms_rm2sm"};
  std::string topic_rm2ba{"mpc_interface_drv_rm_p0040ms_rm2ba"};
  std::string topic_rm2trplon{"mpc_interface_drv_rm_p0040ms_rm2trplon"};

  //******** ilc config *********
  bool ilc_switch{true};
  bool ilc_roadedge_switch{true};
  bool ilc_car_ride_marker_switch{true};
  bool ilc_disable_by_lane_type{true};
  double ilc_lane_width_enable{3.4};
  double ilc_lane_width_disable{3.2};
  double ilc_curve_radius_enable{800.0};
  double ilc_curve_radius_disable{600.0};
  int64_t ilc_speed_lower_limit_enable{45};
  int64_t ilc_speed_lower_limit_disable{40};
  int64_t ilc_speed_upper_limit_enable{130};
  int64_t ilc_speed_upper_limit_disable{135};
  int64_t trplon_curv_sampling_point_gap{2};
  double trplon_curv_sampling_point_start_offset{30.0};
  double ilc_overtaking_ttc{8.0};
  double ilc_overtaked_ttc{3.0};
  double object_in_range_overtaking_ttc{5.0};
  double object_in_range_overtaked_ttc{2.0};
  bool ilc_distance_compensation_enable{true};
  double ilc_distance_compensation_factor{0.1};
  double ilc_base_gap_front{15.0};
  double ilc_base_gap_front_overtaked{6.0};
  double ilc_base_gap_back{1.0};
  bool ilc_overtaking_compensation_enable{true};
  double ilc_overtaking_compensation_factor{0.3};
  double ilc_max_overtaking_gap{80.0};
  double ilc_max_overtaked_gap{30.0};
  double object_in_range_max_gap{30.0};
  double object_in_range_min_gap{3.0};
  double ilc_marker2edge_enable{0.5};
  double ilc_marker2edge_disable{0.6};
  double ilc_refline2edge_enable{2.5};
  double ilc_refline2edge_disable{2.7};
  bool ilc_table_calibration_enable{false};
  std::vector<double> ilc_base_lateral_offset_col{3.2, 3.4, 3.6, 3.8, 4.0, 4.2};
  std::vector<double> ilc_base_lateral_offset_value{0.2,  0.2, 0.25,
                                                    0.25, 0.3, 0.3};
  std::vector<double> ilc_road_edge_lateral_offset_value{0.1, 0.15, 0.15,
                                                         0.2, 0.2,  0.2};
  bool ilc_lateral_offset_compensation_enable{false};
  std::vector<double> ilc_lateral_offset_compensation_col{1.6, 1.8, 2.0,
                                                          2.2, 2.4, 2.6};
  std::vector<double> ilc_lateral_offset_compensation_value{0.1, 0.1, 0.0,
                                                            0.0, 0.0, 0.0};
  double ilc_min_gap_to_lane_marker{0.2};
  double ilc_object_lateral_compensation{0.1};
  double ilc_max_lateral_offset{0.3};
  double ilc_min_lateral_offset{0.1};
  double ilc_road_edge_in_range{1.0};
  double ilc_road_edge_out_range{1.2};
  double boundary_rdedg_lnmrk_dis_threshold{2.7};
  double corridor_generation_width_threshold{10.0};
  // object assignment
  std::vector<double> in_corridor_upper_long_dis_ego_velocity_table_x{
      30.0, 50.0, 70.0, 80.0, 100.0, 120.0};
  std::vector<double> in_corridor_upper_long_dis_ego_velocity_table_y{
      55.0, 75.0, 105.0, 115.0, 135.0, 155.0};
  std::vector<double> in_corridor_lower_long_dis_ego_velocity_table_x{
      30.0, 50.0, 70.0, 80.0, 100.0, 120.0};
  std::vector<double> in_corridor_lower_long_dis_ego_velocity_table_y{
      50.0, 70.0, 100.0, 110.0, 130.0, 150.0};
  std::vector<double> in_corridor_long_dist_inner_table_x{
      -100.0, -20.0, -15.0, -10.0, 0.0,  5.0,  10.0, 15.0,
      20.0,   30.0,  40.0,  50.0,  60.0, 80.0, 100.0};
  std::vector<double> in_corridor_long_dist_inner_table_y{
      1.0, 1.0, 1.0, 1.5, 2.0, 1.1, 1.0, 1.0,
      1.0, 1.0, 1.0, 1.5, 1.5, 4.0, 4.0}; // factor
  std::vector<double> in_corridor_long_dist_outer_table_x{
      -100.0, -20.0, -15.0, -10.0, 10.0, 15.0, 20.0, 30.0, 40.0, 50.0};
  std::vector<double> in_corridor_long_dist_outer_table_y{
      1.0, 1.0, 1.0, 1.0, 0.8, 0.8, 0.9, 1.0, 1.0, 1.0}; // factor
  std::vector<double> in_corridor_inner_width_table_x{2.4, 2.8, 3.2,
                                                      3.6, 4.0, 4.4};
  std::vector<double> in_corridor_inner_width_table_y{-0.1, -0.1, -0.1, -0.2,
                                                      -0.3, -0.35}; // base
  std::vector<double> in_corridor_outer_width_table_x{2.4, 2.8, 3.2,
                                                      3.6, 4.0, 4.4};
  std::vector<double> in_corridor_outer_width_table_y{0.1, 0.1, 0.1,
                                                      0.1, 0.1, 0.2};
  std::vector<double> in_corridor_ego_position_left_table_x{-1.2, -1.0, -0.8,
                                                            -0.2};
  std::vector<double> in_corridor_ego_position_left_table_y{1.2, 1.0, 0.8, 0.0};
  std::vector<double> in_corridor_ego_position_right_table_x{0.2, 0.8, 1.0,
                                                             1.2};
  std::vector<double> in_corridor_ego_position_right_table_y{0.0, -0.8, -1.0,
                                                             -1.2};
  std::vector<double> in_corridor_ego_position_table_x{-1.2, -1.0, -0.8, -0.2,
                                                       0.2,  0.8,  1.0,  1.2};
  std::vector<double> in_corridor_ego_position_table_y{0.5, 0.4,  0.2,  0.0,
                                                       0.0, -0.2, -0.4, -0.5};
  std::vector<double> width_factor_dl_car_x{0.0, 0.1, 0.2, 0.3, 0.4,
                                            0.5, 0.6, 0.7, 0.8, 1.0};
  std::vector<double> width_factor_dl_car_y{0.0, 0.2, 0.5, 0.7, 1.0,
                                            1.0, 1.0, 1.2, 1.2, 1.5};
  std::vector<double> width_factor_dl_trunck_x{0.0, 0.1, 0.2, 0.3, 0.4,
                                               0.5, 0.6, 0.7, 0.8, 1.0};
  std::vector<double> width_factor_dl_trunck_y{0.0, 0.5, 0.9, 1.0, 1.1,
                                               1.2, 1.3, 1.4, 1.4, 1.5};
  std::vector<double> width_factor_s_x{-1.0, 0.0,  1.0,  5.5,  6.0,
                                       8.0,  10.0, 15.0, 20.0, 30.0,
                                       40.0, 50.0, 60.0, 80.0, 100.0};
  std::vector<double> width_factor_s_y{1.0,  0,    0.2,  0.5,  0.9,
                                       0.9,  0.95, 0.95, 0.95, 0.98,
                                       0.98, 1.0,  1.0,  1.0,  1.0};
  std::vector<double> width_factor_ego_yaw_rate_table_x{0.0, 1.0, 2.0, 2.5,
                                                        3.0, 3.5, 4.0};
  std::vector<double> width_factor_ego_yaw_rate_table_y{1.0, 0.85, 0.8, 0.8,
                                                        0.5, 0.2,  0.0};
  std::vector<double> ats_switch_decider_override_hand_trq_ego_velocity_table_x{
      0.0, 10.0, 15.0, 20.0, 25.0, 30.0, 40.0, 50.0};
  std::vector<double>
      ats_switch_decider_override_hand_trq_upper_ego_velocity_table_y{
          1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5};
  std::vector<double>
      ats_switch_decider_override_hand_trq_lower_ego_velocity_table_y{
          1.2, 1.2, 1.2, 1.2, 1.2, 1.2, 1.2, 1.2};
  std::vector<double> ats_switch_decider_counter_threshold_ego_velocity_table_x{
      0.0, 10.0, 15.0, 20.0, 25.0, 30.0, 40.0};
  std::vector<double> ats_switch_decider_counter_threshold_ego_velocity_table_y{
      65.0, 55.0, 50.0, 50.0, 50.0, 50.0, 50.0};
};

// static inline void BindDynamicConfigParams(
//     std::map<std::string, std::pair<rclcpp::ParameterType, void *>>
//         &dynamic_params,
//     RMConfig *config) {
//   /*
//     rclcpp::ParameterType::PARAMETER_BOOL
//     rclcpp::ParameterType::PARAMETER_INTEGER
//     rclcpp::ParameterType::PARAMETER_DOUBLE
//     rclcpp::ParameterType::PARAMETER_STRING
//     rclcpp::ParameterType::PARAMETER_INTEGER_ARRAY
//     rclcpp::ParameterType::PARAMETER_DOUBLE_ARRAY
//   */

//   dynamic_params["node"] = std::pair<rclcpp::ParameterType, void *>(
//       rclcpp::ParameterType::PARAMETER_STRING, &(config->node));
//   dynamic_params["topic_rm2sm"] = std::pair<rclcpp::ParameterType, void *>(
//       rclcpp::ParameterType::PARAMETER_STRING, &(config->topic_rm2sm));
//   dynamic_params["topic_rm2ba"] = std::pair<rclcpp::ParameterType, void *>(
//       rclcpp::ParameterType::PARAMETER_STRING, &(config->topic_rm2ba));
//   dynamic_params["topic_rm2trplon"] = std::pair<rclcpp::ParameterType, void *>(
//       rclcpp::ParameterType::PARAMETER_STRING, &(config->topic_rm2trplon));

//   //******** ilc config *********
//   dynamic_params["ilc_switch"] = std::pair<rclcpp::ParameterType, void *>(
//       rclcpp::ParameterType::PARAMETER_BOOL, &(config->ilc_switch));
//   dynamic_params["ilc_roadedge_switch"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_BOOL,
//           &(config->ilc_roadedge_switch));
//   dynamic_params["ilc_car_ride_marker_switch"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_BOOL,
//           &(config->ilc_car_ride_marker_switch));
//   dynamic_params["ilc_disable_by_lane_type"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_BOOL,
//           &(config->ilc_disable_by_lane_type));
//   dynamic_params["ilc_lane_width_enable"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE,
//           &(config->ilc_lane_width_enable));
//   dynamic_params["ilc_lane_width_disable"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE,
//           &(config->ilc_lane_width_disable));
//   dynamic_params["ilc_curve_radius_enable"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE,
//           &(config->ilc_curve_radius_enable));
//   dynamic_params["ilc_curve_radius_disable"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE,
//           &(config->ilc_curve_radius_disable));
//   dynamic_params["ilc_speed_lower_limit_enable"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_INTEGER,
//           &(config->ilc_speed_lower_limit_enable));
//   dynamic_params["ilc_speed_lower_limit_disable"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_INTEGER,
//           &(config->ilc_speed_lower_limit_disable));
//   dynamic_params["ilc_speed_upper_limit_enable"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_INTEGER,
//           &(config->ilc_speed_upper_limit_enable));
//   dynamic_params["ilc_speed_upper_limit_disable"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_INTEGER,
//           &(config->ilc_speed_upper_limit_disable));
//   dynamic_params["trplon_curv_sampling_point_gap"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_INTEGER,
//           &(config->trplon_curv_sampling_point_gap));
//   dynamic_params["trplon_curv_sampling_point_start_offset"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE,
//           &(config->trplon_curv_sampling_point_start_offset));
//   dynamic_params["ilc_overtaking_ttc"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE,
//           &(config->ilc_overtaking_ttc));
//   dynamic_params["ilc_overtaked_ttc"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE,
//           &(config->ilc_overtaked_ttc));
//   dynamic_params["object_in_range_overtaking_ttc"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE,
//           &(config->object_in_range_overtaking_ttc));
//   dynamic_params["object_in_range_overtaked_ttc"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE,
//           &(config->object_in_range_overtaked_ttc));
//   dynamic_params["ilc_distance_compensation_enable"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_BOOL,
//           &(config->ilc_distance_compensation_enable));
//   dynamic_params["ilc_distance_compensation_factor"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE,
//           &(config->ilc_distance_compensation_factor));
//   dynamic_params["ilc_base_gap_front"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE,
//           &(config->ilc_base_gap_front));
//   dynamic_params["ilc_base_gap_front_overtaked"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE,
//           &(config->ilc_base_gap_front_overtaked));
//   dynamic_params["ilc_base_gap_back"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE,
//           &(config->ilc_base_gap_back));
//   dynamic_params["ilc_overtaking_compensation_enable"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_BOOL,
//           &(config->ilc_overtaking_compensation_enable));
//   dynamic_params["ilc_overtaking_compensation_factor"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE,
//           &(config->ilc_overtaking_compensation_factor));
//   dynamic_params["ilc_max_overtaking_gap"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE,
//           &(config->ilc_max_overtaking_gap));
//   dynamic_params["ilc_max_overtaked_gap"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE,
//           &(config->ilc_max_overtaked_gap));
//   dynamic_params["object_in_range_max_gap"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE,
//           &(config->object_in_range_max_gap));
//   dynamic_params["object_in_range_min_gap"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE,
//           &(config->object_in_range_min_gap));
//   dynamic_params["ilc_marker2edge_enable"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE,
//           &(config->ilc_marker2edge_enable));
//   dynamic_params["ilc_marker2edge_disable"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE,
//           &(config->ilc_marker2edge_disable));
//   dynamic_params["ilc_refline2edge_enable"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE,
//           &(config->ilc_refline2edge_enable));
//   dynamic_params["ilc_refline2edge_disable"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE,
//           &(config->ilc_refline2edge_disable));
//   dynamic_params["ilc_table_calibration_enable"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_BOOL,
//           &(config->ilc_table_calibration_enable));
//   dynamic_params["ilc_base_lateral_offset_col"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE_ARRAY,
//           &(config->ilc_base_lateral_offset_col));
//   dynamic_params["ilc_base_lateral_offset_value"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE_ARRAY,
//           &(config->ilc_base_lateral_offset_value));
//   dynamic_params["ilc_road_edge_lateral_offset_value"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE_ARRAY,
//           &(config->ilc_road_edge_lateral_offset_value));
//   dynamic_params["ilc_lateral_offset_compensation_enable"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_BOOL,
//           &(config->ilc_lateral_offset_compensation_enable));
//   dynamic_params["ilc_lateral_offset_compensation_col"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE_ARRAY,
//           &(config->ilc_lateral_offset_compensation_col));
//   dynamic_params["ilc_lateral_offset_compensation_value"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE_ARRAY,
//           &(config->ilc_lateral_offset_compensation_value));
//   dynamic_params["ilc_min_gap_to_lane_marker"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE,
//           &(config->ilc_min_gap_to_lane_marker));
//   dynamic_params["ilc_object_lateral_compensation"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE,
//           &(config->ilc_object_lateral_compensation));
//   dynamic_params["ilc_max_lateral_offset"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE,
//           &(config->ilc_max_lateral_offset));
//   dynamic_params["ilc_min_lateral_offset"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE,
//           &(config->ilc_min_lateral_offset));
//   dynamic_params["ilc_road_edge_in_range"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE,
//           &(config->ilc_road_edge_in_range));
//   dynamic_params["ilc_road_edge_out_range"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE,
//           &(config->ilc_road_edge_out_range));
//   dynamic_params["boundary_rdedg_lnmrk_dis_threshold"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE,
//           &(config->boundary_rdedg_lnmrk_dis_threshold));
//   dynamic_params["corridor_generation_width_threshold"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE,
//           &(config->corridor_generation_width_threshold));
//   //
//   dynamic_params["in_corridor_upper_long_dis_ego_velocity_table_x"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE_ARRAY,
//           &(config->in_corridor_upper_long_dis_ego_velocity_table_x));
//   dynamic_params["in_corridor_upper_long_dis_ego_velocity_table_y"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE_ARRAY,
//           &(config->in_corridor_upper_long_dis_ego_velocity_table_y));
//   dynamic_params["in_corridor_lower_long_dis_ego_velocity_table_x"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE_ARRAY,
//           &(config->in_corridor_lower_long_dis_ego_velocity_table_x));
//   dynamic_params["in_corridor_lower_long_dis_ego_velocity_table_y"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE_ARRAY,
//           &(config->in_corridor_lower_long_dis_ego_velocity_table_y));
//   dynamic_params["in_corridor_long_dist_inner_table_x"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE_ARRAY,
//           &(config->in_corridor_long_dist_inner_table_x));
//   dynamic_params["in_corridor_long_dist_inner_table_y"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE_ARRAY,
//           &(config->in_corridor_long_dist_inner_table_y));
//   dynamic_params["in_corridor_long_dist_outer_table_x"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE_ARRAY,
//           &(config->in_corridor_long_dist_outer_table_x));
//   dynamic_params["in_corridor_long_dist_outer_table_y"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE_ARRAY,
//           &(config->in_corridor_long_dist_outer_table_y));
//   dynamic_params["in_corridor_inner_width_table_x"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE_ARRAY,
//           &(config->in_corridor_inner_width_table_x));
//   dynamic_params["in_corridor_inner_width_table_y"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE_ARRAY,
//           &(config->in_corridor_inner_width_table_y));
//   dynamic_params["in_corridor_outer_width_table_x"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE_ARRAY,
//           &(config->in_corridor_outer_width_table_x));
//   dynamic_params["in_corridor_outer_width_table_y"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE_ARRAY,
//           &(config->in_corridor_outer_width_table_y));
//   dynamic_params["in_corridor_ego_position_left_table_x"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE_ARRAY,
//           &(config->in_corridor_ego_position_left_table_x));
//   dynamic_params["in_corridor_ego_position_left_table_y"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE_ARRAY,
//           &(config->in_corridor_ego_position_left_table_y));
//   dynamic_params["in_corridor_ego_position_right_table_x"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE_ARRAY,
//           &(config->in_corridor_ego_position_right_table_x));
//   dynamic_params["in_corridor_ego_position_right_table_y"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE_ARRAY,
//           &(config->in_corridor_ego_position_right_table_y));
//   dynamic_params["in_corridor_ego_position_table_x"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE_ARRAY,
//           &(config->in_corridor_ego_position_table_x));
//   dynamic_params["in_corridor_ego_position_table_y"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE_ARRAY,
//           &(config->in_corridor_ego_position_table_y));
//   dynamic_params["width_factor_dl_car_x"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE_ARRAY,
//           &(config->width_factor_dl_car_x));
//   dynamic_params["width_factor_dl_car_y"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE_ARRAY,
//           &(config->width_factor_dl_car_y));
//   dynamic_params["width_factor_dl_trunck_x"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE_ARRAY,
//           &(config->width_factor_dl_trunck_x));
//   dynamic_params["width_factor_dl_trunck_y"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE_ARRAY,
//           &(config->width_factor_dl_trunck_y));
//   dynamic_params["width_factor_s_x"] = std::pair<rclcpp::ParameterType, void *>(
//       rclcpp::ParameterType::PARAMETER_DOUBLE_ARRAY,
//       &(config->width_factor_s_x));
//   dynamic_params["width_factor_s_y"] = std::pair<rclcpp::ParameterType, void *>(
//       rclcpp::ParameterType::PARAMETER_DOUBLE_ARRAY,
//       &(config->width_factor_s_y));
//   dynamic_params["width_factor_ego_yaw_rate_table_x"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE_ARRAY,
//           &(config->width_factor_ego_yaw_rate_table_x));
//   dynamic_params["width_factor_ego_yaw_rate_table_y"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE_ARRAY,
//           &(config->width_factor_ego_yaw_rate_table_y));

//   dynamic_params["ats_switch_decider_override_hand_trq_ego_velocity_table_x"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE_ARRAY,
//           &(config->ats_switch_decider_override_hand_trq_ego_velocity_table_x));
//   dynamic_params["ats_switch_decider_override_hand_trq_upper_ego_velocity_"
//                  "table_y"] = std::pair<rclcpp::ParameterType,
//                                         void *>(
//       rclcpp::ParameterType::PARAMETER_DOUBLE_ARRAY,
//       &(config
//             ->ats_switch_decider_override_hand_trq_upper_ego_velocity_table_y));
//   dynamic_params["ats_switch_decider_override_hand_trq_lower_ego_velocity_"
//                  "table_y"] = std::pair<rclcpp::ParameterType,
//                                         void *>(
//       rclcpp::ParameterType::PARAMETER_DOUBLE_ARRAY,
//       &(config
//             ->ats_switch_decider_override_hand_trq_lower_ego_velocity_table_y));
//   dynamic_params["ats_switch_decider_counter_threshold_ego_velocity_table_x"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE_ARRAY,
//           &(config->ats_switch_decider_counter_threshold_ego_velocity_table_x));
//   dynamic_params["ats_switch_decider_counter_threshold_ego_velocity_table_y"] =
//       std::pair<rclcpp::ParameterType, void *>(
//           rclcpp::ParameterType::PARAMETER_DOUBLE_ARRAY,
//           &(config->ats_switch_decider_counter_threshold_ego_velocity_table_y));
// }

inline const RMConfig &GetRMConfig() { return *(RMConfig::Instance()); }

} // namespace rm
