#pragma once

#include "ats_switch_decider.h"

namespace rm {

void ATSSwitchNoAction::Handle(ATSSwitchDeciderStateMachine *sm) {
  // DLOG(INFO) << "------noaction------";
  // std::cout << "------noaction------" << std::endl;
  bool abs_hand_trq_cond = abs_hand_trq_dd_pair_.debouncer.RunDebouceMoreTure(
      std::abs(sm->input_info().column_trq));
  bool abs_hand_trq_cond_delay =
      abs_hand_trq_dd_pair_.delay.RunDelay(abs_hand_trq_cond);
  if (abs_hand_trq_cond_delay) {
    sm->ChangeState(new ATSSwitchPassive());
  } else {
    // do nothing
  }
}

void ATSSwitchPassive::Handle(ATSSwitchDeciderStateMachine *sm) {
  // DLOG(INFO) << "------passive------";
  // std::cout << "------passive------" << std::endl;
  // passive to active condition
  // heading condition at object location
  bool object_heading_cond = CheckObjectHeadingCond(sm);
  double abs_ref_curv = std::abs(sm->input_info().ref_curv_cog);
  ref_curv_diff_dd_pair_p2a_.debouncer.SetDebounce(
      curv_diff_lower_threshold_ref_curv_table_p2a_.Evaluate(abs_ref_curv),
      curv_diff_upper_threshold_ref_curv_table_p2a_.Evaluate(abs_ref_curv));
  // curv diff condition is flipfloped, only need to be fullfilled once
  bool abs_curv_diff_p2a_cond =
      ref_curv_diff_dd_pair_p2a_.debouncer.RunDebouceMoreTure(std::abs(
          sm->input_info().ref_curv_cog - sm->input_info().ego_veh_curv));
  curv_diff_cond_flipflop_p2a_.Run(abs_curv_diff_p2a_cond, false);
  bool abs_curv_diff_p2a_cond_ff = curv_diff_cond_flipflop_p2a_.get_status();
  // min lat offset wrt left and right lane marker
  double min_abs_lat_offset =
             std::min(std::abs(sm->input_info().left_lat_offset_front_bumper),
                      std::abs(sm->input_info().right_lat_offset_front_bumper)),
         abs_ref_heading = std::abs(sm->input_info().ref_heading_front_bumper);
  bool min_abs_lat_offset_p2a_cond =
           min_lat_offset_dd_pair_p2a_.debouncer.RunDebouceLessTure(
               min_abs_lat_offset),
       min_abs_lat_offset_p2a_cond_delay =
           min_lat_offset_dd_pair_p2a_.delay.RunDelay(
               min_abs_lat_offset_p2a_cond);
  bool ref_heading_p2a_cond =
           ref_heading_dd_pair_p2a_.debouncer.RunDebouceMoreTure(
               abs_ref_heading),
       ref_heading_p2a_cond_delay =
           ref_curv_diff_dd_pair_p2a_.delay.RunDelay(ref_heading_p2a_cond);

  // passive to noaction condition
  ref_curv_diff_dd_pair_p2n_.debouncer.SetDebounce(
      curv_diff_lower_threshold_ref_curv_table_p2n_.Evaluate(abs_ref_curv),
      curv_diff_upper_threshold_ref_curv_table_p2n_.Evaluate(abs_ref_curv));
  bool ref_lat_offset_p2n_cond =
           ref_lat_offset_dd_pair_p2n_.debouncer.RunDebouceLessTure(
               std::abs(sm->input_info().ref_lat_offset_front_bumper)),
       ref_lat_offset_p2n_cond_delay =
           ref_heading_dd_pair_p2n_.delay.RunDelay(ref_lat_offset_p2n_cond);
  bool ref_curv_diff_p2n_cond =
           ref_curv_diff_dd_pair_p2n_.debouncer.RunDebouceLessTure(std::abs(
               sm->input_info().ref_curv_cog - sm->input_info().ego_veh_curv)),
       ref_curv_diff_p2n_cond_delay =
           ref_curv_diff_dd_pair_p2n_.delay.RunDelay(ref_curv_diff_p2n_cond);
  bool ref_heading_p2n_cond =
           ref_heading_dd_pair_p2n_.debouncer.RunDebouceLessTure(
               abs_ref_heading),
       ref_heading_p2n_cond_delay =
           ref_curv_diff_dd_pair_p2n_.delay.RunDelay(ref_heading_p2n_cond);

  // std::cout << 8 * abs_curv_diff_p2a_cond_ff +
  //                  4 * min_abs_lat_offset_p2a_cond_delay +
  //                  2 * ref_heading_p2a_cond_delay + object_heading_cond
  //           << std::endl;

  if (abs_curv_diff_p2a_cond_ff && min_abs_lat_offset_p2a_cond_delay &&
      ref_heading_p2a_cond_delay && object_heading_cond) {
    sm->ChangeState(new ATSSwitchActive());
  } else if (ref_lat_offset_p2n_cond_delay && ref_curv_diff_p2n_cond_delay &&
             ref_heading_p2n_cond_delay) {
    sm->ChangeState(new ATSSwitchNoAction());
  } else {
    // do nothing
  }
}

bool ATSSwitchPassive::CheckObjectHeadingCond(
    ATSSwitchDeciderStateMachine *sm) {
  rm::ObjectLocation front_closest_object_location =
      sm->input_info().ego_corridor_closest_object_location;
  if (front_closest_object_location.object_id == 0U) {
    return true;
  } else {
    rm::common::math::Vec2d left_intersection, right_intersection;
    double long_dis_obj_x = front_closest_object_location.relative_center_x;
    left_intersection.set_x(long_dis_obj_x);
    left_intersection.set_y(
        sm->input_info().left_lane_marker.Evaluate(0U, long_dis_obj_x));
    right_intersection.set_x(long_dis_obj_x);
    right_intersection.set_y(
        sm->input_info().right_lane_marker.Evaluate(0U, long_dis_obj_x));
    double left_angle = left_intersection.Angle(),
           right_angle = right_intersection.Angle();
    bool left_heading_cond_delay = left_intersection_delay_.RunDelay(
        left_angle < 0.0 && right_angle < 0.0);
    bool right_heading_cond_delay = right_intersection_delay_.RunDelay(
        left_angle > 0.0 && right_angle > 0.0);
    return left_heading_cond_delay || right_heading_cond_delay;
  }
}

void ATSSwitchActive::Handle(ATSSwitchDeciderStateMachine *sm) {
  // DLOG(INFO) << "------active------";
  // std::cout << "------active------" << std::endl;
  double abs_ref_curv = std::abs(sm->input_info().ref_curv_cog);
  ref_curv_diff_dd_pair_.debouncer.SetDebounce(
      curv_diff_lower_threshold_ref_curv_table_.Evaluate(abs_ref_curv),
      curv_diff_upper_threshold_ref_curv_table_.Evaluate(abs_ref_curv));
  bool ref_lat_offset_cond =
           ref_lat_offset_dd_pair_.debouncer.RunDebouceLessTure(
               std::abs(sm->input_info().ref_lat_offset_front_bumper)),
       ref_lat_offset_cond_delay =
           ref_heading_dd_pair_.delay.RunDelay(ref_lat_offset_cond);
  bool ref_curv_diff_cond =
           ref_curv_diff_dd_pair_.debouncer.RunDebouceLessTure(std::abs(
               sm->input_info().ref_curv_cog - sm->input_info().ego_veh_curv)),
       ref_curv_diff_cond_delay =
           ref_curv_diff_dd_pair_.delay.RunDelay(ref_curv_diff_cond);
  bool ref_heading_cond = ref_heading_dd_pair_.debouncer.RunDebouceLessTure(
           std::abs(sm->input_info().ref_heading_front_bumper)),
       ref_heading_cond_delay =
           ref_heading_dd_pair_.delay.RunDelay(ref_heading_cond);

  if (ref_lat_offset_cond_delay && ref_curv_diff_cond_delay &&
      ref_heading_cond_delay) {
    sm->ChangeState(new ATSSwitchNoAction());
  } else {
    // do nothing
  }
}

void ATSSwitchDeciderStateMachine::UpdateInputInfo(
    const LaneInfo *ego_lane_info_ptr, LaneLineCrossedType lane_crossed_type) {
  input_info_.left_lat_offset_front_bumper =
      ego_lane_info_ptr->boundary()
          .first.boundary_segment_list()
          .front()
          .geometry.curve()
          .Evaluate(0, rm::DISREARAXEL2FRONT);
  input_info_.right_lat_offset_front_bumper =
      ego_lane_info_ptr->boundary()
          .second.boundary_segment_list()
          .front()
          .geometry.curve()
          .Evaluate(0, rm::DISREARAXEL2FRONT);
  input_info_.left_heading_front_bumper =
      ego_lane_info_ptr->boundary()
          .first.boundary_segment_list()
          .front()
          .geometry.curve()
          .Evaluate(1, rm::DISREARAXEL2FRONT);
  input_info_.right_heading_front_bumper =
      ego_lane_info_ptr->boundary()
          .second.boundary_segment_list()
          .front()
          .geometry.curve()
          .Evaluate(1, rm::DISREARAXEL2FRONT);
  input_info_.lane_crossed_type = lane_crossed_type;
  input_info_.column_trq = VehicleStateProvider::Instance()->column_torque();
  input_info_.ego_velocity = VehicleStateProvider::Instance()->lon_velocity();
  input_info_.is_tja_ica_active =
      (VehicleStateProvider::Instance()->tja_ica_status() ==
       rm::SMTjaIcaStatus::ACTIVE)
          ? true
          : false;
  input_info_.ref_lat_offset_front_bumper =
      ego_lane_info_ptr->corridor().front().center_line().Evaluate(
          0, rm::DISREARAXEL2FRONT);
  input_info_.ref_heading_front_bumper =
      ego_lane_info_ptr->corridor().front().center_line().Evaluate(
          1, rm::DISREARAXEL2FRONT);
  input_info_.ref_curv_cog =
      2.0 * ego_lane_info_ptr->corridor().front().center_line().Evaluate(
                2, rm::DISREARAXEL2COG);
  input_info_.ego_veh_curv =
      VehicleStateProvider::Instance()->driving_curvature();
  std::vector<ObjectLocation> ego_lane_front_object_location;
  for (auto iter =
           ego_lane_info_ptr->corridor().begin()->object_location().begin();
       iter != ego_lane_info_ptr->corridor().begin()->object_location().end();
       ++iter) {
    // front object in ego corridor
    if (iter->in_corridor == true &&
        iter->reference_point2corridor_frenet.s() > 0.0) {
      ego_lane_front_object_location.emplace_back(*iter);
    }
  }
  if (ego_lane_front_object_location.empty()) {
    input_info_.ego_corridor_closest_object_location = ObjectLocation();
  } else {
    input_info_.ego_corridor_closest_object_location =
        *std::min_element(ego_lane_front_object_location.begin(),
                          ego_lane_front_object_location.end());
  }
  input_info_.left_lane_marker = ego_lane_info_ptr->boundary()
                                     .first.boundary_segment_list()
                                     .front()
                                     .geometry.curve();
  input_info_.right_lane_marker = ego_lane_info_ptr->boundary()
                                      .second.boundary_segment_list()
                                      .front()
                                      .geometry.curve();
}

bool ATSSwitchDecider::Run(Frame *frame) {
  //
  const rm::LaneInfo *ego_lane_info_ptr =
      frame->FindLaneInfoPtrFromLanePosition(LanePosition::EGO_LANE);

  // return false if either lane or corridor does not exist
  if (ego_lane_info_ptr == nullptr) {
    ats_switch_detector_state_machine_.Reset();
    return false;
  } else {
    if (ego_lane_info_ptr->corridor().empty()) {
      ats_switch_detector_state_machine_.Reset();
      return false;
    }
  }

  ats_switch_detector_state_machine_.UpdateInputInfo(
      ego_lane_info_ptr, frame->lane_line_crossed());

  ats_switch_detector_state_machine_.Request();

  return ats_switch_detector_state_machine_.get_ats_switch_flag();
}

} // namespace rm