/*
 * @Author: your name
 * @Date: 2021-11-29 14:42:57
 * @LastEditTime: 2022-01-20 09:58:08
 * @LastEditors: Please set LastEditors
 * @Description: 打开koroFileHeader查看配置 进行设置:
 * https://github.com/OBKoro1/koro1FileHeader/wiki/%E9%85%8D%E7%BD%AE
 * @FilePath: /EHR/home/igs/roadmodel/src/CtApRoadModel/source/road/boundary.cc
 */
#pragma once
#include "boundary.h"

namespace rm {
void Boundary::set_geometry(const LineGeometry& geometry) {
  geometry_ = geometry;
}

const LineGeometry& Boundary::geometry() const { return geometry_; }

void Boundary::set_line_type(const LineType& type) { line_type_ = type; }

const LineType& Boundary::line_type() const { return line_type_; }

void Boundary::set_line_color(const LineColor& color) { line_color_ = color; }

const LineColor& Boundary::line_color() const { return line_color_; }

bool Boundary::is_lane_marker() const { return is_lane_marker_; }

bool Boundary::is_front_marker() const { return is_front_marker_; }

PolynomLine Boundary::ComputeShiftLine(bool is_left) {
  // TODO(ningmeng) ：当前平移为临时固定值  后续需要根据线性 是否是roadedge
  // 颜色等更新偏移量
  double shift_distance = 0.1;
  return geometry_.mutable_curve()->MoveCurved(
      (is_left ? -1 : 1) * shift_distance, is_front_marker_);
}

void Boundary::ComputeBoundaryFromLaneMarkers(
    const std::vector<LaneMarker>& lane_markers) {
  if (lane_markers.empty() ||
      lane_markers[0].front_lane_marker_segment().empty()) {
    DLOG(ERROR) << "lane_marker_segments is empty";
    return;
  }
  for (const auto& lane_marker_seg :
       lane_markers[0].front_lane_marker_segment()) {
    // only set the valid_ once
    if (lane_marker_seg.geometry().valid()) {
      if (valid_ == false) valid_ = true;
    } else {
      break;
    }
    boundary_segment_list_.emplace_back(
        lane_marker_seg.geometry(), lane_marker_seg.type(),
        lane_marker_seg.color(), lane_marker_seg.sensor_source());
  }
  is_front_marker_ = true;
  is_lane_marker_ = true;
  sensor_source_ =
      lane_markers.front().front_lane_marker_segment().front().sensor_source();
}

void Boundary::ComputeBackBoundaryFromLaneMarkers(
    const std::vector<LaneMarker>& lane_markers) {
  if (lane_markers.empty() ||
      lane_markers[0].back_lane_marker_segment().empty()) {
    DLOG(ERROR) << "lane_marker_segments is empty";
    return;
  }
  for (const auto& lane_marker_seg :
       lane_markers[0].back_lane_marker_segment()) {
    if (lane_marker_seg.geometry().valid()) {
      if (valid_ == false) valid_ = true;
    } else {
      break;
    }
    boundary_segment_list_.emplace_back(
        lane_marker_seg.geometry(), lane_marker_seg.type(),
        lane_marker_seg.color(), lane_marker_seg.sensor_source());
  }
  is_front_marker_ = false;
  is_lane_marker_ = true;
}

std::ostream& operator<<(std::ostream& os, const Boundary& boundary) {
  os << "geometry_ = " << boundary.geometry() << "\n"
     << "line_type_ = " << (+uint8_t(boundary.line_type())) << "\n"
     << "line_color_ = " << (+uint8_t(boundary.line_color())) << "\n"
     << "is_lane_marker_ = " << boundary.is_lane_marker();
  return os;
}

uint16_t Boundary::sensor_source() const { return sensor_source_; }

void Boundary::set_valid(const bool valid) { valid_ = valid; }

bool Boundary::valid() const { return valid_; }

std::vector<BoundarySegement>* Boundary::mutable_boundary_segment_list() {
  return &boundary_segment_list_;
}

const std::vector<BoundarySegement>& Boundary::boundary_segment_list() const {
  return boundary_segment_list_;
}
}  // namespace rm