#include "control_way_points.h"

namespace rm {

void ControlWayPoints::add_wp_extend_lane_list(
    const WPExtendLane &extend_lane) {
  wp_extend_lane_list_.push_back(extend_lane);
}

void ControlWayPoints::set_reminder_way_point(
    const ReminderWayPoint &reminder_way_point) {
  reminder_way_point_ = reminder_way_point;
}

void ControlWayPoints::set_remain_distance_major_path(
    float remain_distance_major_path) {
  remain_distance_major_path_ = remain_distance_major_path;
}

void ControlWayPoints::set_control_way_point_target_roadtype(
    uint8_t control_way_point_target_roadtype) {
  control_way_point_target_roadtype_ =
      static_cast<rm::TargetRoadType>(control_way_point_target_roadtype);
}

void ControlWayPoints::set_control_way_point_offset(
    float control_way_point_offset) {
  control_way_point_offset_ = control_way_point_offset;
}

void ControlWayPoints::set_speed_limit(uint8_t limit_0, float dis_0,
                                       uint8_t limit_1, float dis_1) {
  if (location_quality_) {
    if (!is_on_ramp_) {
      speed_limit_.set_speed_limit(limit_0, dis_0, limit_1, dis_1);
    } else {
      speed_limit_.set_speed_limit(60U, dis_0, limit_1, dis_1);
    }
  } else {
    speed_limit_.set_default_speed_limit();
    speed_limit_.reset();
  }
}

void ControlWayPoints::set_remain_distance_major_path_type(uint8_t end_type) {
  remain_distance_major_path_type_ =
      static_cast<rm::RemainDistanceMajorPathType>(end_type);
}

void ControlWayPoints::set_location_status(uint8_t location_status) {
  location_quality_ = location_status;
}

void ControlWayPoints::set_navi_status(uint8_t navi_status) {
  navi_sts_ = navi_status;
}

const ReminderWayPoint &ControlWayPoints::reminder_way_point() const {
  return reminder_way_point_;
}

const std::vector<WPExtendLane> &ControlWayPoints::wp_extend_lane_list() const {
  return wp_extend_lane_list_;
}

const rm::TargetRoadType &ControlWayPoints::control_way_point_target_roadtype()
    const {
  return control_way_point_target_roadtype_;
}

float ControlWayPoints::control_way_point_offset() const {
  return control_way_point_offset_;
}

float ControlWayPoints::remain_distance_major_path() const {
  return remain_distance_major_path_;
}

std::vector<WPExtendLane> *ControlWayPoints::mutable_wp_extend_lane_list() {
  return &wp_extend_lane_list_;
}

const SpeedLimit &ControlWayPoints::speed_limit() const { return speed_limit_; }

uint8_t ControlWayPoints::location_quality() const { return location_quality_; }

uint8_t ControlWayPoints::camr_line_mapping() const {
  return camr_line_mapping_;
}

bool ControlWayPoints::is_out_odd_range() const { return is_out_odd_range_; }

bool ControlWayPoints::is_on_ramp() const { return is_on_ramp_; }

uint8_t ControlWayPoints::navi_sts() const { return navi_sts_; }

const rm::RemainDistanceMajorPathType &
ControlWayPoints::remain_distance_major_path_type() const {
  return remain_distance_major_path_type_;
}

void ControlWayPoints::set_control_way_info_bit(uint8_t input) {
  control_way_info_bit_ = std::bitset<8>(input);
  is_out_odd_range_ = control_way_info_bit_[0];
  camr_line_mapping_ = static_cast<uint8_t>(control_way_info_bit_[1]);
  is_on_ramp_ = control_way_info_bit_[2];
}

WPExtendLane *ControlWayPoints::FindWPExtendLaneByLaneId(uint8_t id) {
  auto it =
      std::find(wp_extend_lane_list_.begin(), wp_extend_lane_list_.end(), id);
  if (it == wp_extend_lane_list_.end()) {
    return nullptr;
  }
  return &(*it);
}

}  // namespace rm
