#pragma once
#include <stdint.h>

#include <algorithm>
#include <bitset>
#include <vector>

#include "enum_list.h"
#include "reminder_way_point.h"
#include "speed_limit.h"
#include "wpextend_lane.h"

namespace rm {

class ControlWayPoints {
 public:
  void add_wp_extend_lane_list(const WPExtendLane &extend_lane);

  void set_reminder_way_point(const ReminderWayPoint &reminder_way_point);

  ReminderWayPoint *mutable_reminder_way_point() {
    return &reminder_way_point_;
  }

  void set_remain_distance_major_path(float remain_distance_major_path);

  void set_control_way_point_target_roadtype(
      uint8_t control_way_point_target_roadtype);

  void set_control_way_point_offset(float control_way_point_offset);

  void set_speed_limit(uint8_t limit_0, float dis_0, uint8_t limit_1,
                       float dis_1);

  void set_control_way_info_bit(uint8_t input);

  void set_remain_distance_major_path_type(uint8_t end_type);
  void set_location_status(uint8_t location_status);
  void set_navi_status(uint8_t navi_status);

  const ReminderWayPoint &reminder_way_point() const;

  const std::vector<WPExtendLane> &wp_extend_lane_list() const;

  std::vector<WPExtendLane> *mutable_wp_extend_lane_list();

  const rm::TargetRoadType &control_way_point_target_roadtype() const;

  float control_way_point_offset() const;

  float remain_distance_major_path() const;

  const SpeedLimit &speed_limit() const;

  uint8_t location_quality() const;

  uint8_t camr_line_mapping() const;

  bool is_out_odd_range() const;

  bool is_on_ramp() const;

  uint8_t navi_sts() const;

  const rm::RemainDistanceMajorPathType &remain_distance_major_path_type()
      const;

  WPExtendLane *FindWPExtendLaneByLaneId(uint8_t id);

 private:
  std::vector<WPExtendLane>
      wp_extend_lane_list_;  // 1st: neighbour lane info, 2nd:
  rm::TargetRoadType control_way_point_target_roadtype_;  // target road type
  float control_way_point_offset_;       // distance to target road type
  ReminderWayPoint reminder_way_point_;  // tunnel in front
  float remain_distance_major_path_;     // distance to tunnel
  rm::RemainDistanceMajorPathType remain_distance_major_path_type_;
  SpeedLimit speed_limit_;
  uint8_t location_quality_;
  uint8_t camr_line_mapping_;
  bool is_on_ramp_;
  bool is_out_odd_range_;
  std::bitset<8> control_way_info_bit_;
  uint8_t navi_sts_;
};

}  // namespace rm
