/*
 * @Author: your name
 * @Date: 2021-11-29 15:26:00
 * @LastEditTime: 2022-03-21 19:18:17
 * @LastEditors: Please set LastEditors
 * @Description: 打开koroFileHeader查看配置 进行设置:
 * https://github.com/OBKoro1/koro1FileHeader/wiki/%E9%85%8D%E7%BD%AE
 * @FilePath: /EHR/home/igs/roadmodel/src/CtApRoadModel/source/road/corridor.cc
 */
#include "corridor.h"

// #include "glog/logging.h"

namespace rm {
// void Corridor::ComputeCenterLine(const std::vector<std::pair<Vec2d, double>>
// &vec2d_points)
// {
//     std::vector<std::pair<ReferencePoint, double>> reference_point;
//     for_each(vec2d_points.begin(), vec2d_points.end(), [&](const
//     std::pair<Vec2d, double> &point){
//     reference_point.emplace_back(std::make_pair(ReferencePoint(point.first),
//     point.second)); }); ComputeCenterLine(reference_point);
// }
void Corridor::set_corridor_id(uint32_t id) { corridor_id_ = id; }

void Corridor::set_corridor_id(uint32_t id, bool lane_makers_id_changed,
                               std::shared_ptr<DependencyInjector> injector) {
  auto it = std::find_if(
      injector->mutable_line_follow_corridor_id_info_last_cycle()->begin(),
      injector->mutable_line_follow_corridor_id_info_last_cycle()->end(),
      [id](std::pair<uint16_t, bool> cor_id_info) {
        return (cor_id_info.first - cor_id_info.first % 10) / 10 == id;
      });
  if (it ==
      injector->mutable_line_follow_corridor_id_info_last_cycle()->end()) {
    corridor_id_ = id * 10;
    injector->mutable_line_follow_corridor_id_info_last_cycle()->push_back(
        std::make_pair(corridor_id_, true));
  } else {
    if (it->second == false) {
      // first corridor appears
      if (lane_makers_id_changed) {
        corridor_id_ = id * 10 + ((it->first - id * 10) + 1) % 10;
        it->first = corridor_id_;
        it->second = true;
      } else {
        corridor_id_ = it->first;  // previous value
        it->second = true;
      }
    } else {
      corridor_id_ = it->first;  // previous value
    }
  }
}

void Corridor::set_feature_tag(CorridorFeatureTag tag) { feature_tag_ = tag; }

void Corridor::set_virtualdrivablewidth(float width) {
  drivable_width_ = width;
}

void Corridor::set_ilc_target_id(uint32_t target_id) {
  ilc_target_id_ = target_id;
}

bool Corridor::ComputeCenterLine(
    const std::array<std::vector<std::pair<ReferencePoint, double>>, 2>
        &reference_points) {
  // TODO(all) 调用二乘法计算center_line_
  // arg min_x∥Ax−b∥2
  // there are at least 2 reference points
  if (reference_points.at(0).size() < 2U || reference_points.at(1).size() < 2U)
    return false;
  std::vector<std::pair<ReferencePoint, double>> stacked_references_points;
  uint16_t start_limit = std::max(reference_points.at(0).front().first.x(),
                                  reference_points.at(1).front().first.x());
  uint16_t end_limit = std::min(reference_points.at(0).back().first.x(),
                                reference_points.at(1).back().first.x());
  auto left_start_iter = std::find_if(
      reference_points.at(0).begin(), reference_points.at(0).end(),
      [&](const std::pair<ReferencePoint, double> &ref_pair) {
        return static_cast<uint16_t>(ref_pair.first.x()) == start_limit;
      });
  auto left_end_iter = std::find_if(
      reference_points.at(0).begin(), reference_points.at(0).end(),
      [&](const std::pair<ReferencePoint, double> &ref_pair) {
        return static_cast<uint16_t>(ref_pair.first.x()) == end_limit;
      });
  auto right_start_iter = std::find_if(
      reference_points.at(1).begin(), reference_points.at(1).end(),
      [&](const std::pair<ReferencePoint, double> &ref_pair) {
        return static_cast<uint16_t>(ref_pair.first.x()) == start_limit;
      });
  auto right_end_iter = std::find_if(
      reference_points.at(1).begin(), reference_points.at(1).end(),
      [&](const std::pair<ReferencePoint, double> &ref_pair) {
        return static_cast<uint16_t>(ref_pair.first.x()) == end_limit;
      });
  if (left_start_iter == reference_points.at(0).end() ||
      left_end_iter == reference_points.at(0).end() ||
      right_start_iter == reference_points.at(1).end() ||
      right_end_iter == reference_points.at(1).end())
    return false;
  stacked_references_points.insert(stacked_references_points.end(),
                                   left_start_iter, left_end_iter + 1);
  stacked_references_points.insert(stacked_references_points.end(),
                                   right_start_iter, right_end_iter + 1);

  // if the num of reference points is too little, or the center corridor length
  // is less than threshold, return false status
  if (stacked_references_points.size() > rm::CORRIDORSTACKEDPOINTMAXSIZE ||
      stacked_references_points.size() < rm::CORRIDORSTACKEDPOINTMINSIZE)
    return false;

  // auto start_range_iter = std::min_element(
  //     stacked_references_points.begin(), stacked_references_points.end(),
  //     [](std::pair<ReferencePoint, double> &temp1,
  //        std::pair<ReferencePoint, double> &temp2) {
  //       return temp1.first.x() < temp2.first.x();
  //     });
  // auto end_range_iter = std::max_element(
  //     stacked_references_points.begin(), stacked_references_points.end(),
  //     [](std::pair<ReferencePoint, double> &temp1,
  //        std::pair<ReferencePoint, double> &temp2) {
  //       return temp1.first.x() < temp2.first.x();
  //     });
  double start_range = std::min(reference_points.at(0).front().first.x(),
                                reference_points.at(1).front().first.x());
  double end_range = std::max(reference_points.at(0).back().first.x(),
                              reference_points.at(1).back().first.x());
  // double range = end_range_iter->first.x() - start_range_iter->first.x();
  double range = end_range - start_range;

  if (range < rm::MINCORRIDORRANGE) return false;

  // 1. Set order of polynomial
  int order = 3;
  // 2. Create A matrix with unweighted data
  Eigen::VectorXd point_position_x{stacked_references_points.size()};
  Eigen::VectorXd point_position_y{stacked_references_points.size()};
  for (size_t i = 0; i < stacked_references_points.size(); i++) {
    point_position_x[i] = stacked_references_points.at(i).first.x();
    point_position_y[i] = stacked_references_points.at(i).first.y();
  }

  // debug
  // assert(point_position_x.size() == point_position_y.size());
  // assert(order >= 1 && order <= point_position_x.size() - 1);

  Eigen::MatrixXd A(point_position_x.size(), order + 1);

  // set 1st column of A
  Eigen::VectorXd ones_vec{point_position_x.size()};
  ones_vec.setOnes();
  A.col(0) = ones_vec;
  // set other columns of A
  for (int j = 0; j < point_position_x.size(); j++) {
    for (int i = 0; i < order; i++) {
      A(j, i + 1) = A(j, i) * point_position_x(j);
    }
  }

  // 3. Add weights to points data
  //  Collect weights for all points
  Eigen::VectorXd vec_weights{stacked_references_points.size()};
  for (size_t i = 0; i < stacked_references_points.size(); i++) {
    vec_weights[i] = stacked_references_points.at(i).second;
  }
  //  Create weights matrix
  Eigen::MatrixXd weights_diagonal(vec_weights.size(), vec_weights.size());
  weights_diagonal = vec_weights.asDiagonal();
  //    Assign weights to matrix A:
  Eigen::MatrixXd weighted_A(point_position_x.size(), order + 1);
  weighted_A = weights_diagonal * A;
  //    Assign weights to vec Y:
  Eigen::VectorXd weighted_point_position_y{stacked_references_points.size()};
  weighted_point_position_y = weights_diagonal * point_position_y;

  // 4. Solve:
  Eigen::VectorXd polynomial_coefficient;
  auto Q = weighted_A.householderQr();
  polynomial_coefficient = Q.solve(weighted_point_position_y);

  center_line_.set_start_range(start_range);
  center_line_.set_end_range(end_range);
  center_line_.set_coef(polynomial_coefficient[0], polynomial_coefficient[1],
                        polynomial_coefficient[2], polynomial_coefficient[3]);

  // double c_0 = polynomial_coefficient[0];
  // double c_1 = polynomial_coefficient[1];
  // double c_2 = polynomial_coefficient[2];
  // double c_3 = polynomial_coefficient[3];
  return true;
}

bool Corridor::ComputeCenterLine(
    const std::vector<std::pair<ReferencePoint, double>> &reference_points) {
  // TODO(all) 调用二乘法计算center_line_
  // arg min_x∥Ax−b∥2
  // there are at least 2 reference points
  if (reference_points.size() < 2U) return false;
  // 1. Set order of polynomial
  int order = 3;

  // 2. Create A matrix with unweighted data
  Eigen::VectorXd point_position_x{reference_points.size()};
  Eigen::VectorXd point_position_y{reference_points.size()};
  for (size_t i = 0; i < reference_points.size(); i++) {
    point_position_x[i] = reference_points.at(i).first.x();
    point_position_y[i] = reference_points.at(i).first.y();
  }

  // debug
  // assert(point_position_x.size() == point_position_y.size());
  // assert(order >= 1 && order <= point_position_x.size() - 1);

  Eigen::MatrixXd A(point_position_x.size(), order + 1);

  // set 1st column of A
  Eigen::VectorXd ones_vec{point_position_x.size()};
  ones_vec.setOnes();
  A.col(0) = ones_vec;
  // set other columns of A
  for (int j = 0; j < point_position_x.size(); j++) {
    for (int i = 0; i < order; i++) {
      A(j, i + 1) = A(j, i) * point_position_x(j);
    }
  }

  // 3. Add weights to points data
  //  Collect weights for all points
  Eigen::VectorXd vec_weights{reference_points.size()};
  for (size_t i = 0; i < reference_points.size(); i++) {
    vec_weights[i] = reference_points.at(i).second;
  }
  //  Create weights matrix
  Eigen::MatrixXd weights_diagonal(vec_weights.size(), vec_weights.size());
  weights_diagonal = vec_weights.asDiagonal();
  //    Assign weights to matrix A:
  Eigen::MatrixXd weighted_A(point_position_x.size(), order + 1);
  weighted_A = weights_diagonal * A;
  //    Assign weights to vec Y:
  Eigen::VectorXd weighted_point_position_y{reference_points.size()};
  weighted_point_position_y = weights_diagonal * point_position_y;

  // 4. Solve:
  Eigen::VectorXd polynomial_coefficient;
  auto Q = weighted_A.householderQr();
  polynomial_coefficient = Q.solve(weighted_point_position_y);

  center_line_.set_start_range(reference_points.front().first.x());
  center_line_.set_end_range(reference_points.back().first.x());
  center_line_.set_coef(polynomial_coefficient[0], polynomial_coefficient[1],
                        polynomial_coefficient[2], polynomial_coefficient[3]);

  // double c_0 = polynomial_coefficient[0];
  // double c_1 = polynomial_coefficient[1];
  // double c_2 = polynomial_coefficient[2];
  // double c_3 = polynomial_coefficient[3];
  return true;
}

void Corridor::ComputeDrivableWidth(const LineGeometry &left_line_geometry,
                                    const LineGeometry &right_line_geometry) {
  if (left_line_geometry.valid() && right_line_geometry.valid()) {
    ComputeDrivableWidthFromCurve(left_line_geometry.curve(),
                                  right_line_geometry.curve());
  } else {
    drivable_width_ = 3.5F;
  }
}

void Corridor::ComputeDrivableWidthFromCurve(const PolynomLine &left_curve,
                                             const PolynomLine &right_curve) {
  double x_r = 0.0;
  double x_inter_left = x_r;  // default intersection point
  double x_inter_right = x_r;
  ComputeTangIntersection(left_curve, x_r,
                          x_inter_left);  // evaluate the intersection at x_r
  ComputeTangIntersection(right_curve, x_r,
                          x_inter_right);  // evaluate the intersection at x_r

  drivable_width_ = hypot(x_inter_left - x_inter_right,
                          left_curve.Evaluate(0, x_inter_left) -
                              right_curve.Evaluate(0, x_inter_right));
}

Status Corridor::ComputeTangIntersection(const PolynomLine &curve,
                                         const double x_r, double &x_inter) {
  // x_sol only contains the real solution
  std::vector<double> x_sol;
  double h_r = curve.Evaluate(1, x_r);  // heading of center line at x_r
  double sin_hr = sin(h_r);
  double cos_hr = cos(h_r);
  // a*x^3+b*x^2+c*x + d = 0 --> t^3 + q*t + p = 0
  // x = t - b/(3*a)
  double a = curve.Coef(3) * sin_hr;
  double b = curve.Coef(2) * sin_hr;
  double c = cos_hr + curve.Coef(1) * sin_hr;
  double d =
      (curve.Coef(0) - center_line_.Evaluate(0, x_r)) * sin_hr - x_r * cos_hr;
  double sol_offset = b / (3.0 * a);
  double double_epsilon = std::numeric_limits<double>::epsilon();

  // compute solution
  if (-double_epsilon > a && a > double_epsilon)  // cubic case // https://zhuanlan.zhihu.com/p/137077558
  {
    double p = (3 * a * c - b * b) / (3 * a * a);
    double q =
        (2 * b * b * b - 9 * a * b * c + 27 * a * a * d) / (27 * a * a * a);
    double delta = -(4 * p * p * p + 27 * q * q);
    std::vector<std::pair<double, double>> u;

    if (-double_epsilon <= delta && delta <= double_epsilon)  // 3 real solutions, 2 identical
    {
      std::pair<double, double> temp(-q * 0.5, 0);
      double v_real, threesquare;
      CubicRootCompute(temp, u);  // compute all the root a+bi
      for (int i = 0; i < 3; i++) {
        // v = -p/(3*u);
        threesquare = 3.0 * (u[i].first * u[i].first +
                             u[i].second * u[i].second);  // 3*(a^2 + b^2)
        v_real = -p * u[i].first / threesquare;
        // v.second = p*b / threesquare; // Ignore the imag, since in this case
        // they are cancelled out
        x_sol.push_back(u[i].first + v_real - sol_offset);  // x = t - b/(3*a)
      }
      x_inter =
          (abs(x_sol[0] - x_r) < abs(x_sol[1] - x_r)) ? x_sol[0] : x_sol[1];
      x_inter = (abs(x_inter - x_r) < abs(x_sol[2] - x_r)) ? x_inter : x_sol[2];
      return Status::OK();
    } else if (delta < 0)  // 1 real solution, 2 complex solution
    {
      std::pair<double, double> temp(
          -q * 0.5 + pow(-(delta / 108), (double)0.5), 0.0);
      double v_real;
      if (-double_epsilon <= temp.first && temp.first <= double_epsilon) {
        temp.first = -q * 0.5 - pow(-(delta / 108), (double)0.5);
      }
      // to avoid the case when the temp term is 0
      CubicRootCompute(temp, u);  // TODO this step is not necessary, since we
                                  // only need real solution
      int k = (temp.first < 0) ? 1 : 0;
      v_real = -p / (3.0 * u[k].first);  // only consider the real solution
      x_inter = v_real + u[k].first - sol_offset;
      return Status::OK();
    } else  // 3 real distinct solutions
    {
      std::pair<double, double> temp(-q * 0.5,
                                     pow((double)(delta / 108.0), (double)0.5));
      double v_real, threesquare;
      CubicRootCompute(temp, u);  // compute all the root a+bi
      for (int i = 0; i < 3; i++) {
        // v = -p/(3*u);
        threesquare = 3.0 * (u[i].first * u[i].first +
                             u[i].second * u[i].second);  // 3*(a^2 + b^2)
        v_real = -p * u[i].first / threesquare;
        // v.second = p*b / threesquare; // Ignore the imag, since in this case
        // they are cancelled out
        x_sol.push_back(u[i].first + v_real - sol_offset);
      }
      x_inter =
          (abs(x_sol[0] - x_r) < abs(x_sol[1] - x_r)) ? x_sol[0] : x_sol[1];
      x_inter = (abs(x_inter - x_r) < abs(x_sol[2] - x_r)) ? x_inter : x_sol[2];
      return Status::OK();
    }
  } else if (-double_epsilon > b && b > double_epsilon)  // degenerate to quadratic case
  {
    double delta;
    delta = c * c - 4 * b * d;
    if (0 <= delta) {
      x_sol.push_back(0.5 * (-c + sqrt(delta)) / b - sol_offset);
      x_sol.push_back(0.5 * (-c - sqrt(delta)) / b - sol_offset);
      x_inter =
          (abs(x_sol[0] - x_r) < abs(x_sol[1] - x_r)) ? x_sol[0] : x_sol[1];
      return Status::OK();
    } else {
      // complex solution
      return Status(ErrorCode::RM_COMPUTE_ERROR, "complex solution");
    }
  } else if (-double_epsilon > c && c > double_epsilon)  // degenerate to linear case
  {
    x_inter = -d / c - sol_offset;
    return Status::OK();
  } else {
    return Status(ErrorCode::RM_COMPUTE_ERROR, "no solution");
    ;
    // no solution
  }
}

void Corridor::CubicRootCompute(
    const std::pair<double, double> &value,
    std::vector<std::pair<double, double>> &result) {
  std::pair<double, double> root;
  double s = hypot(value.first, value.second);
  // DLOG(INFO) << PRINT(s);
  double s_root = pow(s, 1.0 / 3.0);
  // DLOG(INFO) << PRINT(s_root);
  for (int k = 0; k < 3; k++) {
    double theta = atan2(value.second, value.first) + 2.0 * k * rm::PI;
    // DLOG(INFO) << PRINT(theta);
    root.first = s_root * cos(theta / 3.0);   // real part
    root.second = s_root * sin(theta / 3.0);  // imaginary part
    result.push_back(root);
  }
}

uint32_t Corridor::corridor_id() const { return corridor_id_; }

CorridorFeatureTag Corridor::feature_tag() const { return feature_tag_; }

float Corridor::drivable_width() const { return drivable_width_; }

const PolynomLine &Corridor::center_line() const { return center_line_; }

std::vector<ObjectLocation> *Corridor::mutable_object_location() {
  return &object_location_;
}

const std::vector<ObjectLocation> &Corridor::object_location() const {
  return object_location_;
}

uint32_t Corridor::ilc_target_id() const { return ilc_target_id_; }

void Corridor::AddObjectLocation(ObjectLocation object_location) {
  object_location_.push_back(object_location);
}

void Corridor::DeleteObjectLocation() { object_location_.pop_back(); }

std::ostream &operator<<(std::ostream &os, const Corridor &corridor) {
  os << "corridor_id_ = " << corridor.corridor_id_ << "\n"
     << "center_line_ = " << corridor.center_line_ << "\n"
     << "drivable_width_ = " << corridor.drivable_width_ << "\n"
     << "start_offset_ = " << corridor.start_offset_;
  return os;
}

void Corridor::set_center_line(std::array<double, 4> coeff, double start_range,
                               double end_range) {
  center_line_.set_start_range(start_range);
  center_line_.set_end_range(end_range);
  center_line_.set_coef(coeff[0], coeff[1], coeff[2], coeff[3]);
}

void Corridor::set_center_line(const PolynomLine &center_line) {
  center_line_ = center_line;
}

void Corridor::ComputeCenterLineArcLength() {
  arc_length_ = center_line_.ComputeArcLengthbySimpson();
}

void Corridor::ComputeCenterLineArcLength(bool is_front_boundary) {
  float sign = (is_front_boundary) ? 1.0 : -1.0;
  arc_length_ = sign * center_line_.ComputeArcLengthbySimpson();
  // DLOG(INFO) << PRINT(arc_length_);
}

float Corridor::arc_length() const { return arc_length_; }

void Corridor::set_start_offset(const float start_offset) {
  start_offset_ = start_offset;
}

float Corridor::start_offset() const { return start_offset_; }

void Corridor::set_is_front_corridor(bool is_front_corridor) {
  is_front_corridor_ = is_front_corridor;
}

void Corridor::set_is_first_seg() { is_first_seg_ = true; }

bool Corridor::is_front_corridor() const { return is_front_corridor_; }

bool Corridor::is_first_seg() const { return is_first_seg_; }

void Corridor::MoveCenterLineLongitudinal(double m) {
  std::array<double, 4> original_coeff = {
      center_line_.Coef(0), center_line_.Coef(1), center_line_.Coef(2),
      center_line_.Coef(3)};
  center_line_.set_coef(
      original_coeff[0] + original_coeff[1] * m + original_coeff[2] * m * m +
          original_coeff[3] * m * m * m,
      original_coeff[1] + 2.0 * original_coeff[2] * m +
          3.0 * original_coeff[3] * m * m,
      original_coeff[2] + 3.0 * m * original_coeff[3], original_coeff[3]);
  center_line_.set_start_range(-m);
  center_line_.set_end_range(center_line_.end_range() - m);
}

void Corridor::MoveCenterLineLateral(double m) {
  center_line_ = center_line_.MoveCurved(m);
}

PolynomLine *Corridor::mutable_center_line() { return &center_line_; }

}  // namespace rm