/*
 * @Author: your name
 * @Date: 2021-09-16 18:23:20
 * @LastEditTime: 2022-03-21 19:17:54
 * @LastEditors: Please set LastEditors
 * @Description: In User Settings Edit
 * @FilePath:
 * /NOP_EHR/media/igs/xnm/ros/RM/roadmodel/modules/common/road/lane_corridor.h
 */
#pragma once

#include <stdint.h>

#include <algorithm>
#include <list>

#include "Eigen/Eigen"  //
#include "dependency_injector.h"
#include "enum_list.h"
#include "global_parameters.h"
#include "glog/logging.h"
#include "line_geometry.h"
#include "macros.h"
#include "object_location.h"
#include "polynom_line.h"
#include "reference_point.h"
#include "status.h"
namespace rm {
using common::math::ReferencePoint;
using common::math::Vec2d;
using rm::common::ErrorCode;
using rm::common::Status;

class Corridor {
 public:
  Corridor() = default;
  /**
   * @brief: 根据离散点及权重拟合三次多项式
   * @param {vector<std::pair<ReferencePoint, double>>} &reference_points
   * first:离散点 second：离散点对应的权重
   * @return {*}
   */
  void set_corridor_id(uint32_t id);
  void set_corridor_id(uint32_t id, bool lane_makers_id_changed,
                       std::shared_ptr<DependencyInjector> injector);
  void set_feature_tag(CorridorFeatureTag tag);
  void set_is_front_corridor(bool is_front_corridor);
  void set_ilc_target_id(uint32_t target_id);

  void set_virtualdrivablewidth(float width);  // set drivable width explictly
  // void ComputeCenterLine(const std::vector<std::pair<Vec2d, double>>
  // &reference_points);
  bool ComputeCenterLine(
      const std::array<std::vector<std::pair<ReferencePoint, double>>, 2>
          &reference_points);
  bool ComputeCenterLine(
      const std::vector<std::pair<ReferencePoint, double>> &reference_points);
  void ComputeDrivableWidth(const LineGeometry &left_line_geometry,
                            const LineGeometry &right_line_geometry);
  /**
   * @brief: 根据中心线及左右边界线 计算corridor的可行驶宽度
   * @param {PolynomLine} &left_curve corridor的左侧宽度边界
   * @param {PolynomLine} &right_curve corridor的右侧宽度边界
   * @return {*}
   */
  void ComputeDrivableWidthFromCurve(const PolynomLine &left_curve,
                                     const PolynomLine &right_curve);
  /**/
  Status ComputeTangIntersection(const PolynomLine &curve, const double x_r,
                                 double &x_inter);
  /**
   * @brief: 存取函数
   * @param {*}
   * @return {*}
   */
  uint32_t corridor_id() const;

  CorridorFeatureTag feature_tag() const;

  float drivable_width() const;

  const PolynomLine &center_line() const;

  PolynomLine *mutable_center_line();

  std::vector<ObjectLocation> *mutable_object_location();

  const std::vector<ObjectLocation> &object_location() const;

  uint32_t ilc_target_id() const;

  /**
   * @brief: 增加映射的障碍物信息
   * @param {ObjectLocation} object_location
   * @return {*}
   */
  void AddObjectLocation(ObjectLocation object_location);

  void DeleteObjectLocation();
  /**
   * @brief: 打印该类成员
   * @param {*}
   * @return {*}
   */
  friend std::ostream &operator<<(std::ostream &os, const Corridor &corridor);

  void set_center_line(std::array<double, 4> coeff, double start_range,
                       double end_range);

  void set_center_line(const PolynomLine &center_line);

  void set_is_first_seg();

  void ComputeCenterLineArcLength();

  void ComputeCenterLineArcLength(bool is_front_boundary);

  float arc_length() const;

  void set_start_offset(const float start_offset);

  float start_offset() const;

  bool is_front_corridor() const;

  bool is_first_seg() const;

  void MoveCenterLineLongitudinal(double m);

  void MoveCenterLineLateral(double m);

 private:
  void CubicRootCompute(const std::pair<double, double> &value,
                        std::vector<std::pair<double, double>> &result);
  uint32_t corridor_id_;                         // corridor的id
  PolynomLine center_line_;                      // corridor的中心线
  float drivable_width_;                         // corridor的宽度
  std::vector<ObjectLocation> object_location_;  // object在该corridor的投影
  float arc_length_ = 0.0F;    // the arc length of the current corridor
  float start_offset_ = 0.0F;  // the accumulated arc length of front corridors
  bool is_front_corridor_;
  CorridorFeatureTag feature_tag_ = CorridorFeatureTag::NOTAVLBL;
  uint32_t ilc_target_id_ = 0U;
  bool is_first_seg_ = false;
};

}  // namespace rm