#include "intersection_virtual_lane_generator.h"

namespace rm {
SINGLETON_STATIC(IntersectionVirtualLaneGenerator);

void IntersectionVirtualLaneGenerator::Terminate() {
  // kalman_gain_ = Eigen::Matrix<double, 6, 6>::Zero();
  // state_cov_ << 0.25, 0.0, 0.0, 0.0, 0.0, 6.0e-4, 0.0, 0.0, 0.0, 0.0, 1.0e-8,
  // 0.0, 0.0, 0.0, 0.0, 0.0; state_vector_ << 0.0, 0.0, 0.0, 0.0;
  // extend_distance_ = 0.0F;
  // extend_duration_ = 0.0F;
  // end_range_ = extend_distance_threshold_;
  corridor_last_time_before_invalid_ = 0.0;
  freespace_collision_delay_.ResetDelay();
  obstacles_distance_update_list_.clear();
  first_time_selection_obstacles_ = true;
  initilized_ = false;
  filter_status_ = FilterStatus::INVALID;
}

float IntersectionVirtualLaneGenerator::extend_distance() const {
  return extend_distance_;
}

float IntersectionVirtualLaneGenerator::extend_duration() const {
  return extend_duration_;
}

bool IntersectionVirtualLaneGenerator::initilized() const {
  return initilized_;
}

void IntersectionVirtualLaneGenerator::SelectObstacles(
    const rm::RoadInfoProvider *road_info_provider,
    const dpc::TargetsInfo &ats_targetsinfo,
    std::vector<std::pair<double, double>> &measure_vector) {
  if (road_info_provider->obstacles().empty())
    return;  // return if there is no object
  // PolynomLine intersec_line({state_vector_(0), state_vector_(1),
  // state_vector_(2), state_vector_(3),}, 0.0, 0.0); frenet_converter_ =
  // FrenetConverter(intersec_line);
  double intersec_line_heading = 0;
  double var_intersec_line_heading = 0;
  double debug_num_of_correct_objs = 0;
  bool is_ats_target = false;
  // DLOG(INFO) << PRINT(state_cov_(0, 0));
  // DLOG(INFO) << PRINT(state_cov_(1, 2));
  // DLOG(INFO) << PRINT(state_cov_(1, 3));
  // DLOG(INFO) << PRINT(state_cov_(2, 2));
  // DLOG(INFO) << PRINT(state_cov_(2, 3));
  // DLOG(INFO) << PRINT(state_cov_(3, 3));
  std::vector<std::pair<uint32_t, IntersectionDisInfo>>
      temp_obstacles_new_disinfo_list_;
  for (const auto &object : road_info_provider->obstacles()) {
    if (object.second.classification() != ObstacleType::BUS &&
        object.second.classification() != ObstacleType::TRUCK &&
        object.second.classification() != ObstacleType::CAR)
      continue;
    if (object.second.existence_duration() < 0.8F) continue;
    if (object.second.id() == ats_targetsinfo.iv.indx_tgtid) {
      is_ats_target = true;
    } else if (object.second.id() == ats_targetsinfo.flniv.indx_tgtid) {
      is_ats_target = true;
    } else if (object.second.id() == ats_targetsinfo.slniv.indx_tgtid) {
      is_ats_target = true;
    } else {
      is_ats_target = false;
    }
    // DLOG(INFO) << PRINT(object.second.id());
    // DLOG(INFO) << PRINT(+static_cast<uint8_t>(object.second.motion_property()));
    // DLOG(INFO) << PRINT(object.second.speed().Length());
    if (object.second.motion_property() != ObstacleMotionProperty::MOVING &&
        object.second.motion_property() !=
            ObstacleMotionProperty::MOVING_SLOWLY &&
        object.second.speed().Length() < 0.00001)
      continue;  // continue if the obstacle is not moving
    // DLOG(INFO) << PRINT(
    // +static_cast<uint8_t>(object.second.moving_direction()));
    if (object.second.moving_direction() == ObstacleMovingDirection::ONCOMING ||
        (object.second.moving_direction() == ObstacleMovingDirection::UNKOWN &&
         std::abs(object.second.heading()) > 1.0))
      continue;
    common::math::ReferencePoint projection_point =
        frenet_converter_.FindRefPointS(object.second.center().x(),
                                        object.second.center().y());
    // DLOG(INFO) << PRINT(projection_point.s());
    // DLOG(INFO) << PRINT(extend_distance_threshold_ - extend_distance_ +
    //                     (0.35 * extend_distance_ - 20.0));
    if (projection_point.s() <= 2.0 || projection_point.s() >= 80.0)
      continue;  // only consider front objects
    if (std::abs(object.second.center().y()) > 8) continue;
    // DLOG(INFO) << PRINT(object.second.id());
    double obj_x = projection_point.x();
    double obj_x_pow2 = obj_x * obj_x;
    double obj_x_pow3 = obj_x_pow2 * obj_x;
    double obj_x_pow4 = obj_x_pow3 * obj_x;
    intersec_line_heading =
        projection_point.heading();  // evalute the heading of polynom line at
                                     // postion of object x
    // compute the variance of heading angle at x
    // h = h0 + c0*x + 0.5*x^2*c1
    // h_bar = h0_bar + c0_bar*x + 0.5*x^2*c1_bar
    // E{(h-h_bar)^2} = ...
    var_intersec_line_heading =
        state_cov_(0, 0) + 2 * obj_x * state_cov_(1, 2) +
        obj_x_pow2 * (state_cov_(1, 3) + state_cov_(2, 2)) +
        obj_x_pow3 * state_cov_(2, 3) + 0.25 * obj_x_pow4 * state_cov_(3, 3);
    // DLOG(INFO) << PRINT(var_intersec_line_heading);
    double heading =
        object.second.heading();  // the heading that has been aligned
                                  // to the same direction of lane
    if (std::abs(heading) > 120 * rm::DEG2RAD) continue;
    // double opposite_heading =
    //     (heading < 0) ? heading + rm::PI : heading - rm::PI;
    double diff_heading = std::abs(heading - intersec_line_heading);
    // double diff_heading_opposite =
    //     std::abs(opposite_heading - intersec_line_heading);
    // if (diff_heading > diff_heading_opposite) {
    //   diff_heading = diff_heading_opposite;
    //   heading = opposite_heading;
    // }
    // DLOG(INFO) << PRINT(obj_x);
    // DLOG(INFO) << PRINT(object.second.center().y());
    // DLOG(INFO) << PRINT(diff_heading * rm::RAD2DEG);
    // DLOG(INFO) << PRINT(diff_heading * diff_heading /
                        // var_intersec_line_heading);
    if (diff_heading * diff_heading / var_intersec_line_heading <
        0.2)  // TODO the threshold shall be depedent on obj distance?
    // if ((std::abs(diff_heading) <
    //      2.0 * rm::DEG2RAD)) // TODO the threshold shall be depedent on obj
    //                          // distance?
    {
      measure_vector.push_back(std::make_pair(obj_x, heading));
      double obj_y = projection_point.y();
      double distance_to_ref_line =
          std::sqrt((obj_x - object.second.center().x()) *
                        (obj_x - object.second.center().x()) +
                    (obj_y - object.second.center().y()) *
                        (obj_y - object.second.center().y()));
      if (std::abs(object.second.center().y()) <= 10.0 &&
          std::abs(object.second.center().x()) <= 50.0) {
        temp_obstacles_new_disinfo_list_.push_back(std::make_pair(
            object.second.id(),
            IntersectionDisInfo(distance_to_ref_line,
                                obj_y - object.second.center().y(), obj_x)));
      }
      // DLOG(INFO) << "***********";
      debug_num_of_correct_objs++;
    }
    // TODO the historical trajectories of oncoming objects might be made use of
  }
  // DLOG(INFO) << PRINT(debug_num_of_correct_objs);

  if (first_time_selection_obstacles_) {
    for (auto &temp_obstacle : temp_obstacles_new_disinfo_list_) {
      obstacles_distance_update_list_[temp_obstacle.first] = std::make_pair(
          temp_obstacle.second.new_distance, temp_obstacle.second);
      // DLOG(INFO) << PRINT(temp_obstacle.second.new_distance);
    }
    first_time_selection_obstacles_ = false;
  } else {
    for (auto iter = obstacles_distance_update_list_.begin();
         iter != obstacles_distance_update_list_.end();) {
      auto it = std::find_if(
          temp_obstacles_new_disinfo_list_.begin(),
          temp_obstacles_new_disinfo_list_.end(),
          [iter](std::pair<uint8_t, IntersectionDisInfo> temp_obstacle) {
            return temp_obstacle.first == iter->first;
          });
      if (it == temp_obstacles_new_disinfo_list_.end()) {
        obstacles_distance_update_list_.erase(iter++);
      } else {
        // overwrite the new distance value
        iter->second.second = it->second;
        iter++;
      }
    }
  }
}  // namespace rm

bool IntersectionVirtualLaneGenerator::GenerateVitrualCenterLine(
    const rm::RoadInfoProvider *road_info_provider,
    dpc::TargetsInfo &ats_targetsinfo) {
  if (!initilized_) return false;
  PolynomLine intersec_line(
      {
          state_vector_(0),
          state_vector_(1),
          0.5 * state_vector_(2),
          state_vector_(3) / 6,
      },
      0.0, 0.0);
  frenet_converter_ = FrenetConverter(intersec_line);
  SetControl(VehicleStateProvider::Instance()->lon_velocity(),
             VehicleStateProvider::Instance()->lon_acceleration(),
             VehicleStateProvider::Instance()->yaw_rate() * rm::DEG2RAD);
  if (CheckGeneratePrecondition(road_info_provider)) {
    // DLOG(INFO) << "CheckGeneratePrecondition terminate";
    Terminate();
    return false;
  }
  if (std::abs(v_x_) < 0.001) {
    // initilized_ = false;
    // DLOG(INFO) << "speed terminate";
    Terminate();
    return false;
  }
  Predict();
  std::vector<std::pair<double, double>> measure_vector;
  SelectObstacles(
      road_info_provider, ats_targetsinfo,
      measure_vector);  // select obstacles that can be used to correct the
                        // coefficients of intersection bridge
  if (!measure_vector.empty()) {
    if (!Correct(measure_vector)) {
      // DLOG(INFO) << "correst terminate";
      Terminate();
      return false;
    };
  }

  if (CheckGeneratedCorridor(road_info_provider)) {
    return true;
  } else {
    // std::cout << "quit because of collision" << std::endl;
    // DLOG(INFO) << "CheckGeneratedCorridor terminate";
    Terminate();
    return false;
  }
}

void IntersectionVirtualLaneGenerator::SetInitialState(
    const rm::Corridor &corridor) {
  // AverageInitialState(corridor.center_line().Coef(0),
  // corridor.center_line().Coef(1), 2.0*corridor.center_line().Coef(2), 6.0*corridor.center_line().Coef(3));

  // state_vector_ << init_state_.at(0), init_state_.at(1), init_state_.at(2),
  // init_state_.at(3);
  state_vector_ << corridor.center_line().Coef(0),
      corridor.center_line().Coef(1), 2.0 * corridor.center_line().Coef(2),
      6.0 * corridor.center_line().Coef(3);
  state_cov_ << 0.008, 0.0, 0.0, 0.0, 0.0, 6.0e-2, 0.0, 0.0, 0.0, 0.0, 1.0e-4,
      0.0, 0.0, 0.0, 0.0, 1.0e-10;
  extend_distance_ = 0.0F;
  extend_duration_ = 0.0F;
  end_range_ = extend_distance_threshold_;
  dot_yaw_rate_low_pass_filter_.reset();
  last_corridor_id_ = corridor.corridor_id();
  corridor_last_time_before_invalid_ += delta_t_;
  obstacles_distance_update_list_.clear();
  first_time_selection_obstacles_ = true;
  initilized_ = true;
}

void IntersectionVirtualLaneGenerator::GenIntersectionCorridor(
    Corridor &corridor) {
  corridor.set_virtualdrivablewidth(3.5);
  corridor.set_corridor_id(last_corridor_id_);
  corridor.set_feature_tag(CorridorFeatureTag::LINEEXTEND);
  corridor.set_center_line({state_vector_(0), state_vector_(1),
                            0.5 * state_vector_(2), state_vector_(3) / 6},
                           0.0, end_range_);
}

void IntersectionVirtualLaneGenerator::SetControl(const double &v_x,
                                                  const double &a_x,
                                                  const double &yaw_rate) {
  // Set the control variables
  v_x_ = v_x;
  a_x_ = a_x;
  pre_yaw_rate_ = yaw_rate_;
  yaw_rate_ = yaw_rate;
  dot_yaw_rate_ =
      dot_yaw_rate_low_pass_filter_.run((yaw_rate_ - pre_yaw_rate_) / delta_t_);
  // DLOG(INFO) << PRINT(v_x_);
  // DLOG(INFO) << PRINT(a_x_);
  // DLOG(INFO) << PRINT(yaw_rate_);
  // DLOG(INFO) << PRINT(dot_yaw_rate_);
  // DLOG(INFO) << PRINT((yaw_rate_ - pre_yaw_rate_) / delta_t_);
}

void IntersectionVirtualLaneGenerator::Predict() {
  // compute the jacobian matrix of transition
  // jacobian_transmatrix_(0, 0) = 1.0;
  // jacobian_transmatrix_(0, 1) = delta_t_ * v_x_ * cos(state_vector_(1));
  // jacobian_transmatrix_(1, 1) = 1.0;
  // jacobian_transmatrix_(1, 2) = delta_t_ * v_x_;
  // jacobian_transmatrix_(2, 2) = 1.0 - a_x_ * delta_t_ / v_x_;
  // state_cov_ =
  //     jacobian_transmatrix_ * state_cov_ * jacobian_transmatrix_.transpose()
  //     + process_cov_;
  double delta_x = v_x_ * delta_t_;
  double delta_x_pow_2 = delta_x * delta_x;
  double delta_x_pow_3 = delta_x_pow_2 * delta_x;
  double delta_phi = yaw_rate_ * delta_t_;
  jacobian_transmatrix_(0, 0) = 1.0;
  jacobian_transmatrix_(0, 1) = delta_x;
  jacobian_transmatrix_(0, 2) = 0.5 * delta_x_pow_2;
  jacobian_transmatrix_(0, 3) = delta_x_pow_3 / 6;
  jacobian_transmatrix_(1, 1) = 1.0;
  jacobian_transmatrix_(1, 2) = delta_x;
  jacobian_transmatrix_(1, 3) = 0.5 * delta_x_pow_2;
  jacobian_transmatrix_(2, 2) = 1.0;
  jacobian_transmatrix_(2, 3) = delta_x;
  jacobian_transmatrix_(3, 3) = 1.0;

  state_cov_ =
      jacobian_transmatrix_ * state_cov_ * jacobian_transmatrix_.transpose() +
      process_cov_;

  Eigen::Matrix<double, 4, 1> input_vector = {-delta_phi * delta_x, -delta_phi,
                                              0.0, 0.0};

  // Eigen::Matrix<double, 4, 1>
  //     temp; // creat temp to avoid state from being overwritten
  // temp(0) = state_vector_(0) + v_x_ * delta_t_ * sin(state_vector_(1));
  // temp(1) = state_vector_(1) + (state_vector_(2) * v_x_ - yaw_rate_) *
  // delta_t_; temp(2) = state_vector_(2) +
  //           (dot_yaw_rate_ - state_vector_(2) * a_x_) * delta_t_ / v_x_;
  // temp(3) = state_vector_(3);
  state_vector_ = jacobian_transmatrix_ * state_vector_ + input_vector;

  // state_vector_ = temp;
  extend_distance_ =
      extend_distance_ + v_x_ * delta_t_ + 0.5F * a_x_ * delta_t_ * delta_t_;
  extend_duration_ = extend_duration_ + delta_t_;
  end_range_ = extend_distance_threshold_ - extend_distance_;

  filter_status_ = FilterStatus::PREDICT;

  // DLOG(INFO) << "<<<<<<<Predicting";
  // DLOG(INFO) << PRINT(state_vector_(0));
  // DLOG(INFO) << PRINT(state_vector_(1));
  // DLOG(INFO) << PRINT(state_vector_(2));
  // DLOG(INFO) << PRINT(state_vector_(3));
}

bool IntersectionVirtualLaneGenerator::Correct(
    const std::vector<std::pair<double, double>> &measure_vector) {
  uint8_t num_meas = measure_vector.size();
  uint8_t num_meas_distance = obstacles_distance_update_list_.size();
  // DLOG(INFO) << PRINT(+num_meas_distance);
  num_meas = num_meas + num_meas_distance;
  kalman_gain_ = Eigen::MatrixXd::Zero(4, num_meas);
  meas_cov_ = Eigen::MatrixXd::Zero(num_meas, num_meas);
  Eigen::MatrixXd innovation_cov = Eigen::MatrixXd::Zero(num_meas, num_meas);
  Eigen::VectorXd measure_innovation(num_meas, 1);
  Eigen::MatrixXd meas_matrix(num_meas, 4);
  double innovation_cov_det;
  int idx = 0;
  for (auto &&meas : measure_vector) {
    measure_innovation(idx) =
        meas.second - (state_vector_(1) + state_vector_(2) * meas.first +
                       0.5 * state_vector_(3) * meas.first * meas.first);
    // DLOG(INFO) << PRINT(measure_innovation(idx));
    meas_matrix(idx, 0) = 0.0;
    meas_matrix(idx, 1) = 1.0;
    meas_matrix(idx, 2) = meas.first;
    meas_matrix(idx, 3) = 0.5 * meas.first * meas.first;
    meas_cov_(idx, idx) = 0.04 + 0.08 * meas.first / 200;
    idx++;
  }

  for (auto &meas_dis : obstacles_distance_update_list_) {
    measure_innovation(idx) =
        meas_dis.second.first - meas_dis.second.second.new_distance;
    // DLOG(INFO) << PRINT(measure_innovation(idx));
    // DLOG(INFO) << PRINT(meas_dis.first);
    double delta_y = meas_dis.second.second.delta_y;
    double x_ref = meas_dis.second.second.x_ref;
    double dist = meas_dis.second.second.new_distance;
    double delta_y_devide_by_dist = delta_y / dist;
    meas_matrix(idx, 0) = delta_y_devide_by_dist;
    meas_matrix(idx, 1) = delta_y_devide_by_dist * x_ref;
    meas_matrix(idx, 2) = delta_y_devide_by_dist * x_ref * x_ref * 0.5;
    meas_matrix(idx, 3) = delta_y_devide_by_dist * x_ref * x_ref * x_ref / 6;
    meas_cov_(idx, idx) = 0.2;
    idx++;
  }

  // predict the measurement output based on the predicted state
  innovation_cov =
      meas_matrix * state_cov_ * meas_matrix.transpose() + meas_cov_;
  Eigen::FullPivLU<Eigen::MatrixXd> LU(innovation_cov);

  if (LU.isInvertible()) {
    kalman_gain_ =
        state_cov_ * meas_matrix.transpose() * innovation_cov.inverse();
    // DLOG(INFO) << PRINT(kalman_gain_);
    state_vector_ = state_vector_ + kalman_gain_ * measure_innovation;
    state_cov_ = (identity_4by4_ - kalman_gain_ * meas_matrix) * state_cov_;
    filter_status_ = FilterStatus::UPDATE;
  } else {
    Terminate();
    return false;
    // otherwise the matrix is singular, abort
  }
  // DLOG(INFO) << "<<<<<<<Correcting";
  // DLOG(INFO) << PRINT(state_vector_(0));
  // DLOG(INFO) << PRINT(state_vector_(1));
  // DLOG(INFO) << PRINT(state_vector_(2));
  // DLOG(INFO) << PRINT(state_vector_(3));
  return true;
}

void IntersectionVirtualLaneGenerator::GetState(std::array<double, 4> &state) {
  state = {state_vector_(0), state_vector_(1), state_vector_(2),
           state_vector_(3)};
}

void IntersectionVirtualLaneGenerator::GetStateCov(
    std::array<std::array<double, 4>, 4> &state_cov) {
  for (int idx = 0; idx < 4; idx++) {
    state_cov.at(idx) = {state_cov_(idx, 0), state_cov_(idx, 1),
                         state_cov_(idx, 2), state_cov_(idx, 3)};
  }
}

void IntersectionVirtualLaneGenerator::AverageInitialState(
    const double &y_0, const double &h_0, const double &kappa,
    const double &kappa_rate) {
  std::array<double, 4> input_array = {y_0, h_0, kappa, kappa_rate};
  if (initilized_ == false) {
    num_of_recstate_ = 1;
    current_state_idx_ = 0;
    int idx = 0;
    for (auto &&array_it : init_state_history_) {
      array_it.fill(0.0);
      array_it.at(0) = input_array.at(idx);
      init_state_.at(idx) = input_array.at(idx);
      idx++;
    }
  } else {
    num_of_recstate_ = (num_of_recstate_ >= 4) ? 4 : num_of_recstate_ + 1;
    current_state_idx_ =
        (current_state_idx_ == (4 - 1)) ? 0 : current_state_idx_ + 1;
    int idx = 0;
    for (auto &&array_it : init_state_history_) {
      array_it.at(current_state_idx_) = input_array.at(idx);
      init_state_.at(idx) =
          std::accumulate(array_it.begin(), array_it.end(), 0.0) /
          num_of_recstate_;
      idx++;
    }
  }
}

bool IntersectionVirtualLaneGenerator::CheckGeneratePrecondition(
    const rm::RoadInfoProvider *road_info_provider) {
  if (corridor_last_time_before_invalid_ < 6.0F) return true;
  // 1. the generated distance or duration exceeds the threshold
  if (extend_distance_ >= extend_distance_threshold_ ||
      extend_duration_ >= extend_duration_threshold_)
    return true;
  // 2. the extended corridor intersectes with oncoming vehilce
  // if (!road_info_provider->mutable_obstacles()->empty()) {
  //   for (const auto &object : *(road_info_provider->mutable_obstacles())) {
  //     if (object.second.moving_direction() ==
  //             ObstacleMovingDirection::PRECEDING ||
  //         object.second.moving_direction() ==
  //         ObstacleMovingDirection::UNKOWN)
  //       continue;
  //     common::math::ReferencePoint projection_point =
  //         frenet_converter_.FindRefPointS(object.second.center().x(),
  //                                         object.second.center().y());
  //     if (projection_point.s() > 0 &&
  //         projection_point.DistanceTo(object.second.center()) < 2.0) {
  //       DLOG(INFO) << PRINT(object.second.id());
  //       DLOG(INFO) << "collision with oncoming vehicle";
  //       // DLOG(INFO) <<
  //       // PRINT(+static_cast<uint8_t>(object.second.moving_direction()));
  //       // DLOG(INFO) << PRINT(+object.first);
  //       return true;
  //     }
  //   }
  // }
  return false;
}

bool IntersectionVirtualLaneGenerator::CheckGeneratedCorridor(
    const rm::RoadInfoProvider *road_info_provider) {
  PolynomLine virtual_corridor({state_vector_(0), state_vector_(1),
                                state_vector_(2) * 0.5, state_vector_(3) / 6},
                               0.0, end_range_);
  double trans_angle_odo2map = road_info_provider->odometry().yaw;
  Vec2d trans_coord_odo2map = road_info_provider->odometry().point -
                              road_info_provider->freespace().origin;
  // transformed_reference_points_map stores the ref points position in map
  // coordinate
  std::vector<rm::Vec2d> transformed_reference_points_map;
  for (double x = 0.0; x < end_range_;) {
    Vec2d inter_pos(x, virtual_corridor.Evaluate(0, x));
    inter_pos.SelfRotate(trans_angle_odo2map);
    transformed_reference_points_map.push_back(inter_pos + trans_coord_odo2map);
    x += 1.0;
  }
  double map_resolution = road_info_provider->freespace().resolution;
  double map_width = road_info_provider->freespace().width * map_resolution;
  double map_height = road_info_provider->freespace().height * map_resolution;
  uint16_t i_x, i_y;
  uint32_t idx;  // idx in x/y direction
  for (const auto &trans_pos_map : transformed_reference_points_map) {
    // continue if the reference point is out of map
    if ((trans_pos_map.x() > map_width || trans_pos_map.x() < 0.0) ||
        (trans_pos_map.y() > map_height || trans_pos_map.y() < 0.0))
      continue;
    i_x = trans_pos_map.x() / map_resolution;
    i_y = trans_pos_map.y() / map_resolution;
    idx = i_x + i_y * road_info_provider->freespace().width;
    if (road_info_provider->freespace().data.at(idx) == 1U) {
      return freespace_collision_delay_.RunDelay(false);
    }
  }
  freespace_collision_delay_.RunDelay(true);
  return true;
}

}  // namespace rm