/*
 * @Author: your name
 * @Date: 2022-03-10 19:07:29
 * @LastEditTime: 2022-03-21 19:33:40
 * @LastEditors: Please set LastEditors
 * @Description: 打开koroFileHeader查看配置 进行设置:
 * https://github.com/OBKoro1/koro1FileHeader/wiki/%E9%85%8D%E7%BD%AE
 * @FilePath:
 * /roadmodel/src/mpc_drv_ctap_rm/src/road/intersection_virtual_lane_generator.h
 */
#include <climits>
#include <cmath>
#include <iostream>
#include <numeric>
#include <utility>
#include <vector>

#include "Eigen/Dense"
#include "corridor.h"
#include "delay.h"
#include "freespace.h"
#include "frenet_converter.h"
#include "gflags.h"
#include "global_parameters.h"
#include "logging.h"
#include "low_pass_filter.h"
#include "macros.h"
#include "road_info_provider.h"
#include "vehicle_state_provider.h"
// #include "mpc_drv_msgs/msg/targets_info.hpp"
#include "dpc/impl_type_targetsinfo.h"

typedef Eigen::Matrix<double, 4, 4> Matrix4d;
typedef Eigen::Matrix<double, 4, 1> Vector4d;

namespace rm {
struct IntersectionDisInfo {
  IntersectionDisInfo(){};
  IntersectionDisInfo(double new_distance_, double delta_y_, double x_ref_)
      : new_distance(new_distance_), delta_y(delta_y_), x_ref(x_ref_){};
  double new_distance;
  double delta_y;
  double x_ref;
};
class IntersectionVirtualLaneGenerator {
  // Reference: [1] Joint Ego-Motion and Road Geometry Estimation
  // [2] long range road geometry estimation using moving vehicles and road side
  // observation
public:
  void SetInitialState(const rm::Corridor &corridor);
  void SelectObstacles(const rm::RoadInfoProvider *road_info_provider,
                       const dpc::TargetsInfo &ats_targetsinfo,
                       std::vector<std::pair<double, double>> &measure_vector);
  bool
  GenerateVitrualCenterLine(const rm::RoadInfoProvider *road_info_provider,
                            dpc::TargetsInfo &ats_targetsinfo);
  void GetState(std::array<double, 4> &state_result);
  void GetStateCov(std::array<std::array<double, 4>, 4> &state_cov);
  float extend_distance() const;
  float extend_duration() const;
  void GenIntersectionCorridor(Corridor &corridor);
  FilterStatus filter_status() const { return filter_status_; };

private:
  void Predict();
  bool
  Correct(const std::vector<std::pair<double, double>>
              &measure_vector); //  measurement vector, the first element is
                                //  long. dis, and the second element is heading
  void AverageInitialState(const double &y_0, const double &h_0,
                           const double &kappa, const double &kappa_rate);
  void Terminate();
  void SetControl(const double &v_x, const double &a_x,
                  const double &yaw_rate); // set the control variables
  bool initilized() const;
  bool
  CheckGeneratePrecondition(const rm::RoadInfoProvider *road_info_provider);
  bool CheckGeneratedCorridor(const rm::RoadInfoProvider *road_info_provider);

  Matrix4d jacobian_transmatrix_ =
      Eigen::Matrix<double, 4, 4>::Zero(); // jacobian of transition 4*4
  Matrix4d identity_4by4_ = Eigen::Matrix<double, 4, 4>::Identity();
  Eigen::MatrixXd kalman_gain_;
  Matrix4d state_cov_;    // covariance of state 4*4
  Vector4d state_vector_; // state vector 4*1
  // l: distance to central lane m
  // delta_r: the heading angle of the central lane wrt ego vehicle's direction
  // of velocity rad c_0: curvature of the central lane 1/m c_1: change of the
  // curvature
  Matrix4d process_cov_ =
      (Eigen::Matrix4d() << 1e-4, 0.0, 0.0, 0.0, 0.0, 0.07, 0.0, 0.0, 0.0, 0.0,
       1.8e-7, 0.0, 0.0, 0.0, 0.0, 1.0e-12)
          .finished();
  Eigen::MatrixXd meas_cov_;
  rm::FrenetConverter frenet_converter_;

  uint8_t num_of_recstate_ = 0;
  uint8_t current_state_idx_ = 0;
  std::array<std::array<double, 4>, 4> init_state_history_;
  std::array<double, 4> init_state_ = {{0.0, 0.0, 0.0, 0.0}};
  // Control input variables
  double v_x_;
  double a_x_;
  double yaw_rate_ = 0.0;
  double pre_yaw_rate_ = 0.0;
  double dot_yaw_rate_ = 0.0;
  const double delta_t_ = rm::ROADMODELCYCLEINS;
  float corridor_last_time_before_invalid_ = 0.0F;
  // the distance and duration the filter has worked over
  float extend_distance_ = 0.0F;
  float extend_duration_ = 0.0F;
  bool initilized_ = false;
  double end_range_ = 100.0F;
  uint32_t last_corridor_id_ = 0;
  const float extend_distance_threshold_ = 100.0; // m
  const float extend_duration_threshold_ = 10.0;  // s
  common::math::LowPassFilter dot_yaw_rate_low_pass_filter_ =
      common::math::LowPassFilter(0.2F);
  Delay<bool> freespace_collision_delay_ = {0.0, 3.0, true};
  FilterStatus filter_status_ = FilterStatus::INVALID;
  bool first_time_selection_obstacles_ = true;
  std::map<uint32_t, std::pair<double, IntersectionDisInfo>>
      obstacles_distance_update_list_;

  DECLARE_SINGLETON(IntersectionVirtualLaneGenerator);
};
} // namespace rm
