#include "lane_change_intention_detector.h"

namespace rm {

bool LaneChangeIntentionDetector::Run(const ObjectLocation &obj_loc,
                                      bool is_fv) {
  if (first_run_) {
    prv_reference_point2corridor_frenet_l_ =
        obj_loc.reference_point2corridor_frenet.l();
    diff_l_ = 0.0;
    first_run_ = false;
  } else {
    diff_l_ = (obj_loc.reference_point2corridor_frenet.l() -
               prv_reference_point2corridor_frenet_l_) /
              rm::ROADMODELCYCLEINS;
    prv_reference_point2corridor_frenet_l_ =
        obj_loc.reference_point2corridor_frenet.l();
  }
  bool cond_flag = false;
  // DLOG(INFO) << "++++++++++++++++++++++";
  // DLOG(INFO) << PRINT(diff_l_);
  // DLOG(INFO) << PRINT(is_fv);
  // DLOG(INFO) << PRINT(obj_loc.reference_point2corridor_frenet.dl());
  double obj_l = obj_loc.reference_point2corridor_frenet.l();
  double obj_abs_l = std::abs(obj_l);
  double dl_upper_threshold = dl_upper_threshold_l_.Evaluate(obj_abs_l);
  double dl_lower_threshold = dl_lower_threshold_l_.Evaluate(obj_abs_l);
  // DLOG(INFO) << PRINT(obj_l);
  if (is_fv) {
    dl_debouncer_.SetDebounce(-1.0 * dl_upper_threshold,
                              -1.0 * dl_lower_threshold);
    cond_flag = dl_debouncer_.RunDebouceLessTure(diff_l_);
  } else {
    dl_debouncer_.SetDebounce(dl_lower_threshold, dl_upper_threshold);
    cond_flag = dl_debouncer_.RunDebouceMoreTure(diff_l_);
  }
  // DLOG(INFO) << PRINT(cond_flag);
  auto accumulated_counters = lane_change_counter_.Run(cond_flag, obj_abs_l);
  // DLOG(INFO) << PRINT(obj_loc.object_id);
  // DLOG(INFO) << PRINT(accumulated_counters);
  double counter_threshold =
      lane_change_fullfill_counter_l_.Evaluate(obj_abs_l);
  // DLOG(INFO) << PRINT(counter_threshold);
  return accumulated_counters > counter_threshold; // TODO counter threshold
};

const OneDimLookupTable
    LaneChangeIntentionDetector::lane_change_fullfill_counter_l_ =
        OneDimLookupTable({1.0, 1.4, 1.8, 2.2, 2.6, 2.8, 3.0, 3.5, 5.0},
                          {0.0, 1.0, 2.0, 3.0, 5.0, 9.0, 25.0, 50.0, 100.0});
const OneDimLookupTable LaneChangeIntentionDetector::dl_upper_threshold_l_ =
    OneDimLookupTable({1.0, 1.4, 1.8, 2.2, 2.6, 2.8, 3.0, 3.5},
                      {0.05, 0.1, 0.12, 0.15, 0.18, 0.2, 0.25, 0.3});
const OneDimLookupTable LaneChangeIntentionDetector::dl_lower_threshold_l_ =
    OneDimLookupTable({1.0, 1.4, 1.8, 2.2, 2.6, 2.8, 3.0, 3.5},
                      {0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.2});

const OneDimLookupTable LaneChangeCounter::increament_l_ =
    OneDimLookupTable({1.0, 1.4, 1.8, 2.2, 2.6, 2.8, 3.0, 3.5, 4.0, 5.0},
                      {1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 0.5, 0.2});

const OneDimLookupTable LaneChangeCounter::decrement_l_ =
    OneDimLookupTable({1.0, 1.4, 1.8, 2.2, 2.6, 2.8, 3.0, 3.5, 4.0, 5.0},
                      {1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.5, 5.0, 5.0});

} // namespace rm