#pragma once

#include "debouncer.h"
#include "global_parameters.h"
#include "glog/logging.h"
#include "lookup_table.h"
#include "macros.h"
#include "object_location.h"
#include <stdint.h>

namespace rm {

class LaneChangeCounter {
public:
  uint16_t Run(bool flag, float abs_l) {
    if (flag) {
      counters_ = std::min(counters_ + increament_l_.Evaluate(abs_l), 250.0);
    } else {
      counters_ = std::max(counters_ - decrement_l_.Evaluate(abs_l), 0.0);
    }
    return counters_;
  };
  void Reset() { counters_ = 0.0; }

private:
  float counters_{0};
  static const OneDimLookupTable increament_l_;
  static const OneDimLookupTable decrement_l_;
};

class LaneChangeIntentionDetector {
public:
  bool Run(const ObjectLocation &obj_loc, bool is_fv);
  void Reset() {
    lane_change_counter_.Reset();
    dl_debouncer_.ResetDebounce();
    first_run_ = true;
  }

private:
  bool first_run_;
  float prv_reference_point2corridor_frenet_l_;
  float diff_l_;
  LaneChangeCounter lane_change_counter_;
  Debouncer dl_debouncer_;
  static const OneDimLookupTable lane_change_fullfill_counter_l_;
  static const OneDimLookupTable dl_upper_threshold_l_;
  static const OneDimLookupTable dl_lower_threshold_l_;
};
} // namespace rm
