/*
 * @Author: your name
 * @Date: 2021-09-28 12:48:40
 * @LastEditTime: 2022-01-20 10:32:01
 * @LastEditors: Please set LastEditors
 * @Description: In User Settings Edit
 * @FilePath: /RM/roadmodel/modules/common/road/lane.h
 */
#pragma once
#include <utility>

#include "boundary.h"
#include "corridor.h"
#include "lane.h"
namespace rm {
/**
 * @class: LaneInfo
 * @brief：task加工的对象 task计算的结果均存储在此处
 */
class LaneInfo {
 public:
  LaneInfo() = default;
  /**
   * @brief: 根据lane构造laneinfo
   * @param {Lane&} lane
   * @return {*}
   */
  LaneInfo(const Lane& lane);
  /**
   * @brief:返回lane信息
   * @param {*}
   * @return {*}
   */
  const Lane& lane() const;

  Lane* mutable_lane() { return &lane_; };
  /**
   * @brief: 存boundary
   * @param {*}
   * @return {*}
   */
  void set_boundary(const std::pair<Boundary, Boundary>& boundary);

  void set_back_boundary(const std::pair<Boundary, Boundary>& boundary);

  void set_boundary_pair_type();
  /**
   * @brief: 返回boundary
   * @param {*}
   * @return {*}
   */
  const std::pair<Boundary, Boundary>& boundary() const;

  std::pair<Boundary, Boundary>* mutable_boundary();

  const std::pair<Boundary, Boundary>& back_boundary() const;
  /**
   * @brief: 返回corridor地址
   * @param {*}
   * @return {*}
   */
  std::vector<Corridor>* mutable_corridor();
  std::vector<Corridor>* mutable_back_corridor();
  /**
   * @brief: 返回corridor的值
   * @param {*}
   * @return {*}
   */
  const std::vector<Corridor>& corridor() const;

  const std::vector<Corridor>& back_corridor() const;

  BoundaryPairType boundary_pair_type() const;

  bool operator==(const LanePosition& lane_position) {
    return lane_.lane_position() == lane_position;
  }

 private:
  Lane lane_;  //原始lane信息

  std::pair<Boundary, Boundary>
      boundary_;  // task1生成的的boundary 可能是车道线或道路边缘
                  // first:left_boundary  second:right_boundary

  BoundaryPairType boundary_pair_type_;

  std::pair<Boundary, Boundary> back_boundary_;  //车道后方的boundary

  std::vector<Corridor>
      corridor_;  // lane的corridor‘列表 对于egolane的corridor较多
                  // 对于其他lane的corridor只有一个

  std::vector<Corridor> back_corridor_;
};
}  // namespace rm