/*
 * @Author: your name
 * @Date: 2021-10-09 14:26:22
 * @LastEditTime: 2022-01-20 09:53:28
 * @LastEditors: Please set LastEditors
 * @Description: In User Settings Edit
 * @FilePath:
 * /c++test/media/igs/xnm/RM/rm/modules/common/road/boundary_segment.h
 */
#pragma once

#include <stdint.h>

#include <list>

#include "enum_list.h"
#include "lane_marker_segment.h"
#include "lane_marker_segment_attribute.h"
namespace rm {
/**
 * @class: LaneMarker
 * @brief: 存储cem的lanemarker
 */
class LaneMarker {
 public:
  LaneMarker() = default;
  ~LaneMarker() = default;
  /**
   * @brief: 存取函数
   * @param {uint32_t} &lane_marker_id
   * @return {*}
   */
  void set_lane_marker_id(const uint32_t &lane_marker_id) {
    lane_marker_id_ = lane_marker_id;
  }

  LaneMarkerSegment *add_front_lane_marker_segment() {
    front_lane_marker_segment_.emplace_back();
    return &front_lane_marker_segment_.back();
  }

  LaneMarkerSegment *add_back_lane_marker_segment() {
    back_lane_marker_segment_.emplace_back();
    return &back_lane_marker_segment_.back();
  }

  LaneMarkerSegmentAttribute *add_lane_marker_seg_attribute() {
    lane_marker_segment_attribute_.emplace_back();
    return &lane_marker_segment_attribute_.back();
  }

  void set_is_start_out_of_range(const bool is_start_out_of_range) {
    is_start_out_of_range_ = is_start_out_of_range;
  }

  void set_is_end_out_of_range(const bool is_end_out_of_range) {
    is_end_out_of_range_ = is_end_out_of_range;
  }

  const std::vector<LaneMarkerSegment> &front_lane_marker_segment() const {
    return front_lane_marker_segment_;
  }

  std::vector<LaneMarkerSegment> *mutable_front_lane_marker_segment() {
    return &front_lane_marker_segment_;
  }

  const std::vector<LaneMarkerSegment> &back_lane_marker_segment() const {
    return back_lane_marker_segment_;
  }

  std::vector<LaneMarkerSegment> *mutable_back_lane_marker_segment() {
    return &back_lane_marker_segment_;
  }

  const std::vector<LaneMarkerSegmentAttribute> &lane_marker_segment_attribute()
      const {
    return lane_marker_segment_attribute_;
  }

  std::vector<LaneMarkerSegmentAttribute>
      *mutable_lane_marker_segment_attribute() {
    return &lane_marker_segment_attribute_;
  }

  uint32_t lane_marker_id() const { return lane_marker_id_; }

  bool operator==(const uint32_t id) const { return id == lane_marker_id_; }

  bool is_start_out_of_range() const { return is_start_out_of_range_; }

  bool is_end_out_of_range() const { return is_end_out_of_range_; }

  rm::LineColor lane_marker_color() const {
    if (lane_marker_segment_attribute_.empty()) {
      return rm::LineColor::UNKNOWN;
    } else {
      auto it = std::find_if(
          lane_marker_segment_attribute_.begin(),
          lane_marker_segment_attribute_.end(),
          [](const LaneMarkerSegmentAttribute &seg) {
            return (0.0 < seg.end_offset() && 0.0 >= seg.start_offset());
          });
      if (it != lane_marker_segment_attribute_.end()) {
        return it->line_color();
      } else {
        return lane_marker_segment_attribute_.back().line_color();
      }
    }
  }

  rm::LineType lane_marker_type() const {
    if (lane_marker_segment_attribute_.empty()) {
      return rm::LineType::UNKNOWN;
    } else {
      auto it = std::find_if(
          lane_marker_segment_attribute_.begin(),
          lane_marker_segment_attribute_.end(),
          [](const LaneMarkerSegmentAttribute &seg) {
            return (0.0 < seg.end_offset() && 0.0 >= seg.start_offset());
          });
      if (it != lane_marker_segment_attribute_.end()) {
        return it->line_type();
      } else {
        return lane_marker_segment_attribute_.back().line_type();
      }
    }
  }

 private:
  uint32_t lane_marker_id_;  //车道线ID
  std::vector<LaneMarkerSegment> front_lane_marker_segment_;
  std::vector<LaneMarkerSegment> back_lane_marker_segment_;
  bool is_start_out_of_range_;
  bool is_end_out_of_range_;
  std::vector<LaneMarkerSegmentAttribute> lane_marker_segment_attribute_;
};
}  // namespace rm
