/*
 * @Author: your name
 * @Date: 2021-12-03 14:28:49
 * @LastEditTime: 2022-03-09 21:01:03
 * @LastEditors: Please set LastEditors
 * @Description: 打开koroFileHeader查看配置 进行设置:
 * https://github.com/OBKoro1/koro1FileHeader/wiki/%E9%85%8D%E7%BD%AE
 * @FilePath:
 * /EHR/home/igs/roadmodel/src/CtApRoadModel/source/road/lane_marker_segment.cc
 */
#pragma once

#include "lane_marker_segment.h"

namespace rm {

void LaneMarkerSegment::set_lane_marker_segment_id(
    const uint32_t &lane_marker_segment_id) {
  lane_marker_segment_id_ = lane_marker_segment_id;
}

void LaneMarkerSegment::set_line_geometry(const LineGeometry &gemotry) {
  geometry_ = gemotry;
}

void LaneMarkerSegment::set_lane_marker_width(const float lane_marker_width) {
  lane_marker_width_ = lane_marker_width;
}

void LaneMarkerSegment::set_color(const LineColor &color) { color_ = color; }

void LaneMarkerSegment::set_type(const LineType &type) { type_ = type; }

LineGeometry *LaneMarkerSegment::mutable_geometry() { return &geometry_; }

uint32_t LaneMarkerSegment::lane_marker_segment_id() const {
  return lane_marker_segment_id_;
}

const LineGeometry &LaneMarkerSegment::geometry() const { return geometry_; }

LineType LaneMarkerSegment::type() const { return type_; }

LineColor LaneMarkerSegment::color() const { return color_; }

bool LaneMarkerSegment::operator==(const uint32_t id) const {
  return lane_marker_segment_id_ == id;
}

void LaneMarkerSegment::set_sampling_point_list(
    const dpc::GeometryLineStruct &geometryline) {
  for (const auto &sample_point : geometryline.sampling_point_list) {
    sampling_point_list_.push_back(
        SamplingPoint(sample_point.x, sample_point.y, sample_point.curve, sample_point.offset));
  }
}

const std::vector<SamplingPoint> &LaneMarkerSegment::sampling_point_list()
    const {
  return sampling_point_list_;
}

void LaneMarkerSegment::compute_arc_length_from_geometry() {
  arc_length_ = geometry_.mutable_curve()->ComputeArcLengthbySimpson();
}

void LaneMarkerSegment::compute_arc_length_from_sampling_points() {
  double sampling_point_list_size = sampling_point_list_.size();
  arc_length_ = 0.0F;
  if (sampling_point_list_size > 1) {
    float diff_x;
    float diff_y;
    for (int i = 0; i < sampling_point_list_size - 1; i++) {
      diff_x = sampling_point_list_.at(i).x - sampling_point_list_.at(i + 1).x;
      diff_y = sampling_point_list_.at(i).y - sampling_point_list_.at(i + 1).y;
      arc_length_ = arc_length_ + std::sqrt(diff_x * diff_x + diff_y * diff_y);
    }
  }
}

void LaneMarkerSegment::compute_arc_length() {
  if (geometry_.valid()) {
    compute_arc_length_from_geometry();
  } else {
    compute_arc_length_from_sampling_points();
  }
}

float LaneMarkerSegment::arc_length() const { return arc_length_; }

void LaneMarkerSegment::set_is_sampling_point_valid(bool is_sampling_point) {
  is_sampling_point_valid_ = is_sampling_point;
}

bool LaneMarkerSegment::is_sampling_point_valid() const {
  return is_sampling_point_valid_;
}

std::vector<SamplingPoint> *LaneMarkerSegment::mutable_sampling_point_list() {
  return &sampling_point_list_;
}

}  // namespace rm