#pragma once
#include <stdint.h>
#include "enum_list.h"

namespace rm{

class LaneSegmentAttribute{
    public:
    void set_start_offset(float start_offset){start_offset_ = start_offset;}
    
    void set_end_offset(float end_offset){end_offset_ = end_offset;}

    void set_lane_type(uint8_t lane_type){
        switch (lane_type){
            case 0U:
                lane_type_ = rm::LaneType::NORMAL;
                break;
            case 1U:
                lane_type_ = rm::LaneType::ENTRY;
                break;
            case 2U:
                lane_type_ = rm::LaneType::EXIT;
                break;
            case 3U:
                lane_type_ = rm::LaneType::EMERGENCY_LANE;
                break;
            case 4U:
                lane_type_ = rm::LaneType::ON_RAMP;
                break;
            case 5U:
                lane_type_ = rm::LaneType::OFF_RAMP;
                break;
            case 6U:
                lane_type_ = rm::LaneType::CONNECTING_RAMP;
                break;
            case 7U:
                lane_type_ = rm::LaneType::ACCLERATION_LANE;
                break;
            case 8U:
                lane_type_ = rm::LaneType::DECELERATION_LANE;
                break;
            case 9U:
                lane_type_ = rm::LaneType::EMERGENCY_PARKING_STRIP;
                break;
            case 10U:
                lane_type_ = rm::LaneType::REGULAR_LANE;
                break;
            case 11U:
                lane_type_ = rm::LaneType::DRIVALE_SHOULDER_LANE;
                break;
            case 12U:
                lane_type_ = rm::LaneType::SLOW_LANE;
                break;
            case 13U:
                lane_type_ = rm::LaneType::CARPOOL_LANE;
                break;
            case 14U:
                lane_type_ = rm::LaneType::BOTH_DIRECTION_LANE;
                break;
            case 15U:
                lane_type_ = rm::LaneType::BUS_LANE;
                break;
            case 16U:
                lane_type_ = rm::LaneType::DRIVABLE_PARING_LANE;
                break;
            case 17U:
                lane_type_ = rm::LaneType::DIVERSION;
                break;
            case 18U:
                lane_type_ = rm::LaneType::EMERGENCY_PARKING_BAR;
                break;
            case 19U:
                lane_type_ = rm::LaneType::VIRTUAL_LANE;
                break;
            case 20U:
                lane_type_ = rm::LaneType::NON_MOTORIZED_VEHICLE_LANE;
                break;
        }
    }

    void set_max_speed_limit(float max_speed_limit){max_speed_limit_ = max_speed_limit;}

    void set_min_speed_limit(float min_speed_limit){min_speed_limit_ = min_speed_limit;}

    void set_transit_type(uint8_t transit_type){transit_type_ = static_cast<rm::TransitType>(transit_type);}

    float start_offset() const {return start_offset_;}

    float end_offset() const {return end_offset_;}

    rm::LaneType lane_type() const {return lane_type_;}

    float max_speed_limit() const {return max_speed_limit_;}

    float min_speed_limit() const {return min_speed_limit_;}

    rm::TransitType transit_type() const {return transit_type_;}

    private:
    float start_offset_;
    float end_offset_;
    rm::LaneType lane_type_;
    float max_speed_limit_;
    float min_speed_limit_;
    rm::TransitType transit_type_;
};

}