/*
 * @Author: your name
 * @Date: 2021-12-03 14:53:07
 * @LastEditTime: 2021-12-03 15:08:55
 * @LastEditors: Please set LastEditors
 * @Description: 打开koroFileHeader查看配置 进行设置:
 * https://github.com/OBKoro1/koro1FileHeader/wiki/%E9%85%8D%E7%BD%AE
 * @FilePath:
 * /EHR/home/igs/roadmodel/src/CtApRoadModel/source/road/line_geometry.cc
 */
#pragma once

#include "line_geometry.h"

namespace rm {

void LineGeometry::set_valid(bool valid) { valid_ = valid; }

void LineGeometry::set_curve(PolynomLine curve) { curve_ = curve; }

void LineGeometry::set_curve_std(PolynomLine curve_std) {
  curve_std_ = curve_std;
}

void LineGeometry::set_exist_probability(float exist_probability) {
  exist_probability_ = exist_probability;
  hdm_c3_ = exist_probability;
}

void LineGeometry::set_life_time(float life_time) { hdm_c2_ = life_time; }

void LineGeometry::set_measurement_state(
    const MeasurementState& measurement_state) {
  measurement_state_ = measurement_state;
}

float LineGeometry::exist_probability() const { return exist_probability_; }

const PolynomLine& LineGeometry::curve() const { return curve_; }

const PolynomLine& LineGeometry::curve_std() const { return curve_std_; }

PolynomLine* LineGeometry::mutable_curve() { return &curve_; }

bool LineGeometry::valid() const { return valid_; }

LineGeometry LineGeometry::MoveCurved(double step, bool is_front_marker) {
  LineGeometry ans;
  ans.set_valid(valid_);
  if (valid_) {
    ans.set_curve(curve_.MoveCurved(step, is_front_marker));
    ans.set_curve_std(curve_std_);
  }
  return ans;
}

LineGeometry LineGeometry::MoveCurved(double step) {
  LineGeometry ans;
  ans.set_valid(valid_);
  if (valid_) {
    ans.set_curve(curve_.MoveCurved(step));
    ans.set_curve_std(curve_std_);
  }
  return ans;
}

LineGeometry LineGeometry::MoveCurvedLongitudinal(double target_range) {
  LineGeometry ans;
  ans.set_valid(valid_);
  if (valid_) {
    PolynomLine curve;
    std::array<double, 4> coef_new;
    coef_new[0] = curve_.Coef(0) + curve_.Coef(1) * target_range +
                  curve_.Coef(2) * target_range * target_range +
                  curve_.Coef(3) * target_range * target_range * target_range;
    coef_new[1] = curve_.Coef(1) + 2 * curve_.Coef(2) * target_range +
                  3 * curve_.Coef(3) * target_range * target_range;
    coef_new[2] = curve_.Coef(2) + 3 * curve_.Coef(3) * target_range;
    coef_new[3] = curve_.Coef(3);
    curve.set_end_range(curve_.end_range() - target_range);
    curve.set_start_range(curve_.start_range() - target_range);
    curve.set_coef(coef_new[0], coef_new[1], coef_new[2], coef_new[3]);
    ans.set_curve(curve);
  }
  return ans;
}

bool LineGeometry::CalculateDiscretePointsWithWeight(double step) {
  reference_points_ = curve_.DiscreteCurved(step);
  weight_.clear();
  weight_.insert(weight_.begin(), reference_points_.size(),
                 1.0);  // TODO(ningmeng):暂时选择平均权重
  return true;
}

bool LineGeometry::CalculateDiscretePointsWithWeight(double end_range,
                                                     double step) {
  reference_points_ = curve_.DiscreteCurved(end_range, step);
  weight_.clear();
  weight_.insert(weight_.begin(), reference_points_.size(),
                 0.5);  // TODO(ningmeng):暂时选择平均权重
  return true;
}

Status LineGeometry::GetReferencePiontWithWeight(
    std::vector<std::pair<ReferencePoint, double>>* reference_points,
    double step) {
  CalculateDiscretePointsWithWeight(step);

  if (reference_points_.size() != weight_.size())
    return Status(ErrorCode::RM_ERROR, "fail to discrete point");

  auto iter1 = reference_points_.begin();
  auto iter2 = weight_.begin();
  for (; iter1 != reference_points_.end(), iter2 != weight_.end();) {
    reference_points->emplace_back(std::make_pair(*iter1, *iter2));
    iter1++;
    iter2++;
  }
  return Status::OK();
}

Status LineGeometry::GetReferencePiontWithWeight(
    std::vector<std::pair<ReferencePoint, double>>* reference_points,
    double end_range, double step) {
  CalculateDiscretePointsWithWeight(end_range, step);

  if (reference_points_.size() != weight_.size())
    return Status(ErrorCode::RM_ERROR, "fail to discrete point");

  auto iter1 = reference_points_.begin();
  auto iter2 = weight_.begin();
  for (; iter1 != reference_points_.end(), iter2 != weight_.end();) {
    reference_points->emplace_back(std::make_pair(*iter1, *iter2));
    iter1++;
    iter2++;
  }
  return Status::OK();
}

std::ostream& operator<<(std::ostream& os, const LineGeometry& line) {
  os << "\n"
     << "curve_ = " << line.curve_ << "\n"
     << "curve_std_ = " << line.curve_std_;
  return os;
}

}  // namespace rm