/*
 * @Author: your name
 * @Date: 2021-09-28 16:08:01
 * @LastEditTime: 2021-12-03 15:18:10
 * @LastEditors: Please set LastEditors
 * @Description: In User Settings Edit
 * @FilePath: /RM/roadmodel/modules/common/math/geometry_line.h
 */
#pragma once
#include <vector>

#include "enum_list.h"
#include "polynom_line.h"
#include "reference_point.h"
#include "status.h"
namespace rm {
using common::math::ReferencePoint;
using rm::common::ErrorCode;
using rm::common::Status;

class LineGeometry {
 public:
  /**
   * @brief: 存取函数
   * @param {bool} valid
   * @return {*}
   */
  void set_valid(bool valid);

  void set_curve(PolynomLine curve);

  void set_curve_std(PolynomLine curve_std);

  void set_exist_probability(float exist_probability);

  void set_life_time(float life_time);

  void set_measurement_state(const MeasurementState& measurement_state);

  float exist_probability() const;

  float hdm_c2() const { return hdm_c2_; }

  float hdm_c3() const { return hdm_c3_; }

  const PolynomLine& curve() const;

  const PolynomLine& curve_std() const;

  bool valid() const;

  PolynomLine* mutable_curve();

  MeasurementState measurement_state() const { return measurement_state_; }

  /**
   * @brief: 根据曲线方程及std方程计算带权重的离散点
   * @param  {*}
   * @return  std::vector<std::tuple<离散点,离散点权重>>
   */
  LineGeometry MoveCurved(double step, bool is_front_marker);

  LineGeometry MoveCurved(double step);

  LineGeometry MoveCurvedLongitudinal(double target_range);

  /**
   * @brief: 获取带权重的离散点
   * @param {vector<std::pair<ReferencePoint,double>>} *
   * @param {double} step
   * @return {*}
   */
  Status GetReferencePiontWithWeight(
      std::vector<std::pair<ReferencePoint, double>>* reference_points,
      double end_range, double step);
  Status GetReferencePiontWithWeight(
      std::vector<std::pair<ReferencePoint, double>>* reference_points,
      double step);
  /**
   * @brief: 重载”<<“打印该类成员
   * @param {*}
   * @return {*}
   */
  friend std::ostream& operator<<(std::ostream& os, const LineGeometry& line);

 private:
  /**
   * @brief: 离散曲线型点并计算权重
   * @param {double} step
   * @return {*}
   */
  bool CalculateDiscretePointsWithWeight(double end_range, double step);

  bool CalculateDiscretePointsWithWeight(double step);

 private:
  bool valid_;  //曲线是否有效
  MeasurementState measurement_state_;
  PolynomLine curve_;                             //曲线表达式
  PolynomLine curve_std_;                         //曲线标准差
  float exist_probability_;                       //存在概率
  std::vector<ReferencePoint> reference_points_;  //离散的参考点
  std::vector<double> weight_;  //离散参考点对应的权重
  float hdm_c2_;                // raw c2 from hdm
  float hdm_c3_;                // raw c3 from hdm
};

}  // namespace rm