#include "road_edge.h"

namespace rm {

void RoadEdge::set_road_edge_id(uint32_t id) { road_edge_id_ = id; }

void RoadEdge::set_position(rm::RoadEdgePostion position) {
  position_ = position;
}

void RoadEdge::add_linked_lane_segment_id(uint32_t lane_seg_id) {
  linked_lane_segment_id_list_.push_back(lane_seg_id);
}

RoadEdgeSegment *RoadEdge::add_road_edge_segment() {
  road_edge_segment_list_.emplace_back();
  return &road_edge_segment_list_.back();
}

uint32_t RoadEdge::road_edge_id() const { return road_edge_id_; }

rm::RoadEdgePostion RoadEdge::position() const { return position_; }

const std::vector<uint32_t> &RoadEdge::linked_lane_segment_id_list() const {
  return linked_lane_segment_id_list_;
}

const std::vector<RoadEdgeSegment> &RoadEdge::road_edge_segment_list() const {
  return road_edge_segment_list_;
}

bool RoadEdge::get_current_road_edge_segment(RoadEdgeSegment &segment) {
  if (!road_edge_segment_list_.empty()) {
    // find the closest road edge segment wrt ego coordinates
    auto road_edge_iter = road_edge_segment_list_.begin();
    for (auto it = road_edge_segment_list_.begin();
         it != road_edge_segment_list_.end(); it++) {
      if (it->geometry().curve().end_range() < 0.0) continue;
      if (std::abs(it->geometry().curve().start_range()) <
          std::abs(road_edge_iter->geometry().curve().start_range())) {
        road_edge_iter = it;
      }
    }
    if (road_edge_iter->geometry().curve().end_range() > 0.0) {
      segment = *road_edge_iter;
      return true;
    }
  }
  return false;
}

std::vector<RoadEdgeSegment> *RoadEdge::mutable_road_edge_segment_list() {
  return &road_edge_segment_list_;
}

}  // namespace rm