/*
 * @Author: your name
 * @Date: 2021-09-28 15:55:04
 * @LastEditTime: 2022-01-06 09:54:43
 * @LastEditors: Please set LastEditors
 * @Description: In User Settings Edit
 * @FilePath: /RM/roadmodel/modules/common/road/road_info_provider.h
 */

/**
 * @file reference_line_provider.h
 *
 * @brief Declaration of the class ReferenceLineProvider.
 */

#pragma once

#include <arpa/inet.h>
#include <sys/socket.h>
//#include <tf2/LinearMath/Quaternion.h>

#include <future>
#include <list>
#include <memory>
#include <queue>
#include <string>
#include <unordered_set>
#include <vector>

#include "boundary.h"
#include "cem_header.h"
#include "control_way_points.h"
#include "freespace.h"
#include "glog/logging.h"
#include "lane.h"
#include "lane_marker.h"
#include "lane_marker_segment.h"
#include "local_view.h"
#include "observer.h"
#include "obstacle.h"
#include "odometry.h"
#include "road_edge.h"
#include "vehicle_state_provider.h"
#include "wpextend_lane.h"
/**
 * @namespace
 * @brief
 */
namespace rm {
// using carla::Road;
/**
 * @class ReferenceLineProvider
 * @brief The class of ReferenceLineProvider.
 *        It provides smoothed reference line to planning.
 */
class RoadInfoProvider {
 public:
  RoadInfoProvider() = default;
  virtual ~RoadInfoProvider() = default;
  RoadInfoProvider(const std::shared_ptr<Observer>& observer)
      : observer_(observer) {}

  virtual void Update(const LocalView& local_view) {}

  bool GetLanes(std::list<Lane>* lanes);

  bool CreateReferenceLine(std::list<Lane>* lanes);

  LaneMarker GetLaneMarkerFromId(uint32_t id) const {
    auto it = find(lane_markers_.begin(), lane_markers_.end(), id);
    if (it != lane_markers_.end()) {
      return *it;
    } else {
      return LaneMarker();
    }
  }
  LaneMarker* GetLaneMarkerPtrFromId(uint32_t id) {
    auto it = find(lane_markers_.begin(), lane_markers_.end(), id);
    if (it != lane_markers_.end()) {
      return &(*it);
    } else {
      return nullptr;
    }
  }
  // TODO replace wp_extend_lane_list_ with way_points_
  WPExtendLane GetExtendLaneFromId(uint8_t id) {
    auto extend_lane_it =
        find(way_points_.mutable_wp_extend_lane_list()->begin(),
             way_points_.mutable_wp_extend_lane_list()->end(), id);
    if (extend_lane_it != way_points_.mutable_wp_extend_lane_list()->end()) {
      return *extend_lane_it;
    } else {
      return WPExtendLane();
    }
  }

  std::vector<LaneMarker>* mutable_lane_markers() { return &lane_markers_; }

  LaneMarker* add_lane_markers() {
    lane_markers_.emplace_back();
    return &lane_markers_.back();
  }

  std::vector<RoadEdge>* mutable_road_edges() { return &road_edges_; }

  RoadEdge* add_road_edges() {
    road_edges_.emplace_back();
    return &road_edges_.back();
  }

  std::list<Lane>* mutable_lanes() { return &lanes_; }

  Lane* add_lanes() {
    lanes_.emplace_back();
    return &lanes_.back();
  }

  CemHeader* mutable_traffic_road_cem_header() {
    return &traffic_road_cem_header_;
  }

  CemHeader* mutable_objects_cem_header() { return &objects_cem_header_; }

  const CemHeader& objects_cem_header() const { return objects_cem_header_; }

  std::map<uint32_t, Obstacle>* mutable_obstacles() { return &obstacles_; }

  const std::map<uint32_t, Obstacle>& obstacles() const { return obstacles_; }

  ControlWayPoints* mutable_way_points() { return &way_points_; }

  const ControlWayPoints& way_points() const { return way_points_; }

  rm::Odometry* mutable_odometry() { return &odometry_; }

  const rm::Odometry& odometry() const { return odometry_; }

  rm::Freespace* mutable_freespace() { return &freespace_; }

  const rm::Freespace& freespace() const { return freespace_; }

  SensorStatus traffic_road_sensor_status() const {
    return traffic_road_sensor_status_;
  }

  const Lane& special_ego_lane() const { return special_ego_lane_; }

  uint8_t desired_lane_id() { return desired_lane_id_; }

  uint8_t ehr_current_road_type() {return ehr_current_road_type_;}

 protected:
  CemHeader traffic_road_cem_header_;
  CemHeader objects_cem_header_;
  RoadType road_type_;
  SensorStatus traffic_road_sensor_status_;
  // std::list<WPExtendLane> wp_extend_lane_list_;
  std::list<Lane> lanes_;                 //存储预处理后的车道信息
  Lane special_ego_lane_;
  std::vector<LaneMarker> lane_markers_;  //存储预处理后的车道线segment信息
  uint8_t desired_lane_id_;
  std::map<uint32_t, Obstacle> obstacles_;  //存储预处理后的障碍物
  std::shared_ptr<Observer> observer_;
  ControlWayPoints way_points_;
  std::vector<RoadEdge> road_edges_;  //存储预处理后的道路边缘
  rm::Freespace freespace_;
  rm::Odometry odometry_;
  uint8_t ehr_current_road_type_;
};

}  // namespace rm
