/*
 * @Author: your name
 * @Date: 2021-11-16 11:26:44
 * @LastEditTime: 2021-12-07 10:43:59
 * @LastEditors: Please set LastEditors
 * @Description: In User Settings Edit
 * @FilePath: /roadmodel/src/CtApRoadModel/source/common/math/sl_point.h
 */
#pragma once
#include "lookup_table.h"
#include <stdint.h>
#include <vector>
namespace rm {

class SLPoint {
public:
  SLPoint() = default;
  SLPoint(float s, float ds, float dds, float l, float dl, float ddl)
      : s_(s), ds_(ds), dds_(dds), l_(l), dl_(dl), ddl_(ddl) {
    dds_valid_time_ = dds_time_table.Evaluate(std::abs(dds));
    dl_valid_time_ = dl_time_table.Evaluate(std::abs(dl));
  };

  void set_s(float s);
  void set_ds(float ds);
  void set_dds(float dds);
  void set_l(float l);
  void set_dl(float dl);
  void set_ddl(float ddl);
  void set_dds_valid_time(float dds_valid_time);
  void set_dl_valid_time(float dl_valid_time);
  void set_ddl_valid_time(float ddl_valid_time);

  float s() const;
  float ds() const;
  float dds() const;
  float l() const;
  float dl() const;
  float ddl() const;
  float dds_valid_time() const;
  float dl_valid_time() const;
  float ddl_valid_time() const;

  void Sat_SLValue();
  void Sat_Rear_SLValue();
  void Saturation(float &value, const std::pair<float, float> &range);

  /**
   * @brief: 根据table计算validtime
   * @param {*}
   * @return {*}
   */
  void ComputeValidTime();
  /**
   * @brief: 友元函数重载<<
   * @param {*}
   * @return {*}
   */
  friend std::ostream &operator<<(std::ostream &os, const SLPoint &slpoint);

private:
  static const OneDimLookupTable dds_time_table;
  static const OneDimLookupTable dl_time_table;
  static const OneDimLookupTable ddl_time_table;

  float s_;
  float ds_;
  float dds_;
  float dds_valid_time_;
  float l_;
  float dl_;
  float dl_valid_time_;
  float ddl_;
  float ddl_valid_time_;
  static const std::pair<float, float> s_range;
  static const std::pair<float, float> ds_range;
  static const std::pair<float, float> dds_range;
  static const std::pair<float, float> l_range;
  static const std::pair<float, float> dl_range;
  static const std::pair<float, float> ddl_range;
};

} // namespace rm