#pragma once

#include <stdint.h>

#include <array>
#include <iostream>

namespace rm {

struct SpeedLimitInfo {
  uint8_t speed_limit;
  float offset;
  uint8_t id;
  bool valid;
  SpeedLimitInfo() : speed_limit(130U), offset(1000.0), id(0U), valid(false){};
  SpeedLimitInfo(uint8_t limit, float off, uint8_t idx)
      : speed_limit(limit), offset(off), id(idx){};
  SpeedLimitInfo(uint8_t limit, float off, uint8_t idx, bool val)
      : speed_limit(limit), offset(off), id(idx), valid(val){};
};

class SpeedLimit {
 public:
  void set_default_speed_limit() {
    speed_limit_ = {SpeedLimitInfo(), SpeedLimitInfo()};
    prv_speed_limit_ = speed_limit_;
  };
  void set_speed_limit(uint8_t limit_0, float offset_0, uint8_t limit_1,
                       float offset_1) {
    if (limit_0 == 0) {
      speed_limit_.at(0) = SpeedLimitInfo();
      speed_limit_.at(1) = SpeedLimitInfo();
      first_run_ = true;
    } else {
      if (first_run_) {
        speed_limit_.at(0) = SpeedLimitInfo(limit_0, offset_0, 1U, true);
        prv_speed_limit_.at(0) = speed_limit_.at(0);
        if (limit_1 != 130) {
          speed_limit_.at(1) = SpeedLimitInfo(limit_1, offset_1, 2U, true);
        } else {
          speed_limit_.at(1) = SpeedLimitInfo();
        }
        prv_speed_limit_.at(1) = speed_limit_.at(1);
        first_run_ = false;
      } else {
        if (offset_0 < 5.0) {
          speed_limit_.at(0).speed_limit = prv_speed_limit_.at(0).speed_limit;
        } else {
          speed_limit_.at(0).speed_limit = limit_0;
        }
        speed_limit_.at(0).offset = offset_0;
        speed_limit_.at(0).valid = true;
        if (limit_1 != 130) {
          speed_limit_.at(1).offset = offset_1;
          speed_limit_.at(1).speed_limit = limit_1;
          speed_limit_.at(1).id = (speed_limit_.at(0).id == 2U) ? 1U : 2U;
          speed_limit_.at(1).valid = true;
        } else {
          speed_limit_.at(1) = SpeedLimitInfo();
        }
        dis_offset =
            (!prv_speed_limit_.at(1).valid && !speed_limit_.at(1).valid)
                ? 100.0
                : prv_speed_limit_.at(1).offset - speed_limit_.at(0).offset;
        if (prv_speed_limit_.at(0).offset < 10.0 && dis_offset < 10.0) {
          // position shift happenes, inherit id
          speed_limit_.at(0).id = prv_speed_limit_.at(1).id;
          speed_limit_.at(1).id = prv_speed_limit_.at(0).id;
        } else {
        }
        prv_speed_limit_.at(0) = speed_limit_.at(0);
        prv_speed_limit_.at(1) = speed_limit_.at(1);
      }
    }
  };

  std::array<SpeedLimitInfo, 2> speed_limit() const { return speed_limit_; };

  void reset() { first_run_ = true; };

 private:
  bool first_run_ = true;
  float dis_offset;
  std::array<SpeedLimitInfo, 2> prv_speed_limit_;
  std::array<SpeedLimitInfo, 2> speed_limit_;
};

}  // namespace rm