#include "straight_driving_detector.h"

namespace rm {

void StraightDrivingDetector::Reset() {
  steering_wheel_deb_.ResetDebounce();
  steeing_wheel_delay_.ResetDelay();
  yaw_rate_deb_.ResetDebounce();
  yaw_rate_delay_.ResetDelay();
}

bool StraightDrivingDetector::RunDetector() {
  UpdateInput();
  bool str_whl_deb_result =
      steering_wheel_deb_.RunDebouceLessTure(abs_steering_wheel_angle_);
  bool str_whl_final_result = steeing_wheel_delay_.RunDelay(str_whl_deb_result);
  bool yaw_rate_deb_result = yaw_rate_deb_.RunDebouceLessTure(abs_yaw_rate_);
  bool yaw_rate_final_result = yaw_rate_delay_.RunDelay(str_whl_deb_result);
  // DLOG(INFO) << PRINT(str_whl_deb_result);
  // DLOG(INFO) << PRINT(str_whl_final_result);
  // DLOG(INFO) << PRINT(yaw_rate_deb_result);
  // DLOG(INFO) << PRINT(yaw_rate_final_result);
  return yaw_rate_final_result && str_whl_final_result;
}

void StraightDrivingDetector::UpdateInput() {
  abs_steering_wheel_angle_ =
      std::abs(VehicleStateProvider::Instance()->steering_wheel_angle());
  abs_yaw_rate_ = std::abs(VehicleStateProvider::Instance()->yaw_rate());
}

} // namespace rm