/*
 * @Author: your name
 * @Date: 2021-11-18 13:54:06
 * @LastEditTime: 2022-03-09 18:37:53
 * @LastEditors: Please set LastEditors
 * @Description: 打开koroFileHeader查看配置 进行设置:
 * https://github.com/OBKoro1/koro1FileHeader/wiki/%E9%85%8D%E7%BD%AEeh
 * @FilePath:
 * /EHR/home/igs/roadmodel/src/CtApRoadModel/source/road/vehicle_state_provider.h
 */

/**
 * @file vehicle_state.h
 *
 * @brief Declaration of the class VehicleStateProvider.
 */
#pragma once

#include "box2d.h"
#include "enum_list.h"
#include "global_parameters.h"
#include "local_view.h"
#include "low_pass_filter.h"
#include "macros.h"
#include "status.h"
#include "vec2d.h"
#include <memory>
#include <string>
#include "dpc/impl_type_ved4dpc.h"
using rm::common::Status;
/**
 * @namespace rm
 * @brief rm::common
 */
namespace rm {

/**
 * @class VehicleStateProvider
 * @brief The class of vehicle state.
 *        It includes basic information and computation
 *        about the state of the vehicle.
 */
class VehicleStateProvider {
public:
    /**
   * @brief Constructor by information of localization and chassis.
   * @param localization Localization information of the vehicle.
   * @param chassis Chassis information of the vehicle.
   */
  Status Update(const dpc::VED4DPC &vehicle_state/*,
                const vehicle_msgs::msg::ApSas10msPdu00 &sas_info,
                const vehicle_msgs::msg::ApEps010msPdu00 &eps_info,
                const mpc_drv_msgs::msg::MsgSM2TRPC &sm_info*/);
  /**
   * @brief: 存取函数
   * @param {float} lon_velocity
   * @return {*}
   */
  void set_lon_velocity(const float lon_velocity) {
    lon_velocity_ = lon_velocity;
  }
  float lon_velocity() const { return lon_velocity_; }
  /**
   * @brief: 存取函数
   * @param {float} lon_acceleration
   * @return {*}
   */
  void set_lon_acceleration(const float lon_acceleration) {
    lon_acceleration_ = lon_acceleration;
  }
  float lon_acceleration() const { return lon_acceleration_; }
  /**
   * @brief: 存取函数
   * @param {float} lat_acceleration
   * @return {*}
   */
  void set_lat_acceleration(const float lat_acceleration) {
    lat_acceleration_ = lat_acceleration;
  }
  float lat_acceleration() const { return lat_acceleration_; }
  /**
   * @brief: 存取函数
   * @param {float} lon_slpoe
   * @return {*}
   */
  void set_lon_slope(const float lon_slope) { lon_slope_ = lon_slope; }
  float lon_slope() const { return lon_slope_; }
  /**
   * @brief: 存取函数
   * @param {float} yaw_rate
   * @return {*}
   */
  void set_yaw_rate(const float yaw_rate) { yaw_rate_ = yaw_rate; }
  float yaw_rate() const { return yaw_rate_; }
  /**
   * @brief: 存取函数
   * @param {float} moving_state
   * @return {*}
   */
  void set_moving_status(const VehicleMovingStatus moving_status) {
    moving_status_ = moving_status;
  }
  VehicleMovingStatus moving_status() const { return moving_status_; }

  float steering_wheel_angle() const { return steering_wheel_angle_; }

  float column_torque() const { return column_torque_; }

  float driving_curvature() const { return driving_curvature_; }

  void CalDrivingCurv();

  void set_tja_ica_status(const SMTjaIcaStatus status) {
    tja_ica_status_ = status;
  }

  SMTjaIcaStatus tja_ica_status() { return tja_ica_status_; }

private:
  float lon_velocity_;         // Longitudinal velocity estimation (m/s)
  float lon_acceleration_;     // Longitudinal acceleration estimation (m/s2)
  float lat_acceleration_;     // Acceleration acceleration estimation (m/s2)
  float lon_slope_;            // Current slope (deg)
  float yaw_rate_;             // Yaw rate estimation (deg/s)
  float steering_wheel_angle_; // steering wheel angle in deg
  bool steering_wheel_angle_validity_;
  float column_torque_;
  bool column_torque_validity_;
  VehicleMovingStatus moving_status_; // 0:vehicle standstill. 1:moving forward.
                                      // 2:moving backward. 3:unknown
  uint16_t vse_cyclecounter_;
  float driving_curvature_;
  common::math::LowPassFilter yaw_rate_lpf_{0.6};
  SMTjaIcaStatus tja_ica_status_;
DECLARE_SINGLETON(VehicleStateProvider)
};

} // namespace rm
