/*
 * @Author: your name
 * @Date: 2021-09-18 10:42:18
 * @LastEditTime: 2022-01-20 14:11:35
 * @LastEditors: Please set LastEditors
 * @Description: In User Settings Edit
 * @FilePath:
 * /NOP_EHR/media/igs/xnm/ros/RM/roadmodel/modules/task/deciders/lane_change_decider/lane_change_decider.cc
 */
#include "boundary_builder.h"

#include <iomanip>
#include <iostream>
#include <limits>
#include <memory>

#include "glog/logging.h"
#include "macros.h"
using namespace std;

namespace rm {

using rm::common::ErrorCode;
using rm::common::Status;

BoundaryBuilder::BoundaryBuilder(
    const std::shared_ptr<Observer> &observer,
    const std::shared_ptr<DependencyInjector> &injector)
    : Builder(observer, injector) {
  // CHECK(config_.has_boundary_builder_config()); // config_在task类中
  // 相当于把config赋值给task中的config_
}

Status BoundaryBuilder::Process(Frame *frame,
                                LaneInfo *const current_lane_info) {
  // DLOG(INFO)
  // << "********************boundary builder start********************";
  CHECK_NOTNULL(frame); // Sanity checks.
  CHECK_NOTNULL(current_lane_info);
  // const auto &boundary_builder_config =
  //     config_.boundary_builder_config(); //获取配置量
  // cout<<observer_->getTaskConfig()->boundary_builder_config->int_param<<endl;
  //获取当前车道
  Lane current_lane = current_lane_info->lane();
  Lane *current_mutable_lane = current_lane_info->mutable_lane();
  rm::LaneMarker *left_lane_marker_ptr{nullptr}, *right_lane_marker_ptr{nullptr};
  // DLOG(INFO) << PRINT(current_lane);

  //将该车道的左侧车道线放入 left_lane_marker_segment
  std::vector<LaneMarker>
      left_lane_markers; //根据lane的左侧id列表筛选出的lane markersegment 列表
  std::vector<LaneMarker> right_lane_markers;
  bool lane_exist_last_cycle;

  auto lane_marker_id_info_iter =
      std::find_if(injector_->mutable_lane_marker_id_info_last_cycle()->begin(),
                   injector_->mutable_lane_marker_id_info_last_cycle()->end(),
                   [&current_lane](LaneMarkerIDInfo id_info) {
                     return current_lane.lane_id() == id_info.lane_id;
                   });
  if (lane_marker_id_info_iter ==
      injector_->mutable_lane_marker_id_info_last_cycle()->end()) {
    lane_exist_last_cycle = false;
    injector_->mutable_lane_marker_id_info_last_cycle()->emplace_back(
        current_lane.lane_id(), 0U, 0U, true);
  } else {
    lane_marker_id_info_iter->updated = true;
    lane_exist_last_cycle = true;
  }
  for_each(
      current_lane.mutable_left_lane_marker_id()->begin(),
      current_lane.mutable_left_lane_marker_id()->end(),
      [&left_lane_markers, &left_lane_marker_ptr, frame, current_mutable_lane,
       lane_marker_id_info_iter, lane_exist_last_cycle,
       this](uint32_t lane_marker_id) {
        left_lane_marker_ptr =
            frame->road_info_provider()->GetLaneMarkerPtrFromId(lane_marker_id);
        rm::LaneMarker left_lane_marker = (left_lane_marker_ptr == nullptr)
                                              ? LaneMarker()
                                              : *left_lane_marker_ptr;
        left_lane_markers.emplace_back(left_lane_marker);
        if (lane_exist_last_cycle) {
          if (lane_marker_id_info_iter->left_lane_marker_id == lane_marker_id) {
            current_mutable_lane->set_is_left_lane_marker_changed(false);
          } else {
            lane_marker_id_info_iter->left_lane_marker_id = lane_marker_id;
            current_mutable_lane->set_is_left_lane_marker_changed(true);
          }
        } else {
          injector_->mutable_lane_marker_id_info_last_cycle()
              ->back()
              .left_lane_marker_id = lane_marker_id;
        }
      });
  // DLOG(INFO) << PRINT(left_lane_markers.size()); // TJA阶段暂为1

  for_each(
      current_lane.mutable_right_lane_marker_id()->begin(),
      current_lane.mutable_right_lane_marker_id()->end(),
      [&right_lane_markers, &right_lane_marker_ptr, frame, current_mutable_lane,
       lane_marker_id_info_iter, lane_exist_last_cycle,
       this](uint32_t lane_marker_id) {
        right_lane_marker_ptr =
            frame->road_info_provider()->GetLaneMarkerPtrFromId(lane_marker_id);
        rm::LaneMarker right_lane_marker = (right_lane_marker_ptr == nullptr)
                                               ? LaneMarker()
                                               : *right_lane_marker_ptr;
        right_lane_markers.emplace_back(right_lane_marker);
        if (lane_exist_last_cycle) {
          if (lane_marker_id_info_iter->right_lane_marker_id ==
              lane_marker_id) {
            current_mutable_lane->set_is_right_lane_marker_changed(false);
          } else {
            lane_marker_id_info_iter->right_lane_marker_id = lane_marker_id;
            current_mutable_lane->set_is_right_lane_marker_changed(true);
          }
        } else {
          injector_->mutable_lane_marker_id_info_last_cycle()
              ->back()
              .right_lane_marker_id = lane_marker_id;
        }
      });
  // DLOG(INFO) << PRINT(right_lane_markers.size());

  Boundary left_boundary, right_boundary;
  left_boundary.ComputeBoundaryFromLaneMarkers(left_lane_markers);
  right_boundary.ComputeBoundaryFromLaneMarkers(right_lane_markers);
  current_lane_info->set_boundary(
      std::make_pair(left_boundary, right_boundary));

  Boundary left_back_boundary, right_back_boundary;
  left_back_boundary.ComputeBackBoundaryFromLaneMarkers(left_lane_markers);
  right_back_boundary.ComputeBackBoundaryFromLaneMarkers(right_lane_markers);

  current_lane_info->set_back_boundary(
      std::make_pair(left_back_boundary, right_back_boundary));

  // change lane marker type if there is road edge nearby
  if (current_lane.lane_position() == rm::LanePosition::EGO_LANE) {
    if (current_lane.lane_id() != injector_->ego_lane_id_last_cycle()) {
      // reset the delay if lane change occured
      le_lnmrk_ovlapped_by_rdedge_.ResetDelay();
      ri_lnmrk_ovrlapped_by_rdedge_.ResetDelay();
    }
    if (left_lane_marker_ptr) {
      RoadEdgeLaneMrkTypeOverwrite(
          frame->FindRoadEdgePtrsFromRoadEdgePosition(RoadEdgePostion::LEFT),
          left_lane_marker_ptr, true);
    } else {
      // reset delay if lane marker does not exist
      le_lnmrk_ovlapped_by_rdedge_.ResetDelay();
    }
    if (right_lane_marker_ptr) {
      RoadEdgeLaneMrkTypeOverwrite(
          frame->FindRoadEdgePtrsFromRoadEdgePosition(RoadEdgePostion::RIGHT),
          right_lane_marker_ptr, false);
    } else {
      // reset delay if lane marker does not exist
      ri_lnmrk_ovrlapped_by_rdedge_.ResetDelay();
    }
  }

  // DLOG(INFO) << "********************boundary builder end********************";
  return Status::OK();
}

void BoundaryBuilder::RoadEdgeLaneMrkTypeOverwrite(
    std::vector<RoadEdge *> road_edge_ptrs, rm::LaneMarker *lane_marker,
    bool is_left) {
  std::vector<double> overlap_ranges;
  double extend_end_range;
  // double overlap_range_x{-1.0};
  bool overlap_flag{false};
  for (auto road_edge_ptr : road_edge_ptrs) {
    // road edges loop
    auto road_edge_segments = road_edge_ptr->mutable_road_edge_segment_list();
    std::vector<Vec2d> road_edge_sampling_points;
    for (auto road_seg_it = road_edge_segments->begin();
         road_seg_it != road_edge_segments->end(); ++road_seg_it) {
      // road edge segments loop
      rm::PolynomLine road_edge_curve = road_seg_it->geometry().curve();
      if (road_edge_curve.end_range() <= 0.0)
        continue;
      for (double x = std::max(0.0, road_edge_curve.start_range());
           x < road_edge_curve.end_range(); ++x) {
        // sampling the road edge as points
        if (x > 100.0)
          break;
        // disregards the points that are 100m away from ego veh
        road_edge_sampling_points.emplace_back(x,
                                               road_edge_curve.Evaluate(0, x));
      }
    }

    for (auto edge_point_iter = road_edge_sampling_points.begin();
         edge_point_iter != road_edge_sampling_points.end();
         ++edge_point_iter) {
      // find the fisrt road edge sampling point that is in dangerous zone, from
      // near end to far end
      auto lane_mrk_seg_it = std::find_if(
          lane_marker->mutable_front_lane_marker_segment()->begin(),
          lane_marker->mutable_front_lane_marker_segment()->end(),
          [&edge_point_iter](rm::LaneMarkerSegment &ln_mkr_seg) {
            return edge_point_iter->x() > ln_mkr_seg.mutable_geometry()
                                              ->mutable_curve()
                                              ->start_range() &&
                   edge_point_iter->x() <= ln_mkr_seg.mutable_geometry()
                                               ->mutable_curve()
                                               ->end_range();
          });
      if (lane_mrk_seg_it !=
          lane_marker->mutable_front_lane_marker_segment()->end()) {
        double delta_y =
            edge_point_iter->y() -
            lane_mrk_seg_it->mutable_geometry()->mutable_curve()->Evaluate(
                0, edge_point_iter->x());
        if (is_left) {
          if (delta_y < GetRMConfig().boundary_rdedg_lnmrk_dis_threshold) {
            // overlap_range_x = edge_point_iter->x();
            // left_overlap_range_x_ = overlap_range_x;
            // store the historical range
            overlap_ranges.emplace_back(edge_point_iter->x());
            break;
          } else {
            // do nothing
          }
        } else {
          if (delta_y > -GetRMConfig().boundary_rdedg_lnmrk_dis_threshold) {
            // overlap_range_x = edge_point_iter->x();
            // right_overlap_range_x_ = overlap_range_x;
            // store the historical range
            overlap_ranges.emplace_back(edge_point_iter->x());
            break;
          } else {
            // do nothing
          }
        }
      } else {
        // do nothing
      }
    }
  }

  if (is_left) {
    if (overlap_ranges.empty()) {
      // overlap_range.empty() means condition was not fulfilled, the
      // overlap_range has been updated
      overlap_flag = le_lnmrk_ovlapped_by_rdedge_.RunDelay(false);
    } else {
      left_overlap_range_x_ =
          *(std::min_element(overlap_ranges.begin(), overlap_ranges.end()));
      overlap_flag = le_lnmrk_ovlapped_by_rdedge_.RunDelay(true);
    }
    extend_end_range = std::max(left_overlap_range_x_ - 10.0, 0.0);
  } else {
    if (overlap_ranges.empty()) {
      overlap_flag = ri_lnmrk_ovrlapped_by_rdedge_.RunDelay(false);
    } else {
      right_overlap_range_x_ =
          *(std::min_element(overlap_ranges.begin(), overlap_ranges.end()));
      overlap_flag = ri_lnmrk_ovrlapped_by_rdedge_.RunDelay(true);
    }
    extend_end_range = std::max(right_overlap_range_x_ - 10.0, 0.0);
  }

  // if overlap exists, set the line type of corresponding att seg to solid
  std::vector<rm::LaneMarkerSegmentAttribute> *lane_marker_seg_attribute_list =
      lane_marker->mutable_lane_marker_segment_attribute();
  // if the lane marker contain virtual/comp lane segment attribute, then skip
  // lane type overwrite for this lane marker
  auto hdmap_virtual_seg_it =
      std::find_if(lane_marker_seg_attribute_list->begin(),
                   lane_marker_seg_attribute_list->end(),
                   [](rm::LaneMarkerSegmentAttribute &seg_att) {
                     return seg_att.line_type() == rm::LineType::COMP ||
                            seg_att.line_type() == rm::LineType::VIRTUAL;
                   });
  // if (hdmap_virtual_seg_it != lane_marker_seg_attribute_list->end())
  //   return;

  if (overlap_flag) {
    auto attribute_seg_end_iter = std::find_if(
        lane_marker_seg_attribute_list->begin(),
        lane_marker_seg_attribute_list->end(),
        [&extend_end_range](rm::LaneMarkerSegmentAttribute &seg_att) {
          return extend_end_range < seg_att.end_offset() &&
                 extend_end_range >= seg_att.start_offset();
        });
    if (attribute_seg_end_iter != lane_marker_seg_attribute_list->end()) {
      if (attribute_seg_end_iter->line_type() == rm::LineType::COMP ||
          attribute_seg_end_iter->line_type() == rm::LineType::VIRTUAL)
        return;
      double original_start_offset = attribute_seg_end_iter->start_offset();
      attribute_seg_end_iter->set_start_offset(extend_end_range);
      auto insert_pos_it = lane_marker_seg_attribute_list->insert(
          attribute_seg_end_iter,
          LaneMarkerSegmentAttribute(original_start_offset, extend_end_range,
                                     attribute_seg_end_iter->line_type(),
                                     attribute_seg_end_iter->line_color()));
      std::for_each(insert_pos_it + 1, lane_marker_seg_attribute_list->end(),
                    [](rm::LaneMarkerSegmentAttribute &att_seg) {
                      att_seg.set_line_type(1U);
                    });
    } else if (attribute_seg_end_iter ==
                   lane_marker_seg_attribute_list->end() &&
               !lane_marker_seg_attribute_list->empty() &&
               extend_end_range >
                   (lane_marker_seg_attribute_list->end() - 1)->end_offset()) {
      if (hdmap_virtual_seg_it != lane_marker_seg_attribute_list->end())
        return;
      std::for_each(lane_marker_seg_attribute_list->begin(),
                    lane_marker_seg_attribute_list->end(),
                    [](rm::LaneMarkerSegmentAttribute &att_seg) {
                      att_seg.set_line_type(1U);
                    });
    } else {
      // do nothing
    }
  }
}

rm::Delay<bool> BoundaryBuilder::le_lnmrk_ovlapped_by_rdedge_{0.4, 1.0, false};

rm::Delay<bool> BoundaryBuilder::ri_lnmrk_ovrlapped_by_rdedge_{0.4, 1.0, false};

void BoundaryBuilder::ResetBoundaryBuilder() {
  le_lnmrk_ovlapped_by_rdedge_.ResetDelay();
  ri_lnmrk_ovrlapped_by_rdedge_.ResetDelay();
}

} // namespace rm
