/*
 * @Author: your name
 * @Date: 2021-09-18 10:42:18
 * @LastEditTime: 2022-01-25 14:23:13
 * @LastEditors: Please set LastEditors
 * @Description: In User Settings Edit
 * @FilePath: /NOP_EHR/media/igs/xnm/ros/RM/roadmodel/modules/task/deciders/lane_change_decider/lane_change_decider.cc
 */
#include "corridor_builder.h"

#include <limits>
#include <memory>
#include <iostream>
#include <iomanip>
#include "corridor.h"
#include "status.h"
#include "macros.h"
#include "glog/logging.h"
#include "rm_config.h"
using namespace std;

namespace rm {
 

 using rm::common::ErrorCode;
 using rm::common::Status;

CorridorBuilder::CorridorBuilder(
    const std::shared_ptr<Observer>& observer,
    const std::shared_ptr<DependencyInjector>& injector)
    : Builder(observer, injector) {
  // CHECK(config_.has_corridor_builder_config());//config_在task类中 相当于把config赋值给task中的config_
}

Corridor CorridorBuilder::GenerateCorridor(BoundarySegement left_seg, BoundarySegement right_seg, bool is_front_boundary){
  LineGeometry left_geometry  = left_seg.geometry; //获取左右boundary的几何
  LineGeometry right_geometry = right_seg.geometry;
  Corridor corridor;
  if (!left_geometry.valid() || !right_geometry.valid()) return corridor;
  // if (left_geometry.measurement_state() == MeasurementState::VIRTUAL && right_geometry.measurement_state() == MeasurementState::VIRTUAL) return corridor;
  // moved the left boundary longitudinally to the min start range, same coordinate
  double target_range = std::min(left_geometry.curve().start_range(), right_geometry.curve().start_range());
  // DLOG(INFO) << PRINT(left_geometry);
  // DLOG(INFO) << PRINT(right_geometry);
  LineGeometry left_geometry_moved = left_geometry.MoveCurvedLongitudinal(target_range);
  LineGeometry right_geometry_moved = right_geometry.MoveCurvedLongitudinal(target_range);
  // DLOG(INFO) << PRINT(left_geometry_moved);
  // DLOG(INFO) << PRINT(right_geometry_moved);

  double left_dy = left_geometry_moved.curve().Evaluate(0,0);
  double right_dy = right_geometry_moved.curve().Evaluate(0,0);
  double shift = abs((left_dy - right_dy)/2.0); //计算两条线的中间起点
  // DLOG(INFO)<<PRINT(left_dy);
  // DLOG(INFO)<<PRINT(right_dy);
  // DLOG(INFO)<<PRINT(shift);

  //2.将平移后的两条boundary离散化 提取std 将离散化后的点付权重
  std::array<std::vector<std::pair<ReferencePoint,double>>, 2> reference_points;  //用于收集两条线的离散点和权重
  Status status;
  // if (left_geometry.valid()){
  LineGeometry center_left_geometry = left_geometry_moved.MoveCurved(-shift);
  // DLOG(INFO)<<PRINT(center_left_geometry);
  // status = center_left_geometry.GetReferencePiontWithWeight(&reference_points, min_length, 1.0);
  status = center_left_geometry.GetReferencePiontWithWeight(&reference_points.at(0), 1.0);
  // DLOG(INFO)<<PRINT(reference_points.at(0).size())<<endl;
  // }
  // if (right_geometry.valid()){
  LineGeometry center_right_geometry = right_geometry_moved.MoveCurved(shift);
  // DLOG(INFO)<<PRINT(center_right_geometry);
  // status = center_right_geometry.GetReferencePiontWithWeight(&reference_points, min_length, 1.0);
  status = center_right_geometry.GetReferencePiontWithWeight(&reference_points.at(1), 1.0);
  // DLOG(INFO)<<PRINT(reference_points.at(1).size())<<endl;
  // }
  //4.最小二乘法你拟合中心线
  if (!corridor.ComputeCenterLine(reference_points))
    return corridor;
  // move the coordinate back to the origin
  corridor.MoveCenterLineLongitudinal(-target_range);
  //5.根据中心线计算宽度
  
  // PolynomLine corridor_left_curve = left_boundary.ComputeShiftLine(true);//boundary偏移后的corriodr边界
  // PolynomLine corridor_right_curve = right_boundary.ComputeShiftLine(false);
  double shift_distance = 0.1;
  LineGeometry corridor_left_geometry = left_geometry_moved.MoveCurved(-1.0 * shift_distance);//boundary偏移后的corriodr边界
  LineGeometry corridor_right_geometry = right_geometry_moved.MoveCurved(shift_distance);

  corridor.ComputeDrivableWidth(corridor_left_geometry,corridor_right_geometry);
  corridor.ComputeCenterLineArcLength(is_front_boundary);
  corridor.set_feature_tag(CorridorFeatureTag::LINEFOLLOW);
  corridor.set_is_front_corridor(is_front_boundary);
  
  return corridor;
}

Status CorridorBuilder::Process(
    Frame* frame, LaneInfo* const current_lane_info) {
  // DLOG(INFO)<<"********************corridor builder start********************";
  // Sanity checks.
  CHECK_NOTNULL(frame);
  CHECK_NOTNULL(current_lane_info);
  // const auto& coridor_builder_config = config_.corridor_builder_config();//获取配置量
  uint8_t ego_lane_id = frame->FindLaneIdFromLanePosition(LanePosition::EGO_LANE);
  
  // DLOG(INFO)<<"***************front corridor***************";
  bool is_front_boundary = true;
  Boundary left_boundary = current_lane_info->boundary().first;
  Boundary right_boundary = current_lane_info->boundary().second;
  //TODO both boundaries are invalid
  std::vector<Corridor>* corridor_list = current_lane_info->mutable_corridor();
  uint8_t current_lane_id = current_lane_info->lane().lane_id();
  uint8_t size_of_boundary_list = std::min(left_boundary.mutable_boundary_segment_list()->size(), right_boundary.mutable_boundary_segment_list()->size());
  float accumulated_start_offset = 0.0F;
  for (uint8_t i = 0U; i < size_of_boundary_list; i++){
    Corridor corridor;
    BoundarySegement left_boundary_seg = left_boundary.mutable_boundary_segment_list()->at(i);
    BoundarySegement right_boundary_seg = right_boundary.mutable_boundary_segment_list()->at(i);
    corridor = GenerateCorridor(left_boundary_seg, right_boundary_seg, is_front_boundary);
    corridor.set_corridor_id(current_lane_id, current_lane_info->lane().is_lane_markers_changed(), injector_);
    // DLOG(INFO) << PRINT(+corridor.corridor_id());
    corridor.set_start_offset(accumulated_start_offset);
    accumulated_start_offset = accumulated_start_offset + corridor.arc_length();
    // DLOG(INFO)<<PRINT(corridor);
    if(corridor.feature_tag() != CorridorFeatureTag::NOTAVLBL){
        if (i == 0U) {
          // if the corridor is first one, set is_first_seg flag
          corridor.set_is_first_seg();
          if (ego_lane_id == current_lane_id){
            //if the ego corridor is first one, then store the corridor center lane info
            // IntersectionVirtualLaneGenerator::Instance()->SetInitialState(corridor);
            
            // break if the drivable width of corridor is greater than 5.5m
            if (corridor.drivable_width() > GetRMConfig().corridor_generation_width_threshold) break;
          }
        }
        corridor_list->push_back(corridor);
    }
    else{
        // DLOG(INFO) << "*************front corridor is not available*****************";
        break;
    }
  }
  // DLOG(INFO)<<"***************back corridor***************";
  is_front_boundary = false;
  Boundary left_back_boundary = current_lane_info->back_boundary().first;
  Boundary right_back_boundary = current_lane_info->back_boundary().second;
  std::vector<Corridor>* back_corridor_list = current_lane_info->mutable_back_corridor();
  uint8_t size_of_back_boundary_list = std::min(left_back_boundary.mutable_boundary_segment_list()->size(), right_back_boundary.mutable_boundary_segment_list()->size());
  accumulated_start_offset = 0.0F;
  for (uint8_t i = 0U; i < size_of_back_boundary_list; i++){
    Corridor back_corridor;
    BoundarySegement left_back_boundary_seg = left_back_boundary.mutable_boundary_segment_list()->at(i);
    BoundarySegement right_back_boundary_seg = right_back_boundary.mutable_boundary_segment_list()->at(i);
    back_corridor = GenerateCorridor(left_back_boundary_seg, right_back_boundary_seg, is_front_boundary);
    back_corridor.set_corridor_id(current_lane_id, current_lane_info->lane().is_lane_markers_changed(), injector_);
    // DLOG(INFO) << PRINT(+back_corridor.corridor_id());
    back_corridor.set_start_offset(accumulated_start_offset);
    accumulated_start_offset = accumulated_start_offset + back_corridor.arc_length();
    // DLOG(INFO)<<PRINT(back_corridor);
    if(back_corridor.feature_tag() != CorridorFeatureTag::NOTAVLBL){
      if (i == 0U) {
          // if the corridor is first one, set is_first_seg flag
          back_corridor.set_is_first_seg();
      }
        back_corridor_list->push_back(back_corridor);
    }
    else{
        // DLOG(INFO) << "*************back corridor is not available*****************";
        break;
    }
  }

  // DLOG(INFO)<<"********************corridor builder end********************";
  return Status::OK();
}

 
}  // namespace rm
