/*
 * @Date: 2021-11-09 13:05:00
 * @LastEditors: adc
 * @LastEditTime: 2021-11-10 09:49:23
 * @FilePath: /Cartesian_to_Frenet_corridor_v3/src/frenet_converter.h
 */
#pragma once

#include <iostream>
#include <vector>
#include <array>
#include <cmath>

#include "reference_point.h"
#include "cartesian_frenet_conversion.h"
#include "polynom_line.h"
#include "obstacle.h"
#include "object_location.h"
#define DERI_STEP 0.1 // Minor step for derivative calculation
#define EPSILON 0.01  // critera condition

#define DebugP(x) std::cout << "line" << __LINE__ << "  " << #x << "=" << x << std::endl;

namespace rm{
    using common::math::ReferencePoint;
    class FrenetConverter
    {
    public:
        FrenetConverter() = default;
        FrenetConverter(PolynomLine ref_line);
        FrenetConverter(const Corridor * const corridor);
        ~FrenetConverter() = default;
        // Input XY in cartesian, calculate coresponding frenet point
        SLPoint CalculateFrenetPoint(const Obstacle &object);
        // Find reference point based on XY in reference line
        ReferencePoint FindRefPointS(double x0, double y0);
        double rx() {return rx_;};
        double ry() {return ry_;};

private:
    // 3rd order poly function
    double y_fun(double x);
    // the distance function is the one to be minimized
    // double distance_to_curve(double x0, double y0, double x);
    // Derived function for curve integration
    double ds_fun(double x);

    // curve integration
    double CurveIntegration(double rx);
    // Simpson rule
    double IntegrateBySimpson(const double a, const double b);
    double IntegrateBySimpson(const double a, const double b,const double m);
    // Alternative integration method
    double IntegrateByFiniteVolume(const double a, const double b);

private: // Data
    // coefficient for poly
    // std::vector<double> Coe = {0, 0, 0, 0};
    PolynomLine ref_line_;
    double start_offset_ = 0.0;
    bool is_front_ref_line_ = false;

    // data for ref point
    double rs_;
    double rx_;
    double ry_;



private: // Debug
    void DebugPlot(const double &x0, const double &y0, const double &rx_move);
};
}