/*
 * @Author: your name
 * @Date: 2021-09-18 10:42:18
 * @LastEditTime: 2022-01-25 11:20:52
 * @LastEditors: Please set LastEditors
 * @Description: In User Settings Edit
 * @FilePath:
 * /NOP_EHR/media/igs/xnm/ros/RM/roadmodel/modules/task/deciders/lane_change_decider/lane_change_decider.cc
 */
#include "object_assignment_builder.h"

#include <iomanip>
#include <iostream>
#include <limits>
#include <memory>

#include "frenet_converter.h"
#include "glog/logging.h"
using namespace std;

namespace rm {

using rm::common::ErrorCode;
using rm::common::Status;

ObjectAssignmentBuilder::ObjectAssignmentBuilder(
    const std::shared_ptr<Observer> &observer,
    const std::shared_ptr<DependencyInjector> &injector)
    : Builder(observer, injector) {
  // CHECK(config_.has_object_assignment_builder_config()); //config_在task类中
  // 相当于把config赋值给task中的config_
}

void ObjectAssignmentBuilder::InCorridorCheck(const Lane &lane,
                                              Corridor *const corridor,
                                              ObjectLocation &object_location,
                                              Obstacle &obstacle, bool option) {
  // option ture: front corridor, need to get rid of the objects in the ego veh
  // body area option false: back corridor
  bool is_not_oncoming =
      obstacle.Run_oncoming_delay(object_location, lane.lane_id());
  if (lane.lane_position() == LanePosition::EGO_LANE) {
    // only discard the rear object on the ego lane
    bool out_ego_zone =
        option ? (object_location.reference_point2corridor_frenet.s() >
                  corridor->mutable_center_line()->ComputeArcLengthbySimpson(
                      0, 3.75))
               : true;
    bool is_out_ego_zone = obstacle.Run_rear_delay(out_ego_zone);
    bool is_in_ego_corrdior = obstacle.In_ego_corridor_check(
        corridor, lane, object_location.reference_point2corridor_frenet);
    if (is_out_ego_zone && is_not_oncoming)
    // the object is in the front of the ego veh and not oncoming
    {
      object_location.in_corridor = is_in_ego_corrdior;
    } else {
      object_location.in_corridor = false;
    }
  } else {
    if (is_not_oncoming)
    // the object is in the front of the ego veh and not oncoming
    {
      object_location.in_corridor = obstacle.In_other_corridor_check(
          lane, corridor, object_location.reference_point2corridor_frenet);
    } else {
      object_location.in_corridor = false;
    }
  }
  // if (obstacle.is_already_assigned()){
  //     // if already assigned to other corridor
  //     object_location.in_corridor = false;
  // } else {
  //     if (object_location.in_corridor) {
  //         obstacle.set_prv_corridor_id(corridor->corridor_id());
  //         obstacle.set_is_already_assigned();
  //         obstacle.Reset_duration_of_intermidiate_status();
  //     }
  // }
}

void ObjectAssignmentBuilder::InCorridorCheck(const Lane &lane,
                                              Corridor *const corridor,
                                              ObjectLocation &object_location,
                                              Obstacle &obstacle, bool option,
                                              double ego_offset,
                                              bool lane_changed_delay) {
  // option ture: front corridor, need to get rid of the objects in the ego veh
  // body area option false: back corridor bool in_corridor_debouncer_result =
  // obstacle.In_corridor_debouncer(corridor,
  // object_location.reference_point2corridor_frenet, ego_offset,
  // lane.lane_width(), lane_changed_delay);
  bool is_not_oncoming =
      obstacle.Run_oncoming_delay(object_location, lane.lane_id());
  if (lane.lane_position() == LanePosition::EGO_LANE) {
    // only discard the rear object on the ego lane
    bool out_ego_zone =
        option ? (object_location.reference_point2corridor_frenet.s() >
                  corridor->mutable_center_line()->ComputeArcLengthbySimpson(
                      0, 3.75))
               : true;
    bool is_out_ego_zone = obstacle.Run_rear_delay(out_ego_zone);
    bool is_in_ego_corridor = obstacle.In_ego_corridor_check(
        corridor, object_location.reference_point2corridor_frenet, ego_offset,
        lane, lane_changed_delay);
    if (is_out_ego_zone && is_not_oncoming)
    // the object is in the front of the ego veh and not oncoming
    {
      object_location.in_corridor = is_in_ego_corridor;
    } else {
      object_location.in_corridor = false;
    }
  } else {
    if (is_not_oncoming)
    // the object is in the front of the ego veh and not oncoming
    {
      object_location.in_corridor = obstacle.In_other_corridor_check(
          lane, corridor, object_location.reference_point2corridor_frenet);
    } else {
      object_location.in_corridor = false;
    }
  }
  // if (obstacle.is_already_assigned()){
  //     // if already assigned to other corridor
  //     object_location.in_corridor = false;
  // } else {
  //     if (object_location.in_corridor) {
  //         obstacle.set_prv_corridor_id(corridor->corridor_id());
  //         obstacle.set_is_already_assigned();
  //         obstacle.Reset_duration_of_intermidiate_status();
  //     }
  // }
}

Status ObjectAssignmentBuilder::Process(Frame *frame,
                                        LaneInfo *const current_lane_info) {
  // DLOG(INFO) << "********************object builder start********************";
  // Sanity checks.
  CHECK_NOTNULL(frame);
  CHECK_NOTNULL(current_lane_info);
  // const auto &object_assignment_builder_config =
  // config_.object_assignment_builder_config(); //获取配置量

  if (current_lane_info->boundary_pair_type() ==
          BoundaryPairType::BOTH_INVALID ||
      current_lane_info->corridor().empty())
    return Status::OK();
  std::map<uint32_t, Obstacle> *obstacles =
      frame->road_info_provider()->mutable_obstacles();
  bool back_corridor_exist =
      !current_lane_info->mutable_back_corridor()->empty();
  std::vector<uint32_t> rear_obstacle_idx_vector;
  // extract key of the map
  std::transform(obstacles->begin(), obstacles->end(),
                 std::back_inserter(rear_obstacle_idx_vector),
                 [](auto &map) { return map.first; });
  double ego_veh_lat_pos =
      current_lane_info->corridor().empty()
          ? 0.0
          : current_lane_info->corridor().front().center_line().Coef(0);
  bool is_ego_lane =
      current_lane_info->lane().lane_position() == LanePosition::EGO_LANE;
  bool lane_changed_delay = false;
  if (is_ego_lane) {
    lane_changed_delay = lane_change_falling_edge_delay_.RunDelay(
        injector_->ego_lane_id_last_cycle() !=
        current_lane_info->lane().lane_id());
  }

  // 1.遍历该车道的corridor from far end
  for (int i = current_lane_info->corridor().size() - 1U; i >= 0; i--) {
    // DLOG(INFO) << "corridor " << i << " here <--------------------------";
    // DLOG(INFO) << PRINT(obstacles->size());
    Corridor *corridor = &current_lane_info->mutable_corridor()->at(i);
    if (corridor->feature_tag() == CorridorFeatureTag::NOTAVLBL)
      continue;
    // FrenetConverter frenet_converter(corridor->center_line());
    FrenetConverter frenet_converter(corridor);

    for (auto idx_iter = rear_obstacle_idx_vector.begin();
         idx_iter != rear_obstacle_idx_vector.end();) {
      // DLOG(INFO) << "objects here<--------------------------";
      uint32_t idx = *idx_iter;

      // if ( obstacles->at(idx).classification() == ObstacleType::PEDESTRIAN ){
      //     //erase pedestrian from the obstacles list
      //     idx_iter = rear_obstacle_idx_vector.erase(idx_iter);
      //     continue;
      // }
      // the obstacle shall be binded with same lane id
      obstacles->at(idx).Bind_with_lane(current_lane_info->lane().lane_id());
      // skip if obj is rear w.r.t current corridor segment(not the first one)
      if (i != 0 && obstacles->at(idx).center().x() <
                        corridor->center_line().start_range()) {
        idx_iter++;
        continue;
      }
      ObjectLocation object_location;
      object_location.reference_point2corridor_frenet =
          frenet_converter.CalculateFrenetPoint(obstacles->at(idx));
      // in the front corridor loop, using the front corridor as reference, and
      // pick out the objects with negative s
      object_location.object_id = obstacles->at(idx).id();
      // 1.查表计算valid time
      object_location.reference_point2corridor_frenet.ComputeValidTime();
      object_location.classification = obstacles->at(idx).classification();
      object_location.relative_center_x = obstacles->at(idx).center().x();
      object_location.relative_center_y = obstacles->at(idx).center().y();
      object_location.size_length = obstacles->at(idx).length();
      object_location.size_width = obstacles->at(idx).width();
      // 2.object贴标钱
      //  InCorridorCheck(current_lane_info->lane(), corridor, object_location,
      //  obstacles->at(idx), true, ego_veh_lat_pos); saturate outputs
      //  object_location.reference_point2corridor_frenet.Sat_SLValue();
      if (i == 0 && obstacles->at(idx).center().x() <
                        corridor->center_line().start_range()) {
        if (!back_corridor_exist) {
          InCorridorCheck(current_lane_info->lane(), corridor, object_location,
                          obstacles->at(idx), true);
          object_location.reference_point2corridor_frenet.Sat_SLValue();
          current_lane_info->mutable_corridor()->front().AddObjectLocation(
              object_location);
        }
        idx_iter++;
      } /*else if (i == current_lane_info->corridor().size() - 1U &&
      !current_lane_info->lane().is_end_out_of_range() &&
      obstacles->at(idx).center().x() > corridor->center_line().end_range()){
          // the end of lane is in the range, and the obstacle is ahead of the
      end range of the furtherest corrdior object_location.in_corridor = false;
          current_lane_info->mutable_corridor()->front().AddObjectLocation(object_location);
          idx_iter = rear_obstacle_idx_vector.erase(idx_iter);
      }*/
      else {
        InCorridorCheck(current_lane_info->lane(), corridor, object_location,
                        obstacles->at(idx), true, ego_veh_lat_pos,
                        lane_changed_delay);
        object_location.reference_point2corridor_frenet.Sat_SLValue();
        current_lane_info->mutable_corridor()->front().AddObjectLocation(
            object_location);
        idx_iter = rear_obstacle_idx_vector.erase(idx_iter);
      }
    }
  }

  // the rear objects are processed after front objects
  Corridor *corridor = &current_lane_info->mutable_corridor()->front();
  for (int i = current_lane_info->back_corridor().size() - 1U; i >= 0; i--) {
    // DLOG(INFO) << "back corridor " << i << " here <--------------------------";
    Corridor *back_corridor =
        &current_lane_info->mutable_back_corridor()->at(i);
    FrenetConverter frenet_converter(back_corridor);
    for (auto idx_iter = rear_obstacle_idx_vector.begin();
         idx_iter != rear_obstacle_idx_vector.end();) {
      uint32_t idx = *idx_iter;
      if (i != 0 && obstacles->at(idx).center().x() >=
                        back_corridor->center_line().end_range()) {
        idx_iter++;
        continue;
      }
      ObjectLocation rear_object_location;
      rear_object_location.object_id = idx;
      rear_object_location.classification = obstacles->at(idx).classification();
      rear_object_location.relative_center_x = obstacles->at(idx).center().x();
      rear_object_location.relative_center_y = obstacles->at(idx).center().y();
      rear_object_location.size_length = obstacles->at(idx).length();
      rear_object_location.size_width = obstacles->at(idx).width();
      rear_object_location.reference_point2corridor_frenet =
          frenet_converter.CalculateFrenetPoint(obstacles->at(idx));
      rear_object_location.reference_point2corridor_frenet.ComputeValidTime();
      InCorridorCheck(current_lane_info->lane(), back_corridor,
                      rear_object_location, obstacles->at(idx), false);
      rear_object_location.reference_point2corridor_frenet.Sat_Rear_SLValue();
      corridor->AddObjectLocation(
          rear_object_location); // attach rear objects in the front corridor
      idx_iter = rear_obstacle_idx_vector.erase(idx_iter);
    }
  }

  // DLOG(INFO) << "********************object builder end********************";
  return Status::OK();
}

} // namespace rm
