#ifndef _FUSION_SLAM_TYPES_H_
#define _FUSION_SLAM_TYPES_H_

#include <stdarg.h>
#include <stdio.h>

#ifndef TUE_CAMERA_RAW_OBJECT_NUM
#define TUE_CAMERA_RAW_OBJECT_NUM 24  // CAMERA object number get from sensor
#endif

#define __IPL_CANTPP__ 1

#ifdef __cplusplus
extern "C"
{
#endif

#if ( (!defined(PLATFORM_TYPES_H)) && ( !( defined(_MSC_VER) && (!defined(__IPL_CANTPP__)) ) ) )

typedef unsigned char boolean;
#endif

#if ( (!defined(PLATFORM_TYPES_H)) && ( !( defined(_MSC_VER) && (!defined(__IPL_CANTPP__)) ) ) )

typedef signed char sint8;
#endif

#if ( (!defined(PLATFORM_TYPES_H)) && ( !( defined(_MSC_VER) && (!defined(__IPL_CANTPP__)) ) ) )

typedef unsigned char uint8;
#endif

#if ( (!defined(PLATFORM_TYPES_H)) && ( !( defined(_MSC_VER) && (!defined(__IPL_CANTPP__)) ) ) )

typedef signed short sint16;
#endif

#if ( (!defined(PLATFORM_TYPES_H)) && ( !( defined(_MSC_VER) && (!defined(__IPL_CANTPP__)) ) ) )

typedef unsigned short uint16;
#endif

#if ( (!defined(PLATFORM_TYPES_H)) && ( !( defined(_MSC_VER) && (!defined(__IPL_CANTPP__)) ) ) )

typedef signed int sint32;
#endif

#if ( (!defined(PLATFORM_TYPES_H)) && ( !( defined(_MSC_VER) && (!defined(__IPL_CANTPP__)) ) ) )

typedef unsigned int uint32;
#endif

#if ( (!defined(PLATFORM_TYPES_H)) && ( !( defined(_MSC_VER) && (!defined(__IPL_CANTPP__)) ) ) )

typedef signed long long sint64;
#endif

#if ( (!defined(PLATFORM_TYPES_H)) && ( !( defined(_MSC_VER) && (!defined(__IPL_CANTPP__)) ) ) )

typedef unsigned long long uint64;
#endif

#if ( (!defined(PLATFORM_TYPES_H)) && ( !( defined(_MSC_VER) && (!defined(__IPL_CANTPP__)) ) ) )

typedef float float32;
#endif

#if ( (!defined(PLATFORM_TYPES_H)) && ( !( defined(_MSC_VER) && (!defined(__IPL_CANTPP__)) ) ) )

typedef double float64;
#endif

typedef sint8* psint8;

typedef uint8* puint8;

typedef sint16* psint16;

typedef uint16* puint16;

typedef sint32* psint32;

typedef uint32* puint32;

typedef void* pvoid;

#if ( (!defined(DISABLE_LEAST_TYPE_GENERATION)) && (!defined(PLATFORM_TYPES_H)) && ( !( defined(_MSC_VER) && (!defined(__IPL_CANTPP__)) ) ) )

typedef signed char sint8_least;
#endif

#if ( (!defined(DISABLE_LEAST_TYPE_GENERATION)) && (!defined(PLATFORM_TYPES_H)) && ( !( defined(_MSC_VER) && (!defined(__IPL_CANTPP__)) ) ) )

typedef unsigned char uint8_least;
#endif

#if ( (!defined(DISABLE_LEAST_TYPE_GENERATION)) && (!defined(PLATFORM_TYPES_H)) && ( !( defined(_MSC_VER) && (!defined(__IPL_CANTPP__)) ) ) )

typedef signed short sint16_least;
#endif

#if ( (!defined(DISABLE_LEAST_TYPE_GENERATION)) && (!defined(PLATFORM_TYPES_H)) && ( !( defined(_MSC_VER) && (!defined(__IPL_CANTPP__)) ) ) )

typedef unsigned short uint16_least;
#endif

#if ( (!defined(DISABLE_LEAST_TYPE_GENERATION)) && (!defined(PLATFORM_TYPES_H)) && ( !( defined(_MSC_VER) && (!defined(__IPL_CANTPP__)) ) ) )

typedef signed int sint32_least;
#endif

#if ( (!defined(DISABLE_LEAST_TYPE_GENERATION)) && (!defined(PLATFORM_TYPES_H)) && ( !( defined(_MSC_VER) && (!defined(__IPL_CANTPP__)) ) ) )

typedef unsigned int uint32_least;
#endif

#ifndef b_FALSE
#define b_FALSE ((boolean)0)
#endif		

#ifndef FALSE
#define FALSE 0
#endif  	

#ifndef b_TRUE
#define b_TRUE ((boolean)1)
#endif

#ifndef TRUE
#define TRUE 1
#endif

#if 0
#ifndef NULL  
#define NULL ((void *)0)
#endif
#endif
#define CONST(consttype, memclass) const consttype
#define CONSTP2CONST(ptrtype, memclass, ptrclass) const ptrtype * const
#define P2VAR(ptrtype, memclass, ptrclass) ptrtype *

#define CT(type) CONST(type, AUTOMATIC)
#define VR(type) VAR(type,   AUTOMATIC)

#define CP2C(type) CONSTP2CONST(type, AUTOMATIC, COM_APPL_CONST)
#define P2C(type)  P2CONST(type,      AUTOMATIC, COM_APPL_CONST)

#define CP2V(type) CONSTP2VAR(type, AUTOMATIC, COMM_APPL_DATA)
#define P2V(type)  P2VAR(type, AUTOMATIC, COMM_APPL_DATA)
#define VAR(vartype, memclass) vartype

#define P2CONST(ptrtype, memclass, ptrclass) const ptrtype *
#define CONSTP2VAR(ptrtype, memclass, ptrclass) ptrtype * const

#define MIN(x,y)        (((x)<(y))?(x):(y))
#define MAX(x,y)        (((x)>(y))?(x):(y))
#define MINMAX(min,max,value) (MIN(MAX(min,value),max))

#ifndef NULL
#define NULL           (0)
#endif

static inline void LogWrite(char* str, ...){
#ifdef FUSION_LOG_MSG
    va_list args;
    va_start(args, str);
    vprintf(str, args);
    printf("\n");
    va_end(args);
#endif
}


/*************************************************************
TM_Rte_Types.h
*/
#define IN
#define OUT
#define INOUT

typedef uint32 AlgoInterfaceVersionNumber_t;
typedef uint8 VEDIoStateTypes_t;
typedef uint8 MotState_t;
typedef uint8 corrQual_t;

typedef struct
{
    float corrAccel;
    float corrAccelVar;
} AccelCorrVehDyn_t;

typedef struct
{
    float Curve;
    float C1;
    float Gradient;
    float varC0;
    float varC1;
    float Quality;
    float CrvError;
    unsigned char CrvConf;
} CurveVehDyn_t;

typedef struct
{
    float Curve;
    float Variance;
    float Gradient;
} DrvIntCurveVehDyn_t;

typedef struct
{
    float LatAccel;
    float Variance;
} LatAccelVehDyn_t;

typedef struct
{
    float YawRateMaxJitter;
    boolean bStandStill;
} LegacyVehDyn_t;

typedef struct
{
    float Velocity;
    float Accel;
    float varVelocity;
    float varAccel;
} MotVarVehDyn_t;

typedef struct
{
    float SideSlipAngle;
    float Variance;
} SideSlipVehDyn_t;

typedef struct
{
    float YawRate;
    float Variance;
    float Quality;
} YawRateVehDyn_t;

typedef struct
{
    uint32 uiTimeStamp;
    uint16 uiMeasurementCounter;
    uint16 uiCycleCounter;
    uint8 eSigStatus;
} SignalHeader_t;

typedef struct
{
    YawRateVehDyn_t YawRate;
    float OffCompStWheelAngle;
    CurveVehDyn_t Curve;
    DrvIntCurveVehDyn_t DrvIntCurve;
    LatAccelVehDyn_t Accel;
    SideSlipVehDyn_t SlipAngle;
} Lateral_t;

typedef struct
{
    MotState_t MotState;
    float Confidence;
} MotionStateVehDyn_t;

typedef VEDIoStateTypes_t State_array_t[12];

typedef struct
{
    float corrFact;
    float corrVar;
    float corrVelo;
    float corrVeloVar;
    float minVelo;
    float maxVelo;
    corrQual_t corrQual;
    boolean bRollerTestBench;
} VeloCorrVehDyn_t;

typedef struct
{
    MotVarVehDyn_t MotVar;
    VeloCorrVehDyn_t VeloCorr;
    AccelCorrVehDyn_t AccelCorr;
} Longitudinal_t;

/*************************************************************
BSW_Interface.h
*/
#define BSW_OBJECT_NUM_MAX 40//radar object number get from sensor
#define BSW_NR_PRIVOBJECTS    (25)
#define BSW_LIDAR_OBJECT_NUM_MAX 40//Lidar object number get from sensor

#define SONSOR_OBJECT_DYNPROP_MOVING               (0)
#define SONSOR_OBJECT_DYNPROP_STATIONARY           (1)
#define SONSOR_OBJECT_DYNPROP_ONCOMING             (2)
#define SONSOR_OBJECT_DYNPROP_STATIONARY_CANDITATE (3)
#define SONSOR_OBJECT_DYNPROP_UNKNOWN              (4)
#define SONSOR_OBJECT_DYNPROP_CROSSING_STATIONARY  (5)
#define SONSOR_OBJECT_DYNPROP_CROSSING_MOVING      (6)
#define SONSOR_OBJECT_DYNPROP_STOPPED              (7)

typedef enum {
    CAMOBJ_REF_POSITON_UNKNOWN = 0,
    CAMOBJ_REF_POSITON_REAR_CENTER = 1,
    CAMOBJ_REF_POSITON_FRONT_CENTER = 2,
    CAMOBJ_REF_POSITON_MIDDLE_CENTER = 3,
    CAMOBJ_REF_POSITON_MAXVALUE = 4,
} eRefpointLocation_Type;

typedef struct
{
    uint32 uiTimeStamp;
    uint16 uiMeasurementCounter;
    uint16 uiCycleCounter;
    uint8 eSigStatus;
} BSW_SignalHeader_t;

//	OBJECT_DYNPROP_UNKNOWN:0, OBJECT_DYNPROP_STATIONARY:1,OBJECT_DYNPROP_STOPPED:2,	OBJECT_DYNPROP_MOVING:3,
//	OBJECT_DYNPROP_CROSSING_MOVING:4,OBJECT_DYNPROP_CROSSING_STATIONARY:5,OBJECT_DYNPROP_ONCOMING:6,OBJECT_DYNPROP_STATIONARY_CANDITATE:7,
typedef uint8 OBJECT_DYNPROP_Type;
//	MEAST_STATE_DELETED=0,
//	MEAST_STATE_NEW=1,
//	MEAST_STATE_MEASURED=2,
///	MEAST_STATE_PREDICTED=3,
//	MEAST_STATE_MERGE_DELETED=4,
//	MEAST_STATE_MERGE_NEW=5
// }eObjMaintenanceStateType;
typedef uint8 eObjMaintenanceStateType;
typedef uint8 eClassification_Type;

typedef struct
{
    float32 fDistX;					// m
    float32 fDistXStd;				//
    float32 fDistY;					//
    float32 fDistYStd;				//
    float32 fVx;					// m/s
    float32 fVxStd;					//
    float32 fVy;					//  m/s
    float32 fVyStd;					//
    float32 fAx;					//  m/s2
    float32 fAxStd;					//
    float32 fAy;					// m/s2
    float32 fAyStd;					//
    float32 fOrientation;			// degree;
    uint8 eRefpointLocation;		////eRefpointLocation_Type
} CamObjKinematics;

typedef struct
{
    float32 fHeight;				//			
    float32 fOffsetToGround;		//
    float32 fLength;				//
    float32 fWidth;					//
} CamObjGeometry;

typedef struct
{
    uint8 probabilityOfClassification;
    uint8  ObjectType;		//eObject_Type
} CamObjClassification;

typedef struct
{
    uint8 eStatusBrakeLight;				//break light
    uint8 uiBrakeLightConfidence;			//break light 
    uint8 eStatusTurnIndicator;				//turn indicator light,
    uint8 uiTurnIndicatorConfidence;		// turn indicator light	
    uint8 eAssociatedLane;					//
    uint8 uiAssociatedLaneConfidence;		//
    uint8 eLaneChangeManeuver;				// lane changed
    uint8 percentageOwnDrivingLane;			//
    uint8 percentageSideDrivingLaneLeft;	//	
    uint8 percentageSideDrivingLaneRight;	//
    uint8 eMainteanceState;					// 0x0:Delete, 0x1: measure, 0x2, predicted, 0x3, invalid
    uint8 percentageProbOfExist;			// object exist percent
    uint8 eMotionState;						// 0x0: nodefined, 0x1:static, 0x2:moving, 0x3:oncoming, 0xf:invalid;
    uint8 eObjectOcclusion;					// 0x0:no define, 0x1, no known, 0x2, part, 0x3:all, 0x4:invalid
    uint32 ObjectLifeCycles;				// lifetime.
} CamObjAttributes;

typedef struct
{
    uint8	ObjectID;
    CamObjKinematics Kinematic;
    CamObjGeometry Geometry;
    CamObjClassification Classification;
    CamObjAttributes Attributes;
} CamObject;

typedef CamObject CamObjectArray[TUE_CAMERA_RAW_OBJECT_NUM];

typedef struct
{
    uint8	  Type;
    uint8	  Quality;
    float	  Position;
    float	  Curvature;
    float	  CurvatureDer;
    float	  WidthMarking;
    float	  HeadingAngle;		// degree
    float 	  ViewRangeStart;
    float	  ViewRangeEnd;
    uint8	  LineCrossing;
    uint8 	  LineMarkColor;	// eLineCol_Type
    uint8	  CanMsgEnd;
} CamLane_t;

typedef CamLane_t 	CamLane_array_t[4];

typedef struct
{
    float32 fDistX;
    float32 fDistXStd;
    float32 fDistY;
    float32 fDistYStd;
    float32 fVrelX;
    float32 fVrelXStd;
    float32 fVrelY;
    float32 fVrelYStd;
    float32 fArelX;
    float32 fArelXStd;
    float32 fArelY;
    float32 fArelYStd;
} Kinematic_t;

typedef struct
{
    float32 fWidth;
    float32 fLength;
    float32 fOrientation;
    float32 fOrientationStd;
    float32 fOrientationValid;
} Geometry_t;

typedef struct
{
    uint8 eDynamicProperty;	// OBJECT_DYNPROP_Type
    uint8 uiStoppedConfidence;
    uint8 eAbsMovingState;
    uint8 eClassification;	// eClassification_Type
    uint8 uiClassConfidence;
} Attributes_t;

typedef struct
{
    float32 fLifeTime;
    uint32 fTimeStamp; // maybe system simulation timestamp
    uint8 eObjMaintenanceState;	// eObjMaintenanceStateType
    uint8 cObjMergingID;
} General_t;

typedef struct
{
    float32 fProbabilityOfExistence;
    uint8 ucObstacleProbability;//add Obstacle Probability for ars410 radar
    uint8 uMeasuredTargetFrequencyNear;
    uint8 uMeasuredTargetFrequencyFar;
} Qualifiers_t;

typedef struct
{
    float32 fRCS;
    boolean bCamConfirmed;//true: object has been observed by camera in last 100 cycle
} SensorSpecific_t;

typedef struct
{
    uint16 eEbaInhibitionMask;
    uint8 ucEbaMovingObjQuality;
    uint8 eEbaHypCat;
    boolean bCrossingPedEbaPresel;
} EBAPresel_t;

typedef struct
{

    float32 fMaxAccelY;
    float32 fRCSTargetThresholdUncomp;
    uint16 uiLifeTime;
    uint8 eDynamicSubProperty;
} LegacyObj_t;

typedef struct
{
    uint8 ObjectId;
    Kinematic_t Kinematic;
    Geometry_t Geometry;
    Attributes_t Attributes;
    General_t General;
    Qualifiers_t Qualifiers;
    SensorSpecific_t SensorSpecific;
    LegacyObj_t Legacy;
} ExtObjects_t;
typedef ExtObjects_t Ext_Objects_array_t[BSW_OBJECT_NUM_MAX];

typedef sint8 iSortedObjectList_array_t[BSW_NR_PRIVOBJECTS];

typedef struct
{
    sint8 iNumOfUsedObjects;
    iSortedObjectList_array_t iDistXSortedObjectList;
} HeaderObjList_t;

typedef uint8 ucAccSelBits_t;

typedef struct
{
    uint8 ucAccObjQuality;
    ucAccSelBits_t ucAccSelBits;
} ACCPresel_t;

typedef struct
{
    uint8 ObjectId;
    Kinematic_t Kinematic;
    Geometry_t Geometry;
    Attributes_t Attributes;
    General_t General;
    Qualifiers_t Qualifiers;
    SensorSpecific_t SensorSpecific;
    ACCPresel_t ACCPresel;
    EBAPresel_t EBAPresel;
    LegacyObj_t Legacy;
    uint16 au16SensorID[2];
} Objects_t;
typedef Objects_t Objects_array_t[BSW_NR_PRIVOBJECTS];

typedef struct
{
    uint32 uiVersionNumber;
    BSW_SignalHeader_t sSigHeader;
    CamObjectArray aObject;
    CamLane_array_t sLaneLines;
} CamObjectList;

typedef struct
{
    AlgoInterfaceVersionNumber_t uiVersionNumber;
    SignalHeader_t sSigHeader;
    Longitudinal_t Longitudinal;
    Lateral_t Lateral;
    MotionStateVehDyn_t MotionState;
    LegacyVehDyn_t Legacy;
    State_array_t State;
} VehDyn_t;

typedef struct
{
    uint8 uiTunnelProb;
    uint8 uiFilteredTimeIndicator;//filtered value of u8TimeIndicator from mini-eye
} SceneRecognize_t;

typedef struct
{
    uint32 uiVersionNumber;
    SignalHeader_t sSigHeader;
    HeaderObjList_t HeaderObjList;
    SceneRecognize_t sSceneDescription;//add a new structure for scene recognize: tunnel recognize, time indicator...
    Objects_array_t Objects;
} ObjectList_t;

typedef struct
{
    uint32 uiVersionNumber;
    BSW_SignalHeader_t sSigHeader;
    HeaderObjList_t HeaderObjList;
    Ext_Objects_array_t Objects;
} ExtObjectList_t;

typedef struct {
    float32 f_X;
    float32 f_Y;
    float32 f_DistToTraj;
    float32 f_DistOnTraj;
} BML_t_TrajRefPoint;

//FIXME: Based on Radar data, must be changed accordingly
// ********************************** LiDAR sensor data *******************************

typedef struct
{
    uint32 uiTimeStamp;
    uint16 uiMeasurementCounter;
    uint16 uiCycleCounter;
    uint8 eSigStatus;
} Lidar_BSW_SignalHeader_t;
typedef struct
{
    float32 fProbabilityOfExistence;
    uint8 ucObstacleProbability;//add Obstacle Probability for ars410 radar
    //uint8 uMeasuredTargetFrequencyNear;
    //uint8 uMeasuredTargetFrequencyFar;
} LidarQualifiers_t;


typedef struct
{
    uint8 ObjectId;
    Kinematic_t Kinematic;
    Geometry_t Geometry;
    Attributes_t Attributes;
    General_t General;
    LidarQualifiers_t Qualifiers;
    // SensorSpecific_t SensorSpecific;
    // LegacyObj_t Legacy;
} LidarObjects_t;

typedef LidarObjects_t Lidar_Objects_array_t[BSW_LIDAR_OBJECT_NUM_MAX];

typedef struct {
    uint32 uiVersionNumber;
    Lidar_BSW_SignalHeader_t sSigHeader;
    HeaderObjList_t HeaderObjList;
    Lidar_Objects_array_t Objects;
} LidarObjectList_t;

// typedef struct {
//     uint32  uiTimeStamp;
//     uint16  uiCycleCounter;
//     uint8   eSigStatus;
//     uint16  uNumOfUsedObjects;
//     FusObjects_array_t Objects;
// } LidarFusObjectList_t;

/********************************
Fusion_Std_Types.h:
FusionCompiler.h
FusionCompiler_Cfg.h
*/

#define E_NOT_OK 0x01U

#define STD_HIGH 0x01U /* Physical state 5V or 3.3V */
#define STD_LOW 0x00U  /* Physical state 0v         */

#define STD_ACTIVE 0x01U /* Logical state active */
#define STD_IDLE 0x00U   /* Logical state idle   */

#define STD_ON 0x01U
#define STD_OFF 0x00U

/*****************************************************************************/
/* Define:       AUTOMATIC                                                   */
/* Description:  Used for the declaration of local pointers                  */
/*****************************************************************************/
#define AUTOMATIC
/*****************************************************************************/
/* Define:       TYPEDEF                                                     */
/* Description:  The memory class shall be used within type definitions,     */
/*               where no memory qualifier can be specified                  */
/*****************************************************************************/
#define TYPEDEF
/*****************************************************************************/
/* Define:       STATIC                                                      */
/* Description:  Abstraction of the keyword static                           */
/*****************************************************************************/
#define STATIC static
/*****************************************************************************/
/* Define:       NULL_PTR                                                    */
/* Description:  Void pointer to 0                                           */
/*****************************************************************************/
#ifndef NULL_PTR
#define NULL_PTR ((void *)0)
#endif
/*****************************************************************************/
/* Macro name: FUNC                                                          */
/* Parameters: rettype     return type of the function                       */
/*             memclass    classification of the function itself             */
/*****************************************************************************/
#ifndef FUNC
#define FUNC(rettype, memclass) rettype memclass
#endif
/*****************************************************************************/
/* Macro name: P2FUNC                                                        */
/* Parameters: rettype     return type of the function                       */
/*             ptrclass    defines the classification of the pointer's       */
/*                         distance                                          */
/*             fctname     function name respectively name of the defined    */
/*                         type                                              */
/*****************************************************************************/
#ifndef P2FUNC
#define P2FUNC(rettype, ptrclass, fctname) ptrclass rettype (*fctname)
#endif
/*****************************************************************************/
/* Macro name: FUNC_P2CONST                                                  */
/* Parameters: rettype     return type of the function                       */
/*             ptrclass    defines the classification of the pointer's       */
/*                         distance                                          */
/*             memclass    classification of the function itself             */
/*****************************************************************************/
#ifndef FUNC_P2CONST
#define FUNC_P2CONST(rettype, ptrclass, memclass) const ptrclass rettype * memclass
#endif 
/*****************************************************************************/
/* Macro name: FUNC_P2CONST                                                  */
/* Parameters: rettype     return type of the function                       */
/*             ptrclass    defines the classification of the pointer's       */
/*                         distance                                          */
/*             memclass    classification of the function itself             */
/*****************************************************************************/
#ifndef FUNC_P2VAR
#define FUNC_P2VAR(rettype, ptrclass, memclass) ptrclass rettype * memclass
#endif

/*
 * Definition of static keyword for enabling external visibility of
 * translation unit objects. The default resolution is 'static' and
 * should be used for production builds.
 * For unit test environment LOCAL shall be defined as LOCAL= on the
 * command line. This will allow unit test harness to access UUT
 * static functions and objects
 */
#ifndef LOCAL
#define LOCAL static
#endif /* LOCAL */

#ifndef EXTERN
#define EXTERN extern
#endif /* EXTERN */

/* used for SWC specific code */
#define ObjFusn_CODE

/* used for code that shall go into fast code memory segments */
#define ObjFusn_CODE_FAST

/* used for global or static SWC specific constants */
#define ObjFusn_CONST

/* used for all global or static SWC specific variables that are not initialized
 * by the startup code of the compiler */
#define ObjFusn_VAR_NOINIT

/* used for global or static SWC specific variables that are initialized by the
 * startup code of the compiler */
#define ObjFusn_VAR_INIT

/* used for global or static SWC specific variables that are initialized with
 * zero by the startup code of the compiler */
#define ObjFusn_VAR_ZERO_INIT

/* used for all global or static SWC specific variables that are not initialized
 * by the startup code of the compiler */
#define ObjFusn_VAR_FAST_NOINIT

/* used for global or static SWC specific variables that are initialized by the
 * startup code of the compiler */
#define ObjFusn_VAR_FAST_INIT

/* used for all global or static SWC specific variables that are not initialized
 * by the startup code of the compiler */
#define ObjFusn_VAR_SLOW_NOINIT

/* used for global or static SWC specific variables that are initialized by the
 * startup code of the compiler */
#define ObjFusn_VAR_SLOW_INIT

#define ObjFusn_VAR_NOSTACK

#define ObjFusn_APPL_CODE

/* To be used for references on application data (expected to be in RAM or ROM)
 * passed via API */
#define ObjFusn_APPL_DATA

/* To be used for references on application constants (expected to be certainly
   in ROM, for instance pointer of Init-function)
   passed via API */
#define ObjFusn_APPL_CONST

/**************************************************************************************************************
 * Utils.h
*/
// Used in ContiRadarConverter_int.h
#define TU_OBJECT_DYNPROP_MOVING               (0)
#define TU_OBJECT_DYNPROP_STATIONARY           (1)
#define TU_OBJECT_DYNPROP_ONCOMING             (2)
#define TU_OBJECT_DYNPROP_STATIONARY_CANDITATE (3)
#define TU_OBJECT_DYNPROP_UNKNOWN              (4)
#define TU_OBJECT_DYNPROP_CROSSING_STATIONARY  (5)
#define TU_OBJECT_DYNPROP_CROSSING_MOVING      (6)
#define TU_OBJECT_DYNPROP_STOPPED              (7)
#ifdef TUERME_CYCLETIME_CHANGE

#define DAP_RADAR_CYCLE_TIME   0.09f//0.066f  
#else
#define DAP_RADAR_CYCLE_TIME   0.066f  
#endif

#ifndef Envm_GEN_OBJECT_CLASS_POINT
#define Envm_GEN_OBJECT_CLASS_POINT 0U
#endif
#ifndef Envm_GEN_OBJECT_CLASS_CAR
#define Envm_GEN_OBJECT_CLASS_CAR 1U
#endif
#ifndef Envm_GEN_OBJECT_CLASS_TRUCK
#define Envm_GEN_OBJECT_CLASS_TRUCK 2U
#endif
#ifndef Envm_GEN_OBJECT_CLASS_PEDESTRIAN
#define Envm_GEN_OBJECT_CLASS_PEDESTRIAN 3U
#endif
#ifndef Envm_GEN_OBJECT_CLASS_MOTORCYCLE
#define Envm_GEN_OBJECT_CLASS_MOTORCYCLE 4U
#endif
#ifndef Envm_GEN_OBJECT_CLASS_BICYCLE
#define Envm_GEN_OBJECT_CLASS_BICYCLE 5U
#endif
#ifndef Envm_GEN_OBJECT_CLASS_WIDE
#define Envm_GEN_OBJECT_CLASS_WIDE 6U
#endif
#ifndef Envm_GEN_OBJECT_CLASS_UNCLASSIFIED
#define Envm_GEN_OBJECT_CLASS_UNCLASSIFIED 7U
#endif
#ifndef Envm_GEN_OBJECT_CLASS_TL
#define Envm_GEN_OBJECT_CLASS_TL 8U
#endif
#ifndef Envm_GEN_OBJECT_CLASS_MAX_DIFF_TYPES
#define Envm_GEN_OBJECT_CLASS_MAX_DIFF_TYPES 9U
#endif

#ifdef __cplusplus
}
#endif

#endif //_FUSION_SLAM_TYPES_H_
