#ifndef BASE_MEASURE_H
#define BASE_MEASURE_H

#include <vector>
#include "ego_msg_info.h"
#include "common_define.h"

//#ifndef EIGEN_DONT_ALIGN_STATICALLY
//#define EIGEN_DONT_ALIGN_STATICALLY
//#endif

namespace Localization {
namespace Ego {

enum class MeasurementType : std::int8_t{
    DEFAULT = 0,
    IMU = 1,
    WHEEL = 2,
    RTK = 3,
    VISION = 4,
    VSL = 5,
    LIDAR = 6,
    CHECKMEASURE = 7
};

class BaseMeasure {

public:
    virtual bool Update(const State &state, State &nextState) = 0;
    virtual Eigen::VectorXd MeasurementEquation(const State &state) const = 0;
    BaseMeasure(const MeasurementType &mType): type_(mType) {};
    BaseMeasure() = delete;
    BaseMeasure(const BaseMeasure &) = delete;
    BaseMeasure & operator=(const BaseMeasure &) = delete;
    //======== function ========//
    void UpdateState(const State &state, State &nextState, const Eigen::VectorXd &dx, const Eigen::MatrixXd &mH,
       const Eigen::MatrixXd &mK) const;
    double GetStamp() { return stamp_; }
    MeasurementType GetMeasurementType() { return type_; }

    double stamp_ = 0.0;
    double seq_ = 0;
    MeasurementType type_ = MeasurementType::DEFAULT;
    Eigen::Vector3d imuGyroCmp_;

    // 状态量
    bool isKFUsed_;
    static bool isStatic_;
    static bool isImuValid_;
    static bool isVehValid_;
    static bool isRtkValid_;
    static bool isVlocValid_;
    static double lastImuStamp_;
    static double lastVehStamp_;
    static double lastRtkStamp_;
    static double lastVlocStamp_;
    static bool isImuBlock_;
    static bool isVehBlock_;
    static bool isRtkBlock_;
    static bool isVlocBlock_;

    static Eigen::Vector3d vLocPos_;
    static Eigen::Vector3d vLocAtt_;
    static Eigen::Vector3d rtkPos_;
    static Eigen::Vector3d rtkAtt_;
    static Eigen::Quaterniond stateQuat_;
    static double vehVelo_;
    // DataCheck
    static double vLocCheckOffset_;
    static bool useVlocCheckOffset_;


    uint16_t imuFreq = 100;
    uint16_t vehFreq = 100;
    uint16_t rtkFreq = 100;
    Eigen::MatrixXd mRm_ = Eigen::Matrix3d::Identity();
    const std::vector<std::string> typeName_{"DEFAULT","IMU", "WHEEL", "RTK", "VISION", "VSL", "LIDAR"}; // 0~6
    const std::vector<double> chiSquare99Table{0.0, 6.63, 9.21, 11.34, 13.28, 15.09, 16.81};

}; // class EgoObserve


} // namespace Ego
} // namespace Localization

#endif
