#ifndef EGO_INTERFACE_H_
#define EGO_INTERFACE_H_

//#ifndef EIGEN_DONT_ALIGN_STATICALLY
//#define EIGEN_DONT_ALIGN_STATICALLY
//#endif

#include <unistd.h>
#include "kf_interface.h"
#include "base_measure.h"
#include "rtk_measure.h"
#include "imu_measure.h"
#include "veh_measure.h"
#include "vloc_measure.h"
#include "check_measure.h"
#include <queue>
#include "base_measure.h"
#include "ego_msg_info.h"
#include "common_define.h"
#include <thread>
#include <mutex>
#include <memory>
#include <condition_variable>
#include "atomic"

namespace Localization {
namespace Ego{

class EgoInterface {
public:
    EgoInterface();
    ~EgoInterface();
    void Init();
    void StopThread();
    void GetEgoOutput(LocalizationResult &egoOuput);
    void SetRtkInput(InsInfo &rtk);
    void SetImuInput(ImuInfo &imu);
    void SetVehInput(VehInfo &veh);
    void SetVLocInput(RecLocInfo &vLoc);
    void SetCheckMeaInput(CheckMeaInfo &checkMea);
private:
    void startThreadLoop();
    void stopThreadLoop();
    void PredictThread();
    void UpdateThread();
    bool InitKF(std::shared_ptr<BaseMeasure> &ptrM);
    void AddMeasureToBuffer(std::shared_ptr<BaseMeasure> &pMeasure);
    void AddMeasureToRecBuf(std::shared_ptr<BaseMeasure> &pMeasure);
    void GetOneMeaFromBuf(std::vector<std::shared_ptr<BaseMeasure>> &meaBuf, std::shared_ptr<Ego::BaseMeasure> &ptrM);
    void SetLocalizationResult(State &stateRes,LocalizationResult &output);
    void SetEgoOutput(LocalizationResult &kfRes);
    void ResetMeaBuf();
    void SetFusionOutput(LocalizationResult &kfRes);

    std::vector<std::shared_ptr<BaseMeasure>> measureBuffer_;
    std::vector<std::shared_ptr<BaseMeasure>> recLocBuffer_;
    std::mutex mutexMeasureBuffer_;
    std::mutex mutexRecLocBuffer_;
    std::mutex mutexOutput_;
    std::thread updateThread_;
    std::thread predictThread_;
    std::atomic<bool> predictThreadFlag_;
    std::atomic<bool> updateThreadFlag_;
    bool isKFInit_ = false;//true
    bool isUsePredict_ = false;//false
    KFInterface kf_;
    LocalizationResult egoOuput_;

};
}// namespace Ego
}// namespace Localizaiton

#endif
