#ifndef EGO_MSG_INFO_H
#define EGO_MSG_INFO_H

//#ifndef EIGEN_DONT_ALIGN_STATICALLY
//#define EIGEN_DONT_ALIGN_STATICALLY
//#endif

#include "Eigen/Core"
#include "Eigen/Dense"
namespace Localization {
namespace Ego {
enum AXIS_DIM
{
    AXIS_X = 0,
    AXIS_Y = 1,
    AXIS_Z = 2
};

struct ImuInfo {
    bool isValid_ = true;
    bool isEgoUsed_ = false;
    bool isVslUsed_ = false;
    bool isLidarUsed_ = false;
    bool isRelativeUsed_ = false;

    unsigned int seq_ = 0U;
    double stamp_ = 0.0;
    Eigen::Vector3d acc_{0,0,0};  // m/s^2
    Eigen::Vector3d gyro_{0,0,0}; // rad/s
};

struct VehInfo {
    bool isValid_ = true;
    bool isEgoUsed_ = false;
    bool isVslUsed_ = false;
    bool isLidarUsed_ = false;
    bool isRelativeUsed_ = false;

    unsigned int gear_ = 3U; // 0-drive;1-return;2-park;3-default
    unsigned int seq_ = 0U;
    double stamp_ = 0.0;
    double velocity_;
    double wheelspeedFL_ = 0.0;
    double wheelspeedFR_ = 0.0;
    double wheelspeedRL_ = 0.0;
    double wheelspeedRR_ = 0.0;
    unsigned int wheelCountFL_ = 0;
    unsigned int wheelCountFR_ = 0;
    unsigned int wheelCountRL_ = 0;
    unsigned int wheelCountRR_ = 0;
    double yawRate_ = 0.0;
};

struct InsInfo {
    unsigned int positionType_ = 0;//PositionQuality
    unsigned int satUseNum_ = 0;// if lack , don't write
    unsigned int satInViewNum_ = 0;// if lack , don't write
    unsigned int solutionStatus = 0;// position status
    unsigned int utmZoneNum = 0;// if lack , don't write
    bool isValid_ = true;
    bool isEgoUsed_ = false;
    bool isVslUsed_ = false;
    bool isLidarUsed_ = false;
    bool isRelativeUsed_ = false;

    unsigned int seq_ = 0U;
    double stamp_ = 0.0;

    double longitude_ = 0;
    double latitude_ = 0;
    double elevation_ = 0;
    double heading = 0;
    double cog = 0;// if lack , don't write
    double tdop = 0;
    double vdop = 0;
    double hdop = 0;
    Eigen::Vector3d utmPosition_{0,0,0};  // if lack , don't write
    Eigen::Vector3d attitude_{0,0,0,};    // x y z, roll, pitch,yaw
    Eigen::Quaterniond quat_{1,0,0,0}; // if lack , don't write
    Eigen::Vector3d attitudeDual_{0,0,0};// if lack , don't write
    Eigen::Vector3d velocityENU_{0,0,0};// Velocity (north, east, down)
    double velocityVeh_ = 0.0;//ground_vel_2d
};

struct GNSSInfo {
    unsigned int positionType_ = 0;//PositionQuality
    unsigned int satUseNum_ = 0;// if lack , don't write
    unsigned int satInViewNum_ = 0;// if lack , don't write
    unsigned int solutionStatus = 0;// position status
    unsigned int utmZoneNum = 0;// if lack , don't write
    bool isValid_ = true;
    bool isEgoUsed_ = false;
    bool isVslUsed_ = false;
    bool isLidarUsed_ = false;
    bool isRelativeUsed_ = false;

    unsigned int seq_ = 0U;
    double stamp_ = 0.0;

    double longitude_ = 0;
    double latitude_ = 0;
    double elevation_ = 0;
    double heading = 0;
    double cog = 0;// if lack , don't write
    Eigen::Vector3d utmPosition_{0,0,0};  // if lack , don't write
    Eigen::Vector3d attitude_{0,0,0,};    // x y z
    Eigen::Quaterniond quat_{1,0,0,0}; // if lack , don't write
    Eigen::Vector3d attitudeDual_{0,0,0};// if lack , don't write
    Eigen::Vector3d velocityENU_{0,0,0};// Velocity (north, east, down)
    double velocityVeh_ = 0.0;//ground_vel_2d
};

struct RecLocInfo {
  enum RecLocStatus{
    bad = 0U, // compute bad
    no_map = 1U, // no map
    no_line = 2U, // no input line
    compute_predict = 3U, // compute bad,icp wrong,and rtknot valid,just predict;如果good 太少再考虑用这个
    good = 4U, /// certainly match map,use this one;
    compute_ref = 5U // loc jump compared with valid rtk,use global ref's result;
  };
  bool isValid_ = true;
  bool isEgoUsed_ = false;
  bool isRelativeUsed_ = false;

  unsigned int status_ = 0U; // filled with RecLocStatus
  unsigned int integratedState = 0U;
  unsigned int seq_ = 0U;
  double stamp_ = 0.0; // unit: second
  double score_ = 0.0; // unit: 100 score
  double heading = 0; // unit:degree

  Eigen::Vector3d posLLA_{0,0,0};
  Eigen::Vector3d posUTM_{0,0,0};
  Eigen::Vector3d attitude_{0,0,0};
  Eigen::Matrix3d rot_ = Eigen::Matrix3d::Identity();
  Eigen::Quaterniond quat_{1,0,0,0};
};
struct CheckMeaInfo{
  unsigned int status_ = 0U;
  double stamp_{0.0};
  double vlocCheckOffset_{0.0}; // unit: m
  double vlocCheckYawDiff_{0.0}; // unit: degree
};
} // namespace Ego
} // namespace Localization
#endif
