#pragma once

#include "data_manager/common_data_manager.h"
#include "interface/slam_vehicle_data.h"
#include "interface/slam_decision.h"
#include "interface/slam_road_data.h"
#include "interface/slam_road_model.h"
#include "interface/slam_rtk_data.h"
#include "interface/slam_odometry.h"

namespace slam
{

    class DataManager
    {
    public:
        static DataManager &getInstance()
        {
            static DataManager instance;
            return instance;
        };

        DataManager();
        DataManager(const DataManager &) = delete;
        DataManager &operator=(const DataManager &) = delete;

        void Init();
        // for Input data
        void InputVisionMsg(RoadFrame *_vis_msg);
        void InputCanMsg(VehicleData *_can_msg);
        void InputImuMsg(ImuData *_imu_msg);
        void InputGnssMsg(GnssData *_gnss_msg);
        void InputInsMsg(InsData *_ins_msg);

        // for Output data
        void SetOdometryData(slam_odometry *_odom_msg)
        {
            odom_manager_->InputData(_odom_msg);
        };

        void SetDecisionData(ControlWaypoints *_dcp_msg)
        {
            decision_manager_->InputData(_dcp_msg);
        };

        void SetRoadModelData(slam_road_model *_rm_msg)
        {
            rm_manager_->InputData(_rm_msg);
        };

        bool GetVisionMsg(RoadFrame *_msg)
        {
            return cam_manager_->GetData(_msg);
        }

        bool GetImuMsg(ImuData *_msg)
        {
            return imu_manager_->GetData(_msg);
        }

        bool GetCanMsg(VehicleData *_msg)
        {
            return can_manager_->GetData(_msg);
        }

        bool GetGnssMsg(GnssData *_msg)
        {
            return gnss_manager_->GetData(_msg);
        }

        bool GetInsMsg(InsData *_msg)
        {
            return ins_manager_->GetData(_msg);
        }

        bool GetOdometryResult(slam_odometry *_msg)
        {
            return odom_manager_->GetData(_msg);
        }

        bool GetDecisionResult(ControlWaypoints *_msg)
        {
            return decision_manager_->GetData(_msg);
        }

        bool GetRoadModelResult(slam_road_model *_msg)
        {
            return rm_manager_->GetData(_msg);
        }

//        slam_odometry* GetOdometry()
//        {
//            return &cur_odom_;
//        }

//        slam_road_model* GetRoadMode()
//        {
//            return &cur_rm_p;
//        }

    private:
        std::shared_ptr<CommonDataManager<RoadFrame>> cam_manager_;
        std::shared_ptr<CommonDataManager<ImuData>> imu_manager_;
        std::shared_ptr<CommonDataManager<VehicleData>> can_manager_;
        std::shared_ptr<CommonDataManager<GnssData>> gnss_manager_;
        std::shared_ptr<CommonDataManager<InsData>> ins_manager_;

        std::shared_ptr<CommonDataManager<slam_odometry>> odom_manager_;
        std::shared_ptr<CommonDataManager<ControlWaypoints>> decision_manager_;
        std::shared_ptr<CommonDataManager<slam_road_model>> rm_manager_;

    };

} // hpp_map_loc
