#ifndef SLAM_INTERFACE_ROAD_DATA_H_
#define SLAM_INTERFACE_ROAD_DATA_H_
#pragma pack(push)
#pragma pack(1)

#include <cstdint>
#include <string>
#include <vector>
#include "interface/common_type.h"

namespace slam {

struct LMLanePoint2f {
    float x;
    float y;
    uint8_t next_segment_lane_line_type;
    uint8_t next_segment_lane_line_color;
    float next_segment_lane_line_width;
    float next_segment_lane_line_type_confidence;
    float next_segment_lane_line_color_confidence;
    float next_segment_lane_line_width_confidence;
};

struct LaneLine {
    int16_t id;
    int16_t position_index;
    long track_id;
    long track_age;
    float distance_to_left_bumper;
    float distance_to_right_bumper;
    float laneline_width;
    uint8_t color_id;
    uint8_t laneline_type_id;
    uint8_t edge_type_id;
    float lane_confidence;
    float color_confidence;
    float laneline_type_confidence;
    float edge_type_confidence;
    float laneline_width_confidence;
    int16_t fitting_degree;
    uint8_t is_line_junction_valid;
    int16_t lane_category;
    uint8_t line_type_id;
    float line_type_confidence;
    int16_t fitting_type;
    std::vector<Vector2f> image_points;
    std::vector<Vector2f> world_points_reproj;
    std::vector<float> polyfit_coefficients;
    std::vector<LMLanePoint2f> junction_points;
    Vector2f start_point;
    Vector2f end_point;
};


struct StopLine {
    long id;
    std::vector<Vector2f> image_points;
    std::vector<Vector2f> world_points;
    std::vector<float> confidence;
};


struct Boundary {
    float direction_x;
    float direction_y;
    float confidence;
    uint8_t is_line;
    long id;
    std::vector<Vector2f> image_points;
    std::vector<Vector2f> world_points;
};


struct Freespace {
    long id;
    uint16_t width;
    uint16_t height;
    std::vector<Vector2f> image_points;
    std::vector<Vector2f> world_points;
    std::vector<uint8_t> bit_map;
    std::vector<float> confidence;
};


struct CameraBBox2DInfo
{
    uint16_t width;
    uint16_t height;
    std::vector<Vector3f> border_points;
};


struct TrafficLightObject {
    int16_t label;
    uint16_t status_label;
    uint16_t color_label;
    uint16_t type_label;
    float type_confidence;
    float existence_confidence;
    long track_id;
    CameraBBox2DInfo camera_bbox_info;
};


struct TrafficSignObject {
    int16_t label;
    float type_confidence;
    float existence_confidence;
    long track_id;
    CameraBBox2DInfo camera_bbox_info;
};


struct RoadStruct {
    std::vector<LaneLine> laneline_results;
    std::vector<StopLine> stopline_results;
    std::vector<Boundary> boundary_results;
    std::vector<Freespace> freespace_results;
    std::vector<TrafficLightObject> traffic_light_results;
    std::vector<TrafficSignObject> traffic_sign_results;
};

// Sync message
struct SyncHeader{
  /// @brief Image timestamp 
  std::uint64_t image_timestamp;
  /// @brief SPI timestamp
  std::uint64_t spi_timestamp;
  /// @brief Frame index
  std::uint32_t frame_index;
  /// @brief Camera id
  std::uint8_t camera_id;
  /// @brief Protocol major version
  std::uint8_t protocol_major_version;
  /// @brief Protocol minor version
  std::uint8_t protocol_minor_version;
  std::uint8_t padding;
};


struct RoadFrame {
    uint32_t instance_id;
    long long frame_timestamp_ns;
    int16_t sensor_id;
    std::string sensor_name;
    SyncHeader header_;
    RoadStruct road_struct;
};

} 

#pragma pack(pop)

#endif//SLAM_INTERFACE_ROAD_DATA_H_