#ifndef SLAM_ROAD_MODEL_H_
#define SLAM_ROAD_MODEL_H_

#include <iostream>
#include <vector>
#include <list>
#include <Eigen/Dense>
#include <Eigen/Geometry>

//#include "common_type.h"

namespace slam
{
enum PositionType : uint8_t
{
    UNKNOW = 0,
    FISRT_ON_THE_LEFT = 1,
    SECOND_ON_THE_LEFT = 2,
    THIRD_ON_THE_LEFT = 3,
    FORTH_ON_THE_LEFT = 4,
    FIFTH_ON_THE_LEFT = 5,
    SIXTH_ON_THE_LEFT = 6,
    SEVENTH_ON_THE_LEFT = 7,
    FISRT_ON_THE_RIGHT = 8,
    SECOND_ON_THE_RIGHT = 9,
    THIRD_ON_THE_RIGHT = 10,
    FORTH_ON_THE_RIGHT = 11,
    FIFTH_ON_THE_RIGHT = 12,
    SIXTH_ON_THE_RIGHT = 13,
    SEVENTH_ON_THE_RIGHT = 14,
    OTHER = 15
};




struct position {
    float x;
    float y;

    position() : x(0.0f), y(0.0f) {}
};

struct attribute {
    float curve;
    float offset;
    uint32_t lane_marker_type;


    attribute() : curve(0.0f), offset(0.0f), lane_marker_type(0) {}
};

struct line_point {
    position point;
    attribute point_attri;

    line_point() : point(), point_attri() {}
};

struct speed_limit {
    float max_speed_limit;
    float min_speed_limit;
    float rec_speed_limit;

    speed_limit()
        : max_speed_limit(0.0f), min_speed_limit(0.0f), rec_speed_limit(0.0f) {}
};

struct traffic_object {
    uint32_t object_id;
    uint8_t object_type;
    uint8_t object_state;
    // uint8_t padding[2];
    float start_offset;
    float end_offset;

    traffic_object()
        : object_id(0), object_type(0), object_state(0),
          start_offset(0.0f), end_offset(0.0f) {}
};

struct road_edge {
    uint8_t type;
    uint8_t side;
    // uint8_t padding[2];
    float start_offset;
    float end_offset;
    std::vector<position> points;

    road_edge()
        : type(0), side(0), start_offset(0.0f), end_offset(0.0f) {}
};

struct line
{
    uint32_t id;
    uint8_t line_type;
    uint8_t direction;
    uint8_t color;
    PositionType position_type;
    float width;
    float start_offset;
    float end_offset;
    std::vector<line_point> points;

    line():
        id(0), line_type(0), direction(0), color(0),
        position_type(PositionType::UNKNOW), width(0.0),
        start_offset(0.0), end_offset(0.0) {}
};


struct traffic_lane
{
    uint8_t lane_type;
    uint8_t lane_no;
    uint32_t lane_id;
    uint32_t center_line_id;
    uint32_t left_line_id;
    uint32_t right_line_id;
    float lane_width;
    float start_offset;
    float end_offset;
    speed_limit speed_limit_value;
    std::vector<traffic_object> traffic_objects;

    traffic_lane()
        : lane_type(0), lane_no(0), lane_id(0), center_line_id(0),
          left_line_id(0), right_line_id(0), lane_width(0.0),
          start_offset(0.0), end_offset(0.0),speed_limit_value(),traffic_objects(){}
};

struct slam_road_model {
    uint64_t utc_timestamp;
    uint32_t cycle_counter;
    std::vector<traffic_lane> lanes;
    std::vector<road_edge> road_edges;
    std::vector<line> lines;

    slam_road_model()
        : utc_timestamp(0), cycle_counter(0), lanes(),road_edges(), lines(){}
};


}
#endif // SLAM_ROAD_MODEL_H_
