#ifndef SLAM_INTERFACE_VEHICLE_DATA_H_
#define SLAM_INTERFACE_VEHICLE_DATA_H_
#pragma pack(push)
#pragma pack(1)

#include <cstdint>

#include "interface/common_type.h"

namespace slam {

// Steering angle speed direction
enum class SteeringAngleSpeedDirection : std::uint8_t {
  CLOCKWISE = 0,        // Clockwise or RotSpd is 0
  COUNTERCLOCKWISE = 1  // counterclockwise
};

// Vehicle data
struct alignas(4) VehicleData {
  /// @brief Timestamp
  std::uint64_t timestamp;
  /// @brief Whether speed is valid
  bool is_speed_valid;
  /// @brief Whether front left wheel speed valid
  bool is_wheel_speed_fl_valid;
  /// @brief Whether front right wheel speed valid
  bool is_wheel_speed_fr_valid;
  /// @brief Whether rear left wheel speed valid
  bool is_wheel_speed_rl_valid;
  /// @brief Whether rear right wheel speed valid
  bool is_wheel_speed_rr_valid;
  /// @brief Whether steering angle is calibrated
  bool is_steering_angle_calibrated;
  /// @brief Whether steering angle sensor is failure
  bool is_steering_angle_sensor_failure;
  /// @brief Steering angle speed direction
  SteeringAngleSpeedDirection steering_angle_speed_direction;
  /// @brief FrontLftWheelSpeedDir
  WheelSpeedDirection wheel_speed_fl_direction;
  /// @brief FrontRgtWheelSpeedDir
  WheelSpeedDirection wheel_speed_fr_direction;
  /// @brief RearLftWheelSpeedDir
  WheelSpeedDirection wheel_speed_rl_direction;
  /// @brief RearRgtWheelSpeedDir
  WheelSpeedDirection wheel_speed_rr_direction;
  /// @brief Speed (km/h)
  float speed;
  /// @brief Front left wheel speed (km/h)
  float wheel_speed_fl;
  /// @brief Front right wheel speed (km/h)
  float wheel_speed_fr;
  /// @brief Rear left wheel speed (km/h)
  float wheel_speed_rl;
  /// @brief Rear right wheel speed (km/h)
  float wheel_speed_rr;
  /// @brief Steering angle (deg)
  float steering_angle;
  /// @brief Steering angle speed (deg/s)
  float steering_angle_speed;
  /// @brief Whether autonomous driving is on
  bool is_auto_mode_on;
  std::uint8_t padding[3]; 
  /// @brief Front left wheel count
  std::uint16_t wheel_count_fl;
  /// @brief Front right wheel count
  std::uint16_t wheel_count_fr;
  /// @brief Rear left wheel count
  std::uint16_t wheel_count_rl;
  /// @brief Rear right wheel count
  std::uint16_t wheel_count_rr;
};

} // namespace slam

#pragma pack(pop)
#endif // SLAM_INTERFACE_VEHICLE_DATA_H_