#include <unistd.h>

#include <fstream>
#include <future>
#include <iostream>
#include <memory>
#include <string>
#include <thread>
#include <vector>

#include "include/slam_manager.h"
#include "include/data/package_line.h"
#include "int_test.h"
#include "SLAM_GeneralWrapper.h"
#include "slam/executable.h"

void package_ins_inputs(slam::InsData* const _insData, CAN2SWC const _canData);
void package_rm_outputs(dpc::slam_road_model* const out_rmData, slam::slam_road_model const in_rmData);



int main(int argc, char **argv)
{
   // setup_test();


    // Logger for wrapper
	// ara headers must be added

	ara::log::InitLogging("DPC", "SLAM_TEST", ara::log::LogLevel::kVerbose, (ara::log::LogMode::kConsole | ara::log::LogMode::kRemote));
	ara::log::Logger &mainLog { ara::log::CreateLogger("main", "slam sample context", ara::log::LogLevel::kVerbose) };
	mainLog.LogInfo() << "Let's produce some slam data...";



	GeneralWrapper gw(mainLog);



	//output
	//***to road model
	dpc::slam_road_model slam_road_model_data;
	dpc::position slam_position_data;
	dpc::attribute slam_attribute_data;
	dpc::line_point slam_line_point_data;
	dpc::line slam_line_data;
	dpc::speed_limit slam_speed_limit_data;
	dpc::traffic_object slam_traffic_object_data;
	dpc::Lane slam_lane_data;
	dpc::RoadEdge slam_road_edge_data;

	//***to decision
	dpc::ControlWaypoints slam_con_wayp_data;

	dpc::MsgHeader slam_header_data;
	dpc::ControlWayPointStruct slam_cps_data;
	dpc::RemindWayPointStruct slam_rwps_data;
	dpc::TransitLaneStruct slam_tls_data;

	dpc::SMAStruct slam_smas_data;
	dpc::WPExtendLaneStruct slam_wpes_data;
	dpc::AdjacentExtendLaneStruct slam_adjacent_data;
	dpc::slam_odometry slam_odom_data;

	//input:
	CAN2SWC can2slam_data; // Executable Sub data

	//Local variables and object declarations
	uint32_t slam_run_counter = 0;
	slam::PackageLine wk_package_line;

	slam::slam_road_model wk_rm_;
	slam::slam_odometry wk_odom;

	wk_rm_.lanes.resize(1);
	wk_rm_.lines.resize(1);
	wk_rm_.lines[0].points.resize(100);


	while(1)
	{
        if (slam_run_counter == 0) {

           wk_package_line.Init();
            printf("slam initialized!\n");

           slam_road_model_data.cycle_counter = 0;
           slam_con_wayp_data.msg_header.cycle_counter = 0;
        }
        else
        {
            //********************************Get input data********************************
           gw._CAN2SLAM_Required.s.GetData(can2slam_data);
//           std::cout << "can data: " << can2slam_data.VCU_Info_1_Signals.ses_en_sts << std::endl;

//            ********************************run slam program********************************
            slam::InsData _insdata;
            package_ins_inputs(&_insdata, can2slam_data);
            wk_package_line.SetHostVehiclePosition(_insdata);
            //bool wk_odom_flag = wk_package_line.PackageOdom(wk_odom);
            //bool wk_rm_flag = wk_package_line.PackageRM(wk_rm_.lines[0].points);

            //********************************publish output data********************************
            gw._SLAM2EMposition_Provide.s.SetData(slam_position_data);
//     		std::cout<< __LINE__ <<std::endl;
             gw._SLAM2EMposition_Provide.Publish();
//     		std::cout<< __LINE__ <<std::endl;


             gw._SLAM2EMattribute_Provide.s.SetData(slam_attribute_data);
             gw._SLAM2EMattribute_Provide.Publish();

             gw._SLAM2EMline_point_Provide.s.SetData(slam_line_point_data);
             gw._SLAM2EMline_point_Provide.Publish();

             //SLAM2EMline_Provide _SLAM2EMline_Provide;
             gw._SLAM2EMline_Provide.s.SetData(slam_line_data);
             gw._SLAM2EMline_Provide.Publish();

             //SLAM2EMspeed_limit_Provide _SLAM2EMspeed_limit_Provide;
             gw._SLAM2EMspeed_limit_Provide.s.SetData(slam_speed_limit_data);
             gw._SLAM2EMspeed_limit_Provide.Publish();

             //SLAM2EMtraffic_object_Provide _SLAM2EMtraffic_object_Provide;
             gw._SLAM2EMtraffic_object_Provide.s.SetData(slam_traffic_object_data);
             gw._SLAM2EMtraffic_object_Provide.Publish();

             //SLAM2EMlane_Provide _SLAM2EMlane_Provide;
             gw._SLAM2EMlane_Provide.s.SetData(slam_lane_data);
             gw._SLAM2EMlane_Provide.Publish();

             //SLAM2EMroad_edge_Provide _SLAM2EMroad_edge_Provide;
             gw._SLAM2EMroad_edge_Provide.s.SetData(slam_road_edge_data);
             gw._SLAM2EMroad_edge_Provide.Publish();

//            SLAM2DMAdjacentExtendLaneStruct_Provide _SLAM2DMAdjacentExtendLaneStruct_Provide;
             gw._SLAM2DMAdjacentExtendLaneStruct_Provide.s.SetData(slam_adjacent_data);
             gw._SLAM2DMAdjacentExtendLaneStruct_Provide.Publish();

             //SLAM2DMSMAStruct_Provide _SLAM2DMSMAStruct_Provide;
             gw._SLAM2DMSMAStruct_Provide.s.SetData(slam_smas_data);
             gw._SLAM2DMSMAStruct_Provide.Publish();

             //SLAM2DMWPExtendLaneStruct_Provide _SLAM2DMWPExtendLaneStruct_Provide;
             gw._SLAM2DMWPExtendLaneStruct_Provide.s.SetData(slam_wpes_data);
             gw._SLAM2DMWPExtendLaneStruct_Provide.Publish();

             //SLAM2DMRemindWaypointStruct_Provide _SLAM2DMRemindWaypointStruct_Provide;
             gw._SLAM2DMRemindWaypointStruct_Provide.s.SetData(slam_rwps_data);
             gw._SLAM2DMRemindWaypointStruct_Provide.Publish();

             //SLAM2DMControlWaypointStruct_Provide _SLAM2DMControlWaypointStruct_Provide;
             gw._SLAM2DMControlWaypointStruct_Provide.s.SetData(slam_cps_data);
             gw._SLAM2DMControlWaypointStruct_Provide.Publish();

             //SLAM2DMMsgHeader_Provide _SLAM2DMMsgHeader_Provide;
             gw._SLAM2DMMsgHeader_Provide.s.SetData(slam_header_data);
             gw._SLAM2DMMsgHeader_Provide.Publish();

             //SLAM2DMTransitLaneStruct_Provide _SLAM2DMTransitLaneStruct_Provide;
             gw._SLAM2DMTransitLaneStruct_Provide.s.SetData(slam_tls_data);
             gw._SLAM2DMTransitLaneStruct_Provide.Publish();

//            SLAM2EMslam_road_model_Provide _SLAM2EMslam_road_model_Provide;
            slam_road_model_data.cycle_counter++;
            package_rm_outputs(&slam_road_model_data, wk_rm_);
            gw._SLAM2EMslam_road_model_Provide.s.SetData(slam_road_model_data);
            gw._SLAM2EMslam_road_model_Provide.Publish();

//            SLAM2DMControlWaypoints_Provide _SLAM2DMControlWaypoints_Provide;
            slam_con_wayp_data.msg_header.cycle_counter++;
            gw._SLAM2DMControlWaypoints_Provide.s.SetData(slam_con_wayp_data);
            gw._SLAM2DMControlWaypoints_Provide.Publish();

              //Slam2PerceptionOdom_Provide _Slam2PerceptionOdom_Provide;
             gw._Slam2PerceptionOdom_Provide.s.SetData(slam_odom_data);
             gw._Slam2PerceptionOdom_Provide.Publish();

        }

        slam_run_counter++;

        std::this_thread::sleep_for(std::chrono::milliseconds(20)); // slam cycle time: 20 ms
	}


    return 0;
}

void package_ins_inputs(slam::InsData* const _insData, CAN2SWC const _canData)
{
    _insData->utc_timestamp = _canData.INS_SOL_RESULT_LONGITUDE_Signals.timestamp_s * 1e3
                + _canData.INS_SOL_RESULT_LONGITUDE_Signals.timestamp_ns*1e-6;//milliseconds

    _insData->longitude = _canData.INS_SOL_RESULT_LONGITUDE_Signals.ins_longitude;
    _insData->latitude = _canData.INS_SOL_RESULT_LATITUDE_Signals.ins_latitude;
    _insData->heading = _canData.INS_SOL_RESULT_ANGLE_Signals.ins_yaw_angle;

    //_insData->status
}

void package_rm_outputs(dpc::slam_road_model* const out_rmData, 
    slam::slam_road_model const in_rmData)
{
    out_rmData->lines.resize(in_rmData.lines.size());

    for(int i = 0; i < in_rmData.lines.size();++i)
    {
        out_rmData->lines[i].points.resize(in_rmData.lines[i].points.size());

        for(int j = 0; j < in_rmData.lines[i].points.size(); ++j)
        {
            dpc::line_point _p;
            _p.point.x = in_rmData.lines[i].points[j].point.x;
            _p.point.y = in_rmData.lines[i].points[j].point.y;

            out_rmData->lines[i].points[j] = _p;
        }
    }
}
