#include "data/package_line.h"
#include "data/data_coordinates.h"
#include "util/slam_util.h"
#include "data_manager/data_manager.h"

namespace slam
{

PackageLine::PackageLine()
{
}

void PackageLine::Init()
{
    start_point_ = GetVectorPoint(coordinates[0].latitude, coordinates[0].longitude);

    slam::Vector2d last_point_ = start_point_;

    for (size_t i = 0; i < COOR_P_NUM && i < 2200; i++)
    {
        slam::Vector2d c_world_p = GetVectorPoint(coordinates[i].latitude, coordinates[i].longitude);
        c_world_p.x -= start_point_.x;
        c_world_p.y -= start_point_.y;

        if(i>0)
        {
            double td_ =  fabs(c_world_p.x - last_point_.x) +fabs(c_world_p.y - last_point_.y);
            if(td_ < 0.5)
            {
                continue;
            }else
            {
                last_point_ = c_world_p;
            }
        }

        ref_line_.push_back(c_world_p);
    }

}

std::vector<slam::Vector2d> extractPoints(const std::vector<slam::Vector2d> &ref_line, int index, int forward_num, int backward_num)
{
    std::vector<slam::Vector2d> extracted_points;

    // 计算起始和结束索引
    int start_index = std::max(index - forward_num, 0);
    int end_index = std::min(index + backward_num, static_cast<int>(ref_line.size()) - 1);

    extracted_points.resize(end_index - start_index + 1);

    int cindex = 0;
    // 提取点
    for (int i = start_index; i <= end_index; ++i)
    {
        extracted_points[cindex] = ref_line[i];
        cindex++;
    }

    return extracted_points;
}

bool PackageLine::PackageOdom(slam::slam_odometry & odom_)
{
    odom_.utc_timestamp = cur_ins_.utc_timestamp;
    odom_.position.position.x = cur_host_pos.x;
    odom_.position.position.y = cur_host_pos.y;

    Quaterniond wk_quat_ret = SlamUtil::EulerToQuaternion(
                cur_ins_.attitude.z * DEG2RAD,
                cur_ins_.attitude.y * DEG2RAD,
                cur_ins_.attitude.x * DEG2RAD);

    odom_.position.orientation.w = (float)wk_quat_ret.w;
    odom_.position.orientation.x = (float)wk_quat_ret.x;
    odom_.position.orientation.y = (float)wk_quat_ret.y;
    odom_.position.orientation.z = (float)wk_quat_ret.z;

    return true;
}

bool PackageLine::PackageRM(std::vector<line_point>& points)
{
    int c_index = SlamUtil::findNearestPointIndex(cur_host_pos, ref_line_, 10, 1.0);

    if (c_index == -1)return false;

    std::vector<slam::Vector2d> c_line = extractPoints(ref_line_, c_index, 20, 180);

    if(points.size() > c_line.size())points.resize(c_line.size());

    for (uint i = 0 ; i< points.size(); ++i)
    {
        slam::Vector2d wk_vcs_p = SlamUtil::globalToLocal(c_line[i], cur_host_pos, cur_host_heading);

        points[i].point.x = wk_vcs_p.x;
        points[i].point.y = wk_vcs_p.y;
    }

    return true;
}

void PackageLine::SetHostVehiclePosition(const InsData& _ins)
{
    if (last_odom_utc_ >= _ins.utc_timestamp) return;
    last_odom_utc_ =  _ins.utc_timestamp;

    cur_host_pos = GetVectorPoint(_ins.latitude, _ins.longitude);
    cur_host_pos.x -= start_point_.x;
    cur_host_pos.y -= start_point_.y;

    cur_host_heading = _ins.heading * DEG2RAD;

    cur_ins_ = _ins;
}

slam::Vector2d PackageLine::GetVectorPoint(double lat, double lon)
{
    slam::Vector2d wk_p0;

    Localization::PointLoc::CoordinateUtil::PositionLL wk_gps;
    wk_gps.latitude = lat;
    wk_gps.longitude = lon;
    Localization::PointLoc::CoordinateUtil::PositionGK world_p_ = Localization::PointLoc::CoordinateUtil::Ll2GK(wk_gps, base_lon_);

    wk_p0.x = world_p_.x;
    wk_p0.y = world_p_.y;

    return wk_p0;
}

}
