#include "functions.h"
#include <cmath>



Functions::Functions()
{

}

std::vector<std::string> Functions::splitWithStl(const std::string &str,const std::string &pattern)
{
    std::vector<std::string> resVec;

    if ("" == str)
    {
        return resVec;
    }
    //方便截取最后一段数据
    std::string strs = str + pattern;

    size_t pos = strs.find(pattern);
    size_t size = strs.size();

    while (pos != std::string::npos)
    {
        std::string x = strs.substr(0,pos);
        resVec.push_back(x);
        strs = strs.substr(pos+1,size);
        pos = strs.find(pattern);
    }

    return resVec;
}

// 获取两个向量的夹角
double Functions::GetTwoVectorsAngle(const Eigen::Vector2d &v1,
                          const Eigen::Vector2d &v2) {
  double cosine_value = v1.dot(v2) / (v1.norm() * v2.norm());
  // 归约到-1到1,防止出错
//  if (cosine_value > 1)
   // cosine_value = 1;
  //else if (cosine_value < -1)
    //cosine_value = -1;

//弧度
  double value = acos(cosine_value);

  return value;
}


bool Functions::IsThreePointsInLine(const Eigen::Vector2d &p1, const Eigen::Vector2d &p2,
                         const Eigen::Vector2d &p3, double radian_deviation) {
  Eigen::Vector2d p1_p2 = p2 - p1;
  Eigen::Vector2d p1_p3 = p3 - p1;
  // 有重合点直接返回true
  if (p1_p2.norm() == 0 || p1_p3.norm() == 0) {
    return true;
  }

  double radian_value_of_p1 = GetTwoVectorsAngle(p1_p2, p1_p3);


  Eigen::Vector2d p3_p1 = p1 - p3;
  Eigen::Vector2d p3_p2 = p2 - p3;
  // 有重合点直接返回true
  if (p3_p1.norm() == 0 || p3_p2.norm() == 0) {
    return true;
  }

  double radian_value_of_p3 = GetTwoVectorsAngle(p3_p1, p3_p2);

  if (radian_value_of_p1 <= radian_deviation &&
      radian_value_of_p3 <= radian_deviation) {
    return true;
  } else {
    return false;
  }
}

double Functions::FitCenterByLeastSquares(std::vector<Eigen::Vector2d> mapPoint) {


    //if(IsThreePointsInLine(mapPoint[0],mapPoint[1],mapPoint[2],0.1))return 0;

    Eigen::Vector2d centerP;
    double radius = 0,curvature = 0;

    double sumX = 0, sumY = 0;
    double sumXX = 0, sumYY = 0, sumXY = 0;
    double sumXXX = 0, sumXXY = 0, sumXYY = 0, sumYYY = 0;

    int pCount = mapPoint.size();
    for (int i = 0; i<pCount; i++){
        sumX += mapPoint[i].x();
        sumY += mapPoint[i].y();
        sumXX += pow(mapPoint[i].x(),2);
        sumYY += pow(mapPoint[i].y(),2);
        sumXY += mapPoint[i].x() * mapPoint[i].y();
        sumXXX += pow(mapPoint[i].x(),3);
        sumXXY += pow(mapPoint[i].x(),2) * mapPoint[i].y();
        sumXYY += mapPoint[i].x() * pow(mapPoint[i].y(),2);
        sumYYY += pow(mapPoint[i].y(),3);
    }

    double M1 = pCount * sumXY - sumX * sumY;
    double M2 = pCount * sumXX - sumX * sumX;
    double M3 = pCount * (sumXXX + sumXYY) - sumX * (sumXX + sumYY);
    double M4 = pCount * sumYY - sumY * sumY;
    double M5 = pCount * (sumYYY + sumXXY) - sumY * (sumXX + sumYY);

    if((M2 * M4 - M1 * M1) == 0)return 0;

    double a = (M1 * M5 - M3 * M4) / (M2 * M4 - M1 * M1);
    double b = (M1 * M3 - M2 * M5) / (M2 * M4 - M1 * M1);
    double c = -(a * sumX + b * sumY + sumXX + sumYY) / pCount;

    //圆心XY 半径
    double xCenter = -0.5 * a;
    double yCenter = -0.5 * b;
    radius = 0.5 * sqrt(a * a + b * b - 4 * c);
    centerP.x() = xCenter;
    centerP.y() = yCenter;

    //计算曲率正负号
    a = (sumXY-((sumX*sumY)/pCount))/(sumXX-(sumX*sumX/pCount));
    b = (sumY-a*sumX)/pCount;
    curvature = (xCenter*a+b<yCenter)?(1/radius):(-1/radius);

    return curvature;
}

Eigen::Vector2d Functions::interpolate_translation(double t1,
                                        Eigen::Vector2d trans1,
                                        double t2,
                                        Eigen::Vector2d trans2,
                                        double t_inter){

    return trans1 + (trans2 - trans1) / (t2 - t1) * (t_inter - t1);
}

double Functions::interpolate_heading(double t1,double heading1,double t2,double heading2, double t_inter){

    return heading1 + (heading2 - heading1) / (t2 - t1) * (t_inter - t1);
}

Eigen::Vector2d Functions::gcs2vcs(Eigen::Vector2d gcs_p, Eigen::Vector2d cen_p, double heading)
{
    Eigen::Vector2d res_p;

    double dev_x = gcs_p.x() - cen_p.x();
    double dev_y = gcs_p.y() - cen_p.y();

    res_p.x() = dev_x * cos(heading) - dev_y*sin(heading);
    res_p.y() = dev_x * sin(heading) + dev_y*cos(heading);

    return res_p;
}

Eigen::Vector2d Functions::vcs2gcs(Eigen::Vector2d host_p, float vh, Eigen::Vector2d vcs_p)
{

    Eigen::Vector2d _np;

    _np.x() = (host_p.x() + (vcs_p.x()*cos(vh) - vcs_p.y()*sin(vh)));
    _np.y() = (host_p.y() + (vcs_p.x()*sin(vh) + vcs_p.y()*cos(vh)));

    return _np;
}
