#include "mainwindow.h"
#include "./ui_mainwindow.h"
#include <memory>
#include <fstream>
#include <chrono>
#include <thread>

#include "mapviewer.h"
#include "include/slam_manager.h"
#include "Inspvaxa.h"

#include "functions.h"

MapViewer map_;
std::shared_ptr<slam::SlamManager> p_slam;
InspvaxaData ins_converter;

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
    , ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    map_.Init(ui->customPlot);

    p_slam = std::make_shared<slam::SlamManager>();
    p_slam->Init();

    QString basePath = QCoreApplication::applicationDirPath();
    basePath.replace(basePath.length() - 6, 6, "");

    std::string file_path = basePath.toStdString()+ "data/ins_240105.dat";
    ReadInsData(file_path);

}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::ReadInsData(std::string fp_)
{
    std::string line;
    std::ifstream ins_file(fp_);

    if (!ins_file.is_open())
    {
        std::cout << "无法打开文件" << std::endl;
        return;
    }

    int index = 0;
    while (std::getline(ins_file, line))
    {
        if (line.substr(0, 9) == "#INSPVAXA") {

            INSPVAXA _inspvaxa = ins_converter.parseINSPVAXA(line);
            slam::InsData _insdata = ins_converter.MapINSPVAXAToInsData(_inspvaxa);

            p_slam->InputInsData2Slam(_insdata);

            //std::this_thread::sleep_for(std::chrono::milliseconds(1));

            std::cout << index++ << std::endl;
        }
    }

    ins_file.close();
}

void MainWindow::on_actionplay_triggered()
{

}

void MainWindow::on_actionfocus_triggered()
{

}

void MainWindow::on_action3DQuat_triggered()
{

}
