#ifndef LANE_MSG_INFO_H
#define LANE_MSG_INFO_H

#include <Eigen/Core>
#include <Eigen/Dense>

namespace Localization
{
namespace PointLoc
{

struct InsInfo {
  unsigned int positionType_ = 0;
  unsigned int satUseNum_ = 0;
  unsigned int satInViewNum_ = 0;
  unsigned int solutionStatus = 0; //此字段放置rtkfix
  unsigned int utmZoneNum = 0;
  bool isValid_ = true;
  bool isEgoUsed_ = false;
  bool isVslUsed_ = false;
  bool isLidarUsed_ = false;
  bool isRelativeUsed_ = false;

  unsigned int seq_ = 0U;
  double stamp_ = 0.0;

    double longitude_ = 0;
    double latitude_ = 0;
    double elevation_ = 0;
    double heading = 0;
    double cog = 0;
    Eigen::Vector3d utmPosition_{0,0,0};  // x y z
    Eigen::Vector3d attitude_{0,0,0,};    // x y z
    Eigen::Quaterniond quat_{1,0,0,0};    // w x y z // Y P R
    Eigen::Vector3d attitudeDual_{0,0,0};
    Eigen::Vector3d velocityENU_{0,0,0};  // m/s
    double velocityVeh_ = 0.0;            // m/s
};

struct PointLocResult {
  enum Status{
    ///如果不使用predict,则状态只看bad和good两个字段
    bad = 0U, // not init,just ref pose
    no_map = 1U, // no map
    no_line = 2U, // no input line
    compute_bad = 3U,// input don't match output,can't converge
    good = 4U, // certainly match map
    compute_ref = 5U, // loc jump compared with valid rtk,use global ref's result;
    compute_predict = 6U // loc jump compared with relative,use relative predict;
  };
  uint8_t status_; // filled with RecLocStatus
  uint64_t seq_;
  double stamp_; // unit: second
  double score_; // unit: 100 score
  double heading; // unit:degree

  Eigen::Vector3d posLLA_;  // lon,lat,altitude
  Eigen::Vector3d posUTM_;  // x:E; y:N; z:U; unit:m
  Eigen::Vector3d attitude_;    // x:roll; y:pitch; z:yaw; unit:rad
  Eigen::Matrix3d rot_; // YPR
  Eigen::Quaterniond quat_; // YPR
  PointLocResult(){
    status_ = 0U;
    seq_ = 0U;
    stamp_ = 0.0;
    score_ = 0.0;
    heading = 0;
    posLLA_ = Eigen::Vector3d::Zero();
    posUTM_ = Eigen::Vector3d::Zero();
    attitude_ = Eigen::Vector3d::Zero();
    rot_ = Eigen::Matrix3d::Identity(); // YPR
    quat_ = Eigen::Quaterniond::Identity();
  }
};

} // namespace PointLoc
} // namespace Localization

#endif
