#ifndef MATCHED_FRAME_H_
#define MATCHED_FRAME_H_

#include <cstdint>
#include <Eigen/Core>
#include <Eigen/Dense>
#include <vector>

#include "lane_msg_info.h"

namespace Localization
{
namespace PointLoc
{
// Sync message
enum class SolutionType : std::uint8_t {
  cubic_line = 0U,
  polygen = 1U,
  points = 2U
};

struct MatchedPoints {
  int ptNums;
  std::vector<Eigen::Vector3d> sensorMatchedPts;
  std::vector<Eigen::Vector3d> mapMatchedPts;
};

// poly status
enum class PolyStatus : std::uint8_t {
  BAD = 0U,
  GOOD = 1U
};
struct MatchedPolys {
  // todo:矢量信息
};

// Lane status
enum class LaneStatus : std::uint8_t {
  BAD = 0U,
  GOOD = 1U,
  SINGLE = 2U
};

struct MatchedCubicLines {
  LaneStatus lanestatus;
  std::vector<std::pair<int, int>> matchedIds;
  std::vector<std::pair<int, std::vector<Eigen::Vector3d>>> camLines;
  std::vector<std::pair<int, std::vector<Eigen::Vector3d>>> mapLines;
};

struct MatchFrame {
  SolutionType inputType;
  double stamp;
  // std::vector<double> MapPoly, LanePoly;
  // std::vector<Eigen::Vector3d> j2Intered,
  // mapIntered;
  MatchedPoints GcsPoints;
  MatchedPoints VcsPoints;
  MatchedPolys GcsPolys;
  MatchedPolys VcsPolys;
  MatchedCubicLines GcsLines;
  MatchedCubicLines VcsLines;
  Localization::PointLoc::InsInfo Egmo, Relative;
  std::vector<Eigen::Vector3d> j2UTMDraw, j2VehDraw, mapUTMDraw, mapVehDraw;
  MatchFrame() = default;
  MatchFrame(const MatchFrame &frame) {
    *this = frame;
  }
};

} // namespace PointLoc
} // namespace Localization
#endif
