#ifndef VISION_INTERFACE_H
#define VISION_INTERFACE_H

#include <cmath>
#include "pose_compute_icp.h"
#include <thread>
#include "lane_msg_info.h"
#include <mutex>
#include "coordinate_util.h"

#include "matched_frame.h"
#include <deque>

namespace Localization {
namespace PointLoc {
class PointLocInterface {
public:
  PointLocInterface();
  ~PointLocInterface();
  void Init();
  // input
  void SetInputFrame(MatchFrame &frame);
  // output
  bool GetLocOutput(PointLocResult &locRes);
  void SetLocOutput(PointLocResult &locRes);

private:
  bool TakeInputFrame(MatchFrame &matchFrame);
  bool CheckInputFrame(MatchFrame &matchFrame);
  // odom func
  void StopOdom();
  void ResetOdom();
  void VehICPThread();
  void OdomResProcess(double &stamp,Eigen::Vector3d &pos,Eigen::Matrix3d &rot,
                      PointLocResult &odomRes);

private:
  // odom
  PoseComputeICP computeICP_;
  PointLocResult locRes_;
  std::thread computeThread_;
  bool computeThreadInited_ = false;
  bool locInited_ = false;
  double outputLocStamp_ = 0.0;

  std::mutex mutexResult_;
  std::mutex mutexFrameBuf_;
  std::deque<shared_ptr<MatchFrame>> frameBuf_;
  //开关
  bool isPrintRes_ = false;
};
} // namespace PointLoc
} // namespace Localization
#endif
