#include "pose_compute_icp.h"
namespace Localization
{
namespace PointLoc
{
PoseComputeICP::PoseComputeICP(){
}

bool PoseComputeICP::ComputeByFrameVCS(const MatchFrame &input,PointLocResult &locRes) {
  bool res = false;
  
  std::vector<std::vector<Eigen::Vector3d>> camLines;
  std::vector<std::vector<Eigen::Vector3d>> mapLines;
  std::vector<float> weights;
  InterPolateFrameLinesVCS(input,camLines,mapLines);
  std::vector<Eigen::Vector3d> camCloud;
  std::vector<Eigen::Vector3d> mapCloud;
  ComputeWeights(camLines,mapLines,camCloud,mapCloud,weights);
  Eigen::Vector3d posIn = Eigen::Vector3d::Zero();
  Eigen::Matrix3d rotYPRIn = Eigen::Matrix3d::Identity();
  Eigen::Vector3d posOpt = Eigen::Vector3d::Zero();
  Eigen::Matrix3d rotYPROpt = Eigen::Matrix3d::Identity();
  if(camCloud.size() != mapCloud.size()) {
    std::cerr<<"ComputeByFrameGCS():点对数目不是一一对应，输入错误，不能icp计算位姿"<<std::endl;
    locRes.status_ = PointLocResult::Status::compute_bad;
    return res;
  } else {
    res = CalTransformCeres(camCloud,mapCloud,weights,posIn,rotYPRIn,posOpt,rotYPROpt);
    if (res) {
      VehAddOpt2Prior(input.Egmo.utmPosition_,input.Egmo.quat_.toRotationMatrix(),
                      posOpt,rotYPROpt,locRes);
    } else {
      locRes.status_ = PointLocResult::Status::compute_bad;
      std::cerr<<std::setprecision(15)<<"OptimizeByCeres():compute not converge,just use prior,stamp:"<<
          input.Egmo.stamp_<<std::endl;
    }
  }
  return res;
}

void PoseComputeICP::VehAddOpt2Prior(Eigen::Vector3d priorPos,Eigen::Matrix3d priorRot,
                                     Eigen::Vector3d dPos,Eigen::Matrix3d dRot,
                                     PointLocResult &locRes) {
  Eigen::Vector3d recPos = priorRot * dPos + priorPos;
  Eigen::Matrix3d recRot = priorRot * dRot;
  locRes.posUTM_ = recPos;
  locRes.quat_ = recRot;
  
  Eigen::Vector3d dYPR = RotYPR2EulerYPR(dRot);
  Eigen::Vector3d priorYPR = RotYPR2EulerYPR(priorRot);
  Eigen::Vector3d recYPR = RotYPR2EulerYPR(recRot);
}

void PoseComputeICP::InterPolateFrameLinesVCS(const MatchFrame &input,
                                              std::vector<std::vector<Eigen::Vector3d>> &camLines,
                                              std::vector<std::vector<Eigen::Vector3d>> &mapLines) {
  for (int i = 0;i < input.VcsLines.matchedIds.size();i++) {
    auto &idPair = input.VcsLines.matchedIds[i];
    auto &camId = input.VcsLines.camLines[i].first;
    auto &mapId = input.VcsLines.mapLines[i].first;
    if (camId != idPair.first || mapId != idPair.second) {
      std::cerr<<"车道线匹配错误，id对为："<<idPair.first<<"-"<<idPair.second<<",但是cam和map的id:"<<
               camId<<"-"<<mapId<<std::endl;
      continue;
    }
    auto &camLine = input.VcsLines.camLines[i].second;
    auto &mapLine = input.VcsLines.mapLines[i].second;
    camLines.emplace_back(camLine);
    mapLines.emplace_back(mapLine);
  }
}

void PoseComputeICP::ComputeWeights(const std::vector<std::vector<Eigen::Vector3d>> &camLines,
                                    const std::vector<std::vector<Eigen::Vector3d>> &mapLines,
                                    std::vector<Eigen::Vector3d> &camCloud,
                                    std::vector<Eigen::Vector3d> &mapCloud,
                                    std::vector<float> &weights)
{
  double oriWeight = 1.0;
  for (int i = 0; i < camLines.size(); ++i) {
    auto &oneCamLine = camLines[i];
    auto &oneMapLine = mapLines[i];

    for (int j = 0; j < oneCamLine.size(); ++j) {
      weights.emplace_back(oriWeight);
      camCloud.emplace_back(oneCamLine[j]);
    }
    for (int j = 0; j < oneMapLine.size(); ++j) {
      mapCloud.emplace_back(oneMapLine[j]);
    }
  }
}

bool PoseComputeICP::CalTransformCeres(std::vector<Eigen::Vector3d> &aimClouds,
                                       std::vector<Eigen::Vector3d> &bigClouds,
                                       std::vector<float> &weights,
                                       const Eigen::Vector3d &posIn,
                                       const Eigen::Matrix3d &rotYPRIn,
                                       Eigen::Vector3d &posOut,Eigen::Matrix3d &rotYPROut)
{
  bool computeRes = true;
  if (aimClouds.size() != bigClouds.size()) {
    std::cerr<< "CalTransformCeres():Ceres failed, aimClouds.size()" <<aimClouds.size()<<",bigClouds.size():"<<bigClouds.size() <<std::endl;
    computeRes = false;
  }
  if (aimClouds.size() == 0) {
    std::cerr<< "CalTransformCeres():Ceres input is 0,failed" << std::endl;
    computeRes = false;
  }
  Eigen::Quaterniond tmpQ(rotYPRIn);
  tmpQ.normalize();
  double optX[1];
  double optY[1];
  double optZ[1];
  optX[0] = posIn.x();
  optY[0] = posIn.y();
  optZ[0] = posIn.z();
  Eigen::Vector3d optRPYEig = tmpQ.toRotationMatrix().eulerAngles(0, 1, 2);
  double optRoll[1];
  double optPitch[1];
  double optYaw[1];
  optRoll[0] = optRPYEig.x();
  optPitch[0] = optRPYEig.y();
  optYaw[0] = optRPYEig.z();
  double weight = 1.0;
  // construct problem
  ceres::Problem problem;
  ceres::Solver::Options options;

  options.linear_solver_type = ceres::DENSE_QR;
  options.max_num_iterations = 10;
  ceres::LossFunction* loss_function = new ceres::CauchyLoss(0.5);
  problem.AddParameterBlock(optX, 1);
  problem.AddParameterBlock(optY, 1);
  problem.AddParameterBlock(optZ, 1);
  problem.AddParameterBlock(optRoll, 1);
  problem.AddParameterBlock(optPitch, 1);
  problem.AddParameterBlock(optYaw, 1);
  problem.SetParameterBlockConstant(optX);
  problem.SetParameterBlockConstant(optZ);
  problem.SetParameterBlockConstant(optRoll);
  problem.SetParameterBlockConstant(optPitch);
  problem.SetParameterLowerBound(optY,0,-1.2);
  problem.SetParameterUpperBound(optY,0,1.2);
  problem.SetParameterLowerBound(optYaw,0,-0.8*DEG_TO_RAD);
  problem.SetParameterUpperBound(optYaw,0,0.8*DEG_TO_RAD);
  std::vector<ceres::ResidualBlockId> resBlockIds;
  int count = 0;
  for (uint16_t i = 0; i <aimClouds.size(); i++) {
    if (count > 30) {
      break;
    }
    count++;
    ceres::CostFunction *cost_function = LineCost::Create(
      aimClouds[i], bigClouds[i], weights[i]);
    ceres::ResidualBlockId resId = problem.AddResidualBlock(
      cost_function,loss_function,
      optX,optY,optZ,optRoll,optPitch,optYaw);
    resBlockIds.push_back(resId);
  }
  ceres::Solver::Summary summary;
  chrono::steady_clock::time_point t1 = chrono::steady_clock::now();
  ceres::Solve ( options, &problem, &summary );
  chrono::steady_clock::time_point t2 = chrono::steady_clock::now();
  chrono::duration<double> time_used = chrono::duration_cast<chrono::duration<double>>( t2-t1 );
  cout<<"time cost = "<<time_used.count()<<" seconds. "<<endl;

  posOut.x() = optX[0];
  posOut.y() = optY[0];
  posOut.z() = optZ[0];
  Eigen::Vector3d ypr{optYaw[0],optPitch[0],optRoll[0]};
  rotYPROut = EulerYPR2RotYPR(ypr);

  return computeRes;
}

} // namespace PointLoc
} // namespace Localization
