#include "CanMessage.h"
#include <chrono>
#include <ctime>
#include <unistd.h>
#include <iomanip>
#include <vector>
#include <thread>
#include <chrono>
#include <set>
#include "PacketHandler.h"
#include "Messages.h"
#include "Utils.h"


Can::Messages* Can::Messages::m_instance_ptr = nullptr;

CanMessage::CanMessage(char** argv, GeneralWrapper* gw) :
	sockfd(0),
	m_argv(argv),
	_general_wrapper(gw)
   // pubWrapper("can_msg_node/can_msg_node/CAN2SWCPPort", mainLog)
{
	//_general_wrapper->_Lat2CAN_Required.s.set_control_req_msg(&_control_request1_msg);
	//_general_wrapper->_Long2CAN_Required.s.set_control_req_msg(&_control_request1_msg);
	receiving_from_software_components_thread = new std::thread(&CanMessage::receiving_from_software_components, this);
	sending_to_mcu_thread = new std::thread(&CanMessage::sending_message_to_mcu, this);
	receiving_thread = new std::thread(&CanMessage::start_listening, this);
    send_to_swc_thread = new std::thread(&CanMessage::send_to_swc, this);

    receiving_from_software_components_thread->join();
	sending_to_mcu_thread->join();
	receiving_thread->join();
	//_general_wrapper->_CAN2SWC_Provide.s.setAllData(Can::Messages::get_instance()->get_all_messages());
   send_to_swc_thread->join();
}

void CanMessage::receiving_from_software_components()
{

}

void CanMessage::send_to_swc()
{
    std::this_thread::sleep_for(std::chrono::seconds(15));
//    pubWrapper.Init();

    while(true)
    {
         _general_wrapper->_CAN2SWC_Provide.s.setAllData(Can::Messages::get_instance()->get_all_messages());
         _general_wrapper->_CAN2SWC_Provide.Publish();
        std::this_thread::sleep_for(std::chrono::milliseconds(50));
    }
}

void CanMessage::receiving_init()
{
    if (!set_receiving_udp_connection())
        exit(EXIT_FAILURE);
}

bool CanMessage::set_receiving_udp_connection()
{
    struct sockaddr_in addr;

    if ((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
    {
        perror("socket Error ...");

        return false;
    }

    std::memset(&addr, 0, sizeof(addr));
    addr.sin_family = AF_INET;
    addr.sin_addr.s_addr = htonl(INADDR_ANY);
    addr.sin_port = htons(atoi(m_argv[2]));
    int res = bind(sockfd, (struct sockaddr*)&addr, sizeof(addr));
    if ( res < 0)
    {
        perror("bind Error ...");

        close(sockfd);

        return false;
    }

    struct ip_mreq ipmr;

    inet_pton(AF_INET, m_argv[1], &ipmr.imr_interface);
    inet_pton(AF_INET, m_argv[3], &ipmr.imr_multiaddr);

    if (setsockopt(sockfd, IPPROTO_IP, IP_ADD_MEMBERSHIP, &ipmr, sizeof(ipmr)) < 0) {
        perror("setsockopt Error");

        close(sockfd);

        return false;
    }

    return true;
}

bool CanMessage::is_timestamp_packet(const char* buffer) const
{
    for (int i = 0; i < 4; i++)
        if (buffer[i] != 0xff)
            return false;

    return true;
}

void CanMessage::calculate_sec_nanosec(uint64_t& seconds,
    uint64_t& nanoseconds,
    char* buffer) const
{
    //buffer 4 to 11 seconds
    // Extract seconds        
    for (int i = 4; i < 12; ++i)
        seconds = (seconds << 8) | static_cast<unsigned char>(buffer[i]);

    //buffer 12 to 19 nano
    // Extract nanoseconds        
    for (int i = 12; i < 20; ++i)
        nanoseconds = (nanoseconds << 8) | static_cast<unsigned char>(buffer[i]);
}

void CanMessage::cal_print_diff_timestampt_soc_mcu(const uint64_t& seconds, const uint64_t& nano_seconds)const
{
    auto now = std::chrono::system_clock::now();
    auto now_us = std::chrono::time_point_cast <std::chrono::microseconds> (now);
    auto epoch_us = now_us.time_since_epoch();
    auto usTimeStampMicro = 0;
    auto usTimeStampSoc = 0;
    auto now_c = std::chrono::duration_cast <std::chrono::seconds>(epoch_us).count();

    usTimeStampSoc = now_c * 1000000 + (epoch_us.count() % 1000000);
    usTimeStampMicro = (seconds * 1e9 + nano_seconds) / 1e6;

    std::cout << "time stamp soc is: " << usTimeStampSoc << std::endl;
    std::cout << "time stamp micro is: " << usTimeStampMicro << std::endl;
    std::cout << "delta is: " << std::abs(usTimeStampMicro - usTimeStampSoc) / 1e3 << "miliseconds." << std::endl;
}


#define PORT 53590
#define TARGET_IP "192.168.10.32"
#define TARGET_PORT 51909
int sockfdTx;
struct sockaddr_in targetAddr;

void init_Send()
{
    struct sockaddr_in senderAddr;

    if ((sockfdTx = socket(AF_INET, SOCK_DGRAM, 0)) == -1)
    {
        perror("socket");

        exit(EXIT_FAILURE);
    }

    memset(&senderAddr, 0, sizeof(senderAddr));
    senderAddr.sin_family = AF_INET;
    senderAddr.sin_port = htons(PORT);
    senderAddr.sin_addr.s_addr = INADDR_ANY;

    if (bind(sockfdTx, (struct sockaddr*)&senderAddr, sizeof(senderAddr)) == -1)
    {
        perror("bind");

        close(sockfdTx);

        exit(EXIT_FAILURE);
    }

    memset(&targetAddr, 0, sizeof(targetAddr));

    targetAddr.sin_family = AF_INET;
    targetAddr.sin_port = htons(TARGET_PORT);

    if (inet_aton(TARGET_IP, &targetAddr.sin_addr) == 0)
    {
        fprintf(stderr, "Invalid address\n");

        close(sockfdTx);

        exit(EXIT_FAILURE);
    }
}

uint8_t CalCRC(uint8_t* data, uint32_t size)
{
    uint8_t sum = 0;

    for (int i = 0; i < size; i++)
    {
        sum ^= data[i];
    }

    return sum;
}

void CanMessage::sending_message_to_mcu()
{
	init_Send();

	static uint8_t TxBuf[1400];
	uint16_t count = 0;
	Rte_CanMsg_t CanList[1];

	std::cout << "start sending...\n";

	while(true)
	{
		uint16_t datalen = sizeof(Rte_CanMsg_t);

		TxBuf[0] = (count>>8) & 0xff;
		TxBuf[1] = (count) & 0xff;
		TxBuf[2] = (datalen>>8) & 0xff;
		TxBuf[3] = (datalen) & 0xff;

		by_nav_x2_can_v0_1_control_request1_t *src_p = (struct by_nav_x2_can_v0_1_control_request1_t *)malloc(sizeof(struct by_nav_x2_can_v0_1_control_request1_t));;

		if(by_nav_x2_can_v0_1_control_request1_init(src_p) == -1)
		{
		 	std::cout << "empty struct" << std::endl;

		 	return;
		}

		src_p->ipc_target_motor_en = by_nav_x2_can_v0_1_control_request1_ipc_target_motor_en_encode(_control_request1_msg.ipc_target_motor_en);
		src_p->ipc_target_gear = by_nav_x2_can_v0_1_control_request1_ipc_target_gear_encode(_control_request1_msg.ipc_target_gear);
		src_p->ipc_target_mode = by_nav_x2_can_v0_1_control_request1_ipc_target_mode_encode(_control_request1_msg.ipc_target_mode);
		src_p->ipc_life_signal = by_nav_x2_can_v0_1_control_request1_ipc_life_signal_encode(_control_request1_msg.ipc_life_signal);
		src_p->ipc_target_speed = by_nav_x2_can_v0_1_control_request1_ipc_target_speed_encode(_control_request1_msg.ipc_target_speed);
		src_p->ipc_target_spd_valid = by_nav_x2_can_v0_1_control_request1_ipc_target_spd_valid_encode(_control_request1_msg.ipc_target_spd_valid);
		src_p->ipc_target_brk_press = by_nav_x2_can_v0_1_control_request1_ipc_target_brk_press_encode(_control_request1_msg.ipc_target_brk_press);
		src_p->ipc_target_angle = by_nav_x2_can_v0_1_control_request1_ipc_target_angle_encode(_control_request1_msg.ipc_target_angle);
		src_p->ipc_target_steering_en = by_nav_x2_can_v0_1_control_request1_ipc_target_steering_en_encode(_control_request1_msg.ipc_target_steering_en);
		src_p->ipc_target_torque = by_nav_x2_can_v0_1_control_request1_ipc_target_torque_encode(_control_request1_msg.ipc_target_torque);
		src_p->ipc_torque_rpm_mode = by_nav_x2_can_v0_1_control_request1_ipc_torque_rpm_mode_encode(_control_request1_msg.ipc_torque_rpm_mode);
		src_p->ipc_turn_light = by_nav_x2_can_v0_1_control_request1_ipc_turn_light_encode(_control_request1_msg.ipc_turn_light);
		src_p->ipc_target_spd_mode = by_nav_x2_can_v0_1_control_request1_ipc_target_spd_mode_encode(_control_request1_msg.ipc_target_spd_mode);

		by_nav_x2_can_v0_1_control_request1_pack(CanList[0].data, src_p, 8u);


		CanList[0].canId = 0x0A1;
		CanList[0].channel = 0x00;
		CanList[0].length = 8;

		memcpy(&TxBuf[4], CanList, datalen);

		TxBuf[datalen+4] = CalCRC(TxBuf, datalen+4);

		sendto(sockfdTx, TxBuf, datalen+5, 0, (struct sockaddr*)&targetAddr, sizeof(targetAddr));

		std::this_thread::sleep_for(std::chrono::milliseconds(50));

		count++;
	}
}

void CanMessage::start_listening()
{
	std::cout << "start listening...\n";

	receiving_init();

    char buffer[MAX_BUFFER_SIZE];

    std::set<uint32_t> can_ids;

    while (true)
    {
        ssize_t nbytes = recvfrom(sockfd, buffer, sizeof(buffer), 0, NULL, 0);


        if (nbytes < 0)
        {
            perror("recvfrom Error ...");

            break;
        }


        buffer[nbytes] = '\0';

        if(is_timestamp_packet(buffer))
        	continue;

        if ((nbytes-5) % 16 == 0)
        {
        	uint16_t data_length = 0;

        	PacketHandler::cal_data_length(buffer, data_length);

        	if(data_length == 0 || data_length % 16 != 0)
        	{
        		std::cout << "The data length is not valid! :" << data_length << std::endl;

        		continue;
        	}

        	uint16_t message_number = data_length / 16;

        	for(int msg_ix=0; msg_ix<message_number; msg_ix++)
        	{
        		Rte_CanMsg_t new_message;

        		std::memcpy(&new_message.canId, buffer + (16*msg_ix + 4), sizeof(new_message.canId));
        		std::memcpy(&new_message.channel, buffer + (16*msg_ix +8), sizeof(new_message.channel));
        		std::memcpy(&new_message.length, buffer + (16*msg_ix + 9), sizeof(new_message.length));
        		std::memcpy(&new_message.resv, buffer + (16*msg_ix + 10), sizeof(new_message.resv));
        		std::memcpy(&new_message.data, buffer + (16*msg_ix + 12), sizeof(new_message.data));

        		auto pos = can_ids.find(new_message.canId);

        		if(pos == can_ids.end())
        		{
        			can_ids.insert(new_message.canId);

        			std::cout << std::hex << "CAN ID: " << new_message.canId << std::endl;
        			std::cout << "Channel: " << static_cast<int>(new_message.channel) << std::endl;
        			std::cout << "Length: " << static_cast<int>(new_message.length) << std::endl;
        			std::cout << "Reserved: " << new_message.resv << std::endl;
        			std::cout << "Data: " << std::hex << new_message.data << std::endl;
        			std::cout << std::endl;
        		}

        		auto now = std::chrono::system_clock::now();
        		auto now_us = std::chrono::time_point_cast <std::chrono::microseconds> (now);
        		auto epoch_us = now_us.time_since_epoch();
        		auto now_c = std::chrono::duration_cast <std::chrono::seconds>(epoch_us).count();
        		auto usTimeStampSoc = now_c * 1000000 + (epoch_us.count() % 1000000);


        		handle_can_message(new_message.data, new_message.canId, usTimeStampSoc/1000000, (usTimeStampSoc%1000000) * 1000, 0);

//        		toPrint(Can::Messages::get_instance()->get_all_messages());
        	}
        }
        else
        {
            printf("May have packet Loss! \n");
        }
    }

    close(sockfd);
}
